@@ -147,7 +147,8 @@ attrd_add_value_xml(xmlNode *parent, const attribute_t *a,
         crm_xml_add_int(xml, PCMK__XA_ATTR_IS_REMOTE, 1);
     }
     crm_xml_add(xml, PCMK__XA_ATTR_VALUE, v->current);
-    crm_xml_add_int(xml, PCMK__XA_ATTR_DAMPENING, a->timeout_ms / 1000);
+    crm_xml_add_int(xml, PCMK__XA_ATTR_DAMPENING,
+                    pcmk__timeout_ms2s(a->timeout_ms));
     crm_xml_add_int(xml, PCMK__XA_ATTR_IS_PRIVATE,
                     pcmk_is_set(a->flags, attrd_attr_is_private));
     crm_xml_add_int(xml, PCMK__XA_ATTRD_IS_FORCE_WRITE, force_write);
@@ -225,9 +226,9 @@ attrd_set_id(const attribute_t *attr, const char *node_state_id)
 {
     char *set_id = NULL;
 
-    CRM_ASSERT((attr != NULL) && (node_state_id != NULL));
+    pcmk__assert((attr != NULL) && (node_state_id != NULL));
 
-    if (attr->set_id == NULL) {
+    if (pcmk__str_empty(attr->set_id)) {
         /* @COMPAT This should really take the set type into account. Currently
          * we use the same XML ID for transient attributes and utilization
          * attributes. It doesn't cause problems because the status section is
@@ -245,7 +246,7 @@ attrd_set_id(const attribute_t *attr, const char *node_state_id)
          */
         set_id = pcmk__str_copy(attr->set_id);
     }
-    crm_xml_sanitize_id(set_id);
+    pcmk__xml_sanitize_id(set_id);
     return set_id;
 }
 
@@ -270,6 +271,6 @@ attrd_nvpair_id(const attribute_t *attr, const char *node_state_id)
         nvpair_id = crm_strdup_printf(PCMK_XE_STATUS "-%s-%s",
                                       node_state_id, attr->id);
     }
-    crm_xml_sanitize_id(nvpair_id);
+    pcmk__xml_sanitize_id(nvpair_id);
     return nvpair_id;
 }
