@@ -62,11 +62,11 @@ attrd_cib_updated_cb(const char *event, xmlNode *msg)
         return;
     }
 
-    if (cib__element_in_patchset(patchset, PCMK_XE_ALERTS)) {
+    if (pcmk__cib_element_in_patchset(patchset, PCMK_XE_ALERTS)) {
         mainloop_set_trigger(attrd_config_read);
     }
 
-    status_changed = cib__element_in_patchset(patchset, PCMK_XE_STATUS);
+    status_changed = pcmk__cib_element_in_patchset(patchset, PCMK_XE_STATUS);
 
     client_name = crm_element_value(msg, PCMK__XA_CIB_CLIENTNAME);
     if (!cib__client_triggers_refresh(client_name)) {
@@ -81,7 +81,8 @@ attrd_cib_updated_cb(const char *event, xmlNode *msg)
         return;
     }
 
-    if (status_changed || cib__element_in_patchset(patchset, PCMK_XE_NODES)) {
+    if (status_changed
+        || pcmk__cib_element_in_patchset(patchset, PCMK_XE_NODES)) {
         /* An unsafe client modified the PCMK_XE_NODES or PCMK_XE_STATUS
          * section. Write transient attributes to ensure they're up-to-date in
          * the CIB.
@@ -114,12 +115,12 @@ attrd_cib_connect(int max_retry)
         }
         attempts++;
         crm_debug("Connection attempt %d to the CIB manager", attempts);
-        rc = the_cib->cmds->signon(the_cib, PCMK__VALUE_ATTRD, cib_command);
+        rc = the_cib->cmds->signon(the_cib, crm_system_name, cib_command);
 
     } while ((rc != pcmk_ok) && (attempts < max_retry));
 
     if (rc != pcmk_ok) {
-        crm_err("Connection to the CIB manager failed: %s " CRM_XS " rc=%d",
+        crm_err("Connection to the CIB manager failed: %s " QB_XS " rc=%d",
                 pcmk_strerror(rc), rc);
         goto cleanup;
     }
@@ -154,6 +155,7 @@ attrd_cib_disconnect(void)
     the_cib->cmds->del_notify_callback(the_cib, PCMK__VALUE_CIB_DIFF_NOTIFY,
                                        attrd_cib_updated_cb);
     cib__clean_up_connection(&the_cib);
+    mainloop_destroy_trigger(attrd_config_read);
 }
 
 static void
@@ -214,12 +216,12 @@ attrd_cib_init(void)
      * this handles the case where the node restarted so quickly that the
      * cluster layer didn't notice.
      *
-     * \todo If pacemaker-attrd respawns after crashing (see PCMK_ENV_RESPAWNED),
-     *       ideally we'd skip this and sync our attributes from the writer.
-     *       However, currently we reject any values for us that the writer has, in
-     *       attrd_peer_update().
+     * \todo If the attribute manager respawns after crashing (see
+     *       PCMK_ENV_RESPAWNED), ideally we'd skip this and sync our attributes
+     *       from the writer. However, currently we reject any values for us
+     *       that the writer has, in attrd_peer_update().
      */
-    attrd_cib_erase_transient_attrs(attrd_cluster->uname);
+    attrd_cib_erase_transient_attrs(attrd_cluster->priv->node_name);
 
     // Set a trigger for reading the CIB (for the alerts section)
     attrd_config_read = mainloop_add_trigger(G_PRIORITY_HIGH, attrd_read_options, NULL);
@@ -278,7 +280,7 @@ attrd_cib_callback(xmlNode *msg, int call_id, int rc, xmlNode *output, void *use
             break;
     }
 
-    do_crm_log(level, "CIB update %d result for %s: %s " CRM_XS " rc=%d",
+    do_crm_log(level, "CIB update %d result for %s: %s " QB_XS " rc=%d",
                call_id, a->id, pcmk_strerror(rc), rc);
 
     g_hash_table_iter_init(&iter, a->values);
@@ -367,7 +369,7 @@ add_set_attr_update(const attribute_t *attr, const char *attr_id,
                                cib_can_create|cib_transaction);
     rc = pcmk_legacy2rc(rc);
 
-    free_xml(update);
+    pcmk__xml_free(update);
     return rc;
 }
 
@@ -546,15 +548,17 @@ write_attribute(attribute_t *a, bool ignore_delay)
 
         } else {
             // This will create a cluster node cache entry if none exists
-            crm_node_t *peer = pcmk__get_node(v->nodeid, v->nodename, NULL,
-                                              pcmk__node_search_any);
+            pcmk__node_status_t *peer = pcmk__get_node(v->nodeid, v->nodename,
+                                                       NULL,
+                                                       pcmk__node_search_any);
 
-            uuid = peer->uuid;
+            uuid = peer->xml_id;
 
             // Remember peer's node ID if we're just now learning it
-            if ((peer->id != 0) && (v->nodeid == 0)) {
-                crm_trace("Learned ID %u for node %s", peer->id, v->nodename);
-                v->nodeid = peer->id;
+            if ((peer->cluster_layer_id != 0) && (v->nodeid == 0)) {
+                crm_trace("Learned ID %" PRIu32 " for node %s",
+                          peer->cluster_layer_id, v->nodename);
+                v->nodeid = peer->cluster_layer_id;
             }
         }
 
@@ -577,7 +581,7 @@ write_attribute(attribute_t *a, bool ignore_delay)
         rc = add_attr_update(a, v->current, uuid);
         if (rc != pcmk_rc_ok) {
             crm_err("Failed to update %s[%s]='%s': %s "
-                    CRM_XS " node uuid=%s id=%" PRIu32,
+                    QB_XS " node uuid=%s id=%" PRIu32,
                     a->id, v->nodename, v->current, pcmk_rc_str(rc),
                     uuid, v->nodeid);
             continue;
