@@ -57,7 +57,7 @@ static xmlNode *build_query_reply(const char *attr, const char *host)
 
         /* Allow caller to use "localhost" to refer to local node */
         if (pcmk__str_eq(host, "localhost", pcmk__str_casei)) {
-            host = attrd_cluster->uname;
+            host = attrd_cluster->priv->node_name;
             crm_trace("Mapped localhost to %s", host);
         }
 
@@ -152,14 +152,14 @@ attrd_client_peer_remove(pcmk__request_t *request)
 
         crm_element_value_int(xml, PCMK__XA_ATTR_HOST_ID, &nodeid);
         if (nodeid > 0) {
-            crm_node_t *node = NULL;
+            pcmk__node_status_t *node = NULL;
             char *host_alloc = NULL;
 
             node = pcmk__search_node_caches(nodeid, NULL,
                                             pcmk__node_search_cluster_member);
-            if (node && node->uname) {
+            if ((node != NULL) && (node->name != NULL)) {
                 // Use cached name if available
-                host = node->uname;
+                host = node->name;
             } else {
                 // Otherwise ask cluster layer
                 host_alloc = pcmk__cluster_node_name(nodeid);
@@ -236,7 +236,8 @@ handle_missing_host(xmlNode *xml)
 
     if (host == NULL) {
         crm_trace("Inferring host");
-        pcmk__xe_add_node(xml, attrd_cluster->uname, attrd_cluster->nodeid);
+        pcmk__xe_add_node(xml, attrd_cluster->priv->node_name,
+                          attrd_cluster->priv->node_id);
     }
 }
 
@@ -409,7 +410,7 @@ attrd_client_update(pcmk__request_t *request)
              */
             for (xmlNode *child = pcmk__xe_first_child(xml, PCMK_XE_OP, NULL,
                                                        NULL);
-                 child != NULL; child = pcmk__xe_next_same(child)) {
+                 child != NULL; child = pcmk__xe_next(child, PCMK_XE_OP)) {
 
                 attr = crm_element_value(child, PCMK__XA_ATTR_NAME);
                 value = crm_element_value(child, PCMK__XA_ATTR_VALUE);
@@ -581,7 +582,7 @@ attrd_ipc_dispatch(qb_ipcs_connection_t * c, void *data, size_t size)
             .result         = PCMK__UNKNOWN_RESULT,
         };
 
-        CRM_ASSERT(client->user != NULL);
+        pcmk__assert(client->user != NULL);
         pcmk__update_acl_user(xml, PCMK__XA_ATTR_USER, client->user);
 
         request.op = crm_element_value_copy(request.xml, PCMK_XA_TASK);
@@ -591,7 +592,7 @@ attrd_ipc_dispatch(qb_ipcs_connection_t * c, void *data, size_t size)
         pcmk__reset_request(&request);
     }
 
-    free_xml(xml);
+    pcmk__xml_free(xml);
     return 0;
 }
 
@@ -611,6 +612,9 @@ attrd_ipc_fini(void)
         qb_ipcs_destroy(ipcs);
         ipcs = NULL;
     }
+
+    attrd_unregister_handlers();
+    pcmk__client_cleanup();
 }
 
 /*!
