@@ -121,20 +121,6 @@ handle_confirm_request(pcmk__request_t *request)
     }
 }
 
-static xmlNode *
-handle_flush_request(pcmk__request_t *request)
-{
-    if (request->peer != NULL) {
-        /* Ignore. The flush command was removed in 2.0.0 but may be
-         * received from peers running older versions.
-         */
-        pcmk__set_result(&request->result, CRM_EX_OK, PCMK_EXEC_DONE, NULL);
-        return NULL;
-    } else {
-        return handle_unknown_request(request);
-    }
-}
-
 static xmlNode *
 handle_query_request(pcmk__request_t *request)
 {
@@ -181,11 +167,13 @@ handle_sync_response_request(pcmk__request_t *request)
         return handle_unknown_request(request);
     } else {
         if (request->peer != NULL) {
-            crm_node_t *peer = pcmk__get_node(0, request->peer, NULL,
-                                              pcmk__node_search_cluster_member);
+            pcmk__node_status_t *peer =
+                pcmk__get_node(0, request->peer, NULL,
+                               pcmk__node_search_cluster_member);
             bool peer_won = attrd_check_for_new_writer(peer, request->xml);
 
-            if (!pcmk__str_eq(peer->uname, attrd_cluster->uname, pcmk__str_casei)) {
+            if (!pcmk__str_eq(peer->name, attrd_cluster->priv->node_name,
+                              pcmk__str_casei)) {
                 attrd_peer_sync_response(peer, peer_won, request->xml);
             }
         }
@@ -200,8 +188,9 @@ handle_update_request(pcmk__request_t *request)
 {
     if (request->peer != NULL) {
         const char *host = crm_element_value(request->xml, PCMK__XA_ATTR_HOST);
-        crm_node_t *peer = pcmk__get_node(0, request->peer, NULL,
-                                          pcmk__node_search_cluster_member);
+        pcmk__node_status_t *peer =
+            pcmk__get_node(0, request->peer, NULL,
+                           pcmk__node_search_cluster_member);
 
         attrd_peer_update(peer, request->xml, host, false);
         pcmk__set_result(&request->result, CRM_EX_OK, PCMK_EXEC_DONE, NULL);
@@ -243,7 +232,6 @@ attrd_register_handlers(void)
     pcmk__server_command_t handlers[] = {
         { PCMK__ATTRD_CMD_CLEAR_FAILURE, handle_clear_failure_request },
         { PCMK__ATTRD_CMD_CONFIRM, handle_confirm_request },
-        { PCMK__ATTRD_CMD_FLUSH, handle_flush_request },
         { PCMK__ATTRD_CMD_PEER_REMOVE, handle_remove_request },
         { PCMK__ATTRD_CMD_QUERY, handle_query_request },
         { PCMK__ATTRD_CMD_REFRESH, handle_refresh_request },
@@ -289,7 +277,7 @@ attrd_handle_request(pcmk__request_t *request)
             crm_err("Not sending CPG reply to client");
         }
 
-        free_xml(reply);
+        pcmk__xml_free(reply);
     }
 
     reason = request->result.exit_reason;
@@ -326,18 +314,19 @@ attrd_broadcast_protocol(void)
     crm_xml_add(attrd_op, PCMK__XA_ATTR_NAME, CRM_ATTR_PROTOCOL);
     crm_xml_add(attrd_op, PCMK__XA_ATTR_VALUE, ATTRD_PROTOCOL_VERSION);
     crm_xml_add_int(attrd_op, PCMK__XA_ATTR_IS_PRIVATE, 1);
-    pcmk__xe_add_node(attrd_op, attrd_cluster->uname, attrd_cluster->nodeid);
+    pcmk__xe_add_node(attrd_op, attrd_cluster->priv->node_name,
+                      attrd_cluster->priv->node_id);
 
     crm_debug("Broadcasting attrd protocol version %s for node %s",
-              ATTRD_PROTOCOL_VERSION, attrd_cluster->uname);
+              ATTRD_PROTOCOL_VERSION, attrd_cluster->priv->node_name);
 
     attrd_send_message(NULL, attrd_op, false); /* ends up at attrd_peer_message() */
 
-    free_xml(attrd_op);
+    pcmk__xml_free(attrd_op);
 }
 
 gboolean
-attrd_send_message(crm_node_t *node, xmlNode *data, bool confirm)
+attrd_send_message(pcmk__node_status_t *node, xmlNode *data, bool confirm)
 {
     const char *op = crm_element_value(data, PCMK_XA_TASK);
 
@@ -353,5 +342,5 @@ attrd_send_message(crm_node_t *node, xmlNode *data, bool confirm)
     }
 
     attrd_xml_add_writer(data);
-    return pcmk__cluster_send_message(node, crm_msg_attrd, data);
+    return pcmk__cluster_send_message(node, pcmk_ipc_attrd, data);
 }
