@@ -193,7 +193,6 @@ readCibXmlFile(const char *dir, const char *file, gboolean discard_status)
     char *filename = NULL;
     const char *name = NULL;
     const char *value = NULL;
-    const char *validation = NULL;
     const char *use_valgrind = pcmk__env_option(PCMK__ENV_VALGRIND_ENABLED);
 
     xmlNode *root = NULL;
@@ -250,8 +249,9 @@ readCibXmlFile(const char *dir, const char *file, gboolean discard_status)
         crm_warn("Continuing with an empty configuration.");
     }
 
-    if (cib_writes_enabled && use_valgrind &&
-        (crm_is_true(use_valgrind) || strstr(use_valgrind, "pacemaker-based"))) {
+    if (cib_writes_enabled && (use_valgrind != NULL)
+        && (crm_is_true(use_valgrind)
+            || (strstr(use_valgrind, PCMK__SERVER_BASED) != NULL))) {
 
         cib_writes_enabled = FALSE;
         crm_err("*** Disabling disk writes to avoid confusing Valgrind ***");
@@ -260,7 +260,7 @@ readCibXmlFile(const char *dir, const char *file, gboolean discard_status)
     status = pcmk__xe_first_child(root, PCMK_XE_STATUS, NULL, NULL);
     if (discard_status && status != NULL) {
         // Strip out the PCMK_XE_STATUS section if there is one
-        free_xml(status);
+        pcmk__xml_free(status);
         status = NULL;
     }
     if (status == NULL) {
@@ -301,25 +301,9 @@ readCibXmlFile(const char *dir, const char *file, gboolean discard_status)
         crm_log_xml_trace(root, "[on-disk]");
     }
 
-    validation = crm_element_value(root, PCMK_XA_VALIDATE_WITH);
     if (!pcmk__configured_schema_validates(root)) {
-        crm_err("CIB does not validate with %s",
-                pcmk__s(validation, "no schema specified"));
         cib_status = -pcmk_err_schema_validation;
-
-    // @COMPAT Not specifying validate-with is deprecated since 2.1.8
-    } else if (validation == NULL) {
-        pcmk__update_schema(&root, NULL, false, false);
-        validation = crm_element_value(root, PCMK_XA_VALIDATE_WITH);
-        if (validation != NULL) {
-            crm_notice("Enabling %s validation on"
-                       " the existing (sane) configuration", validation);
-        } else {
-            crm_err("CIB does not validate with any known schema");
-            cib_status = -pcmk_err_schema_validation;
-        }
     }
-
     return root;
 }
 
@@ -337,7 +321,7 @@ uninitializeCib(void)
 
     crm_debug("Deallocating the CIB.");
 
-    free_xml(tmp_cib);
+    pcmk__xml_free(tmp_cib);
 
     crm_debug("The CIB has been deallocated.");
 
@@ -354,9 +338,9 @@ activateCibXml(xmlNode * new_cib, gboolean to_disk, const char *op)
     if (new_cib) {
         xmlNode *saved_cib = the_cib;
 
-        CRM_ASSERT(new_cib != saved_cib);
+        pcmk__assert(new_cib != saved_cib);
         the_cib = new_cib;
-        free_xml(saved_cib);
+        pcmk__xml_free(saved_cib);
         if (cib_writes_enabled && cib_status == pcmk_ok && to_disk) {
             crm_debug("Triggering CIB write for %s op", op);
             mainloop_set_trigger(cib_writer);
@@ -451,7 +435,7 @@ write_cib_contents(gpointer p)
     exit_rc = cib_file_write_with_digest(cib_local, cib_root, "cib.xml");
 
     /* A nonzero exit code will cause further writes to be disabled */
-    free_xml(cib_local);
+    pcmk__xml_free(cib_local);
     if (p == NULL) {
         crm_exit_t exit_code = CRM_EX_OK;
 
@@ -472,6 +456,7 @@ write_cib_contents(gpointer p)
         }
 
         /* Use _exit() because exit() could affect the parent adversely */
+        pcmk_common_cleanup();
         _exit(exit_code);
     }
     return exit_rc;
