@@ -83,6 +83,7 @@ cib_process_readwrite(const char *op, int options, const char *section, xmlNode
 
     crm_trace("Processing \"%s\" event", op);
 
+    // @COMPAT Pacemaker Remote clients <3.0.0 may send this
     if (pcmk__str_eq(op, PCMK__CIB_REQUEST_IS_PRIMARY, pcmk__str_none)) {
         if (based_is_primary) {
             result = pcmk_ok;
@@ -117,21 +118,20 @@ void
 send_sync_request(const char *host)
 {
     xmlNode *sync_me = pcmk__xe_create(NULL, "sync-me");
-    crm_node_t *peer = NULL;
+    pcmk__node_status_t *peer = NULL;
 
     crm_info("Requesting re-sync from %s", (host? host : "all peers"));
     sync_in_progress = 1;
 
     crm_xml_add(sync_me, PCMK__XA_T, PCMK__VALUE_CIB);
     crm_xml_add(sync_me, PCMK__XA_CIB_OP, PCMK__CIB_REQUEST_SYNC_TO_ONE);
-    crm_xml_add(sync_me, PCMK__XA_CIB_DELEGATED_FROM,
-                stand_alone? "localhost" : crm_cluster->uname);
+    crm_xml_add(sync_me, PCMK__XA_CIB_DELEGATED_FROM, OUR_NODENAME);
 
     if (host != NULL) {
         peer = pcmk__get_node(0, host, NULL, pcmk__node_search_cluster_member);
     }
-    pcmk__cluster_send_message(peer, crm_msg_cib, sync_me);
-    free_xml(sync_me);
+    pcmk__cluster_send_message(peer, pcmk_ipc_based, sync_me);
+    pcmk__xml_free(sync_me);
 }
 
 int
@@ -140,7 +140,7 @@ cib_process_ping(const char *op, int options, const char *section, xmlNode * req
 {
     const char *host = crm_element_value(req, PCMK__XA_SRC);
     const char *seq = crm_element_value(req, PCMK__XA_CIB_PING_ID);
-    char *digest = calculate_xml_versioned_digest(the_cib, FALSE, TRUE, CRM_FEATURE_SET);
+    char *digest = pcmk__digest_xml(the_cib, true);
 
     xmlNode *wrapper = NULL;
 
@@ -218,6 +218,12 @@ cib_process_upgrade_server(const char *op, int options, const char *section, xml
         crm_trace("Processing \"%s\" event", op);
         original_schema = crm_element_value(existing_cib,
                                             PCMK_XA_VALIDATE_WITH);
+        if (original_schema == NULL) {
+            crm_info("Rejecting upgrade request from %s: No "
+                     PCMK_XA_VALIDATE_WITH, host);
+            return -pcmk_err_cib_corrupt;
+        }
+
         rc = pcmk__update_schema(&scratch, NULL, true, true);
         rc = pcmk_rc2legacy(rc);
         new_schema = crm_element_value(scratch, PCMK_XA_VALIDATE_WITH);
@@ -236,15 +242,9 @@ cib_process_upgrade_server(const char *op, int options, const char *section, xml
             crm_xml_add(up, PCMK__XA_CIB_CALLOPT, call_opts);
             crm_xml_add(up, PCMK__XA_CIB_CALLID, call_id);
 
-            if (cib_legacy_mode() && based_is_primary) {
-                rc = cib_process_upgrade(
-                    op, options, section, up, input, existing_cib, result_cib, answer);
+            pcmk__cluster_send_message(NULL, pcmk_ipc_based, up);
 
-            } else {
-                pcmk__cluster_send_message(NULL, crm_msg_cib, up);
-            }
-
-            free_xml(up);
+            pcmk__xml_free(up);
 
         } else if(rc == pcmk_ok) {
             rc = -pcmk_err_schema_unchanged;
@@ -252,14 +252,14 @@ cib_process_upgrade_server(const char *op, int options, const char *section, xml
 
         if (rc != pcmk_ok) {
             // Notify originating peer so it can notify its local clients
-            crm_node_t *origin = NULL;
+            pcmk__node_status_t *origin = NULL;
 
             origin = pcmk__search_node_caches(0, host,
                                               pcmk__node_search_cluster_member);
 
             crm_info("Rejecting upgrade request from %s: %s "
-                     CRM_XS " rc=%d peer=%s", host, pcmk_strerror(rc), rc,
-                     (origin? origin->uname : "lost"));
+                     QB_XS " rc=%d peer=%s", host, pcmk_strerror(rc), rc,
+                     (origin? origin->name : "lost"));
 
             if (origin) {
                 xmlNode *up = pcmk__xe_create(NULL, __func__);
@@ -272,13 +272,13 @@ cib_process_upgrade_server(const char *op, int options, const char *section, xml
                 crm_xml_add(up, PCMK__XA_CIB_CALLOPT, call_opts);
                 crm_xml_add(up, PCMK__XA_CIB_CALLID, call_id);
                 crm_xml_add_int(up, PCMK__XA_CIB_UPGRADE_RC, rc);
-                if (!pcmk__cluster_send_message(origin, crm_msg_cib, up)) {
+                if (!pcmk__cluster_send_message(origin, pcmk_ipc_based, up)) {
                     crm_warn("Could not send CIB upgrade result to %s", host);
                 }
-                free_xml(up);
+                pcmk__xml_free(up);
             }
         }
-        free_xml(scratch);
+        pcmk__xml_free(scratch);
     }
     return rc;
 }
@@ -331,7 +331,7 @@ cib_server_process_diff(const char *op, int options, const char *section, xmlNod
               (based_is_primary? "primary": "secondary"));
 
     if ((rc == -pcmk_err_diff_resync) && !based_is_primary) {
-        free_xml(*result_cib);
+        pcmk__xml_free(*result_cib);
         *result_cib = NULL;
         send_sync_request(NULL);
 
@@ -340,15 +340,6 @@ cib_server_process_diff(const char *op, int options, const char *section, xmlNod
         if (options & cib_force_diff) {
             crm_warn("Not requesting full refresh in R/W mode");
         }
-
-    } else if ((rc != pcmk_ok) && !based_is_primary && cib_legacy_mode()) {
-        crm_warn("Requesting full CIB refresh because update failed: %s"
-                 CRM_XS " rc=%d", pcmk_strerror(rc), rc);
-
-        pcmk__log_xml_patchset(LOG_INFO, input);
-        free_xml(*result_cib);
-        *result_cib = NULL;
-        send_sync_request(NULL);
     }
 
     return rc;
@@ -368,7 +359,9 @@ cib_process_replace_svr(const char *op, int options, const char *section, xmlNod
     return rc;
 }
 
-// @COMPAT: Remove when PCMK__CIB_REQUEST_ABS_DELETE is removed
+/* @COMPAT: Remove when PCMK__CIB_REQUEST_ABS_DELETE is removed
+ * (At least external client code <3.0.0 can send it)
+ */
 int
 cib_process_delete_absolute(const char *op, int options, const char *section, xmlNode * req,
                             xmlNode * input, xmlNode * existing_cib, xmlNode ** result_cib,
@@ -391,8 +384,6 @@ cib_msg_copy(xmlNode *msg)
         PCMK__XA_CIB_HOST,
         PCMK__XA_CIB_RC,
         PCMK__XA_CIB_DELEGATED_FROM,
-        PCMK__XA_CIB_OBJECT,
-        PCMK__XA_CIB_OBJECT_TYPE,
         PCMK__XA_CIB_UPDATE,
         PCMK__XA_CIB_CLIENTNAME,
         PCMK__XA_CIB_USER,
@@ -421,7 +412,7 @@ sync_our_cib(xmlNode * request, gboolean all)
     char *digest = NULL;
     const char *host = crm_element_value(request, PCMK__XA_SRC);
     const char *op = crm_element_value(request, PCMK__XA_CIB_OP);
-    crm_node_t *peer = NULL;
+    pcmk__node_status_t *peer = NULL;
     xmlNode *replace_request = NULL;
     xmlNode *wrapper = NULL;
 
@@ -447,7 +438,7 @@ sync_our_cib(xmlNode * request, gboolean all)
     pcmk__xe_set_bool_attr(replace_request, PCMK__XA_CIB_UPDATE, true);
 
     crm_xml_add(replace_request, PCMK_XA_CRM_FEATURE_SET, CRM_FEATURE_SET);
-    digest = calculate_xml_versioned_digest(the_cib, FALSE, TRUE, CRM_FEATURE_SET);
+    digest = pcmk__digest_xml(the_cib, true);
     crm_xml_add(replace_request, PCMK__XA_DIGEST, digest);
 
     wrapper = pcmk__xe_create(replace_request, PCMK__XE_CIB_CALLDATA);
@@ -456,10 +447,10 @@ sync_our_cib(xmlNode * request, gboolean all)
     if (!all) {
         peer = pcmk__get_node(0, host, NULL, pcmk__node_search_cluster_member);
     }
-    if (!pcmk__cluster_send_message(peer, crm_msg_cib, replace_request)) {
+    if (!pcmk__cluster_send_message(peer, pcmk_ipc_based, replace_request)) {
         result = -ENOTCONN;
     }
-    free_xml(replace_request);
+    pcmk__xml_free(replace_request);
     free(digest);
     return result;
 }
