@@ -82,14 +82,12 @@ cib_notify_send_one(gpointer key, gpointer value, gpointer user_data)
                 rc = pcmk__ipc_send_iov(client, update->iov,
                                         crm_ipc_server_event);
                 if (rc != pcmk_rc_ok) {
-                    crm_warn("Could not notify client %s: %s " CRM_XS " id=%s",
+                    crm_warn("Could not notify client %s: %s " QB_XS " id=%s",
                              pcmk__client_name(client), pcmk_rc_str(rc),
                              client->id);
                 }
                 break;
-#ifdef HAVE_GNUTLS_GNUTLS_H
             case pcmk__client_tls:
-#endif
             case pcmk__client_tcp:
                 crm_debug("Sent %s notification to client %s (id %s)",
                           type, pcmk__client_name(client), client->id);
@@ -97,7 +95,7 @@ cib_notify_send_one(gpointer key, gpointer value, gpointer user_data)
                 break;
             default:
                 crm_err("Unknown transport for client %s "
-                        CRM_XS " flags=%#016" PRIx64,
+                        QB_XS " flags=%#016" PRIx64,
                         pcmk__client_name(client), client->flags);
         }
     }
@@ -119,23 +117,12 @@ cib_notify_send(const xmlNode *xml)
         pcmk__foreach_ipc_client(cib_notify_send_one, &update);
 
     } else {
-        crm_notice("Could not notify clients: %s " CRM_XS " rc=%d",
+        crm_notice("Could not notify clients: %s " QB_XS " rc=%d",
                    pcmk_rc_str(rc), rc);
     }
     pcmk_free_ipc_event(iov);
 }
 
-static void
-attach_cib_generation(xmlNode *msg)
-{
-    xmlNode *wrapper = pcmk__xe_create(msg, PCMK__XE_CIB_GENERATION);
-    xmlNode *generation = pcmk__xe_create(wrapper, PCMK__XE_GENERATION_TUPLE);
-
-    if (the_cib != NULL) {
-        pcmk__xe_copy_attrs(generation, the_cib, pcmk__xaf_none);
-    }
-}
-
 void
 cib_diff_notify(const char *op, int result, const char *call_id,
                 const char *client_id, const char *client_name,
@@ -153,7 +140,6 @@ cib_diff_notify(const char *op, int result, const char *call_id,
 
     xmlNode *update_msg = NULL;
     xmlNode *wrapper = NULL;
-    const char *type = NULL;
 
     if (diff == NULL) {
         return;
@@ -203,30 +189,10 @@ cib_diff_notify(const char *op, int result, const char *call_id,
     crm_xml_add(update_msg, PCMK__XA_SRC, origin);
     crm_xml_add_int(update_msg, PCMK__XA_CIB_RC, result);
 
-    // @COMPAT Unused internally, drop at 3.0.0
-    if (update != NULL) {
-        type = (const char *) update->name;
-        crm_trace("Setting type to update->name: %s", type);
-    } else {
-        type = (const char *) diff->name;
-        crm_trace("Setting type to new_obj->name: %s", type);
-    }
-
-    // @COMPAT Unused internally, drop at 3.0.0
-    crm_xml_add(update_msg, PCMK__XA_CIB_OBJECT, pcmk__xe_id(diff));
-    crm_xml_add(update_msg, PCMK__XA_CIB_OBJECT_TYPE, type);
-    attach_cib_generation(update_msg);
-
-    // @COMPAT Unused internally, drop at 3.0.0
-    if (update != NULL) {
-        wrapper = pcmk__xe_create(update_msg, PCMK__XE_CIB_UPDATE);
-        pcmk__xml_copy(wrapper, update);
-    }
-
     wrapper = pcmk__xe_create(update_msg, PCMK__XE_CIB_UPDATE_RESULT);
     pcmk__xml_copy(wrapper, diff);
 
     crm_log_xml_trace(update_msg, "diff-notify");
     cib_notify_send(update_msg);
-    free_xml(update_msg);
+    pcmk__xml_free(update_msg);
 }
