@@ -48,7 +48,6 @@ int remote_fd = 0;
 int remote_tls_fd = 0;
 
 GHashTable *config_hash = NULL;
-GHashTable *local_notify_queue = NULL;
 
 static void cib_init(void);
 void cib_shutdown(int nsig);
@@ -127,11 +126,13 @@ setup_stand_alone(GError **error)
 
 /* @COMPAT Deprecated since 2.1.8. Use pcmk_list_cluster_options() or
  * crm_attribute --list-options=cluster instead of querying daemon metadata.
+ *
+ * NOTE: pcs (as of at least 0.11.8) uses this
  */
 static int
 based_metadata(pcmk__output_t *out)
 {
-    return pcmk__daemon_metadata(out, "pacemaker-based",
+    return pcmk__daemon_metadata(out, PCMK__SERVER_BASED,
                                  "Cluster Information Base manager options",
                                  "Cluster options used by Pacemaker's Cluster "
                                  "Information Base manager",
@@ -225,7 +226,7 @@ main(int argc, char **argv)
         goto done;
     }
 
-    pcmk__cli_init_logging("pacemaker-based", args->verbosity);
+    pcmk__cli_init_logging(PCMK__SERVER_BASED, args->verbosity);
     crm_log_init(NULL, LOG_INFO, TRUE, FALSE, argc, argv, FALSE);
     crm_notice("Starting Pacemaker CIB manager");
 
@@ -242,7 +243,8 @@ main(int argc, char **argv)
         /* IPC end-point already up */
         crm_ipc_close(old_instance);
         crm_ipc_destroy(old_instance);
-        crm_err("pacemaker-based is already active, aborting startup");
+        crm_crit("Aborting start-up because another CIB manager instance is "
+                 "already active");
         goto done;
     } else {
         /* not up or not authentic, we'll proceed either way */
@@ -293,10 +295,6 @@ done:
 
     pcmk__cluster_destroy_node_caches();
 
-    if (local_notify_queue != NULL) {
-        g_hash_table_destroy(local_notify_queue);
-    }
-
     if (config_hash != NULL) {
         g_hash_table_destroy(config_hash);
     }
@@ -320,26 +318,24 @@ cib_cs_dispatch(cpg_handle_t handle,
                  const struct cpg_name *groupName,
                  uint32_t nodeid, uint32_t pid, void *msg, size_t msg_len)
 {
-    uint32_t kind = 0;
     xmlNode *xml = NULL;
     const char *from = NULL;
-    char *data = pcmk__cpg_message_data(handle, nodeid, pid, msg, &kind, &from);
+    char *data = pcmk__cpg_message_data(handle, nodeid, pid, msg, &from);
 
     if(data == NULL) {
         return;
     }
-    if (kind == crm_class_cluster) {
-        xml = pcmk__xml_parse(data);
-        if (xml == NULL) {
-            crm_err("Invalid XML: '%.120s'", data);
-            free(data);
-            return;
-        }
-        crm_xml_add(xml, PCMK__XA_SRC, from);
-        cib_peer_callback(xml, NULL);
-    }
 
-    free_xml(xml);
+    xml = pcmk__xml_parse(data);
+    if (xml == NULL) {
+        crm_err("Invalid XML: '%.120s'", data);
+        free(data);
+        return;
+    }
+    crm_xml_add(xml, PCMK__XA_SRC, from);
+    cib_peer_callback(xml, NULL);
+
+    pcmk__xml_free(xml);
     free(data);
 }
 
@@ -356,25 +352,12 @@ cib_cs_destroy(gpointer user_data)
 #endif
 
 static void
-cib_peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *data)
+cib_peer_update_callback(enum pcmk__node_update type,
+                         pcmk__node_status_t *node, const void *data)
 {
     switch (type) {
-        case crm_status_processes:
-            if (cib_legacy_mode()
-                && !pcmk_is_set(node->processes, crm_get_cluster_proc())) {
-
-                uint32_t old = data? *(const uint32_t *)data : 0;
-
-                if ((node->processes ^ old) & crm_proc_cpg) {
-                    crm_info("Attempting to disable legacy mode after %s left the cluster",
-                             node->uname);
-                    legacy_mode = FALSE;
-                }
-            }
-            break;
-
-        case crm_status_uname:
-        case crm_status_nstate:
+        case pcmk__node_update_name:
+        case pcmk__node_update_state:
             if (cib_shutdown_flag && (pcmk__cluster_num_active_nodes() < 2)
                 && (pcmk__ipc_client_count() == 0)) {
 
@@ -382,6 +365,9 @@ cib_peer_update_callback(enum crm_status_type type, crm_node_t * node, const voi
                 terminate_cib(__func__, -1);
             }
             break;
+
+        default:
+            break;
     }
 }
 
