@@ -58,8 +58,8 @@ do_cib_updated(const char *event, xmlNode * msg)
         return;
     }
 
-    if (cib__element_in_patchset(patchset, PCMK_XE_ALERTS)
-        || cib__element_in_patchset(patchset, PCMK_XE_CRM_CONFIG)) {
+    if (pcmk__cib_element_in_patchset(patchset, PCMK_XE_ALERTS)
+        || pcmk__cib_element_in_patchset(patchset, PCMK_XE_CRM_CONFIG)) {
 
         controld_trigger_config();
     }
@@ -75,8 +75,8 @@ do_cib_updated(const char *event, xmlNode * msg)
         return;
     }
 
-    if (cib__element_in_patchset(patchset, PCMK_XE_NODES)
-        || cib__element_in_patchset(patchset, PCMK_XE_STATUS)) {
+    if (pcmk__cib_element_in_patchset(patchset, PCMK_XE_NODES)
+        || pcmk__cib_element_in_patchset(patchset, PCMK_XE_STATUS)) {
 
         /* An unsafe client modified the PCMK_XE_NODES or PCMK_XE_STATUS
          * section. Ensure the node list is up-to-date, and start the join
@@ -99,7 +99,7 @@ controld_disconnect_cib_manager(void)
 {
     cib_t *cib_conn = controld_globals.cib_conn;
 
-    CRM_ASSERT(cib_conn != NULL);
+    pcmk__assert(cib_conn != NULL);
 
     crm_debug("Disconnecting from the CIB manager");
 
@@ -110,8 +110,7 @@ controld_disconnect_cib_manager(void)
     cib_free_callbacks(cib_conn);
 
     if (cib_conn->state != cib_disconnected) {
-        cib_conn->cmds->set_secondary(cib_conn,
-                                      cib_scope_local|cib_discard_reply);
+        cib_conn->cmds->set_secondary(cib_conn, cib_discard_reply);
         cib_conn->cmds->signoff(cib_conn);
     }
 }
@@ -132,7 +131,7 @@ do_cib_control(long long action,
 
     int rc = pcmk_ok;
 
-    CRM_ASSERT(cib_conn != NULL);
+    pcmk__assert(cib_conn != NULL);
 
     if (pcmk_is_set(action, A_CIB_STOP)) {
         if ((cib_conn->state != cib_disconnected)
@@ -156,13 +155,13 @@ do_cib_control(long long action,
         return;
     }
 
-    rc = cib_conn->cmds->signon(cib_conn, CRM_SYSTEM_CRMD,
+    rc = cib_conn->cmds->signon(cib_conn, crm_system_name,
                                 cib_command_nonblocking);
 
     if (rc != pcmk_ok) {
         // A short wait that usually avoids stalling the FSA
         sleep(1);
-        rc = cib_conn->cmds->signon(cib_conn, CRM_SYSTEM_CRMD,
+        rc = cib_conn->cmds->signon(cib_conn, crm_system_name,
                                     cib_command_nonblocking);
     }
 
@@ -213,23 +212,13 @@ do_cib_control(long long action,
 unsigned int
 cib_op_timeout(void)
 {
-    // @COMPAT: Drop env_timeout at 3.0.0
-    static int env_timeout = -1;
-    unsigned int calculated_timeout = 0;
+    unsigned int calculated_timeout = 10U * (pcmk__cluster_num_active_nodes()
+                                             + pcmk__cluster_num_remote_nodes()
+                                             + 1U);
 
-    if (env_timeout == -1) {
-        const char *env = pcmk__env_option(PCMK__ENV_CIB_TIMEOUT);
-
-        pcmk__scan_min_int(env, &env_timeout, MIN_CIB_OP_TIMEOUT);
-        crm_trace("Minimum CIB op timeout: %ds (environment: %s)",
-                  env_timeout, (env? env : "none"));
-    }
-
-    calculated_timeout = 10U * (1U
-                                + pcmk__cluster_num_active_nodes()
-                                + pcmk__cluster_num_remote_nodes());
-    calculated_timeout = QB_MAX(calculated_timeout, env_timeout);
-    crm_trace("Calculated timeout: %us", calculated_timeout);
+    calculated_timeout = QB_MAX(calculated_timeout, MIN_CIB_OP_TIMEOUT);
+    crm_trace("Calculated timeout: %s",
+              pcmk__readable_interval(calculated_timeout * 1000));
 
     if (controld_globals.cib_conn) {
         controld_globals.cib_conn->call_timeout = calculated_timeout;
@@ -252,7 +241,7 @@ crmd_cib_smart_opt(void)
         || (controld_globals.fsa_state == S_PENDING)) {
         crm_info("Sending update to local CIB in state: %s",
                  fsa_state2string(controld_globals.fsa_state));
-        cib__set_call_options(call_opt, "update", cib_scope_local);
+        cib__set_call_options(call_opt, "update", cib_none);
     }
     return call_opt;
 }
@@ -266,7 +255,7 @@ cib_delete_callback(xmlNode *msg, int call_id, int rc, xmlNode *output,
     if (rc == 0) {
         crm_debug("Deletion of %s (via CIB call %d) succeeded", desc, call_id);
     } else {
-        crm_warn("Deletion of %s (via CIB call %d) failed: %s " CRM_XS " rc=%d",
+        crm_warn("Deletion of %s (via CIB call %d) failed: %s " QB_XS " rc=%d",
                  desc, call_id, pcmk_strerror(rc), rc);
     }
 }
@@ -345,7 +334,7 @@ controld_node_state_deletion_strings(const char *uname,
             break;
         default:
             // We called this function incorrectly
-            CRM_ASSERT(false);
+            pcmk__assert(false);
             break;
     }
 
@@ -371,7 +360,7 @@ controld_delete_node_state(const char *uname, enum controld_section_e section,
     char *desc = NULL;
     int cib_rc = pcmk_ok;
 
-    CRM_ASSERT((uname != NULL) && (cib != NULL));
+    pcmk__assert((uname != NULL) && (cib != NULL));
 
     controld_node_state_deletion_strings(uname, section, &xpath, &desc);
 
@@ -379,7 +368,7 @@ controld_delete_node_state(const char *uname, enum controld_section_e section,
                           cib_xpath|cib_multiple);
     cib_rc = cib->cmds->remove(cib, xpath, NULL, options);
     fsa_register_cib_callback(cib_rc, desc, cib_delete_callback);
-    crm_info("Deleting %s (via CIB call %d) " CRM_XS " xpath=%s",
+    crm_info("Deleting %s (via CIB call %d) " QB_XS " xpath=%s",
              desc, cib_rc, xpath);
 
     // CIB library handles freeing desc
@@ -433,7 +422,7 @@ controld_delete_resource_history(const char *rsc_id, const char *node,
     if (rc < 0) {
         rc = pcmk_legacy2rc(rc);
         crm_err("Could not delete resource status of %s on %s%s%s: %s "
-                CRM_XS " rc=%d", rsc_id, node,
+                QB_XS " rc=%d", rsc_id, node,
                 (user_name? " for user " : ""), (user_name? user_name : ""),
                 pcmk_rc_str(rc), rc);
         free(desc);
@@ -450,7 +439,7 @@ controld_delete_resource_history(const char *rsc_id, const char *node,
         free(desc);
 
     } else {
-        crm_info("Clearing %s (via CIB call %d) " CRM_XS " xpath=%s",
+        crm_info("Clearing %s (via CIB call %d) " QB_XS " xpath=%s",
                  desc, rc, xpath);
         fsa_register_cib_callback(rc, desc, cib_delete_callback);
         // CIB library handles freeing desc
@@ -477,7 +466,7 @@ controld_delete_resource_history(const char *rsc_id, const char *node,
  *       secure digest, we want parameters that *are* marked private for the
  *       string, but parameters that are *not* marked private for the XML.
  * \note It is the caller's responsibility to free the string return value with
- *       \p g_string_free() and the XML result with \p free_xml().
+ *       \p g_string_free() and the XML result with \p pcmk__xml_free().
  */
 static GString *
 build_parameter_list(const lrmd_event_data_t *op,
@@ -590,7 +579,7 @@ append_restart_list(lrmd_event_data_t *op, struct ra_metadata_s *metadata,
         return;
     }
 
-    digest = calculate_operation_digest(restart, version);
+    digest = pcmk__digest_operation(restart);
     /* Add PCMK__XA_OP_FORCE_RESTART and PCMK__XA_OP_RESTART_DIGEST to indicate
      * the resource supports reload, no matter if it actually supports any
      * reloadable parameters
@@ -608,7 +597,7 @@ append_restart_list(lrmd_event_data_t *op, struct ra_metadata_s *metadata,
     if (list != NULL) {
         g_string_free(list, TRUE);
     }
-    free_xml(restart);
+    pcmk__xml_free(restart);
     free(digest);
 }
 
@@ -628,7 +617,7 @@ append_secure_list(lrmd_event_data_t *op, struct ra_metadata_s *metadata,
     list = build_parameter_list(op, metadata, ra_param_private, &secure);
 
     if (list != NULL) {
-        digest = calculate_operation_digest(secure, version);
+        digest = pcmk__digest_operation(secure);
         crm_xml_add(update, PCMK__XA_OP_SECURE_PARAMS,
                     (const char *) list->str);
         crm_xml_add(update, PCMK__XA_OP_SECURE_DIGEST, digest);
@@ -639,7 +628,7 @@ append_secure_list(lrmd_event_data_t *op, struct ra_metadata_s *metadata,
         crm_trace("%s: no secure parameters", op->rsc_id);
     }
 
-    free_xml(secure);
+    pcmk__xml_free(secure);
     free(digest);
 }
 
@@ -675,7 +664,8 @@ controld_add_resource_history_xml_as(const char *func, xmlNode *parent,
     CRM_CHECK(caller_version != NULL, caller_version = CRM_FEATURE_SET);
 
     xml_op = pcmk__create_history_xml(parent, op, caller_version, target_rc,
-                                      controld_globals.our_nodename, func);
+                                      controld_globals.cluster->priv->node_name,
+                                      func);
     if (xml_op == NULL) {
         return;
     }
@@ -688,7 +678,7 @@ controld_add_resource_history_xml_as(const char *func, xmlNode *parent,
         return;
     }
 
-    lrm_state = lrm_state_find(node_name);
+    lrm_state = controld_get_executor_state(node_name, false);
     if (lrm_state == NULL) {
         crm_warn("Cannot calculate digests for operation " PCMK__OP_FMT
                  " because we have no connection to executor for %s",
@@ -744,6 +734,11 @@ controld_record_pending_op(const char *node_name, const lrmd_rsc_info_t *rsc,
     // Check action's PCMK_META_RECORD_PENDING meta-attribute (defaults to true)
     record_pending = crm_meta_value(op->params, PCMK_META_RECORD_PENDING);
     if ((record_pending != NULL) && !crm_is_true(record_pending)) {
+        pcmk__warn_once(pcmk__wo_record_pending,
+                        "The " PCMK_META_RECORD_PENDING " option (for example, "
+                        "for the %s resource's %s operation) is deprecated and "
+                        "will be removed in a future release",
+                        rsc->id, op->op_type);
         return false;
     }
 
@@ -773,9 +768,9 @@ cib_rsc_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void *use
         default:
             if (call_id > 0) {
                 crm_warn("Resource history update %d failed: %s "
-                         CRM_XS " rc=%d", call_id, pcmk_strerror(rc), rc);
+                         QB_XS " rc=%d", call_id, pcmk_strerror(rc), rc);
             } else {
-                crm_warn("Resource history update failed: %s " CRM_XS " rc=%d",
+                crm_warn("Resource history update failed: %s " QB_XS " rc=%d",
                          pcmk_strerror(rc), rc);
             }
     }
@@ -827,7 +822,7 @@ controld_update_cib(const char *section, xmlNode *data, int options,
     cib_t *cib = controld_globals.cib_conn;
     int cib_rc = -ENOTCONN;
 
-    CRM_ASSERT(data != NULL);
+    pcmk__assert(data != NULL);
 
     if (cib != NULL) {
         cib_rc = cib->cmds->modify(cib, section, data, options);
@@ -893,8 +888,7 @@ controld_update_resource_history(const char *node_name,
 
     //   <node_state ...>
     xml = pcmk__xe_create(update, PCMK__XE_NODE_STATE);
-    if (pcmk__str_eq(node_name, controld_globals.our_nodename,
-                     pcmk__str_casei)) {
+    if (controld_is_local_node(node_name)) {
         node_id = controld_globals.our_uuid;
     } else {
         node_id = node_name;
@@ -945,7 +939,7 @@ controld_update_resource_history(const char *node_name,
      */
     crm_log_xml_trace(update, __func__);
     controld_update_cib(PCMK_XE_STATUS, update, call_opt, cib_rsc_callback);
-    free_xml(update);
+    pcmk__xml_free(update);
 }
 
 /*!
@@ -979,7 +973,7 @@ controld_delete_action_history(const lrmd_event_data_t *op)
     controld_globals.cib_conn->cmds->remove(controld_globals.cib_conn,
                                             PCMK_XE_STATUS, xml_top, cib_none);
     crm_log_xml_trace(xml_top, "op:cancel");
-    free_xml(xml_top);
+    pcmk__xml_free(xml_top);
 }
 
 /* Define xpath to find LRM resource history entry by node and resource */
