@@ -41,14 +41,13 @@ do_ha_control(long long action,
               enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
     gboolean registered = FALSE;
-    static pcmk_cluster_t *cluster = NULL;
 
-    if (cluster == NULL) {
-        cluster = pcmk_cluster_new();
+    if (controld_globals.cluster == NULL) {
+        controld_globals.cluster = pcmk_cluster_new();
     }
 
     if (action & A_HA_DISCONNECT) {
-        pcmk_cluster_disconnect(cluster);
+        pcmk_cluster_disconnect(controld_globals.cluster);
         crm_info("Disconnected from the cluster");
 
         controld_set_fsa_input_flags(R_HA_DISCONNECTED);
@@ -60,15 +59,20 @@ do_ha_control(long long action,
 
 #if SUPPORT_COROSYNC
         if (pcmk_get_cluster_layer() == pcmk_cluster_layer_corosync) {
-            registered = crm_connect_corosync(cluster);
+            registered = crm_connect_corosync(controld_globals.cluster);
         }
 #endif // SUPPORT_COROSYNC
 
         if (registered) {
-            controld_election_init(cluster->uname);
-            controld_globals.our_nodename = cluster->uname;
-            controld_globals.our_uuid = cluster->uuid;
-            if(cluster->uuid == NULL) {
+            pcmk__node_status_t *node = controld_get_local_node_status();
+
+            controld_election_init();
+
+            free(controld_globals.our_uuid);
+            controld_globals.our_uuid =
+                pcmk__str_copy(pcmk__cluster_node_uuid(node));
+
+            if (controld_globals.our_uuid == NULL) {
                 crm_err("Could not obtain local uuid");
                 registered = FALSE;
             }
@@ -115,19 +119,20 @@ do_shutdown_req(long long action,
     //controld_set_fsa_input_flags(R_STAYDOWN);
     crm_info("Sending shutdown request to all peers (DC is %s)",
              pcmk__s(controld_globals.dc_name, "not set"));
-    msg = create_request(CRM_OP_SHUTDOWN_REQ, NULL, NULL, CRM_SYSTEM_CRMD, CRM_SYSTEM_CRMD, NULL);
+    msg = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_CRMD, NULL,
+                            CRM_SYSTEM_CRMD, CRM_OP_SHUTDOWN_REQ, NULL);
 
-    if (!pcmk__cluster_send_message(NULL, crm_msg_crmd, msg)) {
+    if (!pcmk__cluster_send_message(NULL, pcmk_ipc_controld, msg)) {
         register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
     }
-    free_xml(msg);
+    pcmk__xml_free(msg);
 }
 
 void
 crmd_fast_exit(crm_exit_t exit_code)
 {
     if (pcmk_is_set(controld_globals.fsa_input_register, R_STAYDOWN)) {
-        crm_warn("Inhibiting respawn "CRM_XS" remapping exit code %d to %d",
+        crm_warn("Inhibiting respawn " QB_XS " remapping exit code %d to %d",
                  exit_code, CRM_EX_FATAL);
         exit_code = CRM_EX_FATAL;
 
@@ -222,7 +227,7 @@ crmd_exit(crm_exit_t exit_code)
     controld_globals.fsa_message_queue = NULL;
 
     controld_free_node_pending_timers();
-    controld_election_fini();
+    election_reset(controld_globals.cluster); // Stop any election timer
 
     /* Tear down the CIB manager connection, but don't free it yet -- it could
      * be used when we drain the mainloop later.
@@ -247,9 +252,6 @@ crmd_exit(crm_exit_t exit_code)
     te_cleanup_stonith_history_sync(NULL, TRUE);
     controld_free_sched_timer();
 
-    free(controld_globals.our_nodename);
-    controld_globals.our_nodename = NULL;
-
     free(controld_globals.our_uuid);
     controld_globals.our_uuid = NULL;
 
@@ -310,6 +312,9 @@ crmd_exit(crm_exit_t exit_code)
 
     throttle_fini();
 
+    pcmk_cluster_free(controld_globals.cluster);
+    controld_globals.cluster = NULL;
+
     /* Graceful */
     crm_trace("Done preparing for exit with status %d (%s)",
               exit_code, crm_exit_str(exit_code));
@@ -388,7 +393,7 @@ dispatch_controller_ipc(qb_ipcs_connection_t * c, void *data, size_t size)
     pcmk__ipc_send_ack(client, id, flags, PCMK__XE_ACK, NULL,
                        CRM_EX_INDETERMINATE);
 
-    CRM_ASSERT(client->user != NULL);
+    pcmk__assert(client->user != NULL);
     pcmk__update_acl_user(msg, PCMK__XA_CRM_USER, client->user);
 
     crm_xml_add(msg, PCMK__XA_CRM_SYS_FROM, client->id);
@@ -399,7 +404,7 @@ dispatch_controller_ipc(qb_ipcs_connection_t * c, void *data, size_t size)
     }
 
     controld_trigger_fsa();
-    free_xml(msg);
+    pcmk__xml_free(msg);
     return 0;
 }
 
@@ -575,9 +580,10 @@ config_query_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
     controld_verify_stonith_watchdog_timeout(value);
 
     value = g_hash_table_lookup(config_hash, PCMK_OPT_NO_QUORUM_POLICY);
-    if (pcmk__str_eq(value, PCMK_VALUE_FENCE_LEGACY, pcmk__str_casei)
+    if (pcmk__strcase_any_of(value, PCMK_VALUE_FENCE, PCMK_VALUE_FENCE_LEGACY,
+                             NULL)
         && (pcmk__locate_sbd() != 0)) {
-        controld_set_global_flags(controld_no_quorum_suicide);
+        controld_set_global_flags(controld_no_quorum_panic);
     }
 
     value = g_hash_table_lookup(config_hash, PCMK_OPT_SHUTDOWN_LOCK);
@@ -638,7 +644,7 @@ crm_read_options(gpointer user_data)
     int call_id = cib_conn->cmds->query(cib_conn,
                                         "//" PCMK_XE_CRM_CONFIG
                                         " | //" PCMK_XE_ALERTS,
-                                        NULL, cib_xpath|cib_scope_local);
+                                        NULL, cib_xpath);
 
     fsa_register_cib_callback(call_id, NULL, config_query_callback);
     crm_trace("Querying the CIB... call %d", call_id);
