@@ -275,7 +275,8 @@ send_task_ok_ack(const lrm_state_t *lrm_state, const ha_msg_input_t *input,
 static inline const char *
 op_node_name(lrmd_event_data_t *op)
 {
-    return pcmk__s(op->remote_nodename, controld_globals.our_nodename);
+    return pcmk__s(op->remote_nodename,
+                   controld_globals.cluster->priv->node_name);
 }
 
 void
@@ -294,9 +295,10 @@ lrm_op_callback(lrmd_event_data_t * op)
 
         case lrmd_event_exec_complete:
             {
-                lrm_state_t *lrm_state = lrm_state_find(op_node_name(op));
+                lrm_state_t *lrm_state =
+                    controld_get_executor_state(op_node_name(op), false);
 
-                CRM_ASSERT(lrm_state != NULL);
+                pcmk__assert(lrm_state != NULL);
                 process_lrm_event(lrm_state, op, NULL, NULL);
             }
             break;
@@ -355,10 +357,10 @@ do_lrm_control(long long action,
 
     lrm_state_t *lrm_state = NULL;
 
-    if (controld_globals.our_nodename == NULL) {
-        return; /* Nothing to do */
+    if (controld_globals.cluster->priv->node_name == NULL) {
+        return; // Shouldn't be possible
     }
-    lrm_state = lrm_state_find_or_create(controld_globals.our_nodename);
+    lrm_state = controld_get_executor_state(NULL, true);
     if (lrm_state == NULL) {
         register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
         return;
@@ -574,12 +576,11 @@ controld_query_executor_state(void)
     xmlNode *xml_state = NULL;
     xmlNode *xml_data = NULL;
     xmlNode *rsc_list = NULL;
-    crm_node_t *peer = NULL;
-    lrm_state_t *lrm_state = lrm_state_find(controld_globals.our_nodename);
+    pcmk__node_status_t *peer = NULL;
+    lrm_state_t *lrm_state = controld_get_executor_state(NULL, false);
 
     if (!lrm_state) {
-        crm_err("Could not find executor state for node %s",
-                controld_globals.our_nodename);
+        crm_err("Could not get executor state for local node");
         return NULL;
     }
 
@@ -594,7 +595,7 @@ controld_query_executor_state(void)
     }
 
     xml_data = pcmk__xe_create(xml_state, PCMK__XE_LRM);
-    crm_xml_add(xml_data, PCMK_XA_ID, peer->uuid);
+    crm_xml_add(xml_data, PCMK_XA_ID, peer->xml_id);
     rsc_list = pcmk__xe_create(xml_data, PCMK__XE_LRM_RESOURCES);
 
     /* Build a list of active (not always running) resources */
@@ -770,7 +771,7 @@ void
 lrm_clear_last_failure(const char *rsc_id, const char *node_name,
                        const char *operation, guint interval_ms)
 {
-    lrm_state_t *lrm_state = lrm_state_find(node_name);
+    lrm_state_t *lrm_state = controld_get_executor_state(node_name, false);
 
     if (lrm_state == NULL) {
         return;
@@ -943,7 +944,7 @@ get_lrm_resource(lrm_state_t *lrm_state, const xmlNode *rsc_xml,
             fsa_data_t *msg_data = NULL;
 
             crm_err("Could not register resource %s with the executor on %s: %s "
-                    CRM_XS " rc=%d",
+                    QB_XS " rc=%d",
                     id, lrm_state->node_name, pcmk_strerror(rc), rc);
 
             /* Register this as an internal error if this involves the local
@@ -991,7 +992,7 @@ delete_resource(lrm_state_t *lrm_state, const char *id, lrmd_rsc_info_t *rsc,
         return;
     } else {
         crm_warn("Could not delete '%s' from executor for %s%s%s: %s "
-                 CRM_XS " rc=%d", id, sys, (user? " as " : ""),
+                 QB_XS " rc=%d", id, sys, (user? " as " : ""),
                  (user? user : ""), pcmk_strerror(rc), rc);
     }
 
@@ -1051,7 +1052,8 @@ force_reprobe(lrm_state_t *lrm_state, const char *from_sys,
             unregister = false;
 
             if (reprobe_all_nodes) {
-                lrm_state_t *remote_lrm_state = lrm_state_find(entry->id);
+                lrm_state_t *remote_lrm_state =
+                    controld_get_executor_state(entry->id, false);
 
                 if (remote_lrm_state != NULL) {
                     /* If reprobing all nodes, be sure to reprobe the remote
@@ -1072,10 +1074,7 @@ force_reprobe(lrm_state_t *lrm_state, const char *from_sys,
 
     /* Now delete the copy in the CIB */
     controld_delete_node_state(lrm_state->node_name, controld_section_lrm,
-                               cib_scope_local);
-
-    // @COMPAT DCs < 1.1.14 need this deleted (in case it was explicitly false)
-    update_attrd(lrm_state->node_name, CRM_OP_PROBED, NULL, user_name, is_remote_node);
+                               cib_none);
 }
 
 /*!
@@ -1152,7 +1151,7 @@ lrm_op_target(const xmlNode *xml)
         target = crm_element_value(xml, PCMK__META_ON_NODE);
     }
     if (target == NULL) {
-        target = controld_globals.our_nodename;
+        target = controld_globals.cluster->priv->node_name;
     }
     return target;
 }
@@ -1215,7 +1214,7 @@ fail_lrm_resource(xmlNode *xml, lrm_state_t *lrm_state, const char *user_name,
 }
 
 static void
-handle_reprobe_op(lrm_state_t *lrm_state, const char *from_sys,
+handle_reprobe_op(lrm_state_t *lrm_state, xmlNode *msg, const char *from_sys,
                   const char *from_host, const char *user_name,
                   gboolean is_remote_node, bool reprobe_all_nodes)
 {
@@ -1224,17 +1223,14 @@ handle_reprobe_op(lrm_state_t *lrm_state, const char *from_sys,
                   reprobe_all_nodes);
 
     if (!pcmk__strcase_any_of(from_sys, CRM_SYSTEM_PENGINE, CRM_SYSTEM_TENGINE, NULL)) {
-
-        xmlNode *reply = create_request(CRM_OP_INVOKE_LRM, NULL, from_host,
-                                        from_sys, CRM_SYSTEM_LRMD,
-                                        controld_globals.our_uuid);
+        xmlNode *reply = pcmk__new_reply(msg, NULL);
 
         crm_debug("ACK'ing re-probe from %s (%s)", from_sys, from_host);
 
         if (relay_message(reply, TRUE) == FALSE) {
             crm_log_xml_err(reply, "Unable to route reply");
         }
-        free_xml(reply);
+        pcmk__xml_free(reply);
     }
 }
 
@@ -1300,24 +1296,6 @@ static bool do_lrm_cancel(ha_msg_input_t *input, lrm_state_t *lrm_state,
             g_hash_table_remove(lrm_state->active_ops, op_id);
         }
         free(op_id);
-
-    } else {
-        /* No ack is needed since abcdaa8, but peers with older versions
-         * in a rolling upgrade need one. We didn't bump the feature set
-         * at that commit, so we can only compare against the previous
-         * CRM version (3.0.8). If any peers have feature set 3.0.9 but
-         * not abcdaa8, they will time out waiting for the ack (no
-         * released versions of Pacemaker are affected).
-         */
-        const char *peer_version = crm_element_value(params,
-                                                     PCMK_XA_CRM_FEATURE_SET);
-
-        if (compare_version(peer_version, "3.0.8") <= 0) {
-            crm_info("Sending compatibility ack for %s cancellation to %s (CRM version %s)",
-                     op_key, from_host, peer_version);
-            send_task_ok_ack(lrm_state, input, rsc->id, rsc, op_task,
-                             from_host, from_sys);
-        }
     }
 
     free(op_key);
@@ -1377,7 +1355,7 @@ static void
 free_metadata_cb_data(struct metadata_cb_data *data)
 {
     lrmd_free_rsc_info(data->rsc);
-    free_xml(data->input_xml);
+    pcmk__xml_free(data->input_xml);
     free(data);
 }
 
@@ -1395,7 +1373,8 @@ metadata_complete(int pid, const pcmk__action_result_t *result, void *user_data)
     struct metadata_cb_data *data = (struct metadata_cb_data *) user_data;
 
     struct ra_metadata_s *md = NULL;
-    lrm_state_t *lrm_state = lrm_state_find(lrm_op_target(data->input_xml));
+    lrm_state_t *lrm_state =
+        controld_get_executor_state(lrm_op_target(data->input_xml), false);
 
     if ((lrm_state != NULL) && pcmk__result_ok(result)) {
         md = controld_cache_metadata(lrm_state->metadata_cache, data->rsc,
@@ -1426,10 +1405,9 @@ do_lrm_invoke(long long action,
     bool crm_rsc_delete = FALSE;
 
     // Message routed to the local node is targeting a specific, non-local node
-    is_remote_node = !pcmk__str_eq(target_node, controld_globals.our_nodename,
-                                   pcmk__str_casei);
+    is_remote_node = !controld_is_local_node(target_node);
 
-    lrm_state = lrm_state_find(target_node);
+    lrm_state = controld_get_executor_state(target_node, false);
     if ((lrm_state == NULL) && is_remote_node) {
         crm_err("Failing action because local node has never had connection to remote node %s",
                 target_node);
@@ -1438,7 +1416,7 @@ do_lrm_invoke(long long action,
                                 "Local node has no connection to remote");
         return;
     }
-    CRM_ASSERT(lrm_state != NULL);
+    pcmk__assert(lrm_state != NULL);
 
     user_name = pcmk__update_acl_user(input->msg, PCMK__XA_CRM_USER, NULL);
     crm_op = crm_element_value(input->msg, PCMK__XA_CRM_TASK);
@@ -1468,19 +1446,6 @@ do_lrm_invoke(long long action,
         fail_lrm_resource(input->xml, lrm_state, user_name, from_host,
                           from_sys);
 
-    } else if (pcmk__str_eq(crm_op, CRM_OP_LRM_REFRESH, pcmk__str_none)) {
-        /* @COMPAT This can only be sent by crm_resource --refresh on a
-         * Pacemaker Remote node running Pacemaker 1.1.9, which is extremely
-         * unlikely. It previously would cause the controller to re-write its
-         * resource history to the CIB. Just ignore it.
-         */
-        crm_notice("Ignoring refresh request from Pacemaker Remote 1.1.9 node");
-
-    // @COMPAT DCs <1.1.14 in a rolling upgrade might schedule this op
-    } else if (pcmk__str_eq(operation, CRM_OP_PROBED, pcmk__str_none)) {
-        update_attrd(lrm_state->node_name, CRM_OP_PROBED, PCMK_VALUE_TRUE,
-                     user_name, is_remote_node);
-
     } else if (pcmk__str_eq(crm_op, CRM_OP_REPROBE, pcmk__str_none)
                || pcmk__str_eq(operation, CRM_OP_REPROBE, pcmk__str_none)) {
         const char *raw_target = NULL;
@@ -1489,7 +1454,7 @@ do_lrm_invoke(long long action,
             // For CRM_OP_REPROBE, a NULL target means we're targeting all nodes
             raw_target = crm_element_value(input->xml, PCMK__META_ON_NODE);
         }
-        handle_reprobe_op(lrm_state, from_sys, from_host, user_name,
+        handle_reprobe_op(lrm_state, input->msg, from_sys, from_host, user_name,
                           is_remote_node, (raw_target == NULL));
 
     } else if (operation != NULL) {
@@ -1515,10 +1480,10 @@ do_lrm_invoke(long long action,
             /* Delete of malformed or nonexistent resource
              * (deleting something that does not exist is a success)
              */
-            crm_notice("Not registering resource '%s' for a %s event "
-                       CRM_XS " get-rc=%d (%s) transition-key=%s",
-                       pcmk__xe_id(xml_rsc), operation,
-                       rc, pcmk_strerror(rc), pcmk__xe_id(input->xml));
+            crm_debug("Not registering resource '%s' for a %s event "
+                      QB_XS " get-rc=%d (%s) transition-key=%s",
+                      pcmk__xe_id(xml_rsc), operation,
+                      rc, pcmk_strerror(rc), pcmk__xe_id(input->xml));
             delete_rsc_entry(lrm_state, input, pcmk__xe_id(xml_rsc), NULL,
                              pcmk_ok, user_name, true);
             return;
@@ -1535,7 +1500,7 @@ do_lrm_invoke(long long action,
         } else if (rc < 0) {
             // Error communicating with the executor
             crm_err("Could not register resource '%s' with executor: %s "
-                    CRM_XS " rc=%d",
+                    QB_XS " rc=%d",
                     pcmk__xe_id(xml_rsc), pcmk_strerror(rc), rc);
             crm_log_xml_warn(input->msg, "failed registration");
             synthesize_lrmd_failure(lrm_state, input->xml, PCMK_EXEC_ERROR,
@@ -1621,7 +1586,7 @@ construct_op(const lrm_state_t *lrm_state, const xmlNode *rsc_op,
 
     const char *transition = NULL;
 
-    CRM_ASSERT(rsc_id && operation);
+    pcmk__assert((rsc_id != NULL) && (operation != NULL));
 
     op = lrmd_new_event(rsc_id, operation, 0);
     op->type = lrmd_event_exec_complete;
@@ -1751,20 +1716,19 @@ controld_ack_event_directly(const char *to_host, const char *to_sys,
 {
     xmlNode *reply = NULL;
     xmlNode *update, *iter;
-    crm_node_t *peer = NULL;
+    pcmk__node_status_t *peer = NULL;
 
     CRM_CHECK(op != NULL, return);
     if (op->rsc_id == NULL) {
         // op->rsc_id is a (const char *) but lrmd_free_event() frees it
-        CRM_ASSERT(rsc_id != NULL);
+        pcmk__assert(rsc_id != NULL);
         op->rsc_id = pcmk__str_copy(rsc_id);
     }
     if (to_sys == NULL) {
         to_sys = CRM_SYSTEM_TENGINE;
     }
 
-    peer = pcmk__get_node(0, controld_globals.our_nodename, NULL,
-                          pcmk__node_search_cluster_member);
+    peer = controld_get_local_node_status();
     update = create_node_state_update(peer, node_update_none, NULL,
                                       __func__);
 
@@ -1776,8 +1740,16 @@ controld_ack_event_directly(const char *to_host, const char *to_sys,
     crm_xml_add(iter, PCMK_XA_ID, op->rsc_id);
 
     controld_add_resource_history_xml(iter, rsc, op,
-                                      controld_globals.our_nodename);
-    reply = create_request(CRM_OP_INVOKE_LRM, update, to_host, to_sys, CRM_SYSTEM_LRMD, NULL);
+                                      controld_globals.cluster->priv->node_name);
+
+    /* We don't have the original message ID, so use "direct-ack" (we just need
+     * something non-NULL for this to create a reply)
+     *
+     * @TODO It would be better to use the server, message ID, and task from the
+     * original request when callers have it available
+     */
+    reply = pcmk__new_message(pcmk_ipc_controld, "direct-ack", CRM_SYSTEM_LRMD,
+                              to_host, to_sys, CRM_OP_INVOKE_LRM, update);
 
     crm_log_xml_trace(update, "[direct ACK]");
 
@@ -1789,8 +1761,8 @@ controld_ack_event_directly(const char *to_host, const char *to_sys,
         crm_log_xml_err(reply, "Unable to route reply");
     }
 
-    free_xml(update);
-    free_xml(reply);
+    pcmk__xml_free(update);
+    pcmk__xml_free(reply);
 }
 
 gboolean
@@ -1939,7 +1911,7 @@ do_lrm_rsc_op(lrm_state_t *lrm_state, lrmd_rsc_info_t *rsc, xmlNode *msg,
     if (lrm_state == NULL) {
         // This shouldn't be possible, but provide a failsafe just in case
         crm_err("Cannot execute %s of %s: No executor connection "
-                CRM_XS " transition_key=%s",
+                QB_XS " transition_key=%s",
                 operation, rsc->id, pcmk__s(transition, ""));
         synthesize_lrmd_failure(NULL, msg, PCMK_EXEC_INVALID,
                                 PCMK_OCF_UNKNOWN_ERROR,
@@ -1984,7 +1956,7 @@ do_lrm_rsc_op(lrm_state_t *lrm_state, lrmd_rsc_info_t *rsc, xmlNode *msg,
 
     /* now do the op */
     crm_notice("Requesting local execution of %s operation for %s on %s "
-               CRM_XS " transition_key=%s op_key=" PCMK__OP_FMT,
+               QB_XS " transition_key=%s op_key=" PCMK__OP_FMT,
                pcmk__readable_action(op->op_type, op->interval_ms), rsc->id,
                lrm_state->node_name, pcmk__s(transition, ""), rsc->id,
                operation, op->interval_ms);
@@ -2056,7 +2028,7 @@ do_lrm_rsc_op(lrm_state_t *lrm_state, lrmd_rsc_info_t *rsc, xmlNode *msg,
 
     } else if (lrm_state_is_local(lrm_state)) {
         crm_err("Could not initiate %s action for resource %s locally: %s "
-                CRM_XS " rc=%d", operation, rsc->id, pcmk_rc_str(rc), rc);
+                QB_XS " rc=%d", operation, rsc->id, pcmk_rc_str(rc), rc);
         fake_op_status(lrm_state, op, PCMK_EXEC_NOT_CONNECTED,
                        PCMK_OCF_UNKNOWN_ERROR, pcmk_rc_str(rc));
         process_lrm_event(lrm_state, op, NULL, NULL);
@@ -2064,7 +2036,7 @@ do_lrm_rsc_op(lrm_state_t *lrm_state, lrmd_rsc_info_t *rsc, xmlNode *msg,
 
     } else {
         crm_err("Could not initiate %s action for resource %s remotely on %s: "
-                "%s " CRM_XS " rc=%d",
+                "%s " QB_XS " rc=%d",
                 operation, rsc->id, lrm_state->node_name, pcmk_rc_str(rc), rc);
         fake_op_status(lrm_state, op, PCMK_EXEC_NOT_CONNECTED,
                        PCMK_OCF_UNKNOWN_ERROR, pcmk_rc_str(rc));
@@ -2160,7 +2132,7 @@ log_executor_event(const lrmd_event_data_t *op, const char *op_key,
     switch (op->op_status) {
         case PCMK_EXEC_DONE:
             log_level = LOG_NOTICE;
-            pcmk__g_strcat(str, ": ", services_ocf_exitcode_str(op->rc), NULL);
+            pcmk__g_strcat(str, ": ", crm_exit_str((crm_exit_t) op->rc), NULL);
             break;
 
         case PCMK_EXEC_TIMEOUT:
@@ -2171,25 +2143,14 @@ log_executor_event(const lrmd_event_data_t *op, const char *op_key,
 
         case PCMK_EXEC_CANCELLED:
             log_level = LOG_INFO;
-	    /* order of __attribute__ and Fall through comment is IMPORTANT!
-	     * do not change it without proper testing with both clang and gcc
-	     * in multiple versions.
-	     * the clang check allows to build with all versions of clang.
-	     * the has_c_attribute check is to workaround a bug in clang version
-	     * in rhel7. has_attribute would happily return "YES SIR WE GOT IT"
-	     * and fail the build the next line.
-	     */
-#ifdef __clang__
-#ifdef __has_c_attribute
-#if __has_attribute(fallthrough)
-	    __attribute__((fallthrough));
-#endif
-#endif
-#endif
-            // Fall through
+            pcmk__g_strcat(str, ": ", pcmk_exec_status_str(op->op_status),
+                           NULL);
+            break;
+
         default:
             pcmk__g_strcat(str, ": ", pcmk_exec_status_str(op->op_status),
                            NULL);
+            break;
     }
 
     if ((op->exit_reason != NULL)
@@ -2198,7 +2159,7 @@ log_executor_event(const lrmd_event_data_t *op, const char *op_key,
         pcmk__g_strcat(str, " (", op->exit_reason, ")", NULL);
     }
 
-    g_string_append(str, " " CRM_XS);
+    g_string_append(str, " " QB_XS);
     g_string_append_printf(str, " graph action %sconfirmed; call=%d key=%s",
                            (confirmed? "" : "un"), op->call_id, op_key);
     if (op->op_status == PCMK_EXEC_DONE) {
