@@ -78,8 +78,8 @@ fail_pending_op(gpointer key, gpointer value, gpointer user_data)
     event.interval_ms = op->interval_ms;
     lrmd__set_result(&event, PCMK_OCF_UNKNOWN_ERROR, PCMK_EXEC_NOT_CONNECTED,
                      "Action was pending when executor connection was dropped");
-    event.t_run = (unsigned int) op->start_time;
-    event.t_rcchange = (unsigned int) op->start_time;
+    event.t_run = op->start_time;
+    event.t_rcchange = op->start_time;
 
     event.call_id = op->call_id;
     event.remote_nodename = lrm_state->node_name;
@@ -93,9 +93,7 @@ fail_pending_op(gpointer key, gpointer value, gpointer user_data)
 gboolean
 lrm_state_is_local(lrm_state_t *lrm_state)
 {
-    return (lrm_state != NULL)
-           && pcmk__str_eq(lrm_state->node_name, controld_globals.our_nodename,
-                           pcmk__str_casei);
+    return (lrm_state != NULL) && controld_is_local_node(lrm_state->node_name);
 }
 
 /*!
@@ -295,28 +293,32 @@ lrm_state_destroy_all(void)
     }
 }
 
+/*!
+ * \internal
+ * \brief Get executor state object
+ *
+ * \param[in] node_name  Get executor state for this node (local node if NULL)
+ * \param[in] create     If true, create executor state if it doesn't exist
+ *
+ * \return Executor state object for \p node_name
+ */
 lrm_state_t *
-lrm_state_find(const char *node_name)
+controld_get_executor_state(const char *node_name, bool create)
 {
+    lrm_state_t *state = NULL;
+
+    if ((node_name == NULL) && (controld_globals.cluster != NULL)) {
+        node_name = controld_globals.cluster->priv->node_name;
+    }
     if ((node_name == NULL) || (lrm_state_table == NULL)) {
         return NULL;
     }
-    return g_hash_table_lookup(lrm_state_table, node_name);
-}
 
-lrm_state_t *
-lrm_state_find_or_create(const char *node_name)
-{
-    lrm_state_t *lrm_state;
-
-    CRM_CHECK(lrm_state_table != NULL, return NULL);
-
-    lrm_state = g_hash_table_lookup(lrm_state_table, node_name);
-    if (!lrm_state) {
-        lrm_state = lrm_state_create(node_name);
+    state = g_hash_table_lookup(lrm_state_table, node_name);
+    if ((state == NULL) && create) {
+        state = lrm_state_create(node_name);
     }
-
-    return lrm_state;
+    return state;
 }
 
 GList *
@@ -438,7 +440,7 @@ crmd_proxy_send(const char *session, xmlNode *msg)
         return;
     }
     crm_log_xml_trace(msg, "to-proxy");
-    lrm_state = lrm_state_find(proxy->node_name);
+    lrm_state = controld_get_executor_state(proxy->node_name, false);
     if (lrm_state) {
         crm_trace("Sending event to %.8s on %s", proxy->session_id, proxy->node_name);
         remote_proxy_relay_event(proxy, msg);
@@ -506,7 +508,7 @@ crmd_remote_proxy_cb(lrmd_t *lrmd, void *userdata, xmlNode *msg)
                  * remote peer for validation
                  */
                 int rc = cib_conn->cmds->query(cib_conn, PCMK_XE_CRM_CONFIG,
-                                               NULL, cib_scope_local);
+                                               NULL, cib_none);
                 cib_conn->cmds->register_callback_full(cib_conn, rc, 10, FALSE,
                                                        lrmd,
                                                        "remote_config_check",
@@ -545,7 +547,8 @@ crmd_remote_proxy_cb(lrmd_t *lrmd, void *userdata, xmlNode *msg)
         /* This is for the controller, which we are, so don't try
          * to send to ourselves over IPC -- do it directly.
          */
-        int flags = 0;
+        uint32_t flags = 0U;
+        int rc = pcmk_rc_ok;
         xmlNode *wrapper = pcmk__xe_first_child(msg, PCMK__XE_LRMD_IPC_MSG,
                                                 NULL, NULL);
         xmlNode *request = pcmk__xe_first_child(wrapper, NULL, NULL, NULL);
@@ -574,8 +577,12 @@ crmd_remote_proxy_cb(lrmd_t *lrmd, void *userdata, xmlNode *msg)
 
         crmd_proxy_dispatch(session, request);
 
-        crm_element_value_int(msg, PCMK__XA_LRMD_IPC_MSG_FLAGS, &flags);
-        if (flags & crm_ipc_client_response) {
+        rc = pcmk__xe_get_flags(msg, PCMK__XA_LRMD_IPC_MSG_FLAGS, &flags, 0U);
+        if (rc != pcmk_rc_ok) {
+            crm_warn("Couldn't parse controller flags from remote request: %s",
+                     pcmk_rc_str(rc));
+        }
+        if (pcmk_is_set(flags, crm_ipc_client_response)) {
             int msg_id = 0;
             xmlNode *op_reply = pcmk__xe_create(NULL, PCMK__XE_ACK);
 
@@ -585,7 +592,7 @@ crmd_remote_proxy_cb(lrmd_t *lrmd, void *userdata, xmlNode *msg)
             crm_element_value_int(msg, PCMK__XA_LRMD_IPC_MSG_ID, &msg_id);
             remote_proxy_relay_response(proxy, op_reply, msg_id);
 
-            free_xml(op_reply);
+            pcmk__xml_free(op_reply);
         }
 
     } else {
@@ -607,7 +614,7 @@ controld_connect_remote_executor(lrm_state_t *lrm_state, const char *server,
         rc = lrmd__new(&api, lrm_state->node_name, server, port);
         if (rc != pcmk_rc_ok) {
             crm_warn("Pacemaker Remote connection to %s:%s failed: %s "
-                     CRM_XS " rc=%d", server, port, pcmk_rc_str(rc), rc);
+                     QB_XS " rc=%d", server, port, pcmk_rc_str(rc), rc);
 
             return rc;
         }
@@ -784,7 +791,7 @@ lrm_state_register_rsc(lrm_state_t * lrm_state,
     }
 
     if (is_remote_lrmd_ra(agent, provider, NULL)) {
-        return lrm_state_find_or_create(rsc_id)? pcmk_ok : -EINVAL;
+        return controld_get_executor_state(rsc_id, true)? pcmk_ok : -EINVAL;
     }
 
     /* @TODO Implement an asynchronous version of this (currently a blocking
