@@ -30,8 +30,10 @@ struct st_fail_rec {
     int count;
 };
 
+#define DEFAULT_STONITH_MAX_ATTEMPTS 10
+
 static bool fence_reaction_panic = false;
-static unsigned long int stonith_max_attempts = 10;
+static unsigned long int stonith_max_attempts = DEFAULT_STONITH_MAX_ATTEMPTS;
 static GHashTable *stonith_failures = NULL;
 
 /*!
@@ -43,10 +45,17 @@ static GHashTable *stonith_failures = NULL;
 static void
 update_stonith_max_attempts(const char *value)
 {
-    stonith_max_attempts = char2score(value);
-    if (stonith_max_attempts < 1UL) {
-        stonith_max_attempts = 10UL;
+    int score = 0;
+    int rc = pcmk_parse_score(value, &score, DEFAULT_STONITH_MAX_ATTEMPTS);
+
+    // The option validator ensures invalid values shouldn't be possible
+    CRM_CHECK((rc == pcmk_rc_ok) && (score > 0), return);
+
+    if (stonith_max_attempts != score) {
+        crm_debug("Maximum fencing attempts per transition is now %d (was %lu)",
+                  score, stonith_max_attempts);
     }
+    stonith_max_attempts = score;
 }
 
 /*!
@@ -203,7 +212,7 @@ send_stonith_update(pcmk__graph_action_t *action, const char *target,
                     const char *uuid)
 {
     int rc = pcmk_ok;
-    crm_node_t *peer = NULL;
+    pcmk__node_status_t *peer = NULL;
 
     /* We (usually) rely on the membership layer to do node_update_cluster,
      * and the peer status callback to do node_update_peer, because the node
@@ -233,9 +242,9 @@ send_stonith_update(pcmk__graph_action_t *action, const char *target,
         flags |= node_update_cluster;
     }
 
-    if (peer->uuid == NULL) {
-        crm_info("Recording uuid '%s' for node '%s'", uuid, target);
-        peer->uuid = pcmk__str_copy(uuid);
+    if (peer->xml_id == NULL) {
+        crm_info("Recording XML ID '%s' for node '%s'", uuid, target);
+        peer->xml_id = pcmk__str_copy(uuid);
     }
 
     crmd_peer_down(peer, TRUE);
@@ -244,7 +253,7 @@ send_stonith_update(pcmk__graph_action_t *action, const char *target,
     node_state = create_node_state_update(peer, flags, NULL, __func__);
 
     /* we have to mark whether or not remote nodes have already been fenced */
-    if (peer->flags & crm_remote_node) {
+    if (pcmk_is_set(peer->flags, pcmk__node_status_remote)) {
         char *now_s = pcmk__ttoa(time(NULL));
 
         crm_xml_add(node_state, PCMK__XA_NODE_FENCED, now_s);
@@ -256,8 +265,7 @@ send_stonith_update(pcmk__graph_action_t *action, const char *target,
 
     rc = controld_globals.cib_conn->cmds->modify(controld_globals.cib_conn,
                                                  PCMK_XE_STATUS, node_state,
-                                                 cib_scope_local
-                                                 |cib_can_create);
+                                                 cib_can_create);
 
     /* Delay processing the trigger until the update completes */
     crm_debug("Sending fencing update %d for %s", rc, target);
@@ -265,12 +273,11 @@ send_stonith_update(pcmk__graph_action_t *action, const char *target,
 
     // Make sure it sticks
     /* controld_globals.cib_conn->cmds->bump_epoch(controld_globals.cib_conn,
-     *                                             cib_scope_local);
+     *                                             cib_none);
      */
 
-    controld_delete_node_state(peer->uname, controld_section_all,
-                               cib_scope_local);
-    free_xml(node_state);
+    controld_delete_node_state(peer->name, controld_section_all, cib_none);
+    pcmk__xml_free(node_state);
     return;
 }
 
@@ -375,7 +382,7 @@ execute_stonith_cleanup(void)
 
     for (iter = stonith_cleanup_list; iter != NULL; iter = iter->next) {
         char *target = iter->data;
-        crm_node_t *target_node =
+        pcmk__node_status_t *target_node =
             pcmk__get_node(0, target, NULL, pcmk__node_search_cluster_member);
         const char *uuid = pcmk__cluster_node_uuid(target_node);
 
@@ -537,9 +544,7 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
         return;
     }
 
-    if (succeeded
-        && pcmk__str_eq(event->target, controld_globals.our_nodename,
-                        pcmk__str_casei)) {
+    if (succeeded && controld_is_local_node(event->target)) {
         /* We were notified of our own fencing. Most likely, either fencing was
          * misconfigured, or fabric fencing that doesn't cut cluster
          * communication is in use.
@@ -552,7 +557,7 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
         crm_crit("We were allegedly just fenced by %s for %s!",
                  executioner, event->origin); // Dumps blackbox if enabled
         if (fence_reaction_panic) {
-            pcmk__panic(__func__);
+            pcmk__panic("Notified of own fencing");
         } else {
             crm_exit(CRM_EX_FATAL);
         }
@@ -572,7 +577,7 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
     }
 
     crm_notice("Peer %s was%s terminated (%s) by %s on behalf of %s@%s: "
-               "%s%s%s%s " CRM_XS " event=%s",
+               "%s%s%s%s " QB_XS " event=%s",
                event->target, (succeeded? "" : " not"),
                event->action, executioner, client, event->origin,
                (succeeded? "OK" : pcmk_exec_status_str(exec_status)),
@@ -585,7 +590,8 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
         const uint32_t flags = pcmk__node_search_any
                                |pcmk__node_search_cluster_cib;
 
-        crm_node_t *peer = pcmk__search_node_caches(0, event->target, flags);
+        pcmk__node_status_t *peer = pcmk__search_node_caches(0, event->target,
+                                                             flags);
         const char *uuid = NULL;
 
         if (peer == NULL) {
@@ -618,7 +624,7 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
 
         } else if (pcmk__str_eq(controld_globals.dc_name, event->target,
                                 pcmk__str_null_matches|pcmk__str_casei)
-                   && !pcmk_is_set(peer->flags, crm_remote_node)) {
+                   && !pcmk_is_set(peer->flags, pcmk__node_status_remote)) {
             // Assume the target was our DC if we don't currently have one
 
             if (controld_globals.dc_name != NULL) {
@@ -632,8 +638,7 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
              * have one node update the CIB now and, if the new DC is different,
              * have them do so too after the election
              */
-            if (pcmk__str_eq(event->executioner, controld_globals.our_nodename,
-                             pcmk__str_casei)) {
+            if (controld_is_local_node(event->executioner)) {
                 send_stonith_update(NULL, event->target, uuid);
             }
             add_stonith_cleanup(event->target);
@@ -644,7 +649,7 @@ handle_fence_notification(stonith_t *st, stonith_event_t *event)
          * The connection won't necessarily drop when a remote node is fenced,
          * so the failure might not otherwise be detected until the next poke.
          */
-        if (pcmk_is_set(peer->flags, crm_remote_node)) {
+        if (pcmk_is_set(peer->flags, pcmk__node_status_remote)) {
             remote_ra_fail(event->target);
         }
 
@@ -684,7 +689,7 @@ controld_timer_fencer_connect(gpointer user_data)
         rc = stonith_api_connect_retry(stonith_api, crm_system_name, 30);
         if (rc != pcmk_ok) {
             crm_err("Could not connect to fencer in 30 attempts: %s "
-                    CRM_XS " rc=%d", pcmk_strerror(rc), rc);
+                    QB_XS " rc=%d", pcmk_strerror(rc), rc);
         }
     } else {
         // Non-blocking (retry failures later in main loop)
@@ -701,7 +706,7 @@ controld_timer_fencer_connect(gpointer user_data)
             if (pcmk_is_set(controld_globals.fsa_input_register,
                             R_ST_REQUIRED)) {
                 crm_notice("Fencer connection failed (will retry): %s "
-                           CRM_XS " rc=%d", pcmk_strerror(rc), rc);
+                           QB_XS " rc=%d", pcmk_strerror(rc), rc);
 
                 if (!mainloop_timer_running(controld_fencer_connect_timer)) {
                     mainloop_timer_start(controld_fencer_connect_timer);
@@ -710,7 +715,7 @@ controld_timer_fencer_connect(gpointer user_data)
                 return G_SOURCE_CONTINUE;
             } else {
                 crm_info("Fencer connection failed (ignoring because no longer required): %s "
-                         CRM_XS " rc=%d", pcmk_strerror(rc), rc);
+                         QB_XS " rc=%d", pcmk_strerror(rc), rc);
             }
             return G_SOURCE_REMOVE;
         }
@@ -802,7 +807,7 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
         if (reason == NULL) {
            reason = pcmk_exec_status_str(stonith__execution_status(data));
         }
-        crm_notice("Result of fence operation %d: %d (%s) " CRM_XS " key=%s",
+        crm_notice("Result of fence operation %d: %d (%s) " QB_XS " key=%s",
                    data->call_id, stonith__exit_status(data), reason,
                    (const char *) data->userdata);
         return;
@@ -816,7 +821,7 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
         || !pcmk__str_eq(uuid, controld_globals.te_uuid, pcmk__str_none)
         || (controld_globals.transition_graph->id != transition_id)) {
         crm_info("Ignoring fence operation %d result: "
-                 "Not from current transition " CRM_XS
+                 "Not from current transition " QB_XS
                  " complete=%s action=%d uuid=%s (vs %s) transition=%d (vs %d)",
                  data->call_id,
                  pcmk__btoa(controld_globals.transition_graph->complete),
@@ -829,7 +834,7 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
     if (action == NULL) {
         crm_err("Ignoring fence operation %d result: "
                 "Action %d not found in transition graph (bug?) "
-                CRM_XS " uuid=%s transition=%d",
+                QB_XS " uuid=%s transition=%d",
                 data->call_id, stonith_id, uuid, transition_id);
         goto bail;
     }
@@ -863,7 +868,8 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
                  * This allows it to learn more quickly if this node does have
                  * the information.
                  */
-                if (g_hash_table_lookup(crm_remote_peer_cache, uuid) != NULL) {
+                if (g_hash_table_lookup(pcmk__remote_peer_cache,
+                                        uuid) != NULL) {
                     is_remote_node = TRUE;
                 }
 
@@ -936,11 +942,10 @@ static int
 fence_with_delay(const char *target, const char *type, int delay)
 {
     uint32_t options = st_opt_none; // Group of enum stonith_call_options
-    int timeout_sec = (int) (controld_globals.transition_graph->stonith_timeout
-                             / 1000);
+    int timeout_sec = pcmk__timeout_ms2s(controld_globals.transition_graph->stonith_timeout);
 
-    if (crmd_join_phase_count(crm_join_confirmed) == 1) {
-        stonith__set_call_options(options, target, st_opt_allow_suicide);
+    if (crmd_join_phase_count(controld_join_confirmed) == 1) {
+        stonith__set_call_options(options, target, st_opt_allow_self_fencing);
     }
     return stonith_api->cmds->fence_with_delay(stonith_api, options, target,
                                                type, timeout_sec, 0, delay);
@@ -969,8 +974,7 @@ controld_execute_fence_action(pcmk__graph_t *graph,
     const char *priority_delay = NULL;
     int delay_i = 0;
     gboolean invalid_action = FALSE;
-    int stonith_timeout = (int) (controld_globals.transition_graph->stonith_timeout
-                                 / 1000);
+    int stonith_timeout = pcmk__timeout_ms2s(controld_globals.transition_graph->stonith_timeout);
 
     CRM_CHECK(id != NULL, invalid_action = TRUE);
     CRM_CHECK(uuid != NULL, invalid_action = TRUE);
@@ -986,7 +990,7 @@ controld_execute_fence_action(pcmk__graph_t *graph,
                                     PCMK_OPT_PRIORITY_FENCING_DELAY);
 
     crm_notice("Requesting fencing (%s) targeting node %s "
-               CRM_XS " action=%s timeout=%i%s%s",
+               QB_XS " action=%s timeout=%i%s%s",
                type, target, id, stonith_timeout,
                priority_delay ? " priority_delay=" : "",
                priority_delay ? priority_delay : "");
@@ -1012,7 +1016,7 @@ bool
 controld_verify_stonith_watchdog_timeout(const char *value)
 {
     long long st_timeout = (value != NULL)? crm_get_msec(value) : 0;
-    const char *our_nodename = controld_globals.our_nodename;
+    const char *our_nodename = controld_globals.cluster->priv->node_name;
 
     if (st_timeout == 0
         || (stonith_api && (stonith_api->state != stonith_disconnected) &&
