@@ -9,6 +9,14 @@
 
 #include <crm_internal.h>
 
+#include <inttypes.h>               // PRIu32
+#include <stdbool.h>                // bool, true, false
+#include <stdio.h>                  // NULL
+#include <stdlib.h>                 // free(), etc.
+
+#include <glib.h>                   // gboolean, etc.
+#include <libxml/tree.h>            // xmlNode
+
 #include <crm/crm.h>
 
 #include <crm/common/xml.h>
@@ -40,6 +48,35 @@ gboolean check_join_state(enum crmd_fsa_state cur_state, const char *source);
  */
 static int current_join_id = 0;
 
+/*!
+ * \internal
+ * \brief Get log-friendly string equivalent of a controller group join phase
+ *
+ * \param[in] phase  Join phase
+ *
+ * \return Log-friendly string equivalent of \p phase
+ */
+static const char *
+join_phase_text(enum controld_join_phase phase)
+{
+    switch (phase) {
+        case controld_join_nack:
+            return "nack";
+        case controld_join_none:
+            return "none";
+        case controld_join_welcomed:
+            return "welcomed";
+        case controld_join_integrated:
+            return "integrated";
+        case controld_join_finalized:
+            return "finalized";
+        case controld_join_confirmed:
+            return "confirmed";
+        default:
+            return "invalid";
+    }
+}
+
 /*!
  * \internal
  * \brief Destroy the hash table containing failed sync nodes
@@ -116,58 +153,68 @@ lookup_failed_sync_node(const char *node_name, gint *join_id)
 }
 
 void
-crm_update_peer_join(const char *source, crm_node_t * node, enum crm_join_phase phase)
+crm_update_peer_join(const char *source, pcmk__node_status_t *node,
+                     enum controld_join_phase phase)
 {
-    enum crm_join_phase last = 0;
+    enum controld_join_phase last = controld_get_join_phase(node);
 
     CRM_CHECK(node != NULL, return);
 
     /* Remote nodes do not participate in joins */
-    if (pcmk_is_set(node->flags, crm_remote_node)) {
+    if (pcmk_is_set(node->flags, pcmk__node_status_remote)) {
         return;
     }
 
-    last = node->join;
-
-    if(phase == last) {
+    if (phase == last) {
         crm_trace("Node %s join-%d phase is still %s "
-                  CRM_XS " nodeid=%u source=%s",
-                  node->uname, current_join_id, crm_join_phase_str(last),
-                  node->id, source);
-
-    } else if ((phase <= crm_join_none) || (phase == (last + 1))) {
-        node->join = phase;
-        crm_trace("Node %s join-%d phase is now %s (was %s) "
-                  CRM_XS " nodeid=%u source=%s",
-                 node->uname, current_join_id, crm_join_phase_str(phase),
-                 crm_join_phase_str(last), node->id, source);
-
-    } else {
-        crm_warn("Rejecting join-%d phase update for node %s because "
-                 "can't go from %s to %s " CRM_XS " nodeid=%u source=%s",
-                 current_join_id, node->uname, crm_join_phase_str(last),
-                 crm_join_phase_str(phase), node->id, source);
+                  QB_XS " nodeid=%" PRIu32 " source=%s",
+                  node->name, current_join_id, join_phase_text(last),
+                  node->cluster_layer_id, source);
+        return;
     }
+
+    if ((phase <= controld_join_none) || (phase == (last + 1))) {
+        struct controld_node_status_data *data = NULL;
+
+        if (node->user_data == NULL) {
+            node->user_data =
+                pcmk__assert_alloc(1, sizeof(struct controld_node_status_data));
+        }
+        data = node->user_data;
+        data->join_phase = phase;
+
+        crm_trace("Node %s join-%d phase is now %s (was %s) "
+                  QB_XS " nodeid=%" PRIu32 " source=%s",
+                  node->name, current_join_id, join_phase_text(phase),
+                  join_phase_text(last), node->cluster_layer_id,
+                  source);
+        return;
+    }
+
+    crm_warn("Rejecting join-%d phase update for node %s because can't go from "
+             "%s to %s " QB_XS " nodeid=%" PRIu32 " source=%s",
+             current_join_id, node->name, join_phase_text(last),
+             join_phase_text(phase), node->cluster_layer_id, source);
 }
 
 static void
 start_join_round(void)
 {
     GHashTableIter iter;
-    crm_node_t *peer = NULL;
+    pcmk__node_status_t *peer = NULL;
 
     crm_debug("Starting new join round join-%d", current_join_id);
 
-    g_hash_table_iter_init(&iter, crm_peer_cache);
+    g_hash_table_iter_init(&iter, pcmk__peer_cache);
     while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &peer)) {
-        crm_update_peer_join(__func__, peer, crm_join_none);
+        crm_update_peer_join(__func__, peer, controld_join_none);
     }
     if (max_generation_from != NULL) {
         free(max_generation_from);
         max_generation_from = NULL;
     }
     if (max_generation_xml != NULL) {
-        free_xml(max_generation_xml);
+        pcmk__xml_free(max_generation_xml);
         max_generation_xml = NULL;
     }
     controld_clear_fsa_input_flags(R_HAVE_CIB);
@@ -183,8 +230,8 @@ start_join_round(void)
 static xmlNode *
 create_dc_message(const char *join_op, const char *host_to)
 {
-    xmlNode *msg = create_request(join_op, NULL, host_to, CRM_SYSTEM_CRMD,
-                                  CRM_SYSTEM_DC, NULL);
+    xmlNode *msg = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_DC, host_to,
+                                     CRM_SYSTEM_CRMD, join_op, NULL);
 
     /* Identify which election this is a part of */
     crm_xml_add_int(msg, PCMK__XA_JOIN_ID, current_join_id);
@@ -201,15 +248,18 @@ create_dc_message(const char *join_op, const char *host_to)
 static void
 join_make_offer(gpointer key, gpointer value, gpointer user_data)
 {
+    /* @TODO We don't use user_data except to distinguish one particular call
+     * from others. Make this clearer.
+     */
     xmlNode *offer = NULL;
-    crm_node_t *member = (crm_node_t *)value;
+    pcmk__node_status_t *member = (pcmk__node_status_t *) value;
 
-    CRM_ASSERT(member != NULL);
+    pcmk__assert(member != NULL);
     if (!pcmk__cluster_is_node_active(member)) {
         crm_info("Not making join-%d offer to inactive node %s",
-                 current_join_id,
-                 (member->uname? member->uname : "with unknown name"));
-        if(member->expected == NULL && pcmk__str_eq(member->state, CRM_NODE_LOST, pcmk__str_casei)) {
+                 current_join_id, pcmk__s(member->name, "with unknown name"));
+        if ((member->expected == NULL)
+            && pcmk__str_eq(member->state, PCMK__VALUE_LOST, pcmk__str_none)) {
             /* You would think this unsafe, but in fact this plus an
              * active resource is what causes it to be fenced.
              *
@@ -224,37 +274,41 @@ join_make_offer(gpointer key, gpointer value, gpointer user_data)
         return;
     }
 
-    if (member->uname == NULL) {
+    if (member->name == NULL) {
         crm_info("Not making join-%d offer to node uuid %s with unknown name",
-                 current_join_id, member->uuid);
+                 current_join_id, member->xml_id);
         return;
     }
 
-    if (controld_globals.membership_id != crm_peer_seq) {
-        controld_globals.membership_id = crm_peer_seq;
+    if (controld_globals.membership_id != controld_globals.peer_seq) {
+        controld_globals.membership_id = controld_globals.peer_seq;
         crm_info("Making join-%d offers based on membership event %llu",
-                 current_join_id, crm_peer_seq);
+                 current_join_id, controld_globals.peer_seq);
     }
 
-    if(user_data && member->join > crm_join_none) {
-        crm_info("Not making join-%d offer to already known node %s (%s)",
-                 current_join_id, member->uname,
-                 crm_join_phase_str(member->join));
-        return;
+    if (user_data != NULL) {
+        enum controld_join_phase phase = controld_get_join_phase(member);
+
+        if (phase > controld_join_none) {
+            crm_info("Not making join-%d offer to already known node %s (%s)",
+                     current_join_id, member->name, join_phase_text(phase));
+            return;
+        }
     }
 
-    crm_update_peer_join(__func__, (crm_node_t*)member, crm_join_none);
+    crm_update_peer_join(__func__, (pcmk__node_status_t*) member,
+                         controld_join_none);
 
-    offer = create_dc_message(CRM_OP_JOIN_OFFER, member->uname);
+    offer = create_dc_message(CRM_OP_JOIN_OFFER, member->name);
 
     // Advertise our feature set so the joining node can bail if not compatible
     crm_xml_add(offer, PCMK_XA_CRM_FEATURE_SET, CRM_FEATURE_SET);
 
-    crm_info("Sending join-%d offer to %s", current_join_id, member->uname);
-    pcmk__cluster_send_message(member, crm_msg_crmd, offer);
-    free_xml(offer);
+    crm_info("Sending join-%d offer to %s", current_join_id, member->name);
+    pcmk__cluster_send_message(member, pcmk_ipc_controld, offer);
+    pcmk__xml_free(offer);
 
-    crm_update_peer_join(__func__, member, crm_join_welcomed);
+    crm_update_peer_join(__func__, member, controld_join_welcomed);
 }
 
 /*	 A_DC_JOIN_OFFER_ALL	*/
@@ -277,9 +331,9 @@ do_dc_join_offer_all(long long action,
     if (cause == C_HA_MESSAGE && current_input == I_NODE_JOIN) {
         crm_info("A new node joined the cluster");
     }
-    g_hash_table_foreach(crm_peer_cache, join_make_offer, NULL);
+    g_hash_table_foreach(pcmk__peer_cache, join_make_offer, NULL);
 
-    count = crmd_join_phase_count(crm_join_welcomed);
+    count = crmd_join_phase_count(controld_join_welcomed);
     crm_info("Waiting on join-%d requests from %d outstanding node%s",
              current_join_id, count, pcmk__plural_s(count));
 
@@ -293,7 +347,7 @@ do_dc_join_offer_one(long long action,
                      enum crmd_fsa_state cur_state,
                      enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
-    crm_node_t *member;
+    pcmk__node_status_t *member = NULL;
     ha_msg_input_t *welcome = NULL;
     int count;
     const char *join_to = NULL;
@@ -301,7 +355,7 @@ do_dc_join_offer_one(long long action,
     if (msg_data->data == NULL) {
         crm_info("Making join-%d offers to any unconfirmed nodes "
                  "because an unknown node joined", current_join_id);
-        g_hash_table_foreach(crm_peer_cache, join_make_offer, &member);
+        g_hash_table_foreach(pcmk__peer_cache, join_make_offer, &member);
         check_join_state(cur_state, __func__);
         return;
     }
@@ -324,15 +378,14 @@ do_dc_join_offer_one(long long action,
      * due course, or we can re-store the original offer on the client.
      */
 
-    crm_update_peer_join(__func__, member, crm_join_none);
+    crm_update_peer_join(__func__, member, controld_join_none);
     join_make_offer(NULL, member, NULL);
 
     /* If the offer isn't to the local node, make an offer to the local node as
      * well, to ensure the correct value for max_generation_from.
      */
-    if (strcasecmp(join_to, controld_globals.our_nodename) != 0) {
-        member = pcmk__get_node(0, controld_globals.our_nodename, NULL,
-                                pcmk__node_search_cluster_member);
+    if (!controld_is_local_node(join_to)) {
+        member = controld_get_local_node_status();
         join_make_offer(NULL, member, NULL);
     }
 
@@ -342,7 +395,7 @@ do_dc_join_offer_one(long long action,
     abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart, "Node join",
                      NULL);
 
-    count = crmd_join_phase_count(crm_join_welcomed);
+    count = crmd_join_phase_count(controld_join_welcomed);
     crm_info("Waiting on join-%d requests from %d outstanding node%s",
              current_join_id, count, pcmk__plural_s(count));
 
@@ -358,8 +411,19 @@ compare_int_fields(xmlNode * left, xmlNode * right, const char *field)
     long long int_elem_l;
     long long int_elem_r;
 
-    pcmk__scan_ll(elem_l, &int_elem_l, -1LL);
-    pcmk__scan_ll(elem_r, &int_elem_r, -1LL);
+    int rc = pcmk_rc_ok;
+
+    rc = pcmk__scan_ll(elem_l, &int_elem_l, -1LL);
+    if (rc != pcmk_rc_ok) { // Shouldn't be possible
+        crm_warn("Comparing current CIB %s as -1 "
+                 "because '%s' is not an integer", field, elem_l);
+    }
+
+    rc = pcmk__scan_ll(elem_r, &int_elem_r, -1LL);
+    if (rc != pcmk_rc_ok) { // Shouldn't be possible
+        crm_warn("Comparing joining node's CIB %s as -1 "
+                 "because '%s' is not an integer", field, elem_r);
+    }
 
     if (int_elem_l < int_elem_r) {
         return -1;
@@ -391,7 +455,7 @@ do_dc_join_filter_offer(long long action,
     const char *ref = crm_element_value(join_ack->msg, PCMK_XA_REFERENCE);
     const char *join_version = crm_element_value(join_ack->msg,
                                                  PCMK_XA_CRM_FEATURE_SET);
-    crm_node_t *join_node = NULL;
+    pcmk__node_status_t *join_node = NULL;
 
     if (join_from == NULL) {
         crm_err("Ignoring invalid join request without node name");
@@ -438,7 +502,7 @@ do_dc_join_filter_offer(long long action,
 
     if (lookup_failed_sync_node(join_from, &value) == pcmk_rc_ok) {
         crm_err("Rejecting join-%d request from node %s because we failed to "
-                "sync its CIB in join-%d " CRM_XS " ref=%s",
+                "sync its CIB in join-%d " QB_XS " ref=%s",
                 join_id, join_from, value, ref);
         ack_nack_bool = FALSE;
 
@@ -451,23 +515,23 @@ do_dc_join_filter_offer(long long action,
              * cause for alarm.
              */
             crm_debug("Rejecting join-%d request from inactive node %s "
-                      CRM_XS " ref=%s", join_id, join_from, ref);
+                      QB_XS " ref=%s", join_id, join_from, ref);
         } else {
             crm_err("Rejecting join-%d request from inactive node %s "
-                    CRM_XS " ref=%s", join_id, join_from, ref);
+                    QB_XS " ref=%s", join_id, join_from, ref);
         }
         ack_nack_bool = FALSE;
 
     } else if (generation == NULL) {
         crm_err("Rejecting invalid join-%d request from node %s "
-                "missing CIB generation " CRM_XS " ref=%s",
+                "missing CIB generation " QB_XS " ref=%s",
                 join_id, join_from, ref);
         ack_nack_bool = FALSE;
 
     } else if ((join_version == NULL)
                || !feature_set_compatible(CRM_FEATURE_SET, join_version)) {
         crm_err("Rejecting join-%d request from node %s because feature set %s"
-                " is incompatible with ours (%s) " CRM_XS " ref=%s",
+                " is incompatible with ours (%s) " QB_XS " ref=%s",
                 join_id, join_from, (join_version? join_version : "pre-3.1.0"),
                 CRM_FEATURE_SET, ref);
         ack_nack_bool = FALSE;
@@ -478,74 +542,67 @@ do_dc_join_filter_offer(long long action,
 
         if (pcmk__get_schema(validation) == NULL) {
             crm_err("Rejecting join-%d request from %s (with first CIB "
-                    "generation) due to unknown schema version %s "
-                    CRM_XS " ref=%s",
-                    join_id, join_from, pcmk__s(validation, "(missing)"), ref);
+                    "generation) due to %s schema version %s " QB_XS " ref=%s",
+                    join_id, join_from,
+                    ((validation == NULL)? "missing" : "unknown"),
+                    pcmk__s(validation, ""), ref);
             ack_nack_bool = FALSE;
 
         } else {
             crm_debug("Accepting join-%d request from %s (with first CIB "
-                      "generation) " CRM_XS " ref=%s",
+                      "generation) " QB_XS " ref=%s",
                       join_id, join_from, ref);
             max_generation_xml = pcmk__xml_copy(NULL, generation);
             pcmk__str_update(&max_generation_from, join_from);
         }
 
     } else if ((cmp < 0)
-               || ((cmp == 0)
-                   && pcmk__str_eq(join_from, controld_globals.our_nodename,
-                                   pcmk__str_casei))) {
+               || ((cmp == 0) && controld_is_local_node(join_from))) {
         const char *validation = crm_element_value(generation,
                                                    PCMK_XA_VALIDATE_WITH);
 
         if (pcmk__get_schema(validation) == NULL) {
             crm_err("Rejecting join-%d request from %s (with better CIB "
-                    "generation than current best from %s) due to unknown "
-                    "schema version %s " CRM_XS " ref=%s",
+                    "generation than current best from %s) due to %s "
+                    "schema version %s " QB_XS " ref=%s",
                     join_id, join_from, max_generation_from,
-                    pcmk__s(validation, "(missing)"), ref);
+                    ((validation == NULL)? "missing" : "unknown"),
+                    pcmk__s(validation, ""), ref);
             ack_nack_bool = FALSE;
 
         } else {
             crm_debug("Accepting join-%d request from %s (with better CIB "
-                      "generation than current best from %s) " CRM_XS " ref=%s",
+                      "generation than current best from %s) " QB_XS " ref=%s",
                       join_id, join_from, max_generation_from, ref);
             crm_log_xml_debug(max_generation_xml, "Old max generation");
             crm_log_xml_debug(generation, "New max generation");
 
-            free_xml(max_generation_xml);
+            pcmk__xml_free(max_generation_xml);
             max_generation_xml = pcmk__xml_copy(NULL, join_ack->xml);
             pcmk__str_update(&max_generation_from, join_from);
         }
 
     } else {
-        crm_debug("Accepting join-%d request from %s " CRM_XS " ref=%s",
+        crm_debug("Accepting join-%d request from %s " QB_XS " ref=%s",
                   join_id, join_from, ref);
     }
 
     if (!ack_nack_bool) {
-        if (compare_version(join_version, "3.17.0") < 0) {
-            /* Clients with CRM_FEATURE_SET < 3.17.0 may respawn infinitely
-             * after a nack message, don't send one
-             */
-            crm_update_peer_join(__func__, join_node, crm_join_nack_quiet);
-        } else {
-            crm_update_peer_join(__func__, join_node, crm_join_nack);
-        }
+        crm_update_peer_join(__func__, join_node, controld_join_nack);
         pcmk__update_peer_expected(__func__, join_node, CRMD_JOINSTATE_NACK);
 
     } else {
-        crm_update_peer_join(__func__, join_node, crm_join_integrated);
+        crm_update_peer_join(__func__, join_node, controld_join_integrated);
         pcmk__update_peer_expected(__func__, join_node, CRMD_JOINSTATE_MEMBER);
     }
 
-    count = crmd_join_phase_count(crm_join_integrated);
+    count = crmd_join_phase_count(controld_join_integrated);
     crm_debug("%d node%s currently integrated in join-%d",
               count, pcmk__plural_s(count), join_id);
 
     if (check_join_state(cur_state, __func__) == FALSE) {
         // Don't waste time by invoking the scheduler yet
-        count = crmd_join_phase_count(crm_join_welcomed);
+        count = crmd_join_phase_count(controld_join_welcomed);
         crm_debug("Waiting on join-%d requests from %d outstanding node%s",
                   join_id, count, pcmk__plural_s(count));
     }
@@ -560,10 +617,9 @@ do_dc_join_finalize(long long action,
 {
     char *sync_from = NULL;
     int rc = pcmk_ok;
-    int count_welcomed = crmd_join_phase_count(crm_join_welcomed);
-    int count_finalizable = crmd_join_phase_count(crm_join_integrated)
-                            + crmd_join_phase_count(crm_join_nack)
-                            + crmd_join_phase_count(crm_join_nack_quiet);
+    int count_welcomed = crmd_join_phase_count(controld_join_welcomed);
+    int count_finalizable = crmd_join_phase_count(controld_join_integrated)
+                            + crmd_join_phase_count(controld_join_nack);
 
     /* This we can do straight away and avoid clients timing us out
      *  while we compute the latest CIB
@@ -585,8 +641,8 @@ do_dc_join_finalize(long long action,
     }
 
     controld_clear_fsa_input_flags(R_HAVE_CIB);
-    if (pcmk__str_eq(max_generation_from, controld_globals.our_nodename,
-                     pcmk__str_null_matches|pcmk__str_casei)) {
+    if ((max_generation_from == NULL)
+        || controld_is_local_node(max_generation_from)) {
         controld_set_fsa_input_flags(R_HAVE_CIB);
     }
 
@@ -600,7 +656,7 @@ do_dc_join_finalize(long long action,
 
     if (pcmk_is_set(controld_globals.fsa_input_register, R_HAVE_CIB)) {
         // Send our CIB out to everyone
-        sync_from = pcmk__str_copy(controld_globals.our_nodename);
+        sync_from = pcmk__str_copy(controld_globals.cluster->priv->node_name);
         crm_debug("Finalizing join-%d for %d node%s (sync'ing from local CIB)",
                   current_join_id, count_finalizable,
                   pcmk__plural_s(count_finalizable));
@@ -627,7 +683,7 @@ free_max_generation(void)
     free(max_generation_from);
     max_generation_from = NULL;
 
-    free_xml(max_generation_xml);
+    pcmk__xml_free(max_generation_xml);
     max_generation_xml = NULL;
 }
 
@@ -666,14 +722,13 @@ finalize_sync_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, voi
         if (!check_join_state(controld_globals.fsa_state, __func__)) {
             int count_finalizable = 0;
 
-            count_finalizable = crmd_join_phase_count(crm_join_integrated)
-                                + crmd_join_phase_count(crm_join_nack)
-                                + crmd_join_phase_count(crm_join_nack_quiet);
+            count_finalizable = crmd_join_phase_count(controld_join_integrated)
+                                + crmd_join_phase_count(controld_join_nack);
 
             crm_debug("Notifying %d node%s of join-%d results",
                       count_finalizable, pcmk__plural_s(count_finalizable),
                       current_join_id);
-            g_hash_table_foreach(crm_peer_cache, finalize_join_for, NULL);
+            g_hash_table_foreach(pcmk__peer_cache, finalize_join_for, NULL);
         }
     }
 }
@@ -712,7 +767,8 @@ do_dc_join_ack(long long action,
 
     const char *op = crm_element_value(join_ack->msg, PCMK__XA_CRM_TASK);
     char *join_from = crm_element_value_copy(join_ack->msg, PCMK__XA_SRC);
-    crm_node_t *peer = NULL;
+    pcmk__node_status_t *peer = NULL;
+    enum controld_join_phase phase = controld_join_none;
 
     enum controld_section_e section = controld_section_lrm;
     char *xpath = NULL;
@@ -745,11 +801,12 @@ do_dc_join_ack(long long action,
     }
 
     peer = pcmk__get_node(0, join_from, NULL, pcmk__node_search_cluster_member);
-    if (peer->join != crm_join_finalized) {
+    phase = controld_get_join_phase(peer);
+    if (phase != controld_join_finalized) {
         crm_info("Ignoring out-of-sequence join-%d confirmation from %s "
                  "(currently %s not %s)",
-                 join_id, join_from, crm_join_phase_str(peer->join),
-                 crm_join_phase_str(crm_join_finalized));
+                 join_id, join_from, join_phase_text(phase),
+                 join_phase_text(controld_join_finalized));
         goto done;
     }
 
@@ -757,11 +814,11 @@ do_dc_join_ack(long long action,
         crm_err("Rejecting join-%d confirmation from %s "
                 "because currently on join-%d",
                 join_id, join_from, current_join_id);
-        crm_update_peer_join(__func__, peer, crm_join_nack);
+        crm_update_peer_join(__func__, peer, controld_join_nack);
         goto done;
     }
 
-    crm_update_peer_join(__func__, peer, crm_join_confirmed);
+    crm_update_peer_join(__func__, peer, controld_join_confirmed);
 
     /* Update CIB with node's current executor state. A new transition will be
      * triggered later, when the CIB manager notifies us of the change.
@@ -780,17 +837,13 @@ do_dc_join_ack(long long action,
     controld_node_state_deletion_strings(join_from, section, &xpath, NULL);
 
     rc = cib->cmds->remove(cib, xpath, NULL,
-                           cib_scope_local
-                           |cib_xpath
-                           |cib_multiple
-                           |cib_transaction);
+                           cib_xpath|cib_multiple|cib_transaction);
     if (rc != pcmk_ok) {
         goto done;
     }
 
     // Update CIB with node's latest known executor state
-    if (pcmk__str_eq(join_from, controld_globals.our_nodename,
-                     pcmk__str_casei)) {
+    if (controld_is_local_node(join_from)) {
 
         // Use the latest possible state if processing our own join ack
         execd_state = controld_query_executor_state();
@@ -813,14 +866,14 @@ do_dc_join_ack(long long action,
     }
 
     rc = cib->cmds->modify(cib, PCMK_XE_STATUS, state,
-                           cib_scope_local|cib_can_create|cib_transaction);
-    free_xml(execd_state);
+                           cib_can_create|cib_transaction);
+    pcmk__xml_free(execd_state);
     if (rc != pcmk_ok) {
         goto done;
     }
 
     // Commit the transaction
-    rc = cib->cmds->end_transaction(cib, true, cib_scope_local);
+    rc = cib->cmds->end_transaction(cib, true, cib_none);
     fsa_register_cib_callback(rc, join_from, join_node_state_commit_callback);
 
     if (rc > 0) {
@@ -844,21 +897,21 @@ finalize_join_for(gpointer key, gpointer value, gpointer user_data)
 {
     xmlNode *acknak = NULL;
     xmlNode *tmp1 = NULL;
-    crm_node_t *join_node = value;
-    const char *join_to = join_node->uname;
+    pcmk__node_status_t *join_node = value;
+    const char *join_to = join_node->name;
+    enum controld_join_phase phase = controld_get_join_phase(join_node);
     bool integrated = false;
 
-    switch (join_node->join) {
-        case crm_join_integrated:
+    switch (phase) {
+        case controld_join_integrated:
             integrated = true;
             break;
-        case crm_join_nack:
-        case crm_join_nack_quiet:
+        case controld_join_nack:
             break;
         default:
             crm_trace("Not updating non-integrated and non-nacked node %s (%s) "
-                      "for join-%d", join_to,
-                      crm_join_phase_str(join_node->join), current_join_id);
+                      "for join-%d",
+                      join_to, join_phase_text(phase), current_join_id);
             return;
     }
 
@@ -870,13 +923,7 @@ finalize_join_for(gpointer key, gpointer value, gpointer user_data)
     crm_xml_add(tmp1, PCMK_XA_ID, pcmk__cluster_node_uuid(join_node));
     crm_xml_add(tmp1, PCMK_XA_UNAME, join_to);
     fsa_cib_anon_update(PCMK_XE_NODES, tmp1);
-    free_xml(tmp1);
-
-    if (join_node->join == crm_join_nack_quiet) {
-        crm_trace("Not sending nack message to node %s with feature set older "
-                  "than 3.17.0", join_to);
-        return;
-    }
+    pcmk__xml_free(tmp1);
 
     join_node = pcmk__get_node(0, join_to, NULL,
                                pcmk__node_search_cluster_member);
@@ -902,7 +949,7 @@ finalize_join_for(gpointer key, gpointer value, gpointer user_data)
 
     if (integrated) {
         // No change needed for a nacked node
-        crm_update_peer_join(__func__, join_node, crm_join_finalized);
+        crm_update_peer_join(__func__, join_node, controld_join_finalized);
         pcmk__update_peer_expected(__func__, join_node, CRMD_JOINSTATE_MEMBER);
 
         /* Iterate through the remote peer cache and add information on which
@@ -911,10 +958,10 @@ finalize_join_for(gpointer key, gpointer value, gpointer user_data)
          */
         if (pcmk__cluster_num_remote_nodes() > 0) {
             GHashTableIter iter;
-            crm_node_t *node = NULL;
+            pcmk__node_status_t *node = NULL;
             xmlNode *remotes = pcmk__xe_create(acknak, PCMK_XE_NODES);
 
-            g_hash_table_iter_init(&iter, crm_remote_peer_cache);
+            g_hash_table_iter_init(&iter, pcmk__remote_peer_cache);
             while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &node)) {
                 xmlNode *remote = NULL;
 
@@ -924,15 +971,15 @@ finalize_join_for(gpointer key, gpointer value, gpointer user_data)
 
                 remote = pcmk__xe_create(remotes, PCMK_XE_NODE);
                 pcmk__xe_set_props(remote,
-                                   PCMK_XA_ID, node->uname,
+                                   PCMK_XA_ID, node->name,
                                    PCMK__XA_NODE_STATE, node->state,
                                    PCMK__XA_CONNECTION_HOST, node->conn_host,
                                    NULL);
             }
         }
     }
-    pcmk__cluster_send_message(join_node, crm_msg_crmd, acknak);
-    free_xml(acknak);
+    pcmk__cluster_send_message(join_node, pcmk_ipc_controld, acknak);
+    pcmk__xml_free(acknak);
     return;
 }
 
@@ -941,23 +988,24 @@ check_join_state(enum crmd_fsa_state cur_state, const char *source)
 {
     static unsigned long long highest_seq = 0;
 
-    if (controld_globals.membership_id != crm_peer_seq) {
+    if (controld_globals.membership_id != controld_globals.peer_seq) {
         crm_debug("join-%d: Membership changed from %llu to %llu "
-                  CRM_XS " highest=%llu state=%s for=%s",
-                  current_join_id, controld_globals.membership_id, crm_peer_seq,
-                  highest_seq, fsa_state2string(cur_state), source);
-        if(highest_seq < crm_peer_seq) {
+                  QB_XS " highest=%llu state=%s for=%s",
+                  current_join_id, controld_globals.membership_id,
+                  controld_globals.peer_seq, highest_seq,
+                  fsa_state2string(cur_state), source);
+        if (highest_seq < controld_globals.peer_seq) {
             /* Don't spam the FSA with duplicates */
-            highest_seq = crm_peer_seq;
+            highest_seq = controld_globals.peer_seq;
             register_fsa_input_before(C_FSA_INTERNAL, I_NODE_JOIN, NULL);
         }
 
     } else if (cur_state == S_INTEGRATION) {
-        if (crmd_join_phase_count(crm_join_welcomed) == 0) {
-            int count = crmd_join_phase_count(crm_join_integrated);
+        if (crmd_join_phase_count(controld_join_welcomed) == 0) {
+            int count = crmd_join_phase_count(controld_join_integrated);
 
             crm_debug("join-%d: Integration of %d peer%s complete "
-                      CRM_XS " state=%s for=%s",
+                      QB_XS " state=%s for=%s",
                       current_join_id, count, pcmk__plural_s(count),
                       fsa_state2string(cur_state), source);
             register_fsa_input_before(C_FSA_INTERNAL, I_INTEGRATED, NULL);
@@ -967,39 +1015,39 @@ check_join_state(enum crmd_fsa_state cur_state, const char *source)
     } else if (cur_state == S_FINALIZE_JOIN) {
         if (!pcmk_is_set(controld_globals.fsa_input_register, R_HAVE_CIB)) {
             crm_debug("join-%d: Delaying finalization until we have CIB "
-                      CRM_XS " state=%s for=%s",
+                      QB_XS " state=%s for=%s",
                       current_join_id, fsa_state2string(cur_state), source);
             return TRUE;
 
-        } else if (crmd_join_phase_count(crm_join_welcomed) != 0) {
-            int count = crmd_join_phase_count(crm_join_welcomed);
+        } else if (crmd_join_phase_count(controld_join_welcomed) != 0) {
+            int count = crmd_join_phase_count(controld_join_welcomed);
 
             crm_debug("join-%d: Still waiting on %d welcomed node%s "
-                      CRM_XS " state=%s for=%s",
+                      QB_XS " state=%s for=%s",
                       current_join_id, count, pcmk__plural_s(count),
                       fsa_state2string(cur_state), source);
             crmd_join_phase_log(LOG_DEBUG);
 
-        } else if (crmd_join_phase_count(crm_join_integrated) != 0) {
-            int count = crmd_join_phase_count(crm_join_integrated);
+        } else if (crmd_join_phase_count(controld_join_integrated) != 0) {
+            int count = crmd_join_phase_count(controld_join_integrated);
 
             crm_debug("join-%d: Still waiting on %d integrated node%s "
-                      CRM_XS " state=%s for=%s",
+                      QB_XS " state=%s for=%s",
                       current_join_id, count, pcmk__plural_s(count),
                       fsa_state2string(cur_state), source);
             crmd_join_phase_log(LOG_DEBUG);
 
-        } else if (crmd_join_phase_count(crm_join_finalized) != 0) {
-            int count = crmd_join_phase_count(crm_join_finalized);
+        } else if (crmd_join_phase_count(controld_join_finalized) != 0) {
+            int count = crmd_join_phase_count(controld_join_finalized);
 
             crm_debug("join-%d: Still waiting on %d finalized node%s "
-                      CRM_XS " state=%s for=%s",
+                      QB_XS " state=%s for=%s",
                       current_join_id, count, pcmk__plural_s(count),
                       fsa_state2string(cur_state), source);
             crmd_join_phase_log(LOG_DEBUG);
 
         } else {
-            crm_debug("join-%d: Complete " CRM_XS " state=%s for=%s",
+            crm_debug("join-%d: Complete " QB_XS " state=%s for=%s",
                       current_join_id, fsa_state2string(cur_state), source);
             register_fsa_input_later(C_FSA_INTERNAL, I_FINALIZED, NULL);
             return TRUE;
@@ -1016,18 +1064,18 @@ do_dc_join_final(long long action,
                  enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
     crm_debug("Ensuring DC, quorum and node attributes are up-to-date");
-    crm_update_quorum(crm_have_quorum, TRUE);
+    crm_update_quorum(pcmk__cluster_has_quorum(), TRUE);
 }
 
-int crmd_join_phase_count(enum crm_join_phase phase)
+int crmd_join_phase_count(enum controld_join_phase phase)
 {
     int count = 0;
-    crm_node_t *peer;
+    pcmk__node_status_t *peer;
     GHashTableIter iter;
 
-    g_hash_table_iter_init(&iter, crm_peer_cache);
+    g_hash_table_iter_init(&iter, pcmk__peer_cache);
     while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &peer)) {
-        if(peer->join == phase) {
+        if (controld_get_join_phase(peer) == phase) {
             count++;
         }
     }
@@ -1036,12 +1084,12 @@ int crmd_join_phase_count(enum crm_join_phase phase)
 
 void crmd_join_phase_log(int level)
 {
-    crm_node_t *peer;
+    pcmk__node_status_t *peer;
     GHashTableIter iter;
 
-    g_hash_table_iter_init(&iter, crm_peer_cache);
+    g_hash_table_iter_init(&iter, pcmk__peer_cache);
     while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &peer)) {
-        do_crm_log(level, "join-%d: %s=%s", current_join_id, peer->uname,
-                   crm_join_phase_str(peer->join));
+        do_crm_log(level, "join-%d: %s=%s", current_join_id, peer->name,
+                   join_phase_text(controld_get_join_phase(peer)));
     }
 }
