@@ -23,11 +23,12 @@
 
 static enum crmd_fsa_input handle_message(xmlNode *msg,
                                           enum crmd_fsa_cause cause);
+static xmlNode* create_ping_reply(const xmlNode *msg);
 static void handle_response(xmlNode *stored_msg);
 static enum crmd_fsa_input handle_request(xmlNode *stored_msg,
                                           enum crmd_fsa_cause cause);
 static enum crmd_fsa_input handle_shutdown_request(xmlNode *stored_msg);
-static void send_msg_via_ipc(xmlNode * msg, const char *sys);
+static void send_msg_via_ipc(xmlNode * msg, const char *sys, const char *src);
 
 /* debug only, can wrap all it likes */
 static int last_data_id = 0;
@@ -218,7 +219,7 @@ delete_fsa_input(fsa_data_t * fsa_data)
 
             case fsa_dt_xml:
                 foo = fsa_data->data;
-                free_xml(foo);
+                pcmk__xml_free(foo);
                 break;
 
             case fsa_dt_lrm:
@@ -267,7 +268,7 @@ fsa_typed_data_adv(fsa_data_t * fsa_data, enum fsa_data_type a_type, const char
     } else if (fsa_data->data_type != a_type) {
         crm_crit("%s: Message data was the wrong type! %d vs. requested=%d.  Origin: %s",
                  caller, fsa_data->data_type, a_type, fsa_data->origin);
-        CRM_ASSERT(fsa_data->data_type == a_type);
+        pcmk__assert(fsa_data->data_type == a_type);
     } else {
         ret_val = fsa_data->data;
     }
@@ -328,7 +329,7 @@ route_message(enum crmd_fsa_cause cause, xmlNode * input)
 gboolean
 relay_message(xmlNode * msg, gboolean originated_locally)
 {
-    enum crm_ais_msg_types dest = crm_msg_none;
+    enum pcmk_ipc_server dest = pcmk_ipc_unknown;
     bool is_for_dc = false;
     bool is_for_dcib = false;
     bool is_for_te = false;
@@ -342,7 +343,7 @@ relay_message(xmlNode * msg, gboolean originated_locally)
     const char *type = NULL;
     const char *task = NULL;
     const char *ref = NULL;
-    crm_node_t *node_to = NULL;
+    pcmk__node_status_t *node_to = NULL;
 
     CRM_CHECK(msg != NULL, return TRUE);
 
@@ -366,7 +367,7 @@ relay_message(xmlNode * msg, gboolean originated_locally)
         return TRUE;
     }
 
-    // Require message type (set by create_request())
+    // Require message type (set by pcmk__new_request())
     if (!pcmk__str_eq(type, PCMK__VALUE_CRMD, pcmk__str_none)) {
         crm_warn("Ignoring invalid message %s with type '%s' "
                  "(not '" PCMK__VALUE_CRMD "')",
@@ -375,7 +376,7 @@ relay_message(xmlNode * msg, gboolean originated_locally)
         return TRUE;
     }
 
-    // Require a destination subsystem (also set by create_request())
+    // Require a destination subsystem (also set by pcmk__new_request())
     if (sys_to == NULL) {
         crm_warn("Ignoring invalid message %s with no " PCMK__XA_CRM_SYS_TO,
                  ref);
@@ -385,13 +386,13 @@ relay_message(xmlNode * msg, gboolean originated_locally)
 
     // Get the message type appropriate to the destination subsystem
     if (pcmk_get_cluster_layer() == pcmk_cluster_layer_corosync) {
-        dest = pcmk__cluster_parse_msg_type(sys_to);
-        if (dest == crm_msg_none) {
+        dest = pcmk__parse_server(sys_to);
+        if (dest == pcmk_ipc_unknown) {
             /* Unrecognized value, use a sane default
              *
              * @TODO Maybe we should bail instead
              */
-            dest = crm_msg_crmd;
+            dest = pcmk_ipc_controld;
         }
     }
 
@@ -424,8 +425,7 @@ relay_message(xmlNode * msg, gboolean originated_locally)
             is_local = true;
         }
 
-    } else if (pcmk__str_eq(controld_globals.our_nodename, host_to,
-                            pcmk__str_casei)) {
+    } else if (controld_is_local_node(host_to)) {
         is_local = true;
 
     } else if (is_for_crm && pcmk__str_eq(task, CRM_OP_LRM_DELETE, pcmk__str_casei)) {
@@ -440,6 +440,18 @@ relay_message(xmlNode * msg, gboolean originated_locally)
         }
     }
 
+    // If is for DC and DC is not yet selected
+    if (is_for_dc && pcmk__str_eq(task, CRM_OP_PING, pcmk__str_casei) 
+        && (controld_globals.dc_name == NULL)) {
+            
+        xmlNode *reply = create_ping_reply(msg);
+        sys_to = crm_element_value(reply, PCMK__XA_CRM_SYS_TO);
+        // Explicitly leave src empty. It indicates that dc is "not yet selected"
+        send_msg_via_ipc(reply, sys_to, NULL);
+        pcmk__xml_free(reply);
+        return TRUE;
+    }
+
     // Check whether message should be relayed
 
     if (is_for_dc || is_for_dcib || is_for_te) {
@@ -448,7 +460,8 @@ relay_message(xmlNode * msg, gboolean originated_locally)
                 crm_trace("Route message %s locally as transition request",
                           ref);
                 crm_log_xml_trace(msg, sys_to);
-                send_msg_via_ipc(msg, sys_to);
+                send_msg_via_ipc(msg, sys_to, controld_globals.cluster->priv->node_name);
+
                 return TRUE; // No further processing of message is needed
             }
             crm_trace("Route message %s locally as DC request", ref);
@@ -484,7 +497,7 @@ relay_message(xmlNode * msg, gboolean originated_locally)
         }
         crm_trace("Relay message %s locally to %s", ref, sys_to);
         crm_log_xml_trace(msg, "IPC-relay");
-        send_msg_via_ipc(msg, sys_to);
+        send_msg_via_ipc(msg, sys_to, controld_globals.cluster->priv->node_name);
         return TRUE;
     }
 
@@ -518,7 +531,7 @@ authorize_version(xmlNode *message_data, const char *field,
         || (version_num < 0LL)) {
 
         crm_warn("Rejected IPC hello from %s: '%s' is not a valid protocol %s "
-                 CRM_XS " ref=%s uuid=%s",
+                 QB_XS " ref=%s uuid=%s",
                  client_name, ((version == NULL)? "" : version),
                  field, (ref? ref : "none"), uuid);
         return false;
@@ -554,7 +567,7 @@ controld_authorize_ipc_message(const xmlNode *client_msg, pcmk__client_t *curr_c
 
     if (uuid == NULL) {
         crm_warn("IPC message from client rejected: No client identifier "
-                 CRM_XS " ref=%s", (ref? ref : "none"));
+                 QB_XS " ref=%s", (ref? ref : "none"));
         goto rejected;
     }
 
@@ -569,7 +582,7 @@ controld_authorize_ipc_message(const xmlNode *client_msg, pcmk__client_t *curr_c
     client_name = crm_element_value(message_data, PCMK__XA_CLIENT_NAME);
     if (pcmk__str_empty(client_name)) {
         crm_warn("IPC hello from client rejected: No client name",
-                 CRM_XS " ref=%s uuid=%s", (ref? ref : "none"), uuid);
+                 QB_XS " ref=%s uuid=%s", (ref? ref : "none"), uuid);
         goto rejected;
     }
     if (!authorize_version(message_data, PCMK__XA_MAJOR_VERSION, client_name,
@@ -777,7 +790,7 @@ handle_remote_state(const xmlNode *msg)
 {
     const char *conn_host = NULL;
     const char *remote_uname = pcmk__xe_id(msg);
-    crm_node_t *remote_peer;
+    pcmk__node_status_t *remote_peer;
     bool remote_is_up = false;
     int rc = pcmk_rc_ok;
 
@@ -789,7 +802,7 @@ handle_remote_state(const xmlNode *msg)
     CRM_CHECK(remote_peer, return I_NULL);
 
     pcmk__update_peer_state(__func__, remote_peer,
-                            remote_is_up ? CRM_NODE_MEMBER : CRM_NODE_LOST,
+                            remote_is_up ? PCMK_VALUE_MEMBER : PCMK__VALUE_LOST,
                             0);
 
     conn_host = crm_element_value(msg, PCMK__XA_CONNECTION_HOST);
@@ -810,8 +823,8 @@ handle_remote_state(const xmlNode *msg)
  *
  * \return Next FSA input
  */
-static enum crmd_fsa_input
-handle_ping(const xmlNode *msg)
+static xmlNode*
+create_ping_reply(const xmlNode *msg)
 {
     const char *value = NULL;
     xmlNode *ping = NULL;
@@ -832,12 +845,18 @@ handle_ping(const xmlNode *msg)
     // @TODO maybe do some checks to determine meaningful status
     crm_xml_add(ping, PCMK_XA_RESULT, "ok");
 
-    // Send reply
-    reply = create_reply(msg, ping);
-    free_xml(ping);
+    reply = pcmk__new_reply(msg, ping);
+    pcmk__xml_free(ping);
+    return reply;
+}
+
+static enum crmd_fsa_input
+handle_ping(const xmlNode *msg)
+{
+    xmlNode *reply = create_ping_reply(msg);
     if (reply != NULL) {
         (void) relay_message(reply, TRUE);
-        free_xml(reply);
+        pcmk__xml_free(reply);
     }
 
     // Nothing further to do
@@ -855,27 +874,28 @@ static enum crmd_fsa_input
 handle_node_list(const xmlNode *request)
 {
     GHashTableIter iter;
-    crm_node_t *node = NULL;
+    pcmk__node_status_t *node = NULL;
     xmlNode *reply = NULL;
     xmlNode *reply_data = NULL;
 
     // Create message data for reply
     reply_data = pcmk__xe_create(NULL, PCMK_XE_NODES);
-    g_hash_table_iter_init(&iter, crm_peer_cache);
+    g_hash_table_iter_init(&iter, pcmk__peer_cache);
     while (g_hash_table_iter_next(&iter, NULL, (gpointer *) & node)) {
         xmlNode *xml = pcmk__xe_create(reply_data, PCMK_XE_NODE);
 
-        crm_xml_add_ll(xml, PCMK_XA_ID, (long long) node->id); // uint32_t
-        crm_xml_add(xml, PCMK_XA_UNAME, node->uname);
+        crm_xml_add_ll(xml, PCMK_XA_ID,
+                       (long long) node->cluster_layer_id); // uint32_t
+        crm_xml_add(xml, PCMK_XA_UNAME, node->name);
         crm_xml_add(xml, PCMK__XA_IN_CCM, node->state);
     }
 
     // Create and send reply
-    reply = create_reply(request, reply_data);
-    free_xml(reply_data);
+    reply = pcmk__new_reply(request, reply_data);
+    pcmk__xml_free(reply_data);
     if (reply) {
         (void) relay_message(reply, TRUE);
-        free_xml(reply);
+        pcmk__xml_free(reply);
     }
 
     // Nothing further to do
@@ -893,7 +913,7 @@ static enum crmd_fsa_input
 handle_node_info_request(const xmlNode *msg)
 {
     const char *value = NULL;
-    crm_node_t *node = NULL;
+    pcmk__node_status_t *node = NULL;
     int node_id = 0;
     xmlNode *reply = NULL;
     xmlNode *reply_data = NULL;
@@ -921,24 +941,25 @@ handle_node_info_request(const xmlNode *msg)
 
     // Default to local node if none given
     if ((node_id == 0) && (value == NULL)) {
-        value = controld_globals.our_nodename;
+        value = controld_globals.cluster->priv->node_name;
     }
 
     node = pcmk__search_node_caches(node_id, value, pcmk__node_search_any);
     if (node) {
-        crm_xml_add(reply_data, PCMK_XA_ID, node->uuid);
-        crm_xml_add(reply_data, PCMK_XA_UNAME, node->uname);
+        crm_xml_add(reply_data, PCMK_XA_ID, node->xml_id);
+        crm_xml_add(reply_data, PCMK_XA_UNAME, node->name);
         crm_xml_add(reply_data, PCMK_XA_CRMD, node->state);
         pcmk__xe_set_bool_attr(reply_data, PCMK_XA_REMOTE_NODE,
-                               pcmk_is_set(node->flags, crm_remote_node));
+                               pcmk_is_set(node->flags,
+                                           pcmk__node_status_remote));
     }
 
     // Send reply
-    reply = create_reply(msg, reply_data);
-    free_xml(reply_data);
+    reply = pcmk__new_reply(msg, reply_data);
+    pcmk__xml_free(reply_data);
     if (reply != NULL) {
         (void) relay_message(reply, TRUE);
-        free_xml(reply);
+        pcmk__xml_free(reply);
     }
 
     // Nothing further to do
@@ -1049,7 +1070,7 @@ handle_request(xmlNode *stored_msg, enum crmd_fsa_cause cause)
 
     if (strcmp(op, CRM_OP_SHUTDOWN_REQ) == 0) {
         const char *from = crm_element_value(stored_msg, PCMK__XA_SRC);
-        crm_node_t *node =
+        pcmk__node_status_t *node =
             pcmk__search_node_caches(0, from, pcmk__node_search_cluster_member);
 
         pcmk__update_peer_expected(__func__, node, CRMD_JOINSTATE_DOWN);
@@ -1134,7 +1155,6 @@ handle_request(xmlNode *stored_msg, enum crmd_fsa_cause cause)
         return handle_lrm_delete(stored_msg);
 
     } else if ((strcmp(op, CRM_OP_LRM_FAIL) == 0)
-               || (strcmp(op, CRM_OP_LRM_REFRESH) == 0) // @COMPAT
                || (strcmp(op, CRM_OP_REPROBE) == 0)) {
 
         crm_xml_add(stored_msg, PCMK__XA_CRM_SYS_TO, CRM_SYSTEM_LRMD);
@@ -1157,13 +1177,15 @@ handle_request(xmlNode *stored_msg, enum crmd_fsa_cause cause)
         name = crm_element_value(stored_msg, PCMK_XA_UNAME);
 
         if(cause == C_IPC_MESSAGE) {
-            msg = create_request(CRM_OP_RM_NODE_CACHE, NULL, NULL, CRM_SYSTEM_CRMD, CRM_SYSTEM_CRMD, NULL);
-            if (!pcmk__cluster_send_message(NULL, crm_msg_crmd, msg)) {
+            msg = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_CRMD, NULL,
+                                    CRM_SYSTEM_CRMD, CRM_OP_RM_NODE_CACHE,
+                                    NULL);
+            if (!pcmk__cluster_send_message(NULL, pcmk_ipc_controld, msg)) {
                 crm_err("Could not instruct peers to remove references to node %s/%u", name, id);
             } else {
                 crm_notice("Instructing peers to remove references to node %s/%u", name, id);
             }
-            free_xml(msg);
+            pcmk__xml_free(msg);
 
         } else {
             pcmk__cluster_forget_cluster_node(id, name);
@@ -1254,7 +1276,7 @@ handle_shutdown_request(xmlNode * stored_msg)
 
     if (host_from == NULL) {
         /* we're shutting down and the DC */
-        host_from = controld_globals.our_nodename;
+        host_from = controld_globals.cluster->priv->node_name;
     }
 
     crm_info("Creating shutdown request for %s (state=%s)", host_from,
@@ -1270,7 +1292,7 @@ handle_shutdown_request(xmlNode * stored_msg)
 }
 
 static void
-send_msg_via_ipc(xmlNode * msg, const char *sys)
+send_msg_via_ipc(xmlNode * msg, const char *sys, const char *src)
 {
     pcmk__client_t *client_channel = NULL;
 
@@ -1279,7 +1301,7 @@ send_msg_via_ipc(xmlNode * msg, const char *sys)
     client_channel = pcmk__find_client_by_id(sys);
 
     if (crm_element_value(msg, PCMK__XA_SRC) == NULL) {
-        crm_xml_add(msg, PCMK__XA_SRC, controld_globals.our_nodename);
+        crm_xml_add(msg, PCMK__XA_SRC, src);
     }
 
     if (client_channel != NULL) {
@@ -1328,7 +1350,7 @@ delete_ha_msg_input(ha_msg_input_t * orig)
     if (orig == NULL) {
         return;
     }
-    free_xml(orig->msg);
+    pcmk__xml_free(orig->msg);
     free(orig);
 }
 
@@ -1342,8 +1364,9 @@ delete_ha_msg_input(ha_msg_input_t * orig)
 void
 broadcast_remote_state_message(const char *node_name, bool node_up)
 {
-    xmlNode *msg = create_request(CRM_OP_REMOTE_STATE, NULL, NULL,
-                                  CRM_SYSTEM_CRMD, CRM_SYSTEM_CRMD, NULL);
+    xmlNode *msg = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_CRMD, NULL,
+                                     CRM_SYSTEM_CRMD, CRM_OP_REMOTE_STATE,
+                                     NULL);
 
     crm_info("Notifying cluster of Pacemaker Remote node %s %s",
              node_name, node_up? "coming up" : "going down");
@@ -1353,10 +1376,10 @@ broadcast_remote_state_message(const char *node_name, bool node_up)
 
     if (node_up) {
         crm_xml_add(msg, PCMK__XA_CONNECTION_HOST,
-                    controld_globals.our_nodename);
+                    controld_globals.cluster->priv->node_name);
     }
 
-    pcmk__cluster_send_message(NULL, crm_msg_crmd, msg);
-    free_xml(msg);
+    pcmk__cluster_send_message(NULL, pcmk_ipc_controld, msg);
+    pcmk__xml_free(msg);
 }
 
