@@ -164,7 +164,7 @@ recurring_helper(gpointer data)
     lrm_state_t *connection_rsc = NULL;
 
     cmd->interval_id = 0;
-    connection_rsc = lrm_state_find(cmd->rsc_id);
+    connection_rsc = controld_get_executor_state(cmd->rsc_id, false);
     if (connection_rsc && connection_rsc->remote_ra_data) {
         remote_ra_data_t *ra_data = connection_rsc->remote_ra_data;
 
@@ -183,7 +183,7 @@ start_delay_helper(gpointer data)
     lrm_state_t *connection_rsc = NULL;
 
     cmd->delay_id = 0;
-    connection_rsc = lrm_state_find(cmd->rsc_id);
+    connection_rsc = controld_get_executor_state(cmd->rsc_id, false);
     if (connection_rsc && connection_rsc->remote_ra_data) {
         remote_ra_data_t *ra_data = connection_rsc->remote_ra_data;
 
@@ -193,14 +193,13 @@ start_delay_helper(gpointer data)
 }
 
 static bool
-should_purge_attributes(crm_node_t *node)
+should_purge_attributes(pcmk__node_status_t *node)
 {
-    bool purge = true;
-    crm_node_t *conn_node = NULL;
+    pcmk__node_status_t *conn_node = NULL;
     lrm_state_t *connection_rsc = NULL;
 
-    if (!node->conn_host) {
-        return purge;
+    if ((node->conn_host == NULL) || (node->name == NULL)) {
+        return true;
     }
 
     /* Get the node that was hosting the remote connection resource from the
@@ -209,14 +208,14 @@ should_purge_attributes(crm_node_t *node)
     conn_node = pcmk__get_node(0, node->conn_host, NULL,
                                pcmk__node_search_cluster_member);
     if (conn_node == NULL) {
-        return purge;
+        return true;
     }
 
     /* Check the uptime of connection_rsc.  If it hasn't been running long
      * enough, set purge=true.  "Long enough" means it started running earlier
      * than the timestamp when we noticed it went away in the first place.
      */
-    connection_rsc = lrm_state_find(node->uname);
+    connection_rsc = controld_get_executor_state(node->name, false);
 
     if (connection_rsc != NULL) {
         lrmd_t *lrm = connection_rsc->conn;
@@ -230,11 +229,11 @@ should_purge_attributes(crm_node_t *node)
         if (uptime > 0 &&
             conn_node->peer_lost > 0 &&
             uptime + 20 >= now - conn_node->peer_lost) {
-            purge = false;
+            return false;
         }
     }
 
-    return purge;
+    return true;
 }
 
 static enum controld_section_e
@@ -256,17 +255,17 @@ section_to_delete(bool purge)
 }
 
 static void
-purge_remote_node_attrs(int call_opt, crm_node_t *node)
+purge_remote_node_attrs(int call_opt, pcmk__node_status_t *node)
 {
     bool purge = should_purge_attributes(node);
     enum controld_section_e section = section_to_delete(purge);
 
     /* Purge node from attrd's memory */
     if (purge) {
-        update_attrd_remote_node_removed(node->uname, NULL);
+        update_attrd_remote_node_removed(node->name, NULL);
     }
 
-    controld_delete_node_state(node->uname, section, call_opt);
+    controld_delete_node_state(node->name, section, call_opt);
 }
 
 /*!
@@ -280,7 +279,7 @@ remote_node_up(const char *node_name)
 {
     int call_opt;
     xmlNode *update, *state;
-    crm_node_t *node;
+    pcmk__node_status_t *node = NULL;
     lrm_state_t *connection_rsc = NULL;
 
     CRM_CHECK(node_name != NULL, return);
@@ -288,32 +287,34 @@ remote_node_up(const char *node_name)
 
     call_opt = crmd_cib_smart_opt();
 
-    /* Delete node's probe_complete attribute. This serves two purposes:
+    /* Delete node's CRM_OP_PROBED attribute. Deleting any attribute ensures
+     * that the attribute manager learns the node is remote. Deletion of this
+     * specfic attribute is a holdover from when it had special meaning.
      *
-     * - @COMPAT DCs < 1.1.14 in a rolling upgrade might use it
-     * - deleting it (or any attribute for that matter) here ensures the
-     *   attribute manager learns the node is remote
+     * @COMPAT Find another way to tell attrd that the node is remote, without
+     * risking deletion or overwrite of an arbitrary attribute. Then work on
+     * deprecating CRM_OP_PROBED.
      */
     update_attrd(node_name, CRM_OP_PROBED, NULL, NULL, TRUE);
 
     /* Ensure node is in the remote peer cache with member status */
     node = pcmk__cluster_lookup_remote_node(node_name);
-    CRM_CHECK(node != NULL, return);
+    CRM_CHECK((node != NULL) && (node->name != NULL), return);
 
     purge_remote_node_attrs(call_opt, node);
-    pcmk__update_peer_state(__func__, node, CRM_NODE_MEMBER, 0);
+    pcmk__update_peer_state(__func__, node, PCMK_VALUE_MEMBER, 0);
 
     /* Apply any start state that we were given from the environment on the
      * remote node.
      */
-    connection_rsc = lrm_state_find(node->uname);
+    connection_rsc = controld_get_executor_state(node->name, false);
 
     if (connection_rsc != NULL) {
         lrmd_t *lrm = connection_rsc->conn;
         const char *start_state = lrmd__node_start_state(lrm);
 
         if (start_state) {
-            set_join_state(start_state, node->uname, node->uuid, true);
+            set_join_state(start_state, node->name, node->xml_id, true);
         }
     }
 
@@ -343,7 +344,7 @@ remote_node_up(const char *node_name)
      * soon. Ideally, we wouldn't rely on the CIB for the fenced status.
      */
     controld_update_cib(PCMK_XE_STATUS, update, call_opt, NULL);
-    free_xml(update);
+    pcmk__xml_free(update);
 }
 
 enum down_opts {
@@ -363,7 +364,7 @@ remote_node_down(const char *node_name, const enum down_opts opts)
 {
     xmlNode *update;
     int call_opt = crmd_cib_smart_opt();
-    crm_node_t *node;
+    pcmk__node_status_t *node = NULL;
 
     /* Purge node from attrd's memory */
     update_attrd_remote_node_removed(node_name, NULL);
@@ -382,7 +383,7 @@ remote_node_down(const char *node_name, const enum down_opts opts)
     /* Ensure node is in the remote peer cache with lost state */
     node = pcmk__cluster_lookup_remote_node(node_name);
     CRM_CHECK(node != NULL, return);
-    pcmk__update_peer_state(__func__, node, CRM_NODE_LOST, 0);
+    pcmk__update_peer_state(__func__, node, PCMK__VALUE_LOST, 0);
 
     /* Notify DC */
     broadcast_remote_state_message(node_name, false);
@@ -391,7 +392,7 @@ remote_node_down(const char *node_name, const enum down_opts opts)
     update = pcmk__xe_create(NULL, PCMK_XE_STATUS);
     create_node_state_update(node, node_update_cluster, update, __func__);
     controld_update_cib(PCMK_XE_STATUS, update, call_opt, NULL);
-    free_xml(update);
+    pcmk__xml_free(update);
 }
 
 /*!
@@ -420,13 +421,15 @@ check_remote_node_state(const remote_ra_cmd_t *cmd)
          * it hasn't been tracking the remote node, and other code relies on
          * the cache to distinguish remote nodes from unseen cluster nodes.
          */
-        crm_node_t *node = pcmk__cluster_lookup_remote_node(cmd->rsc_id);
+        pcmk__node_status_t *node =
+            pcmk__cluster_lookup_remote_node(cmd->rsc_id);
 
         CRM_CHECK(node != NULL, return);
-        pcmk__update_peer_state(__func__, node, CRM_NODE_MEMBER, 0);
+        pcmk__update_peer_state(__func__, node, PCMK_VALUE_MEMBER, 0);
 
     } else if (pcmk__str_eq(cmd->action, PCMK_ACTION_STOP, pcmk__str_casei)) {
-        lrm_state_t *lrm_state = lrm_state_find(cmd->rsc_id);
+        lrm_state_t *lrm_state = controld_get_executor_state(cmd->rsc_id,
+                                                             false);
         remote_ra_data_t *ra_data = lrm_state? lrm_state->remote_ra_data : NULL;
 
         if (ra_data) {
@@ -467,14 +470,14 @@ report_remote_ra_result(remote_ra_cmd_t * cmd)
     op.user_data = cmd->userdata;
     op.timeout = cmd->timeout;
     op.interval_ms = cmd->interval_ms;
-    op.t_run = (unsigned int) cmd->start_time;
-    op.t_rcchange = (unsigned int) cmd->start_time;
+    op.t_run = cmd->start_time;
+    op.t_rcchange = cmd->start_time;
 
     lrmd__set_result(&op, cmd->result.exit_status, cmd->result.execution_status,
                      cmd->result.exit_reason);
 
     if (pcmk_is_set(cmd->status, cmd_reported_success) && !pcmk__result_ok(&(cmd->result))) {
-        op.t_rcchange = (unsigned int) time(NULL);
+        op.t_rcchange = time(NULL);
         /* This edge case will likely never ever occur, but if it does the
          * result is that a failure will not be processed correctly. This is only
          * remotely possible because we are able to detect a connection resource's tcp
@@ -566,7 +569,7 @@ connection_takeover_timeout_cb(gpointer data)
     crm_info("takeover event timed out for node %s", cmd->rsc_id);
     cmd->takeover_timeout_id = 0;
 
-    lrm_state = lrm_state_find(cmd->rsc_id);
+    lrm_state = controld_get_executor_state(cmd->rsc_id, false);
 
     handle_remote_ra_stop(lrm_state, cmd);
     free_cmd(cmd);
@@ -580,7 +583,7 @@ monitor_timeout_cb(gpointer data)
     lrm_state_t *lrm_state = NULL;
     remote_ra_cmd_t *cmd = data;
 
-    lrm_state = lrm_state_find(cmd->rsc_id);
+    lrm_state = controld_get_executor_state(cmd->rsc_id, false);
 
     crm_info("Timed out waiting for remote poke response from %s%s",
              cmd->rsc_id, (lrm_state? "" : " (no LRM state)"));
@@ -615,14 +618,14 @@ synthesize_lrmd_success(lrm_state_t *lrm_state, const char *rsc_id, const char *
 
     if (lrm_state == NULL) {
         /* if lrm_state not given assume local */
-        lrm_state = lrm_state_find(controld_globals.our_nodename);
+        lrm_state = controld_get_executor_state(NULL, false);
     }
-    CRM_ASSERT(lrm_state != NULL);
+    pcmk__assert(lrm_state != NULL);
 
     op.type = lrmd_event_exec_complete;
     op.rsc_id = rsc_id;
     op.op_type = op_type;
-    op.t_run = (unsigned int) time(NULL);
+    op.t_run = time(NULL);
     op.t_rcchange = op.t_run;
     op.call_id = generate_callid();
     lrmd__set_result(&op, PCMK_OCF_OK, PCMK_EXEC_DONE, NULL);
@@ -637,14 +640,16 @@ remote_lrm_op_callback(lrmd_event_data_t * op)
     remote_ra_data_t *ra_data = NULL;
     remote_ra_cmd_t *cmd = NULL;
 
+    CRM_CHECK((op != NULL) && (op->remote_nodename != NULL), return);
+
     crm_debug("Processing '%s%s%s' event on remote connection to %s: %s "
               "(%d) status=%s (%d)",
               (op->op_type? op->op_type : ""), (op->op_type? " " : ""),
               lrmd_event_type2str(op->type), op->remote_nodename,
-              services_ocf_exitcode_str(op->rc), op->rc,
+              crm_exit_str((crm_exit_t) op->rc), op->rc,
               pcmk_exec_status_str(op->op_status), op->op_status);
 
-    lrm_state = lrm_state_find(op->remote_nodename);
+    lrm_state = controld_get_executor_state(op->remote_nodename, false);
     if (!lrm_state || !lrm_state->remote_ra_data) {
         crm_debug("No state information found for remote connection event");
         return;
@@ -729,7 +734,7 @@ remote_lrm_op_callback(lrmd_event_data_t * op)
 
             } else if (cmd->remaining_timeout > 3000) {
                 crm_trace("rescheduling start, remaining timeout %d", cmd->remaining_timeout);
-                g_timeout_add(1000, retry_start_cmd_cb, lrm_state);
+                pcmk__create_timer(1000, retry_start_cmd_cb, lrm_state);
                 return;
 
             } else {
@@ -774,8 +779,8 @@ remote_lrm_op_callback(lrmd_event_data_t * op)
         /* success, keep rescheduling if interval is present. */
         if (cmd->interval_ms && !pcmk_is_set(cmd->status, cmd_cancel)) {
             ra_data->recurring_cmds = g_list_append(ra_data->recurring_cmds, cmd);
-            cmd->interval_id = g_timeout_add(cmd->interval_ms,
-                                             recurring_helper, cmd);
+            cmd->interval_id = pcmk__create_timer(cmd->interval_ms,
+                                                  recurring_helper, cmd);
             cmd = NULL;         /* prevent free */
         }
         cmd_handled = TRUE;
@@ -795,13 +800,6 @@ remote_lrm_op_callback(lrmd_event_data_t * op)
         }
         cmd_handled = TRUE;
 
-    } else if ((op->type == lrmd_event_new_client)
-               && pcmk__str_eq(cmd->action, PCMK_ACTION_STOP,
-                               pcmk__str_casei)) {
-
-        handle_remote_ra_stop(lrm_state, cmd);
-        cmd_handled = TRUE;
-
     } else {
         crm_debug("Event did not match %s action", ra_data->cur_cmd->action);
     }
@@ -820,7 +818,7 @@ handle_remote_ra_stop(lrm_state_t * lrm_state, remote_ra_cmd_t * cmd)
 {
     remote_ra_data_t *ra_data = NULL;
 
-    CRM_ASSERT(lrm_state);
+    pcmk__assert(lrm_state != NULL);
     ra_data = lrm_state->remote_ra_data;
 
     if (!pcmk_is_set(ra_data->status, takeover_complete)) {
@@ -944,7 +942,7 @@ handle_remote_ra_exec(gpointer user_data)
                 crm_debug("Poked Pacemaker Remote at node %s, waiting for async response",
                           cmd->rsc_id);
                 ra_data->cur_cmd = cmd;
-                cmd->monitor_timeout_id = g_timeout_add(cmd->timeout, monitor_timeout_cb, cmd);
+                cmd->monitor_timeout_id = pcmk__create_timer(cmd->timeout, monitor_timeout_cb, cmd);
                 return TRUE;
             }
             report_remote_ra_result(cmd);
@@ -958,7 +956,9 @@ handle_remote_ra_exec(gpointer user_data)
                  * cleared which will require all the resources running in the remote-node
                  * to be explicitly re-detected via probe actions.  If the takeover does occur
                  * successfully, then we can leave the status section intact. */
-                cmd->takeover_timeout_id = g_timeout_add((cmd->timeout/2), connection_takeover_timeout_cb, cmd);
+                cmd->takeover_timeout_id = pcmk__create_timer((cmd->timeout/2),
+                                                              connection_takeover_timeout_cb,
+                                                              cmd);
                 ra_data->cur_cmd = cmd;
                 return TRUE;
             }
@@ -1033,12 +1033,8 @@ is_remote_lrmd_ra(const char *agent, const char *provider, const char *id)
     if (agent && provider && !strcmp(agent, REMOTE_LRMD_RA) && !strcmp(provider, "pacemaker")) {
         return TRUE;
     }
-    if ((id != NULL) && (lrm_state_find(id) != NULL)
-        && !pcmk__str_eq(id, controld_globals.our_nodename, pcmk__str_casei)) {
-        return TRUE;
-    }
-
-    return FALSE;
+    return (id != NULL) && (controld_get_executor_state(id, false) != NULL)
+           && !controld_is_local_node(id);
 }
 
 lrmd_rsc_info_t *
@@ -1046,7 +1042,9 @@ remote_ra_get_rsc_info(lrm_state_t * lrm_state, const char *rsc_id)
 {
     lrmd_rsc_info_t *info = NULL;
 
-    if ((lrm_state_find(rsc_id))) {
+    CRM_CHECK(rsc_id != NULL, return NULL);
+
+    if (controld_get_executor_state(rsc_id, false) != NULL) {
         info = pcmk__assert_alloc(1, sizeof(lrmd_rsc_info_t));
 
         info->id = pcmk__str_copy(rsc_id);
@@ -1134,7 +1132,9 @@ remote_ra_cancel(lrm_state_t *lrm_state, const char *rsc_id,
     lrm_state_t *connection_rsc = NULL;
     remote_ra_data_t *ra_data = NULL;
 
-    connection_rsc = lrm_state_find(rsc_id);
+    CRM_CHECK(rsc_id != NULL, return -EINVAL);
+
+    connection_rsc = controld_get_executor_state(rsc_id, false);
     if (!connection_rsc || !connection_rsc->remote_ra_data) {
         return -EINVAL;
     }
@@ -1270,7 +1270,7 @@ controld_execute_remote_agent(const lrm_state_t *lrm_state, const char *rsc_id,
         return EOPNOTSUPP;
     }
 
-    connection_rsc = lrm_state_find(rsc_id);
+    connection_rsc = controld_get_executor_state(rsc_id, false);
     if (connection_rsc == NULL) {
         lrmd_key_value_freeall(params);
         return ENOTCONN;
@@ -1301,7 +1301,7 @@ controld_execute_remote_agent(const lrm_state_t *lrm_state, const char *rsc_id,
     cmd->call_id = generate_callid();
 
     if (cmd->start_delay) {
-        cmd->delay_id = g_timeout_add(cmd->start_delay, start_delay_helper, cmd);
+        cmd->delay_id = pcmk__create_timer(cmd->start_delay, start_delay_helper, cmd);
     }
 
     ra_data->cmds = g_list_append(ra_data->cmds, cmd);
@@ -1320,8 +1320,11 @@ controld_execute_remote_agent(const lrm_state_t *lrm_state, const char *rsc_id,
 void
 remote_ra_fail(const char *node_name)
 {
-    lrm_state_t *lrm_state = lrm_state_find(node_name);
+    lrm_state_t *lrm_state = NULL;
 
+    CRM_CHECK(node_name != NULL, return);
+
+    lrm_state = controld_get_executor_state(node_name, false);
     if (lrm_state && lrm_state_is_connected(lrm_state)) {
         remote_ra_data_t *ra_data = lrm_state->remote_ra_data;
 
@@ -1389,7 +1392,7 @@ remote_ra_maintenance(lrm_state_t * lrm_state, gboolean maintenance)
 {
     xmlNode *update, *state;
     int call_opt;
-    crm_node_t *node;
+    pcmk__node_status_t *node = NULL;
 
     call_opt = crmd_cib_smart_opt();
     node = pcmk__cluster_lookup_remote_node(lrm_state->node_name);
@@ -1407,7 +1410,7 @@ remote_ra_maintenance(lrm_state_t * lrm_state, gboolean maintenance)
             lrm_remote_clear_flags(lrm_state, remote_in_maint);
         }
     }
-    free_xml(update);
+    pcmk__xml_free(update);
 }
 
 #define XPATH_PSEUDO_MAINTENANCE "//" PCMK__XE_PSEUDO_EVENT         \
@@ -1431,11 +1434,18 @@ remote_ra_process_maintenance_nodes(xmlNode *xml)
 
         for (node = pcmk__xe_first_child(getXpathResult(search, 0),
                                          PCMK_XE_NODE, NULL, NULL);
-             node != NULL; node = pcmk__xe_next_same(node)) {
+             node != NULL; node = pcmk__xe_next(node, PCMK_XE_NODE)) {
 
-            lrm_state_t *lrm_state = lrm_state_find(pcmk__xe_id(node));
+            lrm_state_t *lrm_state = NULL;
+            const char *id = pcmk__xe_id(node);
 
             cnt++;
+            if (id == NULL) {
+                continue; // Shouldn't be possible
+            }
+
+            lrm_state = controld_get_executor_state(id, false);
+
             if (lrm_state && lrm_state->remote_ra_data &&
                 pcmk_is_set(((remote_ra_data_t *) lrm_state->remote_ra_data)->status, remote_active)) {
 
