@@ -62,9 +62,9 @@ save_cib_contents(xmlNode *msg, int call_id, int rc, xmlNode *output,
     CRM_CHECK(id != NULL, return);
 
     if (rc == pcmk_ok) {
-        char *filename = crm_strdup_printf(PE_STATE_DIR "/pe-core-%s.bz2", id);
+        char *filename = crm_strdup_printf(PCMK_SCHEDULER_INPUT_DIR "/pe-core-%s.bz2", id);
 
-        if (pcmk__xml_write_file(output, filename, true, NULL) != pcmk_rc_ok) {
+        if (pcmk__xml_write_file(output, filename, true) != pcmk_rc_ok) {
             crm_err("Could not save Cluster Information Base to %s after scheduler crash",
                     filename);
         } else {
@@ -90,7 +90,7 @@ handle_disconnect(void)
         char *uuid_str = crm_generate_uuid();
 
         crm_crit("Lost connection to the scheduler "
-                 CRM_XS " CIB will be saved to " PE_STATE_DIR "/pe-core-%s.bz2",
+                 QB_XS " CIB will be saved to " PCMK_SCHEDULER_INPUT_DIR "/pe-core-%s.bz2",
                  uuid_str);
 
         /*
@@ -104,8 +104,7 @@ handle_disconnect(void)
          *
          */
         rc = controld_globals.cib_conn->cmds->query(controld_globals.cib_conn,
-                                                    NULL, NULL,
-                                                    cib_scope_local);
+                                                    NULL, NULL, cib_none);
         fsa_register_cib_callback(rc, uuid_str, save_cib_contents);
     }
 
@@ -152,7 +151,7 @@ handle_reply(pcmk_schedulerd_api_reply_t *reply)
         pcmk__xml_copy(crm_data_node, reply->data.graph.tgraph);
         register_fsa_input_later(C_IPC_MESSAGE, I_PE_SUCCESS, &fsa_input);
 
-        free_xml(fsa_input.msg);
+        pcmk__xml_free(fsa_input.msg);
 
     } else {
         crm_info("%s calculation %s is obsolete", CRM_OP_PECALC, msg_ref);
@@ -360,7 +359,7 @@ do_pe_invoke(long long action,
         return;
     }
 
-    fsa_pe_query = cib_conn->cmds->query(cib_conn, NULL, NULL, cib_scope_local);
+    fsa_pe_query = cib_conn->cmds->query(cib_conn, NULL, NULL, cib_none);
 
     crm_debug("Query %d: Requesting the current CIB: %s", fsa_pe_query,
               fsa_state2string(controld_globals.fsa_state));
@@ -431,8 +430,8 @@ force_local_option(xmlNode *xml, const char *attr_name, const char *attr_value)
 
         xml = pcmk__xe_create(cluster_property_set, PCMK_XE_NVPAIR);
 
-        crm_xml_set_id(xml, "%s-%s",
-                       PCMK_VALUE_CIB_BOOTSTRAP_OPTIONS, attr_name);
+        pcmk__xe_set_id(xml, "%s-%s",
+                        PCMK_VALUE_CIB_BOOTSTRAP_OPTIONS, attr_name);
         crm_xml_add(xml, PCMK_XA_NAME, attr_name);
         crm_xml_add(xml, PCMK_XA_VALUE, attr_value);
     }
@@ -447,7 +446,7 @@ do_pe_invoke_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
 
     if (rc != pcmk_ok) {
         crm_err("Could not retrieve the Cluster Information Base: %s "
-                CRM_XS " rc=%d call=%d", pcmk_strerror(rc), rc, call_id);
+                QB_XS " rc=%d call=%d", pcmk_strerror(rc), rc, call_id);
         register_fsa_error_adv(C_FSA_INTERNAL, I_ERROR, NULL, NULL, __func__);
         return;
 
@@ -490,22 +489,24 @@ do_pe_invoke_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
     force_local_option(output, PCMK_OPT_HAVE_WATCHDOG, pcmk__btoa(watchdog));
 
     if (pcmk_is_set(controld_globals.flags, controld_ever_had_quorum)
-        && !crm_have_quorum) {
+        && !pcmk__cluster_has_quorum()) {
+
         crm_xml_add_int(output, PCMK_XA_NO_QUORUM_PANIC, 1);
     }
 
-    rc = pcmk_rc2legacy(pcmk_schedulerd_api_graph(schedulerd_api, output, &ref));
-
-    if (rc < 0) {
-        crm_err("Could not contact the scheduler: %s " CRM_XS " rc=%d",
-                pcmk_strerror(rc), rc);
+    rc = pcmk_schedulerd_api_graph(schedulerd_api, output, &ref);
+    if (rc != pcmk_rc_ok) {
+        free(ref);
+        crm_err("Could not contact the scheduler: %s " QB_XS " rc=%d",
+                pcmk_rc_str(rc), rc);
         register_fsa_error_adv(C_FSA_INTERNAL, I_ERROR, NULL, NULL, __func__);
     } else {
-        CRM_ASSERT(ref != NULL);
+        pcmk__assert(ref != NULL);
         controld_expect_sched_reply(ref);
         crm_debug("Invoking the scheduler: query=%d, ref=%s, seq=%llu, "
                   "quorate=%s",
-                  fsa_pe_query, controld_globals.fsa_pe_ref, crm_peer_seq,
+                  fsa_pe_query, controld_globals.fsa_pe_ref,
+                  controld_globals.peer_seq,
                   pcmk__flag_text(controld_globals.flags, controld_has_quorum));
     }
 }
