@@ -26,9 +26,9 @@ static void te_update_job_count(pcmk__graph_action_t *action, int offset);
 static void
 te_start_action_timer(const pcmk__graph_t *graph, pcmk__graph_action_t *action)
 {
-    action->timer = g_timeout_add(action->timeout + graph->network_delay,
-                                  action_timer_callback, (void *) action);
-    CRM_ASSERT(action->timer != 0);
+    action->timer = pcmk__create_timer(action->timeout + graph->network_delay,
+                                       action_timer_callback, action);
+    pcmk__assert(action->timer != 0);
 }
 
 /*!
@@ -48,21 +48,21 @@ execute_pseudo_action(pcmk__graph_t *graph, pcmk__graph_action_t *pseudo)
     /* send to peers as well? */
     if (pcmk__str_eq(task, PCMK_ACTION_MAINTENANCE_NODES, pcmk__str_casei)) {
         GHashTableIter iter;
-        crm_node_t *node = NULL;
+        pcmk__node_status_t *node = NULL;
 
-        g_hash_table_iter_init(&iter, crm_peer_cache);
+        g_hash_table_iter_init(&iter, pcmk__peer_cache);
         while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &node)) {
             xmlNode *cmd = NULL;
 
-            if (pcmk__str_eq(controld_globals.our_nodename, node->uname,
-                             pcmk__str_casei)) {
+            if (controld_is_local_node(node->name)) {
                 continue;
             }
 
-            cmd = create_request(task, pseudo->xml, node->uname,
-                                 CRM_SYSTEM_CRMD, CRM_SYSTEM_TENGINE, NULL);
-            pcmk__cluster_send_message(node, crm_msg_crmd, cmd);
-            free_xml(cmd);
+            cmd = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_TENGINE,
+                                    node->name, CRM_SYSTEM_CRMD, task,
+                                    pseudo->xml);
+            pcmk__cluster_send_message(node, pcmk_ipc_controld, cmd);
+            pcmk__xml_free(cmd);
         }
 
         remote_ra_process_maintenance_nodes(pseudo->xml);
@@ -112,7 +112,7 @@ execute_cluster_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
     gboolean rc = TRUE;
     gboolean no_wait = FALSE;
 
-    const crm_node_t *node = NULL;
+    const pcmk__node_status_t *node = NULL;
 
     id = pcmk__xe_id(action->xml);
     CRM_CHECK(!pcmk__str_empty(id), return EPROTO);
@@ -130,13 +130,12 @@ execute_cluster_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
             const char *mode = crm_element_value(action->xml, PCMK__XA_MODE);
 
             if (pcmk__str_eq(mode, PCMK__VALUE_CIB, pcmk__str_none)) {
-                router_node = controld_globals.our_nodename;
+                router_node = controld_globals.cluster->priv->node_name;
             }
         }
     }
 
-    if (pcmk__str_eq(router_node, controld_globals.our_nodename,
-                     pcmk__str_casei)) {
+    if (controld_is_local_node(router_node)) {
         is_local = TRUE;
     }
 
@@ -159,13 +158,15 @@ execute_cluster_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
         return pcmk_rc_ok;
 
     } else if (pcmk__str_eq(task, PCMK_ACTION_DO_SHUTDOWN, pcmk__str_none)) {
-        crm_node_t *peer = pcmk__get_node(0, router_node, NULL,
-                                          pcmk__node_search_cluster_member);
+        pcmk__node_status_t *peer =
+            pcmk__get_node(0, router_node, NULL,
+                           pcmk__node_search_cluster_member);
 
         pcmk__update_peer_expected(__func__, peer, CRMD_JOINSTATE_DOWN);
     }
 
-    cmd = create_request(task, action->xml, router_node, CRM_SYSTEM_CRMD, CRM_SYSTEM_TENGINE, NULL);
+    cmd = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_TENGINE, router_node,
+                            CRM_SYSTEM_CRMD, task, action->xml);
 
     counter = pcmk__transition_key(controld_globals.transition_graph->id,
                                    action->id, get_target_rc(action),
@@ -174,9 +175,9 @@ execute_cluster_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
 
     node = pcmk__get_node(0, router_node, NULL,
                           pcmk__node_search_cluster_member);
-    rc = pcmk__cluster_send_message(node, crm_msg_crmd, cmd);
+    rc = pcmk__cluster_send_message(node, pcmk_ipc_controld, cmd);
     free(counter);
-    free_xml(cmd);
+    pcmk__xml_free(cmd);
 
     if (rc == FALSE) {
         crm_err("Action %d failed: send", action->id);
@@ -306,10 +307,9 @@ controld_record_action_event(pcmk__graph_action_t *action,
     pcmk__create_history_xml(rsc, op, CRM_FEATURE_SET, target_rc, target,
                              __func__);
 
-    rc = cib_conn->cmds->modify(cib_conn, PCMK_XE_STATUS, state,
-                                cib_scope_local);
+    rc = cib_conn->cmds->modify(cib_conn, PCMK_XE_STATUS, state, cib_none);
     fsa_register_cib_callback(rc, NULL, cib_action_updated);
-    free_xml(state);
+    pcmk__xml_free(state);
 
     crm_trace("Sent CIB update (call ID %d) for synthesized event of action %d (%s on %s)",
               rc, action->id, task_uuid, target);
@@ -367,8 +367,7 @@ execute_rsc_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
     const char *router_node = NULL;
     const char *task_uuid = NULL;
 
-    CRM_ASSERT(action != NULL);
-    CRM_ASSERT(action->xml != NULL);
+    pcmk__assert((action != NULL) && (action->xml != NULL));
 
     pcmk__clear_graph_action_flags(action, pcmk__graph_action_executed);
     on_node = crm_element_value(action->xml, PCMK__META_ON_NODE);
@@ -392,8 +391,7 @@ execute_rsc_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
                                    controld_globals.te_uuid);
     crm_xml_add(rsc_op, PCMK__XA_TRANSITION_KEY, counter);
 
-    if (pcmk__str_eq(router_node, controld_globals.our_nodename,
-                     pcmk__str_casei)) {
+    if (controld_is_local_node(router_node)) {
         is_local = TRUE;
     }
 
@@ -402,12 +400,12 @@ execute_rsc_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
         no_wait = TRUE;
     }
 
-    crm_notice("Initiating %s operation %s%s on %s%s "CRM_XS" action %d",
+    crm_notice("Initiating %s operation %s%s on %s%s " QB_XS " action %d",
                task, task_uuid, (is_local? " locally" : ""), on_node,
                (no_wait? " without waiting" : ""), action->id);
 
-    cmd = create_request(CRM_OP_INVOKE_LRM, rsc_op, router_node,
-                         CRM_SYSTEM_LRMD, CRM_SYSTEM_TENGINE, NULL);
+    cmd = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_TENGINE, router_node,
+                            CRM_SYSTEM_LRMD, CRM_OP_INVOKE_LRM, rsc_op);
 
     if (is_local) {
         /* shortcut local resource commands */
@@ -430,15 +428,15 @@ execute_rsc_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
                       I_NULL, &msg);
 
     } else {
-        const crm_node_t *node =
+        const pcmk__node_status_t *node =
             pcmk__get_node(0, router_node, NULL,
                            pcmk__node_search_cluster_member);
 
-        rc = pcmk__cluster_send_message(node, crm_msg_lrmd, cmd);
+        rc = pcmk__cluster_send_message(node, pcmk_ipc_execd, cmd);
     }
 
     free(counter);
-    free_xml(cmd);
+    pcmk__xml_free(cmd);
 
     pcmk__set_graph_action_flags(action, pcmk__graph_action_executed);
 
