@@ -17,11 +17,6 @@
 
 #include <pacemaker-controld.h>
 
-void te_update_confirm(const char *event, xmlNode * msg);
-
-#define RSC_OP_PREFIX "//" PCMK__XE_DIFF_ADDED "//" PCMK_XE_CIB \
-                      "//" PCMK__XE_LRM_RSC_OP "[@" PCMK_XA_ID "='"
-
 // An explicit PCMK_OPT_SHUTDOWN_LOCK of 0 means the lock has been cleared
 static bool
 shutdown_lock_cleared(xmlNode *lrm_resource)
@@ -33,187 +28,11 @@ shutdown_lock_cleared(xmlNode *lrm_resource)
            && (shutdown_lock == 0);
 }
 
-static void
-te_update_diff_v1(const char *event, xmlNode *diff)
-{
-    int lpc, max;
-    xmlXPathObject *xpathObj = NULL;
-    GString *rsc_op_xpath = NULL;
-
-    CRM_CHECK(diff != NULL, return);
-
-    pcmk__output_set_log_level(controld_globals.logger_out, LOG_TRACE);
-    controld_globals.logger_out->message(controld_globals.logger_out,
-                                         "xml-patchset", diff);
-
-    if (cib__config_changed_v1(NULL, NULL, &diff)) {
-        abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
-                         "Non-status change", diff);
-        goto bail;              /* configuration changed */
-    }
-
-    /* Tickets Attributes - Added/Updated */
-    xpathObj =
-        xpath_search(diff,
-                     "//" PCMK__XE_CIB_UPDATE_RESULT
-                     "//" PCMK__XE_DIFF_ADDED
-                     "//" PCMK_XE_TICKETS);
-    if (numXpathResults(xpathObj) > 0) {
-        xmlNode *aborted = getXpathResult(xpathObj, 0);
-
-        abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
-                         "Ticket attribute: update", aborted);
-        goto bail;
-
-    }
-    freeXpathObject(xpathObj);
-
-    /* Tickets Attributes - Removed */
-    xpathObj =
-        xpath_search(diff,
-                     "//" PCMK__XE_CIB_UPDATE_RESULT
-                     "//" PCMK__XE_DIFF_REMOVED
-                     "//" PCMK_XE_TICKETS);
-    if (numXpathResults(xpathObj) > 0) {
-        xmlNode *aborted = getXpathResult(xpathObj, 0);
-
-        abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
-                         "Ticket attribute: removal", aborted);
-        goto bail;
-    }
-    freeXpathObject(xpathObj);
-
-    /* Transient Attributes - Removed */
-    xpathObj =
-        xpath_search(diff,
-                     "//" PCMK__XE_CIB_UPDATE_RESULT
-                     "//" PCMK__XE_DIFF_REMOVED
-                     "//" PCMK__XE_TRANSIENT_ATTRIBUTES);
-    if (numXpathResults(xpathObj) > 0) {
-        xmlNode *aborted = getXpathResult(xpathObj, 0);
-
-        abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
-                         "Transient attribute: removal", aborted);
-        goto bail;
-
-    }
-    freeXpathObject(xpathObj);
-
-    // Check for PCMK__XE_LRM_RESOURCE entries
-    xpathObj = xpath_search(diff,
-                            "//" PCMK__XE_CIB_UPDATE_RESULT
-                            "//" PCMK__XE_DIFF_ADDED
-                            "//" PCMK__XE_LRM_RESOURCE);
-    max = numXpathResults(xpathObj);
-
-    /*
-     * Updates by, or in response to, graph actions will never affect more than
-     * one resource at a time, so such updates indicate an LRM refresh. In that
-     * case, start a new transition rather than check each result individually,
-     * which can result in _huge_ speedups in large clusters.
-     *
-     * Unfortunately, we can only do so when there are no pending actions.
-     * Otherwise, we could mistakenly throw away those results here, and
-     * the cluster will stall waiting for them and time out the operation.
-     */
-    if ((controld_globals.transition_graph->pending == 0) && (max > 1)) {
-        crm_debug("Ignoring resource operation updates due to history refresh of %d resources",
-                  max);
-        crm_log_xml_trace(diff, "lrm-refresh");
-        abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
-                         "History refresh", NULL);
-        goto bail;
-    }
-
-    if (max == 1) {
-        xmlNode *lrm_resource = getXpathResult(xpathObj, 0);
-
-        if (shutdown_lock_cleared(lrm_resource)) {
-            // @TODO would be more efficient to abort once after transition done
-            abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
-                             "Shutdown lock cleared", lrm_resource);
-            // Still process results, so we stop timers and update failcounts
-        }
-    }
-    freeXpathObject(xpathObj);
-
-    /* Process operation updates */
-    xpathObj =
-        xpath_search(diff,
-                     "//" PCMK__XE_CIB_UPDATE_RESULT
-                     "//" PCMK__XE_DIFF_ADDED
-                     "//" PCMK__XE_LRM_RSC_OP);
-    max = numXpathResults(xpathObj);
-    if (max > 0) {
-        int lpc = 0;
-
-        for (lpc = 0; lpc < max; lpc++) {
-            xmlNode *rsc_op = getXpathResult(xpathObj, lpc);
-            const char *node = get_node_id(rsc_op);
-
-            process_graph_event(rsc_op, node);
-        }
-    }
-    freeXpathObject(xpathObj);
-
-    /* Detect deleted (as opposed to replaced or added) actions - eg. crm_resource -C */
-    xpathObj = xpath_search(diff,
-                            "//" PCMK__XE_DIFF_REMOVED
-                            "//" PCMK__XE_LRM_RSC_OP);
-    max = numXpathResults(xpathObj);
-    for (lpc = 0; lpc < max; lpc++) {
-        const char *op_id = NULL;
-        xmlXPathObject *op_match = NULL;
-        xmlNode *match = getXpathResult(xpathObj, lpc);
-
-        CRM_LOG_ASSERT(match != NULL);
-        if(match == NULL) { continue; };
-
-        op_id = pcmk__xe_id(match);
-
-        if (rsc_op_xpath == NULL) {
-            rsc_op_xpath = g_string_new(RSC_OP_PREFIX);
-        } else {
-            g_string_truncate(rsc_op_xpath, sizeof(RSC_OP_PREFIX) - 1);
-        }
-        pcmk__g_strcat(rsc_op_xpath, op_id, "']", NULL);
-
-        op_match = xpath_search(diff, (const char *) rsc_op_xpath->str);
-        if (numXpathResults(op_match) == 0) {
-            /* Prevent false positives by matching cancelations too */
-            const char *node = get_node_id(match);
-            pcmk__graph_action_t *cancelled = get_cancel_action(op_id, node);
-
-            if (cancelled == NULL) {
-                crm_debug("No match for deleted action %s (%s on %s)",
-                          (const char *) rsc_op_xpath->str, op_id, node);
-                abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
-                                 "Resource op removal", match);
-                freeXpathObject(op_match);
-                goto bail;
-
-            } else {
-                crm_debug("Deleted " PCMK__XE_LRM_RSC_OP " %s on %s was for "
-                          "graph event %d",
-                          op_id, node, cancelled->id);
-            }
-        }
-
-        freeXpathObject(op_match);
-    }
-
-  bail:
-    freeXpathObject(xpathObj);
-    if (rsc_op_xpath != NULL) {
-        g_string_free(rsc_op_xpath, TRUE);
-    }
-}
-
 static void
 process_lrm_resource_diff(xmlNode *lrm_resource, const char *node)
 {
     for (xmlNode *rsc_op = pcmk__xe_first_child(lrm_resource, NULL, NULL, NULL);
-         rsc_op != NULL; rsc_op = pcmk__xe_next(rsc_op)) {
+         rsc_op != NULL; rsc_op = pcmk__xe_next(rsc_op, NULL)) {
         process_graph_event(rsc_op, node);
     }
     if (shutdown_lock_cleared(lrm_resource)) {
@@ -262,7 +81,7 @@ process_resource_updates(const char *node, xmlNode *xml, xmlNode *change,
     }
 
     for (rsc = pcmk__xe_first_child(xml, NULL, NULL, NULL); rsc != NULL;
-         rsc = pcmk__xe_next(rsc)) {
+         rsc = pcmk__xe_next(rsc, NULL)) {
         crm_trace("Processing %s", pcmk__xe_id(rsc));
         process_lrm_resource_diff(rsc, node);
     }
@@ -387,7 +206,7 @@ process_status_diff(xmlNode *status, xmlNode *change, const char *op,
                     const char *xpath)
 {
     for (xmlNode *state = pcmk__xe_first_child(status, NULL, NULL, NULL);
-         state != NULL; state = pcmk__xe_next(state)) {
+         state != NULL; state = pcmk__xe_next(state, NULL)) {
 
         process_node_state_diff(state, change, op, xpath);
     }
@@ -411,7 +230,7 @@ process_cib_diff(xmlNode *cib, xmlNode *change, const char *op,
 }
 
 static int
-te_update_diff_element_v2(xmlNode *change, void *userdata)
+te_update_diff_element(xmlNode *change, void *userdata)
 {
     xmlNode *match = NULL;
     const char *name = NULL;
@@ -532,13 +351,6 @@ te_update_diff_element_v2(xmlNode *change, void *userdata)
     return pcmk_rc_ok;
 }
 
-static void
-te_update_diff_v2(xmlNode *diff)
-{
-    crm_log_xml_trace(diff, "Patch:Raw");
-    pcmk__xe_foreach_child(diff, NULL, te_update_diff_element_v2, NULL);
-}
-
 void
 te_update_diff(const char *event, xmlNode * msg)
 {
@@ -581,16 +393,14 @@ te_update_diff(const char *event, xmlNode * msg)
               fsa_state2string(controld_globals.fsa_state));
 
     crm_element_value_int(diff, PCMK_XA_FORMAT, &format);
-    switch (format) {
-        case 1:
-            te_update_diff_v1(event, diff);
-            break;
-        case 2:
-            te_update_diff_v2(diff);
-            break;
-        default:
-            crm_warn("Ignoring malformed CIB update (unknown patch format %d)",
-                     format);
+
+    if (format == 2) {
+        crm_log_xml_trace(diff, "patch");
+        pcmk__xe_foreach_child(diff, NULL, te_update_diff_element, NULL);
+
+    } else {
+        crm_warn("Ignoring malformed CIB update (unknown patch format %d)",
+                 format);
     }
     controld_remove_all_outside_events();
 }
