@@ -120,12 +120,12 @@ fail_incompletable_actions(pcmk__graph_t *graph, const char *down_node)
                                             PCMK__META_ON_NODE_UUID);
             router = crm_element_value(action->xml, PCMK__XA_ROUTER_NODE);
             if (router) {
-                const crm_node_t *node =
+                const pcmk__node_status_t *node =
                     pcmk__get_node(0, router, NULL,
                                    pcmk__node_search_cluster_member);
 
-                if (node) {
-                    router_uuid = node->uuid;
+                if (node != NULL) {
+                    router_uuid = node->xml_id;
                 }
             }
 
@@ -236,7 +236,7 @@ update_failcount(const xmlNode *event, const char *event_node_uuid, int rc,
             value = PCMK_XA_VALUE "++";
         }
 
-        if (g_hash_table_lookup(crm_remote_peer_cache, event_node_uuid)) {
+        if (g_hash_table_lookup(pcmk__remote_peer_cache, event_node_uuid)) {
             opts |= pcmk__node_attr_remote;
         }
 
@@ -436,7 +436,7 @@ process_graph_event(xmlNode *event, const char *event_node)
     const char *magic = NULL;
     const char *uname = NULL;
 
-    CRM_ASSERT(event != NULL);
+    pcmk__assert(event != NULL);
 
 /*
 <lrm_rsc_op id="rsc_east-05_last_0" operation_key="rsc_east-05_monitor_0" operation="monitor" crm-debug-origin="do_update_resource" crm_feature_set="3.0.6" transition-key="9:2:7:be2e97d9-05e2-439d-863e-48f7aecab2aa" transition-magic="0:7;9:2:7:be2e97d9-05e2-439d-863e-48f7aecab2aa" call-id="17" rc-code="7" op-status="0" interval="0" last-rc-change="1355361636" exec-time="128" queue-time="0" op-digest="c81f5f40b1c9e859c992e800b1aa6972"/>
@@ -463,7 +463,7 @@ process_graph_event(xmlNode *event, const char *event_node)
                               &action_num, &target_rc) == FALSE) {
         // decode_transition_key() already logged the bad key
         crm_err("Can't process action %s result: Incompatible versions? "
-                CRM_XS " call-id=%d", id, callid);
+                QB_XS " call-id=%d", id, callid);
         abort_transition(PCMK_SCORE_INFINITY, pcmk__graph_restart,
                          "Bad event", event);
         return;
@@ -579,32 +579,30 @@ process_graph_event(xmlNode *event, const char *event_node)
     } else if (desc && update_failcount(event, event_node, rc, target_rc,
                                         (transition_num == -1), FALSE)) {
         crm_notice("Transition %d action %d (%s on %s): expected '%s' but got '%s' "
-                   CRM_XS " target-rc=%d rc=%d call-id=%d event='%s'",
+                   QB_XS " target-rc=%d rc=%d call-id=%d event='%s'",
                    transition_num, action_num, id, uname,
-                   services_ocf_exitcode_str(target_rc),
-                   services_ocf_exitcode_str(rc),
+                   crm_exit_str(target_rc), crm_exit_str(rc),
                    target_rc, rc, callid, desc);
 
     } else if (desc) {
         crm_info("Transition %d action %d (%s on %s): %s "
-                 CRM_XS " rc=%d target-rc=%d call-id=%d",
+                 QB_XS " rc=%d target-rc=%d call-id=%d",
                  transition_num, action_num, id, uname,
                  desc, rc, target_rc, callid);
 
     } else if (rc == target_rc) {
         crm_info("Transition %d action %d (%s on %s) confirmed: %s "
-                 CRM_XS " rc=%d call-id=%d",
+                 QB_XS " rc=%d call-id=%d",
                  transition_num, action_num, id, uname,
-                 services_ocf_exitcode_str(rc), rc, callid);
+                 crm_exit_str(rc), rc, callid);
 
     } else {
         update_failcount(event, event_node, rc, target_rc,
                          (transition_num == -1), ignore_failures);
         crm_notice("Transition %d action %d (%s on %s): expected '%s' but got '%s' "
-                   CRM_XS " target-rc=%d rc=%d call-id=%d",
+                   QB_XS " target-rc=%d rc=%d call-id=%d",
                    transition_num, action_num, id, uname,
-                   services_ocf_exitcode_str(target_rc),
-                   services_ocf_exitcode_str(rc),
+                   crm_exit_str(target_rc), crm_exit_str(rc),
                    target_rc, rc, callid);
     }
 
