@@ -76,7 +76,7 @@ load2str(enum throttle_state_e mode)
 static char *
 find_cib_loadfile(void)
 {
-    pid_t pid = pcmk__procfs_pid_of("pacemaker-based");
+    pid_t pid = pcmk__procfs_pid_of(PCMK__SERVER_BASED);
 
     return pid? crm_strdup_printf("/proc/%lld/stat", (long long) pid) : NULL;
 }
@@ -367,12 +367,13 @@ throttle_send_command(enum throttle_state_e mode)
                  load2str(mode), load2str(last));
         last = mode;
 
-        xml = create_request(CRM_OP_THROTTLE, NULL, NULL, CRM_SYSTEM_CRMD, CRM_SYSTEM_CRMD, NULL);
+        xml = pcmk__new_request(pcmk_ipc_controld, CRM_SYSTEM_CRMD, NULL,
+                                CRM_SYSTEM_CRMD, CRM_OP_THROTTLE, NULL);
         crm_xml_add_int(xml, PCMK__XA_CRM_LIMIT_MODE, mode);
         crm_xml_add_int(xml, PCMK__XA_CRM_LIMIT_MAX, throttle_job_max);
 
-        pcmk__cluster_send_message(NULL, crm_msg_crmd, xml);
-        free_xml(xml);
+        pcmk__cluster_send_message(NULL, pcmk_ipc_controld, xml);
+        pcmk__xml_free(xml);
     }
 }
 
@@ -408,14 +409,25 @@ static void
 throttle_update_job_max(const char *preference)
 {
     long long max = 0LL;
+
+    // Per-node override
     const char *env_limit = pcmk__env_option(PCMK__ENV_NODE_ACTION_LIMIT);
 
     if (env_limit != NULL) {
-        preference = env_limit; // Per-node override
+        int rc = pcmk__scan_ll(env_limit, &max, 0LL);
+
+        if (rc != pcmk_rc_ok) {
+            crm_warn("Ignoring local option PCMK_" PCMK__ENV_NODE_ACTION_LIMIT
+                     " because '%s' is not a valid value: %s",
+                     env_limit, pcmk_rc_str(rc));
+            env_limit = NULL;
+        }
     }
-    if (preference != NULL) {
-        pcmk__scan_ll(preference, &max, 0LL);
+    if (env_limit == NULL) {
+        // Option validator should prevent invalid values
+        CRM_LOG_ASSERT(pcmk__scan_ll(preference, &max, 0LL) == pcmk_rc_ok);
     }
+
     if (max > 0) {
         throttle_job_max = (max >= INT_MAX)? INT_MAX : (int) max;
     } else {
