@@ -179,7 +179,7 @@ test_exit(crm_exit_t exit_code)
              lrmd_event_type2str(event->type),                          \
              event->rsc_id,                                             \
              event->op_type ? event->op_type : "none",                  \
-             services_ocf_exitcode_str(event->rc),                      \
+             crm_exit_str((crm_exit_t) event->rc),                      \
              pcmk_exec_status_str(event->op_status));                   \
     crm_info("%s", event_buf_v0);
 
@@ -281,7 +281,7 @@ start_test(gpointer user_data)
     lrmd_conn->cmds->set_callback(lrmd_conn, read_events);
 
     if (options.timeout) {
-        g_timeout_add(options.timeout, timeout_err, NULL);
+        pcmk__create_timer(options.timeout, timeout_err, NULL);
     }
 
     if (!options.api_call) {
@@ -472,14 +472,14 @@ generate_params(void)
         crm_crit("Could not allocate scheduler data");
         return ENOMEM;
     }
-    pcmk__set_scheduler_flags(scheduler,
-                              pcmk_sched_no_counts|pcmk_sched_no_compat);
+    pcmk__set_scheduler_flags(scheduler, pcmk__sched_no_counts);
     scheduler->input = cib_xml_copy;
-    scheduler->now = crm_time_new(NULL);
+    scheduler->priv->now = crm_time_new(NULL);
     cluster_status(scheduler);
 
     // Find resource in CIB
-    rsc = pe_find_resource_with_flags(scheduler->resources, options.rsc_id,
+    rsc = pe_find_resource_with_flags(scheduler->priv->resources,
+                                      options.rsc_id,
                                       pcmk_rsc_match_history
                                       |pcmk_rsc_match_basename);
     if (rsc == NULL) {
