@@ -382,7 +382,7 @@ stonith_recurring_op_helper(gpointer data)
 
     rsc = g_hash_table_lookup(rsc_list, cmd->rsc_id);
 
-    CRM_ASSERT(rsc != NULL);
+    pcmk__assert(rsc != NULL);
     /* take it out of recurring_ops list, and put it in the pending ops
      * to be executed */
     rsc->recurring_ops = g_list_remove(rsc->recurring_ops, cmd);
@@ -398,11 +398,13 @@ stonith_recurring_op_helper(gpointer data)
 static inline void
 start_recurring_timer(lrmd_cmd_t *cmd)
 {
-    if (cmd && (cmd->interval_ms > 0)) {
-        cmd->stonith_recurring_id = g_timeout_add(cmd->interval_ms,
-                                                  stonith_recurring_op_helper,
-                                                  cmd);
+    if (!cmd || (cmd->interval_ms <= 0)) {
+        return;
     }
+
+    cmd->stonith_recurring_id = pcmk__create_timer(cmd->interval_ms,
+                                                   stonith_recurring_op_helper,
+                                                   cmd);
 }
 
 static gboolean
@@ -532,7 +534,7 @@ schedule_lrmd_cmd(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
     mainloop_set_trigger(rsc->work);
 
     if (cmd->start_delay) {
-        cmd->delay_id = g_timeout_add(cmd->start_delay, start_delay_helper, cmd);
+        cmd->delay_id = pcmk__create_timer(cmd->start_delay, start_delay_helper, cmd);
     }
 }
 
@@ -587,7 +589,7 @@ send_client_notify(gpointer key, gpointer value, gpointer user_data)
             msg = pcmk_rc_str(rc);
             break;
     }
-    do_crm_log(log_level, "Could not notify client %s: %s " CRM_XS " rc=%d",
+    do_crm_log(log_level, "Could not notify client %s: %s " QB_XS " rc=%d",
                pcmk__client_name(client), msg, rc);
 }
 
@@ -682,7 +684,7 @@ send_cmd_complete_notify(lrmd_cmd_t * cmd)
         pcmk__foreach_ipc_client(send_client_notify, notify);
     }
 
-    free_xml(notify);
+    pcmk__xml_free(notify);
 }
 
 static void
@@ -707,7 +709,7 @@ send_generic_notify(int rc, xmlNode * request)
 
         pcmk__foreach_ipc_client(send_client_notify, notify);
 
-        free_xml(notify);
+        pcmk__xml_free(notify);
     }
 }
 
@@ -790,7 +792,7 @@ notify_of_new_client(pcmk__client_t *new_client)
     crm_xml_add(data.notify, PCMK__XA_LRMD_ORIGIN, __func__);
     crm_xml_add(data.notify, PCMK__XA_LRMD_OP, LRMD_OP_NEW_CLIENT);
     pcmk__foreach_ipc_client(notify_one_client, &data);
-    free_xml(data.notify);
+    pcmk__xml_free(data.notify);
 }
 
 void
@@ -844,10 +846,14 @@ action_complete(svc_action_t * action)
     rsc = cmd->rsc_id ? g_hash_table_lookup(rsc_list, cmd->rsc_id) : NULL;
 
 #ifdef PCMK__TIME_USE_CGT
-    if (rsc && pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_SERVICE, pcmk__str_casei)) {
-        rclass = resources_find_service_class(rsc->type);
-    } else if(rsc) {
+    if (rsc != NULL) {
         rclass = rsc->class;
+#if PCMK__ENABLE_SERVICE
+        if (pcmk__str_eq(rclass, PCMK_RESOURCE_CLASS_SERVICE,
+                         pcmk__str_casei)) {
+            rclass = resources_find_service_class(rsc->type);
+        }
+#endif
     }
 
     if (pcmk__str_eq(rclass, PCMK_RESOURCE_CLASS_SYSTEMD, pcmk__str_casei)) {
@@ -880,7 +886,7 @@ action_complete(svc_action_t * action)
                 crm_debug("%s systemd %s is now complete (elapsed=%dms, "
                           "remaining=%dms): %s (%d)",
                           cmd->rsc_id, cmd->real_action, time_sum, timeout_left,
-                          services_ocf_exitcode_str(cmd->result.exit_status),
+                          crm_exit_str(cmd->result.exit_status),
                           cmd->result.exit_status);
                 cmd_original_times(cmd);
 
@@ -901,22 +907,6 @@ action_complete(svc_action_t * action)
     }
 #endif
 
-#if SUPPORT_NAGIOS
-    if (rsc && pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
-        if (action_matches(cmd, PCMK_ACTION_MONITOR, 0)
-            && pcmk__result_ok(&(cmd->result))) {
-            /* Successfully executed --version for the nagios plugin */
-            cmd->result.exit_status = PCMK_OCF_NOT_RUNNING;
-
-        } else if (pcmk__str_eq(cmd->action, PCMK_ACTION_START, pcmk__str_casei)
-                   && !pcmk__result_ok(&(cmd->result))) {
-#ifdef PCMK__TIME_USE_CGT
-            goagain = true;
-#endif
-        }
-    }
-#endif
-
 #ifdef PCMK__TIME_USE_CGT
     if (goagain) {
         int time_sum = time_diff_ms(NULL, &(cmd->t_first_run));
@@ -941,11 +931,13 @@ action_complete(svc_action_t * action)
                          cmd->rsc_id, cmd->action, time_sum, timeout_left, delay);
 
             } else {
-                crm_notice("%s %s failed '%s' (%d): re-scheduling (elapsed=%dms, remaining=%dms, start_delay=%dms)",
+                crm_notice("%s %s failed: %s: Re-scheduling (remaining "
+                           "timeout %s) " QB_XS
+                           " exitstatus=%d elapsed=%dms start_delay=%dms)",
                            cmd->rsc_id, cmd->action,
-                           services_ocf_exitcode_str(cmd->result.exit_status),
-                           cmd->result.exit_status, time_sum, timeout_left,
-                           delay);
+                           crm_exit_str(cmd->result.exit_status),
+                           pcmk__readable_interval(timeout_left),
+                           cmd->result.exit_status, time_sum, delay);
             }
 
             cmd_reset(cmd);
@@ -1226,7 +1218,7 @@ static inline int
 execd_stonith_monitor(stonith_t *stonith_api, lrmd_rsc_t *rsc, lrmd_cmd_t *cmd)
 {
     int rc = stonith_api->cmds->monitor(stonith_api, 0, cmd->rsc_id,
-                                        cmd->timeout / 1000);
+                                        pcmk__timeout_ms2s(cmd->timeout));
 
     rc = stonith_api->cmds->register_callback(stonith_api, rc, 0, 0, cmd,
                                               "lrmd_stonith_callback",
@@ -1302,23 +1294,11 @@ execute_nonstonith_action(lrmd_rsc_t *rsc, lrmd_cmd_t *cmd)
     svc_action_t *action = NULL;
     GHashTable *params_copy = NULL;
 
-    CRM_ASSERT(rsc);
-    CRM_ASSERT(cmd);
+    pcmk__assert((rsc != NULL) && (cmd != NULL));
 
     crm_trace("Creating action, resource:%s action:%s class:%s provider:%s agent:%s",
               rsc->rsc_id, cmd->action, rsc->class, rsc->provider, rsc->type);
 
-#if SUPPORT_NAGIOS
-    /* Recurring operations are cancelled anyway for a stop operation */
-    if (pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)
-        && pcmk__str_eq(cmd->action, PCMK_ACTION_STOP, pcmk__str_casei)) {
-
-        cmd->result.exit_status = PCMK_OCF_OK;
-        cmd_finalize(cmd, rsc);
-        return;
-    }
-#endif
-
     params_copy = pcmk__str_table_dup(cmd->params);
 
     action = services__create_resource_action(rsc->rsc_id, rsc->class, rsc->provider,
@@ -1953,9 +1933,9 @@ process_lrmd_message(pcmk__client_t *client, uint32_t id, xmlNode *request)
             reply = create_lrmd_reply(__func__, rc, call_id);
         }
         send_rc = lrmd_server_send_reply(client, id, reply);
-        free_xml(reply);
+        pcmk__xml_free(reply);
         if (send_rc != pcmk_rc_ok) {
-            crm_warn("Reply to client %s failed: %s " CRM_XS " rc=%d",
+            crm_warn("Reply to client %s failed: %s " QB_XS " rc=%d",
                      pcmk__client_name(client), pcmk_rc_str(send_rc), send_rc);
         }
     }
