@@ -101,7 +101,7 @@ ipc_proxy_accept(qb_ipcs_connection_t * c, uid_t uid, gid_t gid, const char *ipc
     crm_xml_add(msg, PCMK__XA_LRMD_IPC_SERVER, ipc_channel);
     crm_xml_add(msg, PCMK__XA_LRMD_IPC_SESSION, client->id);
     lrmd_server_send_notify(ipc_proxy, msg);
-    free_xml(msg);
+    pcmk__xml_free(msg);
     crm_debug("Accepted IPC proxy connection (session ID %s) "
               "from uid %d gid %d on channel %s",
               client->id, uid, gid, ipc_channel);
@@ -157,10 +157,6 @@ ipc_proxy_forward_client(pcmk__client_t *ipc_proxy, xmlNode *xml)
     pcmk__client_t *ipc_client;
     int rc = pcmk_rc_ok;
 
-    /* If the IPC provider is acknowledging our shutdown request,
-     * defuse the short exit timer to give the cluster time to
-     * stop any resources we're running.
-     */
     if (pcmk__str_eq(msg_type, LRMD_IPC_OP_SHUTDOWN_ACK, pcmk__str_casei)) {
         handle_shutdown_ack();
         return;
@@ -177,7 +173,7 @@ ipc_proxy_forward_client(pcmk__client_t *ipc_proxy, xmlNode *xml)
         crm_xml_add(msg, PCMK__XA_LRMD_IPC_OP, LRMD_IPC_OP_DESTROY);
         crm_xml_add(msg, PCMK__XA_LRMD_IPC_SESSION, session);
         lrmd_server_send_notify(ipc_proxy, msg);
-        free_xml(msg);
+        pcmk__xml_free(msg);
         return;
     }
 
@@ -217,7 +213,7 @@ ipc_proxy_forward_client(pcmk__client_t *ipc_proxy, xmlNode *xml)
     }
 
     if (rc != pcmk_rc_ok) {
-        crm_warn("Could not proxy IPC to client %s: %s " CRM_XS " rc=%d",
+        crm_warn("Could not proxy IPC to client %s: %s " QB_XS " rc=%d",
                  ipc_client->id, pcmk_rc_str(rc), rc);
     }
 }
@@ -258,9 +254,9 @@ ipc_proxy_dispatch(qb_ipcs_connection_t * c, void *data, size_t size)
     }
 
     CRM_CHECK(client != NULL, crm_err("Invalid client");
-              free_xml(request); return FALSE);
+              pcmk__xml_free(request); return FALSE);
     CRM_CHECK(client->id != NULL, crm_err("Invalid client: %p", client);
-              free_xml(request); return FALSE);
+              pcmk__xml_free(request); return FALSE);
 
     /* This ensures that synced request/responses happen over the event channel
      * in the controller, allowing the controller to process the messages async.
@@ -282,8 +278,8 @@ ipc_proxy_dispatch(qb_ipcs_connection_t * c, void *data, size_t size)
 
     lrmd_server_send_notify(ipc_proxy, msg);
 
-    free_xml(request);
-    free_xml(msg);
+    pcmk__xml_free(request);
+    pcmk__xml_free(msg);
     return 0;
 }
 
@@ -309,7 +305,7 @@ ipc_proxy_shutdown_req(pcmk__client_t *ipc_proxy)
     crm_xml_add(msg, PCMK__XA_LRMD_IPC_SESSION, "0");
 
     rc = (lrmd_server_send_notify(ipc_proxy, msg) != pcmk_rc_ok)? -1 : 0;
-    free_xml(msg);
+    pcmk__xml_free(msg);
     return rc;
 }
 
@@ -332,7 +328,7 @@ ipc_proxy_closed(qb_ipcs_connection_t * c)
         crm_xml_add(msg, PCMK__XA_LRMD_IPC_OP, LRMD_IPC_OP_DESTROY);
         crm_xml_add(msg, PCMK__XA_LRMD_IPC_SESSION, client->id);
         lrmd_server_send_notify(ipc_proxy, msg);
-        free_xml(msg);
+        pcmk__xml_free(msg);
     }
 
     g_hash_table_remove(ipc_clients, client->id);
