@@ -16,6 +16,7 @@
 #include <crm/common/mainloop.h>
 #include <crm/common/xml.h>
 #include <crm/common/remote_internal.h>
+#include <crm/common/tls_internal.h>
 #include <crm/lrmd_internal.h>
 
 #include <netdb.h>
@@ -26,22 +27,14 @@
 
 #include "pacemaker-execd.h"
 
-#ifdef HAVE_GNUTLS_GNUTLS_H
+#include <gnutls/gnutls.h>
 
-#  include <gnutls/gnutls.h>
+#define LRMD_REMOTE_AUTH_TIMEOUT 10000
 
-#  define LRMD_REMOTE_AUTH_TIMEOUT 10000
-gnutls_psk_server_credentials_t psk_cred_s;
-gnutls_dh_params_t dh_params;
+static pcmk__tls_t *tls = NULL;
 static int ssock = -1;
 extern int lrmd_call_id;
 
-static void
-debug_log(int level, const char *str)
-{
-    fputs(str, stderr);
-}
-
 /*!
  * \internal
  * \brief Read (more) TLS handshake data from client
@@ -72,6 +65,13 @@ remoted__read_handshake_data(pcmk__client_t *client)
     pcmk__set_client_flags(client, pcmk__client_tls_handshake_complete);
     crm_notice("Remote client connection accepted");
 
+    /* Now that the handshake is done, see if any client TLS certificate is
+     * close to its expiration date and log if so.  If a TLS certificate is not
+     * in use, this function will just return so we don't need to check for the
+     * session type here.
+     */
+    pcmk__tls_check_cert_expiration(client->remote->tls_session);
+
     /* Only a client with access to the TLS key can connect, so we can treat
      * it as privileged.
      */
@@ -86,57 +86,68 @@ static int
 lrmd_remote_client_msg(gpointer data)
 {
     int id = 0;
-    int rc;
+    int rc = pcmk_rc_ok;
     xmlNode *request = NULL;
     pcmk__client_t *client = data;
 
-    if (!pcmk_is_set(client->flags,
-                     pcmk__client_tls_handshake_complete)) {
+    if (!pcmk_is_set(client->flags, pcmk__client_tls_handshake_complete)) {
         return remoted__read_handshake_data(client);
     }
 
-    switch (pcmk__remote_ready(client->remote, 0)) {
+    rc = pcmk__remote_ready(client->remote, 0);
+    switch (rc) {
         case pcmk_rc_ok:
             break;
-        case ETIME: // No message available to read
+
+        case ETIME:
+            /* No message available to read */
             return 0;
-        default:    // Error
-            crm_info("Remote client disconnected while polling it");
+
+        default:
+            /* Error */
+            crm_info("Error polling remote client: %s", pcmk_rc_str(rc));
             return -1;
     }
 
-    rc = pcmk__read_remote_message(client->remote, -1);
+    rc = pcmk__read_available_remote_data(client->remote);
+    switch (rc) {
+        case pcmk_rc_ok:
+            break;
+
+        case EAGAIN:
+            /* We haven't read the whole message yet */
+            return 0;
+
+        default:
+            /* Error */
+            crm_info("Error reading from remote client: %s", pcmk_rc_str(rc));
+            return -1;
+    }
 
     request = pcmk__remote_message_xml(client->remote);
-    while (request) {
-        crm_element_value_int(request, PCMK__XA_LRMD_REMOTE_MSG_ID, &id);
-        crm_trace("Processing remote client request %d", id);
-        if (!client->name) {
-            client->name = crm_element_value_copy(request,
-                                                  PCMK__XA_LRMD_CLIENTNAME);
-        }
-
-        lrmd_call_id++;
-        if (lrmd_call_id < 1) {
-            lrmd_call_id = 1;
-        }
-
-        crm_xml_add(request, PCMK__XA_LRMD_CLIENTID, client->id);
-        crm_xml_add(request, PCMK__XA_LRMD_CLIENTNAME, client->name);
-        crm_xml_add_int(request, PCMK__XA_LRMD_CALLID, lrmd_call_id);
-
-        process_lrmd_message(client, id, request);
-        free_xml(request);
-
-        /* process all the messages in the current buffer */
-        request = pcmk__remote_message_xml(client->remote);
+    if (request == NULL) {
+        return 0;
     }
 
-    if (rc == ENOTCONN) {
-        crm_info("Remote client disconnected while reading from it");
-        return -1;
+    crm_element_value_int(request, PCMK__XA_LRMD_REMOTE_MSG_ID, &id);
+    crm_trace("Processing remote client request %d", id);
+    if (!client->name) {
+        client->name = crm_element_value_copy(request,
+                                              PCMK__XA_LRMD_CLIENTNAME);
     }
 
+    lrmd_call_id++;
+    if (lrmd_call_id < 1) {
+        lrmd_call_id = 1;
+    }
+
+    crm_xml_add(request, PCMK__XA_LRMD_CLIENTID, client->id);
+    crm_xml_add(request, PCMK__XA_LRMD_CLIENTNAME, client->name);
+    crm_xml_add_int(request, PCMK__XA_LRMD_CALLID, lrmd_call_id);
+
+    process_lrmd_message(client, id, request);
+    pcmk__xml_free(request);
+
     return 0;
 }
 
@@ -164,12 +175,11 @@ lrmd_remote_client_destroy(gpointer user_data)
         void *sock_ptr;
         int csock;
 
-        sock_ptr = gnutls_transport_get_ptr(*client->remote->tls_session);
+        sock_ptr = gnutls_transport_get_ptr(client->remote->tls_session);
         csock = GPOINTER_TO_INT(sock_ptr);
 
-        gnutls_bye(*client->remote->tls_session, GNUTLS_SHUT_RDWR);
-        gnutls_deinit(*client->remote->tls_session);
-        gnutls_free(client->remote->tls_session);
+        gnutls_bye(client->remote->tls_session, GNUTLS_SHUT_RDWR);
+        gnutls_deinit(client->remote->tls_session);
         client->remote->tls_session = NULL;
         close(csock);
     }
@@ -202,7 +212,7 @@ static int
 lrmd_remote_listen(gpointer data)
 {
     int csock = -1;
-    gnutls_session_t *session = NULL;
+    gnutls_session_t session = NULL;
     pcmk__client_t *new_client = NULL;
 
     // For client socket
@@ -217,8 +227,7 @@ lrmd_remote_listen(gpointer data)
         return TRUE;
     }
 
-    session = pcmk__new_tls_session(csock, GNUTLS_SERVER, GNUTLS_CRD_PSK,
-                                    psk_cred_s);
+    session = pcmk__new_tls_session(tls, csock);
     if (session == NULL) {
         close(csock);
         return TRUE;
@@ -230,11 +239,11 @@ lrmd_remote_listen(gpointer data)
     new_client->remote->tls_session = session;
 
     // Require the client to authenticate within this time
-    new_client->remote->auth_timeout = g_timeout_add(LRMD_REMOTE_AUTH_TIMEOUT,
-                                                     lrmd_auth_timeout_cb,
-                                                     new_client);
+    new_client->remote->auth_timeout = pcmk__create_timer(LRMD_REMOTE_AUTH_TIMEOUT,
+                                                          lrmd_auth_timeout_cb,
+                                                          new_client);
     crm_info("Remote client pending authentication "
-             CRM_XS " %p id: %s", new_client, new_client->id);
+             QB_XS " %p id: %s", new_client, new_client->id);
 
     new_client->remote->source =
         mainloop_add_fd("pacemaker-remote-client", G_PRIORITY_DEFAULT, csock,
@@ -340,11 +349,12 @@ get_address_info(const char *bind_name, int port, struct addrinfo **res)
 int
 lrmd_init_remote_tls_server(void)
 {
+    int rc = pcmk_rc_ok;
     int filter;
     int port = crm_default_remote_port();
     struct addrinfo *res = NULL, *iter;
-    gnutls_datum_t psk_key = { NULL, 0 };
     const char *bind_name = pcmk__env_option(PCMK__ENV_REMOTE_ADDRESS);
+    bool use_cert = pcmk__x509_enabled();
 
     static struct mainloop_fd_callbacks remote_listen_fd_callbacks = {
         .dispatch = lrmd_remote_listen,
@@ -355,25 +365,28 @@ lrmd_init_remote_tls_server(void)
 
     crm_debug("Starting TLS listener on %s port %d",
               (bind_name? bind_name : "all addresses on"), port);
-    crm_gnutls_global_init();
-    gnutls_global_set_log_function(debug_log);
 
-    if (pcmk__init_tls_dh(&dh_params) != pcmk_rc_ok) {
+    rc = pcmk__init_tls(&tls, true, use_cert ? GNUTLS_CRD_CERTIFICATE : GNUTLS_CRD_PSK);
+    if (rc != pcmk_rc_ok) {
         return -1;
     }
-    gnutls_psk_allocate_server_credentials(&psk_cred_s);
-    gnutls_psk_set_server_credentials_function(psk_cred_s, lrmd_tls_server_key_cb);
-    gnutls_psk_set_server_dh_params(psk_cred_s, dh_params);
 
-    /* The key callback won't get called until the first client connection
-     * attempt. Do it once here, so we can warn the user at start-up if we can't
-     * read the key. We don't error out, though, because it's fine if the key is
-     * going to be added later.
-     */
-    if (lrmd__init_remote_key(&psk_key) != pcmk_rc_ok) {
-        crm_warn("A cluster connection will not be possible until the key is available");
+    if (!use_cert) {
+        gnutls_datum_t psk_key = { NULL, 0 };
+
+        pcmk__tls_add_psk_callback(tls, lrmd_tls_server_key_cb);
+
+        /* The key callback won't get called until the first client connection
+         * attempt. Do it once here, so we can warn the user at start-up if we can't
+         * read the key. We don't error out, though, because it's fine if the key is
+         * going to be added later.
+         */
+        if (lrmd__init_remote_key(&psk_key) != pcmk_rc_ok) {
+            crm_warn("A cluster connection will not be possible until the key is available");
+        }
+
+        gnutls_free(psk_key.data);
     }
-    gnutls_free(psk_key.data);
 
     if (get_address_info(bind_name, port, &res) != pcmk_rc_ok) {
         return -1;
@@ -419,9 +432,9 @@ lrmd_init_remote_tls_server(void)
 void
 execd_stop_tls_server(void)
 {
-    if (psk_cred_s) {
-        gnutls_psk_free_server_credentials(psk_cred_s);
-        psk_cred_s = 0;
+    if (tls != NULL) {
+        pcmk__free_tls(tls);
+        tls = NULL;
     }
 
     if (ssock >= 0) {
@@ -429,4 +442,3 @@ execd_stop_tls_server(void)
         ssock = -1;
     }
 }
-#endif
