@@ -55,9 +55,9 @@ stonith_send_broadcast_history(xmlNode *history,
         crm_xml_add(call_data, PCMK__XA_ST_TARGET, target);
     }
 
-    pcmk__cluster_send_message(NULL, crm_msg_stonith_ng, bcast);
+    pcmk__cluster_send_message(NULL, pcmk_ipc_fenced, bcast);
 
-    free_xml(bcast);
+    pcmk__xml_free(bcast);
 }
 
 static gboolean
@@ -242,7 +242,7 @@ stonith_xml_history_to_list(const xmlNode *history)
     CRM_LOG_ASSERT(rv != NULL);
 
     for (xml_op = pcmk__xe_first_child(history, NULL, NULL, NULL);
-         xml_op != NULL; xml_op = pcmk__xe_next(xml_op)) {
+         xml_op != NULL; xml_op = pcmk__xe_next(xml_op, NULL)) {
 
         remote_fencing_op_t *op = NULL;
         char *id = crm_element_value_copy(xml_op, PCMK__XA_ST_REMOTE_OP);
@@ -402,7 +402,8 @@ stonith_local_history_diff_and_merge(GHashTable *remote_history,
         g_hash_table_iter_init(&iter, remote_history);
         while (g_hash_table_iter_next(&iter, NULL, (void **)&op)) {
             if (stonith__op_state_pending(op->state) &&
-                pcmk__str_eq(op->originator, stonith_our_uname, pcmk__str_casei)) {
+                pcmk__str_eq(op->originator, fenced_get_local_node(),
+                             pcmk__str_casei)) {
 
                 crm_warn("Failing pending operation %.8s originated by us but "
                          "known only from peer history", op->id);
@@ -439,7 +440,7 @@ stonith_local_history_diff_and_merge(GHashTable *remote_history,
     }
 
     if (cnt == 0) {
-        free_xml(history);
+        pcmk__xml_free(history);
         return NULL;
     } else {
         return history;
@@ -483,14 +484,14 @@ stonith_fence_history(xmlNode *msg, xmlNode **output,
         target = crm_element_value(dev, PCMK__XA_ST_TARGET);
         if (target && (options & st_opt_cs_nodeid)) {
             int nodeid;
-            crm_node_t *node;
+            pcmk__node_status_t *node = NULL;
 
             pcmk__scan_min_int(target, &nodeid, 0);
             node = pcmk__search_node_caches(nodeid, NULL,
                                             pcmk__node_search_any
                                             |pcmk__node_search_cluster_cib);
-            if (node) {
-                target = node->uname;
+            if (node != NULL) {
+                target = node->name;
             }
         }
     }
@@ -522,7 +523,8 @@ stonith_fence_history(xmlNode *msg, xmlNode **output,
                                         st_opt_broadcast | st_opt_discard_reply,
                                         NULL);
         } else if (remote_peer &&
-                   !pcmk__str_eq(remote_peer, stonith_our_uname, pcmk__str_casei)) {
+                   !pcmk__str_eq(remote_peer, fenced_get_local_node(),
+                                 pcmk__str_casei)) {
             xmlNode *history = get_xpath_object("//" PCMK__XE_ST_HISTORY, msg,
                                                 LOG_NEVER);
 
@@ -568,5 +570,5 @@ stonith_fence_history(xmlNode *msg, xmlNode **output,
                   stonith_remote_op_list);
         *output = stonith_local_history(FALSE, target);
     }
-    free_xml(out_history);
+    pcmk__xml_free(out_history);
 }
