@@ -258,7 +258,7 @@ free_remote_op(gpointer data)
         g_list_free_full(op->query_results, free_remote_query);
     }
     if (op->request) {
-        free_xml(op->request);
+        pcmk__xml_free(op->request);
         op->request = NULL;
     }
     if (op->devices_list) {
@@ -304,7 +304,7 @@ static void
 op_phase_off(remote_fencing_op_t *op)
 {
     crm_info("Remapping multiple-device reboot targeting %s to 'off' "
-             CRM_XS " id=%.8s", op->target, op->id);
+             QB_XS " id=%.8s", op->target, op->id);
     op->phase = st_phase_off;
 
     /* Happily, "off" and "on" are shorter than "reboot", so we can reuse the
@@ -325,7 +325,7 @@ op_phase_on(remote_fencing_op_t *op)
     GList *iter = NULL;
 
     crm_info("Remapped 'off' targeting %s complete, "
-             "remapping to 'on' for %s " CRM_XS " id=%.8s",
+             "remapping to 'on' for %s " QB_XS " id=%.8s",
              op->target, op->client_name, op->id);
     op->phase = st_phase_on;
     strcpy(op->action, PCMK_ACTION_ON);
@@ -359,7 +359,7 @@ undo_op_remap(remote_fencing_op_t *op)
 {
     if (op->phase > 0) {
         crm_info("Undoing remap of reboot targeting %s for %s "
-                 CRM_XS " id=%.8s", op->target, op->client_name, op->id);
+                 QB_XS " id=%.8s", op->target, op->client_name, op->id);
         op->phase = st_phase_requested;
         strcpy(op->action, PCMK_ACTION_REBOOT);
     }
@@ -422,8 +422,8 @@ fenced_broadcast_op_result(const remote_fencing_op_t *op, bool op_merged)
     notify_data = fencing_result2xml(wrapper, op);
     stonith__xe_set_result(notify_data, &op->result);
 
-    pcmk__cluster_send_message(NULL, crm_msg_stonith_ng, bcast);
-    free_xml(bcast);
+    pcmk__cluster_send_message(NULL, pcmk_ipc_fenced, bcast);
+    pcmk__xml_free(bcast);
 
     return;
 }
@@ -467,12 +467,12 @@ handle_local_reply_and_notify(remote_fencing_op_t *op, xmlNode *data)
     notify_data = fencing_result2xml(NULL, op);
     fenced_send_notification(PCMK__VALUE_ST_NOTIFY_FENCE, &op->result,
                              notify_data);
-    free_xml(notify_data);
+    pcmk__xml_free(notify_data);
     fenced_send_notification(PCMK__VALUE_ST_NOTIFY_HISTORY, NULL, NULL);
 
     /* mark this op as having notify's already sent */
     op->notify_sent = TRUE;
-    free_xml(reply);
+    pcmk__xml_free(reply);
 }
 
 /*!
@@ -491,7 +491,7 @@ finalize_op_duplicates(remote_fencing_op_t *op, xmlNode *data)
         if (other->state == st_duplicate) {
             other->state = op->state;
             crm_debug("Performing duplicate notification for %s@%s: %s "
-                      CRM_XS " id=%.8s",
+                      QB_XS " id=%.8s",
                       other->client_name, other->originator,
                       pcmk_exec_status_str(op->result.execution_status),
                       other->id);
@@ -501,7 +501,7 @@ finalize_op_duplicates(remote_fencing_op_t *op, xmlNode *data)
         } else {
             // Possible if (for example) it timed out already
             crm_err("Skipping duplicate notification for %s@%s "
-                    CRM_XS " state=%s id=%.8s",
+                    QB_XS " state=%s id=%.8s",
                     other->client_name, other->originator,
                     stonith_op_state_str(other->state), other->id);
         }
@@ -553,7 +553,7 @@ finalize_op(remote_fencing_op_t *op, xmlNode *data, bool dup)
     if (op->notify_sent) {
         // Most likely, this is a timed-out action that eventually completed
         crm_notice("Operation '%s'%s%s by %s for %s@%s%s: "
-                   "Result arrived too late " CRM_XS " id=%.8s",
+                   "Result arrived too late " QB_XS " id=%.8s",
                    op->action, (op->target? " targeting " : ""),
                    (op->target? op->target : ""),
                    (op->delegate? op->delegate : "unknown node"),
@@ -598,16 +598,17 @@ finalize_op(remote_fencing_op_t *op, xmlNode *data, bool dup)
     if (!dup && !pcmk__str_eq(subt, PCMK__VALUE_BROADCAST, pcmk__str_none)) {
         /* Defer notification until the bcast message arrives */
         fenced_broadcast_op_result(op, op_merged);
-        free_xml(local_data);
+        pcmk__xml_free(local_data);
         return;
     }
 
     if (pcmk__result_ok(&op->result) || dup
-        || !pcmk__str_eq(op->originator, stonith_our_uname, pcmk__str_casei)) {
+        || !pcmk__str_eq(op->originator, fenced_get_local_node(),
+                         pcmk__str_casei)) {
         level = LOG_NOTICE;
     }
     do_crm_log(level, "Operation '%s'%s%s by %s for %s@%s%s: %s (%s%s%s) "
-               CRM_XS " id=%.8s", op->action, (op->target? " targeting " : ""),
+               QB_XS " id=%.8s", op->action, (op->target? " targeting " : ""),
                (op->target? op->target : ""),
                (op->delegate? op->delegate : "unknown node"),
                op->client_name, op->originator,
@@ -632,11 +633,11 @@ finalize_op(remote_fencing_op_t *op, xmlNode *data, bool dup)
         op->query_results = NULL;
     }
     if (op->request) {
-        free_xml(op->request);
+        pcmk__xml_free(op->request);
         op->request = NULL;
     }
 
-    free_xml(local_data);
+    pcmk__xml_free(local_data);
 }
 
 /*!
@@ -655,7 +656,7 @@ remote_op_watchdog_done(gpointer userdata)
     op->op_timer_one = 0;
 
     crm_notice("Self-fencing (%s) by %s for %s assumed complete "
-               CRM_XS " id=%.8s",
+               QB_XS " id=%.8s",
                op->action, op->target, op->client_name, op->id);
     op->state = st_done;
     pcmk__set_result(&op->result, CRM_EX_OK, PCMK_EXEC_DONE, NULL);
@@ -670,7 +671,7 @@ remote_op_timeout_one(gpointer userdata)
 
     op->op_timer_one = 0;
 
-    crm_notice("Peer's '%s' action targeting %s for client %s timed out " CRM_XS
+    crm_notice("Peer's '%s' action targeting %s for client %s timed out " QB_XS
                " id=%.8s", op->action, op->target, op->client_name, op->id);
     pcmk__set_result(&op->result, CRM_EX_ERROR, PCMK_EXEC_TIMEOUT,
                      "Peer did not return fence result within timeout");
@@ -679,7 +680,7 @@ remote_op_timeout_one(gpointer userdata)
     if (op->client_delay > 0) {
         op->client_delay = 0;
         crm_trace("Try another device for '%s' action targeting %s "
-                  "for client %s without delay " CRM_XS " id=%.8s",
+                  "for client %s without delay " QB_XS " id=%.8s",
                   op->action, op->target, op->client_name, op->id);
     }
 
@@ -699,7 +700,7 @@ static void
 finalize_timed_out_op(remote_fencing_op_t *op, const char *reason)
 {
     crm_debug("Action '%s' targeting %s for client %s timed out "
-              CRM_XS " id=%.8s",
+              QB_XS " id=%.8s",
               op->action, op->target, op->client_name, op->id);
 
     if (op->phase == st_phase_on) {
@@ -733,7 +734,7 @@ remote_op_timeout(gpointer userdata)
 
     if (op->state == st_done) {
         crm_debug("Action '%s' targeting %s for client %s already completed "
-                  CRM_XS " id=%.8s",
+                  QB_XS " id=%.8s",
                   op->action, op->target, op->client_name, op->id);
     } else {
         finalize_timed_out_op(userdata, "Fencing did not complete within a "
@@ -948,7 +949,7 @@ advance_topology_level(remote_fencing_op_t *op, bool empty_ok)
         return empty_ok? pcmk_rc_ok : ENODEV;
     }
 
-    CRM_ASSERT(tp->levels != NULL);
+    pcmk__assert(tp->levels != NULL);
 
     stonith__set_call_options(op->call_options, op->id, st_opt_topology);
 
@@ -985,7 +986,7 @@ advance_topology_level(remote_fencing_op_t *op, bool empty_ok)
     }
 
     crm_info("All %sfencing options targeting %s for client %s@%s failed "
-             CRM_XS " id=%.8s",
+             QB_XS " id=%.8s",
              (stonith_watchdog_timeout_ms > 0)?"non-watchdog ":"",
              op->target, op->client_name, op->originator, op->id);
     return ENODEV;
@@ -1008,7 +1009,7 @@ merge_duplicates(remote_fencing_op_t *op)
     g_hash_table_iter_init(&iter, stonith_remote_op_list);
     while (g_hash_table_iter_next(&iter, NULL, (void **)&other)) {
         const char *other_action = op_requested_action(other);
-        crm_node_t *node = NULL;
+        pcmk__node_status_t *node = NULL;
 
         if (!strcmp(op->id, other->id)) {
             continue; // Don't compare against self
@@ -1034,7 +1035,7 @@ merge_duplicates(remote_fencing_op_t *op)
             continue;
         }
         if (pcmk__str_eq(other->target, other->originator, pcmk__str_casei)) {
-            crm_trace("%.8s not duplicate of %.8s: suicide for %s",
+            crm_trace("%.8s not duplicate of %.8s: self-fencing for %s",
                       op->id, other->id, other->target);
             continue;
         }
@@ -1044,7 +1045,7 @@ merge_duplicates(remote_fencing_op_t *op)
 
         if (!fencing_peer_active(node)) {
             crm_notice("Failing action '%s' targeting %s originating from "
-                       "client %s@%s: Originator is dead " CRM_XS " id=%.8s",
+                       "client %s@%s: Originator is dead " QB_XS " id=%.8s",
                        other->action, other->target, other->client_name,
                        other->originator, other->id);
             crm_trace("%.8s not duplicate of %.8s: originator dead",
@@ -1072,7 +1073,7 @@ merge_duplicates(remote_fencing_op_t *op)
         }
         crm_notice("Merging fencing action '%s' targeting %s originating from "
                    "client %s with identical request from %s@%s "
-                   CRM_XS " original=%.8s duplicate=%.8s total_timeout=%ds",
+                   QB_XS " original=%.8s duplicate=%.8s total_timeout=%ds",
                    op->action, op->target, op->client_name,
                    other->client_name, other->originator,
                    op->id, other->id, other->total_timeout);
@@ -1084,10 +1085,10 @@ merge_duplicates(remote_fencing_op_t *op)
 static uint32_t fencing_active_peers(void)
 {
     uint32_t count = 0;
-    crm_node_t *entry;
+    pcmk__node_status_t *entry = NULL;
     GHashTableIter gIter;
 
-    g_hash_table_iter_init(&gIter, crm_peer_cache);
+    g_hash_table_iter_init(&gIter, pcmk__peer_cache);
     while (g_hash_table_iter_next(&gIter, NULL, (void **)&entry)) {
         if(fencing_peer_active(entry)) {
             count++;
@@ -1151,7 +1152,7 @@ create_remote_stonith_op(const char *client, xmlNode *request, gboolean peer)
     remote_fencing_op_t *op = NULL;
     xmlNode *dev = get_xpath_object("//@" PCMK__XA_ST_TARGET, request,
                                     LOG_NEVER);
-    int call_options = 0;
+    int rc = pcmk_rc_ok;
     const char *operation = NULL;
 
     init_stonith_remote_op_hash_table(&stonith_remote_op_list);
@@ -1198,7 +1199,7 @@ create_remote_stonith_op(const char *client, xmlNode *request, gboolean peer)
     op->originator = crm_element_value_copy(dev, PCMK__XA_ST_ORIGIN);
     if (op->originator == NULL) {
         /* Local or relayed request */
-        op->originator = pcmk__str_copy(stonith_our_uname);
+        op->originator = pcmk__str_copy(fenced_get_local_node());
     }
 
     // Delegate may not be set
@@ -1223,8 +1224,13 @@ create_remote_stonith_op(const char *client, xmlNode *request, gboolean peer)
 
     // @TODO Figure out how to avoid copying XML here
     op->request = pcmk__xml_copy(NULL, request);
-    crm_element_value_int(request, PCMK__XA_ST_CALLOPT, &call_options);
-    op->call_options = call_options;
+
+    rc = pcmk__xe_get_flags(request, PCMK__XA_ST_CALLOPT, &(op->call_options),
+                            0U);
+    if (rc != pcmk_rc_ok) {
+        crm_warn("Couldn't parse options from request %s: %s",
+                 op->id, pcmk_rc_str(rc));
+    }
 
     crm_element_value_int(request, PCMK__XA_ST_CALLID, &(op->client_callid));
 
@@ -1237,7 +1243,7 @@ create_remote_stonith_op(const char *client, xmlNode *request, gboolean peer)
 
     if (op->call_options & st_opt_cs_nodeid) {
         int nodeid;
-        crm_node_t *node;
+        pcmk__node_status_t *node = NULL;
 
         pcmk__scan_min_int(op->target, &nodeid, 0);
         node = pcmk__search_node_caches(nodeid, NULL,
@@ -1247,8 +1253,8 @@ create_remote_stonith_op(const char *client, xmlNode *request, gboolean peer)
         /* Ensure the conversion only happens once */
         stonith__clear_call_options(op->call_options, op->id, st_opt_cs_nodeid);
 
-        if (node && node->uname) {
-            pcmk__str_update(&(op->target), node->uname);
+        if ((node != NULL) && (node->name != NULL)) {
+            pcmk__str_update(&(op->target), node->name);
 
         } else {
             crm_warn("Could not expand nodeid '%s' into a host name", op->target);
@@ -1315,18 +1321,18 @@ initiate_remote_stonith_op(const pcmk__client_t *client, xmlNode *request,
             pcmk__set_result(&op->result, CRM_EX_ERROR, PCMK_EXEC_ERROR,
                              "All topology levels failed");
             crm_warn("Could not request peer fencing (%s) targeting %s "
-                     CRM_XS " id=%.8s", op->action, op->target, op->id);
+                     QB_XS " id=%.8s", op->action, op->target, op->id);
             finalize_op(op, NULL, false);
             return op;
 
         case st_duplicate:
             crm_info("Requesting peer fencing (%s) targeting %s (duplicate) "
-                     CRM_XS " id=%.8s", op->action, op->target, op->id);
+                     QB_XS " id=%.8s", op->action, op->target, op->id);
             return op;
 
         default:
             crm_notice("Requesting peer fencing (%s) targeting %s "
-                       CRM_XS " id=%.8s state=%s base_timeout=%ds",
+                       QB_XS " id=%.8s state=%s base_timeout=%ds",
                        op->action, op->target, op->id,
                        stonith_op_state_str(op->state), op->base_timeout);
     }
@@ -1351,11 +1357,11 @@ initiate_remote_stonith_op(const pcmk__client_t *client, xmlNode *request,
         }
     }
 
-    pcmk__cluster_send_message(NULL, crm_msg_stonith_ng, query);
-    free_xml(query);
+    pcmk__cluster_send_message(NULL, pcmk_ipc_fenced, query);
+    pcmk__xml_free(query);
 
     query_timeout = op->base_timeout * TIMEOUT_MULTIPLY_FACTOR;
-    op->query_timer = g_timeout_add((1000 * query_timeout), remote_op_query_timeout, op);
+    op->query_timer = pcmk__create_timer((1000 * query_timeout), remote_op_query_timeout, op);
 
     return op;
 }
@@ -1499,7 +1505,8 @@ valid_fencing_timeout(int specified_timeout, bool action_specific,
     }
 
     timeout = (int) QB_MIN(QB_MAX(specified_timeout,
-                                  stonith_watchdog_timeout_ms / 1000), INT_MAX);
+                                  pcmk__timeout_ms2s(stonith_watchdog_timeout_ms)),
+                           INT_MAX);
 
     if (timeout > specified_timeout) {
         if (action_specific) {
@@ -1653,7 +1660,7 @@ get_op_total_timeout(const remote_fencing_op_t *op,
                    if didn't get a reply
                  */
                 if (!found && is_watchdog_fencing(op, device_list->data)) {
-                    total_timeout += stonith_watchdog_timeout_ms / 1000;
+                    total_timeout += pcmk__timeout_ms2s(stonith_watchdog_timeout_ms);
                 }
             }                   /* End Loop2: iterate through devices at a specific level */
         }                       /*End Loop1: iterate through fencing levels */
@@ -1724,7 +1731,7 @@ report_timeout_period(remote_fencing_op_t * op, int op_timeout)
         return;
     }
 
-    if (pcmk__str_eq(client_node, stonith_our_uname, pcmk__str_casei)) {
+    if (pcmk__str_eq(client_node, fenced_get_local_node(), pcmk__str_casei)) {
         // Client is connected to this node, so send update directly to them
         do_stonith_async_timeout_update(client_id, call_id, op_timeout);
         return;
@@ -1739,9 +1746,9 @@ report_timeout_period(remote_fencing_op_t * op, int op_timeout)
 
     pcmk__cluster_send_message(pcmk__get_node(0, client_node, NULL,
                                               pcmk__node_search_cluster_member),
-                               crm_msg_stonith_ng, update);
+                               pcmk_ipc_fenced, update);
 
-    free_xml(update);
+    pcmk__xml_free(update);
 
     for (iter = op->duplicates; iter != NULL; iter = iter->next) {
         remote_fencing_op_t *dup = iter->data;
@@ -1821,15 +1828,15 @@ check_watchdog_fencing_and_wait(remote_fencing_op_t * op)
         guint timeout_ms = QB_MIN(stonith_watchdog_timeout_ms, UINT_MAX);
 
         crm_notice("Waiting %s for %s to self-fence (%s) for "
-                   "client %s " CRM_XS " id=%.8s",
+                   "client %s " QB_XS " id=%.8s",
                    pcmk__readable_interval(timeout_ms), op->target, op->action,
                    op->client_name, op->id);
 
         if (op->op_timer_one) {
             g_source_remove(op->op_timer_one);
         }
-        op->op_timer_one = g_timeout_add(timeout_ms, remote_op_watchdog_done,
-                                         op);
+        op->op_timer_one = pcmk__create_timer(timeout_ms, remote_op_watchdog_done,
+                                              op);
         return TRUE;
     } else {
         crm_debug("Skipping fallback to watchdog-fencing as %s is "
@@ -1894,10 +1901,10 @@ request_peer_fencing(remote_fencing_op_t *op, peer_device_info_t *peer)
 
     if (!op->op_timer_total) {
         op->total_timeout = TIMEOUT_MULTIPLY_FACTOR * get_op_total_timeout(op, peer);
-        op->op_timer_total = g_timeout_add(1000 * op->total_timeout, remote_op_timeout, op);
+        op->op_timer_total = pcmk__create_timer(1000 * op->total_timeout, remote_op_timeout, op);
         report_timeout_period(op, op->total_timeout);
-        crm_info("Total timeout set to %ds for peer's fencing targeting %s for %s"
-                 CRM_XS "id=%.8s",
+        crm_info("Total timeout set to %ds for peer's fencing targeting %s for %s "
+                 QB_XS " id=%.8s",
                  op->total_timeout, op->target, op->client_name, op->id);
     }
 
@@ -1923,7 +1930,7 @@ request_peer_fencing(remote_fencing_op_t *op, peer_device_info_t *peer)
     if (peer) {
         int timeout_one = 0;
         xmlNode *remote_op = stonith_create_op(op->client_callid, op->id, STONITH_OP_FENCE, NULL, 0);
-        const crm_node_t *peer_node =
+        const pcmk__node_status_t *peer_node =
             pcmk__get_node(0, peer->host, NULL,
                            pcmk__node_search_cluster_member);
 
@@ -1948,7 +1955,7 @@ request_peer_fencing(remote_fencing_op_t *op, peer_device_info_t *peer)
             timeout_one += TIMEOUT_MULTIPLY_FACTOR *
                            get_device_timeout(op, peer, device, true);
             crm_notice("Requesting that %s perform '%s' action targeting %s "
-                       "using %s " CRM_XS " for client %s (%ds)",
+                       "using %s " QB_XS " for client %s (%ds)",
                        peer->host, op->action, op->target, device,
                        op->client_name, timeout_one);
             crm_xml_add(remote_op, PCMK__XA_ST_DEVICE_ID, device);
@@ -1956,7 +1963,7 @@ request_peer_fencing(remote_fencing_op_t *op, peer_device_info_t *peer)
         } else {
             timeout_one += TIMEOUT_MULTIPLY_FACTOR * get_peer_timeout(op, peer);
             crm_notice("Requesting that %s perform '%s' action targeting %s "
-                       CRM_XS " for client %s (%ds, %s)",
+                       QB_XS " for client %s (%ds, %s)",
                        peer->host, op->action, op->target, op->client_name,
                        timeout_one,
                        pcmk__readable_interval(stonith_watchdog_timeout_ms));
@@ -1983,20 +1990,20 @@ request_peer_fencing(remote_fencing_op_t *op, peer_device_info_t *peer)
                  come back in between
                - Delicate might be the case where we have watchdog-fencing
                  enabled for a node but the watchdog-fencing-device isn't
-                 explicitly chosen for suicide. Local pe-execution in sbd
-                 may detect the node as unclean and lead to timely suicide.
-                 Otherwise the selection of PCMK_OPT_STONITH_WATCHDOG_TIMEOUT
-                 at least is questionable.
+                 explicitly chosen for self-fencing. Local scheduler execution
+                 in sbd might detect the node as unclean and lead to timely
+                 self-fencing. Otherwise the selection of
+                 PCMK_OPT_STONITH_WATCHDOG_TIMEOUT at least is questionable.
              */
 
             /* coming here we're not waiting for watchdog timeout -
                thus engage timer with timout evaluated before */
-            op->op_timer_one = g_timeout_add((1000 * timeout_one), remote_op_timeout_one, op);
+            op->op_timer_one = pcmk__create_timer((1000 * timeout_one), remote_op_timeout_one, op);
         }
 
-        pcmk__cluster_send_message(peer_node, crm_msg_stonith_ng, remote_op);
+        pcmk__cluster_send_message(peer_node, pcmk_ipc_fenced, remote_op);
         peer->tried = TRUE;
-        free_xml(remote_op);
+        pcmk__xml_free(remote_op);
         return;
 
     } else if (op->phase == st_phase_on) {
@@ -2015,7 +2022,7 @@ request_peer_fencing(remote_fencing_op_t *op, peer_device_info_t *peer)
     } else if (op->query_timer == 0) {
         /* We've exhausted all available peers */
         crm_info("No remaining peers capable of fencing (%s) %s for client %s "
-                 CRM_XS " state=%s", op->action, op->target, op->client_name,
+                 QB_XS " state=%s", op->action, op->target, op->client_name,
                  stonith_op_state_str(op->state));
         CRM_CHECK(op->state < st_done, return);
         finalize_timed_out_op(op, "All nodes failed, or are unable, to "
@@ -2039,7 +2046,7 @@ request_peer_fencing(remote_fencing_op_t *op, peer_device_info_t *peer)
 
         if (op->state == st_query) {
             crm_info("No peers (out of %d) have devices capable of fencing "
-                     "(%s) %s for client %s " CRM_XS " state=%s",
+                     "(%s) %s for client %s " QB_XS " state=%s",
                      op->replies, op->action, op->target, op->client_name,
                      stonith_op_state_str(op->state));
 
@@ -2060,7 +2067,7 @@ request_peer_fencing(remote_fencing_op_t *op, peer_device_info_t *peer)
              */
 
             crm_info("No peers (out of %d) are capable of fencing (%s) %s "
-                     "for client %s " CRM_XS " state=%s",
+                     "for client %s " QB_XS " state=%s",
                      op->replies, op->action, op->target, op->client_name,
                      stonith_op_state_str(op->state));
         }
@@ -2070,7 +2077,7 @@ request_peer_fencing(remote_fencing_op_t *op, peer_device_info_t *peer)
 
     } else {
         crm_info("Waiting for additional peers capable of fencing (%s) %s%s%s "
-                 "for client %s " CRM_XS " id=%.8s",
+                 "for client %s " QB_XS " id=%.8s",
                  op->action, op->target, (device? " using " : ""),
                  (device? device : ""), op->client_name, op->id);
     }
@@ -2221,7 +2228,7 @@ add_device_properties(const xmlNode *xml, remote_fencing_op_t *op,
     int verified = 0;
     device_properties_t *props =
         pcmk__assert_alloc(1, sizeof(device_properties_t));
-    int flags = st_device_supports_on; /* Old nodes that don't set the flag assume they support the on action */
+    int rc = pcmk_rc_ok;
 
     /* Add a new entry to this peer's devices list */
     g_hash_table_insert(peer->devices, pcmk__str_copy(device), props);
@@ -2234,14 +2241,20 @@ add_device_properties(const xmlNode *xml, remote_fencing_op_t *op,
         props->verified = TRUE;
     }
 
-    crm_element_value_int(xml, PCMK__XA_ST_DEVICE_SUPPORT_FLAGS, &flags);
-    props->device_support_flags = flags;
+    // Nodes <2.1.5 won't set this, so assume unfencing in that case
+    rc = pcmk__xe_get_flags(xml, PCMK__XA_ST_DEVICE_SUPPORT_FLAGS,
+                            &(props->device_support_flags),
+                            st_device_supports_on);
+    if (rc != pcmk_rc_ok) {
+        crm_warn("Couldn't determine device support for %s "
+                 "(assuming unfencing): %s", device, pcmk_rc_str(rc));
+    }
 
     /* Parse action-specific device properties */
     parse_action_specific(xml, peer->host, device, op_requested_action(op),
                           op, st_phase_requested, props);
     for (child = pcmk__xe_first_child(xml, NULL, NULL, NULL); child != NULL;
-         child = pcmk__xe_next(child)) {
+         child = pcmk__xe_next(child, NULL)) {
         /* Replies for "reboot" operations will include the action-specific
          * values for "off" and "on" in child elements, just in case the reboot
          * winds up getting remapped.
@@ -2282,7 +2295,7 @@ add_result(remote_fencing_op_t *op, const char *host, int ndevices,
 
     /* Each child element describes one capable device available to the peer */
     for (child = pcmk__xe_first_child(xml, NULL, NULL, NULL); child != NULL;
-         child = pcmk__xe_next(child)) {
+         child = pcmk__xe_next(child, NULL)) {
         const char *device = pcmk__xe_id(child);
 
         if (device) {
@@ -2477,7 +2490,8 @@ fenced_process_fencing_reply(xmlNode *msg)
         finalize_op(op, msg, false);
         return;
 
-    } else if (!pcmk__str_eq(op->originator, stonith_our_uname, pcmk__str_casei)) {
+    } else if (!pcmk__str_eq(op->originator, fenced_get_local_node(),
+                             pcmk__str_casei)) {
         /* If this isn't a remote level broadcast, and we are not the
          * originator of the operation, we should not be receiving this msg. */
         crm_err("Received non-broadcast fencing result for operation %.8s "
