@@ -40,13 +40,11 @@
 
 #define SUMMARY "daemon for executing fencing devices in a Pacemaker cluster"
 
-char *stonith_our_uname = NULL;
 long long stonith_watchdog_timeout_ms = 0;
 GList *stonith_watchdog_targets = NULL;
 
 static GMainLoop *mainloop = NULL;
 
-gboolean stand_alone = FALSE;
 gboolean stonith_shutdown_flag = FALSE;
 
 static qb_ipcs_service_t *ipcs = NULL;
@@ -60,7 +58,7 @@ pcmk__supported_format_t formats[] = {
 };
 
 static struct {
-    bool no_cib_connect;
+    gboolean stand_alone;
     gchar **log_files;
 } options;
 
@@ -89,10 +87,11 @@ st_ipc_dispatch(qb_ipcs_connection_t * qbc, void *data, size_t size)
 {
     uint32_t id = 0;
     uint32_t flags = 0;
-    int call_options = 0;
+    uint32_t call_options = st_opt_none;
     xmlNode *request = NULL;
     pcmk__client_t *c = pcmk__find_client(qbc);
     const char *op = NULL;
+    int rc = pcmk_rc_ok;
 
     if (c == NULL) {
         crm_info("Invalid client: %p", qbc);
@@ -112,10 +111,10 @@ st_ipc_dispatch(qb_ipcs_connection_t * qbc, void *data, size_t size)
         crm_xml_add(request, PCMK__XA_ST_OP, op);
         crm_xml_add(request, PCMK__XA_ST_CLIENTID, c->id);
         crm_xml_add(request, PCMK__XA_ST_CLIENTNAME, pcmk__client_name(c));
-        crm_xml_add(request, PCMK__XA_ST_CLIENTNODE, stonith_our_uname);
+        crm_xml_add(request, PCMK__XA_ST_CLIENTNODE, fenced_get_local_node());
 
-        pcmk__cluster_send_message(NULL, crm_msg_stonith_ng, request);
-        free_xml(request);
+        pcmk__cluster_send_message(NULL, pcmk_ipc_fenced, request);
+        pcmk__xml_free(request);
         return 0;
     }
 
@@ -125,24 +124,30 @@ st_ipc_dispatch(qb_ipcs_connection_t * qbc, void *data, size_t size)
         c->name = crm_strdup_printf("%s.%u", pcmk__s(value, "unknown"), c->pid);
     }
 
-    crm_element_value_int(request, PCMK__XA_ST_CALLOPT, &call_options);
+    rc = pcmk__xe_get_flags(request, PCMK__XA_ST_CALLOPT, &call_options,
+                            st_opt_none);
+    if (rc != pcmk_rc_ok) {
+        crm_warn("Couldn't parse options from IPC request: %s",
+                 pcmk_rc_str(rc));
+    }
+
     crm_trace("Flags %#08" PRIx32 "/%#08x for command %" PRIu32
               " from client %s", flags, call_options, id, pcmk__client_name(c));
 
     if (pcmk_is_set(call_options, st_opt_sync_call)) {
-        CRM_ASSERT(flags & crm_ipc_client_response);
+        pcmk__assert(pcmk_is_set(flags, crm_ipc_client_response));
         CRM_LOG_ASSERT(c->request_id == 0);     /* This means the client has two synchronous events in-flight */
         c->request_id = id;     /* Reply only to the last one */
     }
 
     crm_xml_add(request, PCMK__XA_ST_CLIENTID, c->id);
     crm_xml_add(request, PCMK__XA_ST_CLIENTNAME, pcmk__client_name(c));
-    crm_xml_add(request, PCMK__XA_ST_CLIENTNODE, stonith_our_uname);
+    crm_xml_add(request, PCMK__XA_ST_CLIENTNODE, fenced_get_local_node());
 
     crm_log_xml_trace(request, "ipc-received");
     stonith_command(c, id, flags, request, NULL);
 
-    free_xml(request);
+    pcmk__xml_free(request);
     return 0;
 }
 
@@ -190,28 +195,25 @@ stonith_peer_ais_callback(cpg_handle_t handle,
                           const struct cpg_name *groupName,
                           uint32_t nodeid, uint32_t pid, void *msg, size_t msg_len)
 {
-    uint32_t kind = 0;
     xmlNode *xml = NULL;
     const char *from = NULL;
-    char *data = pcmk__cpg_message_data(handle, nodeid, pid, msg, &kind, &from);
+    char *data = pcmk__cpg_message_data(handle, nodeid, pid, msg, &from);
 
     if(data == NULL) {
         return;
     }
-    if (kind == crm_class_cluster) {
-        xml = pcmk__xml_parse(data);
-        if (xml == NULL) {
-            crm_err("Invalid XML: '%.120s'", data);
-            free(data);
-            return;
-        }
-        crm_xml_add(xml, PCMK__XA_SRC, from);
-        stonith_peer_callback(xml, NULL);
-    }
 
-    free_xml(xml);
+    xml = pcmk__xml_parse(data);
+    if (xml == NULL) {
+        crm_err("Invalid XML: '%.120s'", data);
+        free(data);
+        return;
+    }
+    crm_xml_add(xml, PCMK__XA_SRC, from);
+    stonith_peer_callback(xml, NULL);
+
+    pcmk__xml_free(xml);
     free(data);
-    return;
 }
 
 static void
@@ -298,7 +300,7 @@ stonith_notify_client(gpointer key, gpointer value, gpointer user_data)
 
         if (rc != pcmk_rc_ok) {
             crm_warn("%s notification of client %s failed: %s "
-                     CRM_XS " id=%.8s rc=%d", type, pcmk__client_name(client),
+                     QB_XS " id=%.8s rc=%d", type, pcmk__client_name(client),
                      pcmk_rc_str(rc), client->id, rc);
         } else {
             crm_trace("Sent %s notification to client %s",
@@ -333,7 +335,7 @@ do_stonith_async_timeout_update(const char *client_id, const char *call_id, int
         pcmk__ipc_send_xml(client, 0, notify_data, crm_ipc_server_event);
     }
 
-    free_xml(notify_data);
+    pcmk__xml_free(notify_data);
 }
 
 /*!
@@ -366,7 +368,7 @@ fenced_send_notification(const char *type, const pcmk__action_result_t *result,
 
     crm_trace("Notifying clients");
     pcmk__foreach_ipc_client(stonith_notify_client, update_msg);
-    free_xml(update_msg);
+    pcmk__xml_free(update_msg);
     crm_trace("Notify complete");
 }
 
@@ -392,7 +394,7 @@ fenced_send_config_notification(const char *op,
     crm_xml_add(notify_data, PCMK__XA_ST_DEVICE_ID, desc);
 
     fenced_send_notification(op, result, notify_data);
-    free_xml(notify_data);
+    pcmk__xml_free(notify_data);
 }
 
 /*!
@@ -437,18 +439,6 @@ stonith_cleanup(void)
     free_device_list();
     free_metadata_cache();
     fenced_unregister_handlers();
-
-    free(stonith_our_uname);
-    stonith_our_uname = NULL;
-}
-
-static gboolean
-stand_alone_cpg_cb(const gchar *option_name, const gchar *optarg, gpointer data,
-                   GError **error)
-{
-    stand_alone = FALSE;
-    options.no_cib_connect = true;
-    return TRUE;
 }
 
 struct qb_ipcs_service_handlers ipc_callbacks = {
@@ -468,10 +458,11 @@ struct qb_ipcs_service_handlers ipc_callbacks = {
  * \param[in] data  Previous value of what changed
  */
 static void
-st_peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *data)
+st_peer_update_callback(enum pcmk__node_update type, pcmk__node_status_t *node,
+                        const void *data)
 {
-    if ((type != crm_status_processes)
-        && !pcmk_is_set(node->flags, crm_remote_node)) {
+    if ((type != pcmk__node_update_processes)
+        && !pcmk_is_set(node->flags, pcmk__node_status_remote)) {
         /*
          * This is a hack until we can send to a nodeid and/or we fix node name lookups
          * These messages are ignored in stonith_peer_callback()
@@ -481,37 +472,37 @@ st_peer_update_callback(enum crm_status_type type, crm_node_t * node, const void
         crm_xml_add(query, PCMK__XA_T, PCMK__VALUE_STONITH_NG);
         crm_xml_add(query, PCMK__XA_ST_OP, STONITH_OP_POKE);
 
-        crm_debug("Broadcasting our uname because of node %u", node->id);
-        pcmk__cluster_send_message(NULL, crm_msg_stonith_ng, query);
+        crm_debug("Broadcasting our uname because of node %" PRIu32,
+                  node->cluster_layer_id);
+        pcmk__cluster_send_message(NULL, pcmk_ipc_fenced, query);
 
-        free_xml(query);
+        pcmk__xml_free(query);
     }
 }
 
 /* @COMPAT Deprecated since 2.1.8. Use pcmk_list_fence_attrs() or
  * crm_resource --list-options=fencing instead of querying daemon metadata.
+ *
+ * NOTE: pcs (as of at least 0.11.8) uses this
  */
 static int
 fencer_metadata(void)
 {
-    const char *name = "pacemaker-fenced";
+    const char *name = PCMK__SERVER_FENCED;
     const char *desc_short = N_("Instance attributes available for all "
                                 "\"stonith\"-class resources");
     const char *desc_long = N_("Instance attributes available for all "
                                "\"stonith\"-class resources and used by "
-                               "Pacemaker's fence daemon, formerly known as "
-                               "stonithd");
+                               "Pacemaker's fence daemon");
 
     return pcmk__daemon_metadata(out, name, desc_short, desc_long,
                                  pcmk__opt_fencing);
 }
 
 static GOptionEntry entries[] = {
-    { "stand-alone", 's', G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE, &stand_alone,
-      N_("Deprecated (will be removed in a future release)"), NULL },
-
-    { "stand-alone-w-cpg", 'c', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-      stand_alone_cpg_cb, N_("Intended for use in regression testing only"), NULL },
+    { "stand-alone", 's', G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE,
+      &options.stand_alone, N_("Intended for use in regression testing only"),
+      NULL },
 
     { "logfile", 'l', G_OPTION_FLAG_NONE, G_OPTION_ARG_FILENAME_ARRAY,
       &options.log_files, N_("Send logs to the additional named logfile"), NULL },
@@ -598,7 +589,8 @@ main(int argc, char **argv)
         // IPC endpoint already up
         crm_ipc_close(old_instance);
         crm_ipc_destroy(old_instance);
-        crm_err("pacemaker-fenced is already active, aborting startup");
+        crm_crit("Aborting start-up because another fencer instance is "
+                 "already active");
         goto done;
     } else {
         // Not up or not authentic, we'll proceed either way
@@ -620,32 +612,25 @@ main(int argc, char **argv)
 
     cluster = pcmk_cluster_new();
 
-    if (!stand_alone) {
 #if SUPPORT_COROSYNC
-        if (pcmk_get_cluster_layer() == pcmk_cluster_layer_corosync) {
-            pcmk_cluster_set_destroy_fn(cluster, stonith_peer_cs_destroy);
-            pcmk_cpg_set_deliver_fn(cluster, stonith_peer_ais_callback);
-            pcmk_cpg_set_confchg_fn(cluster, pcmk__cpg_confchg_cb);
-        }
+    if (pcmk_get_cluster_layer() == pcmk_cluster_layer_corosync) {
+        pcmk_cluster_set_destroy_fn(cluster, stonith_peer_cs_destroy);
+        pcmk_cpg_set_deliver_fn(cluster, stonith_peer_ais_callback);
+        pcmk_cpg_set_confchg_fn(cluster, pcmk__cpg_confchg_cb);
+    }
 #endif // SUPPORT_COROSYNC
 
-        pcmk__cluster_set_status_callback(&st_peer_update_callback);
+    pcmk__cluster_set_status_callback(&st_peer_update_callback);
 
-        if (pcmk_cluster_connect(cluster) != pcmk_rc_ok) {
-            exit_code = CRM_EX_FATAL;
-            crm_crit("Cannot sign in to the cluster... terminating");
-            goto done;
-        }
-        pcmk__str_update(&stonith_our_uname, cluster->uname);
+    if (pcmk_cluster_connect(cluster) != pcmk_rc_ok) {
+        exit_code = CRM_EX_FATAL;
+        crm_crit("Cannot sign in to the cluster... terminating");
+        goto done;
+    }
+    fenced_set_local_node(cluster->priv->node_name);
 
-        if (!options.no_cib_connect) {
-            setup_cib();
-        }
-
-    } else {
-        pcmk__str_update(&stonith_our_uname, "localhost");
-        crm_warn("Stand-alone mode is deprecated and will be removed "
-                 "in a future release");
+    if (!options.stand_alone) {
+        setup_cib();
     }
 
     init_device_list();
