@@ -18,6 +18,7 @@
 #include <grp.h>
 #include <signal.h>
 #include <stdbool.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -36,17 +37,15 @@ enum child_daemon_flags {
 };
 
 typedef struct pcmk_child_s {
+    enum pcmk_ipc_server server;
     pid_t pid;
     int respawn_count;
-    const char *name;
     const char *uid;
-    const char *command;
-    const char *endpoint;  /* IPC server name */
     int check_count;
     uint32_t flags;
 } pcmk_child_t;
 
-#define PCMK_PROCESS_CHECK_INTERVAL 1
+#define PCMK_PROCESS_CHECK_INTERVAL 1000    /* 1s */
 #define PCMK_PROCESS_CHECK_RETRIES  5
 #define SHUTDOWN_ESCALATION_PERIOD  180000  /* 3m */
 
@@ -55,33 +54,27 @@ typedef struct pcmk_child_s {
 
 static pcmk_child_t pcmk_children[] = {
     {
-        0, 0, "pacemaker-based", CRM_DAEMON_USER,
-        CRM_DAEMON_DIR "/pacemaker-based", PCMK__SERVER_BASED_RO,
+        pcmk_ipc_based, 0, 0, CRM_DAEMON_USER,
         0, child_respawn | child_needs_cluster
     },
     {
-        0, 0, "pacemaker-fenced", NULL,
-        CRM_DAEMON_DIR "/pacemaker-fenced", "stonith-ng",
+        pcmk_ipc_fenced, 0, 0, NULL,
         0, child_respawn | child_needs_cluster
     },
     {
-        0, 0, "pacemaker-execd", NULL,
-        CRM_DAEMON_DIR "/pacemaker-execd", CRM_SYSTEM_LRMD,
+        pcmk_ipc_execd, 0, 0, NULL,
         0, child_respawn
     },
     {
-        0, 0, "pacemaker-attrd", CRM_DAEMON_USER,
-        CRM_DAEMON_DIR "/pacemaker-attrd", PCMK__VALUE_ATTRD,
+        pcmk_ipc_attrd, 0, 0, CRM_DAEMON_USER,
         0, child_respawn | child_needs_cluster
     },
     {
-        0, 0, "pacemaker-schedulerd", CRM_DAEMON_USER,
-        CRM_DAEMON_DIR "/pacemaker-schedulerd", CRM_SYSTEM_PENGINE,
+        pcmk_ipc_schedulerd, 0, 0, CRM_DAEMON_USER,
         0, child_respawn
     },
     {
-        0, 0, "pacemaker-controld", CRM_DAEMON_USER,
-        CRM_DAEMON_DIR "/pacemaker-controld", CRM_SYSTEM_CRMD,
+        pcmk_ipc_controld, 0, 0, CRM_DAEMON_USER,
         0, child_respawn | child_needs_cluster
     },
 };
@@ -125,6 +118,22 @@ static void pcmk_process_exit(pcmk_child_t * child);
 static gboolean pcmk_shutdown_worker(gpointer user_data);
 static gboolean stop_child(pcmk_child_t * child, int signal);
 
+/*!
+ * \internal
+ * \brief Get path to subdaemon executable
+ *
+ * \param[in] subdaemon  Subdaemon to get path for
+ *
+ * \return Newly allocated string with path to subdaemon executable
+ * \note It is the caller's responsibility to free() the return value
+ */
+static inline char *
+subdaemon_path(pcmk_child_t *subdaemon)
+{
+    return crm_strdup_printf(CRM_DAEMON_DIR "/%s",
+                             pcmk__server_name(subdaemon->server));
+}
+
 static bool
 pcmkd_cluster_connected(void)
 {
@@ -141,11 +150,12 @@ check_next_subdaemon(gpointer user_data)
     static int next_child = 0;
 
     pcmk_child_t *child = &(pcmk_children[next_child]);
+    const char *name = pcmk__server_name(child->server);
     const long long pid = PCMK__SPECIAL_PID_AS_0(child->pid);
     int rc = child_liveness(child);
 
-    crm_trace("Checked %s[%lld]: %s (%d)",
-              child->name, pid, pcmk_rc_str(rc), rc);
+    crm_trace("Checked subdaemon %s[%lld]: %s (%d)",
+              name, pid, pcmk_rc_str(rc), rc);
 
     switch (rc) {
         case pcmk_rc_ok:
@@ -155,9 +165,10 @@ check_next_subdaemon(gpointer user_data)
 
         case pcmk_rc_ipc_pid_only: // Child was previously OK
             if (++(child->check_count) >= PCMK_PROCESS_CHECK_RETRIES) {
-                crm_crit("%s[%lld] is unresponsive to IPC after %d attempt%s "
-                         "and will now be killed",
-                         child->name, pid, child->check_count,
+                // cts-lab looks for this message
+                crm_crit("Subdaemon %s[%lld] is unresponsive to IPC "
+                         "after %d attempt%s and will now be killed",
+                         name, pid, child->check_count,
                          pcmk__plural_s(child->check_count));
                 stop_child(child, SIGKILL);
                 if (pcmk_is_set(child->flags, child_respawn)) {
@@ -165,8 +176,9 @@ check_next_subdaemon(gpointer user_data)
                     child->check_count = 0;
                 }
             } else {
-                crm_notice("%s[%lld] is unresponsive to IPC after %d attempt%s",
-                           child->name, pid, child->check_count,
+                crm_notice("Subdaemon %s[%lld] is unresponsive to IPC "
+                           "after %d attempt%s (will recheck later)",
+                           name, pid, child->check_count,
                            pcmk__plural_s(child->check_count));
                 if (pcmk_is_set(child->flags, child_respawn)) {
                     /* as long as the respawn-limit isn't reached
@@ -194,15 +206,15 @@ check_next_subdaemon(gpointer user_data)
                 }
             }
             if (!pcmk_is_set(child->flags, child_active_before_startup)) {
-                crm_trace("%s[%lld] terminated (relying on SIGCHLD handler)",
-                          child->name, pid);
+                crm_trace("Subdaemon %s[%lld] terminated", name, pid);
                 break;
             }
             if (pcmk_is_set(child->flags, child_respawn)) {
-                crm_err("%s[%lld] terminated", child->name, pid);
+                // cts-lab looks for this message
+                crm_err("Subdaemon %s[%lld] terminated", name, pid);
             } else {
                 /* orderly shutdown */
-                crm_notice("%s[%lld] terminated", child->name, pid);
+                crm_notice("Subdaemon %s[%lld] terminated", name, pid);
             }
             pcmk_process_exit(child);
             break;
@@ -228,7 +240,8 @@ escalate_shutdown(gpointer data)
 
     } else if (child->pid != 0) {
         /* Use SIGSEGV instead of SIGKILL to create a core so we can see what it was up to */
-        crm_err("Child %s not terminating in a timely manner, forcing", child->name);
+        crm_err("Subdaemon %s not terminating in a timely manner, forcing",
+                pcmk__server_name(child->server));
         stop_child(child, SIGSEGV);
     }
     return FALSE;
@@ -241,6 +254,7 @@ pcmk_child_exit(mainloop_child_t * p, pid_t pid, int core, int signo, int exitco
     const char *name = mainloop_child_name(p);
 
     if (signo) {
+        // cts-lab looks for this message
         do_crm_log(((signo == SIGKILL)? LOG_WARNING : LOG_ERR),
                    "%s[%d] terminated with signal %d (%s)%s",
                    name, pid, signo, strsignal(signo),
@@ -262,14 +276,23 @@ pcmk_child_exit(mainloop_child_t * p, pid_t pid, int core, int signo, int exitco
                 break;
 
             case CRM_EX_PANIC:
-                crm_emerg("%s[%d] instructed the machine to reset", name, pid);
-                child->flags &= ~child_respawn;
-                fatal_error = TRUE;
-                pcmk__panic(__func__);
-                pcmk_shutdown(SIGTERM);
+                {
+                    char *msg = NULL;
+
+                    child->flags &= ~child_respawn;
+                    fatal_error = TRUE;
+                    msg = crm_strdup_printf("Subdaemon %s[%d] requested panic",
+                                            name, pid);
+                    pcmk__panic(msg);
+
+                    // Should never get here
+                    free(msg);
+                    pcmk_shutdown(SIGTERM);
+                }
                 break;
 
             default:
+                // cts-lab looks for this message
                 crm_err("%s[%d] exited with status %d (%s)",
                         name, pid, exitcode, crm_exit_str(exitcode));
                 break;
@@ -282,13 +305,14 @@ pcmk_child_exit(mainloop_child_t * p, pid_t pid, int core, int signo, int exitco
 static void
 pcmk_process_exit(pcmk_child_t * child)
 {
+    const char *name = pcmk__server_name(child->server);
     child->pid = 0;
     child->flags &= ~child_active_before_startup;
     child->check_count = 0;
 
     child->respawn_count += 1;
     if (child->respawn_count > MAX_RESPAWN) {
-        crm_err("Child respawn count exceeded by %s", child->name);
+        crm_err("Subdaemon %s exceeded maximum respawn count", name);
         child->flags &= ~child_respawn;
     }
 
@@ -300,22 +324,19 @@ pcmk_process_exit(pcmk_child_t * child)
         /* nothing to do */
 
     } else if (crm_is_true(pcmk__env_option(PCMK__ENV_FAIL_FAST))) {
-        crm_err("Rebooting system because of %s", child->name);
-        pcmk__panic(__func__);
+        pcmk__panic("Subdaemon failed");
 
     } else if (child_liveness(child) == pcmk_rc_ok) {
-        crm_warn("One-off suppressing strict respawning of a child process %s,"
-                 " appears alright per %s IPC end-point",
-                 child->name, child->endpoint);
+        crm_warn("Not respawning subdaemon %s because IPC endpoint %s is OK",
+                 name, pcmk__server_ipc_name(child->server));
 
     } else if (pcmk_is_set(child->flags, child_needs_cluster) && !pcmkd_cluster_connected()) {
-        crm_notice("Not respawning %s subdaemon until cluster returns",
-                   child->name);
+        crm_notice("Not respawning subdaemon %s until cluster returns", name);
         child->flags |= child_needs_retry;
 
     } else {
-        crm_notice("Respawning %s subdaemon after unexpected exit",
-                   child->name);
+        // cts-lab looks for this message
+        crm_notice("Respawning subdaemon %s after unexpected exit", name);
         start_child(child);
     }
 }
@@ -333,42 +354,38 @@ pcmk_shutdown_worker(gpointer user_data)
 
     for (; phase >= 0; phase--) {
         pcmk_child_t *child = &(pcmk_children[phase]);
+        const char *name = pcmk__server_name(child->server);
 
         if (child->pid != 0) {
             time_t now = time(NULL);
 
             if (pcmk_is_set(child->flags, child_respawn)) {
                 if (child->pid == PCMK__SPECIAL_PID) {
-                    crm_warn("The process behind %s IPC cannot be"
-                             " terminated, so either wait the graceful"
-                             " period of %ld s for its native termination"
-                             " if it vitally depends on some other daemons"
-                             " going down in a controlled way already,"
-                             " or locate and kill the correct %s process"
-                             " on your own; set PCMK_" PCMK__ENV_FAIL_FAST "=1"
-                             " to avoid this altogether next time around",
-                             child->name, (long) SHUTDOWN_ESCALATION_PERIOD,
-                             child->command);
+                    crm_warn("Subdaemon %s cannot be terminated (shutdown "
+                             "will be escalated after %ld seconds if it does "
+                             "not terminate on its own; set PCMK_"
+                             PCMK__ENV_FAIL_FAST "=1 to exit immediately "
+                             "instead)",
+                             name, (long) SHUTDOWN_ESCALATION_PERIOD);
                 }
                 next_log = now + 30;
                 child->flags &= ~child_respawn;
                 stop_child(child, SIGTERM);
                 if (phase < PCMK_CHILD_CONTROLD) {
-                    g_timeout_add(SHUTDOWN_ESCALATION_PERIOD,
-                                  escalate_shutdown, child);
+                    pcmk__create_timer(SHUTDOWN_ESCALATION_PERIOD,
+                                       escalate_shutdown, child);
                 }
 
             } else if (now >= next_log) {
                 next_log = now + 30;
-                crm_notice("Still waiting for %s to terminate "
-                           CRM_XS " pid=%lld",
-                           child->name, (long long) child->pid);
+                crm_notice("Still waiting for subdaemon %s to terminate "
+                           QB_XS " pid=%lld", name, (long long) child->pid);
             }
             return TRUE;
         }
 
         /* cleanup */
-        crm_debug("%s confirmed stopped", child->name);
+        crm_debug("Subdaemon %s confirmed stopped", name);
         child->pid = 0;
     }
 
@@ -381,24 +398,11 @@ pcmk_shutdown_worker(gpointer user_data)
         return TRUE;
     }
 
-    // @COMPAT Drop shutdown delay at 3.0.0
-    {
-        const char *delay = pcmk__env_option(PCMK__ENV_SHUTDOWN_DELAY);
-        if(delay) {
-            long long delay_ms = crm_get_msec(delay);
-
-            sync();
-            if (delay_ms > 0) {
-                pcmk__sleep_ms((unsigned int) QB_MIN(delay_ms, UINT_MAX));
-            }
-        }
-    }
-
     g_main_loop_quit(mainloop);
 
     if (fatal_error) {
         crm_notice("Shutting down and staying down after fatal error");
-#ifdef SUPPORT_COROSYNC
+#if SUPPORT_COROSYNC
         pcmkd_shutdown_corosync();
 #endif
         crm_exit(CRM_EX_FATAL);
@@ -420,56 +424,54 @@ start_child(pcmk_child_t * child)
     gid_t gid = 0;
     gboolean use_valgrind = FALSE;
     gboolean use_callgrind = FALSE;
+    const char *name = pcmk__server_name(child->server);
     const char *env_valgrind = pcmk__env_option(PCMK__ENV_VALGRIND_ENABLED);
     const char *env_callgrind = pcmk__env_option(PCMK__ENV_CALLGRIND_ENABLED);
 
     child->flags &= ~child_active_before_startup;
     child->check_count = 0;
 
-    if (child->command == NULL) {
-        crm_info("Nothing to do for child \"%s\"", child->name);
-        return pcmk_rc_ok;
-    }
-
     if (env_callgrind != NULL && crm_is_true(env_callgrind)) {
         use_callgrind = TRUE;
         use_valgrind = TRUE;
 
-    } else if (env_callgrind != NULL && strstr(env_callgrind, child->name)) {
+    } else if ((env_callgrind != NULL)
+               && (strstr(env_callgrind, name) != NULL)) {
         use_callgrind = TRUE;
         use_valgrind = TRUE;
 
     } else if (env_valgrind != NULL && crm_is_true(env_valgrind)) {
         use_valgrind = TRUE;
 
-    } else if (env_valgrind != NULL && strstr(env_valgrind, child->name)) {
+    } else if ((env_valgrind != NULL)
+               && (strstr(env_valgrind, name) != NULL)) {
         use_valgrind = TRUE;
     }
 
-    if (use_valgrind && strlen(VALGRIND_BIN) == 0) {
-        crm_warn("Cannot enable valgrind for %s:"
-                 " The location of the valgrind binary is unknown", child->name);
+    if (use_valgrind && strlen(PCMK__VALGRIND_EXEC) == 0) {
+        crm_warn("Cannot enable valgrind for subdaemon %s: valgrind not found",
+                 name);
         use_valgrind = FALSE;
     }
 
-    if (child->uid) {
-        if (crm_user_lookup(child->uid, &uid, &gid) < 0) {
-            crm_err("Invalid user (%s) for %s: not found", child->uid, child->name);
-            return EACCES;
-        }
-        crm_info("Using uid=%u and group=%u for process %s", uid, gid, child->name);
+    if ((child->uid != NULL) && (crm_user_lookup(child->uid, &uid, &gid) < 0)) {
+        crm_err("Invalid user (%s) for subdaemon %s: not found",
+                child->uid, name);
+        return EACCES;
     }
 
     child->pid = fork();
-    CRM_ASSERT(child->pid != -1);
+    pcmk__assert(child->pid != -1);
 
     if (child->pid > 0) {
         /* parent */
-        mainloop_child_add(child->pid, 0, child->name, child, pcmk_child_exit);
+        mainloop_child_add(child->pid, 0, name, child, pcmk_child_exit);
 
-        crm_info("Forked child %lld for process %s%s",
-                 (long long) child->pid, child->name,
-                 use_valgrind ? " (valgrind enabled: " VALGRIND_BIN ")" : "");
+        crm_info("Forked process %lld using user %lu (%s) and group %lu "
+                 "for subdaemon %s%s",
+                 (long long) child->pid, (unsigned long) uid,
+                 pcmk__s(child->uid, "root"), (unsigned long) gid, name,
+                 use_valgrind ? " (valgrind enabled: " PCMK__VALGRIND_EXEC ")" : "");
         return pcmk_rc_ok;
 
     } else {
@@ -477,39 +479,43 @@ start_child(pcmk_child_t * child)
         (void)setsid();
 
         /* Setup the two alternate arg arrays */
-        opts_vgrind[0] = pcmk__str_copy(VALGRIND_BIN);
+        opts_vgrind[0] = pcmk__str_copy(PCMK__VALGRIND_EXEC);
         if (use_callgrind) {
             opts_vgrind[1] = pcmk__str_copy("--tool=callgrind");
             opts_vgrind[2] = pcmk__str_copy("--callgrind-out-file="
                                             CRM_STATE_DIR "/callgrind.out.%p");
-            opts_vgrind[3] = pcmk__str_copy(child->command);
+            opts_vgrind[3] = subdaemon_path(child);
             opts_vgrind[4] = NULL;
         } else {
-            opts_vgrind[1] = pcmk__str_copy(child->command);
+            opts_vgrind[1] = subdaemon_path(child);
             opts_vgrind[2] = NULL;
             opts_vgrind[3] = NULL;
             opts_vgrind[4] = NULL;
         }
-        opts_default[0] = pcmk__str_copy(child->command);
+        opts_default[0] = subdaemon_path(child);
 
         if(gid) {
             // Drop root group access if not needed
             if (!need_root_group && (setgid(gid) < 0)) {
-                crm_warn("Could not set group to %d: %s", gid, strerror(errno));
+                crm_warn("Could not set subdaemon %s group to %lu: %s",
+                         name, (unsigned long) gid, strerror(errno));
             }
 
             /* Initialize supplementary groups to only those always granted to
              * the user, plus haclient (so we can access IPC).
              */
             if (initgroups(child->uid, gid) < 0) {
-                crm_err("Cannot initialize groups for %s: %s (%d)",
-                        child->uid, pcmk_rc_str(errno), errno);
+                crm_err("Cannot initialize system groups for subdaemon %s: %s "
+                        QB_XS " errno=%d",
+                        name, pcmk_rc_str(errno), errno);
             }
         }
 
         if (uid && setuid(uid) < 0) {
-            crm_warn("Could not set user to %s (id %d): %s",
-                     child->uid, uid, strerror(errno));
+            crm_warn("Could not set subdaemon %s user to %s: %s "
+                     QB_XS " uid=%lu errno=%d",
+                     name, strerror(errno), child->uid, (unsigned long) uid,
+                     errno);
         }
 
         pcmk__close_fds_in_child(true);
@@ -519,11 +525,14 @@ start_child(pcmk_child_t * child)
         pcmk__open_devnull(O_WRONLY);   // stderr (fd 2)
 
         if (use_valgrind) {
-            (void)execvp(VALGRIND_BIN, opts_vgrind);
+            (void)execvp(PCMK__VALGRIND_EXEC, opts_vgrind);
         } else {
-            (void)execvp(child->command, opts_default);
+            char *path = subdaemon_path(child);
+
+            (void) execvp(path, opts_default);
+            free(path);
         }
-        crm_crit("Could not execute %s: %s", child->command, strerror(errno));
+        crm_crit("Could not execute subdaemon %s: %s", name, strerror(errno));
         crm_exit(CRM_EX_FATAL);
     }
     return pcmk_rc_ok;          /* never reached */
@@ -556,49 +565,43 @@ child_liveness(pcmk_child_t *child)
     const gid_t root_gid = 0;
     const uid_t *ref_uid;
     const gid_t *ref_gid;
+    const char *name = pcmk__server_name(child->server);
     int rc = pcmk_rc_ipc_unresponsive;
+    int legacy_rc = pcmk_ok;
     pid_t ipc_pid = 0;
 
-    if (child->endpoint == NULL
-            && (child->pid <= 0 || child->pid == PCMK__SPECIAL_PID)) {
-        crm_err("Cannot track child %s for missing both API end-point and PID",
-                child->name);
-        rc = EINVAL; // Misuse of function when child is not trackable
+    if (child->uid == NULL) {
+        ref_uid = &root_uid;
+        ref_gid = &root_gid;
+    } else {
+        ref_uid = &cl_uid;
+        ref_gid = &cl_gid;
+        legacy_rc = pcmk_daemon_user(&cl_uid, &cl_gid);
+    }
 
-    } else if (child->endpoint != NULL) {
-        int legacy_rc = pcmk_ok;
+    if (legacy_rc < 0) {
+        rc = pcmk_legacy2rc(legacy_rc);
+        crm_err("Could not find user and group IDs for user %s: %s "
+                QB_XS " rc=%d", CRM_DAEMON_USER, pcmk_rc_str(rc), rc);
+    } else {
+        const char *ipc_name = pcmk__server_ipc_name(child->server);
 
-        if (child->uid == NULL) {
-            ref_uid = &root_uid;
-            ref_gid = &root_gid;
-        } else {
-            ref_uid = &cl_uid;
-            ref_gid = &cl_gid;
-            legacy_rc = pcmk_daemon_user(&cl_uid, &cl_gid);
-        }
-
-        if (legacy_rc < 0) {
-            rc = pcmk_legacy2rc(legacy_rc);
-            crm_err("Could not find user and group IDs for user %s: %s "
-                    CRM_XS " rc=%d", CRM_DAEMON_USER, pcmk_rc_str(rc), rc);
-        } else {
-            rc = pcmk__ipc_is_authentic_process_active(child->endpoint,
-                                                       *ref_uid, *ref_gid,
-                                                       &ipc_pid);
-            if ((rc == pcmk_rc_ok) || (rc == pcmk_rc_ipc_unresponsive)) {
-                if (child->pid <= 0) {
-                    /* If rc is pcmk_rc_ok, ipc_pid is nonzero and this
-                     * initializes a new child. If rc is
-                     * pcmk_rc_ipc_unresponsive, ipc_pid is zero, and we will
-                     * investigate further.
-                     */
-                    child->pid = ipc_pid;
-                } else if ((ipc_pid != 0) && (child->pid != ipc_pid)) {
-                    /* An unexpected (but authorized) process is responding to
-                     * IPC. Investigate further.
-                     */
-                    rc = pcmk_rc_ipc_unresponsive;
-                }
+        rc = pcmk__ipc_is_authentic_process_active(ipc_name,
+                                                   *ref_uid, *ref_gid,
+                                                   &ipc_pid);
+        if ((rc == pcmk_rc_ok) || (rc == pcmk_rc_ipc_unresponsive)) {
+            if (child->pid <= 0) {
+                /* If rc is pcmk_rc_ok, ipc_pid is nonzero and this
+                 * initializes a new child. If rc is
+                 * pcmk_rc_ipc_unresponsive, ipc_pid is zero, and we will
+                 * investigate further.
+                 */
+                child->pid = ipc_pid;
+            } else if ((ipc_pid != 0) && (child->pid != ipc_pid)) {
+                /* An unexpected (but authorized) process is responding to
+                 * IPC. Investigate further.
+                 */
+                rc = pcmk_rc_ipc_unresponsive;
             }
         }
     }
@@ -610,12 +613,11 @@ child_liveness(pcmk_child_t *child)
          * the only change possible from a proper child's PID into "special" PID
          * of 1 behind more loosely related process.
          */
-        int ret = pcmk__pid_active(child->pid, child->name);
+        int ret = pcmk__pid_active(child->pid, name);
 
         if (ipc_pid && ((ret != pcmk_rc_ok)
                         || ipc_pid == PCMK__SPECIAL_PID
-                        || (pcmk__pid_active(ipc_pid,
-                                             child->name) == pcmk_rc_ok))) {
+                        || (pcmk__pid_active(ipc_pid, name) == pcmk_rc_ok))) {
             /* An unexpected (but authorized) process was detected at the IPC
              * endpoint, and either it is active, or the child we're tracking is
              * not.
@@ -695,9 +697,10 @@ find_and_track_existing_processes(void)
     for (rounds = 1; rounds <= WAIT_TRIES; rounds++) {
         wait_in_progress = false;
         for (i = 0; i < PCMK__NELEM(pcmk_children); i++) {
+            const char *name = pcmk__server_name(pcmk_children[i].server);
+            const char *ipc_name = NULL;
 
-            if ((pcmk_children[i].endpoint == NULL)
-                || (pcmk_children[i].respawn_count < 0)) {
+            if (pcmk_children[i].respawn_count < 0) {
                 continue;
             }
 
@@ -711,6 +714,7 @@ find_and_track_existing_processes(void)
             }
 
             // @TODO Functionize more of this to reduce nesting
+            ipc_name = pcmk__server_ipc_name(pcmk_children[i].server);
             pcmk_children[i].respawn_count = rounds;
             switch (rc) {
                 case pcmk_rc_ok:
@@ -719,8 +723,7 @@ find_and_track_existing_processes(void)
                             crm_crit("Cannot reliably track pre-existing"
                                      " authentic process behind %s IPC on this"
                                      " platform and PCMK_" PCMK__ENV_FAIL_FAST
-                                     " requested",
-                                     pcmk_children[i].endpoint);
+                                     " requested", ipc_name);
                             return EOPNOTSUPP;
                         } else if (pcmk_children[i].respawn_count == WAIT_TRIES) {
                             crm_notice("Assuming pre-existing authentic, though"
@@ -731,7 +734,7 @@ find_and_track_existing_processes(void)
                                        " not requested), we just switch to a"
                                        " less optimal IPC liveness monitoring"
                                        " (not very suitable for heavy load)",
-                                       pcmk_children[i].name, WAIT_TRIES - 1);
+                                       name, WAIT_TRIES - 1);
                             crm_warn("The process behind %s IPC cannot be"
                                      " terminated, so the overall shutdown"
                                      " will get delayed implicitly (%ld s),"
@@ -739,20 +742,19 @@ find_and_track_existing_processes(void)
                                      " its native termination if it vitally"
                                      " depends on some other daemons going"
                                      " down in a controlled way already",
-                                     pcmk_children[i].name,
-                                     (long) SHUTDOWN_ESCALATION_PERIOD);
+                                     name, (long) SHUTDOWN_ESCALATION_PERIOD);
                         } else {
                             wait_in_progress = true;
                             crm_warn("Cannot reliably track pre-existing"
                                      " authentic process behind %s IPC on this"
                                      " platform, can still disappear in %d"
-                                     " attempt(s)", pcmk_children[i].endpoint,
+                                     " attempt(s)", ipc_name,
                                      WAIT_TRIES - pcmk_children[i].respawn_count);
                             continue;
                         }
                     }
                     crm_notice("Tracking existing %s process (pid=%lld)",
-                               pcmk_children[i].name,
+                               name,
                                (long long) PCMK__SPECIAL_PID_AS_0(
                                                pcmk_children[i].pid));
                     pcmk_children[i].respawn_count = -1;  /* 0~keep watching */
@@ -760,25 +762,25 @@ find_and_track_existing_processes(void)
                     break;
                 case pcmk_rc_ipc_pid_only:
                     if (pcmk_children[i].respawn_count == WAIT_TRIES) {
-                        crm_crit("%s IPC end-point for existing authentic"
+                        crm_crit("%s IPC endpoint for existing authentic"
                                  " process %lld did not (re)appear",
-                                 pcmk_children[i].endpoint,
+                                 ipc_name,
                                  (long long) PCMK__SPECIAL_PID_AS_0(
                                                  pcmk_children[i].pid));
                         return rc;
                     }
                     wait_in_progress = true;
-                    crm_warn("Cannot find %s IPC end-point for existing"
+                    crm_warn("Cannot find %s IPC endpoint for existing"
                              " authentic process %lld, can still (re)appear"
                              " in %d attempts (?)",
-                             pcmk_children[i].endpoint,
+                             ipc_name,
                              (long long) PCMK__SPECIAL_PID_AS_0(
                                              pcmk_children[i].pid),
                              WAIT_TRIES - pcmk_children[i].respawn_count);
                     continue;
                 default:
-                    crm_crit("Checked liveness of %s: %s " CRM_XS " rc=%d",
-                             pcmk_children[i].name, pcmk_rc_str(rc), rc);
+                    crm_crit("Checked liveness of %s: %s " QB_XS " rc=%d",
+                             name, pcmk_rc_str(rc), rc);
                     return rc;
             }
         }
@@ -791,8 +793,8 @@ find_and_track_existing_processes(void)
         pcmk_children[i].respawn_count = 0;  /* restore pristine state */
     }
 
-    g_timeout_add_seconds(PCMK_PROCESS_CHECK_INTERVAL, check_next_subdaemon,
-                          NULL);
+    pcmk__create_timer(PCMK_PROCESS_CHECK_INTERVAL, check_next_subdaemon,
+                       NULL);
     return pcmk_rc_ok;
 }
 
@@ -844,7 +846,8 @@ restart_cluster_subdaemons(void)
             continue;
         }
 
-        crm_notice("Respawning cluster-based subdaemon: %s", pcmk_children[i].name);
+        crm_notice("Respawning cluster-based subdaemon %s",
+                   pcmk__server_name(pcmk_children[i].server));
         if (start_child(&pcmk_children[i])) {
             pcmk_children[i].flags &= ~child_needs_retry;
         }
@@ -854,6 +857,8 @@ restart_cluster_subdaemons(void)
 static gboolean
 stop_child(pcmk_child_t * child, int signal)
 {
+    const char *name = pcmk__server_name(child->server);
+
     if (signal == 0) {
         signal = SIGTERM;
     }
@@ -862,27 +867,26 @@ stop_child(pcmk_child_t * child, int signal)
        - FreeBSD ~ how untrackable process behind IPC is masqueraded as
        - elsewhere: how "init" task is designated; in particular, in systemd
          arrangement of socket-based activation, this is pretty real */
-    if (child->command == NULL || child->pid == PCMK__SPECIAL_PID) {
-        crm_debug("Nothing to do for child \"%s\" (process %lld)",
-                  child->name, (long long) PCMK__SPECIAL_PID_AS_0(child->pid));
+    if (child->pid == PCMK__SPECIAL_PID) {
+        crm_debug("Nothing to do to stop subdaemon %s[%lld]",
+                  name, (long long) PCMK__SPECIAL_PID_AS_0(child->pid));
         return TRUE;
     }
 
     if (child->pid <= 0) {
-        crm_trace("Client %s not running", child->name);
+        crm_trace("Nothing to do to stop subdaemon %s: Not running", name);
         return TRUE;
     }
 
     errno = 0;
     if (kill(child->pid, signal) == 0) {
-        crm_notice("Stopping %s "CRM_XS" sent signal %d to process %lld",
-                   child->name, signal, (long long) child->pid);
-
+        crm_notice("Stopping subdaemon %s "
+                   QB_XS " via signal %d to process %lld",
+                   name, signal, (long long) child->pid);
     } else {
-        crm_err("Could not stop %s (process %lld) with signal %d: %s",
-                child->name, (long long) child->pid, signal, strerror(errno));
+        crm_err("Could not stop subdaemon %s[%lld] with signal %d: %s",
+                name, (long long) child->pid, signal, strerror(errno));
     }
 
     return TRUE;
 }
-
