@@ -28,14 +28,7 @@ init_working_set(void)
     pcmk_scheduler_t *scheduler = pe_new_working_set();
 
     pcmk__mem_assert(scheduler);
-
-    crm_config_error = FALSE;
-    crm_config_warning = FALSE;
-
-    was_processing_error = FALSE;
-    was_processing_warning = FALSE;
-
-    scheduler->priv = logger_out;
+    scheduler->priv->out = logger_out;
     return scheduler;
 }
 
@@ -63,7 +56,7 @@ handle_pecalc_request(pcmk__request_t *request)
     static char *last_digest = NULL;
     static char *filename = NULL;
 
-    unsigned int seq;
+    unsigned int seq = 0U;
     int series_id = 0;
     int series_wrap = 0;
     char *digest = NULL;
@@ -78,13 +71,13 @@ handle_pecalc_request(pcmk__request_t *request)
     pcmk__ipc_send_ack(request->ipc_client, request->ipc_id, request->ipc_flags,
                        PCMK__XE_ACK, NULL, CRM_EX_INDETERMINATE);
 
-    digest = calculate_xml_versioned_digest(xml_data, FALSE, FALSE,
-                                            CRM_FEATURE_SET);
+    digest = pcmk__digest_xml(xml_data, false);
     converted = pcmk__xml_copy(NULL, xml_data);
     if (pcmk__update_configured_schema(&converted, true) != pcmk_rc_ok) {
-        scheduler->graph = pcmk__xe_create(NULL, PCMK__XE_TRANSITION_GRAPH);
-        crm_xml_add_int(scheduler->graph, "transition_id", 0);
-        crm_xml_add_int(scheduler->graph, PCMK_OPT_CLUSTER_DELAY, 0);
+        scheduler->priv->graph = pcmk__xe_create(NULL,
+                                                 PCMK__XE_TRANSITION_GRAPH);
+        crm_xml_add_int(scheduler->priv->graph, "transition_id", 0);
+        crm_xml_add_int(scheduler->priv->graph, PCMK_OPT_CLUSTER_DELAY, 0);
         process = false;
         free(digest);
 
@@ -99,37 +92,38 @@ handle_pecalc_request(pcmk__request_t *request)
 
     if (process) {
         pcmk__schedule_actions(converted,
-                               pcmk_sched_no_counts
-                               |pcmk_sched_no_compat
-                               |pcmk_sched_show_utilization, scheduler);
+                               pcmk__sched_no_counts
+                               |pcmk__sched_show_utilization, scheduler);
     }
 
     // Get appropriate index into series[] array
-    if (was_processing_error || crm_config_error) {
+    if (pcmk_is_set(scheduler->flags, pcmk__sched_processing_error)
+        || pcmk__config_has_error) {
         series_id = 0;
-    } else if (was_processing_warning || crm_config_warning) {
+    } else if (pcmk_is_set(scheduler->flags, pcmk__sched_processing_warning)
+               || pcmk__config_has_warning) {
         series_id = 1;
     } else {
         series_id = 2;
     }
 
-    value = pcmk__cluster_option(scheduler->config_hash,
+    value = pcmk__cluster_option(scheduler->priv->options,
                                  series[series_id].param);
     if ((value == NULL)
         || (pcmk__scan_min_int(value, &series_wrap, -1) != pcmk_rc_ok)) {
         series_wrap = series[series_id].wrap;
     }
 
-    if (pcmk__read_series_sequence(PE_STATE_DIR, series[series_id].name,
+    if (pcmk__read_series_sequence(PCMK_SCHEDULER_INPUT_DIR, series[series_id].name,
                                    &seq) != pcmk_rc_ok) {
         // @TODO maybe handle errors better ...
-        seq = 0;
+        seq = 0U;
     }
     crm_trace("Series %s: wrap=%d, seq=%u, pref=%s",
               series[series_id].name, series_wrap, seq, value);
 
     scheduler->input = NULL;
-    reply = create_reply(msg, scheduler->graph);
+    reply = pcmk__new_reply(msg, scheduler->priv->graph);
 
     if (reply == NULL) {
         pcmk__format_result(&request->result, CRM_EX_ERROR, PCMK_EXEC_ERROR,
@@ -144,17 +138,13 @@ handle_pecalc_request(pcmk__request_t *request)
 
     } else if (!is_repoke) { // Input changed, save to disk
         free(filename);
-        filename = pcmk__series_filename(PE_STATE_DIR,
+        filename = pcmk__series_filename(PCMK_SCHEDULER_INPUT_DIR,
                                          series[series_id].name, seq, true);
     }
 
     crm_xml_add(reply, PCMK__XA_CRM_TGRAPH_IN, filename);
-    crm_xml_add_int(reply, PCMK__XA_GRAPH_ERRORS, was_processing_error);
-    crm_xml_add_int(reply, PCMK__XA_GRAPH_WARNINGS, was_processing_warning);
-    crm_xml_add_int(reply, PCMK__XA_CONFIG_ERRORS, crm_config_error);
-    crm_xml_add_int(reply, PCMK__XA_CONFIG_WARNINGS, crm_config_warning);
 
-    pcmk__log_transition_summary(filename);
+    pcmk__log_transition_summary(scheduler, filename);
 
     if (series_wrap == 0) {
         crm_debug("Not saving input to disk (disabled by configuration)");
@@ -166,15 +156,15 @@ handle_pecalc_request(pcmk__request_t *request)
         unlink(filename);
         crm_xml_add_ll(xml_data, PCMK_XA_EXECUTION_DATE,
                        (long long) execution_date);
-        pcmk__xml_write_file(xml_data, filename, true, NULL);
-        pcmk__write_series_sequence(PE_STATE_DIR, series[series_id].name,
+        pcmk__xml_write_file(xml_data, filename, true);
+        pcmk__write_series_sequence(PCMK_SCHEDULER_INPUT_DIR, series[series_id].name,
                                     ++seq, series_wrap);
     }
 
     pcmk__set_result(&request->result, CRM_EX_OK, PCMK_EXEC_DONE, NULL);
 
 done:
-    free_xml(converted);
+    pcmk__xml_free(converted);
     pe_free_working_set(scheduler);
 
     return reply;
@@ -283,7 +273,7 @@ pe_ipc_dispatch(qb_ipcs_connection_t * qbc, void *data, size_t size)
 
         if (reply != NULL) {
             pcmk__ipc_send_xml(c, id, reply, crm_ipc_server_event);
-            free_xml(reply);
+            pcmk__xml_free(reply);
         }
 
         reason = request.result.exit_reason;
@@ -306,7 +296,7 @@ pe_ipc_dispatch(qb_ipcs_connection_t * qbc, void *data, size_t size)
         pcmk__reset_request(&request);
     }
 
-    free_xml(msg);
+    pcmk__xml_free(msg);
     return 0;
 }
 
