@@ -7,11 +7,18 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef CIB_INTERNAL__H
-#  define CIB_INTERNAL__H
-#  include <crm/cib.h>
-#  include <crm/common/ipc_internal.h>
-#  include <crm/common/output_internal.h>
+#ifndef PCMK__CRM_CIB_INTERNAL__H
+#define PCMK__CRM_CIB_INTERNAL__H
+
+#include <crm/cib.h>
+#include <crm/common/ipc_internal.h>
+#include <crm/common/output_internal.h>
+#include <crm/common/servers_internal.h>
+#include <crm/common/strings_internal.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 // Request types for CIB manager IPC/CPG
 #define PCMK__CIB_REQUEST_SECONDARY     "cib_slave"
@@ -155,7 +162,7 @@ cib_t *cib_new_variant(void);
 static inline bool
 cib__client_triggers_refresh(const char *name)
 {
-    return !crm_is_daemon_name(name)
+    return (pcmk__parse_server(name) == pcmk_ipc_unknown)
            && !pcmk__str_any_of(name,
                                 "attrd_updater",
                                 "crm_attribute",
@@ -167,13 +174,11 @@ cib__client_triggers_refresh(const char *name)
 
 int cib__get_notify_patchset(const xmlNode *msg, const xmlNode **patchset);
 
-bool cib__element_in_patchset(const xmlNode *patchset, const char *element);
-
-int cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
-                   bool is_query, const char *section, xmlNode *req,
-                   xmlNode *input, bool manage_counters, bool *config_changed,
-                   xmlNode **current_cib, xmlNode **result_cib, xmlNode **diff,
-                   xmlNode **output);
+int cib_perform_op(cib_t *cib, const char *op, uint32_t call_options,
+                   cib__op_fn_t fn, bool is_query, const char *section,
+                   xmlNode *req, xmlNode *input, bool manage_counters,
+                   bool *config_changed, xmlNode **current_cib,
+                   xmlNode **result_cib, xmlNode **diff, xmlNode **output);
 
 int cib__create_op(cib_t *cib, const char *op, const char *host,
                    const char *section, xmlNode *data, int call_options,
@@ -249,8 +254,6 @@ int cib_process_xpath(const char *op, int options, const char *section,
                       const xmlNode *req, xmlNode *input, xmlNode *existing_cib,
                       xmlNode **result_cib, xmlNode ** answer);
 
-bool cib__config_changed_v1(xmlNode *last, xmlNode *next, xmlNode **diff);
-
 int cib_internal_op(cib_t * cib, const char *op, const char *host,
                     const char *section, xmlNode * data,
                     xmlNode ** output_data, int call_options, const char *user_name);
@@ -288,8 +291,7 @@ cib_callback_client_t* cib__lookup_id (int call_id);
  */
 int cib__signon_query(pcmk__output_t *out, cib_t **cib, xmlNode **cib_object);
 
-int cib__signon_attempts(cib_t *cib, const char *name, enum cib_conn_type type,
-                         int attempts);
+int cib__signon_attempts(cib_t *cib, enum cib_conn_type type, int attempts);
 
 int cib__clean_up_connection(cib_t **cib);
 
@@ -309,4 +311,8 @@ int cib__delete_node_attr(pcmk__output_t *out, cib_t *cib, int options,
                           const char *set_name, const char *attr_id, const char *attr_name,
                           const char *attr_value, const char *user_name);
 
+#ifdef __cplusplus
+}
 #endif
+
+#endif // PCMK__CRM_CIB_INTERNAL__H
