@@ -7,14 +7,18 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef CRM_COMMON_ACL_INTERNAL__H
-#define CRM_COMMON_ACL_INTERNAL__H
+#ifndef PCMK__CRM_COMMON_ACL_INTERNAL__H
+#define PCMK__CRM_COMMON_ACL_INTERNAL__H
 
 #include <string.h>         // strcmp()
 #include <libxml/tree.h>    // xmlNode
 
 #include <crm/common/xml_internal.h>    // enum xml_private_flags
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* internal ACL-related utilities */
 
 char *pcmk__uid2username(uid_t uid);
@@ -32,4 +36,8 @@ void pcmk__enable_acl(xmlNode *acl_source, xmlNode *target, const char *user);
 bool pcmk__check_acl(xmlNode *xml, const char *name,
                      enum xml_private_flags mode);
 
-#endif /* CRM_COMMON_INTERNAL__H */
+#ifdef __cplusplus
+}
+#endif
+
+#endif // PCMK__CRM_COMMON_INTERNAL__H
