@@ -7,11 +7,16 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PCMK__ALERT_INTERNAL__H
-#define PCMK__ALERT_INTERNAL__H
+#ifndef PCMK__CRM_COMMON_ALERTS_INTERNAL__H
+#define PCMK__CRM_COMMON_ALERTS_INTERNAL__H
 
 #include <glib.h>
 #include <stdbool.h>
+#include <stdint.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /* Default-Timeout to use before killing a alerts script (in milliseconds) */
 #define PCMK__ALERT_DEFAULT_TIMEOUT_MS (30000)
@@ -67,7 +72,7 @@ enum pcmk__alert_keys_e {
 #define PCMK__ALERT_INTERNAL_KEY_MAX 19
 #define PCMK__ALERT_NODE_SEQUENCE "CRM_alert_node_sequence"
 
-extern const char *pcmk__alert_keys[PCMK__ALERT_INTERNAL_KEY_MAX][3];
+extern const char *pcmk__alert_keys[PCMK__ALERT_INTERNAL_KEY_MAX];
 
 pcmk__alert_t *pcmk__dup_alert(const pcmk__alert_t *entry);
 pcmk__alert_t *pcmk__alert_new(const char *id, const char *path);
@@ -88,4 +93,9 @@ pcmk__alert_flag2text(enum pcmk__alert_flags flag)
         default:                    return "unknown";
     }
 }
+
+#ifdef __cplusplus
+}
 #endif
+
+#endif // PCMK__CRM_COMMON_ALERTS_INTERNAL__H
