@@ -10,9 +10,10 @@
 #ifndef PCMK__CRM_COMMON_GROUP_INTERNAL__H
 #define PCMK__CRM_COMMON_GROUP_INTERNAL__H
 
-#include <stdbool.h>                    // bool
-#include <crm/common/scheduler_types.h> // pcmk_resource_t
-#include <crm/common/resources.h>       // pcmk_rsc_variant_group
+#include <stdio.h>                          // NULL
+#include <stdbool.h>                        // bool
+#include <crm/common/scheduler_types.h>     // pcmk_resource_t
+#include <crm/common/resources_internal.h>  // pcmk__rsc_variant_group etc.
 
 #ifdef __cplusplus
 extern "C" {
@@ -37,7 +38,7 @@ enum pcmk__group_flags {
 static inline bool
 pcmk__is_group(const pcmk_resource_t *rsc)
 {
-    return (rsc != NULL) && (rsc->variant == pcmk_rsc_variant_group);
+    return (rsc != NULL) && (rsc->priv->variant == pcmk__rsc_variant_group);
 }
 
 #ifdef __cplusplus
