@@ -26,38 +26,6 @@ extern "C" {
  * \ingroup core
  */
 
-/*
- * Message creation utilities
- *
- * These are used for both IPC messages and cluster layer messages. However,
- * since this is public API, they stay in this header for backward
- * compatibility.
- */
-
-// @COMPAT Make internal when we can break API backward compatibility
-//! \deprecated Do not use
-#define create_reply(request, xml_response_data)    \
-    create_reply_adv(request, xml_response_data, __func__)
-
-// @COMPAT Make internal when we can break API backward compatibility
-//! \deprecated Do not use
-xmlNode *create_reply_adv(const xmlNode *request, xmlNode *xml_response_data,
-                          const char *origin);
-
-// @COMPAT Make internal when we can break API backward compatibility
-//! \deprecated Do not use
-#define create_request(task, xml_data, host_to, sys_to, sys_from, uuid_from) \
-    create_request_adv(task, xml_data, host_to, sys_to, sys_from, uuid_from, \
-                       __func__)
-
-// @COMPAT Make internal when we can break API backward compatibility
-//! \deprecated Do not use
-xmlNode *create_request_adv(const char *task, xmlNode *xml_data,
-                            const char *host_to, const char *sys_to,
-                            const char *sys_from, const char *uuid_from,
-                            const char *origin);
-
-
 /*
  * The library supports two methods of creating IPC connections. The older code
  * allows connecting to any arbitrary IPC name. The newer code only allows
@@ -73,8 +41,12 @@ xmlNode *create_request_adv(const char *task, xmlNode *xml_data,
  * Pacemaker daemon IPC
  */
 
+/* @COMPAT This is also used internally for cluster message types, but it's not
+ * worth the hassle of redefining this public API just to change the name.
+ */
 //! Available IPC interfaces
 enum pcmk_ipc_server {
+    pcmk_ipc_unknown,       //!< Unknown or invalid
     pcmk_ipc_attrd,         //!< Attribute manager
     pcmk_ipc_based,         //!< CIB manager
     pcmk_ipc_controld,      //!< Controller
@@ -173,18 +145,12 @@ enum crm_ipc_flags
     crm_ipc_server_event    = 0x00010000, /* Send an Event instead of a Response */
     crm_ipc_server_free     = 0x00020000, /* Free the iovec after sending */
     crm_ipc_proxied_relay_response = 0x00040000, /* all replies to proxied connections are sent as events, this flag preserves whether the event should be treated as an actual event, or a response.*/
-
-#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
-    crm_ipc_server_info     = 0x00100000, //!< \deprecated Unused
-    crm_ipc_server_error    = 0x00200000, //!< \deprecated Unused
-#endif
 };
 /* *INDENT-ON* */
 
 typedef struct crm_ipc_s crm_ipc_t;
 
 crm_ipc_t *crm_ipc_new(const char *name, size_t max_size);
-bool crm_ipc_connect(crm_ipc_t * client);
 void crm_ipc_close(crm_ipc_t * client);
 void crm_ipc_destroy(crm_ipc_t * client);
 void pcmk_free_ipc_event(struct iovec *event);
@@ -239,11 +205,6 @@ unsigned int crm_ipc_default_buffer_size(void);
 int crm_ipc_is_authentic_process(int sock, uid_t refuid, gid_t refgid,
                                  pid_t *gotpid, uid_t *gotuid, gid_t *gotgid);
 
-// @COMPAT Make internal when we can break API backward compatibility
-//! \deprecated Do not use
-xmlNode *create_hello_message(const char *uuid, const char *client_name,
-                              const char *major_version, const char *minor_version);
-
 #ifdef __cplusplus
 }
 #endif
