@@ -7,14 +7,19 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PCMK__ISO8601_INTERNAL__H
-#define PCMK__ISO8601_INTERNAL__H
+#ifndef PCMK__CRM_COMMON_ISO8601_INTERNAL__H
+#define PCMK__CRM_COMMON_ISO8601_INTERNAL__H
 
+#include <glib.h>
 #include <time.h>
 #include <sys/time.h>
 #include <ctype.h>
 #include <crm/common/iso8601.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct pcmk__time_us pcmk__time_hr_t;
 
 pcmk__time_hr_t *pcmk__time_hr_convert(pcmk__time_hr_t *target,
@@ -39,4 +44,8 @@ struct pcmk__time_us {
     int useconds;
 };
 
+#ifdef __cplusplus
+}
 #endif
+
+#endif // PCMK__CRM_COMMON_ISO8601_INTERNAL__H
