@@ -10,9 +10,10 @@
 #ifndef PCMK__CRM_COMMON_PRIMITIVE_INTERNAL__H
 #define PCMK__CRM_COMMON_PRIMITIVE_INTERNAL__H
 
-#include <stdbool.h>                    // bool
-#include <crm/common/scheduler_types.h> // pcmk_resource_t
-#include <crm/common/resources.h>       // pcmk_rsc_variant_primitive
+#include <stdio.h>                          // NULL
+#include <stdbool.h>                        // bool, false
+#include <crm/common/scheduler_types.h>     // pcmk_resource_t
+#include <crm/common/resources_internal.h>  // pcmk__rsc_variant_primitive etc.
 
 #ifdef __cplusplus
 extern "C" {
@@ -29,7 +30,10 @@ extern "C" {
 static inline bool
 pcmk__is_primitive(const pcmk_resource_t *rsc)
 {
-    return (rsc != NULL) && (rsc->variant == pcmk_rsc_variant_primitive);
+    if (rsc == NULL) {
+        return false;
+    }
+    return rsc->priv->variant == pcmk__rsc_variant_primitive;
 }
 
 #ifdef __cplusplus
