@@ -9,6 +9,8 @@
 #ifndef PCMK__CRM_COMMON_RESULTS__H
 #define PCMK__CRM_COMMON_RESULTS__H
 
+#include <glib.h>           // gboolean
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -39,12 +41,6 @@ extern "C" {
 # endif
 #endif
 
-#define CRM_ASSERT(expr) do {                                                \
-        if (!(expr)) {                                                       \
-            crm_abort(__FILE__, __func__, __LINE__, #expr, TRUE, FALSE);     \
-        }                                                                    \
-    } while(0)
-
 /*
  * Function return codes
  *
@@ -201,26 +197,6 @@ enum ocf_exitcode {
      */
     PCMK_OCF_CONNECTION_DIED      = 189, //!< \deprecated See PCMK_EXEC_NOT_CONNECTED
     PCMK_OCF_UNKNOWN              = 193, //!< Action is pending
-
-#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
-    // Former Pacemaker extensions
-    PCMK_OCF_EXEC_ERROR           = 192, //!< \deprecated (Unused)
-    PCMK_OCF_SIGNAL               = 194, //!< \deprecated (Unused)
-    PCMK_OCF_NOT_SUPPORTED        = 195, //!< \deprecated (Unused)
-    PCMK_OCF_PENDING              = 196, //!< \deprecated (Unused)
-    PCMK_OCF_CANCELLED            = 197, //!< \deprecated (Unused)
-    PCMK_OCF_TIMEOUT              = 198, //!< \deprecated (Unused)
-    PCMK_OCF_OTHER_ERROR          = 199, //!< \deprecated (Unused)
-
-    //! \deprecated Use PCMK_OCF_RUNNING_PROMOTED instead
-    PCMK_OCF_RUNNING_MASTER     = PCMK_OCF_RUNNING_PROMOTED,
-
-    //! \deprecated Use PCMK_OCF_FAILED_PROMOTED instead
-    PCMK_OCF_FAILED_MASTER      = PCMK_OCF_FAILED_PROMOTED,
-
-    //! \deprecated Use PCMK_OCF_DEGRADED_PROMOTED instead
-    PCMK_OCF_DEGRADED_MASTER    = PCMK_OCF_DEGRADED_PROMOTED,
-#endif
 };
 
 // NOTE: sbd (as of at least 1.5.2) uses this
@@ -391,15 +367,19 @@ const char *crm_exit_str(crm_exit_t exit_code);
 
 _Noreturn crm_exit_t crm_exit(crm_exit_t rc);
 
+/* coverity[+kill] */
+void crm_abort(const char *file, const char *function, int line,
+               const char *condition, gboolean do_core, gboolean do_fork);
+
 static inline const char *
 pcmk_exec_status_str(enum pcmk_exec_status status)
 {
     switch (status) {
-        case PCMK_EXEC_PENDING:         return "pending";
-        case PCMK_EXEC_DONE:            return "complete";
+        case PCMK_EXEC_PENDING:         return "Pending";
+        case PCMK_EXEC_DONE:            return "Done";
         case PCMK_EXEC_CANCELLED:       return "Cancelled";
-        case PCMK_EXEC_TIMEOUT:         return "Timed Out";
-        case PCMK_EXEC_NOT_SUPPORTED:   return "NOT SUPPORTED";
+        case PCMK_EXEC_TIMEOUT:         return "Timed out";
+        case PCMK_EXEC_NOT_SUPPORTED:   return "Unsupported";
         case PCMK_EXEC_ERROR:           return "Error";
         case PCMK_EXEC_ERROR_HARD:      return "Hard error";
         case PCMK_EXEC_ERROR_FATAL:     return "Fatal error";
@@ -408,7 +388,7 @@ pcmk_exec_status_str(enum pcmk_exec_status status)
         case PCMK_EXEC_INVALID:         return "Cannot execute now";
         case PCMK_EXEC_NO_FENCE_DEVICE: return "No fence device";
         case PCMK_EXEC_NO_SECRETS:      return "CIB secrets unavailable";
-        default:                        return "UNKNOWN!";
+        default:                        return "Unrecognized status (bug?)";
     }
 }
 
@@ -416,4 +396,8 @@ pcmk_exec_status_str(enum pcmk_exec_status status)
 }
 #endif
 
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+#include <crm/common/results_compat.h>
+#endif
+
 #endif
