@@ -21,16 +21,16 @@ extern "C" {
  */
 
 //! Node object (including information that may vary depending on resource)
-typedef struct pe_node_s pcmk_node_t;
+typedef struct pcmk__scored_node pcmk_node_t;
 
 //! Resource object
-typedef struct pe_resource_s pcmk_resource_t;
+typedef struct pcmk__resource pcmk_resource_t;
 
 //! Action object
-typedef struct pe_action_s pcmk_action_t;
+typedef struct pcmk__action pcmk_action_t;
 
 //! Scheduler object
-typedef struct pe_working_set_s pcmk_scheduler_t;
+typedef struct pcmk__scheduler pcmk_scheduler_t;
 
 #ifdef __cplusplus
 }
