@@ -48,31 +48,9 @@ typedef const xmlChar *pcmkXmlStr;
 // NOTE: sbd (as of at least 1.5.2) uses this
 xmlNode *get_xpath_object(const char *xpath, xmlNode * xml_obj, int error_level);
 
-char *calculate_on_disk_digest(xmlNode * local_cib);
-char *calculate_operation_digest(xmlNode * local_cib, const char *version);
-char *calculate_xml_versioned_digest(xmlNode * input, gboolean sort, gboolean do_filter,
-                                     const char *version);
-
-// NOTE: sbd (as of at least 1.5.2) uses this
-/*!
- * \brief Initialize the CRM XML subsystem
- *
- * This method sets global XML settings and loads pacemaker schemas into the cache.
- */
-void crm_xml_init(void);
-
-void crm_xml_cleanup(void);
-
-void pcmk_free_xml_subtree(xmlNode *xml);
-
-// NOTE: sbd (as of at least 1.5.2) uses this
-void free_xml(xmlNode * child);
-
-xmlNode *sorted_xml(xmlNode * input, xmlNode * parent, gboolean recursive);
 xmlXPathObjectPtr xpath_search(const xmlNode *xml_top, const char *path);
 void crm_foreach_xpath_result(xmlNode *xml, const char *xpath,
                               void (*helper)(xmlNode*, void*), void *user_data);
-xmlNode *expand_idref(xmlNode * input, xmlNode * top);
 
 void freeXpathObject(xmlXPathObjectPtr xpathObj);
 xmlNode *getXpathResult(xmlXPathObjectPtr xpathObj, int index);
@@ -100,15 +78,12 @@ int xml_apply_patchset(xmlNode *xml, xmlNode *patchset, bool check_version);
 
 void patchset_process_digest(xmlNode *patch, xmlNode *source, xmlNode *target, bool with_digest);
 
-void crm_xml_sanitize_id(char *id);
-void crm_xml_set_id(xmlNode *xml, const char *format, ...) G_GNUC_PRINTF(2, 3);
+#ifdef __cplusplus
+}
+#endif
 
 #if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
 #include <crm/common/xml_compat.h>
 #endif
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
