@@ -7,8 +7,8 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PCMK__XML_INTERNAL__H
-#define PCMK__XML_INTERNAL__H
+#ifndef PCMK__CRM_COMMON_XML_INTERNAL__H
+#define PCMK__CRM_COMMON_XML_INTERNAL__H
 
 /*
  * Internal-only wrappers for and extensions to libxml2 (libxslt)
@@ -17,15 +17,24 @@
 #include <stdlib.h>
 #include <stdint.h>   // uint32_t
 #include <stdio.h>
-#include <string.h>
 
 #include <crm/crm.h>  /* transitively imports qblog.h */
 #include <crm/common/output_internal.h>
+#include <crm/common/xml_names.h>             // PCMK_XA_ID, PCMK_XE_CLONE
+
+// This file is a wrapper for other xml_*_internal.h headers
+#include <crm/common/xml_comment_internal.h>
+#include <crm/common/xml_element_internal.h>
+#include <crm/common/xml_idref_internal.h>
 #include <crm/common/xml_io_internal.h>
-#include <crm/common/xml_names_internal.h>    // PCMK__XE_PROMOTABLE_LEGACY
+#include <crm/common/xml_names_internal.h>
 
 #include <libxml/relaxng.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*!
  * \brief Base for directing lib{xml2,xslt} log into standard libqb backend
  *
@@ -151,20 +160,11 @@ enum pcmk__xml_fmt_options {
     // @COMPAT Can we start including text nodes unconditionally?
     //! Include XML text nodes
     pcmk__xml_fmt_text       = (1 << 6),
-
-    // @COMPAT Remove when v1 patchsets are removed
-    //! Log a created XML subtree
-    pcmk__xml_fmt_diff_plus  = (1 << 7),
-
-    // @COMPAT Remove when v1 patchsets are removed
-    //! Log a removed XML subtree
-    pcmk__xml_fmt_diff_minus = (1 << 8),
-
-    // @COMPAT Remove when v1 patchsets are removed
-    //! Log a minimal version of an XML diff (only showing the changes)
-    pcmk__xml_fmt_diff_short = (1 << 9),
 };
 
+void pcmk__xml_init(void);
+void pcmk__xml_cleanup(void);
+
 int pcmk__xml_show(pcmk__output_t *out, const char *prefix, const xmlNode *data,
                    int depth, uint32_t options);
 int pcmk__xml_show_changes(pcmk__output_t *out, const xmlNode *xml);
@@ -193,16 +193,6 @@ int pcmk__xml_show_changes(pcmk__output_t *out, const xmlNode *xml);
 #define PCMK__XP_REMOTE_NODE_STATUS                                 \
     "//" PCMK_XE_CIB "//" PCMK_XE_STATUS "//" PCMK__XE_NODE_STATE   \
     "[@" PCMK_XA_REMOTE_NODE "='" PCMK_VALUE_TRUE "']"
-/*!
- * \internal
- * \brief Serialize XML (using libxml) into provided descriptor
- *
- * \param[in] fd  File descriptor to (piece-wise) write to
- * \param[in] cur XML subtree to proceed
- * 
- * \return a standard Pacemaker return code
- */
-int pcmk__xml2fd(int fd, xmlNode *cur);
 
 enum pcmk__xml_artefact_ns {
     pcmk__xml_artefact_ns_legacy_rng = 1,
@@ -212,20 +202,6 @@ enum pcmk__xml_artefact_ns {
 };
 
 void pcmk__strip_xml_text(xmlNode *xml);
-const char *pcmk__xe_add_last_written(xmlNode *xe);
-
-xmlNode *pcmk__xe_first_child(const xmlNode *parent, const char *node_name,
-                              const char *attr_n, const char *attr_v);
-
-
-void pcmk__xe_remove_attr(xmlNode *element, const char *name);
-bool pcmk__xe_remove_attr_cb(xmlNode *xml, void *user_data);
-void pcmk__xe_remove_matching_attrs(xmlNode *element,
-                                    bool (*match)(xmlAttrPtr, void *),
-                                    void *user_data);
-int pcmk__xe_delete_match(xmlNode *xml, xmlNode *search);
-int pcmk__xe_replace_match(xmlNode *xml, xmlNode *replace);
-int pcmk__xe_update_match(xmlNode *xml, xmlNode *update, uint32_t flags);
 
 GString *pcmk__element_xpath(const xmlNode *xml);
 
@@ -334,36 +310,6 @@ pcmk__xml_artefact_root(enum pcmk__xml_artefact_ns ns);
 char *pcmk__xml_artefact_path(enum pcmk__xml_artefact_ns ns,
                               const char *filespec);
 
-/*!
- * \internal
- * \brief Retrieve the value of the \c PCMK_XA_ID XML attribute
- *
- * \param[in] xml  XML element to check
- *
- * \return Value of the \c PCMK_XA_ID attribute (may be \c NULL)
- */
-static inline const char *
-pcmk__xe_id(const xmlNode *xml)
-{
-    return crm_element_value(xml, PCMK_XA_ID);
-}
-
-/*!
- * \internal
- * \brief Check whether an XML element is of a particular type
- *
- * \param[in] xml   XML element to compare
- * \param[in] name  XML element name to compare
- *
- * \return \c true if \p xml is of type \p name, otherwise \c false
- */
-static inline bool
-pcmk__xe_is(const xmlNode *xml, const char *name)
-{
-    return (xml != NULL) && (xml->name != NULL) && (name != NULL)
-           && (strcmp((const char *) xml->name, name) == 0);
-}
-
 /*!
  * \internal
  * \brief Return first non-text child node of an XML node
@@ -402,31 +348,9 @@ pcmk__xml_next(const xmlNode *child)
     return next;
 }
 
-/*!
- * \internal
- * \brief Return next non-text sibling element of an XML element
- *
- * \param[in] child  XML element to check
- *
- * \return Next sibling element of \p child (or NULL if none)
- */
-static inline xmlNode *
-pcmk__xe_next(const xmlNode *child)
-{
-    xmlNode *next = child? child->next : NULL;
-
-    while (next && (next->type != XML_ELEMENT_NODE)) {
-        next = next->next;
-    }
-    return next;
-}
-
-xmlNode *pcmk__xe_create(xmlNode *parent, const char *name);
+void pcmk__xml_free(xmlNode *xml);
+void pcmk__xml_free_doc(xmlDoc *doc);
 xmlNode *pcmk__xml_copy(xmlNode *parent, xmlNode *src);
-xmlNode *pcmk__xe_next_same(const xmlNode *node);
-
-void pcmk__xe_set_content(xmlNode *node, const char *format, ...)
-    G_GNUC_PRINTF(2, 3);
 
 /*!
  * \internal
@@ -447,46 +371,7 @@ enum pcmk__xa_flags {
     pcmk__xaf_score_update  = (1U << 1),
 };
 
-int pcmk__xe_copy_attrs(xmlNode *target, const xmlNode *src, uint32_t flags);
-
-/*!
- * \internal
- * \brief Like pcmk__xe_set_props, but takes a va_list instead of
- *        arguments directly.
- *
- * \param[in,out] node   XML to add attributes to
- * \param[in]     pairs  NULL-terminated list of name/value pairs to add
- */
-void
-pcmk__xe_set_propv(xmlNodePtr node, va_list pairs);
-
-/*!
- * \internal
- * \brief Add a NULL-terminated list of name/value pairs to the given
- *        XML node as properties.
- *
- * \param[in,out] node XML node to add properties to
- * \param[in]     ...  NULL-terminated list of name/value pairs
- *
- * \note A NULL name terminates the arguments; a NULL value will be skipped.
- */
-void
-pcmk__xe_set_props(xmlNodePtr node, ...)
-G_GNUC_NULL_TERMINATED;
-
-/*!
- * \internal
- * \brief Get first attribute of an XML element
- *
- * \param[in] xe  XML element to check
- *
- * \return First attribute of \p xe (or NULL if \p xe is NULL or has none)
- */
-static inline xmlAttr *
-pcmk__xe_first_attr(const xmlNode *xe)
-{
-    return (xe == NULL)? NULL : xe->properties;
-}
+void pcmk__xml_sanitize_id(char *id);
 
 /*!
  * \internal
@@ -540,31 +425,6 @@ enum xml_private_flags {
 
 void pcmk__set_xml_doc_flag(xmlNode *xml, enum xml_private_flags flag);
 
-/*!
- * \internal
- * \brief Iterate over child elements of \p xml
- *
- * This function iterates over the children of \p xml, performing the
- * callback function \p handler on each node.  If the callback returns
- * a value other than pcmk_rc_ok, the iteration stops and the value is
- * returned.  It is therefore possible that not all children will be
- * visited.
- *
- * \param[in,out] xml                 The starting XML node.  Can be NULL.
- * \param[in]     child_element_name  The name that the node must match in order
- *                                    for \p handler to be run.  If NULL, all
- *                                    child elements will match.
- * \param[in]     handler             The callback function.
- * \param[in,out] userdata            User data to pass to the callback function.
- *                                    Can be NULL.
- *
- * \return Standard Pacemaker return code
- */
-int
-pcmk__xe_foreach_child(xmlNode *xml, const char *child_element_name,
-                       int (*handler)(xmlNode *xml, void *userdata),
-                       void *userdata);
-
 bool pcmk__xml_tree_foreach(xmlNode *xml, bool (*fn)(xmlNode *, void *),
                             void *user_data);
 
@@ -575,20 +435,22 @@ pcmk__xml_attr_value(const xmlAttr *attr)
            : (const char *) attr->children->content;
 }
 
-// @COMPAT Remove when v1 patchsets are removed
-xmlNode *pcmk__diff_v1_xml_object(xmlNode *left, xmlNode *right, bool suppress);
+/*!
+ * \internal
+ * \brief Check whether a given CIB element was modified in a CIB patchset
+ *
+ * \param[in] patchset  CIB XML patchset
+ * \param[in] element   XML tag of CIB element to check (\c NULL is equivalent
+ *                      to \c PCMK_XE_CIB). Supported values include any CIB
+ *                      element supported by \c pcmk__cib_abs_xpath_for().
+ *
+ * \return \c true if \p element was modified, or \c false otherwise
+ */
+bool pcmk__cib_element_in_patchset(const xmlNode *patchset,
+                                   const char *element);
 
-// @COMPAT Drop when PCMK__XE_PROMOTABLE_LEGACY is removed
-static inline const char *
-pcmk__map_element_name(const xmlNode *xml)
-{
-    if (xml == NULL) {
-        return NULL;
-    } else if (pcmk__xe_is(xml, PCMK__XE_PROMOTABLE_LEGACY)) {
-        return PCMK_XE_CLONE;
-    } else {
-        return (const char *) xml->name;
-    }
+#ifdef __cplusplus
 }
+#endif
 
-#endif // PCMK__XML_INTERNAL__H
+#endif // PCMK__CRM_COMMON_XML_INTERNAL__H
