@@ -7,14 +7,19 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef STONITH_NG_INTERNAL__H
-#  define STONITH_NG_INTERNAL__H
+#ifndef PCMK__CRM_FENCING_INTERNAL__H
+#define PCMK__CRM_FENCING_INTERNAL__H
 
-#  include <glib.h>
-#  include <crm/common/ipc.h>
-#  include <crm/common/xml.h>
-#  include <crm/common/output_internal.h>
-#  include <crm/stonith-ng.h>
+#include <glib.h>
+#include <crm/common/ipc.h>
+#include <crm/common/xml.h>
+#include <crm/common/output_internal.h>
+#include <crm/common/results_internal.h>
+#include <crm/stonith-ng.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 enum st_device_flags {
     st_device_supports_none             = (0 << 0),
@@ -83,7 +88,7 @@ xmlNode *create_level_registration_xml(const char *node, const char *pattern,
                                        const stonith_key_value_t *device_list);
 
 xmlNode *create_device_registration_xml(const char *id,
-                                        enum stonith_namespace namespace,
+                                        enum stonith_namespace standard,
                                         const char *agent,
                                         const stonith_key_value_t *params,
                                         const char *rsc_provides);
@@ -106,7 +111,6 @@ void stonith__device_parameter_flags(uint32_t *device_flags,
  */
 #  define ST__LEVEL_COUNT 10
 
-// @COMPAT Deprecated since 1.1.17 (and see T773 to drop it)
 #  define STONITH_ATTR_ACTION_OP   "action"
 
 #  define STONITH_OP_EXEC        "st_execute"
@@ -167,4 +171,29 @@ stonith__op_state_pending(enum op_state state)
 gboolean stonith__watchdog_fencing_enabled_for_node(const char *node);
 gboolean stonith__watchdog_fencing_enabled_for_node_api(stonith_t *st, const char *node);
 
+/*!
+ * \internal
+ * \brief Validate a fencing configuration
+ *
+ * \param[in,out] st            Fencer connection to use
+ * \param[in]     call_options  Group of enum stonith_call_options
+ * \param[in]     rsc_id        Resource to validate
+ * \param[in]     namespace_s   Type of fence agent to search for
+ * \param[in]     agent         Fence agent to validate
+ * \param[in,out] params        Fence device configuration parameters
+ * \param[in]     timeout_sec   How long to wait for operation to complete
+ * \param[in,out] output        If non-NULL, where to store any agent output
+ * \param[in,out] error_output  If non-NULL, where to store agent error output
+ *
+ * \return Standard Pacemaker return code
+ */
+int stonith__validate(stonith_t *st, int call_options, const char *rsc_id,
+                      const char *namespace_s, const char *agent,
+                      GHashTable *params, int timeout_sec, char **output,
+                      char **error_output);
+
+#ifdef __cplusplus
+}
 #endif
+
+#endif // PCMK__CRM_FENCING_INTERNAL__H
