@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2023 the Pacemaker project contributors
+ * Copyright 2015-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,8 +7,8 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef LRMD_INTERNAL__H
-#define LRMD_INTERNAL__H
+#ifndef PCMK__CRM_LRMD_INTERNAL__H
+#define PCMK__CRM_LRMD_INTERNAL__H
 
 #include <stdint.h>                     // uint32_t
 #include <glib.h>                       // GList, GHashTable, gpointer
@@ -17,8 +17,13 @@
 #include <crm/common/mainloop.h>        // mainloop_io_t, ipc_client_callbacks
 #include <crm/common/output_internal.h> // pcmk__output_t
 #include <crm/common/remote_internal.h> // pcmk__remote_t
+#include <crm/common/results_internal.h> // pcmk__action_result_t
 #include <crm/lrmd.h>           // lrmd_t, lrmd_event_data_t, lrmd_rsc_info_t
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int lrmd__new(lrmd_t **api, const char *nodename, const char *server, int port);
 
 int lrmd_send_attribute_alert(lrmd_t *lrmd, const GList *alert_list,
@@ -85,8 +90,10 @@ void remote_proxy_relay_response(remote_proxy_t *proxy, xmlNode *msg,
 
 void lrmd__register_messages(pcmk__output_t *out);
 
-#ifdef HAVE_GNUTLS_GNUTLS_H
 int lrmd__init_remote_key(gnutls_datum_t *key);
+
+#ifdef __cplusplus
+}
 #endif
 
-#endif
+#endif // PCMK__CRM_LRMD_INTERNAL__H
