@@ -7,19 +7,23 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PE_INTERNAL__H
-#  define PE_INTERNAL__H
+#ifndef PCMK__CRM_PENGINE_INTERNAL__H
+#define PCMK__CRM_PENGINE_INTERNAL__H
 
-#  include <stdbool.h>
-#  include <stdint.h>
-#  include <string.h>
-#  include <crm/common/xml.h>
-#  include <crm/pengine/status.h>
-#  include <crm/pengine/remote_internal.h>
-#  include <crm/common/internal.h>
-#  include <crm/common/options_internal.h>
-#  include <crm/common/output_internal.h>
-#  include <crm/common/scheduler_internal.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <string.h>
+#include <crm/common/xml.h>
+#include <crm/pengine/status.h>
+#include <crm/pengine/remote_internal.h>
+#include <crm/common/internal.h>
+#include <crm/common/options_internal.h>
+#include <crm/common/output_internal.h>
+#include <crm/common/scheduler_internal.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 const char *pe__resource_description(const pcmk_resource_t *rsc,
                                      uint32_t show_opts);
@@ -57,7 +61,7 @@ bool pe_can_fence(const pcmk_scheduler_t *scheduler, const pcmk_node_t *node);
 char *native_parameter(pcmk_resource_t *rsc, pcmk_node_t *node, gboolean create,
                        const char *name, pcmk_scheduler_t *scheduler);
 pcmk_node_t *native_location(const pcmk_resource_t *rsc, GList **list,
-                             int current);
+                             uint32_t target);
 void native_add_running(pcmk_resource_t *rsc, pcmk_node_t *node,
                         pcmk_scheduler_t *scheduler, gboolean failed);
 
@@ -74,22 +78,6 @@ gboolean group_active(pcmk_resource_t *rsc, gboolean all);
 gboolean clone_active(pcmk_resource_t *rsc, gboolean all);
 gboolean pe__bundle_active(pcmk_resource_t *rsc, gboolean all);
 
-//! \deprecated This function will be removed in a future release
-void native_print(pcmk_resource_t *rsc, const char *pre_text, long options,
-                  void *print_data);
-
-//! \deprecated This function will be removed in a future release
-void group_print(pcmk_resource_t *rsc, const char *pre_text, long options,
-                 void *print_data);
-
-//! \deprecated This function will be removed in a future release
-void clone_print(pcmk_resource_t *rsc, const char *pre_text, long options,
-                 void *print_data);
-
-//! \deprecated This function will be removed in a future release
-void pe__print_bundle(pcmk_resource_t *rsc, const char *pre_text, long options,
-                      void *print_data);
-
 gchar *pcmk__native_output_string(const pcmk_resource_t *rsc, const char *name,
                                   const pcmk_node_t *node, uint32_t show_opts,
                                   const char *target_role, bool show_nodes);
@@ -165,7 +153,7 @@ pcmk_node_t *pe__find_active_requires(const pcmk_resource_t *rsc,
 GHashTable *pe__node_list2table(const GList *list);
 
 pcmk_action_t *get_pseudo_op(const char *name, pcmk_scheduler_t *scheduler);
-gboolean order_actions(pcmk_action_t *lh_action, pcmk_action_t *rh_action,
+gboolean order_actions(pcmk_action_t *first, pcmk_action_t *then,
                        uint32_t flags);
 
 void pe__show_node_scores_as(const char *file, const char *function,
@@ -188,54 +176,54 @@ xmlNode *pcmk__find_action_config(const pcmk_resource_t *rsc,
                                   const char *action_name, guint interval_ms,
                                   bool include_disabled);
 
-enum rsc_start_requirement pcmk__action_requires(const pcmk_resource_t *rsc,
-                                                 const char *action_name);
+enum pcmk__requires pcmk__action_requires(const pcmk_resource_t *rsc,
+                                          const char *action_name);
 
-enum action_fail_response pcmk__parse_on_fail(const pcmk_resource_t *rsc,
-                                              const char *action_name,
-                                              guint interval_ms,
-                                              const char *value);
+enum pcmk__on_fail pcmk__parse_on_fail(const pcmk_resource_t *rsc,
+                                       const char *action_name,
+                                       guint interval_ms, const char *value);
 
 enum rsc_role_e pcmk__role_after_failure(const pcmk_resource_t *rsc,
                                          const char *action_name,
-                                         enum action_fail_response on_fail,
+                                         enum pcmk__on_fail on_fail,
                                          GHashTable *meta);
 
 pcmk_action_t *custom_action(pcmk_resource_t *rsc, char *key, const char *task,
                              const pcmk_node_t *on_node, gboolean optional,
                              pcmk_scheduler_t *scheduler);
 
-#  define delete_key(rsc) pcmk__op_key(rsc->id, PCMK_ACTION_DELETE, 0)
-#  define delete_action(rsc, node, optional) custom_action(		\
-		rsc, delete_key(rsc), PCMK_ACTION_DELETE, node, \
-		optional, rsc->cluster);
+#define delete_key(rsc)  pcmk__op_key((rsc)->id, PCMK_ACTION_DELETE, 0)
+#define stop_key(rsc)    pcmk__op_key((rsc)->id, PCMK_ACTION_STOP, 0)
+#define reload_key(rsc)  pcmk__op_key((rsc)->id, PCMK_ACTION_RELOAD_AGENT, 0)
+#define start_key(rsc)   pcmk__op_key((rsc)->id, PCMK_ACTION_START, 0)
+#define promote_key(rsc) pcmk__op_key((rsc)->id, PCMK_ACTION_PROMOTE, 0)
+#define demote_key(rsc)  pcmk__op_key((rsc)->id, PCMK_ACTION_DEMOTE, 0)
 
-#  define stop_key(rsc) pcmk__op_key(rsc->id, PCMK_ACTION_STOP, 0)
-#  define stop_action(rsc, node, optional) custom_action(			\
-		rsc, stop_key(rsc), PCMK_ACTION_STOP, node,		\
-		optional, rsc->cluster);
+#define delete_action(rsc, node, optional)                          \
+    custom_action((rsc), delete_key(rsc), PCMK_ACTION_DELETE,       \
+                  (node), (optional), (rsc)->priv->scheduler)
 
-#  define reload_key(rsc) pcmk__op_key(rsc->id, PCMK_ACTION_RELOAD_AGENT, 0)
-#  define start_key(rsc) pcmk__op_key(rsc->id, PCMK_ACTION_START, 0)
-#  define start_action(rsc, node, optional) custom_action(		\
-		rsc, start_key(rsc), PCMK_ACTION_START, node,           \
-		optional, rsc->cluster)
+#define stop_action(rsc, node, optional)                            \
+    custom_action((rsc), stop_key(rsc), PCMK_ACTION_STOP,           \
+                  (node), (optional), (rsc)->priv->scheduler)
 
-#  define promote_key(rsc) pcmk__op_key(rsc->id, PCMK_ACTION_PROMOTE, 0)
-#  define promote_action(rsc, node, optional) custom_action(		\
-		rsc, promote_key(rsc), PCMK_ACTION_PROMOTE, node,	\
-		optional, rsc->cluster)
+#define start_action(rsc, node, optional)                           \
+    custom_action((rsc), start_key(rsc), PCMK_ACTION_START,         \
+                  (node), (optional), (rsc)->priv->scheduler)
 
-#  define demote_key(rsc) pcmk__op_key(rsc->id, PCMK_ACTION_DEMOTE, 0)
-#  define demote_action(rsc, node, optional) custom_action(		\
-		rsc, demote_key(rsc), PCMK_ACTION_DEMOTE, node, \
-		optional, rsc->cluster)
+#define promote_action(rsc, node, optional)                         \
+    custom_action((rsc), promote_key(rsc), PCMK_ACTION_PROMOTE,     \
+                  (node), (optional), (rsc)->priv->scheduler)
+
+#define demote_action(rsc, node, optional)                          \
+    custom_action((rsc), demote_key(rsc), PCMK_ACTION_DEMOTE,       \
+                  (node), (optional), (rsc)->priv->scheduler)
 
 pcmk_action_t *find_first_action(const GList *input, const char *uuid,
                                  const char *task, const pcmk_node_t *on_node);
 
-enum action_tasks get_complex_task(const pcmk_resource_t *rsc,
-                                   const char *name);
+enum pcmk__action_type get_complex_task(const pcmk_resource_t *rsc,
+                                        const char *name);
 
 GList *find_actions(GList *input, const char *key, const pcmk_node_t *on_node);
 GList *find_actions_exact(GList *input, const char *key,
@@ -248,18 +236,14 @@ extern void pe_free_action(pcmk_action_t *action);
 void resource_location(pcmk_resource_t *rsc, const pcmk_node_t *node, int score,
                        const char *tag, pcmk_scheduler_t *scheduler);
 
-extern int pe__is_newer_op(const xmlNode *xml_a, const xmlNode *xml_b,
-                           bool same_node_default);
+int pe__is_newer_op(const xmlNode *xml_a, const xmlNode *xml_b);
 extern gint sort_op_by_callid(gconstpointer a, gconstpointer b);
 gboolean get_target_role(const pcmk_resource_t *rsc, enum rsc_role_e *role);
 void pe__set_next_role(pcmk_resource_t *rsc, enum rsc_role_e role,
                        const char *why);
 
-pcmk_resource_t *find_clone_instance(const pcmk_resource_t *rsc,
-                                     const char *sub_id);
-
 extern void destroy_ticket(gpointer data);
-pcmk_ticket_t *ticket_new(const char *ticket_id, pcmk_scheduler_t *scheduler);
+pcmk__ticket_t *ticket_new(const char *ticket_id, pcmk_scheduler_t *scheduler);
 
 // Resources for manipulating resource names
 const char *pe_base_name_end(const char *id);
@@ -305,7 +289,8 @@ void trigger_unfencing(pcmk_resource_t *rsc, pcmk_node_t *node,
                        const char *reason, pcmk_action_t *dependency,
                        pcmk_scheduler_t *scheduler);
 
-char *pe__action2reason(const pcmk_action_t *action, enum pe_action_flags flag);
+char *pe__action2reason(const pcmk_action_t *action,
+                        enum pcmk__action_flags flag);
 void pe_action_set_reason(pcmk_action_t *action, const char *reason,
                           bool overwrite);
 void pe__add_action_expected_result(pcmk_action_t *action, int expected_result);
@@ -315,21 +300,13 @@ void pe__clear_resource_flags_recursive(pcmk_resource_t *rsc, uint64_t flags);
 void pe__clear_resource_flags_on_all(pcmk_scheduler_t *scheduler,
                                      uint64_t flag);
 
-gboolean add_tag_ref(GHashTable * tags, const char * tag_name,  const char * obj_ref);
-
-//! \deprecated This function will be removed in a future release
-void print_rscs_brief(GList *rsc_list, const char * pre_text, long options,
-                      void * print_data, gboolean print_all);
 int pe__rscs_brief_output(pcmk__output_t *out, GList *rsc_list, unsigned int options);
 void pe_fence_node(pcmk_scheduler_t *scheduler, pcmk_node_t *node,
                    const char *reason, bool priority_delay);
 
 pcmk_node_t *pe_create_node(const char *id, const char *uname, const char *type,
-                            const char *score, pcmk_scheduler_t *scheduler);
+                            int score, pcmk_scheduler_t *scheduler);
 
-//! \deprecated This function will be removed in a future release
-void common_print(pcmk_resource_t *rsc, const char *pre_text, const char *name,
-                  const pcmk_node_t *node, long options, void *print_data);
 int pe__common_output_text(pcmk__output_t *out, const pcmk_resource_t *rsc,
                            const char *name, const pcmk_node_t *node,
                            unsigned int options);
@@ -383,7 +360,6 @@ void pe__register_messages(pcmk__output_t *out);
 void pe__unpack_dataset_nvpairs(const xmlNode *xml_obj, const char *set_name,
                                 const pe_rule_eval_data_t *rule_data,
                                 GHashTable *hash, const char *always_first,
-                                gboolean overwrite,
                                 pcmk_scheduler_t *scheduler);
 
 bool pe__resource_is_disabled(const pcmk_resource_t *rsc);
@@ -424,18 +400,14 @@ int pe__node_health(pcmk_node_t *node);
 static inline enum pcmk__health_strategy
 pe__health_strategy(pcmk_scheduler_t *scheduler)
 {
-    const char *strategy = pcmk__cluster_option(scheduler->config_hash,
+    const char *strategy = pcmk__cluster_option(scheduler->priv->options,
                                                 PCMK_OPT_NODE_HEALTH_STRATEGY);
 
     return pcmk__parse_health_strategy(strategy);
 }
 
-static inline int
-pe__health_score(const char *option, pcmk_scheduler_t *scheduler)
-{
-    const char *value = pcmk__cluster_option(scheduler->config_hash, option);
-
-    return char2score(value);
+#ifdef __cplusplus
 }
-
 #endif
+
+#endif // PCMK__CRM_PENGINE_INTERNAL__H
