@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2022 the Pacemaker project contributors
+ * Copyright 2010-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,8 +7,10 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PCMK__SERVICES_INTERNAL__H
-#  define PCMK__SERVICES_INTERNAL__H
+#ifndef PCMK__CRM_SERVICES_INTERNAL__H
+#define PCMK__CRM_SERVICES_INTERNAL__H
+
+#include <crm/services.h>       // svc_action_t
 
 #ifdef __cplusplus
 extern "C" {
@@ -43,7 +45,7 @@ svc_action_t *services__create_resource_action(const char *name,
                                                int timeout, GHashTable *params,
                                                enum svc_action_flags flags);
 
-const char *services__exit_reason(svc_action_t *action);
+const char *services__exit_reason(const svc_action_t *action);
 char *services__grab_stdout(svc_action_t *action);
 char *services__grab_stderr(svc_action_t *action);
 
@@ -55,8 +57,8 @@ void services__format_result(svc_action_t *action, int agent_status,
                              enum pcmk_exec_status exec_status,
                              const char *format, ...) G_GNUC_PRINTF(4, 5);
 
-#  ifdef __cplusplus
+#ifdef __cplusplus
 }
-#  endif
+#endif
 
-#endif                          /* PCMK__SERVICES_INTERNAL__H */
+#endif // PCMK__CRM_SERVICES_INTERNAL__H
