@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2022 the Pacemaker project contributors
+ * Copyright 2019-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -7,10 +7,14 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 #ifndef PCMK__PCMKI_PCMKI_FENCE__H
-#  define PCMK__PCMKI_PCMKI_FENCE__H
+#define PCMK__PCMKI_PCMKI_FENCE__H
 
-#  include <crm/stonith-ng.h>
-#  include <crm/common/output_internal.h>
+#include <crm/stonith-ng.h>
+#include <crm/common/output_internal.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /*!
  * \brief Control how much of the fencing history is output.
@@ -44,7 +48,7 @@ enum pcmk__fence_history {
  * \return Standard Pacemaker return code
  * \note If \p reason is not NULL, the caller is responsible for freeing its
  *       returned value.
- * \todo delay is eventually used with g_timeout_add() and should be guint
+ * \todo delay is eventually used with pcmk__create_timer() and should be guint
  */
 int pcmk__request_fencing(stonith_t *st, const char *target, const char *action,
                           const char *name, unsigned int timeout,
@@ -184,13 +188,12 @@ int pcmk__fence_registered(pcmk__output_t *out, stonith_t *st,
  * \param[in,out] st          A connection to the fencer API
  * \param[in]     target      The object to register a fencing level for
  * \param[in]     fence_level Index number of level to add
- * \param[in]     devices     Devices to use in level
+ * \param[in]     devices     Devices to use in level as a list of char *
  *
  * \return Standard Pacemaker return code
  */
 int pcmk__fence_register_level(stonith_t *st, const char *target,
-                               int fence_level,
-                               const stonith_key_value_t *devices);
+                               int fence_level, GList *devices);
 
 /*!
  * \brief Unregister a fencing level for specific node, node regex, or attribute
@@ -232,8 +235,7 @@ int pcmk__fence_unregister_level(stonith_t *st, const char *target,
  * \return Standard Pacemaker return code
  */
 int pcmk__fence_validate(pcmk__output_t *out, stonith_t *st, const char *agent,
-                         const char *id, const stonith_key_value_t *params,
-                         unsigned int timeout);
+                         const char *id, GHashTable *params, unsigned int timeout);
 
 /*!
  * \brief Fetch fencing history, optionally reducing it
@@ -247,4 +249,9 @@ int pcmk__fence_validate(pcmk__output_t *out, stonith_t *st, const char *agent,
 int
 pcmk__get_fencing_history(stonith_t *st, stonith_history_t **stonith_history,
                           enum pcmk__fence_history fence_history);
+
+#ifdef __cplusplus
+}
 #endif
+
+#endif // PCMK__PCMKI_PCMKI_FENCE__H
