@@ -12,8 +12,16 @@
 
 #include <crm/common/output_internal.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int pcmk__list_cluster_options(pcmk__output_t *out, bool all);
 int pcmk__list_fencing_params(pcmk__output_t *out, bool all);
 int pcmk__list_primitive_meta(pcmk__output_t *out, bool all);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif  // PCMK__PCMKI_PCMKI_OPTIONS__H
