@@ -26,29 +26,6 @@
 
 static GHashTable *cib_op_callback_table = NULL;
 
-#define op_common(cib) do {                                             \
-        if(cib == NULL) {                                               \
-            return -EINVAL;						\
-        } else if(cib->delegate_fn == NULL) {                           \
-            return -EPROTONOSUPPORT;                                    \
-        }                                                               \
-    } while(0)
-
-static int
-cib_client_set_op_callback(cib_t *cib,
-                           void (*callback) (const xmlNode * msg, int call_id,
-                                             int rc, xmlNode * output))
-{
-    if (callback == NULL) {
-        crm_info("Un-Setting operation callback");
-
-    } else {
-        crm_trace("Setting operation callback");
-    }
-    cib->op_callback = callback;
-    return pcmk_ok;
-}
-
 static gint
 ciblib_GCompareFunc(gconstpointer a, gconstpointer b)
 {
@@ -224,9 +201,9 @@ cib_client_register_callback_full(cib_t *cib, int call_id, int timeout,
         async_timer->cib = cib;
         async_timer->call_id = call_id;
         async_timer->timeout = timeout * 1000;
-        async_timer->ref = g_timeout_add(async_timer->timeout,
-                                         cib_async_timeout_handler,
-                                         async_timer);
+        async_timer->ref = pcmk__create_timer(async_timer->timeout,
+                                              cib_async_timeout_handler,
+                                              async_timer);
     }
 
     crm_trace("Adding callback %s for call %d", callback_name, call_id);
@@ -250,7 +227,6 @@ cib_client_register_callback(cib_t *cib, int call_id, int timeout,
 static int
 cib_client_noop(cib_t * cib, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_NOOP, NULL, NULL, NULL, NULL,
                            call_options, cib->user);
 }
@@ -258,7 +234,6 @@ cib_client_noop(cib_t * cib, int call_options)
 static int
 cib_client_ping(cib_t * cib, xmlNode ** output_data, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, CRM_OP_PING, NULL, NULL, NULL, output_data,
                            call_options, cib->user);
 }
@@ -273,46 +248,27 @@ static int
 cib_client_query_from(cib_t * cib, const char *host, const char *section,
                       xmlNode ** output_data, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_QUERY, host, section, NULL,
                            output_data, call_options, cib->user);
 }
 
-static int
-is_primary(cib_t *cib)
-{
-    op_common(cib);
-    return cib_internal_op(cib, PCMK__CIB_REQUEST_IS_PRIMARY, NULL, NULL, NULL,
-                           NULL, cib_scope_local|cib_sync_call, cib->user);
-}
-
 static int
 set_secondary(cib_t *cib, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_SECONDARY, NULL, NULL, NULL,
                            NULL, call_options, cib->user);
 }
 
-static int
-set_all_secondary(cib_t * cib, int call_options)
-{
-    return -EPROTONOSUPPORT;
-}
-
 static int
 set_primary(cib_t *cib, int call_options)
 {
-    op_common(cib);
-    crm_trace("Adding cib_scope_local to options");
     return cib_internal_op(cib, PCMK__CIB_REQUEST_PRIMARY, NULL, NULL, NULL,
-                           NULL, call_options|cib_scope_local, cib->user);
+                           NULL, call_options, cib->user);
 }
 
 static int
 cib_client_bump_epoch(cib_t * cib, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_BUMP, NULL, NULL, NULL, NULL,
                            call_options, cib->user);
 }
@@ -320,7 +276,6 @@ cib_client_bump_epoch(cib_t * cib, int call_options)
 static int
 cib_client_upgrade(cib_t * cib, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_UPGRADE, NULL, NULL, NULL,
                            NULL, call_options, cib->user);
 }
@@ -334,7 +289,6 @@ cib_client_sync(cib_t * cib, const char *section, int call_options)
 static int
 cib_client_sync_from(cib_t * cib, const char *host, const char *section, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_SYNC_TO_ALL, host, section,
                            NULL, NULL, call_options, cib->user);
 }
@@ -342,7 +296,6 @@ cib_client_sync_from(cib_t * cib, const char *host, const char *section, int cal
 static int
 cib_client_create(cib_t * cib, const char *section, xmlNode * data, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_CREATE, NULL, section, data,
                            NULL, call_options, cib->user);
 }
@@ -350,7 +303,6 @@ cib_client_create(cib_t * cib, const char *section, xmlNode * data, int call_opt
 static int
 cib_client_modify(cib_t * cib, const char *section, xmlNode * data, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_MODIFY, NULL, section, data,
                            NULL, call_options, cib->user);
 }
@@ -358,7 +310,6 @@ cib_client_modify(cib_t * cib, const char *section, xmlNode * data, int call_opt
 static int
 cib_client_replace(cib_t * cib, const char *section, xmlNode * data, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_REPLACE, NULL, section, data,
                            NULL, call_options, cib->user);
 }
@@ -366,23 +317,13 @@ cib_client_replace(cib_t * cib, const char *section, xmlNode * data, int call_op
 static int
 cib_client_delete(cib_t * cib, const char *section, xmlNode * data, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_DELETE, NULL, section, data,
                            NULL, call_options, cib->user);
 }
 
-static int
-cib_client_delete_absolute(cib_t * cib, const char *section, xmlNode * data, int call_options)
-{
-    op_common(cib);
-    return cib_internal_op(cib, PCMK__CIB_REQUEST_ABS_DELETE, NULL, section,
-                           data, NULL, call_options, cib->user);
-}
-
 static int
 cib_client_erase(cib_t * cib, xmlNode ** output_data, int call_options)
 {
-    op_common(cib);
     return cib_internal_op(cib, PCMK__CIB_REQUEST_ERASE, NULL, NULL, NULL,
                            output_data, call_options, cib->user);
 }
@@ -392,7 +333,9 @@ cib_client_init_transaction(cib_t *cib)
 {
     int rc = pcmk_rc_ok;
 
-    op_common(cib);
+    if (cib == NULL) {
+        return -EINVAL;
+    }
 
     if (cib->transaction != NULL) {
         // A client can have at most one transaction at a time
@@ -419,7 +362,10 @@ cib_client_end_transaction(cib_t *cib, bool commit, int call_options)
     const char *client_id = NULL;
     int rc = pcmk_ok;
 
-    op_common(cib);
+    if (cib == NULL) {
+        return -EINVAL;
+    }
+
     cib->cmds->client_id(cib, NULL, &client_id);
     client_id = pcmk__s(client_id, "(unidentified)");
 
@@ -442,7 +388,7 @@ cib_client_end_transaction(cib_t *cib, bool commit, int call_options)
             crm_trace("No transaction found for CIB client %s", client_id);
         }
     }
-    free_xml(cib->transaction);
+    pcmk__xml_free(cib->transaction);
     cib->transaction = NULL;
     return rc;
 }
@@ -458,7 +404,7 @@ cib_client_fetch_schemas(cib_t *cib, xmlNode **output_data, const char *after_ve
 
     rc = cib_internal_op(cib, PCMK__CIB_REQUEST_SCHEMAS, NULL, NULL, data,
                          output_data, call_options, NULL);
-    free_xml(data);
+    pcmk__xml_free(data);
     return rc;
 }
 
@@ -616,51 +562,51 @@ cib_t *
 cib_new(void)
 {
     const char *value = getenv("CIB_shadow");
+    const char *server = NULL;
+    const char *user = NULL;
+    const char *pass = NULL;
+    gboolean encrypted = TRUE;
     int port;
 
-    if (value && value[0] != 0) {
+    if (!pcmk__str_empty(value)) {
         return cib_shadow_new(value);
     }
 
     value = getenv("CIB_file");
-    if (value) {
+    if (!pcmk__str_empty(value)) {
         return cib_file_new(value);
     }
 
     value = getenv("CIB_port");
-    if (value) {
-        gboolean encrypted = TRUE;
-        const char *server = getenv("CIB_server");
-        const char *user = getenv("CIB_user");
-        const char *pass = getenv("CIB_passwd");
-
-        /* We don't ensure port is valid (>= 0) because cib_new() currently
-         * can't return NULL in practice, and introducing a NULL return here
-         * could cause core dumps that would previously just cause signon()
-         * failures.
-         */
-        pcmk__scan_port(value, &port);
-
-        value = getenv("CIB_encrypted");
-        if (value && crm_is_true(value) == FALSE) {
-            crm_info("Disabling TLS");
-            encrypted = FALSE;
-        }
-
-        if (user == NULL) {
-            user = CRM_DAEMON_USER;
-            crm_info("Defaulting to user: %s", user);
-        }
-
-        if (server == NULL) {
-            server = "localhost";
-            crm_info("Defaulting to localhost");
-        }
-
-        return cib_remote_new(server, user, pass, port, encrypted);
+    if (pcmk__str_empty(value)) {
+        return cib_native_new();
     }
 
-    return cib_native_new();
+    /* We don't ensure port is valid (>= 0) because cib_new() currently can't
+     * return NULL in practice, and introducing a NULL return here could cause
+     * core dumps that would previously just cause signon() failures.
+     */
+    pcmk__scan_port(value, &port);
+
+    if (!crm_is_true(getenv("CIB_encrypted"))) {
+        encrypted = FALSE;
+    }
+
+    server = getenv("CIB_server");
+    user = getenv("CIB_user");
+    pass = getenv("CIB_passwd");
+
+    if (pcmk__str_empty(user)) {
+        user = CRM_DAEMON_USER;
+    }
+
+    if (pcmk__str_empty(server)) {
+        server = "localhost";
+    }
+
+    crm_debug("Initializing %s remote CIB access to %s:%d as user %s",
+              (encrypted? "encrypted" : "plain-text"), server, port, user);
+    return cib_remote_new(server, user, pass, port, encrypted);
 }
 
 /*!
@@ -690,8 +636,6 @@ cib_new_variant(void)
 
     new_cib->type = cib_no_connection;
     new_cib->state = cib_disconnected;
-
-    new_cib->op_callback = NULL;
     new_cib->variant_opaque = NULL;
     new_cib->notify_list = NULL;
 
@@ -703,9 +647,6 @@ cib_new_variant(void)
         return NULL;
     }
 
-    // Deprecated method
-    new_cib->cmds->set_op_callback = cib_client_set_op_callback;
-
     new_cib->cmds->add_notify_callback = cib_client_add_notify_callback;
     new_cib->cmds->del_notify_callback = cib_client_del_notify_callback;
     new_cib->cmds->register_callback = cib_client_register_callback;
@@ -719,29 +660,18 @@ cib_new_variant(void)
     new_cib->cmds->query_from = cib_client_query_from;
     new_cib->cmds->sync_from = cib_client_sync_from;
 
-    new_cib->cmds->is_master = is_primary; // Deprecated method
-
     new_cib->cmds->set_primary = set_primary;
-    new_cib->cmds->set_master = set_primary; // Deprecated method
-
     new_cib->cmds->set_secondary = set_secondary;
-    new_cib->cmds->set_slave = set_secondary; // Deprecated method
-
-    new_cib->cmds->set_slave_all = set_all_secondary; // Deprecated method
 
     new_cib->cmds->upgrade = cib_client_upgrade;
     new_cib->cmds->bump_epoch = cib_client_bump_epoch;
 
     new_cib->cmds->create = cib_client_create;
     new_cib->cmds->modify = cib_client_modify;
-    new_cib->cmds->update = cib_client_modify; // Deprecated method
     new_cib->cmds->replace = cib_client_replace;
     new_cib->cmds->remove = cib_client_delete;
     new_cib->cmds->erase = cib_client_erase;
 
-    // Deprecated method
-    new_cib->cmds->delete_absolute = cib_client_delete_absolute;
-
     new_cib->cmds->init_transaction = cib_client_init_transaction;
     new_cib->cmds->end_transaction = cib_client_end_transaction;
 
