@@ -170,7 +170,7 @@ file_get_op_function(const cib__operation_t *operation)
 {
     enum cib__op_type type = operation->type;
 
-    CRM_ASSERT(type >= 0);
+    pcmk__assert(type >= 0);
 
     if (type >= PCMK__NELEM(cib_op_functions)) {
         return NULL;
@@ -217,7 +217,7 @@ cib_file_process_request(cib_t *cib, xmlNode *request, xmlNode **output)
     cib__op_fn_t op_function = NULL;
 
     int call_id = 0;
-    int call_options = cib_none;
+    uint32_t call_options = cib_none;
     const char *op = crm_element_value(request, PCMK__XA_CIB_OP);
     const char *section = crm_element_value(request, PCMK__XA_CIB_SECTION);
     xmlNode *wrapper = pcmk__xe_first_child(request, PCMK__XE_CIB_CALLDATA,
@@ -236,11 +236,15 @@ cib_file_process_request(cib_t *cib, xmlNode *request, xmlNode **output)
     op_function = file_get_op_function(operation);
 
     crm_element_value_int(request, PCMK__XA_CIB_CALLID, &call_id);
-    crm_element_value_int(request, PCMK__XA_CIB_CALLOPT, &call_options);
+    rc = pcmk__xe_get_flags(request, PCMK__XA_CIB_CALLOPT, &call_options,
+                            cib_none);
+    if (rc != pcmk_rc_ok) {
+        crm_warn("Couldn't parse options from request: %s", pcmk_rc_str(rc));
+    }
 
     read_only = !pcmk_is_set(operation->flags, cib__op_attr_modifies);
 
-    // Mirror the logic in prepare_input() in pacemaker-based
+    // Mirror the logic in prepare_input() in the CIB manager
     if ((section != NULL) && pcmk__xe_is(data, PCMK_XE_CIB)) {
 
         data = pcmk_find_cib_element(data, section);
@@ -265,22 +269,17 @@ cib_file_process_request(cib_t *cib, xmlNode *request, xmlNode **output)
         pcmk__log_xml_patchset(LOG_DEBUG, cib_diff);
 
         if (result_cib != private->cib_xml) {
-            free_xml(private->cib_xml);
+            pcmk__xml_free(private->cib_xml);
             private->cib_xml = result_cib;
         }
         cib_set_file_flags(private, cib_file_flag_dirty);
     }
 
-    // Global operation callback (deprecated)
-    if (cib->op_callback != NULL) {
-        cib->op_callback(NULL, call_id, rc, *output);
-    }
-
 done:
     if ((result_cib != private->cib_xml) && (result_cib != *output)) {
-        free_xml(result_cib);
+        pcmk__xml_free(result_cib);
     }
-    free_xml(cib_diff);
+    pcmk__xml_free(cib_diff);
     return rc;
 }
 
@@ -329,7 +328,7 @@ cib_file_perform_op_delegate(cib_t *cib, const char *op, const char *host,
     if (rc != pcmk_ok) {
         return rc;
     }
-    crm_xml_add(request, PCMK_XE_ACL_TARGET, user_name);
+    crm_xml_add(request, PCMK__XA_ACL_TARGET, user_name);
     crm_xml_add(request, PCMK__XA_CIB_CLIENTID, private->id);
 
     if (pcmk_is_set(call_options, cib_transaction)) {
@@ -352,9 +351,9 @@ done:
         && (output->doc != private->cib_xml->doc)
         && ((output_data == NULL) || (output != *output_data))) {
 
-        free_xml(output);
+        pcmk__xml_free(output);
     }
-    free_xml(request);
+    pcmk__xml_free(request);
     return rc;
 }
 
@@ -397,10 +396,7 @@ load_file_cib(const char *filename, xmlNode **output)
 
     /* Validate XML against its specified schema */
     if (!pcmk__configured_schema_validates(root)) {
-        const char *schema = crm_element_value(root, PCMK_XA_VALIDATE_WITH);
-
-        crm_err("CIB does not validate against %s, or that schema is unknown", schema);
-        free_xml(root);
+        pcmk__xml_free(root);
         return -pcmk_err_schema_validation;
     }
 
@@ -423,14 +419,15 @@ cib_file_signon(cib_t *cib, const char *name, enum cib_conn_type type)
 
     if (rc == pcmk_ok) {
         crm_debug("Opened connection to local file '%s' for %s",
-                  private->filename, name);
+                  private->filename, pcmk__s(name, "client"));
         cib->state = cib_connected_command;
         cib->type = cib_command;
         register_client(cib);
 
     } else {
         crm_info("Connection to local file '%s' for %s (client %s) failed: %s",
-                 private->filename, name, private->id, pcmk_strerror(rc));
+                 private->filename, pcmk__s(name, "client"), private->id,
+                 pcmk_strerror(rc));
     }
     return rc;
 }
@@ -552,7 +549,7 @@ cib_file_signoff(cib_t *cib)
             bool compress = pcmk__ends_with_ext(private->filename, ".bz2");
 
             if (pcmk__xml_write_file(private->cib_xml, private->filename,
-                                     compress, NULL) != pcmk_rc_ok) {
+                                     compress) != pcmk_rc_ok) {
                 rc = pcmk_err_generic;
             }
         }
@@ -566,7 +563,7 @@ cib_file_signoff(cib_t *cib)
     }
 
     /* Free the in-memory CIB */
-    free_xml(private->cib_xml);
+    pcmk__xml_free(private->cib_xml);
     private->cib_xml = NULL;
     return rc;
 }
@@ -597,12 +594,6 @@ cib_file_free(cib_t *cib)
     return rc;
 }
 
-static int
-cib_file_inputfd(cib_t *cib)
-{
-    return -EPROTONOSUPPORT;
-}
-
 static int
 cib_file_register_notification(cib_t *cib, const char *callback, int enabled)
 {
@@ -647,42 +638,52 @@ cib_file_client_id(const cib_t *cib, const char **async_id,
 cib_t *
 cib_file_new(const char *cib_location)
 {
+    cib_t *cib = NULL;
     cib_file_opaque_t *private = NULL;
-    cib_t *cib = cib_new_variant();
+    char *filename = NULL;
 
+    if (cib_location == NULL) {
+        cib_location = getenv("CIB_file");
+        if (cib_location == NULL) {
+            return NULL; // Shouldn't be possible if we were called internally
+        }
+    }
+
+    cib = cib_new_variant();
     if (cib == NULL) {
         return NULL;
     }
 
-    private = calloc(1, sizeof(cib_file_opaque_t));
-
-    if (private == NULL) {
+    filename = strdup(cib_location);
+    if (filename == NULL) {
         free(cib);
         return NULL;
     }
+
+    private = calloc(1, sizeof(cib_file_opaque_t));
+    if (private == NULL) {
+        free(cib);
+        free(filename);
+        return NULL;
+    }
+
     private->id = crm_generate_uuid();
+    private->filename = filename;
 
     cib->variant = cib_file;
     cib->variant_opaque = private;
 
-    if (cib_location == NULL) {
-        cib_location = getenv("CIB_file");
-        CRM_CHECK(cib_location != NULL, return NULL); // Shouldn't be possible
-    }
     private->flags = 0;
     if (cib_file_is_live(cib_location)) {
         cib_set_file_flags(private, cib_file_flag_live);
         crm_trace("File %s detected as live CIB", cib_location);
     }
-    private->filename = strdup(cib_location);
 
     /* assign variant specific ops */
     cib->delegate_fn = cib_file_perform_op_delegate;
     cib->cmds->signon = cib_file_signon;
     cib->cmds->signoff = cib_file_signoff;
     cib->cmds->free = cib_file_free;
-    cib->cmds->inputfd = cib_file_inputfd; // Deprecated method
-
     cib->cmds->register_notification = cib_file_register_notification;
     cib->cmds->set_connection_dnotify = cib_file_set_connection_dnotify;
 
@@ -750,7 +751,7 @@ cib_file_read_and_verify(const char *filename, const char *sigfile, xmlNode **ro
     char *local_sigfile = NULL;
     xmlNode *local_root = NULL;
 
-    CRM_ASSERT(filename != NULL);
+    pcmk__assert(filename != NULL);
     if (root) {
         *root = NULL;
     }
@@ -780,7 +781,7 @@ cib_file_read_and_verify(const char *filename, const char *sigfile, xmlNode **ro
     /* Verify that digests match */
     if (cib_file_verify_digest(local_root, sigfile) == FALSE) {
         free(local_sigfile);
-        free_xml(local_root);
+        pcmk__xml_free(local_root);
         return -pcmk_err_cib_modified;
     }
 
@@ -788,7 +789,7 @@ cib_file_read_and_verify(const char *filename, const char *sigfile, xmlNode **ro
     if (root) {
         *root = local_root;
     } else {
-        free_xml(local_root);
+        pcmk__xml_free(local_root);
     }
     return pcmk_ok;
 }
@@ -806,7 +807,7 @@ static int
 cib_file_backup(const char *cib_dirname, const char *cib_filename)
 {
     int rc = 0;
-    unsigned int seq;
+    unsigned int seq = 0U;
     char *cib_path = crm_strdup_printf("%s/%s", cib_dirname, cib_filename);
     char *cib_digest = crm_strdup_printf("%s.sig", cib_path);
     char *backup_path;
@@ -816,7 +817,7 @@ cib_file_backup(const char *cib_dirname, const char *cib_filename)
     if (pcmk__read_series_sequence(cib_dirname, CIB_SERIES,
                                    &seq) != pcmk_rc_ok) {
         // @TODO maybe handle errors better ...
-        seq = 0;
+        seq = 0U;
     }
     backup_path = pcmk__series_filename(cib_dirname, CIB_SERIES, seq,
                                         CIB_SERIES_BZIP);
@@ -898,7 +899,7 @@ cib_file_prepare_xml(xmlNode *root)
      * we discard it on startup anyway, and users get confused by it */
     cib_status_root = pcmk__xe_first_child(root, PCMK_XE_STATUS, NULL, NULL);
     CRM_CHECK(cib_status_root != NULL, return);
-    free_xml(cib_status_root);
+    pcmk__xml_free(cib_status_root);
 }
 
 /*!
@@ -978,15 +979,15 @@ cib_file_write_with_digest(xmlNode *cib_root, const char *cib_dirname,
     }
 
     /* Write out the CIB */
-    if (pcmk__xml_write_fd(cib_root, tmp_cib, fd, false, NULL) != pcmk_rc_ok) {
+    if (pcmk__xml_write_fd(cib_root, tmp_cib, fd) != pcmk_rc_ok) {
         crm_err("Changes couldn't be written to %s", tmp_cib);
         exit_rc = pcmk_err_cib_save;
         goto cleanup;
     }
 
     /* Calculate CIB digest */
-    digest = calculate_on_disk_digest(cib_root);
-    CRM_ASSERT(digest != NULL);
+    digest = pcmk__digest_on_disk_cib(cib_root);
+    pcmk__assert(digest != NULL);
     crm_info("Wrote version %s.%s.0 of the CIB to disk (digest: %s)",
              (admin_epoch ? admin_epoch : "0"), (epoch ? epoch : "0"), digest);
 
@@ -1019,7 +1020,7 @@ cib_file_write_with_digest(xmlNode *cib_root, const char *cib_dirname,
     crm_info("Reading cluster configuration file %s (digest: %s)",
              tmp_cib, tmp_digest);
     rc = cib_file_read_and_verify(tmp_cib, tmp_digest, NULL);
-    CRM_ASSERT(rc == 0);
+    pcmk__assert(rc == 0);
 
     /* Rename temporary files to live, and sync directory changes to media */
     crm_debug("Activating %s", tmp_cib);
@@ -1062,7 +1063,8 @@ cib_file_process_transaction_requests(cib_t *cib, xmlNode *transaction)
     for (xmlNode *request = pcmk__xe_first_child(transaction,
                                                  PCMK__XE_CIB_COMMAND, NULL,
                                                  NULL);
-         request != NULL; request = pcmk__xe_next_same(request)) {
+         request != NULL;
+         request = pcmk__xe_next(request, PCMK__XE_CIB_COMMAND)) {
 
         xmlNode *output = NULL;
         const char *op = crm_element_value(request, PCMK__XA_CIB_OP);
@@ -1099,7 +1101,7 @@ cib_file_process_transaction_requests(cib_t *cib, xmlNode *transaction)
  *
  * \note The caller is responsible for replacing the \p cib argument's
  *       \p private->cib_xml with \p result_cib on success, and for freeing
- *       \p result_cib using \p free_xml() on failure.
+ *       \p result_cib using \p pcmk__xml_free() on failure.
  */
 static int
 cib_file_commit_transaction(cib_t *cib, xmlNode *transaction,
