@@ -10,10 +10,6 @@
 
 #include <crm_internal.h>
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <errno.h>
 #include <crm_internal.h>
 #include <unistd.h>
@@ -169,8 +165,8 @@ cib_native_perform_op_delegate(cib_t *cib, const char *op, const char *host,
         cib->state = cib_disconnected;
     }
 
-    free_xml(op_msg);
-    free_xml(op_reply);
+    pcmk__xml_free(op_msg);
+    pcmk__xml_free(op_reply);
     return rc;
 }
 
@@ -212,7 +208,7 @@ cib_native_dispatch_internal(const char *buffer, ssize_t length,
         crm_err("Unknown message type: %s", type);
     }
 
-    free_xml(msg);
+    pcmk__xml_free(msg);
     return 0;
 }
 
@@ -265,8 +261,7 @@ cib_native_signoff(cib_t *cib)
 }
 
 static int
-cib_native_signon_raw(cib_t *cib, const char *name, enum cib_conn_type type,
-                      int *async_fd)
+cib_native_signon(cib_t *cib, const char *name, enum cib_conn_type type)
 {
     int rc = pcmk_ok;
     const char *channel = NULL;
@@ -278,6 +273,10 @@ cib_native_signon_raw(cib_t *cib, const char *name, enum cib_conn_type type,
         .destroy = cib_native_destroy
     };
 
+    if (name == NULL) {
+        name = pcmk__s(crm_system_name, "client");
+    }
+
     cib->call_timeout = PCMK__IPC_TIMEOUT;
 
     if (type == cib_command) {
@@ -298,26 +297,9 @@ cib_native_signon_raw(cib_t *cib, const char *name, enum cib_conn_type type,
 
     crm_trace("Connecting %s channel", channel);
 
-    if (async_fd != NULL) {
-        native->ipc = crm_ipc_new(channel, 0);
-        if (native->ipc != NULL) {
-            rc = pcmk__connect_generic_ipc(native->ipc);
-            if (rc == pcmk_rc_ok) {
-                rc = pcmk__ipc_fd(native->ipc, async_fd);
-                if (rc != pcmk_rc_ok) {
-                    crm_info("Couldn't get file descriptor for %s IPC",
-                             channel);
-                }
-            }
-            rc = pcmk_rc2legacy(rc);
-        }
-
-    } else {
-        native->source =
-            mainloop_add_ipc_client(channel, G_PRIORITY_HIGH, 512 * 1024 /* 512k */ , cib,
-                                    &cib_callbacks);
-        native->ipc = mainloop_get_ipc_client(native->source);
-    }
+    native->source = mainloop_add_ipc_client(channel, G_PRIORITY_HIGH,
+                                             512 * 1024, cib, &cib_callbacks);
+    native->ipc = mainloop_get_ipc_client(native->source);
 
     if (rc != pcmk_ok || native->ipc == NULL || !crm_ipc_connected(native->ipc)) {
         crm_info("Could not connect to CIB manager for %s", name);
@@ -351,12 +333,12 @@ cib_native_signon_raw(cib_t *cib, const char *name, enum cib_conn_type type,
                     rc = -EPROTO;
                 }
             }
-            free_xml(reply);
+            pcmk__xml_free(reply);
 
         } else {
             rc = -ECOMM;
         }
-        free_xml(hello);
+        pcmk__xml_free(hello);
     }
 
     if (rc == pcmk_ok) {
@@ -370,12 +352,6 @@ cib_native_signon_raw(cib_t *cib, const char *name, enum cib_conn_type type,
     return rc;
 }
 
-static int
-cib_native_signon(cib_t *cib, const char *name, enum cib_conn_type type)
-{
-    return cib_native_signon_raw(cib, name, type, NULL);
-}
-
 static int
 cib_native_free(cib_t *cib)
 {
@@ -417,7 +393,7 @@ cib_native_register_notification(cib_t *cib, const char *callback, int enabled)
         }
     }
 
-    free_xml(notify_msg);
+    pcmk__xml_free(notify_msg);
     return rc;
 }
 
@@ -498,7 +474,6 @@ cib_native_new(void)
     /* assign variant specific ops */
     cib->delegate_fn = cib_native_perform_op_delegate;
     cib->cmds->signon = cib_native_signon;
-    cib->cmds->signon_raw = cib_native_signon_raw;
     cib->cmds->signoff = cib_native_signoff;
     cib->cmds->free = cib_native_free;
 
