@@ -143,7 +143,7 @@ static const cib__operation_t cib_ops[] = {
 int
 cib__get_operation(const char *op, const cib__operation_t **operation)
 {
-    CRM_ASSERT((op != NULL) && (operation != NULL));
+    pcmk__assert((op != NULL) && (operation != NULL));
 
     if (operation_table == NULL) {
         operation_table = pcmk__strkey_table(NULL, NULL);
@@ -179,7 +179,7 @@ cib_process_query(const char *op, int options, const char *section, xmlNode * re
                                  existing_cib, result_cib, answer);
     }
 
-    CRM_CHECK(*answer == NULL, free_xml(*answer));
+    CRM_CHECK(*answer == NULL, pcmk__xml_free(*answer));
     *answer = NULL;
 
     if (pcmk__str_eq(PCMK__XE_ALL, section, pcmk__str_casei)) {
@@ -246,7 +246,7 @@ cib_process_erase(const char *op, int options, const char *section, xmlNode * re
     crm_trace("Processing \"%s\" event", op);
 
     if (*result_cib != existing_cib) {
-        free_xml(*result_cib);
+        pcmk__xml_free(*result_cib);
     }
     *result_cib = createEmptyCib(0);
     pcmk__xe_copy_attrs(*result_cib, existing_cib, pcmk__xaf_none);
@@ -342,11 +342,7 @@ cib_process_replace(const char *op, int options, const char *section, xmlNode *
         const char *digest = crm_element_value(req, PCMK__XA_DIGEST);
 
         if (digest) {
-            const char *version = crm_element_value(req,
-                                                    PCMK_XA_CRM_FEATURE_SET);
-            char *digest_verify = calculate_xml_versioned_digest(input, FALSE, TRUE,
-                                                                 version ? version :
-                                                                 CRM_FEATURE_SET);
+            char *digest_verify = pcmk__digest_xml(input, true);
 
             if (!pcmk__str_eq(digest_verify, digest, pcmk__str_casei)) {
                 crm_err("Digest mis-match on replace from %s: %s vs. %s (expected)", peer,
@@ -394,7 +390,7 @@ cib_process_replace(const char *op, int options, const char *section, xmlNode *
         }
 
         if (*result_cib != existing_cib) {
-            free_xml(*result_cib);
+            pcmk__xml_free(*result_cib);
         }
         *result_cib = pcmk__xml_copy(NULL, input);
 
@@ -484,7 +480,7 @@ cib_process_modify(const char *op, int options, const char *section, xmlNode * r
         tmp_section = pcmk__xe_create(NULL, section);
         cib_process_xpath(PCMK__CIB_REQUEST_CREATE, 0, path, NULL, tmp_section,
                           NULL, result_cib, answer);
-        free_xml(tmp_section);
+        pcmk__xml_free(tmp_section);
 
         obj_root = pcmk_find_cib_element(*result_cib, section);
     }
@@ -503,27 +499,6 @@ cib_process_modify(const char *op, int options, const char *section, xmlNode * r
         }
     }
 
-    // @COMPAT cib_mixed_update is deprecated as of 2.1.7
-    if (pcmk_is_set(options, cib_mixed_update)) {
-        int max = 0, lpc;
-        xmlXPathObjectPtr xpathObj = xpath_search(*result_cib, "//@__delete__");
-
-        if (xpathObj) {
-            max = numXpathResults(xpathObj);
-            crm_log_xml_trace(*result_cib, "Mixed result");
-        }
-
-        for (lpc = 0; lpc < max; lpc++) {
-            xmlNode *match = getXpathResult(xpathObj, lpc);
-            xmlChar *match_path = xmlGetNodePath(match);
-
-            crm_debug("Destroying %s", match_path);
-            free(match_path);
-            free_xml(match);
-        }
-
-        freeXpathObject(xpathObj);
-    }
     return pcmk_ok;
 }
 
@@ -659,7 +634,7 @@ cib_process_create(const char *op, int options, const char *section, xmlNode * r
         *answer = failed;
 
     } else {
-        free_xml(failed);
+        pcmk__xml_free(failed);
     }
 
     return result;
@@ -680,85 +655,13 @@ cib_process_diff(const char *op, int options, const char *section, xmlNode * req
               (pcmk_is_set(options, cib_force_diff)? " (global update)" : ""));
 
     if (*result_cib != existing_cib) {
-        free_xml(*result_cib);
+        pcmk__xml_free(*result_cib);
     }
     *result_cib = pcmk__xml_copy(NULL, existing_cib);
 
     return xml_apply_patchset(*result_cib, input, TRUE);
 }
 
-// @COMPAT: v1-only
-bool
-cib__config_changed_v1(xmlNode *last, xmlNode *next, xmlNode **diff)
-{
-    int lpc = 0, max = 0;
-    bool config_changes = false;
-    xmlXPathObject *xpathObj = NULL;
-    int format = 1;
-
-    CRM_ASSERT(diff != NULL);
-
-    if (*diff == NULL && last != NULL && next != NULL) {
-        *diff = pcmk__diff_v1_xml_object(last, next, false);
-    }
-
-    if (*diff == NULL) {
-        goto done;
-    }
-
-    crm_element_value_int(*diff, PCMK_XA_FORMAT, &format);
-    CRM_LOG_ASSERT(format == 1);
-
-    xpathObj = xpath_search(*diff, "//" PCMK_XE_CONFIGURATION);
-    if (numXpathResults(xpathObj) > 0) {
-        config_changes = true;
-        goto done;
-    }
-    freeXpathObject(xpathObj);
-
-    /*
-     * Do not check PCMK__XE_DIFF_ADDED "//" PCMK_XE_CIB
-     * This always contains every field and would produce a false positive
-     * every time if the checked value existed
-     */
-    xpathObj = xpath_search(*diff, "//" PCMK__XE_DIFF_REMOVED "//" PCMK_XE_CIB);
-    max = numXpathResults(xpathObj);
-
-    for (lpc = 0; lpc < max; lpc++) {
-        xmlNode *top = getXpathResult(xpathObj, lpc);
-
-        if (crm_element_value(top, PCMK_XA_EPOCH) != NULL) {
-            config_changes = true;
-            goto done;
-        }
-        if (crm_element_value(top, PCMK_XA_ADMIN_EPOCH) != NULL) {
-            config_changes = true;
-            goto done;
-        }
-
-        if (crm_element_value(top, PCMK_XA_VALIDATE_WITH) != NULL) {
-            config_changes = true;
-            goto done;
-        }
-        if (crm_element_value(top, PCMK_XA_CRM_FEATURE_SET) != NULL) {
-            config_changes = true;
-            goto done;
-        }
-        if (crm_element_value(top, PCMK_XA_REMOTE_CLEAR_PORT) != NULL) {
-            config_changes = true;
-            goto done;
-        }
-        if (crm_element_value(top, PCMK_XA_REMOTE_TLS_PORT) != NULL) {
-            config_changes = true;
-            goto done;
-        }
-    }
-
-  done:
-    freeXpathObject(xpathObj);
-    return config_changes;
-}
-
 int
 cib_process_xpath(const char *op, int options, const char *section,
                   const xmlNode *req, xmlNode *input, xmlNode *existing_cib,
@@ -820,7 +723,7 @@ cib_process_xpath(const char *op, int options, const char *section,
                 break;
             }
 
-            free_xml(match);
+            pcmk__xml_free(match);
             if ((options & cib_multiple) == 0) {
                 break;
             }
@@ -895,7 +798,7 @@ cib_process_xpath(const char *op, int options, const char *section,
                                 pcmk__str_none)) {
             xmlNode *parent = match->parent;
 
-            free_xml(match);
+            pcmk__xml_free(match);
             pcmk__xml_copy(parent, input);
 
             if ((options & cib_multiple) == 0) {
