@@ -26,19 +26,15 @@
 #include <crm/common/mainloop.h>
 #include <crm/common/xml.h>
 #include <crm/common/remote_internal.h>
+#include <crm/common/tls_internal.h>
 #include <crm/common/output_internal.h>
 
-#ifdef HAVE_GNUTLS_GNUTLS_H
-
-#  include <gnutls/gnutls.h>
+#include <gnutls/gnutls.h>
 
 // GnuTLS handshake timeout in seconds
 #define TLS_HANDSHAKE_TIMEOUT 5
 
-static gnutls_anon_client_credentials_t anon_cred_c;
-static gboolean remote_gnutls_credentials_init = FALSE;
-
-#endif // HAVE_GNUTLS_GNUTLS_H
+static pcmk__tls_t *tls = NULL;
 
 #include <arpa/inet.h>
 
@@ -51,6 +47,8 @@ typedef struct cib_remote_opaque_s {
     pcmk__remote_t command;
     pcmk__remote_t callback;
     pcmk__output_t *out;
+    time_t start_time;
+    int timeout_sec;
 } cib_remote_opaque_t;
 
 static int
@@ -89,7 +87,7 @@ cib_remote_perform_op(cib_t *cib, const char *op, const char *host,
 
     if (pcmk_is_set(call_options, cib_transaction)) {
         rc = cib__extend_transaction(cib, op_msg);
-        free_xml(op_msg);
+        pcmk__xml_free(op_msg);
         return rc;
     }
 
@@ -99,7 +97,7 @@ cib_remote_perform_op(cib_t *cib, const char *op, const char *host,
     } else {
         pcmk__remote_send_xml(&private->command, op_msg);
     }
-    free_xml(op_msg);
+    pcmk__xml_free(op_msg);
 
     if ((call_options & cib_discard_reply)) {
         crm_trace("Discarding reply");
@@ -144,19 +142,13 @@ cib_remote_perform_op(cib_t *cib, const char *op, const char *host,
             crm_err("Received a __future__ reply:" " %d (wanted %d)", reply_id, msg_id);
         }
 
-        free_xml(op_reply);
+        pcmk__xml_free(op_reply);
         op_reply = NULL;
 
         /* wasn't the right reply, try and read some more */
         remaining_time = time(NULL) - start_time;
     }
 
-    /* if(IPC_ISRCONN(native->command_channel) == FALSE) { */
-    /*      crm_err("The CIB manager disconnected: %d",  */
-    /*              native->command_channel->ch_status); */
-    /*      cib->state = cib_disconnected; */
-    /* } */
-
     if (rc == ENOTCONN) {
         crm_err("Disconnected while waiting for reply.");
         return -ENOTCONN;
@@ -181,7 +173,6 @@ cib_remote_perform_op(cib_t *cib, const char *op, const char *host,
         crm_log_xml_debug(op_reply, "passed");
 
     } else {
-/*  } else if(rc == -ETIME) { */
         crm_err("Call failed: %s", pcmk_strerror(rc));
         crm_log_xml_warn(op_reply, "failed");
     }
@@ -201,7 +192,7 @@ cib_remote_perform_op(cib_t *cib, const char *op, const char *host,
         }
     }
 
-    free_xml(op_reply);
+    pcmk__xml_free(op_reply);
 
     return rc;
 }
@@ -214,35 +205,60 @@ cib_remote_callback_dispatch(gpointer user_data)
     cib_remote_opaque_t *private = cib->variant_opaque;
 
     xmlNode *msg = NULL;
+    const char *type = NULL;
 
-    crm_info("Message on callback channel");
+    /* If start time is 0, we've previously handled a complete message and this
+     * connection is being reused for a new message.  Reset the start_time,
+     * giving this new message timeout_sec from now to complete.
+     */
+    if (private->start_time == 0) {
+        private->start_time = time(NULL);
+    }
 
-    rc = pcmk__read_remote_message(&private->callback, -1);
+    rc = pcmk__read_available_remote_data(&private->callback);
+    switch (rc) {
+        case pcmk_rc_ok:
+            /* We have the whole message so process it */
+            break;
+
+        case EAGAIN:
+            /* Have we timed out? */
+            if (time(NULL) >= private->start_time + private->timeout_sec) {
+                crm_info("Error reading from CIB manager connection: %s",
+                         pcmk_rc_str(ETIME));
+                return -1;
+            }
+
+            /* We haven't read the whole message yet */
+            return 0;
+
+        default:
+            /* Error */
+            crm_info("Error reading from CIB manager connection: %s",
+                     pcmk_rc_str(rc));
+            return -1;
+    }
 
     msg = pcmk__remote_message_xml(&private->callback);
-    while (msg) {
-        const char *type = crm_element_value(msg, PCMK__XA_T);
-
-        crm_trace("Activating %s callbacks...", type);
-
-        if (pcmk__str_eq(type, PCMK__VALUE_CIB, pcmk__str_none)) {
-            cib_native_callback(cib, msg, 0, 0);
-
-        } else if (pcmk__str_eq(type, PCMK__VALUE_CIB_NOTIFY, pcmk__str_none)) {
-            g_list_foreach(cib->notify_list, cib_native_notify, msg);
-
-        } else {
-            crm_err("Unknown message type: %s", type);
-        }
-
-        free_xml(msg);
-        msg = pcmk__remote_message_xml(&private->callback);
+    if (msg == NULL) {
+        private->start_time = 0;
+        return 0;
     }
 
-    if (rc == ENOTCONN) {
-        return -1;
+    type = crm_element_value(msg, PCMK__XA_T);
+
+    crm_trace("Activating %s callbacks...", type);
+
+    if (pcmk__str_eq(type, PCMK__VALUE_CIB, pcmk__str_none)) {
+        cib_native_callback(cib, msg, 0, 0);
+    } else if (pcmk__str_eq(type, PCMK__VALUE_CIB_NOTIFY, pcmk__str_none)) {
+        g_list_foreach(cib->notify_list, cib_native_notify, msg);
+    } else {
+        crm_err("Unknown message type: %s", type);
     }
 
+    pcmk__xml_free(msg);
+    private->start_time = 0;
     return 0;
 }
 
@@ -253,15 +269,35 @@ cib_remote_command_dispatch(gpointer user_data)
     cib_t *cib = user_data;
     cib_remote_opaque_t *private = cib->variant_opaque;
 
-    rc = pcmk__read_remote_message(&private->command, -1);
+    /* See cib_remote_callback_dispatch */
+    if (private->start_time == 0) {
+        private->start_time = time(NULL);
+    }
+
+    rc = pcmk__read_available_remote_data(&private->command);
+    if (rc == EAGAIN) {
+        /* Have we timed out? */
+        if (time(NULL) >= private->start_time + private->timeout_sec) {
+            crm_info("Error reading from CIB manager connection: %s",
+                     pcmk_rc_str(ETIME));
+            return -1;
+        }
+
+        /* We haven't read the whole message yet */
+        return 0;
+    }
 
     free(private->command.buffer);
     private->command.buffer = NULL;
     crm_err("received late reply for remote cib connection, discarding");
 
-    if (rc == ENOTCONN) {
+    if (rc != pcmk_rc_ok) {
+        crm_info("Error reading from CIB manager connection: %s",
+                 pcmk_rc_str(rc));
         return -1;
     }
+
+    private->start_time = 0;
     return 0;
 }
 
@@ -270,28 +306,22 @@ cib_tls_close(cib_t *cib)
 {
     cib_remote_opaque_t *private = cib->variant_opaque;
 
-#ifdef HAVE_GNUTLS_GNUTLS_H
     if (private->encrypted) {
         if (private->command.tls_session) {
-            gnutls_bye(*(private->command.tls_session), GNUTLS_SHUT_RDWR);
-            gnutls_deinit(*(private->command.tls_session));
-            gnutls_free(private->command.tls_session);
+            gnutls_bye(private->command.tls_session, GNUTLS_SHUT_RDWR);
+            gnutls_deinit(private->command.tls_session);
         }
 
         if (private->callback.tls_session) {
-            gnutls_bye(*(private->callback.tls_session), GNUTLS_SHUT_RDWR);
-            gnutls_deinit(*(private->callback.tls_session));
-            gnutls_free(private->callback.tls_session);
+            gnutls_bye(private->callback.tls_session, GNUTLS_SHUT_RDWR);
+            gnutls_deinit(private->callback.tls_session);
         }
+
         private->command.tls_session = NULL;
         private->callback.tls_session = NULL;
-        if (remote_gnutls_credentials_init) {
-            gnutls_anon_free_client_credentials(anon_cred_c);
-            gnutls_global_deinit();
-            remote_gnutls_credentials_init = FALSE;
-        }
+        pcmk__free_tls(tls);
+        tls = NULL;
     }
-#endif
 
     if (private->command.tcp_socket) {
         shutdown(private->command.tcp_socket, SHUT_RDWR);       /* no more receptions */
@@ -316,9 +346,7 @@ static void
 cib_remote_connection_destroy(gpointer user_data)
 {
     crm_err("Connection destroyed");
-#ifdef HAVE_GNUTLS_GNUTLS_H
     cib_tls_close(user_data);
-#endif
 }
 
 static int
@@ -337,33 +365,26 @@ cib_tls_signon(cib_t *cib, pcmk__remote_t *connection, gboolean event_channel)
     cib_fd_callbacks.destroy = cib_remote_connection_destroy;
 
     connection->tcp_socket = -1;
-#ifdef HAVE_GNUTLS_GNUTLS_H
     connection->tls_session = NULL;
-#endif
     rc = pcmk__connect_remote(private->server, private->port, 0, NULL,
                               &(connection->tcp_socket), NULL, NULL);
     if (rc != pcmk_rc_ok) {
-        crm_info("Remote connection to %s:%d failed: %s " CRM_XS " rc=%d",
+        crm_info("Remote connection to %s:%d failed: %s " QB_XS " rc=%d",
                  private->server, private->port, pcmk_rc_str(rc), rc);
         return -ENOTCONN;
     }
 
     if (private->encrypted) {
+        bool use_cert = pcmk__x509_enabled();
         int tls_rc = GNUTLS_E_SUCCESS;
 
-        /* initialize GnuTls lib */
-#ifdef HAVE_GNUTLS_GNUTLS_H
-        if (remote_gnutls_credentials_init == FALSE) {
-            crm_gnutls_global_init();
-            gnutls_anon_allocate_client_credentials(&anon_cred_c);
-            remote_gnutls_credentials_init = TRUE;
+        rc = pcmk__init_tls(&tls, false, use_cert ? GNUTLS_CRD_CERTIFICATE : GNUTLS_CRD_ANON);
+        if (rc != pcmk_rc_ok) {
+            return -1;
         }
 
         /* bind the socket to GnuTls lib */
-        connection->tls_session = pcmk__new_tls_session(connection->tcp_socket,
-                                                        GNUTLS_CLIENT,
-                                                        GNUTLS_CRD_ANON,
-                                                        anon_cred_c);
+        connection->tls_session = pcmk__new_tls_session(tls, connection->tcp_socket);
         if (connection->tls_session == NULL) {
             cib_tls_close(cib);
             return -1;
@@ -375,17 +396,20 @@ cib_tls_signon(cib_t *cib, pcmk__remote_t *connection, gboolean event_channel)
             crm_err("Remote CIB session creation for %s:%d failed: %s",
                     private->server, private->port,
                     (rc == EPROTO)? gnutls_strerror(tls_rc) : pcmk_rc_str(rc));
-            gnutls_deinit(*connection->tls_session);
-            gnutls_free(connection->tls_session);
+            gnutls_deinit(connection->tls_session);
             connection->tls_session = NULL;
             cib_tls_close(cib);
             return -1;
         }
-#else
-        return -EPROTONOSUPPORT;
-#endif
     }
 
+    /* Now that the handshake is done, see if any client TLS certificate is
+     * close to its expiration date and log if so.  If a TLS certificate is not
+     * in use, this function will just return so we don't need to check for the
+     * session type here.
+     */
+    pcmk__tls_check_cert_expiration(connection->tls_session);
+
     /* login to server */
     login = pcmk__xe_create(NULL, PCMK__XE_CIB_COMMAND);
     crm_xml_add(login, PCMK_XA_OP, "authenticate");
@@ -394,7 +418,7 @@ cib_tls_signon(cib_t *cib, pcmk__remote_t *connection, gboolean event_channel)
     crm_xml_add(login, PCMK__XA_HIDDEN, PCMK__VALUE_PASSWORD);
 
     pcmk__remote_send_xml(connection, login);
-    free_xml(login);
+    pcmk__xml_free(login);
 
     rc = pcmk_ok;
     if (pcmk__read_remote_message(connection, -1) == ENOTCONN) {
@@ -424,7 +448,7 @@ cib_tls_signon(cib_t *cib, pcmk__remote_t *connection, gboolean event_channel)
             connection->token = strdup(tmp_ticket);
         }
     }
-    free_xml(answer);
+    pcmk__xml_free(answer);
     answer = NULL;
 
     if (rc != 0) {
@@ -433,6 +457,7 @@ cib_tls_signon(cib_t *cib, pcmk__remote_t *connection, gboolean event_channel)
     }
 
     crm_trace("remote client connection established");
+    private->timeout_sec = 60;
     connection->source = mainloop_add_fd("cib-remote", G_PRIORITY_HIGH,
                                          connection->tcp_socket, cib,
                                          &cib_fd_callbacks);
@@ -446,6 +471,10 @@ cib_remote_signon(cib_t *cib, const char *name, enum cib_conn_type type)
     cib_remote_opaque_t *private = cib->variant_opaque;
     xmlNode *hello = NULL;
 
+    if (name == NULL) {
+        name = pcmk__s(crm_system_name, "client");
+    }
+
     if (private->passwd == NULL) {
         if (private->out == NULL) {
             /* If no pcmk__output_t is set, just assume that a text prompt
@@ -459,27 +488,30 @@ cib_remote_signon(cib_t *cib, const char *name, enum cib_conn_type type)
 
     if (private->server == NULL || private->user == NULL) {
         rc = -EINVAL;
+        goto done;
     }
 
-    if (rc == pcmk_ok) {
-        rc = cib_tls_signon(cib, &(private->command), FALSE);
+    rc = cib_tls_signon(cib, &(private->command), FALSE);
+    if (rc != pcmk_ok) {
+        goto done;
     }
 
-    if (rc == pcmk_ok) {
-        rc = cib_tls_signon(cib, &(private->callback), TRUE);
+    rc = cib_tls_signon(cib, &(private->callback), TRUE);
+    if (rc != pcmk_ok) {
+        goto done;
     }
 
-    if (rc == pcmk_ok) {
-        rc = cib__create_op(cib, CRM_OP_REGISTER, NULL, NULL, NULL, cib_none,
-                            NULL, name, &hello);
+    rc = cib__create_op(cib, CRM_OP_REGISTER, NULL, NULL, NULL, cib_none, NULL,
+                        name, &hello);
+    if (rc != pcmk_ok) {
+        goto done;
     }
 
-    if (rc == pcmk_ok) {
-        rc = pcmk__remote_send_xml(&private->command, hello);
-        rc = pcmk_rc2legacy(rc);
-        free_xml(hello);
-    }
+    rc = pcmk__remote_send_xml(&private->command, hello);
+    rc = pcmk_rc2legacy(rc);
+    pcmk__xml_free(hello);
 
+done:
     if (rc == pcmk_ok) {
         crm_info("Opened connection to %s:%d for %s",
                  private->server, private->port, name);
@@ -500,9 +532,7 @@ cib_remote_signoff(cib_t *cib)
     int rc = pcmk_ok;
 
     crm_debug("Disconnecting from the CIB manager");
-#ifdef HAVE_GNUTLS_GNUTLS_H
     cib_tls_close(cib);
-#endif
 
     cib->cmds->end_transaction(cib, false, cib_none);
     cib->state = cib_disconnected;
@@ -535,14 +565,6 @@ cib_remote_free(cib_t *cib)
     return rc;
 }
 
-static int
-cib_remote_inputfd(cib_t * cib)
-{
-    cib_remote_opaque_t *private = cib->variant_opaque;
-
-    return private->callback.tcp_socket;
-}
-
 static int
 cib_remote_register_notification(cib_t * cib, const char *callback, int enabled)
 {
@@ -553,7 +575,7 @@ cib_remote_register_notification(cib_t * cib, const char *callback, int enabled)
     crm_xml_add(notify_msg, PCMK__XA_CIB_NOTIFY_TYPE, callback);
     crm_xml_add_int(notify_msg, PCMK__XA_CIB_NOTIFY_ACTIVATE, enabled);
     pcmk__remote_send_xml(&private->callback, notify_msg);
-    free_xml(notify_msg);
+    pcmk__xml_free(notify_msg);
     return pcmk_ok;
 }
 
@@ -629,8 +651,6 @@ cib_remote_new(const char *server, const char *user, const char *passwd, int por
     cib->cmds->signon = cib_remote_signon;
     cib->cmds->signoff = cib_remote_signoff;
     cib->cmds->free = cib_remote_free;
-    cib->cmds->inputfd = cib_remote_inputfd; // Deprecated method
-
     cib->cmds->register_notification = cib_remote_register_notification;
     cib->cmds->set_connection_dnotify = cib_remote_set_connection_dnotify;
 
