@@ -77,7 +77,7 @@ cib__get_notify_patchset(const xmlNode *msg, const xmlNode **patchset)
     int rc = pcmk_err_generic;
     xmlNode *wrapper = NULL;
 
-    CRM_ASSERT(patchset != NULL);
+    pcmk__assert(patchset != NULL);
     *patchset = NULL;
 
     if (msg == NULL) {
@@ -88,7 +88,7 @@ cib__get_notify_patchset(const xmlNode *msg, const xmlNode **patchset)
     if ((crm_element_value_int(msg, PCMK__XA_CIB_RC, &rc) != 0)
         || (rc != pcmk_ok)) {
 
-        crm_warn("Ignore failed CIB update: %s " CRM_XS " rc=%d",
+        crm_warn("Ignore failed CIB update: %s " QB_XS " rc=%d",
                  pcmk_strerror(rc), rc);
         crm_log_xml_debug(msg, "failed");
         return pcmk_legacy2rc(rc);
@@ -104,125 +104,15 @@ cib__get_notify_patchset(const xmlNode *msg, const xmlNode **patchset)
     return pcmk_rc_ok;
 }
 
-#define XPATH_DIFF_V1 "//" PCMK__XE_CIB_UPDATE_RESULT "//" PCMK__XE_DIFF_ADDED
-
-/*!
- * \internal
- * \brief Check whether a given CIB element was modified in a CIB patchset (v1)
- *
- * \param[in] patchset  CIB XML patchset
- * \param[in] element   XML tag of CIB element to check (\c NULL is equivalent
- *                      to \c PCMK_XE_CIB)
- *
- * \return \c true if \p element was modified, or \c false otherwise
- */
-static bool
-element_in_patchset_v1(const xmlNode *patchset, const char *element)
-{
-    char *xpath = crm_strdup_printf(XPATH_DIFF_V1 "//%s",
-                                    pcmk__s(element, PCMK_XE_CIB));
-    xmlXPathObject *xpath_obj = xpath_search(patchset, xpath);
-
-    free(xpath);
-
-    if (xpath_obj == NULL) {
-        return false;
-    }
-    freeXpathObject(xpath_obj);
-    return true;
-}
-
-/*!
- * \internal
- * \brief Check whether a given CIB element was modified in a CIB patchset (v2)
- *
- * \param[in] patchset  CIB XML patchset
- * \param[in] element   XML tag of CIB element to check (\c NULL is equivalent
- *                      to \c PCMK_XE_CIB). Supported values include any CIB
- *                      element supported by \c pcmk__cib_abs_xpath_for().
- *
- * \return \c true if \p element was modified, or \c false otherwise
- */
-static bool
-element_in_patchset_v2(const xmlNode *patchset, const char *element)
-{
-    const char *element_xpath = pcmk__cib_abs_xpath_for(element);
-    const char *parent_xpath = pcmk_cib_parent_name_for(element);
-    char *element_regex = NULL;
-    bool rc = false;
-
-    CRM_CHECK(element_xpath != NULL, return false); // Unsupported element
-
-    // Matches if and only if element_xpath is part of a changed path
-    element_regex = crm_strdup_printf("^%s(/|$)", element_xpath);
-
-    for (const xmlNode *change = pcmk__xe_first_child(patchset, PCMK_XE_CHANGE,
-                                                      NULL, NULL);
-         change != NULL; change = pcmk__xe_next_same(change)) {
-
-        const char *op = crm_element_value(change, PCMK__XA_CIB_OP);
-        const char *diff_xpath = crm_element_value(change, PCMK_XA_PATH);
-
-        if (pcmk__str_eq(diff_xpath, element_regex, pcmk__str_regex)) {
-            // Change to an existing element
-            rc = true;
-            break;
-        }
-
-        if (pcmk__str_eq(op, PCMK_VALUE_CREATE, pcmk__str_none)
-            && pcmk__str_eq(diff_xpath, parent_xpath, pcmk__str_none)
-            && pcmk__xe_is(pcmk__xe_first_child(change, NULL, NULL, NULL),
-                                                element)) {
-
-            // Newly added element
-            rc = true;
-            break;
-        }
-    }
-
-    free(element_regex);
-    return rc;
-}
-
-/*!
- * \internal
- * \brief Check whether a given CIB element was modified in a CIB patchset
- *
- * \param[in] patchset  CIB XML patchset
- * \param[in] element   XML tag of CIB element to check (\c NULL is equivalent
- *                      to \c PCMK_XE_CIB). Supported values include any CIB
- *                      element supported by \c pcmk__cib_abs_xpath_for().
- *
- * \return \c true if \p element was modified, or \c false otherwise
- */
-bool
-cib__element_in_patchset(const xmlNode *patchset, const char *element)
-{
-    int format = 1;
-
-    CRM_ASSERT(patchset != NULL);
-
-    crm_element_value_int(patchset, PCMK_XA_FORMAT, &format);
-    switch (format) {
-        case 1:
-            return element_in_patchset_v1(patchset, element);
-
-        case 2:
-            return element_in_patchset_v2(patchset, element);
-
-        default:
-            crm_warn("Unknown patch format: %d", format);
-            return false;
-    }
-}
-
 /*!
  * \brief Create XML for a new (empty) CIB
  *
  * \param[in] cib_epoch  What to use as \c PCMK_XA_EPOCH CIB attribute
  *
  * \return Newly created XML for empty CIB
- * \note It is the caller's responsibility to free the result with free_xml().
+ *
+ * \note It is the caller's responsibility to free the result with
+ *       \c pcmk__xml_free().
  */
 xmlNode *
 createEmptyCib(int cib_epoch)
@@ -329,9 +219,10 @@ should_copy_cib(const char *op, const char *section, int call_options)
 }
 
 int
-cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
-               bool is_query, const char *section, xmlNode *req, xmlNode *input,
-               bool manage_counters, bool *config_changed, xmlNode **current_cib,
+cib_perform_op(cib_t *cib, const char *op, uint32_t call_options,
+               cib__op_fn_t fn, bool is_query, const char *section,
+               xmlNode *req, xmlNode *input, bool manage_counters,
+               bool *config_changed, xmlNode **current_cib,
                xmlNode **result_cib, xmlNode **diff, xmlNode **output)
 {
     int rc = pcmk_ok;
@@ -401,7 +292,7 @@ cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
             *output = pcmk__xml_copy(NULL, *output);
         }
 
-        free_xml(cib_filtered);
+        pcmk__xml_free(cib_filtered);
         return rc;
     }
 
@@ -466,8 +357,8 @@ cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
 
         rc = pcmk__check_feature_set(new_version);
         if (rc != pcmk_rc_ok) {
-            pcmk__config_err("Discarding update with feature set '%s' greater than our own '%s'",
-                             new_version, CRM_FEATURE_SET);
+            crm_err("Discarding update with feature set '%s' greater than "
+                    "our own '%s'", new_version, CRM_FEATURE_SET);
             rc = pcmk_rc2legacy(rc);
             goto done;
         }
@@ -503,17 +394,7 @@ cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
     crm_trace("Massaging CIB contents");
     pcmk__strip_xml_text(scratch);
 
-    if (!make_copy) {
-        /* At this point, patchset_cib is just the PCMK_XE_CIB tag and its
-         * properties.
-         *
-         * The v1 format would barf on this, but we know the v2 patch
-         * format only needs it for the top-level version fields
-         */
-        local_diff = xml_create_patchset(2, patchset_cib, scratch,
-                                         config_changed, manage_counters);
-
-    } else {
+    if (make_copy) {
         static time_t expires = 0;
         time_t tm_now = time(NULL);
 
@@ -521,11 +402,11 @@ cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
             expires = tm_now + 60;  /* Validate clients are correctly applying v2-style diffs at most once a minute */
             with_digest = true;
         }
-
-        local_diff = xml_create_patchset(0, patchset_cib, scratch,
-                                         config_changed, manage_counters);
     }
 
+    local_diff = xml_create_patchset(0, patchset_cib, scratch,
+                                     config_changed, manage_counters);
+
     pcmk__log_xml_changes(LOG_TRACE, scratch);
     xml_accept_changes(scratch);
 
@@ -558,7 +439,7 @@ cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
                             format, pcmk_rc_str(pcmk_legacy2rc(test_rc)),
                             test_rc);
                 }
-                free_xml(cib_copy);
+                pcmk__xml_free(cib_copy);
             },
             {}
         );
@@ -587,6 +468,10 @@ cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
     if (*config_changed && !pcmk_is_set(call_options, cib_no_mtime)) {
         const char *schema = crm_element_value(scratch, PCMK_XA_VALIDATE_WITH);
 
+        if (schema == NULL) {
+            rc = -pcmk_err_cib_corrupt;
+        }
+
         pcmk__xe_add_last_written(scratch);
         pcmk__warn_if_schema_deprecated(schema);
 
@@ -621,11 +506,6 @@ cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
     crm_trace("Perform validation: %s", pcmk__btoa(check_schema));
     if ((rc == pcmk_ok) && check_schema
         && !pcmk__configured_schema_validates(scratch)) {
-        const char *current_schema = crm_element_value(scratch,
-                                                       PCMK_XA_VALIDATE_WITH);
-
-        crm_warn("Updated CIB does not validate against %s schema",
-                 pcmk__s(current_schema, "unspecified"));
         rc = -pcmk_err_schema_validation;
     }
 
@@ -642,16 +522,16 @@ cib_perform_op(cib_t *cib, const char *op, int call_options, cib__op_fn_t fn,
         if (*result_cib == NULL) {
             crm_debug("Pre-filtered the entire cib result");
         }
-        free_xml(scratch);
+        pcmk__xml_free(scratch);
     }
 
     if(diff) {
         *diff = local_diff;
     } else {
-        free_xml(local_diff);
+        pcmk__xml_free(local_diff);
     }
 
-    free_xml(top);
+    pcmk__xml_free(top);
     crm_trace("Done");
     return rc;
 }
@@ -688,10 +568,6 @@ cib__create_op(cib_t *cib, const char *op, const char *host,
         pcmk__xml_copy(wrapper, data);
     }
 
-    if (pcmk_is_set(call_options, cib_inhibit_bcast)) {
-        CRM_CHECK(pcmk_is_set(call_options, cib_scope_local),
-                  free_xml(*op_msg); return -EPROTO);
-    }
     return pcmk_ok;
 }
 
@@ -744,7 +620,7 @@ cib__extend_transaction(cib_t *cib, xmlNode *request)
 {
     int rc = pcmk_rc_ok;
 
-    CRM_ASSERT((cib != NULL) && (request != NULL));
+    pcmk__assert((cib != NULL) && (request != NULL));
 
     rc = validate_transaction_request(request);
 
@@ -802,7 +678,7 @@ cib_native_callback(cib_t * cib, xmlNode * msg, int call_id, int rc)
                   pcmk__s(blob->id, "without ID"), call_id);
         blob->callback(msg, call_id, rc, output, blob->user_data);
 
-    } else if (cib && cib->op_callback == NULL && rc != pcmk_ok) {
+    } else if ((cib != NULL) && (rc != pcmk_ok)) {
         crm_warn("CIB command failed: %s", pcmk_strerror(rc));
         crm_log_xml_debug(msg, "Failed CIB Update");
     }
@@ -812,10 +688,6 @@ cib_native_callback(cib_t * cib, xmlNode * msg, int call_id, int rc)
         remove_cib_op_callback(call_id, FALSE);
     }
 
-    if (cib && cib->op_callback != NULL) {
-        crm_trace("Invoking global callback for call %d", call_id);
-        cib->op_callback(msg, call_id, rc, output);
-    }
     crm_trace("OP callback activated for %d", call_id);
 }
 
@@ -867,9 +739,9 @@ cib_read_config(GHashTable * options, xmlNode * current_cib)
 
     config = pcmk_find_cib_element(current_cib, PCMK_XE_CRM_CONFIG);
     if (config) {
-        pe_unpack_nvpairs(current_cib, config, PCMK_XE_CLUSTER_PROPERTY_SET,
-                          NULL, options, PCMK_VALUE_CIB_BOOTSTRAP_OPTIONS, TRUE,
-                          now, NULL);
+        pe_unpack_nvpairs(NULL, config, PCMK_XE_CLUSTER_PROPERTY_SET, NULL,
+                          options, PCMK_VALUE_CIB_BOOTSTRAP_OPTIONS, FALSE, now,
+                          NULL);
     }
 
     pcmk__validate_cluster_options(options);
@@ -884,15 +756,21 @@ cib_internal_op(cib_t * cib, const char *op, const char *host,
                 const char *section, xmlNode * data,
                 xmlNode ** output_data, int call_options, const char *user_name)
 {
-    int (*delegate) (cib_t * cib, const char *op, const char *host,
-                     const char *section, xmlNode * data,
-                     xmlNode ** output_data, int call_options, const char *user_name) =
-        cib->delegate_fn;
+    int (*delegate)(cib_t *cib, const char *op, const char *host,
+                    const char *section, xmlNode *data, xmlNode **output_data,
+                    int call_options, const char *user_name) = NULL;
 
-    if(user_name == NULL) {
-        user_name = getenv("CIB_user");
+    if (cib == NULL) {
+        return -EINVAL;
     }
 
+    delegate = cib->delegate_fn;
+    if (delegate == NULL) {
+        return -EPROTONOSUPPORT;
+    }
+    if (user_name == NULL) {
+        user_name = getenv("CIB_user");
+    }
     return delegate(cib, op, host, section, data, output_data, call_options, user_name);
 }
 
@@ -916,9 +794,7 @@ cib_apply_patch_event(xmlNode *event, xmlNode *input, xmlNode **output,
     xmlNode *wrapper = NULL;
     xmlNode *diff = NULL;
 
-    CRM_ASSERT(event);
-    CRM_ASSERT(input);
-    CRM_ASSERT(output);
+    pcmk__assert((event != NULL) && (input != NULL) && (output != NULL));
 
     crm_element_value_int(event, PCMK__XA_CIB_RC, &rc);
     wrapper = pcmk__xe_first_child(event, PCMK__XE_CIB_UPDATE_RESULT, NULL,
@@ -945,7 +821,7 @@ cib_apply_patch_event(xmlNode *event, xmlNode *input, xmlNode **output,
                 crm_trace("Masking error, we already have the supplied update");
                 return pcmk_ok;
             }
-            free_xml(*output);
+            pcmk__xml_free(*output);
             *output = NULL;
             return rc;
         }
@@ -967,7 +843,7 @@ cib__signon_query(pcmk__output_t *out, cib_t **cib, xmlNode **cib_object)
     int rc = pcmk_rc_ok;
     cib_t *cib_conn = NULL;
 
-    CRM_ASSERT(cib_object != NULL);
+    pcmk__assert(cib_object != NULL);
 
     if (cib == NULL) {
         cib_conn = cib_new();
@@ -996,8 +872,7 @@ cib__signon_query(pcmk__output_t *out, cib_t **cib, xmlNode **cib_object)
     if (out != NULL) {
         out->transient(out, "Querying CIB...");
     }
-    rc = cib_conn->cmds->query(cib_conn, NULL, cib_object,
-                               cib_scope_local|cib_sync_call);
+    rc = cib_conn->cmds->query(cib_conn, NULL, cib_object, cib_sync_call);
     rc = pcmk_legacy2rc(rc);
 
     if (rc != pcmk_rc_ok) {
@@ -1016,8 +891,7 @@ done:
 }
 
 int
-cib__signon_attempts(cib_t *cib, const char *name, enum cib_conn_type type,
-                     int attempts)
+cib__signon_attempts(cib_t *cib, enum cib_conn_type type, int attempts)
 {
     int rc = pcmk_rc_ok;
 
@@ -1025,7 +899,7 @@ cib__signon_attempts(cib_t *cib, const char *name, enum cib_conn_type type,
               attempts, pcmk__plural_s(attempts));
 
     for (int remaining = attempts - 1; remaining >= 0; --remaining) {
-        rc = cib->cmds->signon(cib, name, type);
+        rc = cib->cmds->signon(cib, crm_system_name, type);
 
         if ((rc == pcmk_rc_ok)
             || (remaining == 0)
@@ -1056,71 +930,3 @@ cib__clean_up_connection(cib_t **cib)
     *cib = NULL;
     return pcmk_legacy2rc(rc);
 }
-
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-#include <crm/cib/util_compat.h>
-
-xmlNode *
-cib_get_generation(cib_t * cib)
-{
-    xmlNode *the_cib = NULL;
-    xmlNode *generation = pcmk__xe_create(NULL, PCMK__XE_GENERATION_TUPLE);
-
-    cib->cmds->query(cib, NULL, &the_cib, cib_scope_local | cib_sync_call);
-    if (the_cib != NULL) {
-        pcmk__xe_copy_attrs(generation, the_cib, pcmk__xaf_none);
-        free_xml(the_cib);
-    }
-
-    return generation;
-}
-
-const char *
-get_object_path(const char *object_type)
-{
-    return pcmk_cib_xpath_for(object_type);
-}
-
-const char *
-get_object_parent(const char *object_type)
-{
-    return pcmk_cib_parent_name_for(object_type);
-}
-
-xmlNode *
-get_object_root(const char *object_type, xmlNode *the_root)
-{
-    return pcmk_find_cib_element(the_root, object_type);
-}
-
-const char *
-cib_pref(GHashTable * options, const char *name)
-{
-    return pcmk__cluster_option(options, name);
-}
-
-void
-cib_metadata(void)
-{
-    pcmk__output_t *out = NULL;
-    int rc = pcmk__output_new(&out, "text", NULL, NULL);
-
-    if (rc != pcmk_rc_ok) {
-        crm_err("Unable to output metadata: %s", pcmk_rc_str(rc));
-        return;
-    }
-
-    pcmk__daemon_metadata(out, "pacemaker-based",
-                          "Cluster Information Base manager options",
-                          "Cluster options used by Pacemaker's Cluster "
-                          "Information Base manager",
-                          pcmk__opt_based);
-
-    out->finish(out, CRM_EX_OK, true, NULL);
-    pcmk__output_free(out);
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
