@@ -10,7 +10,7 @@
 #include <crm_internal.h>
 
 #include <arpa/inet.h>
-#include <inttypes.h>                   // PRIu64, PRIx32
+#include <inttypes.h>                   // PRIu64, etc.
 #include <netdb.h>
 #include <netinet/in.h>
 #include <stdbool.h>
@@ -50,15 +50,15 @@ static gboolean (*quorum_app_callback)(unsigned long long seq,
  * \note It is the caller's responsibility to free the result with free().
  */
 char *
-pcmk__corosync_uuid(const crm_node_t *node)
+pcmk__corosync_uuid(const pcmk__node_status_t *node)
 {
-    CRM_ASSERT(pcmk_get_cluster_layer() == pcmk_cluster_layer_corosync);
+    pcmk__assert(pcmk_get_cluster_layer() == pcmk_cluster_layer_corosync);
 
     if (node != NULL) {
-        if (node->id > 0) {
-            return crm_strdup_printf("%u", node->id);
+        if (node->cluster_layer_id > 0) {
+            return crm_strdup_printf("%" PRIu32, node->cluster_layer_id);
         } else {
-            crm_info("Node %s is not yet known by Corosync", node->uname);
+            crm_info("Node %s is not yet known by Corosync", node->name);
         }
     }
     return NULL;
@@ -270,23 +270,27 @@ quorum_notification_cb(quorum_handle_t handle, uint32_t quorate,
 {
     int i;
     GHashTableIter iter;
-    crm_node_t *node = NULL;
+    pcmk__node_status_t *node = NULL;
     static gboolean init_phase = TRUE;
 
-    if (quorate != crm_have_quorum) {
-        if (quorate) {
-            crm_notice("Quorum acquired " CRM_XS " membership=%" PRIu64 " members=%lu",
-                       ring_id, (long unsigned int)view_list_entries);
-        } else {
-            crm_warn("Quorum lost " CRM_XS " membership=%" PRIu64 " members=%lu",
-                     ring_id, (long unsigned int)view_list_entries);
-        }
-        crm_have_quorum = quorate;
+    bool is_quorate = (quorate != 0);
+    bool was_quorate = pcmk__cluster_has_quorum();
+
+    if (is_quorate && !was_quorate) {
+        crm_notice("Quorum acquired " QB_XS " membership=%" PRIu64
+                   " members=%" PRIu32,
+                   ring_id, view_list_entries);
+        pcmk__cluster_set_quorum(true);
+
+    } else if (!is_quorate && was_quorate) {
+        crm_warn("Quorum lost " QB_XS " membership=%" PRIu64 " members=" PRIu32,
+                 ring_id, view_list_entries);
+        pcmk__cluster_set_quorum(false);
 
     } else {
-        crm_info("Quorum %s " CRM_XS " membership=%" PRIu64 " members=%lu",
-                 (quorate? "retained" : "still lost"), ring_id,
-                 (long unsigned int)view_list_entries);
+        crm_info("Quorum %s " QB_XS " membership=%" PRIu64 " members=%" PRIu32,
+                 (is_quorate? "retained" : "still lost"), ring_id,
+                 view_list_entries);
     }
 
     if (view_list_entries == 0 && init_phase) {
@@ -296,11 +300,11 @@ quorum_notification_cb(quorum_handle_t handle, uint32_t quorate,
 
     init_phase = FALSE;
 
-    /* Reset last_seen for all cached nodes so we can tell which ones aren't
+    /* Reset membership_id for all cached nodes so we can tell which ones aren't
      * in the view list */
-    g_hash_table_iter_init(&iter, crm_peer_cache);
+    g_hash_table_iter_init(&iter, pcmk__peer_cache);
     while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &node)) {
-        node->last_seen = 0;
+        node->membership_id = 0;
     }
 
     /* Update the peer cache for each node in view list */
@@ -311,7 +315,7 @@ quorum_notification_cb(quorum_handle_t handle, uint32_t quorate,
 
         /* Get this node's peer cache entry (adding one if not already there) */
         node = pcmk__get_node(id, NULL, NULL, pcmk__node_search_cluster_member);
-        if (node->uname == NULL) {
+        if (node->name == NULL) {
             char *name = pcmk__corosync_name(0, id);
 
             crm_info("Obtaining name for new node %u", id);
@@ -320,15 +324,15 @@ quorum_notification_cb(quorum_handle_t handle, uint32_t quorate,
             free(name);
         }
 
-        /* Update the node state (including updating last_seen to ring_id) */
-        pcmk__update_peer_state(__func__, node, CRM_NODE_MEMBER, ring_id);
+        // Update the node state (including updating membership_id to ring_id)
+        pcmk__update_peer_state(__func__, node, PCMK_VALUE_MEMBER, ring_id);
     }
 
     /* Remove any peer cache entries we didn't update */
     pcmk__reap_unseen_nodes(ring_id);
 
     if (quorum_app_callback) {
-        quorum_app_callback(ring_id, quorate);
+        quorum_app_callback(ring_id, is_quorate);
     }
 }
 
@@ -425,7 +429,7 @@ pcmk__corosync_quorum_connect(gboolean (*dispatch)(unsigned long long,
         crm_warn("No quorum");
     }
     quorum_app_callback = dispatch;
-    crm_have_quorum = quorate;
+    pcmk__cluster_set_quorum(quorate != 0);
 
     rc = quorum_trackstart(pcmk_quorum_handle, CS_TRACK_CHANGES | CS_TRACK_CURRENT);
     if (rc != CS_OK) {
@@ -454,7 +458,6 @@ pcmk__corosync_quorum_connect(gboolean (*dispatch)(unsigned long long,
 int
 pcmk__corosync_connect(pcmk_cluster_t *cluster)
 {
-    crm_node_t *peer = NULL;
     const enum pcmk_cluster_layer cluster_layer = pcmk_get_cluster_layer();
     const char *cluster_layer_s = pcmk_cluster_layer_text(cluster_layer);
     int rc = pcmk_rc_ok;
@@ -462,7 +465,7 @@ pcmk__corosync_connect(pcmk_cluster_t *cluster)
     pcmk__cluster_init_node_caches();
 
     if (cluster_layer != pcmk_cluster_layer_corosync) {
-        crm_err("Invalid cluster layer: %s " CRM_XS " cluster_layer=%d",
+        crm_err("Invalid cluster layer: %s " QB_XS " cluster_layer=%d",
                 cluster_layer_s, cluster_layer);
         return EINVAL;
     }
@@ -474,22 +477,21 @@ pcmk__corosync_connect(pcmk_cluster_t *cluster)
     }
     crm_info("Connection to %s established", cluster_layer_s);
 
-    cluster->nodeid = pcmk__cpg_local_nodeid(0);
-    if (cluster->nodeid == 0) {
+    cluster->priv->node_id = pcmk__cpg_local_nodeid(0);
+    if (cluster->priv->node_id == 0) {
         crm_err("Could not determine local node ID");
         return ENXIO;
     }
 
-    cluster->uname = pcmk__cluster_node_name(0);
-    if (cluster->uname == NULL) {
+    cluster->priv->node_name = pcmk__cluster_node_name(0);
+    if (cluster->priv->node_name == NULL) {
         crm_err("Could not determine local node name");
         return ENXIO;
     }
 
     // Ensure local node always exists in peer cache
-    peer = pcmk__get_node(cluster->nodeid, cluster->uname, NULL,
-                          pcmk__node_search_cluster_member);
-    cluster->uuid = pcmk__corosync_uuid(peer);
+    pcmk__get_node(cluster->priv->node_id, cluster->priv->node_name, NULL,
+                   pcmk__node_search_cluster_member);
 
     return pcmk_rc_ok;
 }
@@ -525,20 +527,20 @@ pcmk__corosync_is_active(void)
  * \return \c true if \p node is an active Corosync peer, or \c false otherwise
  */
 bool
-pcmk__corosync_is_peer_active(const crm_node_t *node)
+pcmk__corosync_is_peer_active(const pcmk__node_status_t *node)
 {
     if (node == NULL) {
         crm_trace("Corosync peer inactive: NULL");
         return false;
     }
-    if (!pcmk__str_eq(node->state, CRM_NODE_MEMBER, pcmk__str_none)) {
+    if (!pcmk__str_eq(node->state, PCMK_VALUE_MEMBER, pcmk__str_none)) {
         crm_trace("Corosync peer %s inactive: state=%s",
-                  node->uname, node->state);
+                  node->name, node->state);
         return false;
     }
     if (!pcmk_is_set(node->processes, crm_proc_cpg)) {
-        crm_trace("Corosync peer %s inactive " CRM_XS " processes=%.16" PRIx32,
-                  node->uname, node->processes);
+        crm_trace("Corosync peer %s inactive " QB_XS " processes=%.16" PRIx32,
+                  node->name, node->processes);
         return false;
     }
     return true;
@@ -621,16 +623,19 @@ pcmk__corosync_add_nodes(xmlNode *xml_parent)
         name = pcmk__corosync_name(cmap_handle, nodeid);
         if (name != NULL) {
             GHashTableIter iter;
-            crm_node_t *node = NULL;
+            pcmk__node_status_t *node = NULL;
 
-            g_hash_table_iter_init(&iter, crm_peer_cache);
+            g_hash_table_iter_init(&iter, pcmk__peer_cache);
             while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &node)) {
-                if(node && node->uname && strcasecmp(node->uname, name) == 0) {
-                    if (node->id && node->id != nodeid) {
-                        crm_crit("Nodes %u and %u share the same name '%s': shutting down", node->id,
-                                 nodeid, name);
-                        crm_exit(CRM_EX_FATAL);
-                    }
+                if ((node != NULL)
+                    && (node->cluster_layer_id > 0)
+                    && (node->cluster_layer_id != nodeid)
+                    && pcmk__str_eq(node->name, name, pcmk__str_casei)) {
+
+                    crm_crit("Nodes %" PRIu32 " and %" PRIu32 " share the "
+                             "same name '%s': shutting down",
+                             node->cluster_layer_id, nodeid, name);
+                    crm_exit(CRM_EX_FATAL);
                 }
             }
         }
@@ -756,7 +761,7 @@ pcmk__corosync_has_nodelist(void)
     } while ((retries < 5) && (cs_rc != CS_OK));
     if (cs_rc != CS_OK) {
         crm_warn("Assuming Corosync does not have node list: "
-                 "CMAP connection failed (%s) " CRM_XS " rc=%d",
+                 "CMAP connection failed (%s) " QB_XS " rc=%d",
                  cs_strerror(cs_rc), cs_rc);
         return false;
     }
@@ -765,7 +770,7 @@ pcmk__corosync_has_nodelist(void)
     cs_rc = cmap_fd_get(cmap_handle, &fd);
     if (cs_rc != CS_OK) {
         crm_warn("Assuming Corosync does not have node list: "
-                 "CMAP unusable (%s) " CRM_XS " rc=%d",
+                 "CMAP unusable (%s) " QB_XS " rc=%d",
                  cs_strerror(cs_rc), cs_rc);
         goto bail;
     }
@@ -776,13 +781,13 @@ pcmk__corosync_has_nodelist(void)
     if (rc == 0) {
         crm_warn("Assuming Corosync does not have node list: "
                  "CMAP provider is inauthentic "
-                 CRM_XS " pid=%lld uid=%lld gid=%lld",
+                 QB_XS " pid=%lld uid=%lld gid=%lld",
                  (long long) PCMK__SPECIAL_PID_AS_0(found_pid),
                  (long long) found_uid, (long long) found_gid);
         goto bail;
     } else if (rc < 0) {
         crm_warn("Assuming Corosync does not have node list: "
-                 "Could not verify CMAP authenticity (%s) " CRM_XS " rc=%d",
+                 "Could not verify CMAP authenticity (%s) " QB_XS " rc=%d",
                   pcmk_strerror(rc), rc);
         goto bail;
     }
@@ -791,7 +796,7 @@ pcmk__corosync_has_nodelist(void)
     cs_rc = cmap_iter_init(cmap_handle, "nodelist", &iter_handle);
     if (cs_rc != CS_OK) {
         crm_warn("Assuming Corosync does not have node list: "
-                 "CMAP not readable (%s) " CRM_XS " rc=%d",
+                 "CMAP not readable (%s) " QB_XS " rc=%d",
                  cs_strerror(cs_rc), cs_rc);
         goto bail;
     }
@@ -809,17 +814,3 @@ bail:
     cmap_finalize(cmap_handle);
     return result;
 }
-
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-#include <crm/cluster/compat.h>
-
-gboolean
-crm_is_corosync_peer_active(const crm_node_t *node)
-{
-    return pcmk__corosync_is_peer_active(node);
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
