@@ -46,11 +46,16 @@ static bool cpg_evicted = false;
 static GList *cs_message_queue = NULL;
 static int cs_message_timer = 0;
 
+/* @COMPAT Any changes to these structs (other than renames) will break all
+ * rolling upgrades, and should be avoided if possible or done at a major
+ * version bump if not
+ */
+
 struct pcmk__cpg_host_s {
     uint32_t id;
     uint32_t pid;
-    gboolean local;
-    enum crm_ais_msg_types type;
+    gboolean local;             // Unused but needed for compatibility
+    enum pcmk_ipc_server type;  // For logging only
     uint32_t size;
     char uname[MAX_NAME];
 } __attribute__ ((packed));
@@ -252,7 +257,7 @@ crm_cs_flush(gpointer data)
             /* Proportionally more if sending failed but cap at 1s */
             delay_ms = QB_MIN(1000, CS_SEND_MAX + (10 * queue_len));
         }
-        cs_message_timer = g_timeout_add(delay_ms, crm_cs_flush_cb, data);
+        cs_message_timer = pcmk__create_timer(delay_ms, crm_cs_flush_cb, data);
     }
 }
 
@@ -270,12 +275,12 @@ pcmk_cpg_dispatch(gpointer user_data)
     cs_error_t rc = CS_OK;
     pcmk_cluster_t *cluster = (pcmk_cluster_t *) user_data;
 
-    rc = cpg_dispatch(cluster->cpg_handle, CS_DISPATCH_ONE);
+    rc = cpg_dispatch(cluster->priv->cpg_handle, CS_DISPATCH_ONE);
     if (rc != CS_OK) {
         crm_err("Connection to the CPG API failed: %s (%d)",
                 pcmk__cs_err_str(rc), rc);
-        cpg_finalize(cluster->cpg_handle);
-        cluster->cpg_handle = 0;
+        cpg_finalize(cluster->priv->cpg_handle);
+        cluster->priv->cpg_handle = 0;
         return -1;
 
     } else if (cpg_evicted) {
@@ -288,53 +293,15 @@ pcmk_cpg_dispatch(gpointer user_data)
 static inline const char *
 ais_dest(const pcmk__cpg_host_t *host)
 {
-    if (host->local) {
-        return "local";
-    } else if (host->size > 0) {
-        return host->uname;
-    } else {
-        return "<all>";
-    }
+    return (host->size > 0)? host->uname : "<all>";
 }
 
 static inline const char *
-msg_type2text(enum crm_ais_msg_types type)
+msg_type2text(enum pcmk_ipc_server type)
 {
-    const char *text = "unknown";
+    const char *name = pcmk__server_message_type(type);
 
-    switch (type) {
-        case crm_msg_none:
-            text = "unknown";
-            break;
-        case crm_msg_ais:
-            text = "ais";
-            break;
-        case crm_msg_cib:
-            text = "cib";
-            break;
-        case crm_msg_crmd:
-            text = "crmd";
-            break;
-        case crm_msg_pe:
-            text = "pengine";
-            break;
-        case crm_msg_te:
-            text = "tengine";
-            break;
-        case crm_msg_lrmd:
-            text = "lrmd";
-            break;
-        case crm_msg_attrd:
-            text = "attrd";
-            break;
-        case crm_msg_stonithd:
-            text = "stonithd";
-            break;
-        case crm_msg_stonith_ng:
-            text = "stonith-ng";
-            break;
-    }
-    return text;
+    return pcmk__s(name, "unknown");
 }
 
 /*!
@@ -353,7 +320,7 @@ check_message_sanity(const pcmk__cpg_msg_t *msg)
     if (payload_size < 1) {
         crm_err("%sCPG message %d from %s invalid: "
                 "Claimed size of %d bytes is too small "
-                CRM_XS " from %s[%u] to %s@%s",
+                QB_XS " from %s[%u] to %s@%s",
                 (msg->is_compressed? "Compressed " : ""),
                 msg->id, ais_dest(&(msg->sender)),
                 (int) msg->header.size,
@@ -365,7 +332,7 @@ check_message_sanity(const pcmk__cpg_msg_t *msg)
     if (msg->header.error != CS_OK) {
         crm_err("%sCPG message %d from %s invalid: "
                 "Sender indicated error %d "
-                CRM_XS " from %s[%u] to %s@%s",
+                QB_XS " from %s[%u] to %s@%s",
                 (msg->is_compressed? "Compressed " : ""),
                 msg->id, ais_dest(&(msg->sender)),
                 msg->header.error,
@@ -377,7 +344,7 @@ check_message_sanity(const pcmk__cpg_msg_t *msg)
     if (msg_data_len(msg) != payload_size) {
         crm_err("%sCPG message %d from %s invalid: "
                 "Total size %d inconsistent with payload size %d "
-                CRM_XS " from %s[%u] to %s@%s",
+                QB_XS " from %s[%u] to %s@%s",
                 (msg->is_compressed? "Compressed " : ""),
                 msg->id, ais_dest(&(msg->sender)),
                 (int) msg->header.size, (int) msg_data_len(msg),
@@ -394,7 +361,7 @@ check_message_sanity(const pcmk__cpg_msg_t *msg)
          || (msg->data[msg->size - 1] != '\0'))) {
         crm_err("CPG message %d from %s invalid: "
                 "Payload does not end at byte %llu "
-                CRM_XS " from %s[%u] to %s@%s",
+                QB_XS " from %s[%u] to %s@%s",
                 msg->id, ais_dest(&(msg->sender)),
                 (unsigned long long) msg->size,
                 msg_type2text(msg->sender.type), msg->sender.pid,
@@ -418,126 +385,116 @@ check_message_sanity(const pcmk__cpg_msg_t *msg)
  * \param[in]     sender_id  Corosync ID of node that sent message
  * \param[in]     pid        Process ID of message sender (for logging only)
  * \param[in,out] content    CPG message
- * \param[out]    kind       If not \c NULL, will be set to CPG header ID
- *                           (which should be an <tt>enum crm_ais_msg_class</tt>
- *                           value, currently always \c crm_class_cluster)
  * \param[out]    from       If not \c NULL, will be set to sender uname
  *                           (valid for the lifetime of \p content)
  *
- * \return Newly allocated string with message data
+ * \return Newly allocated string with message data, or NULL for errors and
+ *         messages not intended for the local node
  *
  * \note The caller is responsible for freeing the return value using \c free().
  */
 char *
 pcmk__cpg_message_data(cpg_handle_t handle, uint32_t sender_id, uint32_t pid,
-                       void *content, uint32_t *kind, const char **from)
+                       void *content, const char **from)
 {
     char *data = NULL;
     pcmk__cpg_msg_t *msg = content;
 
+    if (from != NULL) {
+        *from = NULL;
+    }
+
     if (handle != 0) {
-        // Do filtering and field massaging
         uint32_t local_nodeid = pcmk__cpg_local_nodeid(handle);
         const char *local_name = pcmk__cluster_local_node_name();
 
-        if ((msg->sender.id != 0) && (msg->sender.id != sender_id)) {
-            crm_err("Nodeid mismatch from %" PRIu32 ".%" PRIu32
-                    ": claimed nodeid=%" PRIu32,
+        // Update or validate message sender ID
+        if (msg->sender.id == 0) {
+            msg->sender.id = sender_id;
+        } else if (msg->sender.id != sender_id) {
+            crm_warn("Ignoring CPG message from ID %" PRIu32 " PID %" PRIu32
+                     ": claimed ID %" PRIu32,
                     sender_id, pid, msg->sender.id);
             return NULL;
         }
+
+        // Ignore messages that aren't for the local node
         if ((msg->host.id != 0) && (local_nodeid != msg->host.id)) {
-            crm_trace("Not for us: %" PRIu32" != %" PRIu32,
-                      msg->host.id, local_nodeid);
+            crm_trace("Ignoring CPG message from ID %" PRIu32 " PID %" PRIu32
+                      ": for ID %" PRIu32 " not %" PRIu32,
+                      sender_id, pid, msg->host.id, local_nodeid);
             return NULL;
         }
         if ((msg->host.size > 0)
             && !pcmk__str_eq(msg->host.uname, local_name, pcmk__str_casei)) {
 
-            crm_trace("Not for us: %s != %s", msg->host.uname, local_name);
+            crm_trace("Ignoring CPG message from ID %" PRIu32 " PID %" PRIu32
+                      ": for name %s not %s",
+                      sender_id, pid, msg->host.uname, local_name);
             return NULL;
         }
 
-        msg->sender.id = sender_id;
+        // Add sender name if not in original message
         if (msg->sender.size == 0) {
-            const crm_node_t *peer =
+            const pcmk__node_status_t *peer =
                 pcmk__get_node(sender_id, NULL, NULL,
                                pcmk__node_search_cluster_member);
 
-            if (peer->uname == NULL) {
-                crm_err("No uname for peer with nodeid=%u", sender_id);
-
+            if (peer->name == NULL) {
+                crm_debug("Received CPG message from node with ID %" PRIu32
+                          " but its name is unknown", sender_id);
             } else {
-                crm_notice("Fixing uname for peer with nodeid=%u", sender_id);
-                msg->sender.size = strlen(peer->uname);
+                crm_debug("Updating name of CPG message sender with ID %" PRIu32
+                          " to %s", sender_id, peer->name);
+                msg->sender.size = strlen(peer->name);
                 memset(msg->sender.uname, 0, MAX_NAME);
-                memcpy(msg->sender.uname, peer->uname, msg->sender.size);
+                memcpy(msg->sender.uname, peer->name, msg->sender.size);
             }
         }
     }
 
-    crm_trace("Got new%s message (size=%d, %d, %d)",
-              msg->is_compressed ? " compressed" : "",
-              msg_data_len(msg), msg->size, msg->compressed_size);
+    // Ensure sender is in peer cache (though it should already be)
+    pcmk__get_node(msg->sender.id, msg->sender.uname, NULL,
+                   pcmk__node_search_cluster_member);
 
-    if (kind != NULL) {
-        *kind = msg->header.id;
-    }
     if (from != NULL) {
         *from = msg->sender.uname;
     }
 
-    if (msg->is_compressed && (msg->size > 0)) {
-        int rc = BZ_OK;
-        char *uncompressed = NULL;
-        unsigned int new_size = msg->size + 1;
-
-        if (!check_message_sanity(msg)) {
-            goto badmsg;
-        }
-
-        crm_trace("Decompressing message data");
-        uncompressed = pcmk__assert_alloc(1, new_size);
-        rc = BZ2_bzBuffToBuffDecompress(uncompressed, &new_size, msg->data,
-                                        msg->compressed_size, 1, 0);
-
-        rc = pcmk__bzlib2rc(rc);
-
-        if (rc != pcmk_rc_ok) {
-            crm_err("Decompression failed: %s " CRM_XS " rc=%d",
-                    pcmk_rc_str(rc), rc);
-            free(uncompressed);
-            goto badmsg;
-        }
-
-        CRM_ASSERT(new_size == msg->size);
-
-        data = uncompressed;
-
-    } else if (!check_message_sanity(msg)) {
-        goto badmsg;
-
-    } else {
-        data = strdup(msg->data);
+    if (!check_message_sanity(msg)) {
+        return NULL;
     }
 
-    // Is this necessary?
-    pcmk__get_node(msg->sender.id, msg->sender.uname, NULL,
-                   pcmk__node_search_cluster_member);
+    if (msg->is_compressed && (msg->size > 0)) {
+        int rc = BZ_OK;
+        unsigned int new_size = msg->size + 1;
+        char *uncompressed = pcmk__assert_alloc(1, new_size);
 
-    crm_trace("Payload: %.200s", data);
+        rc = BZ2_bzBuffToBuffDecompress(uncompressed, &new_size, msg->data,
+                                        msg->compressed_size, 1, 0);
+        rc = pcmk__bzlib2rc(rc);
+        if ((rc == pcmk_rc_ok) && (msg->size != new_size)) { // libbz2 bug?
+            rc = pcmk_rc_compression;
+        }
+        if (rc != pcmk_rc_ok) {
+            free(uncompressed);
+            crm_warn("Ignoring compressed CPG message %d from %s (ID %" PRIu32
+                    " PID %" PRIu32 "): %s",
+                     msg->id, ais_dest(&(msg->sender)), sender_id, pid,
+                     pcmk_rc_str(rc));
+            return NULL;
+        }
+        data = uncompressed;
+
+    } else {
+        data = pcmk__str_copy(msg->data);
+    }
+
+    crm_trace("Received %sCPG message %d from %s (ID %" PRIu32
+              " PID %" PRIu32 "): %.40s...",
+              (msg->is_compressed? "compressed " : ""),
+              msg->id, ais_dest(&(msg->sender)), sender_id, pid, msg->data);
     return data;
-
-  badmsg:
-    crm_err("Invalid message (id=%d, dest=%s:%s, from=%s:%s.%d):"
-            " min=%d, total=%d, size=%d, bz2_size=%d",
-            msg->id, ais_dest(&(msg->host)), msg_type2text(msg->host.type),
-            ais_dest(&(msg->sender)), msg_type2text(msg->sender.type),
-            msg->sender.pid, (int)sizeof(pcmk__cpg_msg_t),
-            msg->header.size, msg->size, msg->compressed_size);
-
-    free(data);
-    return NULL;
 }
 
 /*!
@@ -596,15 +553,9 @@ cpgreason2str(cpg_reason_t reason)
  * \return Node's uname, or readable string if not known
  */
 static inline const char *
-peer_name(const crm_node_t *peer)
+peer_name(const pcmk__node_status_t *peer)
 {
-    if (peer == NULL) {
-        return "unknown node";
-    } else if (peer->uname == NULL) {
-        return "peer node";
-    } else {
-        return peer->uname;
-    }
+    return (peer != NULL)? pcmk__s(peer->name, "peer node") : "unknown node";
 }
 
 /*!
@@ -624,7 +575,7 @@ node_left(const char *cpg_group_name, int event_counter,
           const struct cpg_address **sorted_member_list,
           size_t member_list_entries)
 {
-    crm_node_t *peer =
+    pcmk__node_status_t *peer =
         pcmk__search_node_caches(cpg_peer->nodeid, NULL,
                                  pcmk__node_search_cluster_member);
     const struct cpg_address **rival = NULL;
@@ -726,8 +677,9 @@ pcmk__cpg_confchg_cb(cpg_handle_t handle,
     }
 
     for (int i = 0; i < member_list_entries; i++) {
-        crm_node_t *peer = pcmk__get_node(member_list[i].nodeid, NULL, NULL,
-                                          pcmk__node_search_cluster_member);
+        pcmk__node_status_t *peer =
+            pcmk__get_node(member_list[i].nodeid, NULL, NULL,
+                           pcmk__node_search_cluster_member);
 
         if (member_list[i].nodeid == local_nodeid
                 && member_list[i].pid != getpid()) {
@@ -746,7 +698,7 @@ pcmk__cpg_confchg_cb(cpg_handle_t handle,
         peer = crm_update_peer_proc(__func__, peer, crm_proc_cpg,
                                     PCMK_VALUE_ONLINE);
 
-        if (peer && peer->state && strcmp(peer->state, CRM_NODE_MEMBER)) {
+        if (peer && peer->state && strcmp(peer->state, PCMK_VALUE_MEMBER)) {
             /* The node is a CPG member, but we currently think it's not a
              * cluster member. This is possible only if auto-reaping was
              * disabled. The node may be joining, and we happened to get the CPG
@@ -765,7 +717,7 @@ pcmk__cpg_confchg_cb(cpg_handle_t handle,
                 crm_warn("Node %u is member of group %s but was believed "
                          "offline",
                          member_list[i].nodeid, group_name->value);
-                pcmk__update_peer_state(__func__, peer, CRM_NODE_MEMBER, 0);
+                pcmk__update_peer_state(__func__, peer, PCMK_VALUE_MEMBER, 0);
             }
         }
 
@@ -832,9 +784,9 @@ pcmk__cpg_connect(pcmk_cluster_t *cluster)
     int fd = -1;
     int retries = 0;
     uint32_t id = 0;
-    crm_node_t *peer = NULL;
+    pcmk__node_status_t *peer = NULL;
     cpg_handle_t handle = 0;
-    const char *message_name = pcmk__message_name(crm_system_name);
+    const char *cpg_group_name = NULL;
     uid_t found_uid = 0;
     gid_t found_gid = 0;
     pid_t found_pid = 0;
@@ -854,13 +806,18 @@ pcmk__cpg_connect(pcmk_cluster_t *cluster)
     };
 
     cpg_evicted = false;
-    cluster->group.length = 0;
-    cluster->group.value[0] = 0;
 
-    /* group.value is char[128] */
-    strncpy(cluster->group.value, message_name, 127);
-    cluster->group.value[127] = 0;
-    cluster->group.length = 1 + QB_MIN(127, strlen(cluster->group.value));
+    cpg_group_name = pcmk__server_message_type(cluster->priv->server);
+    if (cpg_group_name == NULL) {
+        /* The name will already be non-NULL for Pacemaker servers. If a
+         * command-line tool or external caller connects to the cluster,
+         * they will join this CPG group.
+         */
+        cpg_group_name = pcmk__s(crm_system_name, "unknown");
+    }
+    memset(cluster->priv->group.value, 0, 128);
+    strncpy(cluster->priv->group.value, cpg_group_name, 127);
+    cluster->priv->group.length = strlen(cluster->priv->group.value) + 1;
 
     cs_repeat(rc, retries, 30, cpg_model_initialize(&handle, CPG_MODEL_V1, (cpg_model_data_t *)&cpg_model_info, NULL));
     if (rc != CS_OK) {
@@ -898,17 +855,17 @@ pcmk__cpg_connect(pcmk_cluster_t *cluster)
         goto bail;
 
     }
-    cluster->nodeid = id;
+    cluster->priv->node_id = id;
 
     retries = 0;
-    cs_repeat(rc, retries, 30, cpg_join(handle, &cluster->group));
+    cs_repeat(rc, retries, 30, cpg_join(handle, &cluster->priv->group));
     if (rc != CS_OK) {
-        crm_err("Could not join the CPG group '%s': %d", message_name, rc);
+        crm_err("Could not join the CPG group '%s': %d", cpg_group_name, rc);
         goto bail;
     }
 
     pcmk_cpg_handle = handle;
-    cluster->cpg_handle = handle;
+    cluster->priv->cpg_handle = handle;
     mainloop_add_fd("corosync-cpg", G_PRIORITY_MEDIUM, fd, cluster, &cpg_fd_callbacks);
 
   bail:
@@ -933,11 +890,11 @@ void
 pcmk__cpg_disconnect(pcmk_cluster_t *cluster)
 {
     pcmk_cpg_handle = 0;
-    if (cluster->cpg_handle != 0) {
+    if (cluster->priv->cpg_handle != 0) {
         crm_trace("Disconnecting CPG");
-        cpg_leave(cluster->cpg_handle, &cluster->group);
-        cpg_finalize(cluster->cpg_handle);
-        cluster->cpg_handle = 0;
+        cpg_leave(cluster->priv->cpg_handle, &cluster->priv->group);
+        cpg_finalize(cluster->priv->cpg_handle);
+        cluster->priv->cpg_handle = 0;
 
     } else {
         crm_info("No CPG connection");
@@ -949,17 +906,15 @@ pcmk__cpg_disconnect(pcmk_cluster_t *cluster)
  * \brief Send string data via Corosync CPG
  *
  * \param[in] data   Data to send
- * \param[in] local  What to set as host "local" value (which is never used)
  * \param[in] node   Cluster node to send message to
  * \param[in] dest   Type of message to send
  *
  * \return \c true on success, or \c false otherwise
  */
 static bool
-send_cpg_text(const char *data, bool local, const crm_node_t *node,
-              enum crm_ais_msg_types dest)
+send_cpg_text(const char *data, const pcmk__node_status_t *node,
+              enum pcmk_ipc_server dest)
 {
-    // @COMPAT Drop local argument when send_cluster_text is dropped
     static int msg_id = 0;
     static int local_pid = 0;
     static int local_name_len = 0;
@@ -969,8 +924,6 @@ send_cpg_text(const char *data, bool local, const crm_node_t *node,
     struct iovec *iov;
     pcmk__cpg_msg_t *msg = NULL;
 
-    CRM_CHECK(dest != crm_msg_ais, return false);
-
     if (local_name == NULL) {
         local_name = pcmk__cluster_local_node_name();
     }
@@ -990,30 +943,28 @@ send_cpg_text(const char *data, bool local, const crm_node_t *node,
 
     msg_id++;
     msg->id = msg_id;
-    msg->header.id = crm_class_cluster;
     msg->header.error = CS_OK;
 
     msg->host.type = dest;
-    msg->host.local = local;
 
     if (node != NULL) {
-        if (node->uname != NULL) {
-            target = pcmk__str_copy(node->uname);
-            msg->host.size = strlen(node->uname);
+        if (node->name != NULL) {
+            target = pcmk__str_copy(node->name);
+            msg->host.size = strlen(node->name);
             memset(msg->host.uname, 0, MAX_NAME);
-            memcpy(msg->host.uname, node->uname, msg->host.size);
+            memcpy(msg->host.uname, node->name, msg->host.size);
 
         } else {
-            target = crm_strdup_printf("%u", node->id);
+            target = crm_strdup_printf("%" PRIu32, node->cluster_layer_id);
         }
-        msg->host.id = node->id;
+        msg->host.id = node->cluster_layer_id;
 
     } else {
         target = pcmk__str_copy("all");
     }
 
     msg->sender.id = 0;
-    msg->sender.type = pcmk__cluster_parse_msg_type(crm_system_name);
+    msg->sender.type = pcmk__parse_server(crm_system_name);
     msg->sender.pid = local_pid;
     msg->sender.size = local_name_len;
     memset(msg->sender.uname, 0, MAX_NAME);
@@ -1088,117 +1039,15 @@ send_cpg_text(const char *data, bool local, const crm_node_t *node,
  * \return TRUE on success, otherwise FALSE
  */
 bool
-pcmk__cpg_send_xml(const xmlNode *msg, const crm_node_t *node,
-                   enum crm_ais_msg_types dest)
+pcmk__cpg_send_xml(const xmlNode *msg, const pcmk__node_status_t *node,
+                   enum pcmk_ipc_server dest)
 {
     bool rc = true;
     GString *data = g_string_sized_new(1024);
 
     pcmk__xml_string(msg, 0, data, 0);
 
-    rc = send_cpg_text(data->str, false, node, dest);
+    rc = send_cpg_text(data->str, node, dest);
     g_string_free(data, TRUE);
     return rc;
 }
-
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-#include <crm/cluster/compat.h>
-
-gboolean
-cluster_connect_cpg(pcmk_cluster_t *cluster)
-{
-    return pcmk__cpg_connect(cluster) == pcmk_rc_ok;
-}
-
-void
-cluster_disconnect_cpg(pcmk_cluster_t *cluster)
-{
-    pcmk__cpg_disconnect(cluster);
-}
-
-uint32_t
-get_local_nodeid(cpg_handle_t handle)
-{
-    return pcmk__cpg_local_nodeid(handle);
-}
-
-void
-pcmk_cpg_membership(cpg_handle_t handle,
-                    const struct cpg_name *group_name,
-                    const struct cpg_address *member_list,
-                    size_t member_list_entries,
-                    const struct cpg_address *left_list,
-                    size_t left_list_entries,
-                    const struct cpg_address *joined_list,
-                    size_t joined_list_entries)
-{
-    pcmk__cpg_confchg_cb(handle, group_name, member_list, member_list_entries,
-                         left_list, left_list_entries,
-                         joined_list, joined_list_entries);
-}
-
-gboolean
-send_cluster_text(enum crm_ais_msg_class msg_class, const char *data,
-                  gboolean local, const crm_node_t *node,
-                  enum crm_ais_msg_types dest)
-{
-    switch (msg_class) {
-        case crm_class_cluster:
-            return send_cpg_text(data, local, node, dest);
-        default:
-            crm_err("Invalid message class: %d", msg_class);
-            return FALSE;
-    }
-}
-
-char *
-pcmk_message_common_cs(cpg_handle_t handle, uint32_t nodeid, uint32_t pid,
-                       void *content, uint32_t *kind, const char **from)
-{
-    return pcmk__cpg_message_data(handle, nodeid, pid, content, kind, from);
-}
-
-enum crm_ais_msg_types
-text2msg_type(const char *text)
-{
-    int type = crm_msg_none;
-
-    CRM_CHECK(text != NULL, return type);
-    text = pcmk__message_name(text);
-    if (pcmk__str_eq(text, "ais", pcmk__str_casei)) {
-        type = crm_msg_ais;
-    } else if (pcmk__str_eq(text, CRM_SYSTEM_CIB, pcmk__str_casei)) {
-        type = crm_msg_cib;
-    } else if (pcmk__strcase_any_of(text, CRM_SYSTEM_CRMD, CRM_SYSTEM_DC, NULL)) {
-        type = crm_msg_crmd;
-    } else if (pcmk__str_eq(text, CRM_SYSTEM_TENGINE, pcmk__str_casei)) {
-        type = crm_msg_te;
-    } else if (pcmk__str_eq(text, CRM_SYSTEM_PENGINE, pcmk__str_casei)) {
-        type = crm_msg_pe;
-    } else if (pcmk__str_eq(text, CRM_SYSTEM_LRMD, pcmk__str_casei)) {
-        type = crm_msg_lrmd;
-    } else if (pcmk__str_eq(text, CRM_SYSTEM_STONITHD, pcmk__str_casei)) {
-        type = crm_msg_stonithd;
-    } else if (pcmk__str_eq(text, "stonith-ng", pcmk__str_casei)) {
-        type = crm_msg_stonith_ng;
-    } else if (pcmk__str_eq(text, "attrd", pcmk__str_casei)) {
-        type = crm_msg_attrd;
-
-    } else {
-        /* This will normally be a transient client rather than
-         * a cluster daemon.  Set the type to the pid of the client
-         */
-        int scan_rc = sscanf(text, "%d", &type);
-
-        if (scan_rc != 1 || type <= crm_msg_stonith_ng) {
-            /* Ensure it's sane */
-            type = crm_msg_none;
-        }
-    }
-    return type;
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
