@@ -55,15 +55,6 @@ create_acl(const xmlNode *xml, GList *acls, enum xml_private_flags mode)
     const char *xpath = crm_element_value(xml, PCMK_XA_XPATH);
     const char *attr = crm_element_value(xml, PCMK_XA_ATTRIBUTE);
 
-    if (tag == NULL) {
-        // @COMPAT Deprecated since 1.1.12 (needed for rolling upgrades)
-        tag = crm_element_value(xml, PCMK_XA_TAG);
-    }
-    if (ref == NULL) {
-        // @COMPAT Deprecated since 1.1.12 (needed for rolling upgrades)
-        ref = crm_element_value(xml, PCMK__XA_REF);
-    }
-
     if ((tag == NULL) && (ref == NULL) && (xpath == NULL)) {
         // Schema should prevent this, but to be safe ...
         crm_trace("Ignoring ACL <%s> element without selection criteria",
@@ -123,66 +114,59 @@ create_acl(const xmlNode *xml, GList *acls, enum xml_private_flags mode)
 static GList *
 parse_acl_entry(const xmlNode *acl_top, const xmlNode *acl_entry, GList *acls)
 {
-    xmlNode *child = NULL;
-
-    for (child = pcmk__xe_first_child(acl_entry, NULL, NULL, NULL);
-         child != NULL; child = pcmk__xe_next(child)) {
-
-        const char *tag = (const char *) child->name;
-        const char *kind = crm_element_value(child, PCMK_XA_KIND);
+    for (const xmlNode *child = pcmk__xe_first_child(acl_entry, NULL, NULL,
+                                                     NULL);
+         child != NULL; child = pcmk__xe_next(child, NULL)) {
 
         if (pcmk__xe_is(child, PCMK_XE_ACL_PERMISSION)) {
-            CRM_ASSERT(kind != NULL);
-            crm_trace("Unpacking ACL <%s> element of kind '%s'", tag, kind);
-            tag = kind;
-        } else {
-            crm_trace("Unpacking ACL <%s> element", tag);
-        }
+            const char *kind = crm_element_value(child, PCMK_XA_KIND);
 
-        /* @COMPAT PCMK__XE_ROLE_REF was deprecated in Pacemaker 1.1.12 (needed
-         * for rolling upgrades)
-         */
-        if (pcmk__str_any_of(tag, PCMK_XE_ROLE, PCMK__XE_ROLE_REF, NULL)) {
-            const char *ref_role = crm_element_value(child, PCMK_XA_ID);
+            pcmk__assert(kind != NULL);
+            crm_trace("Unpacking <" PCMK_XE_ACL_PERMISSION "> element of "
+                      "kind '%s'",
+                      kind);
 
-            if (ref_role) {
-                xmlNode *role = NULL;
+            if (pcmk__str_eq(kind, PCMK_VALUE_READ, pcmk__str_none)) {
+                acls = create_acl(child, acls, pcmk__xf_acl_read);
 
-                for (role = pcmk__xe_first_child(acl_top, NULL, NULL, NULL);
-                     role != NULL; role = pcmk__xe_next(role)) {
+            } else if (pcmk__str_eq(kind, PCMK_VALUE_WRITE, pcmk__str_none)) {
+                acls = create_acl(child, acls, pcmk__xf_acl_write);
 
-                    if (!strcmp(PCMK_XE_ACL_ROLE, (const char *) role->name)) {
-                        const char *role_id = crm_element_value(role,
-                                                                PCMK_XA_ID);
+            } else if (pcmk__str_eq(kind, PCMK_VALUE_DENY, pcmk__str_none)) {
+                acls = create_acl(child, acls, pcmk__xf_acl_deny);
 
-                        if (role_id && strcmp(ref_role, role_id) == 0) {
-                            crm_trace("Unpacking referenced role '%s' in ACL <%s> element",
-                                      role_id, acl_entry->name);
-                            acls = parse_acl_entry(acl_top, role, acls);
-                            break;
-                        }
-                    }
-                }
+            } else {
+                crm_warn("Ignoring unknown ACL kind '%s'", kind);
             }
 
-        /* @COMPAT Use of a tag instead of a PCMK_XA_KIND attribute was
-         * deprecated in 1.1.12. We still need to look for tags named
-         * PCMK_VALUE_READ, etc., to support rolling upgrades. However,
-         * eventually we can clean this up and make the variables more intuitive
-         * (for example, don't assign a PCMK_XA_KIND value to the tag variable).
-         */
-        } else if (strcmp(tag, PCMK_VALUE_READ) == 0) {
-            acls = create_acl(child, acls, pcmk__xf_acl_read);
+        } else if (pcmk__xe_is(child, PCMK_XE_ROLE)) {
+            const char *ref_role = crm_element_value(child, PCMK_XA_ID);
 
-        } else if (strcmp(tag, PCMK_VALUE_WRITE) == 0) {
-            acls = create_acl(child, acls, pcmk__xf_acl_write);
+            crm_trace("Unpacking <" PCMK_XE_ROLE "> element");
 
-        } else if (strcmp(tag, PCMK_VALUE_DENY) == 0) {
-            acls = create_acl(child, acls, pcmk__xf_acl_deny);
+            if (ref_role == NULL) {
+                continue;
+            }
 
-        } else {
-            crm_warn("Ignoring unknown ACL %s '%s'",
-                     (kind? "kind" : "element"), tag);
+            for (xmlNode *role = pcmk__xe_first_child(acl_top, NULL, NULL,
+                                                      NULL);
+                 role != NULL; role = pcmk__xe_next(role, NULL)) {
+
+                const char *role_id = NULL;
+
+                if (!pcmk__xe_is(role, PCMK_XE_ACL_ROLE)) {
+                    continue;
+                }
+
+                role_id = crm_element_value(role, PCMK_XA_ID);
+
+                if (pcmk__str_eq(ref_role, role_id, pcmk__str_none)) {
+                    crm_trace("Unpacking referenced role '%s' in <%s> element",
+                              role_id, acl_entry->name);
+                    acls = parse_acl_entry(acl_top, role, acls);
+                    break;
+                }
+            }
         }
     }
 
@@ -308,13 +292,9 @@ pcmk__unpack_acl(xmlNode *source, xmlNode *target, const char *user)
             xmlNode *child = NULL;
 
             for (child = pcmk__xe_first_child(acls, NULL, NULL, NULL);
-                 child != NULL; child = pcmk__xe_next(child)) {
+                 child != NULL; child = pcmk__xe_next(child, NULL)) {
 
-                /* @COMPAT PCMK__XE_ACL_USER was deprecated in Pacemaker 1.1.12
-                 * (needed for rolling upgrades)
-                 */
-                if (pcmk__xe_is(child, PCMK_XE_ACL_TARGET)
-                    || pcmk__xe_is(child, PCMK__XE_ACL_USER)) {
+                if (pcmk__xe_is(child, PCMK_XE_ACL_TARGET)) {
                     const char *id = crm_element_value(child, PCMK_XA_NAME);
 
                     if (id == NULL) {
@@ -413,7 +393,7 @@ purge_xml_attributes(xmlNode *xml)
             continue;
         }
 
-        xmlUnsetProp(xml, tmp->name);
+        pcmk__xa_remove(tmp, true);
     }
 
     child = pcmk__xml_first_child(xml);
@@ -425,7 +405,8 @@ purge_xml_attributes(xmlNode *xml)
     }
 
     if (!readable_children) {
-        free_xml(xml); /* Nothing readable under here, purge completely */
+        // Nothing readable under here, so purge completely
+        pcmk__xml_free(xml);
     }
     return readable_children;
 }
@@ -506,7 +487,7 @@ xml_acl_filtered_copy(const char *user, xmlNode *acl_source, xmlNode *xml,
     } else {
         crm_trace("User '%s' without ACLs denied access to entire XML document",
                   user);
-        free_xml(target);
+        pcmk__xml_free(target);
         target = NULL;
     }
 
@@ -541,7 +522,7 @@ implicitly_allowed(const xmlNode *xml)
     }
 
     path = pcmk__element_xpath(xml);
-    CRM_ASSERT(path != NULL);
+    pcmk__assert(path != NULL);
 
     if (strstr((const char *) path->str, "/" PCMK_XE_ACLS "/") != NULL) {
         g_string_free(path, TRUE);
@@ -585,9 +566,25 @@ pcmk__apply_creation_acl(xmlNode *xml, bool check_top)
                       xml->name, display_id(xml));
 
         } else if (check_top) {
-            crm_trace("ACLs disallow creation of <%s> with "
-                      PCMK_XA_ID "=\"%s\"", xml->name, display_id(xml));
-            pcmk_free_xml_subtree(xml);
+            /* is_root=true should be impossible with check_top=true, but check
+             * for sanity
+             */
+            bool is_root = (xmlDocGetRootElement(xml->doc) == xml);
+            xml_doc_private_t *docpriv = xml->doc->_private;
+
+            crm_trace("ACLs disallow creation of %s<%s> with "
+                      PCMK_XA_ID "=\"%s\"",
+                      (is_root? "root element " : ""), xml->name,
+                      display_id(xml));
+
+            // pcmk__xml_free() checks ACLs if enabled, which would fail
+            pcmk__clear_xml_flags(docpriv, pcmk__xf_acl_enabled);
+            pcmk__xml_free(xml);
+
+            if (!is_root) {
+                // If root, the document was freed. Otherwise re-enable ACLs.
+                pcmk__set_xml_flags(docpriv, pcmk__xf_acl_enabled);
+            }
             return;
 
         } else {
@@ -657,9 +654,8 @@ xml_acl_enabled(const xmlNode *xml)
 bool
 pcmk__check_acl(xmlNode *xml, const char *name, enum xml_private_flags mode)
 {
-    CRM_ASSERT(xml);
-    CRM_ASSERT(xml->doc);
-    CRM_ASSERT(xml->doc->_private);
+    pcmk__assert((xml != NULL) && (xml->doc != NULL)
+                 && (xml->doc->_private != NULL));
 
     if (pcmk__tracking_xml_changes(xml, false) && xml_acl_enabled(xml)) {
         xmlNode *parent = xml;
@@ -811,19 +807,22 @@ pcmk__update_acl_user(xmlNode *request, const char *field,
         }
     }
 
-    requested_user = crm_element_value(request, PCMK_XE_ACL_TARGET);
+    requested_user = crm_element_value(request, PCMK__XA_ACL_TARGET);
     if (requested_user == NULL) {
-        /* @COMPAT rolling upgrades <=1.1.11
+        /* Currently, different XML attribute names are used for the ACL user in
+         * different contexts (PCMK__XA_ATTR_USER, PCMK__XA_CIB_USER, etc.).
+         * The caller may specify that name as the field argument.
          *
-         * field is checked for backward compatibility with older versions that
-         * did not use PCMK_XE_ACL_TARGET.
+         * @TODO Standardize on PCMK__XA_ACL_TARGET and eventually drop the
+         * others once rolling upgrades from versions older than that are no
+         * longer supported.
          */
         requested_user = crm_element_value(request, field);
     }
 
     if (!pcmk__is_privileged(effective_user)) {
         /* We're not running as a privileged user, set or overwrite any existing
-         * value for PCMK_XE_ACL_TARGET
+         * value for PCMK__XA_ACL_TARGET
          */
         user = effective_user;
 
@@ -839,7 +838,7 @@ pcmk__update_acl_user(xmlNode *request, const char *field,
 
     } else if (!pcmk__is_privileged(peer_user)) {
         /* The peer is not a privileged user, set or overwrite any existing
-         * value for PCMK_XE_ACL_TARGET
+         * value for PCMK__XA_ACL_TARGET
          */
         user = peer_user;
 
@@ -853,8 +852,8 @@ pcmk__update_acl_user(xmlNode *request, const char *field,
     }
 
     // This requires pointer comparison, not string comparison
-    if (user != crm_element_value(request, PCMK_XE_ACL_TARGET)) {
-        crm_xml_add(request, PCMK_XE_ACL_TARGET, user);
+    if (user != crm_element_value(request, PCMK__XA_ACL_TARGET)) {
+        crm_xml_add(request, PCMK__XA_ACL_TARGET, user);
     }
 
     if (field != NULL && user != crm_element_value(request, field)) {
