@@ -7,8 +7,8 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef CRMCOMMON_PRIVATE__H
-#  define CRMCOMMON_PRIVATE__H
+#ifndef PCMK__COMMON_CRMCOMMON_PRIVATE__H
+#define PCMK__COMMON_CRMCOMMON_PRIVATE__H
 
 /* This header is for the sole use of libcrmcommon, so that functions can be
  * declared with G_GNUC_INTERNAL for efficiency.
@@ -17,10 +17,25 @@
 #include <stdint.h>         // uint8_t, uint32_t
 #include <stdbool.h>        // bool
 #include <sys/types.h>      // size_t
-#include <glib.h>           // gchar, GList
+
+#include <glib.h>           // G_GNUC_INTERNAL, G_GNUC_PRINTF, gchar, etc.
 #include <libxml/tree.h>    // xmlNode, xmlAttr
 #include <qb/qbipcc.h>      // struct qb_ipc_response_header
 
+#include <crm/common/ipc.h>             // pcmk_ipc_api_t, crm_ipc_t, etc.
+#include <crm/common/iso8601.h>         // crm_time_t
+#include <crm/common/logging.h>         // LOG_NEVER
+#include <crm/common/mainloop.h>        // mainloop_io_t
+#include <crm/common/output_internal.h> // pcmk__output_t
+#include <crm/common/results.h>         // crm_exit_t
+#include <crm/common/rules.h>           // pcmk_rule_input_t
+#include <crm/common/xml.h>             // pcmkXmlStr
+#include <crm/common/xml_internal.h>    // enum xml_private_flags
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 // Decent chunk size for processing large amounts of data
 #define PCMK__BUFFER_SIZE 4096
 
@@ -50,16 +65,16 @@ typedef struct xml_doc_private_s {
         GList *deleted_objs; // List of pcmk__deleted_xml_t
 } xml_doc_private_t;
 
-// XML entity references
+// XML private data magic numbers
+#define PCMK__XML_DOC_PRIVATE_MAGIC     0x81726354UL
+#define PCMK__XML_NODE_PRIVATE_MAGIC    0x54637281UL
 
+// XML entity references
 #define PCMK__XML_ENTITY_AMP    "&amp;"
 #define PCMK__XML_ENTITY_GT     "&gt;"
 #define PCMK__XML_ENTITY_LT     "&lt;"
 #define PCMK__XML_ENTITY_QUOT   "&quot;"
 
-//! libxml2 supports only XML version 1.0, at least as of libxml2-2.12.5
-#define PCMK__XML_VERSION ((pcmkXmlStr) "1.0")
-
 #define pcmk__set_xml_flags(xml_priv, flags_to_set) do {                    \
         (xml_priv)->flags = pcmk__set_flags_as(__func__, __LINE__,          \
             LOG_NEVER, "XML", "XML node", (xml_priv)->flags,                \
@@ -73,10 +88,25 @@ typedef struct xml_doc_private_s {
     } while (0)
 
 G_GNUC_INTERNAL
-bool pcmk__tracking_xml_changes(xmlNode *xml, bool lazy);
+bool pcmk__xml_reset_node_flags(xmlNode *xml, void *user_data);
 
 G_GNUC_INTERNAL
-void pcmk__xml_mark_created(xmlNode *xml);
+void pcmk__xml_set_parent_flags(xmlNode *xml, uint64_t flags);
+
+G_GNUC_INTERNAL
+void pcmk__xml_new_private_data(xmlNode *xml);
+
+G_GNUC_INTERNAL
+void pcmk__xml_free_private_data(xmlNode *xml);
+
+G_GNUC_INTERNAL
+void pcmk__xml_free_node(xmlNode *xml);
+
+G_GNUC_INTERNAL
+xmlDoc *pcmk__xml_new_doc(void);
+
+G_GNUC_INTERNAL
+bool pcmk__tracking_xml_changes(xmlNode *xml, bool lazy);
 
 G_GNUC_INTERNAL
 int pcmk__xml_position(const xmlNode *xml,
@@ -86,10 +116,6 @@ G_GNUC_INTERNAL
 xmlNode *pcmk__xml_match(const xmlNode *haystack, const xmlNode *needle,
                          bool exact);
 
-G_GNUC_INTERNAL
-void pcmk__xml_update(xmlNode *parent, xmlNode *target, xmlNode *update,
-                      uint32_t flags, bool as_diff);
-
 G_GNUC_INTERNAL
 xmlNode *pcmk__xc_match(const xmlNode *root, const xmlNode *search_comment,
                         bool exact);
@@ -112,6 +138,9 @@ void pcmk__apply_acl(xmlNode *xml);
 G_GNUC_INTERNAL
 void pcmk__apply_creation_acl(xmlNode *xml, bool check_top);
 
+G_GNUC_INTERNAL
+int pcmk__xa_remove(xmlAttr *attr, bool force);
+
 G_GNUC_INTERNAL
 void pcmk__mark_xml_attr_dirty(xmlAttr *a);
 
@@ -134,6 +163,12 @@ void pcmk__dump_xml_attr(const xmlAttr *attr, GString *buffer);
 G_GNUC_INTERNAL
 int pcmk__xe_set_score(xmlNode *target, const char *name, const char *value);
 
+G_GNUC_INTERNAL
+bool pcmk__xml_is_name_start_char(const char *utf8, int *len);
+
+G_GNUC_INTERNAL
+bool pcmk__xml_is_name_char(const char *utf8, int *len);
+
 /*
  * Date/times
  */
@@ -407,16 +442,23 @@ enum pcmk__schema_validator {
 typedef struct {
     int schema_index;
     char *name;
-    char *transform;
+
+    /*!
+     * List of XSLT stylesheets for upgrading from this schema version to the
+     * next one. Sorted by the order in which they should be applied to the CIB.
+     */
+    GList *transforms;
+
     void *cache;
     enum pcmk__schema_validator validator;
     pcmk__schema_version_t version;
-    char *transform_enter;
-    bool transform_onleave;
 } pcmk__schema_t;
 
 G_GNUC_INTERNAL
 GList *pcmk__find_x_0_schema(void);
 
+#ifdef __cplusplus
+}
+#endif
 
-#endif  // CRMCOMMON_PRIVATE__H
+#endif  // PCMK__COMMON_CRMCOMMON_PRIVATE__H
