@@ -9,11 +9,15 @@
 
 #include <crm_internal.h>
 
+#include <stdbool.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
-#include <md5.h>
+
+#include <glib.h>               // GString, etc.
+#include <gnutls/crypto.h>      // gnutls_hash_fast(), gnutls_hash_get_len()
+#include <gnutls/gnutls.h>      // gnutls_strerror()
 
 #include <crm/crm.h>
 #include <crm/common/xml.h>
@@ -21,6 +25,30 @@
 
 #define BEST_EFFORT_STATUS 0
 
+/*
+ * Pacemaker uses digests (MD5 hashes) of stringified XML to detect changes in
+ * the CIB as a whole, a particular resource's agent parameters, and the device
+ * parameters last used to unfence a particular node.
+ *
+ * "v2" digests hash pcmk__xml_string() directly, while less efficient "v1"
+ * digests do the same with a prefixed space, suffixed newline, and optional
+ * pre-sorting.
+ *
+ * On-disk CIB digests use v1 without sorting.
+ *
+ * Operation digests use v1 with sorting, and are stored in a resource's
+ * operation history in the CIB status section. They come in three flavors:
+ * - a digest of (nearly) all resource parameters and options, used to detect
+ *   any resource configuration change;
+ * - a digest of resource parameters marked as nonreloadable, used to decide
+ *   whether a reload or full restart is needed after a configuration change;
+ * - and a digest of resource parameters not marked as private, used in
+ *   simulations where private parameters have been removed from the input.
+ *
+ * Unfencing digests are set as node attributes, and are used to require
+ * that nodes be unfenced again after a device's configuration changes.
+ */
+
 /*!
  * \internal
  * \brief Dump XML in a format used with v1 digests
@@ -43,31 +71,23 @@ dump_xml_for_digest(xmlNodePtr xml)
 }
 
 /*!
+ * \internal
  * \brief Calculate and return v1 digest of XML tree
  *
- * \param[in] input Root of XML to digest
- * \param[in] sort Whether to sort the XML before calculating digest
- * \param[in] ignored Not used
+ * \param[in] input  Root of XML to digest
  *
  * \return Newly allocated string containing digest
+ *
  * \note Example return value: "c048eae664dba840e1d2060f00299e9d"
  */
 static char *
-calculate_xml_digest_v1(xmlNode *input, gboolean sort, gboolean ignored)
+calculate_xml_digest_v1(xmlNode *input)
 {
+    GString *buffer = dump_xml_for_digest(input);
     char *digest = NULL;
-    GString *buffer = NULL;
-    xmlNode *copy = NULL;
 
-    if (sort) {
-        crm_trace("Sorting xml...");
-        copy = sorted_xml(input, NULL, TRUE);
-        crm_trace("Done");
-        input = copy;
-    }
-
-    buffer = dump_xml_for_digest(input);
-    CRM_CHECK(buffer->len > 0, free_xml(copy);
+    // buffer->len > 2 for initial space and trailing newline
+    CRM_CHECK(buffer->len > 2,
               g_string_free(buffer, TRUE);
               return NULL);
 
@@ -75,27 +95,77 @@ calculate_xml_digest_v1(xmlNode *input, gboolean sort, gboolean ignored)
     crm_log_xml_trace(input, "digest:source");
 
     g_string_free(buffer, TRUE);
-    free_xml(copy);
     return digest;
 }
 
 /*!
- * \brief Calculate and return v2 digest of XML tree
+ * \internal
+ * \brief Calculate and return the digest of a CIB, suitable for storing on disk
  *
- * \param[in] source Root of XML to digest
- * \param[in] do_filter Whether to filter certain XML attributes
+ * \param[in] input  Root of XML to digest
  *
  * \return Newly allocated string containing digest
  */
-static char *
-calculate_xml_digest_v2(const xmlNode *source, gboolean do_filter)
+char *
+pcmk__digest_on_disk_cib(xmlNode *input)
 {
+    /* Always use the v1 format for on-disk digests.
+     * * Switching to v2 affects even full-restart upgrades, so it would be a
+     *   compatibility nightmare.
+     * * We only use this once at startup. All other invocations are in a
+     *   separate child process.
+     */
+    return calculate_xml_digest_v1(input);
+}
+
+/*!
+ * \internal
+ * \brief Calculate and return digest of an operation XML element
+ *
+ * The digest is invariant to changes in the order of XML attributes, provided
+ * that \p input has no children.
+ *
+ * \param[in] input  Root of XML to digest
+ *
+ * \return Newly allocated string containing digest
+ */
+char *
+pcmk__digest_operation(xmlNode *input)
+{
+    /* Switching to v2 digests would likely cause restarts during rolling
+     * upgrades.
+     *
+     * @TODO Confirm this. Switch to v2 if safe, or drop this TODO otherwise.
+     */
+    xmlNode *sorted = pcmk__xml_copy(NULL, input);
+    char *digest = NULL;
+
+    pcmk__xe_sort_attrs(sorted);
+    digest = calculate_xml_digest_v1(sorted);
+
+    pcmk__xml_free(sorted);
+    return digest;
+}
+
+/*!
+ * \internal
+ * \brief Calculate and return the digest of an XML tree
+ *
+ * \param[in] xml     XML tree to digest
+ * \param[in] filter  Whether to filter certain XML attributes
+ *
+ * \return Newly allocated string containing digest
+ */
+char *
+pcmk__digest_xml(xmlNode *xml, bool filter)
+{
+    /* @TODO Filtering accounts for significant CPU usage. Consider removing if
+     * possible.
+     */
     char *digest = NULL;
     GString *buf = g_string_sized_new(1024);
 
-    crm_trace("Begin digest %s", do_filter?"filtered":"");
-
-    pcmk__xml_string(source, (do_filter? pcmk__xml_fmt_filtered : 0), buf, 0);
+    pcmk__xml_string(xml, (filter? pcmk__xml_fmt_filtered : 0), buf, 0);
     digest = crm_md5sum(buf->str);
 
     pcmk__if_tracing(
@@ -104,89 +174,19 @@ calculate_xml_digest_v2(const xmlNode *source, gboolean do_filter)
                                                  pcmk__get_tmpdir(), digest);
 
             crm_trace("Saving %s.%s.%s to %s",
-                      crm_element_value(source, PCMK_XA_ADMIN_EPOCH),
-                      crm_element_value(source, PCMK_XA_EPOCH),
-                      crm_element_value(source, PCMK_XA_NUM_UPDATES),
+                      crm_element_value(xml, PCMK_XA_ADMIN_EPOCH),
+                      crm_element_value(xml, PCMK_XA_EPOCH),
+                      crm_element_value(xml, PCMK_XA_NUM_UPDATES),
                       trace_file);
-            save_xml_to_file(source, "digest input", trace_file);
+            save_xml_to_file(xml, "digest input", trace_file);
             free(trace_file);
         },
         {}
     );
-    crm_trace("End digest");
     g_string_free(buf, TRUE);
     return digest;
 }
 
-/*!
- * \brief Calculate and return digest of XML tree, suitable for storing on disk
- *
- * \param[in] input Root of XML to digest
- *
- * \return Newly allocated string containing digest
- */
-char *
-calculate_on_disk_digest(xmlNode *input)
-{
-    /* Always use the v1 format for on-disk digests
-     * a) it's a compatibility nightmare
-     * b) we only use this once at startup, all other
-     *    invocations are in a separate child process
-     */
-    return calculate_xml_digest_v1(input, FALSE, FALSE);
-}
-
-/*!
- * \brief Calculate and return digest of XML operation
- *
- * \param[in] input    Root of XML to digest
- * \param[in] version  Unused
- *
- * \return Newly allocated string containing digest
- */
-char *
-calculate_operation_digest(xmlNode *input, const char *version)
-{
-    /* We still need the sorting for operation digests */
-    return calculate_xml_digest_v1(input, TRUE, FALSE);
-}
-
-/*!
- * \brief Calculate and return digest of XML tree
- *
- * \param[in] input      Root of XML to digest
- * \param[in] sort       Whether to sort XML before calculating digest
- * \param[in] do_filter  Whether to filter certain XML attributes
- * \param[in] version    CRM feature set version (used to select v1/v2 digest)
- *
- * \return Newly allocated string containing digest
- */
-char *
-calculate_xml_versioned_digest(xmlNode *input, gboolean sort,
-                               gboolean do_filter, const char *version)
-{
-    /*
-     * @COMPAT digests (on-disk or in diffs/patchsets) created <1.1.4;
-     * removing this affects even full-restart upgrades from old versions
-     *
-     * The sorting associated with v1 digest creation accounted for 23% of
-     * the CIB manager's CPU usage on the server. v2 drops this.
-     *
-     * The filtering accounts for an additional 2.5% and we may want to
-     * remove it in future.
-     *
-     * v2 also uses the xmlBuffer contents directly to avoid additional copying
-     */
-    if (version == NULL || compare_version("3.0.5", version) > 0) {
-        crm_trace("Using v1 digest algorithm for %s",
-                  pcmk__s(version, "unknown feature set"));
-        return calculate_xml_digest_v1(input, sort, do_filter);
-    }
-    crm_trace("Using v2 digest algorithm for %s",
-              pcmk__s(version, "unknown feature set"));
-    return calculate_xml_digest_v2(input, do_filter);
-}
-
 /*!
  * \internal
  * \brief Check whether calculated digest of given XML matches expected digest
@@ -203,7 +203,7 @@ pcmk__verify_digest(xmlNode *input, const char *expected)
     bool passed;
 
     if (input != NULL) {
-        calculated = calculate_on_disk_digest(input);
+        calculated = pcmk__digest_on_disk_cib(input);
         if (calculated == NULL) {
             crm_perror(LOG_ERR, "Could not calculate digest for comparison");
             return false;
@@ -250,28 +250,24 @@ pcmk__xa_filterable(const char *name)
 char *
 crm_md5sum(const char *buffer)
 {
-    int lpc = 0, len = 0;
     char *digest = NULL;
-    unsigned char raw_digest[MD5_DIGEST_SIZE];
+    gchar *raw_digest = NULL;
 
     if (buffer == NULL) {
-        buffer = "";
+        return NULL;
     }
-    len = strlen(buffer);
 
-    crm_trace("Beginning digest of %d bytes", len);
-    digest = malloc(2 * MD5_DIGEST_SIZE + 1);
-    if (digest) {
-        md5_buffer(buffer, len, raw_digest);
-        for (lpc = 0; lpc < MD5_DIGEST_SIZE; lpc++) {
-            sprintf(digest + (2 * lpc), "%02x", raw_digest[lpc]);
-        }
-        digest[(2 * MD5_DIGEST_SIZE)] = 0;
-        crm_trace("Digest %s.", digest);
+    raw_digest = g_compute_checksum_for_string(G_CHECKSUM_MD5, buffer, -1);
 
-    } else {
-        crm_err("Could not create digest");
+    if (raw_digest == NULL) {
+        crm_err("Failed to calculate hash");
+        return NULL;
     }
+
+    digest = pcmk__str_copy(raw_digest);
+    g_free(raw_digest);
+
+    crm_trace("Digest %s.", digest);
     return digest;
 }
 
@@ -334,3 +330,54 @@ pcmk__filter_op_for_digest(xmlNode *param_set)
     free(timeout);
     free(key);
 }
+
+// Deprecated functions kept only for backward API compatibility
+// LCOV_EXCL_START
+
+#include <crm/common/xml_compat.h>
+#include <crm/common/xml_element_compat.h>
+
+char *
+calculate_on_disk_digest(xmlNode *input)
+{
+    return calculate_xml_digest_v1(input);
+}
+
+char *
+calculate_operation_digest(xmlNode *input, const char *version)
+{
+    xmlNode *sorted = sorted_xml(input, NULL, true);
+    char *digest = calculate_xml_digest_v1(sorted);
+
+    pcmk__xml_free(sorted);
+    return digest;
+}
+
+char *
+calculate_xml_versioned_digest(xmlNode *input, gboolean sort,
+                               gboolean do_filter, const char *version)
+{
+    if ((version == NULL) || (compare_version("3.0.5", version) > 0)) {
+        xmlNode *sorted = NULL;
+        char *digest = NULL;
+
+        if (sort) {
+            xmlNode *sorted = sorted_xml(input, NULL, true);
+
+            input = sorted;
+        }
+
+        crm_trace("Using v1 digest algorithm for %s",
+                  pcmk__s(version, "unknown feature set"));
+
+        digest = calculate_xml_digest_v1(input);
+
+        pcmk__xml_free(sorted);
+        return digest;
+    }
+    crm_trace("Using v2 digest algorithm for %s", version);
+    return pcmk__digest_xml(input, do_filter);
+}
+
+// LCOV_EXCL_STOP
+// End deprecated API
