@@ -9,10 +9,6 @@
 
 #include <crm_internal.h>
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -118,11 +114,11 @@ pcmk__real_path(const char *path, char **resolved_path)
  * \note The caller is responsible for freeing the return value.
  */
 char *
-pcmk__series_filename(const char *directory, const char *series, int sequence,
-                      bool bzip)
+pcmk__series_filename(const char *directory, const char *series,
+                      unsigned int sequence, bool bzip)
 {
-    CRM_ASSERT((directory != NULL) && (series != NULL));
-    return crm_strdup_printf("%s/%s-%d.%s", directory, series, sequence,
+    pcmk__assert((directory != NULL) && (series != NULL));
+    return crm_strdup_printf("%s/%s-%u.%s", directory, series, sequence,
                              (bzip? "bz2" : "raw"));
 }
 
@@ -265,14 +261,14 @@ pcmk__daemon_user_can_write(const char *target_name, struct stat *target_stat)
         return FALSE;
     }
     if (target_stat->st_uid != sys_user->pw_uid) {
-        crm_notice("%s is not owned by user %s " CRM_XS " uid %d != %d",
+        crm_notice("%s is not owned by user %s " QB_XS " uid %d != %d",
                    target_name, CRM_DAEMON_USER, sys_user->pw_uid,
                    target_stat->st_uid);
         return FALSE;
     }
     if ((target_stat->st_mode & (S_IRUSR | S_IWUSR)) == 0) {
         crm_notice("%s is not readable and writable by user %s "
-                   CRM_XS " st_mode=0%lo",
+                   QB_XS " st_mode=0%lo",
                    target_name, CRM_DAEMON_USER,
                    (unsigned long) target_stat->st_mode);
         return FALSE;
@@ -294,7 +290,7 @@ pcmk__daemon_group_can_write(const char *target_name, struct stat *target_stat)
     }
 
     if (target_stat->st_gid != sys_grp->gr_gid) {
-        crm_notice("%s is not owned by group %s " CRM_XS " uid %d != %d",
+        crm_notice("%s is not owned by group %s " QB_XS " uid %d != %d",
                    target_name, CRM_DAEMON_GROUP,
                    sys_grp->gr_gid, target_stat->st_gid);
         return FALSE;
@@ -302,7 +298,7 @@ pcmk__daemon_group_can_write(const char *target_name, struct stat *target_stat)
 
     if ((target_stat->st_mode & (S_IRGRP | S_IWGRP)) == 0) {
         crm_notice("%s is not readable and writable by group %s "
-                   CRM_XS " st_mode=0%lo",
+                   QB_XS " st_mode=0%lo",
                    target_name, CRM_DAEMON_GROUP,
                    (unsigned long) target_stat->st_mode);
         return FALSE;
@@ -333,7 +329,7 @@ pcmk__daemon_can_write(const char *dir, const char *file)
     const char *target = NULL;
 
     // Caller must supply directory
-    CRM_ASSERT(dir != NULL);
+    pcmk__assert(dir != NULL);
 
     // If file is given, check whether it exists as a regular file
     if (file != NULL) {
@@ -348,7 +344,7 @@ pcmk__daemon_can_write(const char *dir, const char *file)
             target = NULL;
 
         } else if (S_ISREG(buf.st_mode) == FALSE) {
-            crm_err("%s must be a regular file " CRM_XS " st_mode=0%lo",
+            crm_err("%s must be a regular file " QB_XS " st_mode=0%lo",
                     target, (unsigned long) buf.st_mode);
             free(full_file);
             return false;
@@ -364,7 +360,7 @@ pcmk__daemon_can_write(const char *dir, const char *file)
             return false;
 
         } else if (S_ISDIR(buf.st_mode) == FALSE) {
-            crm_err("%s must be a directory " CRM_XS " st_mode=0%lo",
+            crm_err("%s must be a directory " QB_XS " st_mode=0%lo",
                     dir, (unsigned long) buf.st_mode);
             return false;
         }
@@ -374,7 +370,7 @@ pcmk__daemon_can_write(const char *dir, const char *file)
         && !pcmk__daemon_group_can_write(target, &buf)) {
 
         crm_err("%s must be owned and writable by either user %s or group %s "
-                CRM_XS " st_mode=0%lo",
+                QB_XS " st_mode=0%lo",
                 target, CRM_DAEMON_USER, CRM_DAEMON_GROUP,
                 (unsigned long) buf.st_mode);
         free(full_file);
@@ -633,30 +629,11 @@ pcmk__close_fds_in_child(bool all)
 char *
 pcmk__full_path(const char *filename, const char *dirname)
 {
-    CRM_ASSERT(filename != NULL);
+    pcmk__assert(filename != NULL);
 
     if (filename[0] == '/') {
         return pcmk__str_copy(filename);
     }
-    CRM_ASSERT(dirname != NULL);
+    pcmk__assert(dirname != NULL);
     return crm_strdup_printf("%s/%s", dirname, filename);
 }
-
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-#include <crm/common/util_compat.h>
-
-void
-crm_build_path(const char *path_c, mode_t mode)
-{
-    int rc = pcmk__build_path(path_c, mode);
-
-    if (rc != pcmk_rc_ok) {
-        crm_err("Could not create directory '%s': %s",
-                path_c, pcmk_rc_str(rc));
-    }
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
