@@ -7,10 +7,6 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <crm_internal.h>
 
 #include <stdio.h>
@@ -32,7 +28,7 @@ set_pairs_data(pcmk__attrd_api_reply_t *data, xmlNode *msg_data)
 
     for (xmlNode *node = pcmk__xe_first_child(msg_data, PCMK_XE_NODE, NULL,
                                               NULL);
-         node != NULL; node = pcmk__xe_next_same(node)) {
+         node != NULL; node = pcmk__xe_next(node, PCMK_XE_NODE)) {
 
         pair = pcmk__assert_alloc(1, sizeof(pcmk__attrd_query_pair_t));
 
@@ -131,11 +127,11 @@ pcmk__attrd_api_methods(void)
 
 /*!
  * \internal
- * \brief Create a generic pacemaker-attrd operation
+ * \brief Create a generic attribute manager operation
  *
  * \param[in] user_name  If not NULL, ACL user to set for operation
  *
- * \return XML of pacemaker-attrd operation
+ * \return XML of attribute manager operation
  */
 static xmlNode *
 create_attrd_op(const char *user_name)
@@ -154,6 +150,7 @@ connect_and_send_attrd_request(pcmk_ipc_api_t *api, const xmlNode *request)
 {
     int rc = pcmk_rc_ok;
     bool created_api = false;
+    enum pcmk_ipc_dispatch dispatch = pcmk_ipc_dispatch_sync;
 
     if (api == NULL) {
         rc = pcmk_new_ipc_api(&api, pcmk_ipc_attrd);
@@ -161,9 +158,11 @@ connect_and_send_attrd_request(pcmk_ipc_api_t *api, const xmlNode *request)
             return rc;
         }
         created_api = true;
+    } else {
+        dispatch = api->dispatch_type;
     }
 
-    rc = pcmk__connect_ipc(api, pcmk_ipc_dispatch_sync, 5);
+    rc = pcmk__connect_ipc(api, dispatch, 5);
     if (rc == pcmk_rc_ok) {
         rc = pcmk__send_ipc_request(api, request);
     }
@@ -211,7 +210,7 @@ pcmk__attrd_api_clear_failures(pcmk_ipc_api_t *api, const char *node,
 
     rc = connect_and_send_attrd_request(api, request);
 
-    free_xml(request);
+    pcmk__xml_free(request);
     return rc;
 }
 
@@ -262,7 +261,7 @@ pcmk__attrd_api_purge(pcmk_ipc_api_t *api, const char *node, bool reap)
 
     rc = connect_and_send_attrd_request(api, request);
 
-    free_xml(request);
+    pcmk__xml_free(request);
     return rc;
 }
 
@@ -285,6 +284,8 @@ pcmk__attrd_api_query(pcmk_ipc_api_t *api, const char *node, const char *name,
 
         if (target != NULL) {
             node = target;
+        } else if (node == NULL) {
+            node = "localhost";
         }
     }
 
@@ -299,7 +300,7 @@ pcmk__attrd_api_query(pcmk_ipc_api_t *api, const char *node, const char *name,
     pcmk__xe_add_node(request, node, 0);
 
     rc = connect_and_send_attrd_request(api, request);
-    free_xml(request);
+    pcmk__xml_free(request);
     return rc;
 }
 
@@ -324,7 +325,7 @@ pcmk__attrd_api_refresh(pcmk_ipc_api_t *api, const char *node)
 
     rc = connect_and_send_attrd_request(api, request);
 
-    free_xml(request);
+    pcmk__xml_free(request);
     return rc;
 }
 
@@ -402,7 +403,7 @@ pcmk__attrd_api_update(pcmk_ipc_api_t *api, const char *node, const char *name,
 
     rc = connect_and_send_attrd_request(api, request);
 
-    free_xml(request);
+    pcmk__xml_free(request);
     return rc;
 }
 
@@ -478,7 +479,7 @@ pcmk__attrd_api_update_list(pcmk_ipc_api_t *api, GList *attrs, const char *dampe
      */
     if (pcmk__is_daemon) {
         rc = connect_and_send_attrd_request(api, request);
-        free_xml(request);
+        pcmk__xml_free(request);
     }
 
     return rc;
