@@ -10,14 +10,9 @@
 #include <crm_internal.h>
 
 #if defined(HAVE_UCRED) || defined(HAVE_SOCKPEERCRED)
-#  ifdef HAVE_UCRED
-#    ifndef _GNU_SOURCE
-#      define _GNU_SOURCE
-#    endif
-#  endif
-#  include <sys/socket.h>
+#include <sys/socket.h>
 #elif defined(HAVE_GETPEERUCRED)
-#  include <ucred.h>
+#include <ucred.h>
 #endif
 
 #include <stdio.h>
@@ -97,6 +92,11 @@ pcmk_new_ipc_api(pcmk_ipc_api_t **api, enum pcmk_ipc_server server)
             // @TODO max_size could vary by client, maybe take as argument?
             (*api)->ipc_size_max = 5 * 1024 * 1024; // 5MB
             break;
+
+        default: // pcmk_ipc_unknown
+            pcmk_free_ipc_api(*api);
+            *api = NULL;
+            return EINVAL;
     }
     if ((*api)->cmds == NULL) {
         pcmk_free_ipc_api(*api);
@@ -249,30 +249,20 @@ pcmk_ipc_name(const pcmk_ipc_api_t *api, bool for_log)
     if (api == NULL) {
         return for_log? "Pacemaker" : NULL;
     }
+    if (for_log) {
+        const char *name = pcmk__server_log_name(api->server);
+
+        return pcmk__s(name, "Pacemaker");
+    }
     switch (api->server) {
-        case pcmk_ipc_attrd:
-            return for_log? "attribute manager" : PCMK__VALUE_ATTRD;
-
+        // These servers do not have pcmk_ipc_api_t implementations yet
         case pcmk_ipc_based:
-            return for_log? "CIB manager" : NULL /* PCMK__SERVER_BASED_RW */;
-
-        case pcmk_ipc_controld:
-            return for_log? "controller" : CRM_SYSTEM_CRMD;
-
         case pcmk_ipc_execd:
-            return for_log? "executor" : NULL /* CRM_SYSTEM_LRMD */;
-
         case pcmk_ipc_fenced:
-            return for_log? "fencer" : NULL /* "stonith-ng" */;
-
-        case pcmk_ipc_pacemakerd:
-            return for_log? "launcher" : CRM_SYSTEM_MCP;
-
-        case pcmk_ipc_schedulerd:
-            return for_log? "scheduler" : CRM_SYSTEM_PENGINE;
+            return NULL;
 
         default:
-            return for_log? "Pacemaker" : NULL;
+            return pcmk__server_ipc_name(api->server);
     }
 }
 
@@ -346,7 +336,7 @@ dispatch_ipc_data(const char *buffer, pcmk_ipc_api_t *api)
     }
 
     more = call_api_dispatch(api, msg);
-    free_xml(msg);
+    pcmk__xml_free(msg);
 
     if (more) {
         return EINPROGRESS;
@@ -697,7 +687,7 @@ pcmk__send_ipc_request(pcmk_ipc_api_t *api, const xmlNode *request)
     if (reply != NULL) {
         bool more = call_api_dispatch(api, reply);
 
-        free_xml(reply);
+        pcmk__xml_free(reply);
 
         while (more) {
             rc = crm_ipc_read(api->ipc);
@@ -766,8 +756,9 @@ create_purge_node_request(const pcmk_ipc_api_t *api, const char *node_name,
         case pcmk_ipc_controld:
         case pcmk_ipc_fenced:
         case pcmk_ipc_pacemakerd:
-            request = create_request(CRM_OP_RM_NODE_CACHE, NULL, NULL,
-                                     pcmk_ipc_name(api, false), client, NULL);
+            request = pcmk__new_request(api->server, client, NULL,
+                                        pcmk_ipc_name(api, false),
+                                        CRM_OP_RM_NODE_CACHE, NULL);
             if (nodeid > 0) {
                 crm_xml_add_ll(request, PCMK_XA_ID, (long long) nodeid);
             }
@@ -778,6 +769,9 @@ create_purge_node_request(const pcmk_ipc_api_t *api, const char *node_name,
         case pcmk_ipc_execd:
         case pcmk_ipc_schedulerd:
             break;
+
+        default: // pcmk_ipc_unknown (shouldn't be possible)
+            return NULL;
     }
     return request;
 }
@@ -811,7 +805,7 @@ pcmk_ipc_purge_node(pcmk_ipc_api_t *api, const char *node_name, uint32_t nodeid)
         return EOPNOTSUPP;
     }
     rc = pcmk__send_ipc_request(api, request);
-    free_xml(request);
+    pcmk__xml_free(request);
 
     crm_debug("%s peer cache purge of node %s[%lu]: rc=%d",
               pcmk_ipc_name(api, true), node_name, (unsigned long) nodeid, rc);
@@ -955,40 +949,6 @@ pcmk__connect_generic_ipc(crm_ipc_t *ipc)
     return pcmk_rc_ok;
 }
 
-/*!
- * \brief Establish an IPC connection to a Pacemaker component
- *
- * \param[in,out] client  Connection instance obtained from crm_ipc_new()
- *
- * \return true on success, false otherwise (in which case errno will be set;
- *         specifically, in case of discovering the remote side is not
- *         authentic, its value is set to ECONNABORTED).
- */
-bool
-crm_ipc_connect(crm_ipc_t *client)
-{
-    int rc = pcmk__connect_generic_ipc(client);
-
-    if (rc == pcmk_rc_ok) {
-        return true;
-    }
-    if ((client != NULL) && (client->ipc == NULL)) {
-        errno = (rc > 0)? rc : ENOTCONN;
-        crm_debug("Could not establish %s IPC connection: %s (%d)",
-                  client->server_name, pcmk_rc_str(errno), errno);
-    } else if (rc == pcmk_rc_ipc_unauthorized) {
-        crm_err("%s IPC provider authentication failed",
-                (client == NULL)? "Pacemaker" : client->server_name);
-        errno = ECONNABORTED;
-    } else {
-        crm_perror(LOG_ERR,
-                   "Could not verify authenticity of %s IPC provider",
-                   (client == NULL)? "Pacemaker" : client->server_name);
-        errno = ENOTCONN;
-    }
-    return false;
-}
-
 void
 crm_ipc_close(crm_ipc_t * client)
 {
@@ -1101,7 +1061,7 @@ crm_ipc_ready(crm_ipc_t *client)
 {
     int rc;
 
-    CRM_ASSERT(client != NULL);
+    pcmk__assert(client != NULL);
 
     if (!crm_ipc_connected(client)) {
         return -ENOTCONN;
@@ -1133,21 +1093,13 @@ crm_ipc_decompress(crm_ipc_t * client)
         rc = pcmk__bzlib2rc(rc);
 
         if (rc != pcmk_rc_ok) {
-            crm_err("Decompression failed: %s " CRM_XS " rc=%d",
+            crm_err("Decompression failed: %s " QB_XS " rc=%d",
                     pcmk_rc_str(rc), rc);
             free(uncompressed);
             return rc;
         }
 
-        /*
-         * This assert no longer holds true.  For an identical msg, some clients may
-         * require compression, and others may not. If that same msg (event) is sent
-         * to multiple clients, it could result in some clients receiving a compressed
-         * msg even though compression was not explicitly required for them.
-         *
-         * CRM_ASSERT((header->size_uncompressed + sizeof(pcmk__ipc_header_t)) >= ipc_buffer_max);
-         */
-        CRM_ASSERT(size_u == header->size_uncompressed);
+        pcmk__assert(size_u == header->size_uncompressed);
 
         memcpy(uncompressed, client->buffer, sizeof(pcmk__ipc_header_t));       /* Preserve the header */
         header = (pcmk__ipc_header_t *)(void*)uncompressed;
@@ -1157,7 +1109,8 @@ crm_ipc_decompress(crm_ipc_t * client)
         client->buffer = uncompressed;
     }
 
-    CRM_ASSERT(client->buffer[sizeof(pcmk__ipc_header_t) + header->size_uncompressed - 1] == 0);
+    pcmk__assert(client->buffer[sizeof(pcmk__ipc_header_t)
+                                + header->size_uncompressed - 1] == 0);
     return pcmk_rc_ok;
 }
 
@@ -1166,9 +1119,8 @@ crm_ipc_read(crm_ipc_t * client)
 {
     pcmk__ipc_header_t *header = NULL;
 
-    CRM_ASSERT(client != NULL);
-    CRM_ASSERT(client->ipc != NULL);
-    CRM_ASSERT(client->buffer != NULL);
+    pcmk__assert((client != NULL) && (client->ipc != NULL)
+                 && (client->buffer != NULL));
 
     client->buffer[0] = 0;
     client->msg_size = qb_ipcc_event_recv(client->ipc, client->buffer,
@@ -1213,7 +1165,7 @@ crm_ipc_read(crm_ipc_t * client)
 const char *
 crm_ipc_buffer(crm_ipc_t * client)
 {
-    CRM_ASSERT(client != NULL);
+    pcmk__assert(client != NULL);
     return client->buffer + sizeof(pcmk__ipc_header_t);
 }
 
@@ -1222,7 +1174,7 @@ crm_ipc_buffer_flags(crm_ipc_t * client)
 {
     pcmk__ipc_header_t *header = NULL;
 
-    CRM_ASSERT(client != NULL);
+    pcmk__assert(client != NULL);
     if (client->buffer == NULL) {
         return 0;
     }
@@ -1234,7 +1186,7 @@ crm_ipc_buffer_flags(crm_ipc_t * client)
 const char *
 crm_ipc_name(crm_ipc_t * client)
 {
-    CRM_ASSERT(client != NULL);
+    pcmk__assert(client != NULL);
     return client->server_name;
 }
 
@@ -1243,7 +1195,7 @@ static int
 internal_ipc_get_reply(crm_ipc_t *client, int request_id, int ms_timeout,
                        ssize_t *bytes)
 {
-    time_t timeout = time(NULL) + 1 + (ms_timeout / 1000);
+    time_t timeout = time(NULL) + 1 + pcmk__timeout_ms2s(ms_timeout);
     int rc = pcmk_rc_ok;
 
     /* get the reply */
@@ -1275,7 +1227,7 @@ internal_ipc_get_reply(crm_ipc_t *client, int request_id, int ms_timeout,
 
                 crm_err("Discarding newer reply %d (need %d)", hdr->qb.id, request_id);
                 crm_log_xml_notice(bad, "ImpossibleReply");
-                CRM_ASSERT(hdr->qb.id <= request_id);
+                pcmk__assert(hdr->qb.id <= request_id);
             }
         } else if (!crm_ipc_connected(client)) {
             crm_err("%s IPC provider disconnected while waiting for message %d",
@@ -1350,7 +1302,7 @@ crm_ipc_send(crm_ipc_t *client, const xmlNode *message,
     CRM_LOG_ASSERT(id != 0); /* Crude wrap-around detection */
     rc = pcmk__ipc_prepare_iov(id, message, client->max_buf_size, &iov, &bytes);
     if (rc != pcmk_rc_ok) {
-        crm_warn("Couldn't prepare %s IPC request: %s " CRM_XS " rc=%d",
+        crm_warn("Couldn't prepare %s IPC request: %s " QB_XS " rc=%d",
                  client->server_name, pcmk_rc_str(rc), rc);
         return pcmk_rc2legacy(rc);
     }
@@ -1378,7 +1330,7 @@ crm_ipc_send(crm_ipc_t *client, const xmlNode *message,
 
     if ((ms_timeout > 0) || !pcmk_is_set(flags, crm_ipc_client_response)) {
 
-        time_t timeout = time(NULL) + 1 + (ms_timeout / 1000);
+        time_t timeout = time(NULL) + 1 + pcmk__timeout_ms2s(ms_timeout);
 
         do {
             /* @TODO Is this check really needed? Won't qb_ipcc_sendv() return
@@ -1441,16 +1393,16 @@ crm_ipc_send(crm_ipc_t *client, const xmlNode *message,
   send_cleanup:
     if (!crm_ipc_connected(client)) {
         crm_notice("Couldn't send %s IPC request %d: Connection closed "
-                   CRM_XS " rc=%d", client->server_name, header->qb.id, rc);
+                   QB_XS " rc=%d", client->server_name, header->qb.id, rc);
 
     } else if (rc == -ETIMEDOUT) {
-        crm_warn("%s IPC request %d failed: %s after %dms " CRM_XS " rc=%d",
+        crm_warn("%s IPC request %d failed: %s after %dms " QB_XS " rc=%d",
                  client->server_name, header->qb.id, pcmk_strerror(rc),
                  ms_timeout, rc);
         crm_write_blackbox(0, NULL);
 
     } else if (rc <= 0) {
-        crm_warn("%s IPC request %d failed: %s " CRM_XS " rc=%d",
+        crm_warn("%s IPC request %d failed: %s " QB_XS " rc=%d",
                  client->server_name, header->qb.id,
                  ((rc == 0)? "No bytes sent" : pcmk_strerror(rc)), rc);
     }
@@ -1641,7 +1593,7 @@ pcmk__ipc_is_authentic_process_active(const char *name, uid_t refuid,
     qb_rc = qb_ipcc_fd_get(c, &fd);
     if (qb_rc != 0) {
         rc = (int) -qb_rc; // System errno
-        crm_err("Could not get fd from %s IPC: %s " CRM_XS " rc=%d",
+        crm_err("Could not get fd from %s IPC: %s " QB_XS " rc=%d",
                 name, pcmk_rc_str(rc), rc);
         goto bail;
     }
@@ -1660,7 +1612,7 @@ pcmk__ipc_is_authentic_process_active(const char *name, uid_t refuid,
     if (auth_rc != pcmk_rc_ok) {
         rc = auth_rc;
         crm_err("Could not get peer credentials from %s IPC: %s "
-                CRM_XS " rc=%d", name, pcmk_rc_str(rc), rc);
+                QB_XS " rc=%d", name, pcmk_rc_str(rc), rc);
         goto bail;
     }
 
@@ -1692,3 +1644,35 @@ bail:
     }
     return rc;
 }
+
+// Deprecated functions kept only for backward API compatibility
+// LCOV_EXCL_START
+
+#include <crm/common/ipc_client_compat.h>
+
+bool
+crm_ipc_connect(crm_ipc_t *client)
+{
+    int rc = pcmk__connect_generic_ipc(client);
+
+    if (rc == pcmk_rc_ok) {
+        return true;
+    }
+    if ((client != NULL) && (client->ipc == NULL)) {
+        errno = (rc > 0)? rc : ENOTCONN;
+        crm_debug("Could not establish %s IPC connection: %s (%d)",
+                  client->server_name, pcmk_rc_str(errno), errno);
+    } else if (rc == pcmk_rc_ipc_unauthorized) {
+        crm_err("%s IPC provider authentication failed",
+                (client == NULL)? "Pacemaker" : client->server_name);
+        errno = ECONNABORTED;
+    } else {
+        crm_err("Could not verify authenticity of %s IPC provider",
+                (client == NULL)? "Pacemaker" : client->server_name);
+        errno = ENOTCONN;
+    }
+    return false;
+}
+
+// LCOV_EXCL_STOP
+// End deprecated API
