@@ -316,14 +316,12 @@ pcmk__free_client(pcmk__client_t *c)
         if (c->remote->auth_timeout) {
             g_source_remove(c->remote->auth_timeout);
         }
-#ifdef HAVE_GNUTLS_GNUTLS_H
         if (c->remote->tls_session != NULL) {
             /* @TODO Reduce duplication at callers. Put here everything
              * necessary to tear down and free tls_session.
              */
-            gnutls_free(c->remote->tls_session);
+            gnutls_deinit(c->remote->tls_session);
         }
-#endif  // HAVE_GNUTLS_GNUTLS_H
         free(c->remote->buffer);
         free(c->remote);
     }
@@ -335,23 +333,42 @@ pcmk__free_client(pcmk__client_t *c)
  * \brief Raise IPC eviction threshold for a client, if allowed
  *
  * \param[in,out] client     Client to modify
- * \param[in]     qmax       New threshold (as non-NULL string)
- *
- * \return true if change was allowed, false otherwise
+ * \param[in]     qmax       New threshold
  */
-bool
+void
 pcmk__set_client_queue_max(pcmk__client_t *client, const char *qmax)
 {
-    if (pcmk_is_set(client->flags, pcmk__client_privileged)) {
-        long long qmax_ll;
+    int rc = pcmk_rc_ok;
+    long long qmax_ll = 0LL;
+    unsigned int orig_value = 0U;
 
-        if ((pcmk__scan_ll(qmax, &qmax_ll, 0LL) == pcmk_rc_ok)
-            && (qmax_ll > 0LL) && (qmax_ll <= UINT_MAX)) {
-            client->queue_max = (unsigned int) qmax_ll;
-            return true;
+    CRM_CHECK(client != NULL, return);
+
+    orig_value = client->queue_max;
+
+    if (pcmk_is_set(client->flags, pcmk__client_privileged)) {
+        rc = pcmk__scan_ll(qmax, &qmax_ll, 0LL);
+        if (rc == pcmk_rc_ok) {
+            if ((qmax_ll <= 0LL) || (qmax_ll > UINT_MAX)) {
+                rc = ERANGE;
+            } else {
+                client->queue_max = (unsigned int) qmax_ll;
+            }
         }
+    } else {
+        rc = EACCES;
+    }
+
+    if (rc != pcmk_rc_ok) {
+        crm_info("Could not set IPC threshold for client %s[%u] to %s: %s",
+                  pcmk__client_name(client), client->pid,
+                  pcmk__s(qmax, "default"), pcmk_rc_str(rc));
+
+    } else if (client->queue_max != orig_value) {
+        crm_debug("IPC threshold for client %s[%u] is now %u (was %u)",
+                  pcmk__client_name(client), client->pid,
+                  client->queue_max, orig_value);
     }
-    return false;
 }
 
 int
@@ -417,14 +434,14 @@ pcmk__client_data2xml(pcmk__client_t *c, void *data, uint32_t *id,
         rc = pcmk__bzlib2rc(rc);
 
         if (rc != pcmk_rc_ok) {
-            crm_err("Decompression failed: %s " CRM_XS " rc=%d",
+            crm_err("Decompression failed: %s " QB_XS " rc=%d",
                     pcmk_rc_str(rc), rc);
             free(uncompressed);
             return NULL;
         }
     }
 
-    CRM_ASSERT(text[header->size_uncompressed - 1] == 0);
+    pcmk__assert(text[header->size_uncompressed - 1] == 0);
 
     xml = pcmk__xml_parse(text);
     crm_log_xml_trace(xml, "[IPC received]");
@@ -458,7 +475,7 @@ delay_next_flush(pcmk__client_t *c, unsigned int queue_len)
     /* Delay a maximum of 1.5 seconds */
     guint delay = (queue_len < 5)? (1000 + 100 * queue_len) : 1500;
 
-    c->event_timer = g_timeout_add(delay, crm_ipcs_flush_events_cb, c);
+    c->event_timer = pcmk__create_timer(delay, crm_ipcs_flush_events_cb, c);
 }
 
 /*!
@@ -538,10 +555,10 @@ crm_ipcs_flush_events(pcmk__client_t *c)
             if ((c->queue_backlog <= 1) || (queue_len < c->queue_backlog)) {
                 /* Don't evict for a new or shrinking backlog */
                 crm_warn("Client with process ID %u has a backlog of %u messages "
-                         CRM_XS " %p", c->pid, queue_len, c->ipcs);
+                         QB_XS " %p", c->pid, queue_len, c->ipcs);
             } else {
                 crm_err("Evicting client with process ID %u due to backlog of %u messages "
-                         CRM_XS " %p", c->pid, queue_len, c->ipcs);
+                         QB_XS " %p", c->pid, queue_len, c->ipcs);
                 c->queue_backlog = 0;
                 qb_ipcs_disconnect(c->ipcs);
                 return rc;
@@ -653,7 +670,7 @@ pcmk__ipc_prepare_iov(uint32_t request, const xmlNode *message,
     header->qb.id = (int32_t)request;    /* Replying to a specific request */
 
     *result = iov;
-    CRM_ASSERT(header->qb.size > 0);
+    pcmk__assert(header->qb.size > 0);
     if (bytes != NULL) {
         *bytes = header->qb.size;
     }
@@ -718,7 +735,7 @@ pcmk__ipc_send_iov(pcmk__client_t *c, struct iovec *iov, uint32_t flags)
                 rc = (int) -qb_rc;
             }
             crm_notice("Response %d to pid %d failed: %s "
-                       CRM_XS " bytes=%u rc=%lld ipcs=%p",
+                       QB_XS " bytes=%u rc=%lld ipcs=%p",
                        header->qb.id, c->pid, pcmk_rc_str(rc),
                        header->qb.size, (long long) qb_rc, c->ipcs);
 
@@ -761,7 +778,7 @@ pcmk__ipc_send_xml(pcmk__client_t *c, uint32_t request, const xmlNode *message,
         rc = pcmk__ipc_send_iov(c, iov, flags);
     } else {
         pcmk_free_ipc_event(iov);
-        crm_notice("IPC message to pid %d failed: %s " CRM_XS " rc=%d",
+        crm_notice("IPC message to pid %d failed: %s " QB_XS " rc=%d",
                    c->pid, pcmk_rc_str(rc), rc);
     }
     return rc;
@@ -779,7 +796,9 @@ pcmk__ipc_send_xml(pcmk__client_t *c, uint32_t request, const xmlNode *message,
  * \param[in] status    Exit status code to add to ack
  *
  * \return Newly created XML for ack
- * \note The caller is responsible for freeing the return value with free_xml().
+ *
+ * \note The caller is responsible for freeing the return value with
+ *       \c pcmk__xml_free().
  */
 xmlNode *
 pcmk__ipc_create_ack_as(const char *function, int line, uint32_t flags,
@@ -826,22 +845,24 @@ pcmk__ipc_send_ack_as(const char *function, int line, pcmk__client_t *c,
         crm_log_xml_trace(ack, "sent-ack");
         c->request_id = 0;
         rc = pcmk__ipc_send_xml(c, request, ack, flags);
-        free_xml(ack);
+        pcmk__xml_free(ack);
     }
     return rc;
 }
 
 /*!
  * \internal
- * \brief Add an IPC server to the main loop for the pacemaker-based API
+ * \brief Add an IPC server to the main loop for the CIB manager API
  *
- * \param[out] ipcs_ro   New IPC server for read-only pacemaker-based API
- * \param[out] ipcs_rw   New IPC server for read/write pacemaker-based API
- * \param[out] ipcs_shm  New IPC server for shared-memory pacemaker-based API
+ * \param[out] ipcs_ro   New IPC server for read-only CIB manager API
+ * \param[out] ipcs_rw   New IPC server for read/write CIB manager API
+ * \param[out] ipcs_shm  New IPC server for shared-memory CIB manager API
  * \param[in]  ro_cb     IPC callbacks for read-only API
  * \param[in]  rw_cb     IPC callbacks for read/write and shared-memory APIs
  *
  * \note This function exits fatally if unable to create the servers.
+ * \note There is no actual difference between the three IPC endpoints other
+ *       than their names.
  */
 void pcmk__serve_based_ipc(qb_ipcs_service_t **ipcs_ro,
                            qb_ipcs_service_t **ipcs_rw,
@@ -867,11 +888,11 @@ void pcmk__serve_based_ipc(qb_ipcs_service_t **ipcs_ro,
 
 /*!
  * \internal
- * \brief Destroy IPC servers for pacemaker-based API
+ * \brief Destroy IPC servers for the CIB manager API
  *
- * \param[out] ipcs_ro   IPC server for read-only pacemaker-based API
- * \param[out] ipcs_rw   IPC server for read/write pacemaker-based API
- * \param[out] ipcs_shm  IPC server for shared-memory pacemaker-based API
+ * \param[out] ipcs_ro   IPC server for read-only the CIB manager API
+ * \param[out] ipcs_rw   IPC server for read/write the CIB manager API
+ * \param[out] ipcs_shm  IPC server for shared-memory the CIB manager API
  *
  * \note This is a convenience function for calling qb_ipcs_destroy() for each
  *       argument.
@@ -888,7 +909,7 @@ pcmk__stop_based_ipc(qb_ipcs_service_t *ipcs_ro,
 
 /*!
  * \internal
- * \brief Add an IPC server to the main loop for the pacemaker-controld API
+ * \brief Add an IPC server to the main loop for the controller API
  *
  * \param[in] cb  IPC callbacks
  *
@@ -902,7 +923,7 @@ pcmk__serve_controld_ipc(struct qb_ipcs_service_handlers *cb)
 
 /*!
  * \internal
- * \brief Add an IPC server to the main loop for the pacemaker-attrd API
+ * \brief Add an IPC server to the main loop for the attribute manager API
  *
  * \param[out] ipcs  Where to store newly created IPC server
  * \param[in] cb  IPC callbacks
@@ -916,15 +937,16 @@ pcmk__serve_attrd_ipc(qb_ipcs_service_t **ipcs,
     *ipcs = mainloop_add_ipc_server(PCMK__VALUE_ATTRD, QB_IPC_NATIVE, cb);
 
     if (*ipcs == NULL) {
-        crm_err("Failed to create pacemaker-attrd server: exiting and inhibiting respawn");
-        crm_warn("Verify pacemaker and pacemaker_remote are not both enabled.");
+        crm_crit("Exiting fatally because unable to serve " PCMK__SERVER_ATTRD
+                 " IPC (verify pacemaker and pacemaker_remote are not both "
+                 "enabled)");
         crm_exit(CRM_EX_FATAL);
     }
 }
 
 /*!
  * \internal
- * \brief Add an IPC server to the main loop for the pacemaker-fenced API
+ * \brief Add an IPC server to the main loop for the fencer API
  *
  * \param[out] ipcs  Where to store newly created IPC server
  * \param[in]  cb    IPC callbacks
@@ -974,7 +996,7 @@ pcmk__serve_pacemakerd_ipc(qb_ipcs_service_t **ipcs,
 
 /*!
  * \internal
- * \brief Add an IPC server to the main loop for the pacemaker-schedulerd API
+ * \brief Add an IPC server to the main loop for the scheduler API
  *
  * \param[in] cb  IPC callbacks
  *
@@ -986,31 +1008,3 @@ pcmk__serve_schedulerd_ipc(struct qb_ipcs_service_handlers *cb)
 {
     return mainloop_add_ipc_server(CRM_SYSTEM_PENGINE, QB_IPC_NATIVE, cb);
 }
-
-/*!
- * \brief Check whether string represents a client name used by cluster daemons
- *
- * \param[in] name  String to check
- *
- * \return true if name is standard client name used by daemons, false otherwise
- *
- * \note This is provided by the client, and so cannot be used by itself as a
- *       secure means of authentication.
- */
-bool
-crm_is_daemon_name(const char *name)
-{
-    return pcmk__str_any_of(pcmk__message_name(name),
-                            "attrd",
-                            CRM_SYSTEM_CIB,
-                            CRM_SYSTEM_CRMD,
-                            CRM_SYSTEM_DC,
-                            CRM_SYSTEM_LRMD,
-                            CRM_SYSTEM_MCP,
-                            CRM_SYSTEM_PENGINE,
-                            CRM_SYSTEM_STONITHD,
-                            CRM_SYSTEM_TENGINE,
-                            "pacemaker-remoted",
-                            "stonith-ng",
-                            NULL);
-}
