@@ -215,15 +215,15 @@ static int month_days[13] = {
 /*!
  * \brief Return number of days in given month of given year
  *
- * \param[in]  Ordinal month (1-12)
- * \param[in]  Gregorian year
+ * \param[in] month  Ordinal month (1-12)
+ * \param[in] year   Gregorian year
  *
- * \return Number of days in given month (0 if given month is invalid)
+ * \return Number of days in given month (0 if given month or year is invalid)
  */
 int
 crm_time_days_in_month(int month, int year)
 {
-    if ((month < 1) || (month > 12)) {
+    if ((month < 1) || (month > 12) || (year < 1)) {
         return 0;
     }
     if ((month == 2) && crm_time_leapyear(year)) {
@@ -246,15 +246,30 @@ crm_time_leapyear(int year)
     return is_leap;
 }
 
-static uint32_t
+/*!
+ * \internal
+ * \brief Get ordinal day number of year corresponding to given date
+ *
+ * \param[in] y   Year
+ * \param[in] m   Month (1-12)
+ * \param[in] d   Day of month (1-31)
+ *
+ * \return Day number of year \p y corresponding to month \p m and day \p d,
+ *         or 0 for invalid arguments
+ */
+static int
 get_ordinal_days(uint32_t y, uint32_t m, uint32_t d)
 {
-    int lpc;
+    int result = 0;
 
-    for (lpc = 1; lpc < m; lpc++) {
-        d += crm_time_days_in_month(lpc, y);
+    CRM_CHECK((y > 0) && (y <= INT_MAX) && (m >= 1) && (m <= 12)
+              && (d >= 1) && (d <= 31), return 0);
+
+    result = d;
+    for (int lpc = 1; lpc < m; lpc++) {
+        result += crm_time_days_in_month(lpc, y);
     }
-    return d;
+    return result;
 }
 
 void
@@ -493,7 +508,7 @@ crm_duration_as_string(const crm_time_t *dt, int usec, bool show_usec,
 {
     size_t offset = 0;
 
-    CRM_ASSERT(valid_sec_usec(dt->seconds, usec));
+    pcmk__assert(valid_sec_usec(dt->seconds, usec));
 
     if (dt->years) {
         offset += snprintf(result + offset, DATE_MAX - offset, "%4d year%s ",
@@ -583,7 +598,7 @@ time_as_string_common(const crm_time_t *dt, int usec, uint32_t flags,
         return;
     }
 
-    CRM_ASSERT(valid_sec_usec(dt->seconds, usec));
+    pcmk__assert(valid_sec_usec(dt->seconds, usec));
 
     /* Simple cases: as duration, seconds, or seconds since epoch.
      * These never depend on time zone.
@@ -877,10 +892,11 @@ parse_date(const char *date_str)
     const char *time_s = NULL;
     crm_time_t *dt = NULL;
 
-    int year = 0;
-    int month = 0;
-    int week = 0;
-    int day = 0;
+    uint32_t year = 0U;
+    uint32_t month = 0U;
+    uint32_t day = 0U;
+    uint32_t week = 0U;
+
     int rc = 0;
 
     if (pcmk__str_empty(date_str)) {
@@ -913,37 +929,52 @@ parse_date(const char *date_str)
     }
 
     /* YYYY-MM-DD */
-    rc = sscanf(date_str, "%d-%d-%d", &year, &month, &day);
+    rc = sscanf(date_str, "%" SCNu32 "-%" SCNu32 "-%" SCNu32 "",
+                &year, &month, &day);
     if (rc == 1) {
         /* YYYYMMDD */
-        rc = sscanf(date_str, "%4d%2d%2d", &year, &month, &day);
+        rc = sscanf(date_str, "%4" SCNu32 "%2" SCNu32 "%2" SCNu32 "",
+                    &year, &month, &day);
     }
     if (rc == 3) {
-        if (month > 12) {
+        if ((month < 1U) || (month > 12U)) {
             crm_err("'%s' is not a valid ISO 8601 date/time specification "
-                    "because '%d' is not a valid month", date_str, month);
+                    "because '%" PRIu32 "' is not a valid month",
+                    date_str, month);
             goto invalid;
-        } else if (day > crm_time_days_in_month(month, year)) {
+        } else if ((year < 1U) || (year > INT_MAX)) {
             crm_err("'%s' is not a valid ISO 8601 date/time specification "
-                    "because '%d' is not a valid day of the month",
+                    "because '%" PRIu32 "' is not a valid year",
+                    date_str, year);
+            goto invalid;
+        } else if ((day < 1) || (day > INT_MAX)
+                   || (day > crm_time_days_in_month(month, year))) {
+            crm_err("'%s' is not a valid ISO 8601 date/time specification "
+                    "because '%" PRIu32 "' is not a valid day of the month",
                     date_str, day);
             goto invalid;
         } else {
             dt->years = year;
             dt->days = get_ordinal_days(year, month, day);
-            crm_trace("Parsed Gregorian date '%.4d-%.3d' from date string '%s'",
-                      year, dt->days, date_str);
+            crm_trace("Parsed Gregorian date '%.4" PRIu32 "-%.3d' "
+                      "from date string '%s'", year, dt->days, date_str);
         }
         goto parse_time;
     }
 
     /* YYYY-DDD */
-    rc = sscanf(date_str, "%d-%d", &year, &day);
+    rc = sscanf(date_str, "%" SCNu32 "-%" SCNu32, &year, &day);
     if (rc == 2) {
-        if (day > year_days(year)) {
+        if ((year < 1U) || (year > INT_MAX)) {
             crm_err("'%s' is not a valid ISO 8601 date/time specification "
-                    "because '%d' is not a valid day of the year (max %d)",
-                    date_str, day, year_days(year));
+                    "because '%" PRIu32 "' is not a valid year",
+                    date_str, year);
+            goto invalid;
+        } else if ((day < 1U) || (day > INT_MAX) || (day > year_days(year))) {
+            crm_err("'%s' is not a valid ISO 8601 date/time specification "
+                    "because '%" PRIu32 "' is not a valid day of year %"
+                    PRIu32 " (1-%d)",
+                    date_str, day, year, year_days(year));
             goto invalid;
         }
         crm_trace("Parsed ordinal year %d and days %d from date string '%s'",
@@ -954,16 +985,18 @@ parse_date(const char *date_str)
     }
 
     /* YYYY-Www-D */
-    rc = sscanf(date_str, "%d-W%d-%d", &year, &week, &day);
+    rc = sscanf(date_str, "%" SCNu32 "-W%" SCNu32 "-%" SCNu32,
+                &year, &week, &day);
     if (rc == 3) {
-        if (week > crm_time_weeks_in_year(year)) {
+        if ((week < 1U) || (week > crm_time_weeks_in_year(year))) {
             crm_err("'%s' is not a valid ISO 8601 date/time specification "
-                    "because '%d' is not a valid week of the year (max %d)",
-                    date_str, week, crm_time_weeks_in_year(year));
+                    "because '%" PRIu32 "' is not a valid week of year %"
+                    PRIu32 " (1-%d)",
+                    date_str, week, year, crm_time_weeks_in_year(year));
             goto invalid;
-        } else if (day < 1 || day > 7) {
+        } else if ((day < 1U) || (day > 7U)) {
             crm_err("'%s' is not a valid ISO 8601 date/time specification "
-                    "because '%d' is not a valid day of the week",
+                    "because '%" PRIu32 "' is not a valid day of the week",
                     date_str, day);
             goto invalid;
         } else {
@@ -974,12 +1007,14 @@ parse_date(const char *date_str)
              * Sunday 3 January 2010 is written "2009-W53-7"
              * Saturday 27 September 2008 is written "2008-W37-6"
              *
-             * If 1 January is on a Monday, Tuesday, Wednesday or Thursday, it is in week 01.
-             * If 1 January is on a Friday, Saturday or Sunday, it is in week 52 or 53 of the previous year.
+             * If 1 January is on a Monday, Tuesday, Wednesday or Thursday, it
+             * is in week 1. If 1 January is on a Friday, Saturday or Sunday,
+             * it is in week 52 or 53 of the previous year.
              */
             int jan1 = crm_time_january1_weekday(year);
 
-            crm_trace("Got year %d (Jan 1 = %d), week %d, and day %d from date string '%s'",
+            crm_trace("Parsed year %" PRIu32 " (Jan 1 = %d), week %" PRIu32
+                      ", and day %" PRIu32 " from date string '%s'",
                       year, jan1, week, day, date_str);
 
             dt->years = year;
@@ -1110,6 +1145,7 @@ crm_time_parse_duration(const char *period_s)
          ++current) {
 
         int an_int = 0, rc;
+        long long result = 0LL;
 
         if (current[0] == 'T') {
             /* A 'T' separates year/month/day from hour/minute/seconds. We don't
@@ -1138,60 +1174,70 @@ crm_time_parse_duration(const char *period_s)
             case 'M':
                 if (!is_time) { // Months
                     diff->months = an_int;
-
-                // Minutes
-                } else if ((diff->seconds + (an_int * 60LL)) > INT_MAX) {
-                    crm_err("'%s' is not a valid ISO 8601 time duration "
-                            "because integer at '%s' is too large",
-                            period_s, current - rc);
-                    goto invalid;
-                } else {
-                    diff->seconds += an_int * 60;
+                } else { // Minutes
+                    result = diff->seconds + an_int * 60LL;
+                    if ((result < INT_MIN) || (result > INT_MAX)) {
+                        crm_err("'%s' is not a valid ISO 8601 time duration "
+                                "because integer at '%s' is too %s",
+                                period_s, current - rc,
+                                ((result > 0)? "large" : "small"));
+                        goto invalid;
+                    } else {
+                        diff->seconds = (int) result;
+                    }
                 }
+
                 break;
 
             case 'W':
-                if ((diff->days + (an_int * 7LL)) > INT_MAX) {
+                result = diff->days + an_int * 7LL;
+                if ((result < INT_MIN) || (result > INT_MAX)) {
                     crm_err("'%s' is not a valid ISO 8601 time duration "
-                            "because integer at '%s' is too large",
-                            period_s, current - rc);
+                            "because integer at '%s' is too %s",
+                            period_s, current - rc,
+                            ((result > 0)? "large" : "small"));
                     goto invalid;
                 } else {
-                    diff->days += an_int * 7;
+                    diff->days = (int) result;
                 }
                 break;
 
             case 'D':
-                if ((diff->days + (long long) an_int) > INT_MAX) {
+                result = diff->days + (long long) an_int;
+                if ((result < INT_MIN) || (result > INT_MAX)) {
                     crm_err("'%s' is not a valid ISO 8601 time duration "
-                            "because integer at '%s' is too large",
-                            period_s, current - rc);
+                            "because integer at '%s' is too %s",
+                            period_s, current - rc,
+                            ((result > 0)? "large" : "small"));
                     goto invalid;
                 } else {
-                    diff->days += an_int;
+                    diff->days = (int) result;
                 }
                 break;
 
             case 'H':
-                if ((diff->seconds + ((long long) an_int * HOUR_SECONDS))
-                    > INT_MAX) {
+                result = diff->seconds + (long long) an_int * HOUR_SECONDS;
+                if ((result < INT_MIN) || (result > INT_MAX)) {
                     crm_err("'%s' is not a valid ISO 8601 time duration "
-                            "because integer at '%s' is too large",
-                            period_s, current - rc);
+                            "because integer at '%s' is too %s",
+                            period_s, current - rc,
+                            ((result > 0)? "large" : "small"));
                     goto invalid;
                 } else {
-                    diff->seconds += an_int * HOUR_SECONDS;
+                    diff->seconds = (int) result;
                 }
                 break;
 
             case 'S':
-                if ((diff->seconds + (long long) an_int) > INT_MAX) {
+                result = diff->seconds + (long long) an_int;
+                if ((result < INT_MIN) || (result > INT_MAX)) {
                     crm_err("'%s' is not a valid ISO 8601 time duration "
-                            "because integer at '%s' is too large",
-                            period_s, current - rc);
+                            "because integer at '%s' is too %s",
+                            period_s, current - rc,
+                            ((result > 0)? "large" : "small"));
                     goto invalid;
                 } else {
-                    diff->seconds += an_int;
+                    diff->seconds = (int) result;
                 }
                 break;
 
@@ -1369,7 +1415,8 @@ ha_set_tm_time(crm_time_t *target, const struct tm *source)
 
     if (source->tm_year > 0) {
         /* years since 1900 */
-        target->years = 1900 + source->tm_year;
+        target->years = 1900;
+        crm_time_add_years(target, source->tm_year);
     }
 
     if (source->tm_yday >= 0) {
@@ -1465,7 +1512,7 @@ crm_time_add(const crm_time_t *dt, const crm_time_t *value)
         return NULL;
     }
 
-    answer->years += utc->years;
+    crm_time_add_years(answer, utc->years);
     crm_time_add_months(answer, utc->months);
     crm_time_add_days(answer, utc->days);
     crm_time_add_seconds(answer, utc->seconds);
@@ -1618,7 +1665,7 @@ crm_time_calculate_duration(const crm_time_t *dt, const crm_time_t *value)
     }
     answer->duration = TRUE;
 
-    answer->years -= utc->years;
+    crm_time_add_years(answer, -utc->years);
     if(utc->months != 0) {
         crm_time_add_months(answer, -utc->months);
     }
@@ -1646,7 +1693,7 @@ crm_time_subtract(const crm_time_t *dt, const crm_time_t *value)
     }
 
     answer = pcmk_copy_time(dt);
-    answer->years -= utc->years;
+    crm_time_add_years(answer, -utc->years);
     if(utc->months != 0) {
         crm_time_add_months(answer, -utc->months);
     }
@@ -1718,46 +1765,65 @@ crm_time_compare(const crm_time_t *a, const crm_time_t *b)
 void
 crm_time_add_seconds(crm_time_t *a_time, int extra)
 {
-    int days = 0;
+    int days = extra / DAY_SECONDS;
 
-    crm_trace("Adding %d seconds to %d (max=%d)",
-              extra, a_time->seconds, DAY_SECONDS);
-    a_time->seconds += extra;
-    days = a_time->seconds / DAY_SECONDS;
-    a_time->seconds %= DAY_SECONDS;
+    pcmk__assert(a_time != NULL);
 
-    // Don't have negative seconds
-    if (a_time->seconds < 0) {
-        a_time->seconds += DAY_SECONDS;
+    crm_trace("Adding %d seconds (including %d whole day%s) to %d",
+              extra, days, pcmk__plural_s(days), a_time->seconds);
+
+    a_time->seconds += extra % DAY_SECONDS;
+
+    // Check whether the addition crossed a day boundary
+    if (a_time->seconds > DAY_SECONDS) {
+        ++days;
+        a_time->seconds -= DAY_SECONDS;
+
+    } else if (a_time->seconds < 0) {
         --days;
+        a_time->seconds += DAY_SECONDS;
     }
 
     crm_time_add_days(a_time, days);
 }
 
+#define ydays(t) (crm_time_leapyear((t)->years)? 366 : 365)
+
+/*!
+ * \brief Add days to a date/time
+ *
+ * \param[in,out] a_time  Time to modify
+ * \param[in]     extra   Number of days to add (may be negative to subtract)
+ */
 void
-crm_time_add_days(crm_time_t * a_time, int extra)
+crm_time_add_days(crm_time_t *a_time, int extra)
 {
-    int lower_bound = 1;
-    int ydays = crm_time_leapyear(a_time->years) ? 366 : 365;
+    pcmk__assert(a_time != NULL);
 
     crm_trace("Adding %d days to %.4d-%.3d", extra, a_time->years, a_time->days);
 
+    if (extra > 0) {
+        while ((a_time->days + (long long) extra) > ydays(a_time)) {
+            if ((a_time->years + 1LL) > INT_MAX) {
+                a_time->days = ydays(a_time); // Clip to latest we can handle
+                return;
+            }
+            extra -= ydays(a_time);
+            a_time->years++;
+        }
+    } else if (extra < 0) {
+        const int min_days = a_time->duration? 0 : 1;
+
+        while ((a_time->days + (long long) extra) < min_days) {
+            if ((a_time->years - 1) < 1) {
+                a_time->days = 1; // Clip to earliest we can handle (no BCE)
+                return;
+            }
+            a_time->years--;
+            extra += ydays(a_time);
+        }
+    }
     a_time->days += extra;
-    while (a_time->days > ydays) {
-        a_time->years++;
-        a_time->days -= ydays;
-        ydays = crm_time_leapyear(a_time->years) ? 366 : 365;
-    }
-
-    if(a_time->duration) {
-        lower_bound = 0;
-    }
-
-    while (a_time->days < lower_bound) {
-        a_time->years--;
-        a_time->days += crm_time_leapyear(a_time->years) ? 366 : 365;
-    }
 }
 
 void
@@ -1824,7 +1890,15 @@ crm_time_add_weeks(crm_time_t * a_time, int extra)
 void
 crm_time_add_years(crm_time_t * a_time, int extra)
 {
-    a_time->years += extra;
+    pcmk__assert(a_time != NULL);
+
+    if ((extra > 0) && ((a_time->years + (long long) extra) > INT_MAX)) {
+        a_time->years = INT_MAX;
+    } else if ((extra < 0) && ((a_time->years + (long long) extra) < 1)) {
+        a_time->years = 1; // Clip to earliest we can handle (no BCE)
+    } else {
+        a_time->years += extra;
+    }
 }
 
 static void
@@ -1851,8 +1925,6 @@ ha_get_tm_time(struct tm *target, const crm_time_t *source)
  * @TODO The long-term goal is to come up with a clean, unified design for a
  *       time type (or types) that meets all the various needs, to replace
  *       crm_time_t, pcmk__time_hr_t, and struct timespec (in lrmd_cmd_t).
- *       Using glib's GDateTime is a possibility (if we are willing to require
- *       glib >= 2.26).
  */
 
 pcmk__time_hr_t *
@@ -1882,7 +1954,7 @@ pcmk__time_hr_convert(pcmk__time_hr_t *target, const crm_time_t *dt)
 void
 pcmk__time_set_hr_dt(crm_time_t *target, const pcmk__time_hr_t *hr_dt)
 {
-    CRM_ASSERT((hr_dt) && (target));
+    pcmk__assert((target != NULL) && (hr_dt != NULL));
     *target = (crm_time_t) {
         .years = hr_dt->years,
         .months = hr_dt->months,
