@@ -45,10 +45,13 @@ typedef time_t log_time_t;
 unsigned int crm_log_level = LOG_INFO;
 unsigned int crm_trace_nonlog = 0;
 bool pcmk__is_daemon = false;
-char *pcmk__our_nodename = NULL;
 
 static unsigned int crm_log_priority = LOG_NOTICE;
-static GLogFunc glib_log_default = NULL;
+static guint pcmk__log_id = 0;
+static guint pcmk__glib_log_id = 0;
+static guint pcmk__gio_log_id = 0;
+static guint pcmk__gmodule_log_id = 0;
+static guint pcmk__gthread_log_id = 0;
 static pcmk__output_t *logger_out = NULL;
 
 pcmk__config_error_func pcmk__config_error_handler = NULL;
@@ -130,9 +133,20 @@ crm_trigger_blackbox(int nsig)
 void
 crm_log_deinit(void)
 {
-    if (glib_log_default != NULL) {
-        g_log_set_default_handler(glib_log_default, NULL);
+    if (pcmk__log_id == 0) {
+        return;
     }
+
+    g_log_remove_handler(G_LOG_DOMAIN, pcmk__log_id);
+    pcmk__log_id = 0;
+    g_log_remove_handler("GLib", pcmk__glib_log_id);
+    pcmk__glib_log_id = 0;
+    g_log_remove_handler("GLib-GIO", pcmk__gio_log_id);
+    pcmk__gio_log_id = 0;
+    g_log_remove_handler("GModule", pcmk__gmodule_log_id);
+    pcmk__gmodule_log_id = 0;
+    g_log_remove_handler("GThread", pcmk__gthread_log_id);
+    pcmk__gthread_log_id = 0;
 }
 
 #define FMT_MAX 256
@@ -155,7 +169,7 @@ set_format_string(int method, const char *daemon, pid_t use_pid,
 {
     if (method == QB_LOG_SYSLOG) {
         // The system log gets a simplified, user-friendly format
-        crm_extended_logging(method, QB_FALSE);
+        qb_log_ctl(method, QB_LOG_CONF_EXTENDED, QB_FALSE);
         qb_log_format_set(method, "%g %p: %b");
 
     } else {
@@ -352,14 +366,14 @@ pcmk__add_logfile(const char *filename)
     logfile = fopen(filename, "a");
     if (logfile == NULL) {
         rc = errno;
-        crm_warn("Logging to '%s' is disabled: %s " CRM_XS " uid=%u gid=%u",
+        crm_warn("Logging to '%s' is disabled: %s " QB_XS " uid=%u gid=%u",
                  filename, strerror(rc), geteuid(), getegid());
         return rc;
     }
 
     rc = set_logfile_permissions(filename, logfile);
     if (rc != pcmk_rc_ok) {
-        crm_warn("Logging to '%s' is disabled: %s " CRM_XS " permissions",
+        crm_warn("Logging to '%s' is disabled: %s " QB_XS " permissions",
                  filename, strerror(rc));
         fclose(logfile);
         return rc;
@@ -369,7 +383,7 @@ pcmk__add_logfile(const char *filename)
     fclose(logfile);
     fd = qb_log_file_open(filename);
     if (fd < 0) {
-        crm_warn("Logging to '%s' is disabled: %s " CRM_XS " qb_log_file_open",
+        crm_warn("Logging to '%s' is disabled: %s " QB_XS " qb_log_file_open",
                  filename, strerror(-fd));
         return -fd; // == +errno
     }
@@ -816,6 +830,7 @@ crm_log_preinit(const char *entity, int argc, char *const *argv)
     pid_t pid = getpid();
     const char *nodename = "localhost";
     static bool have_logging = false;
+    GLogLevelFlags log_levels;
 
     if (have_logging) {
         return;
@@ -823,7 +838,7 @@ crm_log_preinit(const char *entity, int argc, char *const *argv)
 
     have_logging = true;
 
-    crm_xml_init(); /* Sets buffer allocation strategy */
+    pcmk__xml_init();
 
     if (crm_trace_nonlog == 0) {
         crm_trace_nonlog = g_quark_from_static_string("Pacemaker non-logging tracepoint");
@@ -831,11 +846,21 @@ crm_log_preinit(const char *entity, int argc, char *const *argv)
 
     umask(S_IWGRP | S_IWOTH | S_IROTH);
 
-    /* Redirect messages from glib functions to our handler */
-    glib_log_default = g_log_set_default_handler(crm_glib_handler, NULL);
+    /* Add a log handler for messages from our log domain at any log level. */
+    log_levels = G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION;
+    pcmk__log_id = g_log_set_handler(G_LOG_DOMAIN, log_levels, crm_glib_handler, NULL);
+    /* Add a log handler for messages from the GLib domains at any log level. */
+    pcmk__glib_log_id = g_log_set_handler("GLib", log_levels, crm_glib_handler, NULL);
+    pcmk__gio_log_id = g_log_set_handler("GLib-GIO", log_levels, crm_glib_handler, NULL);
+    pcmk__gmodule_log_id = g_log_set_handler("GModule", log_levels, crm_glib_handler, NULL);
+    pcmk__gthread_log_id = g_log_set_handler("GThread", log_levels, crm_glib_handler, NULL);
 
-    /* and for good measure... - this enum is a bit field (!) */
-    g_log_set_always_fatal((GLogLevelFlags) 0); /*value out of range */
+    /* glib should not abort for any messages from the Pacemaker domain, but
+     * other domains are still free to specify their own behavior.  However,
+     * note that G_LOG_LEVEL_ERROR is always fatal regardless of what we do
+     * here.
+     */
+    g_log_set_fatal_mask(G_LOG_DOMAIN, 0);
 
     /* Set crm_system_name, which is used as the logging name. It may also
      * be used for other purposes such as an IPC client name.
@@ -880,10 +905,10 @@ crm_log_preinit(const char *entity, int argc, char *const *argv)
     setlocale(LC_ALL, "");
 
     // Tell gettext where to find Pacemaker message catalogs
-    CRM_ASSERT(bindtextdomain(PACKAGE, PCMK__LOCALE_DIR) != NULL);
+    pcmk__assert(bindtextdomain(PACKAGE, PCMK__LOCALE_DIR) != NULL);
 
     // Tell gettext to use the Pacemaker message catalogs
-    CRM_ASSERT(textdomain(PACKAGE) != NULL);
+    pcmk__assert(textdomain(PACKAGE) != NULL);
 
     // Tell gettext that the translated strings are stored in UTF-8
     bind_textdomain_codeset(PACKAGE, "UTF-8");
@@ -1257,33 +1282,6 @@ pcmk__free_common_logger(void)
     }
 }
 
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-#include <crm/common/logging_compat.h>
-
-gboolean
-crm_log_cli_init(const char *entity)
-{
-    pcmk__cli_init_logging(entity, 0);
-    return TRUE;
-}
-
-gboolean
-crm_add_logfile(const char *filename)
-{
-    return pcmk__add_logfile(filename) == pcmk_rc_ok;
-}
-
-void
-pcmk_log_xml_impl(uint8_t level, const char *text, const xmlNode *xml)
-{
-    pcmk_log_xml_as(__FILE__, __func__, __LINE__, 0, level, text, xml);
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
-
 void pcmk__set_config_error_handler(pcmk__config_error_func error_handler, void *error_context)
 {
     pcmk__config_error_handler = error_handler;
