@@ -9,10 +9,6 @@
 
 #include <crm_internal.h>
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
@@ -189,7 +185,7 @@ mainloop_add_trigger(int priority, int (*dispatch) (gpointer user_data),
 {
     GSource *source = NULL;
 
-    CRM_ASSERT(sizeof(crm_trigger_t) > sizeof(GSource));
+    pcmk__assert(sizeof(crm_trigger_t) > sizeof(GSource));
     source = g_source_new(&crm_trigger_funcs, sizeof(crm_trigger_t));
 
     return mainloop_setup_trigger(source, priority, dispatch, userdata);
@@ -255,7 +251,7 @@ crm_signal_dispatch(GSource *source, GSourceFunc callback, gpointer userdata)
     crm_signal_t *sig = (crm_signal_t *) source;
 
     if(sig->signal != SIGCHLD) {
-        crm_notice("Caught '%s' signal "CRM_XS" %d (%s handler)",
+        crm_notice("Caught '%s' signal " QB_XS " %d (%s handler)",
                    strsignal(sig->signal), sig->signal,
                    (sig->handler? "invoking" : "no"));
     }
@@ -335,10 +331,11 @@ mainloop_destroy_signal_entry(int sig)
 {
     crm_signal_t *tmp = crm_signals[sig];
 
-    crm_signals[sig] = NULL;
-
-    crm_trace("Destroying signal %d", sig);
-    mainloop_destroy_trigger((crm_trigger_t *) tmp);
+    if (tmp != NULL) {
+        crm_signals[sig] = NULL;
+        crm_trace("Unregistering mainloop handler for signal %d", sig);
+        mainloop_destroy_trigger((crm_trigger_t *) tmp);
+    }
 }
 
 /*!
@@ -379,11 +376,11 @@ mainloop_add_signal(int sig, void (*dispatch) (int sig))
         return FALSE;
     }
 
-    CRM_ASSERT(sizeof(crm_signal_t) > sizeof(GSource));
+    pcmk__assert(sizeof(crm_signal_t) > sizeof(GSource));
     source = g_source_new(&crm_signal_funcs, sizeof(crm_signal_t));
 
     crm_signals[sig] = (crm_signal_t *) mainloop_setup_trigger(source, priority, NULL, NULL);
-    CRM_ASSERT(crm_signals[sig] != NULL);
+    pcmk__assert(crm_signals[sig] != NULL);
 
     crm_signals[sig]->handler = dispatch;
     crm_signals[sig]->signal = sig;
@@ -419,8 +416,9 @@ static qb_array_t *gio_map = NULL;
 void
 mainloop_cleanup(void) 
 {
-    if (gio_map) {
+    if (gio_map != NULL) {
         qb_array_free(gio_map);
+        gio_map = NULL;
     }
 
     for (int sig = 0; sig < NSIG; ++sig) {
@@ -450,7 +448,7 @@ gio_read_socket(GIOChannel * gio, GIOCondition condition, gpointer data)
 
     /* if this assert get's hit, then there is a race condition between
      * when we destroy a fd and when mainloop actually gives it up */
-    CRM_ASSERT(adaptor->is_used > 0);
+    pcmk__assert(adaptor->is_used > 0);
 
     return (adaptor->fn(fd, condition, adaptor->data) == 0);
 }
@@ -461,7 +459,7 @@ gio_poll_destroy(gpointer data)
     struct gio_to_qb_poll *adaptor = (struct gio_to_qb_poll *)data;
 
     adaptor->is_used--;
-    CRM_ASSERT(adaptor->is_used >= 0);
+    pcmk__assert(adaptor->is_used >= 0);
 
     if (adaptor->is_used == 0) {
         crm_trace("Marking adaptor %p unused", adaptor);
@@ -666,7 +664,7 @@ mainloop_add_ipc_server_with_prio(const char *name, enum qb_ipc_type type,
         qb_ipcs_request_rate_limit(server, conv_libqb_prio2ratelimit(prio));
     }
 
-    /* All clients should use at least ipc_buffer_max as their buffer size */
+    // All clients should use at least PCMK_ipc_buffer as their buffer size
     qb_ipcs_enforce_buffer_size(server, crm_ipc_default_buffer_size());
     qb_ipcs_poll_handlers_set(server, &gio_poll_funcs);
 
@@ -718,7 +716,7 @@ mainloop_gio_callback(GIOChannel *gio, GIOCondition condition, gpointer data)
     gboolean rc = G_SOURCE_CONTINUE;
     mainloop_io_t *client = data;
 
-    CRM_ASSERT(client->fd == g_io_channel_unix_get_fd(gio));
+    pcmk__assert(client->fd == g_io_channel_unix_get_fd(gio));
 
     if (condition & G_IO_IN) {
         if (client->ipc) {
@@ -758,7 +756,7 @@ mainloop_gio_callback(GIOChannel *gio, GIOCondition condition, gpointer data)
     }
 
     if (client->ipc && !crm_ipc_connected(client->ipc)) {
-        crm_err("Connection to %s closed " CRM_XS "client=%p condition=%d",
+        crm_err("Connection to %s closed " QB_XS " client=%p condition=%d",
                 client->name, client, condition);
         rc = G_SOURCE_REMOVE;
 
@@ -1100,7 +1098,7 @@ child_timeout_callback(gpointer p)
     child->timeout = TRUE;
     crm_debug("%s process (PID %d) timed out", child->desc, (int)child->pid);
 
-    child->timerid = g_timeout_add(5000, child_timeout_callback, child);
+    child->timerid = pcmk__create_timer(5000, child_timeout_callback, child);
     return FALSE;
 }
 
@@ -1266,7 +1264,7 @@ mainloop_child_add_with_flags(pid_t pid, int timeout, const char *desc, void *pr
     child->desc = pcmk__str_copy(desc);
 
     if (timeout) {
-        child->timerid = g_timeout_add(timeout, child_timeout_callback, child);
+        child->timerid = pcmk__create_timer(timeout, child_timeout_callback, child);
     }
 
     child_list = g_list_append(child_list, child);
@@ -1277,7 +1275,7 @@ mainloop_child_add_with_flags(pid_t pid, int timeout, const char *desc, void *pr
          * We do not want it to be possible to both add a child pid
          * to mainloop, and have the pid's exit callback invoked within
          * the same callstack. */
-        g_timeout_add(1, child_signal_init, NULL);
+        pcmk__create_timer(1, child_signal_init, NULL);
     }
 }
 
@@ -1295,7 +1293,7 @@ mainloop_timer_cb(gpointer user_data)
     bool repeat = FALSE;
     struct mainloop_timer_s *t = user_data;
 
-    CRM_ASSERT(t != NULL);
+    pcmk__assert(t != NULL);
 
     id = t->id;
     t->id = 0; /* Ensure it's unset during callbacks so that
@@ -1334,7 +1332,7 @@ mainloop_timer_start(mainloop_timer_t *t)
     mainloop_timer_stop(t);
     if(t && t->period_ms > 0) {
         crm_trace("Starting timer %s", t->name);
-        t->id = g_timeout_add(t->period_ms, mainloop_timer_cb, t);
+        t->id = pcmk__create_timer(t->period_ms, mainloop_timer_cb, t);
     }
 }
 
@@ -1455,7 +1453,7 @@ pcmk_drain_main_loop(GMainLoop *mloop, guint timer_ms, bool (*check)(guint))
     if (ctx) {
         time_t start_time = time(NULL);
 
-        timer = g_timeout_add(timer_ms, drain_timeout_cb, &timeout_popped);
+        timer = pcmk__create_timer(timer_ms, drain_timeout_cb, &timeout_popped);
         while (!timeout_popped
                && check(timer_ms - (time(NULL) - start_time) * 1000)) {
             g_main_context_iteration(ctx, TRUE);
@@ -1465,17 +1463,3 @@ pcmk_drain_main_loop(GMainLoop *mloop, guint timer_ms, bool (*check)(guint))
         g_source_remove(timer);
     }
 }
-
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-#include <crm/common/mainloop_compat.h>
-
-gboolean
-crm_signal(int sig, void (*dispatch) (int sig))
-{
-    return crm_signal_handler(sig, dispatch) != SIG_ERR;
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
