@@ -7,10 +7,6 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <crm_internal.h>
 
 #include <stdio.h>
@@ -131,8 +127,7 @@ static const pcmk__cluster_option_t cluster_options[] = {
         NULL,
     },
     {
-        PCMK_OPT_JOIN_INTEGRATION_TIMEOUT, "crmd-integration-timeout",
-            PCMK_VALUE_DURATION, NULL,
+        PCMK_OPT_JOIN_INTEGRATION_TIMEOUT, NULL, PCMK_VALUE_DURATION, NULL,
         "3min", pcmk__valid_interval_spec,
         pcmk__opt_controld|pcmk__opt_advanced,
         N_("If you need to adjust this value, it probably indicates "
@@ -140,8 +135,7 @@ static const pcmk__cluster_option_t cluster_options[] = {
         NULL,
     },
     {
-        PCMK_OPT_JOIN_FINALIZATION_TIMEOUT, "crmd-finalization-timeout",
-            PCMK_VALUE_DURATION, NULL,
+        PCMK_OPT_JOIN_FINALIZATION_TIMEOUT, NULL, PCMK_VALUE_DURATION, NULL,
         "30min", pcmk__valid_interval_spec,
         pcmk__opt_controld|pcmk__opt_advanced,
         N_("If you need to adjust this value, it probably indicates "
@@ -149,8 +143,7 @@ static const pcmk__cluster_option_t cluster_options[] = {
         NULL,
     },
     {
-        PCMK_OPT_TRANSITION_DELAY, "crmd-transition-delay", PCMK_VALUE_DURATION,
-            NULL,
+        PCMK_OPT_TRANSITION_DELAY, NULL, PCMK_VALUE_DURATION, NULL,
         "0s", pcmk__valid_interval_spec,
         pcmk__opt_controld|pcmk__opt_advanced,
         N_("Enabling this option will slow down cluster recovery under all "
@@ -162,7 +155,8 @@ static const pcmk__cluster_option_t cluster_options[] = {
     {
         PCMK_OPT_NO_QUORUM_POLICY, NULL, PCMK_VALUE_SELECT,
             PCMK_VALUE_STOP ", " PCMK_VALUE_FREEZE ", " PCMK_VALUE_IGNORE
-                ", " PCMK_VALUE_DEMOTE ", " PCMK_VALUE_FENCE_LEGACY,
+                ", " PCMK_VALUE_DEMOTE ", " PCMK_VALUE_FENCE ", "
+                PCMK_VALUE_FENCE_LEGACY,
         PCMK_VALUE_STOP, pcmk__valid_no_quorum_policy,
         pcmk__opt_schedulerd,
         N_("What to do when the cluster does not have quorum"),
@@ -247,11 +241,10 @@ static const pcmk__cluster_option_t cluster_options[] = {
     },
     {
         PCMK_OPT_STONITH_ACTION, NULL, PCMK_VALUE_SELECT,
-            PCMK_ACTION_REBOOT ", " PCMK_ACTION_OFF ", " PCMK__ACTION_POWEROFF,
+            PCMK_ACTION_REBOOT ", " PCMK_ACTION_OFF,
         PCMK_ACTION_REBOOT, pcmk__is_fencing_action,
         pcmk__opt_schedulerd,
-        N_("Action to send to fence device when a node needs to be fenced "
-            "(\"poweroff\" is a deprecated alias for \"off\")"),
+        N_("Action to send to fence device when a node needs to be fenced"),
         NULL,
     },
     {
@@ -316,8 +309,13 @@ static const pcmk__cluster_option_t cluster_options[] = {
     },
     {
         PCMK_OPT_CONCURRENT_FENCING, NULL, PCMK_VALUE_BOOLEAN, NULL,
-        PCMK__CONCURRENT_FENCING_DEFAULT, pcmk__valid_boolean,
-        pcmk__opt_schedulerd,
+#if PCMK__CONCURRENT_FENCING_DEFAULT_TRUE
+        PCMK_VALUE_TRUE,
+#else
+        PCMK_VALUE_FALSE,
+#endif
+        pcmk__valid_boolean,
+        pcmk__opt_schedulerd|pcmk__opt_deprecated,
         N_("Allow performing fencing operations in parallel"),
         NULL,
     },
@@ -454,14 +452,6 @@ static const pcmk__cluster_option_t cluster_options[] = {
             "configuration"),
         NULL,
     },
-    {
-        PCMK__OPT_REMOVE_AFTER_STOP, NULL, PCMK_VALUE_BOOLEAN, NULL,
-        PCMK_VALUE_FALSE, pcmk__valid_boolean,
-        pcmk__opt_schedulerd|pcmk__opt_deprecated,
-        N_("Whether to remove stopped resources from the executor"),
-        N_("Values other than default are poorly tested and potentially "
-            "dangerous."),
-    },
 
     // Storing inputs
     {
@@ -558,14 +548,13 @@ static const pcmk__cluster_option_t fencing_params[] = {
      */
     {
         PCMK_STONITH_HOST_ARGUMENT, NULL, PCMK_VALUE_STRING, NULL,
-        "port", NULL,
+        NULL, NULL,
         pcmk__opt_advanced,
-        N_("An alternate parameter to supply instead of 'port'"),
-        N_("Some devices do not support the standard 'port' parameter or may "
-            "provide additional ones. Use this to specify an alternate, device-"
-            "specific, parameter that should indicate the machine to be "
-            "fenced. A value of \"none\" can be used to tell the cluster not "
-            "to supply any additional parameters."),
+        N_("Name of agent parameter that should be set to the fencing target"),
+        N_("If the fencing agent metadata advertises support for the \"port\" "
+            "or \"plug\" parameter, that will be used as the default, "
+            "otherwise \"none\" will be used, which tells the cluster not to "
+            "supply any additional parameters."),
     },
     {
         PCMK_STONITH_HOST_MAP, NULL, PCMK_VALUE_STRING, NULL,
@@ -633,11 +622,9 @@ static const pcmk__cluster_option_t fencing_params[] = {
         pcmk__opt_none,
         N_("The maximum number of actions can be performed in parallel on this "
             "device"),
-        N_("Cluster property concurrent-fencing=\"true\" needs to be "
-            "configured first. Then use this to specify the maximum number of "
-            "actions can be performed in parallel on this device. A value of "
-            "-1 means an unlimited number of actions can be performed in "
-            "parallel."),
+        N_("If the concurrent-fencing cluster property is \"true\", this "
+            "specifies the maximum number of actions that can be performed in "
+            "parallel on this device. A value of -1 means unlimited."),
     },
     {
         "pcmk_reboot_action", NULL, PCMK_VALUE_STRING, NULL,
@@ -1291,7 +1278,8 @@ pcmk__valid_no_quorum_policy(const char *value)
     return pcmk__strcase_any_of(value,
                                 PCMK_VALUE_STOP, PCMK_VALUE_FREEZE,
                                 PCMK_VALUE_IGNORE, PCMK_VALUE_DEMOTE,
-                                PCMK_VALUE_FENCE_LEGACY, NULL);
+                                PCMK_VALUE_FENCE, PCMK_VALUE_FENCE_LEGACY,
+                                NULL);
 }
 
 /*!
@@ -1346,7 +1334,7 @@ cluster_option_value(GHashTable *table, const pcmk__cluster_option_t *option)
 {
     const char *value = NULL;
 
-    CRM_ASSERT((option != NULL) && (option->name != NULL));
+    pcmk__assert((option != NULL) && (option->name != NULL));
 
     if (table != NULL) {
         value = g_hash_table_lookup(table, option->name);
@@ -1549,7 +1537,7 @@ pcmk__daemon_metadata(pcmk__output_t *out, const char *name,
     out->output_xml(out, PCMK_XE_METADATA, metadata_s->str);
 
     pcmk__output_free(tmp_out);
-    free_xml(top);
+    pcmk__xml_free(top);
     g_string_free(metadata_s, TRUE);
     return pcmk_rc_ok;
 }
