@@ -17,18 +17,6 @@
 
 #include "crmcommon_private.h"
 
-// @COMPAT Drop at 3.0.0
-static gboolean fancy = FALSE;
-
-// @COMPAT Drop at 3.0.0
-GOptionEntry pcmk__text_output_entries[] = {
-    { "text-fancy", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &fancy,
-      "Use more highly formatted output (requires --output-as=text)",
-      NULL },
-
-    { NULL }
-};
-
 typedef struct text_list_data_s {
     unsigned int len;
     char *singular_noun;
@@ -67,7 +55,7 @@ static bool
 text_init(pcmk__output_t *out) {
     private_data_t *priv = NULL;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     /* If text_init was previously called on this output struct, just return. */
     if (out->priv != NULL) {
@@ -85,8 +73,9 @@ text_init(pcmk__output_t *out) {
 }
 
 static void
-text_finish(pcmk__output_t *out, crm_exit_t exit_status, bool print, void **copy_dest) {
-    CRM_ASSERT(out != NULL && out->dest != NULL);
+text_finish(pcmk__output_t *out, crm_exit_t exit_status, bool print, void **copy_dest)
+{
+    pcmk__assert((out != NULL) && (out->dest != NULL));
     fflush(out->dest);
 }
 
@@ -95,13 +84,13 @@ text_reset(pcmk__output_t *out) {
     private_data_t *priv = NULL;
     bool old_fancy = false;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     if (out->dest != stdout) {
         out->dest = freopen(NULL, "w", out->dest);
     }
 
-    CRM_ASSERT(out->dest != NULL);
+    pcmk__assert(out->dest != NULL);
 
     // Save priv->fancy before free/init sequence overwrites it
     priv = out->priv;
@@ -117,7 +106,7 @@ text_reset(pcmk__output_t *out) {
 static void
 text_subprocess_output(pcmk__output_t *out, int exit_status,
                        const char *proc_stdout, const char *proc_stderr) {
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     if (proc_stdout != NULL) {
         fprintf(out->dest, "%s\n", proc_stdout);
@@ -129,8 +118,9 @@ text_subprocess_output(pcmk__output_t *out, int exit_status,
 }
 
 static void
-text_version(pcmk__output_t *out, bool extended) {
-    CRM_ASSERT(out != NULL && out->dest != NULL);
+text_version(pcmk__output_t *out, bool extended)
+{
+    pcmk__assert((out != NULL) && (out->dest != NULL));
 
     if (extended) {
         fprintf(out->dest, "Pacemaker %s (Build: %s): %s\n", PACEMAKER_VERSION, BUILD_VERSION, CRM_FEATURES);
@@ -145,17 +135,15 @@ G_GNUC_PRINTF(2, 3)
 static void
 text_err(pcmk__output_t *out, const char *format, ...) {
     va_list ap;
-    int len = 0;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     va_start(ap, format);
 
     /* Informational output does not get indented, to separate it from other
      * potentially indented list output.
      */
-    len = vfprintf(stderr, format, ap);
-    CRM_ASSERT(len >= 0);
+    vfprintf(stderr, format, ap);
     va_end(ap);
 
     /* Add a newline. */
@@ -166,9 +154,8 @@ G_GNUC_PRINTF(2, 3)
 static int
 text_info(pcmk__output_t *out, const char *format, ...) {
     va_list ap;
-    int len = 0;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     if (out->is_quiet(out)) {
         return pcmk_rc_no_output;
@@ -179,8 +166,7 @@ text_info(pcmk__output_t *out, const char *format, ...) {
     /* Informational output does not get indented, to separate it from other
      * potentially indented list output.
      */
-    len = vfprintf(out->dest, format, ap);
-    CRM_ASSERT(len >= 0);
+    vfprintf(out->dest, format, ap);
     va_end(ap);
 
     /* Add a newline. */
@@ -197,7 +183,7 @@ text_transient(pcmk__output_t *out, const char *format, ...)
 
 static void
 text_output_xml(pcmk__output_t *out, const char *name, const char *buf) {
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
     pcmk__indented_printf(out, "%s", buf);
 }
 
@@ -209,12 +195,12 @@ text_begin_list(pcmk__output_t *out, const char *singular_noun, const char *plur
     text_list_data_t *new_list = NULL;
     va_list ap;
 
-    CRM_ASSERT(out != NULL && out->priv != NULL);
+    pcmk__assert((out != NULL) && (out->priv != NULL));
     priv = out->priv;
 
     va_start(ap, format);
 
-    if ((fancy || priv->fancy) && (format != NULL)) {
+    if (priv->fancy && (format != NULL)) {
         pcmk__indented_vprintf(out, format, ap);
         fprintf(out->dest, ":\n");
     }
@@ -235,12 +221,12 @@ text_list_item(pcmk__output_t *out, const char *id, const char *format, ...) {
     private_data_t *priv = NULL;
     va_list ap;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     priv = out->priv;
     va_start(ap, format);
 
-    if (fancy || priv->fancy) {
+    if (priv->fancy) {
         if (id != NULL) {
             /* Not really a good way to do this all in one call, so make it two.
              * The first handles the indentation and list styling.  The second
@@ -267,11 +253,11 @@ text_increment_list(pcmk__output_t *out) {
     private_data_t *priv = NULL;
     gpointer tail;
 
-    CRM_ASSERT(out != NULL && out->priv != NULL);
+    pcmk__assert((out != NULL) && (out->priv != NULL));
     priv = out->priv;
 
     tail = g_queue_peek_tail(priv->parent_q);
-    CRM_ASSERT(tail != NULL);
+    pcmk__assert(tail != NULL);
     ((text_list_data_t *) tail)->len++;
 }
 
@@ -280,7 +266,7 @@ text_end_list(pcmk__output_t *out) {
     private_data_t *priv = NULL;
     text_list_data_t *node = NULL;
 
-    CRM_ASSERT(out != NULL && out->priv != NULL);
+    pcmk__assert((out != NULL) && (out->priv != NULL));
     priv = out->priv;
 
     node = g_queue_pop_tail(priv->parent_q);
@@ -298,19 +284,19 @@ text_end_list(pcmk__output_t *out) {
 
 static bool
 text_is_quiet(pcmk__output_t *out) {
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
     return out->quiet;
 }
 
 static void
 text_spacer(pcmk__output_t *out) {
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
     fprintf(out->dest, "\n");
 }
 
 static void
 text_progress(pcmk__output_t *out, bool end) {
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     if (out->dest == stdout) {
         fprintf(out->dest, ".");
@@ -373,12 +359,12 @@ pcmk__mk_text_output(char **argv) {
 bool
 pcmk__output_text_get_fancy(pcmk__output_t *out)
 {
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     if (pcmk__str_eq(out->fmt_name, "text", pcmk__str_none)) {
         private_data_t *priv = out->priv;
 
-        CRM_ASSERT(priv != NULL);
+        pcmk__assert(priv != NULL);
         return priv->fancy;
     }
     return false;
@@ -396,12 +382,12 @@ pcmk__output_text_get_fancy(pcmk__output_t *out)
 void
 pcmk__output_text_set_fancy(pcmk__output_t *out, bool enabled)
 {
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     if (pcmk__str_eq(out->fmt_name, "text", pcmk__str_none)) {
         private_data_t *priv = out->priv;
 
-        CRM_ASSERT(priv != NULL);
+        pcmk__assert(priv != NULL);
         priv->fancy = enabled;
     }
 }
@@ -409,13 +395,9 @@ pcmk__output_text_set_fancy(pcmk__output_t *out, bool enabled)
 G_GNUC_PRINTF(2, 0)
 void
 pcmk__formatted_vprintf(pcmk__output_t *out, const char *format, va_list args) {
-    int len = 0;
-
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
     CRM_CHECK(pcmk__str_eq(out->fmt_name, "text", pcmk__str_none), return);
-
-    len = vfprintf(out->dest, format, args);
-    CRM_ASSERT(len >= 0);
+    vfprintf(out->dest, format, args);
 }
 
 G_GNUC_PRINTF(2, 3)
@@ -423,7 +405,7 @@ void
 pcmk__formatted_printf(pcmk__output_t *out, const char *format, ...) {
     va_list ap;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     va_start(ap, format);
     pcmk__formatted_vprintf(out, format, ap);
@@ -435,16 +417,16 @@ void
 pcmk__indented_vprintf(pcmk__output_t *out, const char *format, va_list args) {
     private_data_t *priv = NULL;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
     CRM_CHECK(pcmk__str_eq(out->fmt_name, "text", pcmk__str_none), return);
 
     priv = out->priv;
 
-    if (fancy || priv->fancy) {
+    if (priv->fancy) {
         int level = 0;
         private_data_t *priv = out->priv;
 
-        CRM_ASSERT(priv != NULL);
+        pcmk__assert(priv != NULL);
 
         level = g_queue_get_length(priv->parent_q);
 
@@ -465,7 +447,7 @@ void
 pcmk__indented_printf(pcmk__output_t *out, const char *format, ...) {
     va_list ap;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     va_start(ap, format);
     pcmk__indented_vprintf(out, format, ap);
@@ -479,8 +461,7 @@ pcmk__text_prompt(const char *prompt, bool echo, char **dest)
     struct termios settings;
     tcflag_t orig_c_lflag = 0;
 
-    CRM_ASSERT(prompt != NULL);
-    CRM_ASSERT(dest != NULL);
+    pcmk__assert((prompt != NULL) && (dest != NULL));
 
     if (!echo) {
         rc = tcgetattr(0, &settings);
