@@ -32,10 +32,9 @@
 #include <crm/common/xml.h>
 #include <crm/common/mainloop.h>
 #include <crm/common/remote_internal.h>
+#include <crm/common/tls_internal.h>
 
-#ifdef HAVE_GNUTLS_GNUTLS_H
-#  include <gnutls/gnutls.h>
-#endif
+#include <gnutls/gnutls.h>
 
 /* Swab macros from linux/swab.h */
 #ifdef HAVE_LINUX_SWAB_H
@@ -126,261 +125,9 @@ localized_remote_header(pcmk__remote_t *remote)
     return header;
 }
 
-#ifdef HAVE_GNUTLS_GNUTLS_H
-
-int
-pcmk__tls_client_handshake(pcmk__remote_t *remote, int timeout_sec,
-                           int *gnutls_rc)
-{
-    const time_t time_limit = time(NULL) + timeout_sec;
-
-    if (gnutls_rc != NULL) {
-        *gnutls_rc = GNUTLS_E_SUCCESS;
-    }
-    do {
-        int rc = gnutls_handshake(*remote->tls_session);
-
-        switch (rc) {
-            case GNUTLS_E_SUCCESS:
-                return pcmk_rc_ok;
-
-            case GNUTLS_E_INTERRUPTED:
-            case GNUTLS_E_AGAIN:
-                rc = pcmk__remote_ready(remote, 1000);
-                if ((rc != pcmk_rc_ok) && (rc != ETIME)) { // Fatal error
-                    return rc;
-                }
-                break;
-
-            default:
-                if (gnutls_rc != NULL) {
-                    *gnutls_rc = rc;
-                }
-                return EPROTO;
-        }
-    } while (time(NULL) < time_limit);
-    return ETIME;
-}
-
-/*!
- * \internal
- * \brief Set minimum prime size required by TLS client
- *
- * \param[in] session  TLS session to affect
- */
-static void
-set_minimum_dh_bits(const gnutls_session_t *session)
-{
-    int dh_min_bits;
-
-    pcmk__scan_min_int(pcmk__env_option(PCMK__ENV_DH_MIN_BITS), &dh_min_bits,
-                       0);
-
-    /* This function is deprecated since GnuTLS 3.1.7, in favor of letting
-     * the priority string imply the DH requirements, but this is the only
-     * way to give the user control over compatibility with older servers.
-     */
-    if (dh_min_bits > 0) {
-        crm_info("Requiring server use a Diffie-Hellman prime of at least %d bits",
-                 dh_min_bits);
-        crm_warn("Support for the " PCMK__ENV_DH_MIN_BITS " "
-                 "environment variable is deprecated and will be removed "
-                 "in a future release");
-        gnutls_dh_set_prime_bits(*session, dh_min_bits);
-    }
-}
-
-static unsigned int
-get_bound_dh_bits(unsigned int dh_bits)
-{
-    int dh_min_bits;
-    int dh_max_bits;
-
-    pcmk__scan_min_int(pcmk__env_option(PCMK__ENV_DH_MIN_BITS), &dh_min_bits,
-                       0);
-    pcmk__scan_min_int(pcmk__env_option(PCMK__ENV_DH_MAX_BITS), &dh_max_bits,
-                       0);
-
-    if ((dh_max_bits > 0) && (dh_max_bits < dh_min_bits)) {
-        crm_warn("Ignoring PCMK_dh_max_bits less than PCMK_dh_min_bits");
-        dh_max_bits = 0;
-    }
-    if ((dh_min_bits > 0) && (dh_bits < dh_min_bits)) {
-        return dh_min_bits;
-    }
-    if ((dh_max_bits > 0) && (dh_bits > dh_max_bits)) {
-        return dh_max_bits;
-    }
-    return dh_bits;
-}
-
-/*!
- * \internal
- * \brief Initialize a new TLS session
- *
- * \param[in] csock       Connected socket for TLS session
- * \param[in] conn_type   GNUTLS_SERVER or GNUTLS_CLIENT
- * \param[in] cred_type   GNUTLS_CRD_ANON or GNUTLS_CRD_PSK
- * \param[in] credentials TLS session credentials
- *
- * \return Pointer to newly created session object, or NULL on error
- */
-gnutls_session_t *
-pcmk__new_tls_session(int csock, unsigned int conn_type,
-                      gnutls_credentials_type_t cred_type, void *credentials)
-{
-    int rc = GNUTLS_E_SUCCESS;
-    const char *prio_base = NULL;
-    char *prio = NULL;
-    gnutls_session_t *session = NULL;
-
-    /* Determine list of acceptable ciphers, etc. Pacemaker always adds the
-     * values required for its functionality.
-     *
-     * For an example of anonymous authentication, see:
-     * http://www.manpagez.com/info/gnutls/gnutls-2.10.4/gnutls_81.php#Echo-Server-with-anonymous-authentication
-     */
-
-    prio_base = pcmk__env_option(PCMK__ENV_TLS_PRIORITIES);
-    if (prio_base == NULL) {
-        prio_base = PCMK_GNUTLS_PRIORITIES;
-    }
-    prio = crm_strdup_printf("%s:%s", prio_base,
-                             (cred_type == GNUTLS_CRD_ANON)? "+ANON-DH" : "+DHE-PSK:+PSK");
-
-    session = gnutls_malloc(sizeof(gnutls_session_t));
-    if (session == NULL) {
-        rc = GNUTLS_E_MEMORY_ERROR;
-        goto error;
-    }
-
-    rc = gnutls_init(session, conn_type);
-    if (rc != GNUTLS_E_SUCCESS) {
-        goto error;
-    }
-
-    /* @TODO On the server side, it would be more efficient to cache the
-     * priority with gnutls_priority_init2() and set it with
-     * gnutls_priority_set() for all sessions.
-     */
-    rc = gnutls_priority_set_direct(*session, prio, NULL);
-    if (rc != GNUTLS_E_SUCCESS) {
-        goto error;
-    }
-    if (conn_type == GNUTLS_CLIENT) {
-        set_minimum_dh_bits(session);
-    }
-
-    gnutls_transport_set_ptr(*session,
-                             (gnutls_transport_ptr_t) GINT_TO_POINTER(csock));
-
-    rc = gnutls_credentials_set(*session, cred_type, credentials);
-    if (rc != GNUTLS_E_SUCCESS) {
-        goto error;
-    }
-    free(prio);
-    return session;
-
-error:
-    crm_err("Could not initialize %s TLS %s session: %s "
-            CRM_XS " rc=%d priority='%s'",
-            (cred_type == GNUTLS_CRD_ANON)? "anonymous" : "PSK",
-            (conn_type == GNUTLS_SERVER)? "server" : "client",
-            gnutls_strerror(rc), rc, prio);
-    free(prio);
-    if (session != NULL) {
-        gnutls_free(session);
-    }
-    return NULL;
-}
-
-/*!
- * \internal
- * \brief Initialize Diffie-Hellman parameters for a TLS server
- *
- * \param[out] dh_params  Parameter object to initialize
- *
- * \return Standard Pacemaker return code
- * \todo The current best practice is to allow the client and server to
- *       negotiate the Diffie-Hellman parameters via a TLS extension (RFC 7919).
- *       However, we have to support both older versions of GnuTLS (<3.6) that
- *       don't support the extension on our side, and older Pacemaker versions
- *       that don't support the extension on the other side. The next best
- *       practice would be to use a known good prime (see RFC 5114 section 2.2),
- *       possibly stored in a file distributed with Pacemaker.
- */
-int
-pcmk__init_tls_dh(gnutls_dh_params_t *dh_params)
-{
-    int rc = GNUTLS_E_SUCCESS;
-    unsigned int dh_bits = 0;
-
-    rc = gnutls_dh_params_init(dh_params);
-    if (rc != GNUTLS_E_SUCCESS) {
-        goto error;
-    }
-
-    dh_bits = gnutls_sec_param_to_pk_bits(GNUTLS_PK_DH,
-                                          GNUTLS_SEC_PARAM_NORMAL);
-    if (dh_bits == 0) {
-        rc = GNUTLS_E_DH_PRIME_UNACCEPTABLE;
-        goto error;
-    }
-    dh_bits = get_bound_dh_bits(dh_bits);
-
-    crm_info("Generating Diffie-Hellman parameters with %u-bit prime for TLS",
-             dh_bits);
-    rc = gnutls_dh_params_generate2(*dh_params, dh_bits);
-    if (rc != GNUTLS_E_SUCCESS) {
-        goto error;
-    }
-
-    return pcmk_rc_ok;
-
-error:
-    crm_err("Could not initialize Diffie-Hellman parameters for TLS: %s "
-            CRM_XS " rc=%d", gnutls_strerror(rc), rc);
-    return EPROTO;
-}
-
-/*!
- * \internal
- * \brief Process handshake data from TLS client
- *
- * Read as much TLS handshake data as is available.
- *
- * \param[in] client  Client connection
- *
- * \return Standard Pacemaker return code (of particular interest, EAGAIN
- *         if some data was successfully read but more data is needed)
- */
-int
-pcmk__read_handshake_data(const pcmk__client_t *client)
-{
-    int rc = 0;
-
-    CRM_ASSERT(client && client->remote && client->remote->tls_session);
-
-    do {
-        rc = gnutls_handshake(*client->remote->tls_session);
-    } while (rc == GNUTLS_E_INTERRUPTED);
-
-    if (rc == GNUTLS_E_AGAIN) {
-        /* No more data is available at the moment. This function should be
-         * invoked again once the client sends more.
-         */
-        return EAGAIN;
-    } else if (rc != GNUTLS_E_SUCCESS) {
-        crm_err("TLS handshake with remote client failed: %s "
-                CRM_XS " rc=%d", gnutls_strerror(rc), rc);
-        return EPROTO;
-    }
-    return pcmk_rc_ok;
-}
-
 // \return Standard Pacemaker return code
 static int
-send_tls(gnutls_session_t *session, struct iovec *iov)
+send_tls(gnutls_session_t session, struct iovec *iov)
 {
     const char *unsent = iov->iov_base;
     size_t unsent_len = iov->iov_len;
@@ -393,7 +140,7 @@ send_tls(gnutls_session_t *session, struct iovec *iov)
     crm_trace("Sending TLS message of %llu bytes",
               (unsigned long long) unsent_len);
     while (true) {
-        gnutls_rc = gnutls_record_send(*session, unsent, unsent_len);
+        gnutls_rc = gnutls_record_send(session, unsent, unsent_len);
 
         if (gnutls_rc == GNUTLS_E_INTERRUPTED || gnutls_rc == GNUTLS_E_AGAIN) {
             crm_trace("Retrying to send %llu bytes remaining",
@@ -401,7 +148,7 @@ send_tls(gnutls_session_t *session, struct iovec *iov)
 
         } else if (gnutls_rc < 0) {
             // Caller can log as error if necessary
-            crm_info("TLS connection terminated: %s " CRM_XS " rc=%lld",
+            crm_info("TLS connection terminated: %s " QB_XS " rc=%lld",
                      gnutls_strerror((int) gnutls_rc),
                      (long long) gnutls_rc);
             return ECONNABORTED;
@@ -418,7 +165,6 @@ send_tls(gnutls_session_t *session, struct iovec *iov)
     }
     return pcmk_rc_ok;
 }
-#endif
 
 // \return Standard Pacemaker return code
 static int
@@ -446,7 +192,7 @@ send_plaintext(int sock, struct iovec *iov)
             }
 
             // Caller can log as error if necessary
-            crm_info("Could not send message: %s " CRM_XS " rc=%d socket=%d",
+            crm_info("Could not send message: %s " QB_XS " rc=%d socket=%d",
                      pcmk_rc_str(rc), rc, sock);
             return rc;
 
@@ -473,12 +219,10 @@ remote_send_iovs(pcmk__remote_t *remote, struct iovec *iov, int iovs)
     int rc = pcmk_rc_ok;
 
     for (int lpc = 0; (lpc < iovs) && (rc == pcmk_rc_ok); lpc++) {
-#ifdef HAVE_GNUTLS_GNUTLS_H
         if (remote->tls_session) {
             rc = send_tls(remote->tls_session, &(iov[lpc]));
             continue;
         }
-#endif
         if (remote->tcp_socket) {
             rc = send_plaintext(remote->tcp_socket, &(iov[lpc]));
         } else {
@@ -532,7 +276,7 @@ pcmk__remote_send_xml(pcmk__remote_t *remote, const xmlNode *msg)
 
     rc = remote_send_iovs(remote, iov, 2);
     if (rc != pcmk_rc_ok) {
-        crm_err("Could not send remote message: %s " CRM_XS " rc=%d",
+        crm_err("Could not send remote message: %s " QB_XS " rc=%d",
                 pcmk_rc_str(rc), rc);
     }
 
@@ -582,13 +326,13 @@ pcmk__remote_message_xml(pcmk__remote_t *remote)
             return NULL;
 
         } else if (rc != pcmk_rc_ok) {
-            crm_err("Decompression failed: %s " CRM_XS " rc=%d",
+            crm_err("Decompression failed: %s " QB_XS " rc=%d",
                     pcmk_rc_str(rc), rc);
             free(uncompressed);
             return NULL;
         }
 
-        CRM_ASSERT(size_u == header->payload_uncompressed);
+        pcmk__assert(size_u == header->payload_uncompressed);
 
         memcpy(uncompressed, remote->buffer, header->payload_offset);       /* Preserve the header */
         remote->buffer_size = header->payload_offset + size_u;
@@ -612,19 +356,18 @@ pcmk__remote_message_xml(pcmk__remote_t *remote)
         crm_err("Couldn't parse: '%.120s'", remote->buffer + header->payload_offset);
     }
 
+    crm_log_xml_trace(xml, "[remote msg]");
     return xml;
 }
 
 static int
 get_remote_socket(const pcmk__remote_t *remote)
 {
-#ifdef HAVE_GNUTLS_GNUTLS_H
     if (remote->tls_session) {
-        void *sock_ptr = gnutls_transport_get_ptr(*remote->tls_session);
+        void *sock_ptr = gnutls_transport_get_ptr(remote->tls_session);
 
         return GPOINTER_TO_INT(sock_ptr);
     }
-#endif
 
     if (remote->tcp_socket) {
         return remote->tcp_socket;
@@ -697,13 +440,12 @@ pcmk__remote_ready(const pcmk__remote_t *remote, int timeout_ms)
  * \note This function will return when the socket read buffer is empty or an
  *       error is encountered.
  */
-static int
-read_available_remote_data(pcmk__remote_t *remote)
+int
+pcmk__read_available_remote_data(pcmk__remote_t *remote)
 {
     int rc = pcmk_rc_ok;
     size_t read_len = sizeof(struct remote_header_v0);
     struct remote_header_v0 *header = localized_remote_header(remote);
-    bool received = false;
     ssize_t read_rc;
 
     if(header) {
@@ -719,9 +461,8 @@ read_available_remote_data(pcmk__remote_t *remote)
         remote->buffer = pcmk__realloc(remote->buffer, remote->buffer_size + 1);
     }
 
-#ifdef HAVE_GNUTLS_GNUTLS_H
-    if (!received && remote->tls_session) {
-        read_rc = gnutls_record_recv(*(remote->tls_session),
+    if (remote->tls_session) {
+        read_rc = gnutls_record_recv(remote->tls_session,
                                      remote->buffer + remote->buffer_offset,
                                      remote->buffer_size - remote->buffer_offset);
         if (read_rc == GNUTLS_E_INTERRUPTED) {
@@ -733,21 +474,14 @@ read_available_remote_data(pcmk__remote_t *remote)
                       gnutls_strerror(read_rc), (long long) read_rc);
             rc = EIO;
         }
-        received = true;
-    }
-#endif
-
-    if (!received && remote->tcp_socket) {
+    } else if (remote->tcp_socket) {
         read_rc = read(remote->tcp_socket,
                        remote->buffer + remote->buffer_offset,
                        remote->buffer_size - remote->buffer_offset);
         if (read_rc < 0) {
             rc = errno;
         }
-        received = true;
-    }
-
-    if (!received) {
+    } else {
         crm_err("Remote connection type undetermined (bug?)");
         return ESOCKTNOSUPPORT;
     }
@@ -830,16 +564,16 @@ pcmk__read_remote_message(pcmk__remote_t *remote, int timeout_ms)
 
         } else if (rc != pcmk_rc_ok) {
             crm_debug("Wait for remote data aborted (will retry): %s "
-                      CRM_XS " rc=%d", pcmk_rc_str(rc), rc);
+                      QB_XS " rc=%d", pcmk_rc_str(rc), rc);
 
         } else {
-            rc = read_available_remote_data(remote);
+            rc = pcmk__read_available_remote_data(remote);
             if (rc == pcmk_rc_ok) {
                 return rc;
             } else if (rc == EAGAIN) {
                 crm_trace("Waiting for more remote data");
             } else {
-                crm_debug("Could not receive remote data: %s " CRM_XS " rc=%d",
+                crm_debug("Could not receive remote data: %s " QB_XS " rc=%d",
                           pcmk_rc_str(rc), rc);
             }
         }
@@ -887,7 +621,7 @@ check_connect_finished(gpointer userdata)
     if (rc < 0) { // select() error
         rc = errno;
         if ((rc == EINPROGRESS) || (rc == EAGAIN)) {
-            if ((time(NULL) - cb_data->start) < (cb_data->timeout_ms / 1000)) {
+            if ((time(NULL) - cb_data->start) < pcmk__timeout_ms2s(cb_data->timeout_ms)) {
                 return TRUE; // There is time left, so reschedule timer
             } else {
                 rc = ETIMEDOUT;
@@ -897,7 +631,7 @@ check_connect_finished(gpointer userdata)
                   cb_data->sock, pcmk_rc_str(rc), rc);
 
     } else if (rc == 0) { // select() timeout
-        if ((time(NULL) - cb_data->start) < (cb_data->timeout_ms / 1000)) {
+        if ((time(NULL) - cb_data->start) < pcmk__timeout_ms2s(cb_data->timeout_ms)) {
             return TRUE; // There is time left, so reschedule timer
         }
         crm_debug("Timed out while waiting for socket %d connection success",
@@ -976,7 +710,7 @@ connect_socket_retry(int sock, const struct sockaddr *addr, socklen_t addrlen,
 
     rc = pcmk__set_nonblocking(sock);
     if (rc != pcmk_rc_ok) {
-        crm_warn("Could not set socket non-blocking: %s " CRM_XS " rc=%d",
+        crm_warn("Could not set socket non-blocking: %s " QB_XS " rc=%d",
                  pcmk_rc_str(rc), rc);
         return rc;
     }
@@ -984,7 +718,7 @@ connect_socket_retry(int sock, const struct sockaddr *addr, socklen_t addrlen,
     rc = connect(sock, addr, addrlen);
     if (rc < 0 && (errno != EINPROGRESS) && (errno != EAGAIN)) {
         rc = errno;
-        crm_warn("Could not connect socket: %s " CRM_XS " rc=%d",
+        crm_warn("Could not connect socket: %s " QB_XS " rc=%d",
                  pcmk_rc_str(rc), rc);
         return rc;
     }
@@ -1017,7 +751,7 @@ connect_socket_retry(int sock, const struct sockaddr *addr, socklen_t addrlen,
      */
     crm_trace("Scheduling check in %dms for whether connect to fd %d finished",
               interval, sock);
-    timer = g_timeout_add(interval, check_connect_finished, cb_data);
+    timer = pcmk__create_timer(interval, check_connect_finished, cb_data);
     if (timer_id) {
         *timer_id = timer;
     }
@@ -1044,14 +778,14 @@ connect_socket_once(int sock, const struct sockaddr *addr, socklen_t addrlen)
 
     if (rc < 0) {
         rc = errno;
-        crm_warn("Could not connect socket: %s " CRM_XS " rc=%d",
+        crm_warn("Could not connect socket: %s " QB_XS " rc=%d",
                  pcmk_rc_str(rc), rc);
         return rc;
     }
 
     rc = pcmk__set_nonblocking(sock);
     if (rc != pcmk_rc_ok) {
-        crm_warn("Could not set socket non-blocking: %s " CRM_XS " rc=%d",
+        crm_warn("Could not set socket non-blocking: %s " QB_XS " rc=%d",
                  pcmk_rc_str(rc), rc);
         return rc;
     }
@@ -1128,7 +862,7 @@ pcmk__connect_remote(const char *host, int port, int timeout, int *timer_id,
         if (sock == -1) {
             rc = errno;
             crm_warn("Could not create socket for remote connection to %s:%d: "
-                     "%s " CRM_XS " rc=%d", server, port, pcmk_rc_str(rc), rc);
+                     "%s " QB_XS " rc=%d", server, port, pcmk_rc_str(rc), rc);
             continue;
         }
 
@@ -1227,7 +961,7 @@ pcmk__accept_remote_connection(int ssock, int *csock)
     if (*csock == -1) {
         rc = errno;
         crm_err("Could not accept remote client connection: %s "
-                CRM_XS " rc=%d", pcmk_rc_str(rc), rc);
+                QB_XS " rc=%d", pcmk_rc_str(rc), rc);
         return rc;
     }
     pcmk__sockaddr2str(&addr, addr_str);
@@ -1235,7 +969,7 @@ pcmk__accept_remote_connection(int ssock, int *csock)
 
     rc = pcmk__set_nonblocking(*csock);
     if (rc != pcmk_rc_ok) {
-        crm_err("Could not set socket non-blocking: %s " CRM_XS " rc=%d",
+        crm_err("Could not set socket non-blocking: %s " QB_XS " rc=%d",
                 pcmk_rc_str(rc), rc);
         close(*csock);
         *csock = -1;
@@ -1254,7 +988,7 @@ pcmk__accept_remote_connection(int ssock, int *csock)
         if (rc < 0) {
             rc = errno;
             crm_err("Could not set TCP timeout to %d ms on remote connection: "
-                    "%s " CRM_XS " rc=%d", optval, pcmk_rc_str(rc), rc);
+                    "%s " QB_XS " rc=%d", optval, pcmk_rc_str(rc), rc);
             close(*csock);
             *csock = -1;
             return rc;
