@@ -9,15 +9,13 @@
 
 #include <crm_internal.h>
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <bzlib.h>
 #include <errno.h>
 #include <netdb.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <qb/qbdefs.h>
 
 #include <crm/common/mainloop.h>
@@ -93,7 +91,7 @@ pcmk_result_get_strings(int code, enum pcmk_result_type type, const char **name,
 int
 pcmk__result_bounds(enum pcmk_result_type type, int *lower, int *upper)
 {
-    CRM_ASSERT((lower != NULL) && (upper != NULL));
+    pcmk__assert((lower != NULL) && (upper != NULL));
 
     switch (type) {
         case pcmk_result_legacy:
@@ -116,6 +114,116 @@ pcmk__result_bounds(enum pcmk_result_type type, int *lower, int *upper)
     return pcmk_rc_ok;
 }
 
+/*!
+ * \internal
+ * \brief Log a failed assertion
+ *
+ * \param[in] file              File making the assertion
+ * \param[in] function          Function making the assertion
+ * \param[in] line              Line of file making the assertion
+ * \param[in] assert_condition  String representation of assertion
+ */
+static void
+log_assertion_as(const char *file, const char *function, int line,
+                 const char *assert_condition)
+{
+    if (!pcmk__is_daemon) {
+        crm_enable_stderr(TRUE); // Make sure command-line user sees message
+    }
+    crm_err("%s: Triggered fatal assertion at %s:%d : %s",
+            function, file, line, assert_condition);
+}
+
+/* coverity[+kill] */
+/*!
+ * \internal
+ * \brief Log a failed assertion and abort
+ *
+ * \param[in] file              File making the assertion
+ * \param[in] function          Function making the assertion
+ * \param[in] line              Line of file making the assertion
+ * \param[in] assert_condition  String representation of assertion
+ *
+ * \note This does not return
+ */
+_Noreturn void
+pcmk__abort_as(const char *file, const char *function, int line,
+               const char *assert_condition)
+{
+    log_assertion_as(file, function, line, assert_condition);
+    abort();
+}
+
+/* coverity[+kill] */
+/*!
+ * \internal
+ * \brief Handle a failed assertion
+ *
+ * When called by a daemon, fork a child that aborts (to dump core), otherwise
+ * abort the current process.
+ *
+ * \param[in] file              File making the assertion
+ * \param[in] function          Function making the assertion
+ * \param[in] line              Line of file making the assertion
+ * \param[in] assert_condition  String representation of assertion
+ */
+static void
+fail_assert_as(const char *file, const char *function, int line,
+               const char *assert_condition)
+{
+    int status = 0;
+    pid_t pid = 0;
+
+    if (!pcmk__is_daemon) {
+        pcmk__abort_as(file, function, line, assert_condition); // No return
+    }
+
+    pid = fork();
+    switch (pid) {
+        case -1: // Fork failed
+            crm_warn("%s: Cannot dump core for non-fatal assertion at %s:%d "
+                     ": %s", function, file, line, assert_condition);
+            break;
+
+        case 0: // Child process: just abort to dump core
+            abort();
+            break;
+
+        default: // Parent process: wait for child
+            crm_err("%s: Forked child [%d] to record non-fatal assertion at "
+                    "%s:%d : %s", function, pid, file, line, assert_condition);
+            crm_write_blackbox(SIGTRAP, NULL);
+            do {
+                if (waitpid(pid, &status, 0) == pid) {
+                    return; // Child finished dumping core
+                }
+            } while (errno == EINTR);
+            if (errno == ECHILD) {
+                // crm_mon ignores SIGCHLD
+                crm_trace("Cannot wait on forked child [%d] "
+                          "(SIGCHLD is probably ignored)", pid);
+            } else {
+                crm_err("Cannot wait on forked child [%d]: %s",
+                        pid, pcmk_rc_str(errno));
+            }
+            break;
+    }
+}
+
+/* coverity[+kill] */
+void
+crm_abort(const char *file, const char *function, int line,
+          const char *assert_condition, gboolean do_core, gboolean do_fork)
+{
+    if (!do_fork) {
+        pcmk__abort_as(file, function, line, assert_condition); // No return
+    } else if (do_core) {
+        fail_assert_as(file, function, line, assert_condition);
+    } else {
+        log_assertion_as(file, function, line, assert_condition);
+    }
+}
+
 // @COMPAT Legacy function return codes
 
 //! \deprecated Use standard return codes and pcmk_rc_name() instead
@@ -933,30 +1041,20 @@ pcmk__bzlib2rc(int bz2)
 }
 
 crm_exit_t
-crm_exit(crm_exit_t rc)
+crm_exit(crm_exit_t exit_status)
 {
     /* A compiler could theoretically use any type for crm_exit_t, but an int
      * should always hold it, so cast to int to keep static analysis happy.
      */
-    if ((((int) rc) < 0) || (((int) rc) > CRM_EX_MAX)) {
-        rc = CRM_EX_ERROR;
+    if ((((int) exit_status) < 0) || (((int) exit_status) > CRM_EX_MAX)) {
+        exit_status = CRM_EX_ERROR;
     }
 
-    mainloop_cleanup();
-    crm_xml_cleanup();
-
-    free(pcmk__our_nodename);
-
-    if (crm_system_name) {
-        crm_info("Exiting %s " CRM_XS " with status %d", crm_system_name, rc);
-        free(crm_system_name);
-    } else {
-        crm_trace("Exiting with status %d", rc);
-    }
-    pcmk__free_common_logger();
-    qb_log_fini(); // Don't log anything after this point
-
-    exit(rc);
+    crm_info("Exiting %s " QB_XS " with status %d (%s: %s)",
+             pcmk__s(crm_system_name, "process"), exit_status,
+             crm_exit_name(exit_status), crm_exit_str(exit_status));
+    pcmk_common_cleanup();
+    exit(exit_status);
 }
 
 /*
@@ -1021,7 +1119,7 @@ pcmk__format_result(pcmk__action_result_t *result, int exit_status,
     if (format != NULL) {
         va_start(ap, format);
         len = vasprintf(&reason, format, ap);
-        CRM_ASSERT(len > 0);
+        pcmk__assert(len > 0);
         va_end(ap);
     }
     free(result->exit_reason);
@@ -1095,50 +1193,3 @@ pcmk__copy_result(const pcmk__action_result_t *src, pcmk__action_result_t *dst)
     dst->action_stdout = pcmk__str_copy(src->action_stdout);
     dst->action_stderr = pcmk__str_copy(src->action_stderr);
 }
-
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-#include <crm/common/results_compat.h>
-
-const char *
-bz2_strerror(int rc)
-{
-    // See ftp://sources.redhat.com/pub/bzip2/docs/manual_3.html#SEC17
-    switch (rc) {
-        case BZ_OK:
-        case BZ_RUN_OK:
-        case BZ_FLUSH_OK:
-        case BZ_FINISH_OK:
-        case BZ_STREAM_END:
-            return "Ok";
-        case BZ_CONFIG_ERROR:
-            return "libbz2 has been improperly compiled on your platform";
-        case BZ_SEQUENCE_ERROR:
-            return "library functions called in the wrong order";
-        case BZ_PARAM_ERROR:
-            return "parameter is out of range or otherwise incorrect";
-        case BZ_MEM_ERROR:
-            return "memory allocation failed";
-        case BZ_DATA_ERROR:
-            return "data integrity error is detected during decompression";
-        case BZ_DATA_ERROR_MAGIC:
-            return "the compressed stream does not start with the correct magic bytes";
-        case BZ_IO_ERROR:
-            return "error reading or writing in the compressed file";
-        case BZ_UNEXPECTED_EOF:
-            return "compressed file finishes before the logical end of stream is detected";
-        case BZ_OUTBUFF_FULL:
-            return "output data will not fit into the buffer provided";
-    }
-    return "Data compression error";
-}
-
-crm_exit_t
-crm_errno2exit(int rc)
-{
-    return pcmk_rc2exitc(pcmk_legacy2rc(rc));
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
