@@ -9,10 +9,6 @@
 
 #include <crm_internal.h>
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <stdio.h>      // snprintf(), NULL
 #include <string.h>     // strcpy(), strdup()
 #include <sys/types.h>  // size_t
@@ -22,51 +18,71 @@ int pcmk__score_green = 0;
 int pcmk__score_yellow = 0;
 
 /*!
- * \brief Get the integer value of a score string
+ * \brief Parse an integer score from a string
  *
- * Given a string representation of a score, return the integer equivalent.
- * This accepts infinity strings as well as red, yellow, and green, and
- * bounds the result to +/-INFINITY.
+ * Parse an integer score from a string. This accepts infinity strings as well
+ * as red, yellow, and green, and bounds the result to +/-INFINITY.
  *
- * \param[in] score  Score as string
+ * \param[in]  score_s        Score as string
+ * \param[out] score          Where to store integer value corresponding to
+ *                            \p score_s (may be NULL to only check validity)
+ * \param[in]  default_score  Value to use if \p score_s is NULL or invalid
  *
- * \return Integer value corresponding to \p score
+ * \return Standard Pacemaker return code
  */
 int
-char2score(const char *score)
+pcmk_parse_score(const char *score_s, int *score, int default_score)
 {
-    if (score == NULL) {
-        return 0;
+    int rc = pcmk_rc_ok;
+    int local_score = 0;
 
-    } else if (pcmk_str_is_minus_infinity(score)) {
-        return -PCMK_SCORE_INFINITY;
+    // Ensure default score is in bounds
+    default_score = QB_MIN(default_score, PCMK_SCORE_INFINITY);
+    default_score = QB_MAX(default_score, -PCMK_SCORE_INFINITY);
+    local_score = default_score;
 
-    } else if (pcmk_str_is_infinity(score)) {
-        return PCMK_SCORE_INFINITY;
+    if (score_s == NULL) {
 
-    } else if (pcmk__str_eq(score, PCMK_VALUE_RED, pcmk__str_casei)) {
-        return pcmk__score_red;
+    } else if (pcmk_str_is_minus_infinity(score_s)) {
+        local_score = -PCMK_SCORE_INFINITY;
 
-    } else if (pcmk__str_eq(score, PCMK_VALUE_YELLOW, pcmk__str_casei)) {
-        return pcmk__score_yellow;
+    } else if (pcmk_str_is_infinity(score_s)) {
+        local_score = PCMK_SCORE_INFINITY;
 
-    } else if (pcmk__str_eq(score, PCMK_VALUE_GREEN, pcmk__str_casei)) {
-        return pcmk__score_green;
+    } else if (pcmk__str_eq(score_s, PCMK_VALUE_RED, pcmk__str_casei)) {
+        local_score = pcmk__score_red;
+
+    } else if (pcmk__str_eq(score_s, PCMK_VALUE_YELLOW, pcmk__str_casei)) {
+        local_score = pcmk__score_yellow;
+
+    } else if (pcmk__str_eq(score_s, PCMK_VALUE_GREEN, pcmk__str_casei)) {
+        local_score = pcmk__score_green;
 
     } else {
-        long long score_ll;
+        long long score_ll = 0LL;
 
-        pcmk__scan_ll(score, &score_ll, 0LL);
-        if (score_ll > PCMK_SCORE_INFINITY) {
-            return PCMK_SCORE_INFINITY;
+        rc = pcmk__scan_ll(score_s, &score_ll, default_score);
+        if (rc == ERANGE) {
+            rc = pcmk_rc_ok;
+        }
+        if (rc != pcmk_rc_ok) {
+            local_score = default_score;
+
+        } else if (score_ll > PCMK_SCORE_INFINITY) {
+            local_score = PCMK_SCORE_INFINITY;
 
         } else if (score_ll < -PCMK_SCORE_INFINITY) {
-            return -PCMK_SCORE_INFINITY;
+            local_score = -PCMK_SCORE_INFINITY;
 
         } else {
-            return (int) score_ll;
+            local_score = (int) score_ll;
         }
     }
+
+    if (score != NULL) {
+        *score = local_score;
+    }
+    return rc;
 }
 
 /*!
@@ -102,6 +118,33 @@ pcmk_readable_score(int score)
     return score_s;
 }
 
+/*!
+ * \internal
+ * \brief Check whether a string represents an infinite value
+ *
+ * \param[in] s  String to check
+ *
+ * \return \c true if \p s is "INFINITY" or "+INFINITY", otherwise \c false
+ */
+bool
+pcmk_str_is_infinity(const char *s) {
+    return pcmk__str_any_of(s, PCMK_VALUE_INFINITY, PCMK_VALUE_PLUS_INFINITY,
+                            NULL);
+}
+
+/*!
+ * \internal
+ * \brief Check whether a string represents an negatively infinite value
+ *
+ * \param[in] s  String to check
+ *
+ * \return \c true if \p s is "-INFINITY", otherwise \c false
+ */
+bool
+pcmk_str_is_minus_infinity(const char *s) {
+    return pcmk__str_eq(s, PCMK_VALUE_MINUS_INFINITY, pcmk__str_none);
+}
+
 /*!
  * \internal
  * \brief Add two scores, bounding to +/-INFINITY
@@ -144,19 +187,13 @@ pcmk__add_scores(int score1, int score2)
 
 #include <crm/common/scores_compat.h>
 
-char *
-score2char(int score)
+int
+char2score(const char *score)
 {
-    return pcmk__str_copy(pcmk_readable_score(score));
-}
+    int result = 0;
 
-char *
-score2char_stack(int score, char *buf, size_t len)
-{
-    CRM_CHECK((buf != NULL) && (len >= sizeof(PCMK_VALUE_MINUS_INFINITY)),
-              return NULL);
-    strcpy(buf, pcmk_readable_score(score));
-    return buf;
+    (void) pcmk_parse_score(score, &result, 0);
+    return result;
 }
 
 // LCOV_EXCL_STOP
