@@ -7,13 +7,8 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#include "crm/common/results.h"
 #include <crm_internal.h>
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <regex.h>
 #include <stdio.h>
 #include <string.h>
@@ -35,8 +30,8 @@
  *                            non-integer character
  *
  * \return Standard Pacemaker return code (\c pcmk_rc_ok on success,
- *         \c EINVAL on failed string conversion due to invalid input,
- *         or \c EOVERFLOW on arithmetic overflow)
+ *         \c pcmk_rc_bad_input on failed string conversion due to invalid
+ *         input, or \c ERANGE if outside long long range)
  * \note Sets \c errno on error
  */
 static int
@@ -51,26 +46,26 @@ scan_ll(const char *text, long long *result, long long default_value,
     if (text != NULL) {
         local_result = strtoll(text, &local_end_text, 10);
         if (errno == ERANGE) {
-            rc = EOVERFLOW;
-            crm_warn("Integer parsed from '%s' was clipped to %lld",
-                     text, local_result);
+            rc = errno;
+            crm_debug("Integer parsed from '%s' was clipped to %lld",
+                      text, local_result);
+
+        } else if (local_end_text == text) {
+            rc = pcmk_rc_bad_input;
+            local_result = default_value;
+            crm_debug("Could not parse integer from '%s' (using %lld instead): "
+                      "No digits found", text, default_value);
 
         } else if (errno != 0) {
             rc = errno;
             local_result = default_value;
-            crm_warn("Could not parse integer from '%s' (using %lld instead): "
-                     "%s", text, default_value, pcmk_rc_str(rc));
-
-        } else if (local_end_text == text) {
-            rc = EINVAL;
-            local_result = default_value;
-            crm_warn("Could not parse integer from '%s' (using %lld instead): "
-                    "No digits found", text, default_value);
+            crm_debug("Could not parse integer from '%s' (using %lld instead): "
+                      "%s", text, default_value, pcmk_rc_str(rc));
         }
 
         if ((end_text == NULL) && !pcmk__str_empty(local_end_text)) {
-            crm_warn("Characters left over after parsing '%s': '%s'",
-                     text, local_end_text);
+            crm_debug("Characters left over after parsing '%s': '%s'",
+                      text, local_end_text);
         }
         errno = rc;
     }
@@ -97,14 +92,8 @@ int
 pcmk__scan_ll(const char *text, long long *result, long long default_value)
 {
     long long local_result = default_value;
-    int rc = pcmk_rc_ok;
+    int rc = scan_ll(text, &local_result, default_value, NULL);
 
-    if (text != NULL) {
-        rc = scan_ll(text, &local_result, default_value, NULL);
-        if (rc != pcmk_rc_ok) {
-            local_result = default_value;
-        }
-    }
     if (result != NULL) {
         *result = local_result;
     }
@@ -163,7 +152,10 @@ pcmk__scan_port(const char *text, int *port)
     long long port_ll;
     int rc = pcmk__scan_ll(text, &port_ll, -1LL);
 
-    if ((text != NULL) && (rc == pcmk_rc_ok) // wasn't default or invalid
+    if (rc != pcmk_rc_ok) {
+        crm_warn("'%s' is not a valid port: %s", text, pcmk_rc_str(rc));
+
+    } else if ((text != NULL) // wasn't default or invalid
         && ((port_ll < 0LL) || (port_ll > 65535LL))) {
         crm_warn("Ignoring port specification '%s' "
                  "not in valid range (0-65535)", text);
@@ -202,7 +194,7 @@ pcmk__scan_double(const char *text, double *result, const char *default_text,
     int rc = pcmk_rc_ok;
     char *local_end_text = NULL;
 
-    CRM_ASSERT(result != NULL);
+    pcmk__assert(result != NULL);
     *result = PCMK__PARSE_DBL_DEFAULT;
 
     text = (text != NULL) ? text : default_text;
@@ -328,11 +320,14 @@ pcmk__guint_from_hash(GHashTable *table, const char *key, guint default_val,
 
     rc = pcmk__scan_ll(value, &value_ll, 0LL);
     if (rc != pcmk_rc_ok) {
+        crm_warn("Using default (%u) for %s because '%s' is not a "
+                 "valid integer: %s", default_val, key, value, pcmk_rc_str(rc));
         return rc;
     }
 
     if ((value_ll < 0) || (value_ll > G_MAXUINT)) {
-        crm_warn("Could not parse non-negative integer from %s", value);
+        crm_warn("Using default (%u) for %s because '%s' is not in valid range",
+                 default_val, key, value);
         return ERANGE;
     }
 
@@ -359,6 +354,7 @@ crm_get_msec(const char *input)
     long long multiplier = 1000;
     long long divisor = 1;
     long long msec = PCMK__PARSE_INT_DEFAULT;
+    int rc = pcmk_rc_ok;
 
     if (input == NULL) {
         return PCMK__PARSE_INT_DEFAULT;
@@ -369,9 +365,14 @@ crm_get_msec(const char *input)
         input++;
     }
 
-    // Reject negative and unparsable inputs
-    scan_ll(input, &msec, -1, &units);
-    if (msec < 0) {
+    rc = scan_ll(input, &msec, PCMK__PARSE_INT_DEFAULT, &units);
+
+    if ((rc == ERANGE) && (msec > 0)) {
+        crm_warn("'%s' will be clipped to %lld", input, msec);
+
+    } else if ((rc != pcmk_rc_ok) || (msec < 0)) {
+        crm_warn("'%s' is not a valid time duration: %s",
+                 input, ((rc == pcmk_rc_ok)? "Negative" : pcmk_rc_str(rc)));
         return PCMK__PARSE_INT_DEFAULT;
     }
 
@@ -462,7 +463,8 @@ pcmk_parse_interval_spec(const char *input, guint *result_ms)
         crm_time_t *period_s = crm_time_parse_duration(input);
 
         if (period_s != NULL) {
-            msec = 1000 * crm_time_get_seconds(period_s);
+            msec = crm_time_get_seconds(period_s);
+            msec = QB_MIN(msec, G_MAXUINT / 1000) * 1000;
             crm_time_free(period_s);
         }
 
@@ -470,7 +472,7 @@ pcmk_parse_interval_spec(const char *input, guint *result_ms)
         msec = crm_get_msec(input);
     }
 
-    if (msec == PCMK__PARSE_INT_DEFAULT) {
+    if (msec < 0) {
         crm_warn("Using 0 instead of invalid interval specification '%s'",
                  input);
         msec = 0;
@@ -506,8 +508,8 @@ crm_str_to_boolean(const char *s, int *ret)
         return 1;
     }
 
-    if (pcmk__strcase_any_of(s, PCMK_VALUE_FALSE, "off", "no", "n", "0",
-                             NULL)) {
+    if (pcmk__strcase_any_of(s, PCMK_VALUE_FALSE, PCMK_VALUE_OFF, "no", "n",
+                             "0", NULL)) {
         if (ret != NULL) {
             *ret = FALSE;
         }
@@ -652,9 +654,9 @@ pcmk__ends_with_ext(const char *s, const char *match)
  * a correct implementation of DJB's hash), but we need to preserve existing
  * behaviour, because the hash key ultimately determines the "sort" order
  * when iterating through GHashTables, which affects allocation of scores to
- * clone instances when iterating through rsc->allowed_nodes.  It (somehow)
- * also appears to have some minor impact on the ordering of a few
- * pseudo_event IDs in the transition graph.
+ * clone instances when iterating through allowed nodes. It (somehow) also
+ * appears to have some minor impact on the ordering of a few pseudo_event IDs
+ * in the transition graph.
  */
 static guint
 pcmk__str_hash(gconstpointer v)
@@ -700,7 +702,7 @@ pcmk__strkey_table(GDestroyNotify key_destroy_func,
 void
 pcmk__insert_dup(GHashTable *table, const char *name, const char *value)
 {
-    CRM_ASSERT((table != NULL) && (name != NULL));
+    pcmk__assert((table != NULL) && (name != NULL));
 
     g_hash_table_insert(table, pcmk__str_copy(name), pcmk__str_copy(value));
 }
@@ -794,7 +796,7 @@ void
 pcmk__add_separated_word(GString **list, size_t init_size, const char *word,
                          const char *separator)
 {
-    CRM_ASSERT(list != NULL);
+    pcmk__assert(list != NULL);
 
     if (pcmk__str_empty(word)) {
         return;
@@ -863,7 +865,7 @@ pcmk__compress(const char *data, unsigned int length, unsigned int max,
     free(uncompressed);
 
     if (rc != pcmk_rc_ok) {
-        crm_err("Compression of %d bytes failed: %s " CRM_XS " rc=%d",
+        crm_err("Compression of %d bytes failed: %s " QB_XS " rc=%d",
                 length, pcmk_rc_str(rc), rc);
         free(compressed);
         return rc;
@@ -893,8 +895,8 @@ crm_strdup_printf(char const *format, ...)
     char *string = NULL;
 
     va_start(ap, format);
-    len = vasprintf (&string, format, ap);
-    CRM_ASSERT(len > 0);
+    len = vasprintf(&string, format, ap);
+    pcmk__assert(len > 0);
     va_end(ap);
     return string;
 }
@@ -905,7 +907,7 @@ pcmk__parse_ll_range(const char *srcstring, long long *start, long long *end)
     char *remainder = NULL;
     int rc = pcmk_rc_ok;
 
-    CRM_ASSERT(start != NULL && end != NULL);
+    pcmk__assert((start != NULL) && (end != NULL));
 
     *start = PCMK__PARSE_INT_DEFAULT;
     *end = PCMK__PARSE_INT_DEFAULT;
@@ -923,11 +925,10 @@ pcmk__parse_ll_range(const char *srcstring, long long *start, long long *end)
     if (*srcstring == '-') {
         int rc = scan_ll(srcstring+1, end, PCMK__PARSE_INT_DEFAULT, &remainder);
 
-        if (rc != pcmk_rc_ok || *remainder != '\0') {
-            return pcmk_rc_bad_input;
-        } else {
-            return pcmk_rc_ok;
+        if ((rc == pcmk_rc_ok) && (*remainder != '\0')) {
+            rc = pcmk_rc_bad_input;
         }
+        return rc;
     }
 
     rc = scan_ll(srcstring, start, PCMK__PARSE_INT_DEFAULT, &remainder);
@@ -1077,7 +1078,7 @@ pcmk__str_any_of(const char *s, ...)
 int
 pcmk__numeric_strcasecmp(const char *s1, const char *s2)
 {
-    CRM_ASSERT((s1 != NULL) && (s2 != NULL));
+    pcmk__assert((s1 != NULL) && (s2 != NULL));
 
     while (*s1 && *s2) {
         if (isdigit(*s1) && isdigit(*s2)) {
@@ -1297,7 +1298,7 @@ pcmk__g_strcat(GString *buffer, ...)
 {
     va_list ap;
 
-    CRM_ASSERT(buffer != NULL);
+    pcmk__assert(buffer != NULL);
     va_start(ap, buffer);
 
     while (true) {
@@ -1310,133 +1311,3 @@ pcmk__g_strcat(GString *buffer, ...)
     }
     va_end(ap);
 }
-
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-#include <crm/common/util_compat.h>
-
-gboolean
-safe_str_neq(const char *a, const char *b)
-{
-    if (a == b) {
-        return FALSE;
-
-    } else if (a == NULL || b == NULL) {
-        return TRUE;
-
-    } else if (strcasecmp(a, b) == 0) {
-        return FALSE;
-    }
-    return TRUE;
-}
-
-gboolean
-crm_str_eq(const char *a, const char *b, gboolean use_case)
-{
-    if (use_case) {
-        return g_strcmp0(a, b) == 0;
-
-        /* TODO - Figure out which calls, if any, really need to be case independent */
-    } else if (a == b) {
-        return TRUE;
-
-    } else if (a == NULL || b == NULL) {
-        /* shouldn't be comparing NULLs */
-        return FALSE;
-
-    } else if (strcasecmp(a, b) == 0) {
-        return TRUE;
-    }
-    return FALSE;
-}
-
-char *
-crm_itoa_stack(int an_int, char *buffer, size_t len)
-{
-    if (buffer != NULL) {
-        snprintf(buffer, len, "%d", an_int);
-    }
-    return buffer;
-}
-
-guint
-g_str_hash_traditional(gconstpointer v)
-{
-    return pcmk__str_hash(v);
-}
-
-gboolean
-crm_strcase_equal(gconstpointer a, gconstpointer b)
-{
-    return pcmk__strcase_equal(a, b);
-}
-
-guint
-crm_strcase_hash(gconstpointer v)
-{
-    return pcmk__strcase_hash(v);
-}
-
-GHashTable *
-crm_str_table_dup(GHashTable *old_table)
-{
-    return pcmk__str_table_dup(old_table);
-}
-
-long long
-crm_parse_ll(const char *text, const char *default_text)
-{
-    long long result;
-
-    if (text == NULL) {
-        text = default_text;
-        if (text == NULL) {
-            crm_err("No default conversion value supplied");
-            errno = EINVAL;
-            return PCMK__PARSE_INT_DEFAULT;
-        }
-    }
-    scan_ll(text, &result, PCMK__PARSE_INT_DEFAULT, NULL);
-    return result;
-}
-
-int
-crm_parse_int(const char *text, const char *default_text)
-{
-    long long result = crm_parse_ll(text, default_text);
-
-    if (result < INT_MIN) {
-        // If errno is ERANGE, crm_parse_ll() has already logged a message
-        if (errno != ERANGE) {
-            crm_err("Conversion of %s was clipped: %lld", text, result);
-            errno = ERANGE;
-        }
-        return INT_MIN;
-
-    } else if (result > INT_MAX) {
-        // If errno is ERANGE, crm_parse_ll() has already logged a message
-        if (errno != ERANGE) {
-            crm_err("Conversion of %s was clipped: %lld", text, result);
-            errno = ERANGE;
-        }
-        return INT_MAX;
-    }
-
-    return (int) result;
-}
-
-char *
-crm_strip_trailing_newline(char *str)
-{
-    return pcmk__trim(str);
-}
-
-int
-pcmk_numeric_strcasecmp(const char *s1, const char *s2)
-{
-    return pcmk__numeric_strcasecmp(s1, s2);
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
