@@ -74,7 +74,7 @@ null_invalid(void **state)
     assert_int_equal(pcmk__evaluate_date_expression(NULL, t, NULL), EINVAL);
 
     crm_time_free(t);
-    free_xml(xml);
+    pcmk__xml_free(xml);
 }
 
 static void
@@ -83,7 +83,7 @@ null_next_change_ok(void **state)
     xmlNodePtr xml = pcmk__xml_parse(EXPR_LT_VALID);
 
     assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_within_range);
-    free_xml(xml);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_ID_MISSING                             \
@@ -94,11 +94,10 @@ null_next_change_ok(void **state)
 static void
 id_missing(void **state)
 {
-    // Currently acceptable
     xmlNodePtr xml = pcmk__xml_parse(EXPR_ID_MISSING);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_within_range);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_OP_INVALID                                 \
@@ -110,8 +109,8 @@ op_invalid(void **state)
 {
     xmlNodePtr xml = pcmk__xml_parse(EXPR_OP_INVALID);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_undetermined);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_LT_MISSING_END                             \
@@ -123,8 +122,8 @@ lt_missing_end(void **state)
 {
     xmlNodePtr xml = pcmk__xml_parse(EXPR_LT_MISSING_END);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_undetermined);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_LT_INVALID_END                             \
@@ -137,8 +136,8 @@ lt_invalid_end(void **state)
 {
     xmlNodePtr xml = pcmk__xml_parse(EXPR_LT_INVALID_END);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_undetermined);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 static void
@@ -162,7 +161,7 @@ lt_valid(void **state)
     assert_date_expression(xml, "2024-03-01 12:00:00", "2024-02-01 20:00:00",
                            "2024-02-01 20:00:00", pcmk_rc_after_range);
 
-    free_xml(xml);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_GT_MISSING_START                           \
@@ -174,8 +173,8 @@ gt_missing_start(void **state)
 {
     xmlNodePtr xml = pcmk__xml_parse(EXPR_GT_MISSING_START);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_undetermined);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_GT_INVALID_START                           \
@@ -188,8 +187,8 @@ gt_invalid_start(void **state)
 {
     xmlNodePtr xml = pcmk__xml_parse(EXPR_GT_INVALID_START);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_undetermined);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_GT_VALID                                   \
@@ -222,7 +221,7 @@ gt_valid(void **state)
     assert_date_expression(xml, "2024-03-01 05:03:11", "2024-04-04 04:04:04",
                            "2024-04-04 04:04:04", pcmk_rc_within_range);
 
-    free_xml(xml);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_MISSING                              \
@@ -236,10 +235,10 @@ range_missing(void **state)
     crm_time_t *t = crm_time_new("2024-01-01");
 
     assert_int_equal(pcmk__evaluate_date_expression(xml, t, NULL),
-                     pcmk_rc_undetermined);
+                     pcmk_rc_unpack_error);
 
     crm_time_free(t);
-    free_xml(xml);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_INVALID_START_INVALID_END            \
@@ -253,8 +252,8 @@ range_invalid_start_invalid_end(void **state)
 {
     xmlNodePtr xml = pcmk__xml_parse(EXPR_RANGE_INVALID_START_INVALID_END);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_undetermined);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_INVALID_START_ONLY                   \
@@ -267,8 +266,8 @@ range_invalid_start_only(void **state)
 {
     xmlNodePtr xml = pcmk__xml_parse(EXPR_RANGE_INVALID_START_ONLY);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_undetermined);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_VALID_START_ONLY                     \
@@ -297,7 +296,7 @@ range_valid_start_only(void **state)
     assert_date_expression(xml, "2024-03-01 05:03:11", "2024-04-04 04:04:04",
                            "2024-04-04 04:04:04", pcmk_rc_within_range);
 
-    free_xml(xml);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_INVALID_END_ONLY                   \
@@ -310,8 +309,8 @@ range_invalid_end_only(void **state)
 {
     xmlNodePtr xml = pcmk__xml_parse(EXPR_RANGE_INVALID_END_ONLY);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_undetermined);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_VALID_END_ONLY                     \
@@ -340,7 +339,7 @@ range_valid_end_only(void **state)
     assert_date_expression(xml, "2024-02-01 15:00:01", "2024-04-04 04:04:04",
                            "2024-04-04 04:04:04", pcmk_rc_after_range);
 
-    free_xml(xml);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_VALID_START_INVALID_END              \
@@ -352,26 +351,11 @@ range_valid_end_only(void **state)
 static void
 range_valid_start_invalid_end(void **state)
 {
-    // Currently treated same as start without end
     xmlNodePtr xml = pcmk__xml_parse(EXPR_RANGE_VALID_START_INVALID_END);
 
-    // Now and next change are before start
-    assert_date_expression(xml, "2024-01-01 04:30:05", "2024-01-01 11:00:00",
-                           "2024-01-01 11:00:00", pcmk_rc_before_range);
-
-    // Now is before start, next change is after start
-    assert_date_expression(xml, "2024-02-01 11:59:59", "2024-02-01 18:00:00",
-                           "2024-02-01 12:00:00", pcmk_rc_before_range);
-
-    // Now is equal to start, next change is after start
-    assert_date_expression(xml, "2024-02-01 12:00:00", "2024-02-01 18:00:00",
-                           "2024-02-01 18:00:00", pcmk_rc_within_range);
-
-    // Now and next change are after start
-    assert_date_expression(xml, "2024-03-01 05:03:11", "2024-04-04 04:04:04",
-                           "2024-04-04 04:04:04", pcmk_rc_within_range);
-
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01 04:30:05", NULL, NULL,
+                           pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_INVALID_START_VALID_END              \
@@ -383,26 +367,11 @@ range_valid_start_invalid_end(void **state)
 static void
 range_invalid_start_valid_end(void **state)
 {
-    // Currently treated same as end without start
     xmlNodePtr xml = pcmk__xml_parse(EXPR_RANGE_INVALID_START_VALID_END);
 
-    // Now and next change are before end
-    assert_date_expression(xml, "2024-01-01 04:30:05", "2024-01-01 11:00:00",
-                           "2024-01-01 11:00:00", pcmk_rc_within_range);
-
-    // Now is before end, next change is after end
-    assert_date_expression(xml, "2024-02-01 14:59:59", "2024-02-01 18:00:00",
-                           "2024-02-01 15:00:01", pcmk_rc_within_range);
-
-    // Now is equal to end, next change is after end
-    assert_date_expression(xml, "2024-02-01 15:00:00", "2024-02-01 18:00:00",
-                           "2024-02-01 15:00:01", pcmk_rc_within_range);
-
-    // Now and next change are after end
-    assert_date_expression(xml, "2024-02-01 15:00:01", "2024-04-04 04:04:04",
-                           "2024-04-04 04:04:04", pcmk_rc_after_range);
-
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01 04:30:05", NULL, NULL,
+                           pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_VALID_START_VALID_END                \
@@ -440,7 +409,7 @@ range_valid_start_valid_end(void **state)
     assert_date_expression(xml, "2024-02-01 15:00:01", "2028-04-04 04:04:04",
                            "2028-04-04 04:04:04", pcmk_rc_after_range);
 
-    free_xml(xml);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_VALID_START_INVALID_DURATION         \
@@ -454,26 +423,11 @@ range_valid_start_valid_end(void **state)
 static void
 range_valid_start_invalid_duration(void **state)
 {
-    // Currently treated same as end equals start
     xmlNodePtr xml = pcmk__xml_parse(EXPR_RANGE_VALID_START_INVALID_DURATION);
 
-    // Now and next change are before start
-    assert_date_expression(xml, "2024-02-01 04:30:05", "2024-01-01 11:00:00",
-                           "2024-01-01 11:00:00", pcmk_rc_before_range);
-
-    // Now is before start, next change is after start
-    assert_date_expression(xml, "2024-02-01 11:59:59", "2024-02-01 18:00:00",
-                           "2024-02-01 12:00:00", pcmk_rc_before_range);
-
-    // Now is equal to start, next change is after start
-    assert_date_expression(xml, "2024-02-01 12:00:00", "2024-02-01 14:30:00",
-                           "2024-02-01 12:00:01", pcmk_rc_within_range);
-
-    // Now and next change are after start
-    assert_date_expression(xml, "2024-02-01 12:00:01", "2024-02-01 14:30:00",
-                           "2024-02-01 14:30:00", pcmk_rc_after_range);
-
-    free_xml(xml);
+    assert_date_expression(xml, "2024-02-01 04:30:05", NULL, NULL,
+                           pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_VALID_START_VALID_DURATION           \
@@ -513,50 +467,26 @@ range_valid_start_valid_duration(void **state)
     assert_date_expression(xml, "2024-02-01 15:00:01", "2028-04-04 04:04:04",
                            "2028-04-04 04:04:04", pcmk_rc_after_range);
 
-    free_xml(xml);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_RANGE_VALID_START_DURATION_MISSING_ID      \
-    "<" PCMK_XE_DATE_EXPRESSION " "                     \
+    "<" PCMK_XE_DATE_EXPRESSION " " PCMK_XA_ID "='d' "  \
     PCMK_XA_OPERATION "='" PCMK_VALUE_IN_RANGE "' "     \
     PCMK_XA_START "='2024-02-01 12:00:00'>"             \
-    "<" PCMK_XE_DURATION " " PCMK_XA_ID "='d' "         \
-    PCMK_XA_HOURS "='3' />"                             \
+    "<" PCMK_XE_DURATION " " PCMK_XA_HOURS "='3' />"    \
     "</" PCMK_XE_DATE_EXPRESSION ">"
 
 static void
 range_valid_start_duration_missing_id(void **state)
 {
-    // Currently acceptable
     xmlNodePtr xml = NULL;
 
     xml = pcmk__xml_parse(EXPR_RANGE_VALID_START_DURATION_MISSING_ID);
 
-    // Now and next change are before start
-    assert_date_expression(xml, "2024-01-01 04:30:05", "2024-01-01 11:00:00",
-                           "2024-01-01 11:00:00", pcmk_rc_before_range);
-
-    // Now is before start, next change is between start and end
-    assert_date_expression(xml, "2024-02-01 11:59:59", "2024-02-01 14:00:00",
-                           "2024-02-01 12:00:00", pcmk_rc_before_range);
-
-    // Now is equal to start, next change is between start and end
-    assert_date_expression(xml, "2024-02-01 12:00:00", "2024-02-01 14:30:00",
-                           "2024-02-01 14:30:00", pcmk_rc_within_range);
-
-    // Now is between start and end, next change is after end
-    assert_date_expression(xml, "2024-02-01 14:03:11", "2024-04-04 04:04:04",
-                           "2024-02-01 15:00:01", pcmk_rc_within_range);
-
-    // Now is equal to end, next change is after end
-    assert_date_expression(xml, "2024-02-01 15:00:00", "2028-04-04 04:04:04",
-                           "2024-02-01 15:00:01", pcmk_rc_within_range);
-
-    // Now and next change are after end
-    assert_date_expression(xml, "2024-02-01 15:00:01", "2028-04-04 04:04:04",
-                           "2028-04-04 04:04:04", pcmk_rc_after_range);
-
-    free_xml(xml);
+    assert_date_expression(xml, "2024-02-01 04:30:05", NULL, NULL,
+                           pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_SPEC_MISSING                               \
@@ -568,8 +498,8 @@ spec_missing(void **state)
 {
     xmlNodePtr xml = pcmk__xml_parse(EXPR_SPEC_MISSING);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_undetermined);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_SPEC_INVALID                               \
@@ -582,11 +512,10 @@ spec_missing(void **state)
 static void
 spec_invalid(void **state)
 {
-    // Currently treated as date_spec with no ranges (which passes)
     xmlNodePtr xml = pcmk__xml_parse(EXPR_SPEC_INVALID);
 
-    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_ok);
-    free_xml(xml);
+    assert_date_expression(xml, "2024-01-01", NULL, NULL, pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_SPEC_VALID                                 \
@@ -619,7 +548,7 @@ spec_valid(void **state)
     assert_date_expression(xml, "2024-03-01 00:00:00", NULL, NULL,
                            pcmk_rc_after_range);
 
-    free_xml(xml);
+    pcmk__xml_free(xml);
 }
 
 #define EXPR_SPEC_MISSING_ID                            \
@@ -632,30 +561,14 @@ spec_valid(void **state)
 static void
 spec_missing_id(void **state)
 {
-    // Currently acceptable; date_spec does not currently support next_change
     xmlNodePtr xml = pcmk__xml_parse(EXPR_SPEC_MISSING_ID);
 
-    // Now is just before spec start
     assert_date_expression(xml, "2024-01-01 23:59:59", NULL, NULL,
-                           pcmk_rc_before_range);
-
-    // Now matches spec start
-    assert_date_expression(xml, "2024-02-01 00:00:00", NULL, NULL, pcmk_rc_ok);
-
-    // Now is within spec range
-    assert_date_expression(xml, "2024-02-22 22:22:22", NULL, NULL, pcmk_rc_ok);
-
-    // Now matches spec end
-    assert_date_expression(xml, "2024-02-29 23:59:59", NULL, NULL, pcmk_rc_ok);
-
-    // Now is just past spec end
-    assert_date_expression(xml, "2024-03-01 00:00:00", NULL, NULL,
-                           pcmk_rc_after_range);
-
-    free_xml(xml);
+                           pcmk_rc_unpack_error);
+    pcmk__xml_free(xml);
 }
 
-PCMK__UNIT_TEST(pcmk__xml_test_setup_group, NULL,
+PCMK__UNIT_TEST(pcmk__xml_test_setup_group, pcmk__xml_test_teardown_group,
                 cmocka_unit_test(null_invalid),
                 cmocka_unit_test(null_next_change_ok),
                 cmocka_unit_test(id_missing),
