@@ -9,12 +9,6 @@
 
 #include <crm_internal.h>
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
-#include <sys/types.h>
-#include <sys/wait.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
 
@@ -45,10 +39,30 @@
 
 CRM_TRACE_INIT_DATA(common);
 
-gboolean crm_config_error = FALSE;
-gboolean crm_config_warning = FALSE;
+bool pcmk__config_has_error = false;
+bool pcmk__config_has_warning = false;
 char *crm_system_name = NULL;
 
+/*!
+ * \brief Free all memory used by libcrmcommon
+ *
+ * Free all global memory allocated by the libcrmcommon library. This should be
+ * called before exiting a process that uses the library, and the process should
+ * not call any libcrmcommon or libxml2 APIs after calling this one.
+ */
+void
+pcmk_common_cleanup(void)
+{
+    // @TODO This isn't really everything, move all cleanup here
+    mainloop_cleanup();
+    pcmk__xml_cleanup();
+    pcmk__free_common_logger();
+    qb_log_fini(); // Don't log anything after this point
+
+    free(crm_system_name);
+    crm_system_name = NULL;
+}
+
 bool
 pcmk__is_user_in_group(const char *user, const char *group)
 {
@@ -158,7 +172,7 @@ version_helper(const char *text, const char **end_text)
 {
     int atoi_result = -1;
 
-    CRM_ASSERT(end_text != NULL);
+    pcmk__assert(end_text != NULL);
 
     errno = 0;
 
@@ -255,116 +269,6 @@ compare_version(const char *version1, const char *version2)
     return rc;
 }
 
-/*!
- * \internal
- * \brief Log a failed assertion
- *
- * \param[in] file              File making the assertion
- * \param[in] function          Function making the assertion
- * \param[in] line              Line of file making the assertion
- * \param[in] assert_condition  String representation of assertion
- */
-static void
-log_assertion_as(const char *file, const char *function, int line,
-                 const char *assert_condition)
-{
-    if (!pcmk__is_daemon) {
-        crm_enable_stderr(TRUE); // Make sure command-line user sees message
-    }
-    crm_err("%s: Triggered fatal assertion at %s:%d : %s",
-            function, file, line, assert_condition);
-}
-
-/* coverity[+kill] */
-/*!
- * \internal
- * \brief Log a failed assertion and abort
- *
- * \param[in] file              File making the assertion
- * \param[in] function          Function making the assertion
- * \param[in] line              Line of file making the assertion
- * \param[in] assert_condition  String representation of assertion
- *
- * \note This does not return
- */
-static _Noreturn void
-abort_as(const char *file, const char *function, int line,
-         const char *assert_condition)
-{
-    log_assertion_as(file, function, line, assert_condition);
-    abort();
-}
-
-/* coverity[+kill] */
-/*!
- * \internal
- * \brief Handle a failed assertion
- *
- * When called by a daemon, fork a child that aborts (to dump core), otherwise
- * abort the current process.
- *
- * \param[in] file              File making the assertion
- * \param[in] function          Function making the assertion
- * \param[in] line              Line of file making the assertion
- * \param[in] assert_condition  String representation of assertion
- */
-static void
-fail_assert_as(const char *file, const char *function, int line,
-               const char *assert_condition)
-{
-    int status = 0;
-    pid_t pid = 0;
-
-    if (!pcmk__is_daemon) {
-        abort_as(file, function, line, assert_condition); // does not return
-    }
-
-    pid = fork();
-    switch (pid) {
-        case -1: // Fork failed
-            crm_warn("%s: Cannot dump core for non-fatal assertion at %s:%d "
-                     ": %s", function, file, line, assert_condition);
-            break;
-
-        case 0: // Child process: just abort to dump core
-            abort();
-            break;
-
-        default: // Parent process: wait for child
-            crm_err("%s: Forked child [%d] to record non-fatal assertion at "
-                    "%s:%d : %s", function, pid, file, line, assert_condition);
-            crm_write_blackbox(SIGTRAP, NULL);
-            do {
-                if (waitpid(pid, &status, 0) == pid) {
-                    return; // Child finished dumping core
-                }
-            } while (errno == EINTR);
-            if (errno == ECHILD) {
-                // crm_mon ignores SIGCHLD
-                crm_trace("Cannot wait on forked child [%d] "
-                          "(SIGCHLD is probably ignored)", pid);
-            } else {
-                crm_err("Cannot wait on forked child [%d]: %s",
-                        pid, pcmk_rc_str(errno));
-            }
-            break;
-    }
-}
-
-/* coverity[+kill] */
-void
-crm_abort(const char *file, const char *function, int line,
-          const char *assert_condition, gboolean do_core, gboolean do_fork)
-{
-    if (!do_fork) {
-        abort_as(file, function, line, assert_condition);
-    } else if (do_core) {
-        fail_assert_as(file, function, line, assert_condition);
-    } else {
-        log_assertion_as(file, function, line, assert_condition);
-    }
-}
-
 /*!
  * \internal
  * \brief Convert the current process to a daemon process
@@ -404,7 +308,7 @@ pcmk__daemonize(const char *name, const char *pidfile)
 
     rc = pcmk__lock_pidfile(pidfile, name);
     if (rc != pcmk_rc_ok) {
-        crm_err("Could not lock '%s' for %s: %s " CRM_XS " rc=%d",
+        crm_err("Could not lock '%s' for %s: %s " QB_XS " rc=%d",
                 pidfile, name, pcmk_rc_str(rc), rc);
         printf("Could not lock '%s' for %s: %s (%d)\n",
                pidfile, name, pcmk_rc_str(rc), rc);
@@ -439,26 +343,6 @@ crm_generate_uuid(void)
     return buffer;
 }
 
-#ifdef HAVE_GNUTLS_GNUTLS_H
-void
-crm_gnutls_global_init(void)
-{
-    signal(SIGPIPE, SIG_IGN);
-    gnutls_global_init();
-}
-#endif
-
-bool
-pcmk_str_is_infinity(const char *s) {
-    return pcmk__str_any_of(s, PCMK_VALUE_INFINITY, PCMK_VALUE_PLUS_INFINITY,
-                            NULL);
-}
-
-bool
-pcmk_str_is_minus_infinity(const char *s) {
-    return pcmk__str_eq(s, PCMK_VALUE_MINUS_INFINITY, pcmk__str_none);
-}
-
 /*!
  * \internal
  * \brief Sleep for given milliseconds
@@ -503,46 +387,113 @@ pcmk__sleep_ms(unsigned int ms)
 #endif
 }
 
+/*!
+ * \internal
+ * \brief Add a timer
+ *
+ * \param[in] interval_ms The interval for the function to be called, in ms
+ * \param[in] fn          The function to be called
+ * \param[in] data        Data to be passed to fn (can be NULL)
+ *
+ * \return The ID of the event source
+ */
+guint
+pcmk__create_timer(guint interval_ms, GSourceFunc fn, gpointer data)
+{
+    pcmk__assert(interval_ms != 0 && fn != NULL);
+
+    if (interval_ms % 1000 == 0) {
+        /* In case interval_ms is 0, the call to pcmk__timeout_ms2s ensures
+         * an interval of one second.
+         */
+        return g_timeout_add_seconds(pcmk__timeout_ms2s(interval_ms), fn, data);
+    } else {
+        return g_timeout_add(interval_ms, fn, data);
+    }
+}
+
+/*!
+ * \internal
+ * \brief Convert milliseconds to seconds
+ *
+ * \param[in] timeout_ms The interval, in ms
+ *
+ * \return If \p timeout_ms is 0, return 0.  Otherwise, return the number of
+ *         seconds, rounded to the nearest integer, with a minimum of 1.
+ */
+guint
+pcmk__timeout_ms2s(guint timeout_ms)
+{
+    guint quot, rem;
+
+    if (timeout_ms == 0) {
+        return 0;
+    } else if (timeout_ms < 1000) {
+        return 1;
+    }
+
+    quot = timeout_ms / 1000;
+    rem = timeout_ms % 1000;
+
+    if (rem >= 500) {
+        quot += 1;
+    }
+
+    return quot;
+}
+
 // Deprecated functions kept only for backward API compatibility
 // LCOV_EXCL_START
 
 #include <crm/common/util_compat.h>
 
-guint
-crm_parse_interval_spec(const char *input)
+static void
+_gnutls_log_func(int level, const char *msg)
 {
-    long long msec = -1;
-
-    errno = 0;
-    if (input == NULL) {
-        return 0;
-
-    } else if (input[0] == 'P') {
-        crm_time_t *period_s = crm_time_parse_duration(input);
-
-        if (period_s) {
-            msec = 1000 * crm_time_get_seconds(period_s);
-            crm_time_free(period_s);
-        }
-
-    } else {
-        msec = crm_get_msec(input);
-    }
-
-    if (msec < 0) {
-        crm_warn("Using 0 instead of '%s'", input);
-        errno = EINVAL;
-        return 0;
-    }
-    return (msec >= G_MAXUINT)? G_MAXUINT : (guint) msec;
+    crm_trace("%s", msg);
 }
 
-char *
-pcmk_hostname(void)
+void
+crm_gnutls_global_init(void)
 {
-    struct utsname hostinfo;
+    signal(SIGPIPE, SIG_IGN);
+    gnutls_global_init();
+    gnutls_global_set_log_level(8);
+    gnutls_global_set_log_function(_gnutls_log_func);
+}
 
-    return (uname(&hostinfo) < 0)? NULL : strdup(hostinfo.nodename);
+/*!
+ * \brief Check whether string represents a client name used by cluster daemons
+ *
+ * \param[in] name  String to check
+ *
+ * \return true if name is standard client name used by daemons, false otherwise
+ *
+ * \note This is provided by the client, and so cannot be used by itself as a
+ *       secure means of authentication.
+ */
+bool
+crm_is_daemon_name(const char *name)
+{
+    return pcmk__str_any_of(name,
+                            "attrd",
+                            CRM_SYSTEM_CIB,
+                            CRM_SYSTEM_CRMD,
+                            CRM_SYSTEM_DC,
+                            CRM_SYSTEM_LRMD,
+                            CRM_SYSTEM_MCP,
+                            CRM_SYSTEM_PENGINE,
+                            CRM_SYSTEM_TENGINE,
+                            "pacemaker-attrd",
+                            "pacemaker-based",
+                            "pacemaker-controld",
+                            "pacemaker-execd",
+                            "pacemaker-fenced",
+                            "pacemaker-remoted",
+                            "pacemaker-schedulerd",
+                            "stonith-ng",
+                            "stonithd",
+                            NULL);
 }
 
 // LCOV_EXCL_STOP
