@@ -24,46 +24,6 @@
 #include <crm/common/xml_io.h>
 #include "crmcommon_private.h"
 
-/* @COMPAT XML_PARSE_RECOVER allows some XML errors to be silently worked around
- * by libxml2, which is potentially ambiguous and dangerous. We should drop it
- * when we can break backward compatibility with configurations that might be
- * relying on it (i.e. pacemaker 3.0.0).
- */
-#define PCMK__XML_PARSE_OPTS_WITHOUT_RECOVER    (XML_PARSE_NOBLANKS)
-#define PCMK__XML_PARSE_OPTS_WITH_RECOVER       (XML_PARSE_NOBLANKS \
-                                                 |XML_PARSE_RECOVER)
-
-/*!
- * \internal
- * \brief Read from \c stdin until EOF or error
- *
- * \return Newly allocated string containing the bytes read from \c stdin, or
- *         \c NULL on error
- *
- * \note The caller is responsible for freeing the return value using \c free().
- */
-static char *
-read_stdin(void)
-{
-    char *buf = NULL;
-    size_t length = 0;
-
-    do {
-        buf = pcmk__realloc(buf, length + PCMK__BUFFER_SIZE + 1);
-        length += fread(buf + length, 1, PCMK__BUFFER_SIZE, stdin);
-    } while ((feof(stdin) == 0) && (ferror(stdin) == 0));
-
-    if (ferror(stdin) != 0) {
-        crm_err("Error reading input from stdin");
-        free(buf);
-        buf = NULL;
-    } else {
-        buf[length] = '\0';
-    }
-    clearerr(stdin);
-    return buf;
-}
-
 /*!
  * \internal
  * \brief Decompress a <tt>bzip2</tt>-compressed file into a string buffer
@@ -93,7 +53,7 @@ decompress_file(const char *filename)
     rc = pcmk__bzlib2rc(rc);
     if (rc != pcmk_rc_ok) {
         crm_err("Could not prepare to read compressed %s: %s "
-                CRM_XS " rc=%d", filename, pcmk_rc_str(rc), rc);
+                QB_XS " rc=%d", filename, pcmk_rc_str(rc), rc);
         goto done;
     }
 
@@ -114,7 +74,7 @@ decompress_file(const char *filename)
     rc = pcmk__bzlib2rc(rc);
     if (rc != pcmk_rc_ok) {
         rc = pcmk__bzlib2rc(rc);
-        crm_err("Could not read compressed %s: %s " CRM_XS " rc=%d",
+        crm_err("Could not read compressed %s: %s " QB_XS " rc=%d",
                 filename, pcmk_rc_str(rc), rc);
         free(buffer);
         buffer = NULL;
@@ -128,29 +88,6 @@ done:
     return buffer;
 }
 
-// @COMPAT Remove macro at 3.0.0 when we drop XML_PARSE_RECOVER
-/*!
- * \internal
- * \brief Try to parse XML first without and then with recovery enabled
- *
- * \param[out] result  Where to store the resulting XML doc (<tt>xmlDoc **</tt>)
- * \param[in]  fn      XML parser function
- * \param[in]  ...     All arguments for \p fn except the final one (an
- *                     \c xmlParserOption group)
- */
-#define parse_xml_recover(result, fn, ...) do {                             \
-        *result = fn(__VA_ARGS__, PCMK__XML_PARSE_OPTS_WITHOUT_RECOVER);    \
-        if (*result == NULL) {                                              \
-            *result = fn(__VA_ARGS__, PCMK__XML_PARSE_OPTS_WITH_RECOVER);   \
-                                                                            \
-            if (*result != NULL) {                                          \
-                crm_warn("Successfully recovered from XML errors "          \
-                         "(note: a future release will treat this as a "    \
-                         "fatal failure)");                                 \
-            }                                                               \
-        }                                                                   \
-    } while (0);
-
 /*!
  * \internal
  * \brief Parse XML from a file
@@ -159,8 +96,7 @@ done:
  *                      \c stdin); if \p filename ends in \c ".bz2", the file
  *                      will be decompressed using \c bzip2
  *
- * \return XML tree parsed from the given file; may be \c NULL or only partial
- *         on error
+ * \return XML tree parsed from the given file on success, otherwise \c NULL
  */
 xmlNode *
 pcmk__xml_read(const char *filename)
@@ -179,34 +115,24 @@ pcmk__xml_read(const char *filename)
     xmlSetGenericErrorFunc(ctxt, pcmk__log_xmllib_err);
 
     if (use_stdin) {
-        /* @COMPAT After dropping XML_PARSE_RECOVER, we can avoid capturing
-         * stdin into a buffer and instead call
-         * xmlCtxtReadFd(ctxt, STDIN_FILENO, NULL, NULL, XML_PARSE_NOBLANKS);
-         *
-         * For now we have to save the input so that we can use it twice.
-         */
-        char *input = read_stdin();
-
-        if (input != NULL) {
-            parse_xml_recover(&output, xmlCtxtReadDoc, ctxt, (pcmkXmlStr) input,
-                              NULL, NULL);
-            free(input);
-        }
+        output = xmlCtxtReadFd(ctxt, STDIN_FILENO, NULL, NULL,
+                               XML_PARSE_NOBLANKS);
 
     } else if (pcmk__ends_with_ext(filename, ".bz2")) {
         char *input = decompress_file(filename);
 
         if (input != NULL) {
-            parse_xml_recover(&output, xmlCtxtReadDoc, ctxt, (pcmkXmlStr) input,
-                              NULL, NULL);
+            output = xmlCtxtReadDoc(ctxt, (pcmkXmlStr) input, NULL, NULL,
+                                    XML_PARSE_NOBLANKS);
             free(input);
         }
 
     } else {
-        parse_xml_recover(&output, xmlCtxtReadFile, ctxt, filename, NULL);
+        output = xmlCtxtReadFile(ctxt, filename, NULL, XML_PARSE_NOBLANKS);
     }
 
     if (output != NULL) {
+        pcmk__xml_new_private_data((xmlNode *) output);
         xml = xmlDocGetRootElement(output);
         if (xml != NULL) {
             /* @TODO Should we really be stripping out text? This seems like an
@@ -219,12 +145,11 @@ pcmk__xml_read(const char *filename)
         }
     }
 
-    // @COMPAT At 3.0.0, free xml and return NULL if xml != NULL on error
     last_error = xmlCtxtGetLastError(ctxt);
-    if (last_error != NULL) {
-        if (xml != NULL) {
-            crm_log_xml_info(xml, "Partial");
-        }
+    if ((last_error != NULL) && (xml != NULL)) {
+        crm_log_xml_debug(xml, "partial");
+        pcmk__xml_free(xml);
+        xml = NULL;
     }
 
     xmlFreeParserCtxt(ctxt);
@@ -237,8 +162,7 @@ pcmk__xml_read(const char *filename)
  *
  * \param[in] input  String to parse
  *
- * \return XML tree parsed from the given string; may be \c NULL or only partial
- *         on error
+ * \return XML tree parsed from the given string on success, otherwise \c NULL
  */
 xmlNode *
 pcmk__xml_parse(const char *input)
@@ -260,19 +184,18 @@ pcmk__xml_parse(const char *input)
     xmlCtxtResetLastError(ctxt);
     xmlSetGenericErrorFunc(ctxt, pcmk__log_xmllib_err);
 
-    parse_xml_recover(&output, xmlCtxtReadDoc, ctxt, (pcmkXmlStr) input, NULL,
-                      NULL);
-
+    output = xmlCtxtReadDoc(ctxt, (pcmkXmlStr) input, NULL, NULL,
+                            XML_PARSE_NOBLANKS);
     if (output != NULL) {
+        pcmk__xml_new_private_data((xmlNode *) output);
         xml = xmlDocGetRootElement(output);
     }
 
-    // @COMPAT At 3.0.0, free xml and return NULL if xml != NULL; update doxygen
     last_error = xmlCtxtGetLastError(ctxt);
-    if (last_error != NULL) {
-        if (xml != NULL) {
-            crm_log_xml_info(xml, "Partial");
-        }
+    if ((last_error != NULL) && (xml != NULL)) {
+        crm_log_xml_debug(xml, "partial");
+        pcmk__xml_free(xml);
+        xml = NULL;
     }
 
     xmlFreeParserCtxt(ctxt);
@@ -344,7 +267,7 @@ dump_xml_element(const xmlNode *data, uint32_t options, GString *buffer,
  * \brief Append XML text content to a buffer
  *
  * \param[in]     data     XML whose content to append
- * \param[in]     options  Group of \p xml_log_options flags
+ * \param[in]     options  Group of <tt>enum pcmk__xml_fmt_options</tt>
  * \param[in,out] buffer   Where to append the content (must not be \p NULL)
  * \param[in]     depth    Current indentation level
  */
@@ -493,7 +416,7 @@ pcmk__xml_string(const xmlNode *data, uint32_t options, GString *buffer,
         return;
     }
 
-    CRM_ASSERT(buffer != NULL);
+    pcmk__assert(buffer != NULL);
     CRM_CHECK(depth >= 0, depth = 0);
 
     switch(data->type) {
@@ -513,7 +436,7 @@ pcmk__xml_string(const xmlNode *data, uint32_t options, GString *buffer,
             dump_xml_cdata(data, options, buffer, depth);
             break;
         default:
-            crm_warn("Cannot convert XML %s node to text " CRM_XS " type=%d",
+            crm_warn("Cannot convert XML %s node to text " QB_XS " type=%d",
                      xml_element_type_text(data->type), data->type);
             break;
     }
@@ -543,7 +466,7 @@ write_compressed_stream(char *text, const char *filename, FILE *stream,
     rc = pcmk__bzlib2rc(rc);
     if (rc != pcmk_rc_ok) {
         crm_warn("Not compressing %s: could not prepare file stream: %s "
-                 CRM_XS " rc=%d",
+                 QB_XS " rc=%d",
                  filename, pcmk_rc_str(rc), rc);
         goto done;
     }
@@ -552,7 +475,7 @@ write_compressed_stream(char *text, const char *filename, FILE *stream,
     rc = pcmk__bzlib2rc(rc);
     if (rc != pcmk_rc_ok) {
         crm_warn("Not compressing %s: could not compress data: %s "
-                 CRM_XS " rc=%d errno=%d",
+                 QB_XS " rc=%d errno=%d",
                  filename, pcmk_rc_str(rc), rc, errno);
         goto done;
     }
@@ -562,7 +485,7 @@ write_compressed_stream(char *text, const char *filename, FILE *stream,
     rc = pcmk__bzlib2rc(rc);
     if (rc != pcmk_rc_ok) {
         crm_warn("Not compressing %s: could not write compressed data: %s "
-                 CRM_XS " rc=%d errno=%d",
+                 QB_XS " rc=%d errno=%d",
                  filename, pcmk_rc_str(rc), rc, errno);
         goto done;
     }
@@ -586,15 +509,13 @@ done:
  * \param[in,out] stream    Open file stream corresponding to filename (closed
  *                          when this function returns)
  * \param[in]     compress  Whether to compress XML before writing
- * \param[out]    nbytes    Number of bytes written
  *
  * \return Standard Pacemaker return code
  */
 static int
 write_xml_stream(const xmlNode *xml, const char *filename, FILE *stream,
-                 bool compress, unsigned int *nbytes)
+                 bool compress)
 {
-    // @COMPAT Drop nbytes as arg when we drop write_xml_fd()/write_xml_file()
     GString *buffer = g_string_sized_new(1024);
     unsigned int bytes_out = 0;
     int rc = pcmk_rc_ok;
@@ -637,9 +558,6 @@ done:
     fclose(stream);
     crm_trace("Saved %u bytes to %s as XML", bytes_out, filename);
 
-    if (nbytes != NULL) {
-        *nbytes = bytes_out;
-    }
     g_string_free(buffer, TRUE);
     return rc;
 }
@@ -651,16 +569,12 @@ done:
  * \param[in]  xml       XML to write
  * \param[in]  filename  Name of file being written (for logging only)
  * \param[in]  fd        Open file descriptor corresponding to \p filename
- * \param[in]  compress  If \c true, compress XML before writing
- * \param[out] nbytes    Number of bytes written (can be \c NULL)
  *
  * \return Standard Pacemaker return code
  */
 int
-pcmk__xml_write_fd(const xmlNode *xml, const char *filename, int fd,
-                   bool compress, unsigned int *nbytes)
+pcmk__xml_write_fd(const xmlNode *xml, const char *filename, int fd)
 {
-    // @COMPAT Drop compress and nbytes arguments when we drop write_xml_fd()
     FILE *stream = NULL;
 
     CRM_CHECK((xml != NULL) && (fd > 0), return EINVAL);
@@ -670,7 +584,7 @@ pcmk__xml_write_fd(const xmlNode *xml, const char *filename, int fd,
     }
 
     return write_xml_stream(xml, pcmk__s(filename, "unnamed file"), stream,
-                            compress, nbytes);
+                            false);
 }
 
 /*!
@@ -680,15 +594,12 @@ pcmk__xml_write_fd(const xmlNode *xml, const char *filename, int fd,
  * \param[in]  xml       XML to write
  * \param[in]  filename  Name of file to write
  * \param[in]  compress  If \c true, compress XML before writing
- * \param[out] nbytes    Number of bytes written (can be \c NULL)
  *
  * \return Standard Pacemaker return code
  */
 int
-pcmk__xml_write_file(const xmlNode *xml, const char *filename, bool compress,
-                     unsigned int *nbytes)
+pcmk__xml_write_file(const xmlNode *xml, const char *filename, bool compress)
 {
-    // @COMPAT Drop nbytes argument when we drop write_xml_fd()
     FILE *stream = NULL;
 
     CRM_CHECK((xml != NULL) && (filename != NULL), return EINVAL);
@@ -697,7 +608,7 @@ pcmk__xml_write_file(const xmlNode *xml, const char *filename, bool compress,
         return errno;
     }
 
-    return write_xml_stream(xml, filename, stream, compress, nbytes);
+    return write_xml_stream(xml, filename, stream, compress);
 }
 
 /*!
@@ -744,97 +655,6 @@ save_xml_to_file(const xmlNode *xml, const char *desc, const char *filename)
     }
 
     crm_info("Saving %s to %s", desc, filename);
-    pcmk__xml_write_file(xml, filename, false, NULL);
+    pcmk__xml_write_file(xml, filename, false);
     free(f);
 }
-
-
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-#include <crm/common/xml_io_compat.h>
-
-xmlNode *
-filename2xml(const char *filename)
-{
-    return pcmk__xml_read(filename);
-}
-
-xmlNode *
-stdin2xml(void)
-{
-    return pcmk__xml_read(NULL);
-}
-
-xmlNode *
-string2xml(const char *input)
-{
-    return pcmk__xml_parse(input);
-}
-
-char *
-dump_xml_formatted(const xmlNode *xml)
-{
-    char *str = NULL;
-    GString *buffer = g_string_sized_new(1024);
-
-    pcmk__xml_string(xml, pcmk__xml_fmt_pretty, buffer, 0);
-
-    str = pcmk__str_copy(buffer->str);
-    g_string_free(buffer, TRUE);
-    return str;
-}
-
-char *
-dump_xml_formatted_with_text(const xmlNode *xml)
-{
-    char *str = NULL;
-    GString *buffer = g_string_sized_new(1024);
-
-    pcmk__xml_string(xml, pcmk__xml_fmt_pretty|pcmk__xml_fmt_text, buffer, 0);
-
-    str = pcmk__str_copy(buffer->str);
-    g_string_free(buffer, TRUE);
-    return str;
-}
-
-char *
-dump_xml_unformatted(const xmlNode *xml)
-{
-    char *str = NULL;
-    GString *buffer = g_string_sized_new(1024);
-
-    pcmk__xml_string(xml, 0, buffer, 0);
-
-    str = pcmk__str_copy(buffer->str);
-    g_string_free(buffer, TRUE);
-    return str;
-}
-
-int
-write_xml_fd(const xmlNode *xml, const char *filename, int fd,
-             gboolean compress)
-{
-    unsigned int nbytes = 0;
-    int rc = pcmk__xml_write_fd(xml, filename, fd, compress, &nbytes);
-
-    if (rc != pcmk_rc_ok) {
-        return pcmk_rc2legacy(rc);
-    }
-    return (int) nbytes;
-}
-
-int
-write_xml_file(const xmlNode *xml, const char *filename, gboolean compress)
-{
-    unsigned int nbytes = 0;
-    int rc = pcmk__xml_write_file(xml, filename, compress, &nbytes);
-
-    if (rc != pcmk_rc_ok) {
-        return pcmk_rc2legacy(rc);
-    }
-    return (int) nbytes;
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
