@@ -310,7 +310,7 @@ pcmk__xpath_node_id(const char *xpath, const char *node)
     start++;
 
     end = strstr(start, "\'");
-    CRM_ASSERT(end);
+    pcmk__assert(end != NULL);
     retval = strndup(start, end-start);
 
     free(patt);
@@ -340,57 +340,3 @@ pcmk__warn_multiple_name_matches(pcmk__output_t *out, xmlNode *search,
     out->info(out, "Multiple attributes match " PCMK_XA_NAME "=%s", name);
     pcmk__xe_foreach_child(search, NULL, output_attr_child, out);
 }
-
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-#include <crm/common/xml_compat.h>
-
-/*!
- * \deprecated This function will be removed in a future release
- * \brief Get an XPath string that matches an XML element as closely as possible
- *
- * \param[in] xml  The XML element for which to build an XPath string
- *
- * \return A string that matches \p xml, or \p NULL if \p xml is \p NULL.
- *
- * \note The caller is responsible for freeing the string using free().
- */
-char *
-xml_get_path(const xmlNode *xml)
-{
-    char *path = NULL;
-    GString *g_path = pcmk__element_xpath(xml);
-
-    if (g_path == NULL) {
-        return NULL;
-    }
-    path = pcmk__str_copy(g_path->str);
-    g_string_free(g_path, TRUE);
-    return path;
-}
-
-xmlNode *
-get_xpath_object_relative(const char *xpath, xmlNode *xml_obj, int error_level)
-{
-    xmlNode *result = NULL;
-    char *xpath_full = NULL;
-    char *xpath_prefix = NULL;
-
-    if (xml_obj == NULL || xpath == NULL) {
-        return NULL;
-    }
-
-    xpath_prefix = (char *)xmlGetNodePath(xml_obj);
-
-    xpath_full = crm_strdup_printf("%s%s", xpath_prefix, xpath);
-
-    result = get_xpath_object(xpath_full, xml_obj, error_level);
-
-    free(xpath_prefix);
-    free(xpath_full);
-    return result;
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
