@@ -88,10 +88,11 @@ log_action(stonith_action_t *action, pid_t pid)
 static void
 append_config_arg(gpointer key, gpointer value, gpointer user_data)
 {
-    /* The fencer will filter "action" out when it registers the device,
-     * but ignore it here in case any external API users don't.
+    /* Filter out parameters handled directly by Pacemaker.
      *
-     * Also filter out parameters handled directly by Pacemaker.
+     * STONITH_ATTR_ACTION_OP is added elsewhere and should never be part of the
+     * fencing resource's parameter list. We should ignore its value if it is
+     * configured there.
      */
     if (!pcmk__str_eq(key, STONITH_ATTR_ACTION_OP, pcmk__str_casei)
         && !pcmk_stonith_param(key)
@@ -142,6 +143,8 @@ make_args(const char *agent, const char *action, const char *target,
             action = value;
         }
     }
+
+    // Tell the fence agent what action to perform
     pcmk__insert_dup(arg_list, STONITH_ATTR_ACTION_OP, action);
 
     /* If this is a fencing operation against another node, add more standard
@@ -165,23 +168,21 @@ make_args(const char *agent, const char *action, const char *target,
             g_hash_table_insert(arg_list, strdup("nodeid"), nodeid);
         }
 
-        // Check whether target must be specified in some other way
+        // Check whether target should be specified as some other argument
         param = g_hash_table_lookup(device_args, PCMK_STONITH_HOST_ARGUMENT);
-        if (!pcmk__str_eq(agent, "fence_legacy", pcmk__str_none)
+        if (param == NULL) {
+            // Use caller's default (likely from agent metadata)
+            param = host_arg;
+        }
+        if ((param != NULL)
+            && !pcmk__str_eq(agent, "fence_legacy", pcmk__str_none)
             && !pcmk__str_eq(param, PCMK_VALUE_NONE, pcmk__str_casei)) {
 
-            if (param == NULL) {
-                /* Use the caller's default for pcmk_host_argument, or "port" if
-                 * none was given
-                 */
-                param = (host_arg == NULL)? "port" : host_arg;
-            }
             value = g_hash_table_lookup(device_args, param);
-
             if (pcmk__str_eq(value, "dynamic",
                              pcmk__str_casei|pcmk__str_null_matches)) {
-                /* If the host argument was "dynamic" or not explicitly specified,
-                 * add it with the target
+                /* If the host argument is "dynamic" or not configured,
+                 * reset it to the target
                  */
                 const char *alias = NULL;
 
