@@ -164,7 +164,6 @@ stonith_get_namespace(const char *agent, const char *namespace_s)
     }
 #endif
 
-    crm_err("Unknown fence agent: %s", agent);
     return st_namespace_invalid;
 }
 
@@ -286,11 +285,11 @@ stonith_connection_destroy(gpointer user_data)
     crm_xml_add(blob.xml, PCMK__XA_SUBT, PCMK__VALUE_ST_NOTIFY_DISCONNECT);
 
     foreach_notify_entry(native, stonith_send_notification, &blob);
-    free_xml(blob.xml);
+    pcmk__xml_free(blob.xml);
 }
 
 xmlNode *
-create_device_registration_xml(const char *id, enum stonith_namespace namespace,
+create_device_registration_xml(const char *id, enum stonith_namespace standard,
                                const char *agent,
                                const stonith_key_value_t *params,
                                const char *rsc_provides)
@@ -299,10 +298,10 @@ create_device_registration_xml(const char *id, enum stonith_namespace namespace,
     xmlNode *args = pcmk__xe_create(data, PCMK__XE_ATTRIBUTES);
 
 #if HAVE_STONITH_STONITH_H
-    if (namespace == st_namespace_any) {
-        namespace = stonith_get_namespace(agent, NULL);
+    if (standard == st_namespace_any) {
+        standard = stonith_get_namespace(agent, NULL);
     }
-    if (namespace == st_namespace_lha) {
+    if (standard == st_namespace_lha) {
         hash2field((gpointer) "plugin", (gpointer) agent, args);
         agent = "fence_legacy";
     }
@@ -311,9 +310,9 @@ create_device_registration_xml(const char *id, enum stonith_namespace namespace,
     crm_xml_add(data, PCMK_XA_ID, id);
     crm_xml_add(data, PCMK__XA_ST_ORIGIN, __func__);
     crm_xml_add(data, PCMK_XA_AGENT, agent);
-    if ((namespace != st_namespace_any) && (namespace != st_namespace_invalid)) {
+    if ((standard != st_namespace_any) && (standard != st_namespace_invalid)) {
         crm_xml_add(data, PCMK__XA_NAMESPACE,
-                    stonith_namespace2text(namespace));
+                    stonith_namespace2text(standard));
     }
     if (rsc_provides) {
         crm_xml_add(data, PCMK__XA_RSC_PROVIDES, rsc_provides);
@@ -340,7 +339,7 @@ stonith_api_register_device(stonith_t *st, int call_options,
                                           agent, params, NULL);
 
     rc = stonith_send_command(st, STONITH_OP_DEVICE_ADD, data, NULL, call_options, 0);
-    free_xml(data);
+    pcmk__xml_free(data);
 
     return rc;
 }
@@ -355,7 +354,7 @@ stonith_api_remove_device(stonith_t * st, int call_options, const char *name)
     crm_xml_add(data, PCMK__XA_ST_ORIGIN, __func__);
     crm_xml_add(data, PCMK_XA_ID, name);
     rc = stonith_send_command(st, STONITH_OP_DEVICE_DEL, data, NULL, call_options, 0);
-    free_xml(data);
+    pcmk__xml_free(data);
 
     return rc;
 }
@@ -386,7 +385,7 @@ stonith_api_remove_level_full(stonith_t *st, int options,
 
     crm_xml_add_int(data, PCMK_XA_INDEX, level);
     rc = stonith_send_command(st, STONITH_OP_LEVEL_DEL, data, NULL, options, 0);
-    free_xml(data);
+    pcmk__xml_free(data);
 
     return rc;
 }
@@ -463,7 +462,7 @@ stonith_api_register_level_full(stonith_t *st, int options, const char *node,
     CRM_CHECK(data != NULL, return -EINVAL);
 
     rc = stonith_send_command(st, STONITH_OP_LEVEL_ADD, data, NULL, options, 0);
-    free_xml(data);
+    pcmk__xml_free(data);
 
     return rc;
 }
@@ -517,7 +516,7 @@ stonith_api_device_metadata(stonith_t *stonith, int call_options,
     enum stonith_namespace ns = stonith_get_namespace(agent, namespace_s);
 
     if (timeout_sec <= 0) {
-        timeout_sec = PCMK_DEFAULT_METADATA_TIMEOUT_MS;
+        timeout_sec = PCMK_DEFAULT_ACTION_TIMEOUT_MS;
     }
 
     crm_trace("Looking up metadata for %s agent %s",
@@ -584,8 +583,8 @@ stonith_api_query(stonith_t * stonith, int call_options, const char *target,
         freeXpathObject(xpathObj);
     }
 
-    free_xml(output);
-    free_xml(data);
+    pcmk__xml_free(output);
+    pcmk__xml_free(data);
     return max;
 }
 
@@ -617,7 +616,7 @@ stonith_api_call(stonith_t *stonith, int call_options, const char *id,
 
     rc = stonith_send_command(stonith, STONITH_OP_EXEC, data, output,
                               call_options, timeout_sec);
-    free_xml(data);
+    pcmk__xml_free(data);
 
     return rc;
 }
@@ -643,7 +642,7 @@ stonith_api_list(stonith_t * stonith, int call_options, const char *id, char **l
     }
 
     if (output) {
-        free_xml(output);
+        pcmk__xml_free(output);
     }
 
     return rc;
@@ -679,7 +678,7 @@ stonith_api_fence_with_delay(stonith_t * stonith, int call_options, const char *
     crm_xml_add_int(data, PCMK__XA_ST_DELAY, delay);
 
     rc = stonith_send_command(stonith, STONITH_OP_FENCE, data, NULL, call_options, timeout);
-    free_xml(data);
+    pcmk__xml_free(data);
 
     return rc;
 }
@@ -719,7 +718,7 @@ stonith_api_history(stonith_t * stonith, int call_options, const char *node,
     stonith__set_call_options(call_options, node, st_opt_sync_call);
     rc = stonith_send_command(stonith, STONITH_OP_FENCE_HISTORY, data, &output,
                               call_options, timeout);
-    free_xml(data);
+    pcmk__xml_free(data);
 
     if (rc == 0) {
         xmlNode *op = NULL;
@@ -727,7 +726,7 @@ stonith_api_history(stonith_t * stonith, int call_options, const char *node,
                                           LOG_NEVER);
 
         for (op = pcmk__xe_first_child(reply, NULL, NULL, NULL); op != NULL;
-             op = pcmk__xe_next(op)) {
+             op = pcmk__xe_next(op, NULL)) {
             stonith_history_t *kvp;
             long long completed;
             long long completed_nsec = 0L;
@@ -754,7 +753,7 @@ stonith_api_history(stonith_t * stonith, int call_options, const char *node,
         }
     }
 
-    free_xml(output);
+    pcmk__xml_free(output);
 
     return rc;
 }
@@ -1024,7 +1023,8 @@ set_callback_timeout(stonith_callback_client_t * callback, stonith_t * stonith,
         g_source_remove(async_timer->ref);
     }
     async_timer->ref =
-        g_timeout_add(async_timer->timeout, stonith_async_timeout_handler, async_timer);
+        pcmk__create_timer(async_timer->timeout, stonith_async_timeout_handler,
+                           async_timer);
 }
 
 static void
@@ -1051,7 +1051,7 @@ stonith_dispatch_internal(const char *buffer, ssize_t length, gpointer userdata)
     stonith_t *st = userdata;
     stonith_private_t *private = NULL;
 
-    CRM_ASSERT(st != NULL);
+    pcmk__assert(st != NULL);
     private = st->st_private;
 
     blob.stonith = st;
@@ -1085,7 +1085,7 @@ stonith_dispatch_internal(const char *buffer, ssize_t length, gpointer userdata)
         crm_log_xml_warn(blob.xml, "BadReply");
     }
 
-    free_xml(blob.xml);
+    pcmk__xml_free(blob.xml);
     return 1;
 }
 
@@ -1104,7 +1104,7 @@ stonith_api_signon(stonith_t * stonith, const char *name, int *stonith_fd)
     CRM_CHECK(stonith != NULL, return -EINVAL);
 
     native = stonith->st_private;
-    CRM_ASSERT(native != NULL);
+    pcmk__assert(native != NULL);
 
     crm_debug("Attempting fencer connection by %s with%s mainloop",
               display_name, (stonith_fd? "out" : ""));
@@ -1149,7 +1149,7 @@ stonith_api_signon(stonith_t * stonith, const char *name, int *stonith_fd)
 
         if (rc < 0) {
             crm_debug("Couldn't register with the fencer: %s "
-                      CRM_XS " rc=%d", pcmk_strerror(rc), rc);
+                      QB_XS " rc=%d", pcmk_strerror(rc), rc);
             rc = -ECOMM;
 
         } else if (reply == NULL) {
@@ -1178,13 +1178,13 @@ stonith_api_signon(stonith_t * stonith, const char *name, int *stonith_fd)
             }
         }
 
-        free_xml(reply);
-        free_xml(hello);
+        pcmk__xml_free(reply);
+        pcmk__xml_free(hello);
     }
 
     if (rc != pcmk_ok) {
         crm_debug("Connection attempt to fencer by %s failed: %s "
-                  CRM_XS " rc=%d", display_name, pcmk_strerror(rc), rc);
+                  QB_XS " rc=%d", display_name, pcmk_strerror(rc), rc);
         stonith->cmds->disconnect(stonith);
     }
     return rc;
@@ -1215,7 +1215,7 @@ stonith_set_notification(stonith_t * stonith, const char *callback, int enabled)
         }
     }
 
-    free_xml(notify_msg);
+    pcmk__xml_free(notify_msg);
     return rc;
 }
 
@@ -1474,8 +1474,6 @@ event_free(stonith_event_t * event)
     struct event_private *event_private = event->opaque;
 
     free(event->id);
-    free(event->type);
-    free(event->message);
     free(event->operation);
     free(event->origin);
     free(event->action);
@@ -1554,7 +1552,8 @@ stonith_send_command(stonith_t * stonith, const char *op, xmlNode * data, xmlNod
     xmlNode *op_reply = NULL;
     stonith_private_t *native = NULL;
 
-    CRM_ASSERT(stonith && stonith->st_private && op);
+    pcmk__assert((stonith != NULL) && (stonith->st_private != NULL)
+                 && (op != NULL));
     native = stonith->st_private;
 
     if (output_data != NULL) {
@@ -1600,7 +1599,7 @@ stonith_send_command(stonith_t * stonith, const char *op, xmlNode * data, xmlNod
         rc = crm_ipc_send(native->ipc, op_msg, ipc_flags,
                           1000 * (timeout + 60), &op_reply);
     }
-    free_xml(op_msg);
+    pcmk__xml_free(op_msg);
 
     if (rc < 0) {
         crm_perror(LOG_ERR, "Couldn't perform %s operation (timeout=%ds): %d", op, timeout, rc);
@@ -1612,7 +1611,7 @@ stonith_send_command(stonith_t * stonith, const char *op, xmlNode * data, xmlNod
 
     if (!(call_options & st_opt_sync_call)) {
         crm_trace("Async call %d, returning", stonith->call_id);
-        free_xml(op_reply);
+        pcmk__xml_free(op_reply);
         return stonith->call_id;
     }
 
@@ -1638,14 +1637,14 @@ stonith_send_command(stonith_t * stonith, const char *op, xmlNode * data, xmlNod
     } else if (reply_id <= 0) {
         crm_err("Received bad reply: No id set");
         crm_log_xml_err(op_reply, "Bad reply");
-        free_xml(op_reply);
+        pcmk__xml_free(op_reply);
         op_reply = NULL;
         rc = -ENOMSG;
 
     } else {
         crm_err("Received bad reply: %d (wanted %d)", reply_id, stonith->call_id);
         crm_log_xml_err(op_reply, "Old reply");
-        free_xml(op_reply);
+        pcmk__xml_free(op_reply);
         op_reply = NULL;
         rc = -ENOMSG;
     }
@@ -1657,7 +1656,7 @@ stonith_send_command(stonith_t * stonith, const char *op, xmlNode * data, xmlNod
         stonith->state = stonith_disconnected;
     }
 
-    free_xml(op_reply);
+    pcmk__xml_free(op_reply);
     return rc;
 }
 
@@ -1668,7 +1667,7 @@ stonith_dispatch(stonith_t * st)
     gboolean stay_connected = TRUE;
     stonith_private_t *private = NULL;
 
-    CRM_ASSERT(st != NULL);
+    pcmk__assert(st != NULL);
     private = st->st_private;
 
     while (crm_ipc_ready(private->ipc)) {
@@ -1731,41 +1730,36 @@ stonith_api_delete(stonith_t * stonith)
     }
 }
 
-static int
-stonith_api_validate(stonith_t *st, int call_options, const char *rsc_id,
-                     const char *namespace_s, const char *agent,
-                     const stonith_key_value_t *params, int timeout_sec,
-                     char **output, char **error_output)
+static gboolean
+is_stonith_param(gpointer key, gpointer value, gpointer user_data)
 {
-    /* Validation should be done directly via the agent, so we can get it from
-     * stonith_admin when the cluster is not running, which is important for
-     * higher-level tools.
-     */
+    return pcmk_stonith_param(key);
+}
 
-    int rc = pcmk_ok;
+int
+stonith__validate(stonith_t *st, int call_options, const char *rsc_id,
+                  const char *namespace_s, const char *agent,
+                  GHashTable *params, int timeout_sec, char **output,
+                  char **error_output)
+{
+    int rc = pcmk_rc_ok;
 
     /* Use a dummy node name in case the agent requires a target. We assume the
      * actual target doesn't matter for validation purposes (if in practice,
      * that is incorrect, we will need to allow the caller to pass the target).
      */
     const char *target = "node1";
-    const char *host_arg = NULL;
+    char *host_arg = NULL;
 
-    GHashTable *params_table = pcmk__strkey_table(free, free);
+    if (params != NULL) {
+        host_arg = pcmk__str_copy(g_hash_table_lookup(params, PCMK_STONITH_HOST_ARGUMENT));
 
-    // Convert parameter list to a hash table
-    for (; params; params = params->next) {
-        if (pcmk__str_eq(params->key, PCMK_STONITH_HOST_ARGUMENT,
-                         pcmk__str_none)) {
-            host_arg = params->value;
-        }
-        if (!pcmk_stonith_param(params->key)) {
-            pcmk__insert_dup(params_table, params->key, params->value);
-        }
+        /* Remove special stonith params from the table before doing anything else */
+        g_hash_table_foreach_remove(params, is_stonith_param, NULL);
     }
 
-#if SUPPORT_CIBSECRETS
-    rc = pcmk__substitute_secrets(rsc_id, params_table);
+#if PCMK__ENABLE_CIBSECRETS
+    rc = pcmk__substitute_secrets(rsc_id, params);
     if (rc != pcmk_rc_ok) {
         crm_warn("Could not replace secret parameters for validation of %s: %s",
                  agent, pcmk_rc_str(rc));
@@ -1781,27 +1775,29 @@ stonith_api_validate(stonith_t *st, int call_options, const char *rsc_id,
     }
 
     if (timeout_sec <= 0) {
-        timeout_sec = PCMK_DEFAULT_METADATA_TIMEOUT_MS; // Questionable
+        timeout_sec = PCMK_DEFAULT_ACTION_TIMEOUT_MS;
     }
 
     switch (stonith_get_namespace(agent, namespace_s)) {
         case st_namespace_rhcs:
             rc = stonith__rhcs_validate(st, call_options, target, agent,
-                                        params_table, host_arg, timeout_sec,
+                                        params, host_arg, timeout_sec,
                                         output, error_output);
+            rc = pcmk_legacy2rc(rc);
             break;
 
 #if HAVE_STONITH_STONITH_H
         case st_namespace_lha:
             rc = stonith__lha_validate(st, call_options, target, agent,
-                                       params_table, timeout_sec, output,
+                                       params, timeout_sec, output,
                                        error_output);
+            rc = pcmk_legacy2rc(rc);
             break;
 #endif
 
         case st_namespace_invalid:
             errno = ENOENT;
-            rc = -errno;
+            rc = errno;
 
             if (error_output) {
                 *error_output = crm_strdup_printf("Agent %s not found", agent);
@@ -1813,7 +1809,7 @@ stonith_api_validate(stonith_t *st, int call_options, const char *rsc_id,
 
         default:
             errno = EOPNOTSUPP;
-            rc = -errno;
+            rc = errno;
 
             if (error_output) {
                 *error_output = crm_strdup_printf("Agent %s does not support validation",
@@ -1825,6 +1821,35 @@ stonith_api_validate(stonith_t *st, int call_options, const char *rsc_id,
             break;
     }
 
+    free(host_arg);
+    return rc;
+}
+
+static int
+stonith_api_validate(stonith_t *st, int call_options, const char *rsc_id,
+                     const char *namespace_s, const char *agent,
+                     const stonith_key_value_t *params, int timeout_sec,
+                     char **output, char **error_output)
+{
+    /* Validation should be done directly via the agent, so we can get it from
+     * stonith_admin when the cluster is not running, which is important for
+     * higher-level tools.
+     */
+
+    int rc = pcmk_ok;
+
+    GHashTable *params_table = pcmk__strkey_table(free, free);
+
+    // Convert parameter list to a hash table
+    for (; params; params = params->next) {
+        if (!pcmk_stonith_param(params->key)) {
+            pcmk__insert_dup(params_table, params->key, params->value);
+        }
+    }
+
+    rc = stonith__validate(st, call_options, rsc_id, namespace_s, agent,
+                           params_table, timeout_sec, output, error_output);
+
     g_hash_table_destroy(params_table);
     return rc;
 }
@@ -1918,12 +1943,12 @@ stonith_api_connect_retry(stonith_t *st, const char *name, int max_attempts)
             return pcmk_ok;
         } else if (attempt < max_attempts) {
             crm_notice("Fencer connection attempt %d of %d failed (retrying in 2s): %s "
-                       CRM_XS " rc=%d",
+                       QB_XS " rc=%d",
                        attempt, max_attempts, pcmk_strerror(rc), rc);
             sleep(2);
         }
     }
-    crm_notice("Could not connect to fencer: %s " CRM_XS " rc=%d",
+    crm_notice("Could not connect to fencer: %s " QB_XS " rc=%d",
                pcmk_strerror(rc), rc);
     return rc;
 }
@@ -1995,7 +2020,7 @@ stonith_api_kick(uint32_t nodeid, const char *uname, int timeout, bool off)
         int opts = 0;
 
         stonith__set_call_options(opts, name,
-                                  st_opt_sync_call|st_opt_allow_suicide);
+                                  st_opt_sync_call|st_opt_allow_self_fencing);
         if ((uname == NULL) && (nodeid > 0)) {
             stonith__set_call_options(opts, name, st_opt_cs_nodeid);
         }
@@ -2171,7 +2196,7 @@ parse_list_line(const char *line, int len, GList **output)
             rc = sscanf(line + entry_start, "%[a-zA-Z0-9_-.]", entry);
             if (rc != 1) {
                 crm_warn("Could not parse list output entry: %s "
-                         CRM_XS " entry_start=%d position=%d",
+                         QB_XS " entry_start=%d position=%d",
                          line + entry_start, entry_start, i);
                 free(entry);
 
@@ -2702,18 +2727,3 @@ stonith__event_description(const stonith_event_t *event)
                              ((reason == NULL)? "" : ")"),
                              pcmk__s(event->id, "(none)"));
 }
-
-
-// Deprecated functions kept only for backward API compatibility
-// LCOV_EXCL_START
-
-const char *get_stonith_provider(const char *agent, const char *provider);
-
-const char *
-get_stonith_provider(const char *agent, const char *provider)
-{
-    return stonith_namespace2text(stonith_get_namespace(agent, provider));
-}
-
-// LCOV_EXCL_STOP
-// End deprecated API
