@@ -33,7 +33,7 @@ remote_proxy_notify_destroy(lrmd_t *lrmd, const char *session_id)
     crm_xml_add(msg, PCMK__XA_LRMD_IPC_OP, LRMD_IPC_OP_DESTROY);
     crm_xml_add(msg, PCMK__XA_LRMD_IPC_SESSION, session_id);
     lrmd_internal_proxy_send(lrmd, msg);
-    free_xml(msg);
+    pcmk__xml_free(msg);
 }
 
 /*!
@@ -48,7 +48,7 @@ remote_proxy_ack_shutdown(lrmd_t *lrmd)
     xmlNode *msg = pcmk__xe_create(NULL, PCMK__XE_LRMD_IPC_PROXY);
     crm_xml_add(msg, PCMK__XA_LRMD_IPC_OP, LRMD_IPC_OP_SHUTDOWN_ACK);
     lrmd_internal_proxy_send(lrmd, msg);
-    free_xml(msg);
+    pcmk__xml_free(msg);
 }
 
 /*!
@@ -63,7 +63,7 @@ remote_proxy_nack_shutdown(lrmd_t *lrmd)
     xmlNode *msg = pcmk__xe_create(NULL, PCMK__XE_LRMD_IPC_PROXY);
     crm_xml_add(msg, PCMK__XA_LRMD_IPC_OP, LRMD_IPC_OP_SHUTDOWN_NACK);
     lrmd_internal_proxy_send(lrmd, msg);
-    free_xml(msg);
+    pcmk__xml_free(msg);
 }
 
 void
@@ -81,7 +81,7 @@ remote_proxy_relay_event(remote_proxy_t *proxy, xmlNode *msg)
 
     crm_log_xml_explicit(event, "EventForProxy");
     lrmd_internal_proxy_send(proxy->lrm, event);
-    free_xml(event);
+    pcmk__xml_free(event);
 }
 
 void
@@ -99,7 +99,7 @@ remote_proxy_relay_response(remote_proxy_t *proxy, xmlNode *msg, int msg_id)
     pcmk__xml_copy(wrapper, msg);
 
     lrmd_internal_proxy_send(proxy->lrm, response);
-    free_xml(response);
+    pcmk__xml_free(response);
 }
 
 static void
@@ -129,7 +129,7 @@ remote_proxy_free(gpointer data)
 int
 remote_proxy_dispatch(const char *buffer, ssize_t length, gpointer userdata)
 {
-    // Async responses from cib and friends to clients via pacemaker-remoted
+    // Async responses from servers to clients via the remote executor
     xmlNode *xml = NULL;
     uint32_t flags = 0;
     remote_proxy_t *proxy = userdata;
@@ -150,7 +150,7 @@ remote_proxy_dispatch(const char *buffer, ssize_t length, gpointer userdata)
         crm_trace("Passing event back to %.8s on %s: %.200s", proxy->session_id, proxy->node_name, buffer);
         remote_proxy_relay_event(proxy, xml);
     }
-    free_xml(xml);
+    pcmk__xml_free(xml);
     return 1;
 }
 
@@ -191,8 +191,8 @@ remote_proxy_new(lrmd_t *lrmd, struct ipc_client_callbacks *proxy_callbacks,
     proxy->session_id = strdup(session_id);
     proxy->lrm = lrmd;
 
-    if (!strcmp(pcmk__message_name(crm_system_name), CRM_SYSTEM_CRMD)
-        && !strcmp(pcmk__message_name(channel), CRM_SYSTEM_CRMD)) {
+    if ((pcmk__parse_server(crm_system_name) == pcmk_ipc_controld)
+        && (pcmk__parse_server(channel) == pcmk_ipc_controld)) {
         // The controller doesn't need to connect to itself
         proxy->is_local = TRUE;
 
@@ -235,7 +235,8 @@ remote_proxy_cb(lrmd_t *lrmd, const char *node_name, xmlNode *msg)
         remote_proxy_end_session(proxy);
 
     } else if (pcmk__str_eq(op, LRMD_IPC_OP_REQUEST, pcmk__str_casei)) {
-        int flags = 0;
+        uint32_t flags = 0U;
+        int rc = pcmk_rc_ok;
         const char *name = crm_element_value(msg, PCMK__XA_LRMD_IPC_CLIENT);
 
         xmlNode *wrapper = pcmk__xe_first_child(msg, PCMK__XE_LRMD_IPC_MSG,
@@ -259,10 +260,15 @@ remote_proxy_cb(lrmd_t *lrmd, const char *node_name, xmlNode *msg)
             return;
         }
         proxy->last_request_id = 0;
-        crm_element_value_int(msg, PCMK__XA_LRMD_IPC_MSG_FLAGS, &flags);
         crm_xml_add(request, PCMK_XE_ACL_ROLE, "pacemaker-remote");
 
-        CRM_ASSERT(node_name);
+        rc = pcmk__xe_get_flags(msg, PCMK__XA_LRMD_IPC_MSG_FLAGS, &flags, 0U);
+        if (rc != pcmk_rc_ok) {
+            crm_warn("Couldn't parse controller flags from remote request: %s",
+                     pcmk_rc_str(rc));
+        }
+
+        pcmk__assert(node_name != NULL);
         pcmk__update_acl_user(request, PCMK__XA_LRMD_IPC_USER, node_name);
 
         if (pcmk_is_set(flags, crm_ipc_proxied)) {
@@ -291,7 +297,7 @@ remote_proxy_cb(lrmd_t *lrmd, const char *node_name, xmlNode *msg)
                 crm_xml_add_int(op_reply, PCMK__XA_LINE, __LINE__);
                 crm_xml_add_int(op_reply, PCMK_XA_RC, rc);
                 remote_proxy_relay_response(proxy, op_reply, msg_id);
-                free_xml(op_reply);
+                pcmk__xml_free(op_reply);
 
             } else {
                 crm_trace("Relayed %s request %d from %s to %s for %s",
@@ -318,7 +324,7 @@ remote_proxy_cb(lrmd_t *lrmd, const char *node_name, xmlNode *msg)
 
             if(op_reply) {
                 remote_proxy_relay_response(proxy, op_reply, msg_id);
-                free_xml(op_reply);
+                pcmk__xml_free(op_reply);
             }
         }
     } else {
