@@ -7,16 +7,30 @@
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
-#ifndef PCMK__LIBPACEMAKER_PRIVATE__H
-#  define PCMK__LIBPACEMAKER_PRIVATE__H
+#ifndef PCMK__PACEMAKER_LIBPACEMAKER_PRIVATE__H
+#define PCMK__PACEMAKER_LIBPACEMAKER_PRIVATE__H
 
 /* This header is for the sole use of libpacemaker, so that functions can be
  * declared with G_GNUC_INTERNAL for efficiency.
  */
 
-#include <crm/lrmd_events.h>      // lrmd_event_data_t
-#include <crm/common/scheduler.h> // pcmk_action_t, pcmk_node_t, etc.
-#include <crm/pengine/internal.h> // pcmk__location_t
+#include <stdio.h>                  // NULL
+#include <stdint.h>                 // uint32_t
+#include <stdbool.h>                // bool, false
+#include <glib.h>                   // guint, gpointer, GList, GHashTable
+#include <libxml/tree.h>            // xmlNode
+
+#include <crm/common/scheduler.h>   // pcmk_action_t, pcmk_node_t, etc.
+#include <crm/common/scheduler_internal.h>  // pcmk__location_t, etc.
+#include <crm/cib.h>                // cib_t
+#include <crm/lrmd_events.h>        // lrmd_event_data_t
+#include <crm/pengine/internal.h>   // pe__const_top_resource(), etc.
+#include <pacemaker.h>              // pcmk_injections_t
+#include <pacemaker-internal.h>     // pcmk__colocation_t
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 // Colocation flags
 enum pcmk__coloc_flags {
@@ -66,7 +80,7 @@ enum pcmk__updated {
     } while (0)
 
 // Resource assignment methods
-struct resource_alloc_functions_s {
+struct pcmk__assignment_methods {
     /*!
      * \internal
      * \brief Assign a resource to a node
@@ -274,9 +288,10 @@ struct resource_alloc_functions_s {
      *                           node (only used when interleaving instances)
      * \param[in]     flags      Action flags for \p first for ordering purposes
      * \param[in]     filter     Action flags to limit scope of certain updates
-     *                           (may include pcmk_action_optional to affect
-     *                           only mandatory actions and pcmk_action_runnable
-     *                           to affect only runnable actions)
+     *                           (may include pcmk__action_optional to affect
+     *                           only mandatory actions and
+     *                           pcmk__action_runnable to affect only runnable
+     *                           actions)
      * \param[in]     type       Group of enum pcmk__action_relation_flags
      * \param[in,out] scheduler  Scheduler data
      *
@@ -440,6 +455,7 @@ void pcmk__order_restart_vs_unfence(gpointer data, gpointer user_data);
 
 // Injected scheduler inputs (pcmk_sched_injections.c)
 
+G_GNUC_INTERNAL
 void pcmk__inject_scheduler_input(pcmk_scheduler_t *scheduler, cib_t *cib,
                                   const pcmk_injections_t *injections);
 
@@ -450,6 +466,10 @@ G_GNUC_INTERNAL
 pcmk_resource_t *pcmk__find_constraint_resource(GList *rsc_list,
                                                 const char *id);
 
+G_GNUC_INTERNAL
+int pcmk__parse_constraint_role(const char *id, const char *role_spec,
+                                enum rsc_role_e *role);
+
 G_GNUC_INTERNAL
 xmlNode *pcmk__expand_tags_in_sets(xmlNode *xml_obj,
                                    const pcmk_scheduler_t *scheduler);
@@ -457,7 +477,7 @@ xmlNode *pcmk__expand_tags_in_sets(xmlNode *xml_obj,
 G_GNUC_INTERNAL
 bool pcmk__valid_resource_or_tag(const pcmk_scheduler_t *scheduler,
                                  const char *id, pcmk_resource_t **rsc,
-                                 pcmk_tag_t **tag);
+                                 pcmk__idref_t **tag);
 
 G_GNUC_INTERNAL
 bool pcmk__tag_to_set(xmlNode *xml_obj, xmlNode **rsc_set, const char *attr,
@@ -492,38 +512,10 @@ enum pcmk__coloc_affects {
     pcmk__coloc_affects_role,
 };
 
-/*!
- * \internal
- * \brief Get the value of a colocation's node attribute
- *
- * \param[in] node  Node on which to look up the attribute
- * \param[in] attr  Name of attribute to look up
- * \param[in] rsc   Resource on whose behalf to look up the attribute
- *
- * \return Value of \p attr on \p node or on the host of \p node, as appropriate
- */
-static inline const char *
-pcmk__colocation_node_attr(const pcmk_node_t *node, const char *attr,
-                           const pcmk_resource_t *rsc)
-{
-    const char *target = NULL;
-
-    /* A resource colocated with a bundle or its primitive can't run on the
-     * bundle node itself (where only the primitive, if any, can run). Instead,
-     * we treat it as a colocation with the bundle's containers, so always look
-     * up colocation node attributes on the container host.
-     */
-    if (pcmk__is_bundle_node(node) && pcmk__is_bundled(rsc)
-        && (pe__const_top_resource(rsc, false) == pe__bundled_resource(rsc))) {
-        target = PCMK_VALUE_HOST;
-
-    } else if (rsc != NULL) {
-        target = g_hash_table_lookup(rsc->meta,
-                                     PCMK_META_CONTAINER_ATTRIBUTE_TARGET);
-    }
-
-    return pcmk__node_attr(node, attr, target, pcmk__rsc_node_assigned);
-}
+G_GNUC_INTERNAL
+const char *pcmk__colocation_node_attr(const pcmk_node_t *node,
+                                       const char *attr,
+                                       const pcmk_resource_t *rsc);
 
 G_GNUC_INTERNAL
 enum pcmk__coloc_affects pcmk__colocation_affects(const pcmk_resource_t
@@ -589,57 +581,15 @@ GList *pcmk__this_with_colocations(const pcmk_resource_t *rsc);
 G_GNUC_INTERNAL
 void pcmk__new_colocation(const char *id, const char *node_attr, int score,
                           pcmk_resource_t *dependent, pcmk_resource_t *primary,
-                          const char *dependent_role, const char *primary_role,
-                          uint32_t flags);
+                          const char *dependent_role_spec,
+                          const char *primary_role_spec, uint32_t flags);
 
 G_GNUC_INTERNAL
 void pcmk__block_colocation_dependents(pcmk_action_t *action);
 
-/*!
- * \internal
- * \brief Check whether colocation's dependent preferences should be considered
- *
- * \param[in] colocation  Colocation constraint
- * \param[in] rsc         Primary instance (normally this will be
- *                        colocation->primary, which NULL will be treated as,
- *                        but for clones or bundles with multiple instances
- *                        this can be a particular instance)
- *
- * \return true if colocation influence should be effective, otherwise false
- */
-static inline bool
-pcmk__colocation_has_influence(const pcmk__colocation_t *colocation,
-                               const pcmk_resource_t *rsc)
-{
-    if (rsc == NULL) {
-        rsc = colocation->primary;
-    }
-
-    /* A bundle replica colocates its remote connection with its container,
-     * using a finite score so that the container can run on Pacemaker Remote
-     * nodes.
-     *
-     * Moving a connection is lightweight and does not interrupt the service,
-     * while moving a container is heavyweight and does interrupt the service,
-     * so don't move a clean, active container based solely on the preferences
-     * of its connection.
-     *
-     * This also avoids problematic scenarios where two containers want to
-     * perpetually swap places.
-     */
-    if (pcmk_is_set(colocation->dependent->flags,
-                    pcmk_rsc_remote_nesting_allowed)
-        && !pcmk_is_set(rsc->flags, pcmk_rsc_failed)
-        && pcmk__list_of_1(rsc->running_on)) {
-        return false;
-    }
-
-    /* The dependent in a colocation influences the primary's location
-     * if the PCMK_XA_INFLUENCE option is true or the primary is not yet active.
-     */
-    return pcmk_is_set(colocation->flags, pcmk__coloc_influence)
-           || (rsc->running_on == NULL);
-}
+G_GNUC_INTERNAL
+bool pcmk__colocation_has_influence(const pcmk__colocation_t *colocation,
+                                    const pcmk_resource_t *rsc);
 
 
 // Ordering constraints (pcmk_sched_ordering.c)
@@ -684,7 +634,7 @@ void pcmk__order_after_each(pcmk_action_t *after, GList *list);
                        NULL,                                                \
                        (then_rsc),                                          \
                        pcmk__op_key((then_rsc)->id, (then_task), 0),        \
-                       NULL, (flags), (first_rsc)->cluster)
+                       NULL, (flags), (first_rsc)->priv->scheduler)
 
 #define pcmk__order_starts(rsc1, rsc2, flags)                \
     pcmk__order_resource_actions((rsc1), PCMK_ACTION_START,  \
@@ -1033,11 +983,13 @@ xmlNode *pcmk__inject_resource_history(pcmk__output_t *out, xmlNode *cib_node,
 G_GNUC_INTERNAL
 void pcmk__inject_failcount(pcmk__output_t *out, cib_t *cib_conn,
                             xmlNode *cib_node, const char *resource,
-                            const char *task, guint interval_ms, int rc);
+                            const char *task, guint interval_ms, int rc,
+                            bool infinity);
 
 G_GNUC_INTERNAL
 xmlNode *pcmk__inject_action_result(xmlNode *cib_resource,
-                                    lrmd_event_data_t *op, int target_rc);
+                                    lrmd_event_data_t *op, const char *node,
+                                    int target_rc);
 
 
 // Nodes (pcmk_sched_nodes.c)
@@ -1180,4 +1132,11 @@ G_GNUC_INTERNAL
 int pcmk__setup_output_cib_sched(pcmk__output_t **out, cib_t **cib,
                                  pcmk_scheduler_t **scheduler, xmlNode **xml);
 
-#endif // PCMK__LIBPACEMAKER_PRIVATE__H
+G_GNUC_INTERNAL
+int pcmk__setup_output_fencing(pcmk__output_t **out, stonith_t **st, xmlNode **xml);
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif // PCMK__PACEMAKER_LIBPACEMAKER_PRIVATE__H
