@@ -89,11 +89,13 @@ inject_transient_attr(pcmk__output_t *out, xmlNode *cib_node,
  * \param[in]     exit_status  Action result for fail count to inject (if
  *                             \c PCMK_OCF_OK, or \c PCMK_OCF_NOT_RUNNING when
  *                             \p interval_ms is 0, inject nothing)
+ * \param[in]     infinity     If true, set fail count to "INFINITY", otherwise
+ *                             increase it by 1
  */
 void
 pcmk__inject_failcount(pcmk__output_t *out, cib_t *cib_conn, xmlNode *cib_node,
                        const char *resource, const char *task,
-                       guint interval_ms, int exit_status)
+                       guint interval_ms, int exit_status, bool infinity)
 {
     char *name = NULL;
     char *value = NULL;
@@ -120,12 +122,19 @@ pcmk__inject_failcount(pcmk__output_t *out, cib_t *cib_conn, xmlNode *cib_node,
             failcount = 0;
         }
     }
-    value = pcmk__itoa(failcount + 1);
+
+    if (infinity) {
+        value = pcmk__str_copy(PCMK_VALUE_INFINITY);
+
+    } else {
+        value = pcmk__itoa(failcount + 1);
+    }
+
     inject_transient_attr(out, cib_node, name, value);
 
     free(name);
     free(value);
-    free_xml(output);
+    pcmk__xml_free(output);
 
     name = pcmk__lastfailure_name(resource, task, interval_ms);
     value = pcmk__ttoa(time(NULL));
@@ -148,8 +157,7 @@ create_node_entry(cib_t *cib_conn, const char *node)
     int rc = pcmk_ok;
     char *xpath = crm_strdup_printf(XPATH_NODE_CONFIG, node);
 
-    rc = cib_conn->cmds->query(cib_conn, xpath, NULL,
-                               cib_xpath|cib_sync_call|cib_scope_local);
+    rc = cib_conn->cmds->query(cib_conn, xpath, NULL, cib_xpath|cib_sync_call);
 
     if (rc == -ENXIO) { // Only add if not already existing
         xmlNode *cib_object = pcmk__xe_create(NULL, PCMK_XE_NODE);
@@ -157,11 +165,11 @@ create_node_entry(cib_t *cib_conn, const char *node)
         crm_xml_add(cib_object, PCMK_XA_ID, node); // Use node name as ID
         crm_xml_add(cib_object, PCMK_XA_UNAME, node);
         cib_conn->cmds->create(cib_conn, PCMK_XE_NODES, cib_object,
-                               cib_sync_call|cib_scope_local);
+                               cib_sync_call);
         /* Not bothering with subsequent query to see if it exists,
            we'll bomb out later in the call to query_node_uuid()... */
 
-        free_xml(cib_object);
+        pcmk__xml_free(cib_object);
     }
 
     free(xpath);
@@ -190,13 +198,13 @@ create_op(const xmlNode *cib_resource, const char *task, guint interval_ms,
     op = lrmd_new_event(pcmk__xe_id(cib_resource), task, interval_ms);
     lrmd__set_result(op, outcome, PCMK_EXEC_DONE, "Simulated action result");
     op->params = NULL; // Not needed for simulation purposes
-    op->t_run = (unsigned int) time(NULL);
+    op->t_run = time(NULL);
     op->t_rcchange = op->t_run;
 
     // Use a call ID higher than any existing history entries
     op->call_id = 0;
     for (xop = pcmk__xe_first_child(cib_resource, NULL, NULL, NULL);
-         xop != NULL; xop = pcmk__xe_next(xop)) {
+         xop != NULL; xop = pcmk__xe_next(xop, NULL)) {
 
         int tmp = 0;
 
@@ -216,16 +224,17 @@ create_op(const xmlNode *cib_resource, const char *task, guint interval_ms,
  *
  * \param[in,out] cib_resource  Resource history XML to inject entry into
  * \param[in,out] op            Action result to inject
+ * \param[in]     node          Name of node where the action occurred
  * \param[in]     target_rc     Expected result for action to inject
  *
  * \return XML of injected resource history entry
  */
 xmlNode *
 pcmk__inject_action_result(xmlNode *cib_resource, lrmd_event_data_t *op,
-                           int target_rc)
+                           const char *node, int target_rc)
 {
     return pcmk__create_history_xml(cib_resource, op, CRM_FEATURE_SET,
-                                    target_rc, NULL, crm_system_name);
+                                    target_rc, node, crm_system_name);
 }
 
 /*!
@@ -255,7 +264,7 @@ pcmk__inject_node(cib_t *cib_conn, const char *node, const char *uuid)
     }
 
     rc = cib_conn->cmds->query(cib_conn, xpath, &cib_object,
-                               cib_xpath|cib_sync_call|cib_scope_local);
+                               cib_xpath|cib_sync_call);
 
     if ((cib_object != NULL) && (pcmk__xe_id(cib_object) == NULL)) {
         crm_err("Detected multiple " PCMK__XE_NODE_STATE " entries for "
@@ -280,7 +289,7 @@ pcmk__inject_node(cib_t *cib_conn, const char *node, const char *uuid)
              * PCMK_XA_UNAME yet
              */
             rc = cib_conn->cmds->query(cib_conn, xpath_by_uuid, &cib_object,
-                                       cib_xpath|cib_sync_call|cib_scope_local);
+                                       cib_xpath|cib_sync_call);
 
             if ((cib_object != NULL) && (pcmk__xe_id(cib_object) == NULL)) {
                 crm_err("Can't inject node state for %s because multiple "
@@ -293,8 +302,7 @@ pcmk__inject_node(cib_t *cib_conn, const char *node, const char *uuid)
                 crm_xml_add(cib_object, PCMK_XA_UNAME, node);
 
                 rc = cib_conn->cmds->modify(cib_conn, PCMK_XE_STATUS,
-                                            cib_object,
-                                            cib_sync_call|cib_scope_local);
+                                            cib_object, cib_sync_call);
             }
 
             free(xpath_by_uuid);
@@ -306,11 +314,11 @@ pcmk__inject_node(cib_t *cib_conn, const char *node, const char *uuid)
         crm_xml_add(cib_object, PCMK_XA_ID, found_uuid);
         crm_xml_add(cib_object, PCMK_XA_UNAME, node);
         cib_conn->cmds->create(cib_conn, PCMK_XE_STATUS, cib_object,
-                               cib_sync_call|cib_scope_local);
-        free_xml(cib_object);
+                               cib_sync_call);
+        pcmk__xml_free(cib_object);
 
         rc = cib_conn->cmds->query(cib_conn, xpath, &cib_object,
-                                   cib_xpath|cib_sync_call|cib_scope_local);
+                                   cib_xpath|cib_sync_call);
         crm_trace("Injecting node state for %s (rc=%d)", node, rc);
     }
 
@@ -324,7 +332,7 @@ done:
         return NULL; // not reached, but makes static analysis happy
     }
 
-    CRM_ASSERT(rc == pcmk_ok);
+    pcmk__assert(rc == pcmk_ok);
     return cib_object;
 }
 
@@ -437,7 +445,6 @@ pcmk__inject_resource_history(pcmk__output_t *out, xmlNode *cib_node,
                                      PCMK_RESOURCE_CLASS_OCF,
                                      PCMK_RESOURCE_CLASS_STONITH,
                                      PCMK_RESOURCE_CLASS_SERVICE,
-                                     PCMK_RESOURCE_CLASS_UPSTART,
                                      PCMK_RESOURCE_CLASS_SYSTEMD,
                                      PCMK_RESOURCE_CLASS_LSB, NULL)) {
         out->err(out, "Invalid class for %s: %s", resource, rclass);
@@ -529,11 +536,10 @@ set_ticket_state_attr(pcmk__output_t *out, const char *ticket_id,
     crm_log_xml_debug(xml_top, "Update");
 
     // Commit the change to the CIB
-    rc = cib->cmds->modify(cib, PCMK_XE_STATUS, xml_top,
-                           cib_sync_call|cib_scope_local);
+    rc = cib->cmds->modify(cib, PCMK_XE_STATUS, xml_top, cib_sync_call);
     rc = pcmk_legacy2rc(rc);
 
-    free_xml(xml_top);
+    pcmk__xml_free(xml_top);
     return rc;
 }
 
@@ -568,6 +574,7 @@ inject_action(pcmk__output_t *out, const char *spec, cib_t *cib,
     xmlNode *cib_resource = NULL;
     const pcmk_resource_t *rsc = NULL;
     lrmd_event_data_t *op = NULL;
+    bool infinity = false;
 
     out->message(out, "inject-spec", spec);
 
@@ -582,37 +589,45 @@ inject_action(pcmk__output_t *out, const char *spec, cib_t *cib,
 
     parse_op_key(key, &resource, &task, &interval_ms);
 
-    rsc = pe_find_resource(scheduler->resources, resource);
+    rsc = pe_find_resource(scheduler->priv->resources, resource);
     if (rsc == NULL) {
         out->err(out, "Invalid resource name: %s", resource);
         goto done;
     }
 
-    rclass = crm_element_value(rsc->xml, PCMK_XA_CLASS);
-    rtype = crm_element_value(rsc->xml, PCMK_XA_TYPE);
-    rprovider = crm_element_value(rsc->xml, PCMK_XA_PROVIDER);
+    rclass = crm_element_value(rsc->priv->xml, PCMK_XA_CLASS);
+    rtype = crm_element_value(rsc->priv->xml, PCMK_XA_TYPE);
+    rprovider = crm_element_value(rsc->priv->xml, PCMK_XA_PROVIDER);
 
     cib_node = pcmk__inject_node(cib, node, NULL);
-    CRM_ASSERT(cib_node != NULL);
+    pcmk__assert(cib_node != NULL);
+
+    if (pcmk__str_eq(task, PCMK_ACTION_STOP, pcmk__str_none)) {
+        infinity = true;
+
+    } else if (pcmk__str_eq(task, PCMK_ACTION_START, pcmk__str_none)
+               && pcmk_is_set(scheduler->flags,
+                              pcmk__sched_start_failure_fatal)) {
+        infinity = true;
+    }
 
     pcmk__inject_failcount(out, cib, cib_node, resource, task, interval_ms,
-                           outcome);
+                           outcome, infinity);
 
     cib_resource = pcmk__inject_resource_history(out, cib_node,
                                                  resource, resource,
                                                  rclass, rtype, rprovider);
-    CRM_ASSERT(cib_resource != NULL);
+    pcmk__assert(cib_resource != NULL);
 
     op = create_op(cib_resource, task, interval_ms, outcome);
-    CRM_ASSERT(op != NULL);
+    pcmk__assert(op != NULL);
 
-    cib_op = pcmk__inject_action_result(cib_resource, op, 0);
-    CRM_ASSERT(cib_op != NULL);
+    cib_op = pcmk__inject_action_result(cib_resource, op, node, 0);
+    pcmk__assert(cib_op != NULL);
     lrmd_free_event(op);
 
-    rc = cib->cmds->modify(cib, PCMK_XE_STATUS, cib_node,
-                           cib_sync_call|cib_scope_local);
-    CRM_ASSERT(rc == pcmk_ok);
+    rc = cib->cmds->modify(cib, PCMK_XE_STATUS, cib_node, cib_sync_call);
+    pcmk__assert(rc == pcmk_ok);
 
 done:
     free(task);
@@ -635,7 +650,7 @@ pcmk__inject_scheduler_input(pcmk_scheduler_t *scheduler, cib_t *cib,
     int rc = pcmk_ok;
     const GList *iter = NULL;
     xmlNode *cib_node = NULL;
-    pcmk__output_t *out = scheduler->priv;
+    pcmk__output_t *out = scheduler->priv->out;
 
     out->message(out, "inject-modify-config", injections->quorum,
                  injections->watchdog);
@@ -645,16 +660,16 @@ pcmk__inject_scheduler_input(pcmk_scheduler_t *scheduler, cib_t *cib,
         /* crm_xml_add(top, PCMK_XA_DC_UUID, dc_uuid);      */
         crm_xml_add(top, PCMK_XA_HAVE_QUORUM, injections->quorum);
 
-        rc = cib->cmds->modify(cib, NULL, top, cib_sync_call|cib_scope_local);
-        CRM_ASSERT(rc == pcmk_ok);
+        rc = cib->cmds->modify(cib, NULL, top, cib_sync_call);
+        pcmk__assert(rc == pcmk_ok);
     }
 
     if (injections->watchdog != NULL) {
-        rc = cib__update_node_attr(out, cib, cib_sync_call|cib_scope_local,
-                                   PCMK_XE_CRM_CONFIG, NULL, NULL, NULL,
-                                   NULL, PCMK_OPT_HAVE_WATCHDOG,
-                                   injections->watchdog, NULL, NULL);
-        CRM_ASSERT(rc == pcmk_rc_ok);
+        rc = cib__update_node_attr(out, cib, cib_sync_call, PCMK_XE_CRM_CONFIG,
+                                   NULL, NULL, NULL, NULL,
+                                   PCMK_OPT_HAVE_WATCHDOG, injections->watchdog,
+                                   NULL, NULL);
+        pcmk__assert(rc == pcmk_rc_ok);
     }
 
     for (iter = injections->node_up; iter != NULL; iter = iter->next) {
@@ -663,12 +678,11 @@ pcmk__inject_scheduler_input(pcmk_scheduler_t *scheduler, cib_t *cib,
         out->message(out, "inject-modify-node", "Online", node);
 
         cib_node = pcmk__inject_node_state_change(cib, node, true);
-        CRM_ASSERT(cib_node != NULL);
+        pcmk__assert(cib_node != NULL);
 
-        rc = cib->cmds->modify(cib, PCMK_XE_STATUS, cib_node,
-                               cib_sync_call|cib_scope_local);
-        CRM_ASSERT(rc == pcmk_ok);
-        free_xml(cib_node);
+        rc = cib->cmds->modify(cib, PCMK_XE_STATUS, cib_node, cib_sync_call);
+        pcmk__assert(rc == pcmk_ok);
+        pcmk__xml_free(cib_node);
     }
 
     for (iter = injections->node_down; iter != NULL; iter = iter->next) {
@@ -678,27 +692,24 @@ pcmk__inject_scheduler_input(pcmk_scheduler_t *scheduler, cib_t *cib,
         out->message(out, "inject-modify-node", "Offline", node);
 
         cib_node = pcmk__inject_node_state_change(cib, node, false);
-        CRM_ASSERT(cib_node != NULL);
+        pcmk__assert(cib_node != NULL);
 
-        rc = cib->cmds->modify(cib, PCMK_XE_STATUS, cib_node,
-                               cib_sync_call|cib_scope_local);
-        CRM_ASSERT(rc == pcmk_ok);
-        free_xml(cib_node);
+        rc = cib->cmds->modify(cib, PCMK_XE_STATUS, cib_node, cib_sync_call);
+        pcmk__assert(rc == pcmk_ok);
+        pcmk__xml_free(cib_node);
 
         xpath = crm_strdup_printf("//" PCMK__XE_NODE_STATE
                                   "[@" PCMK_XA_UNAME "='%s']"
                                   "/" PCMK__XE_LRM,
                                   node);
-        cib->cmds->remove(cib, xpath, NULL,
-                          cib_xpath|cib_sync_call|cib_scope_local);
+        cib->cmds->remove(cib, xpath, NULL, cib_xpath|cib_sync_call);
         free(xpath);
 
         xpath = crm_strdup_printf("//" PCMK__XE_NODE_STATE
                                   "[@" PCMK_XA_UNAME "='%s']"
                                   "/" PCMK__XE_TRANSIENT_ATTRIBUTES,
                                   node);
-        cib->cmds->remove(cib, xpath, NULL,
-                          cib_xpath|cib_sync_call|cib_scope_local);
+        cib->cmds->remove(cib, xpath, NULL, cib_xpath|cib_sync_call);
         free(xpath);
     }
 
@@ -709,12 +720,11 @@ pcmk__inject_scheduler_input(pcmk_scheduler_t *scheduler, cib_t *cib,
 
         cib_node = pcmk__inject_node_state_change(cib, node, true);
         crm_xml_add(cib_node, PCMK__XA_IN_CCM, PCMK_VALUE_FALSE);
-        CRM_ASSERT(cib_node != NULL);
+        pcmk__assert(cib_node != NULL);
 
-        rc = cib->cmds->modify(cib, PCMK_XE_STATUS, cib_node,
-                               cib_sync_call|cib_scope_local);
-        CRM_ASSERT(rc == pcmk_ok);
-        free_xml(cib_node);
+        rc = cib->cmds->modify(cib, PCMK_XE_STATUS, cib_node, cib_sync_call);
+        pcmk__assert(rc == pcmk_ok);
+        pcmk__xml_free(cib_node);
     }
 
     for (iter = injections->ticket_grant; iter != NULL; iter = iter->next) {
@@ -723,7 +733,7 @@ pcmk__inject_scheduler_input(pcmk_scheduler_t *scheduler, cib_t *cib,
         out->message(out, "inject-modify-ticket", "Granting", ticket_id);
 
         rc = set_ticket_state_attr(out, ticket_id, PCMK__XA_GRANTED, true, cib);
-        CRM_ASSERT(rc == pcmk_rc_ok);
+        pcmk__assert(rc == pcmk_rc_ok);
     }
 
     for (iter = injections->ticket_revoke; iter != NULL; iter = iter->next) {
@@ -733,7 +743,7 @@ pcmk__inject_scheduler_input(pcmk_scheduler_t *scheduler, cib_t *cib,
 
         rc = set_ticket_state_attr(out, ticket_id, PCMK__XA_GRANTED, false,
                                    cib);
-        CRM_ASSERT(rc == pcmk_rc_ok);
+        pcmk__assert(rc == pcmk_rc_ok);
     }
 
     for (iter = injections->ticket_standby; iter != NULL; iter = iter->next) {
@@ -742,7 +752,7 @@ pcmk__inject_scheduler_input(pcmk_scheduler_t *scheduler, cib_t *cib,
         out->message(out, "inject-modify-ticket", "Standby", ticket_id);
 
         rc = set_ticket_state_attr(out, ticket_id, PCMK_XA_STANDBY, true, cib);
-        CRM_ASSERT(rc == pcmk_rc_ok);
+        pcmk__assert(rc == pcmk_rc_ok);
     }
 
     for (iter = injections->ticket_activate; iter != NULL; iter = iter->next) {
@@ -751,7 +761,7 @@ pcmk__inject_scheduler_input(pcmk_scheduler_t *scheduler, cib_t *cib,
         out->message(out, "inject-modify-ticket", "Activating", ticket_id);
 
         rc = set_ticket_state_attr(out, ticket_id, PCMK_XA_STANDBY, false, cib);
-        CRM_ASSERT(rc == pcmk_rc_ok);
+        pcmk__assert(rc == pcmk_rc_ok);
     }
 
     for (iter = injections->op_inject; iter != NULL; iter = iter->next) {
