@@ -42,30 +42,26 @@ colocations_header(pcmk_resource_t *rsc, pcmk__colocation_t *cons,
 static void
 colocations_xml_node(pcmk__output_t *out, pcmk_resource_t *rsc,
                      pcmk__colocation_t *cons) {
-    xmlNodePtr node = NULL;
-
-    node = pcmk__output_create_xml_node(out, PCMK_XE_RSC_COLOCATION,
-                                        PCMK_XA_ID, cons->id,
-                                        PCMK_XA_RSC, cons->dependent->id,
-                                        PCMK_XA_WITH_RSC, cons->primary->id,
-                                        PCMK_XA_SCORE,
-                                        pcmk_readable_score(cons->score),
-                                        NULL);
-
-    if (cons->node_attribute) {
-        xmlSetProp(node, (pcmkXmlStr) PCMK_XA_NODE_ATTRIBUTE,
-                   (pcmkXmlStr) cons->node_attribute);
-    }
+    const char *score = pcmk_readable_score(cons->score);
+    const char *dependent_role = NULL;
+    const char *primary_role = NULL;
 
     if (cons->dependent_role != pcmk_role_unknown) {
-        xmlSetProp(node, (pcmkXmlStr) PCMK_XA_RSC_ROLE,
-                   (pcmkXmlStr) pcmk_role_text(cons->dependent_role));
+        dependent_role = pcmk_role_text(cons->dependent_role);
+    }
+    if (cons->primary_role != pcmk_role_unknown) {
+        primary_role = pcmk_role_text(cons->primary_role);
     }
 
-    if (cons->primary_role != pcmk_role_unknown) {
-        xmlSetProp(node, (pcmkXmlStr) PCMK_XA_WITH_RSC_ROLE,
-                   (pcmkXmlStr) pcmk_role_text(cons->primary_role));
-    }
+    pcmk__output_create_xml_node(out, PCMK_XE_RSC_COLOCATION,
+                                 PCMK_XA_ID, cons->id,
+                                 PCMK_XA_RSC, cons->dependent->id,
+                                 PCMK_XA_WITH_RSC, cons->primary->id,
+                                 PCMK_XA_SCORE, score,
+                                 PCMK_XA_NODE_ATTRIBUTE, cons->node_attribute,
+                                 PCMK_XA_RSC_ROLE, dependent_role,
+                                 PCMK_XA_WITH_RSC_ROLE, primary_role,
+                                 NULL);
 }
 
 static int
@@ -73,10 +69,10 @@ do_locations_list_xml(pcmk__output_t *out, pcmk_resource_t *rsc,
                       bool add_header)
 {
     GList *lpc = NULL;
-    GList *list = rsc->rsc_location;
     int rc = pcmk_rc_no_output;
 
-    for (lpc = list; lpc != NULL; lpc = lpc->next) {
+    for (lpc = rsc->priv->location_constraints;
+         lpc != NULL; lpc = lpc->next) {
         pcmk__location_t *cons = lpc->data;
 
         GList *lpc2 = NULL;
@@ -89,11 +85,11 @@ do_locations_list_xml(pcmk__output_t *out, pcmk_resource_t *rsc,
             }
 
             pcmk__output_create_xml_node(out, PCMK_XE_RSC_LOCATION,
-                                         PCMK_XA_NODE, node->details->uname,
+                                         PCMK_XA_NODE, node->priv->name,
                                          PCMK_XA_RSC, rsc->id,
                                          PCMK_XA_ID, cons->id,
                                          PCMK_XA_SCORE,
-                                         pcmk_readable_score(node->weight),
+                                         pcmk_readable_score(node->assign->score),
                                          NULL);
         }
     }
@@ -128,8 +124,8 @@ rsc_action_item(pcmk__output_t *out, va_list args)
     static int rsc_width = 5;
     static int detail_width = 5;
 
-    CRM_ASSERT(action);
-    CRM_ASSERT(destination != NULL || origin != NULL);
+    pcmk__assert((action != NULL)
+                 && ((destination != NULL) || (origin != NULL)));
 
     if (source == NULL) {
         source = action;
@@ -140,8 +136,8 @@ rsc_action_item(pcmk__output_t *out, va_list args)
         rsc_width = len + 2;
     }
 
-    if ((rsc->role > pcmk_role_started)
-        || (rsc->next_role > pcmk_role_unpromoted)) {
+    if ((rsc->priv->orig_role > pcmk_role_started)
+        || (rsc->priv->next_role > pcmk_role_unpromoted)) {
         need_role = true;
     }
 
@@ -149,14 +145,15 @@ rsc_action_item(pcmk__output_t *out, va_list args)
         same_host = true;
     }
 
-    if (rsc->role == rsc->next_role) {
+    if (rsc->priv->orig_role == rsc->priv->next_role) {
         same_role = true;
     }
 
     if (need_role && (origin == NULL)) {
         /* Starting and promoting a promotable clone instance */
-        details = crm_strdup_printf("%s -> %s %s", pcmk_role_text(rsc->role),
-                                    pcmk_role_text(rsc->next_role),
+        details = crm_strdup_printf("%s -> %s %s",
+                                    pcmk_role_text(rsc->priv->orig_role),
+                                    pcmk_role_text(rsc->priv->next_role),
                                     pcmk__node_name(destination));
 
     } else if (origin == NULL) {
@@ -165,7 +162,8 @@ rsc_action_item(pcmk__output_t *out, va_list args)
 
     } else if (need_role && (destination == NULL)) {
         /* Stopping a promotable clone instance */
-        details = crm_strdup_printf("%s %s", pcmk_role_text(rsc->role),
+        details = crm_strdup_printf("%s %s",
+                                    pcmk_role_text(rsc->priv->orig_role),
                                     pcmk__node_name(origin));
 
     } else if (destination == NULL) {
@@ -174,7 +172,8 @@ rsc_action_item(pcmk__output_t *out, va_list args)
 
     } else if (need_role && same_role && same_host) {
         /* Recovering, restarting or re-promoting a promotable clone instance */
-        details = crm_strdup_printf("%s %s", pcmk_role_text(rsc->role),
+        details = crm_strdup_printf("%s %s",
+                                    pcmk_role_text(rsc->priv->orig_role),
                                     pcmk__node_name(origin));
 
     } else if (same_role && same_host) {
@@ -185,7 +184,7 @@ rsc_action_item(pcmk__output_t *out, va_list args)
         /* Moving a promotable clone instance */
         details = crm_strdup_printf("%s -> %s %s", pcmk__node_name(origin),
                                     pcmk__node_name(destination),
-                                    pcmk_role_text(rsc->role));
+                                    pcmk_role_text(rsc->priv->orig_role));
 
     } else if (same_role) {
         /* Moving a normal resource */
@@ -194,16 +193,17 @@ rsc_action_item(pcmk__output_t *out, va_list args)
 
     } else if (same_host) {
         /* Promoting or demoting a promotable clone instance */
-        details = crm_strdup_printf("%s -> %s %s", pcmk_role_text(rsc->role),
-                                    pcmk_role_text(rsc->next_role),
+        details = crm_strdup_printf("%s -> %s %s",
+                                    pcmk_role_text(rsc->priv->orig_role),
+                                    pcmk_role_text(rsc->priv->next_role),
                                     pcmk__node_name(origin));
 
     } else {
         /* Moving and promoting/demoting */
         details = crm_strdup_printf("%s %s -> %s %s",
-                                    pcmk_role_text(rsc->role),
+                                    pcmk_role_text(rsc->priv->orig_role),
                                     pcmk__node_name(origin),
-                                    pcmk_role_text(rsc->next_role),
+                                    pcmk_role_text(rsc->priv->next_role),
                                     pcmk__node_name(destination));
     }
 
@@ -213,13 +213,13 @@ rsc_action_item(pcmk__output_t *out, va_list args)
     }
 
     if ((source->reason != NULL)
-        && !pcmk_is_set(action->flags, pcmk_action_runnable)) {
+        && !pcmk_is_set(action->flags, pcmk__action_runnable)) {
         reason = crm_strdup_printf("due to %s (blocked)", source->reason);
 
     } else if (source->reason) {
         reason = crm_strdup_printf("due to %s", source->reason);
 
-    } else if (!pcmk_is_set(action->flags, pcmk_action_runnable)) {
+    } else if (!pcmk_is_set(action->flags, pcmk__action_runnable)) {
         reason = strdup("blocked");
 
     }
@@ -253,15 +253,15 @@ rsc_action_item_xml(pcmk__output_t *out, va_list args)
     bool need_role = false;
     xmlNode *xml = NULL;
 
-    CRM_ASSERT(action);
-    CRM_ASSERT(destination != NULL || origin != NULL);
+    pcmk__assert((action != NULL)
+                 && ((destination != NULL) || (origin != NULL)));
 
     if (source == NULL) {
         source = action;
     }
 
-    if ((rsc->role > pcmk_role_started)
-        || (rsc->next_role > pcmk_role_unpromoted)) {
+    if ((rsc->priv->orig_role > pcmk_role_started)
+        || (rsc->priv->next_role > pcmk_role_unpromoted)) {
         need_role = true;
     }
 
@@ -269,7 +269,7 @@ rsc_action_item_xml(pcmk__output_t *out, va_list args)
         same_host = true;
     }
 
-    if (rsc->role == rsc->next_role) {
+    if (rsc->priv->orig_role == rsc->priv->next_role) {
         same_role = true;
     }
 
@@ -283,72 +283,81 @@ rsc_action_item_xml(pcmk__output_t *out, va_list args)
     if (need_role && (origin == NULL)) {
         /* Starting and promoting a promotable clone instance */
         pcmk__xe_set_props(xml,
-                           PCMK_XA_ROLE, pcmk_role_text(rsc->role),
-                           PCMK_XA_NEXT_ROLE, pcmk_role_text(rsc->next_role),
-                           PCMK_XA_DEST, destination->details->uname,
+                           PCMK_XA_ROLE,
+                           pcmk_role_text(rsc->priv->orig_role),
+                           PCMK_XA_NEXT_ROLE,
+                           pcmk_role_text(rsc->priv->next_role),
+                           PCMK_XA_DEST, destination->priv->name,
                            NULL);
 
     } else if (origin == NULL) {
         /* Starting a resource */
-        crm_xml_add(xml, PCMK_XA_NODE, destination->details->uname);
+        crm_xml_add(xml, PCMK_XA_NODE, destination->priv->name);
 
     } else if (need_role && (destination == NULL)) {
         /* Stopping a promotable clone instance */
         pcmk__xe_set_props(xml,
-                           PCMK_XA_ROLE, pcmk_role_text(rsc->role),
-                           PCMK_XA_NODE, origin->details->uname,
+                           PCMK_XA_ROLE,
+                           pcmk_role_text(rsc->priv->orig_role),
+                           PCMK_XA_NODE, origin->priv->name,
                            NULL);
 
     } else if (destination == NULL) {
         /* Stopping a resource */
-        crm_xml_add(xml, PCMK_XA_NODE, origin->details->uname);
+        crm_xml_add(xml, PCMK_XA_NODE, origin->priv->name);
 
     } else if (need_role && same_role && same_host) {
         /* Recovering, restarting or re-promoting a promotable clone instance */
         pcmk__xe_set_props(xml,
-                           PCMK_XA_ROLE, pcmk_role_text(rsc->role),
-                           PCMK_XA_SOURCE, origin->details->uname,
+                           PCMK_XA_ROLE,
+                           pcmk_role_text(rsc->priv->orig_role),
+                           PCMK_XA_SOURCE, origin->priv->name,
                            NULL);
 
     } else if (same_role && same_host) {
         /* Recovering or Restarting a normal resource */
-        crm_xml_add(xml, PCMK_XA_SOURCE, origin->details->uname);
+        crm_xml_add(xml, PCMK_XA_SOURCE, origin->priv->name);
 
     } else if (need_role && same_role) {
         /* Moving a promotable clone instance */
         pcmk__xe_set_props(xml,
-                           PCMK_XA_SOURCE, origin->details->uname,
-                           PCMK_XA_DEST, destination->details->uname,
-                           PCMK_XA_ROLE, pcmk_role_text(rsc->role),
+                           PCMK_XA_SOURCE, origin->priv->name,
+                           PCMK_XA_DEST, destination->priv->name,
+                           PCMK_XA_ROLE,
+                           pcmk_role_text(rsc->priv->orig_role),
                            NULL);
 
     } else if (same_role) {
         /* Moving a normal resource */
         pcmk__xe_set_props(xml,
-                           PCMK_XA_SOURCE, origin->details->uname,
-                           PCMK_XA_DEST, destination->details->uname,
+                           PCMK_XA_SOURCE, origin->priv->name,
+                           PCMK_XA_DEST, destination->priv->name,
                            NULL);
 
     } else if (same_host) {
         /* Promoting or demoting a promotable clone instance */
         pcmk__xe_set_props(xml,
-                           PCMK_XA_ROLE, pcmk_role_text(rsc->role),
-                           PCMK_XA_NEXT_ROLE, pcmk_role_text(rsc->next_role),
-                           PCMK_XA_SOURCE, origin->details->uname,
+                           PCMK_XA_ROLE,
+                           pcmk_role_text(rsc->priv->orig_role),
+                           PCMK_XA_NEXT_ROLE,
+                           pcmk_role_text(rsc->priv->next_role),
+                           PCMK_XA_SOURCE, origin->priv->name,
                            NULL);
 
     } else {
         /* Moving and promoting/demoting */
         pcmk__xe_set_props(xml,
-                           PCMK_XA_ROLE, pcmk_role_text(rsc->role),
-                           PCMK_XA_SOURCE, origin->details->uname,
-                           PCMK_XA_NEXT_ROLE, pcmk_role_text(rsc->next_role),
-                           PCMK_XA_DEST, destination->details->uname,
+                           PCMK_XA_ROLE,
+                           pcmk_role_text(rsc->priv->orig_role),
+                           PCMK_XA_SOURCE, origin->priv->name,
+                           PCMK_XA_NEXT_ROLE,
+                           pcmk_role_text(rsc->priv->next_role),
+                           PCMK_XA_DEST, destination->priv->name,
                            NULL);
     }
 
     if ((source->reason != NULL)
-        && !pcmk_is_set(action->flags, pcmk_action_runnable)) {
+        && !pcmk_is_set(action->flags, pcmk__action_runnable)) {
         pcmk__xe_set_props(xml,
                            PCMK_XA_REASON, source->reason,
                            PCMK_XA_BLOCKED, PCMK_VALUE_TRUE,
@@ -357,7 +366,7 @@ rsc_action_item_xml(pcmk__output_t *out, va_list args)
     } else if (source->reason != NULL) {
         crm_xml_add(xml, PCMK_XA_REASON, source->reason);
 
-    } else if (!pcmk_is_set(action->flags, pcmk_action_runnable)) {
+    } else if (!pcmk_is_set(action->flags, pcmk__action_runnable)) {
         pcmk__xe_set_bool_attr(xml, PCMK_XA_BLOCKED, true);
 
     }
@@ -373,22 +382,24 @@ rsc_is_colocated_with_list(pcmk__output_t *out, va_list args) {
 
     int rc = pcmk_rc_no_output;
 
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_detect_loop)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_detect_loop)) {
         return rc;
     }
 
-    /* We're listing constraints explicitly involving rsc, so use rsc->rsc_cons
-     * directly rather than rsc->cmds->this_with_colocations().
+    /* We're listing constraints explicitly involving rsc, so use
+     * rsc->private->this_with_colocations directly rather than call
+     * rsc->private->cmds->this_with_colocations().
      */
-    pcmk__set_rsc_flags(rsc, pcmk_rsc_detect_loop);
-    for (GList *lpc = rsc->rsc_cons; lpc != NULL; lpc = lpc->next) {
+    pcmk__set_rsc_flags(rsc, pcmk__rsc_detect_loop);
+    for (GList *lpc = rsc->priv->this_with_colocations;
+         lpc != NULL; lpc = lpc->next) {
         pcmk__colocation_t *cons = (pcmk__colocation_t *) lpc->data;
         char *hdr = NULL;
 
         PCMK__OUTPUT_LIST_HEADER(out, false, rc,
                                  "Resources %s is colocated with", rsc->id);
 
-        if (pcmk_is_set(cons->primary->flags, pcmk_rsc_detect_loop)) {
+        if (pcmk_is_set(cons->primary->flags, pcmk__rsc_detect_loop)) {
             out->list_item(out, NULL, "%s (id=%s - loop)",
                            cons->primary->id, cons->id);
             continue;
@@ -422,18 +433,20 @@ rsc_is_colocated_with_list_xml(pcmk__output_t *out, va_list args) {
 
     int rc = pcmk_rc_no_output;
 
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_detect_loop)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_detect_loop)) {
         return rc;
     }
 
-    /* We're listing constraints explicitly involving rsc, so use rsc->rsc_cons
-     * directly rather than rsc->cmds->this_with_colocations().
+    /* We're listing constraints explicitly involving rsc, so use
+     * rsc->private->this_with_colocations directly rather than call
+     * rsc->private->cmds->this_with_colocations().
      */
-    pcmk__set_rsc_flags(rsc, pcmk_rsc_detect_loop);
-    for (GList *lpc = rsc->rsc_cons; lpc != NULL; lpc = lpc->next) {
+    pcmk__set_rsc_flags(rsc, pcmk__rsc_detect_loop);
+    for (GList *lpc = rsc->priv->this_with_colocations;
+         lpc != NULL; lpc = lpc->next) {
         pcmk__colocation_t *cons = (pcmk__colocation_t *) lpc->data;
 
-        if (pcmk_is_set(cons->primary->flags, pcmk_rsc_detect_loop)) {
+        if (pcmk_is_set(cons->primary->flags, pcmk__rsc_detect_loop)) {
             colocations_xml_node(out, cons->primary, cons);
             continue;
         }
@@ -458,23 +471,24 @@ rscs_colocated_with_list(pcmk__output_t *out, va_list args) {
 
     int rc = pcmk_rc_no_output;
 
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_detect_loop)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_detect_loop)) {
         return rc;
     }
 
     /* We're listing constraints explicitly involving rsc, so use
-     * rsc->rsc_cons_lhs directly rather than
-     * rsc->cmds->with_this_colocations().
+     * rsc->private->with_this_colocations directly rather than
+     * rsc->private->cmds->with_this_colocations().
      */
-    pcmk__set_rsc_flags(rsc, pcmk_rsc_detect_loop);
-    for (GList *lpc = rsc->rsc_cons_lhs; lpc != NULL; lpc = lpc->next) {
+    pcmk__set_rsc_flags(rsc, pcmk__rsc_detect_loop);
+    for (GList *lpc = rsc->priv->with_this_colocations;
+         lpc != NULL; lpc = lpc->next) {
         pcmk__colocation_t *cons = (pcmk__colocation_t *) lpc->data;
         char *hdr = NULL;
 
         PCMK__OUTPUT_LIST_HEADER(out, false, rc, "Resources colocated with %s",
                                  rsc->id);
 
-        if (pcmk_is_set(cons->dependent->flags, pcmk_rsc_detect_loop)) {
+        if (pcmk_is_set(cons->dependent->flags, pcmk__rsc_detect_loop)) {
             out->list_item(out, NULL, "%s (id=%s - loop)",
                            cons->dependent->id, cons->id);
             continue;
@@ -508,19 +522,20 @@ rscs_colocated_with_list_xml(pcmk__output_t *out, va_list args) {
 
     int rc = pcmk_rc_no_output;
 
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_detect_loop)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_detect_loop)) {
         return rc;
     }
 
     /* We're listing constraints explicitly involving rsc, so use
-     * rsc->rsc_cons_lhs directly rather than
-     * rsc->cmds->with_this_colocations().
+     * rsc->private->with_this_colocations directly rather than
+     * rsc->private->cmds->with_this_colocations().
      */
-    pcmk__set_rsc_flags(rsc, pcmk_rsc_detect_loop);
-    for (GList *lpc = rsc->rsc_cons_lhs; lpc != NULL; lpc = lpc->next) {
+    pcmk__set_rsc_flags(rsc, pcmk__rsc_detect_loop);
+    for (GList *lpc = rsc->priv->with_this_colocations;
+         lpc != NULL; lpc = lpc->next) {
         pcmk__colocation_t *cons = (pcmk__colocation_t *) lpc->data;
 
-        if (pcmk_is_set(cons->dependent->flags, pcmk_rsc_detect_loop)) {
+        if (pcmk_is_set(cons->dependent->flags, pcmk__rsc_detect_loop)) {
             colocations_xml_node(out, cons->dependent, cons);
             continue;
         }
@@ -543,10 +558,10 @@ locations_list(pcmk__output_t *out, va_list args) {
     pcmk_resource_t *rsc = va_arg(args, pcmk_resource_t *);
 
     GList *lpc = NULL;
-    GList *list = rsc->rsc_location;
     int rc = pcmk_rc_no_output;
 
-    for (lpc = list; lpc != NULL; lpc = lpc->next) {
+    for (lpc = rsc->priv->location_constraints;
+         lpc != NULL; lpc = lpc->next) {
         pcmk__location_t *cons = lpc->data;
 
         GList *lpc2 = NULL;
@@ -557,7 +572,7 @@ locations_list(pcmk__output_t *out, va_list args) {
             PCMK__OUTPUT_LIST_HEADER(out, false, rc, "Locations");
             out->list_item(out, NULL, "Node %s (score=%s, id=%s, rsc=%s)",
                            pcmk__node_name(node),
-                           pcmk_readable_score(node->weight), cons->id,
+                           pcmk_readable_score(node->assign->score), cons->id,
                            rsc->id);
         }
     }
@@ -582,7 +597,7 @@ locations_and_colocations(pcmk__output_t *out, va_list args)
     bool recursive = va_arg(args, int);
     bool force = va_arg(args, int);
 
-    pcmk__unpack_constraints(rsc->cluster);
+    pcmk__unpack_constraints(rsc->priv->scheduler);
 
     // Constraints apply to group/clone, not member/instance
     if (!force) {
@@ -591,10 +606,12 @@ locations_and_colocations(pcmk__output_t *out, va_list args)
 
     out->message(out, "locations-list", rsc);
 
-    pe__clear_resource_flags_on_all(rsc->cluster, pcmk_rsc_detect_loop);
+    pe__clear_resource_flags_on_all(rsc->priv->scheduler,
+                                    pcmk__rsc_detect_loop);
     out->message(out, "rscs-colocated-with-list", rsc, recursive);
 
-    pe__clear_resource_flags_on_all(rsc->cluster, pcmk_rsc_detect_loop);
+    pe__clear_resource_flags_on_all(rsc->priv->scheduler,
+                                    pcmk__rsc_detect_loop);
     out->message(out, "rsc-is-colocated-with-list", rsc, recursive);
     return pcmk_rc_ok;
 }
@@ -608,7 +625,7 @@ locations_and_colocations_xml(pcmk__output_t *out, va_list args)
     bool recursive = va_arg(args, int);
     bool force = va_arg(args, int);
 
-    pcmk__unpack_constraints(rsc->cluster);
+    pcmk__unpack_constraints(rsc->priv->scheduler);
 
     // Constraints apply to group/clone, not member/instance
     if (!force) {
@@ -618,10 +635,12 @@ locations_and_colocations_xml(pcmk__output_t *out, va_list args)
     pcmk__output_xml_create_parent(out, PCMK_XE_CONSTRAINTS, NULL);
     do_locations_list_xml(out, rsc, false);
 
-    pe__clear_resource_flags_on_all(rsc->cluster, pcmk_rsc_detect_loop);
+    pe__clear_resource_flags_on_all(rsc->priv->scheduler,
+                                    pcmk__rsc_detect_loop);
     out->message(out, "rscs-colocated-with-list", rsc, recursive);
 
-    pe__clear_resource_flags_on_all(rsc->cluster, pcmk_rsc_detect_loop);
+    pe__clear_resource_flags_on_all(rsc->priv->scheduler,
+                                    pcmk__rsc_detect_loop);
     out->message(out, "rsc-is-colocated-with-list", rsc, recursive);
 
     pcmk__output_xml_pop_parent(out);
@@ -700,7 +719,7 @@ pacemakerd_health(pcmk__output_t *out, va_list args)
 
     if (sys_from == NULL) {
         if (state == pcmk_pacemakerd_state_remote) {
-            sys_from = "pacemaker-remoted";
+            sys_from = PCMK__SERVER_REMOTED;
         } else {
             sys_from = CRM_SYSTEM_MCP;
         }
@@ -741,7 +760,7 @@ pacemakerd_health_html(pcmk__output_t *out, va_list args)
 
     if (sys_from == NULL) {
         if (state == pcmk_pacemakerd_state_remote) {
-            sys_from = "pacemaker-remoted";
+            sys_from = PCMK__SERVER_REMOTED;
         } else {
             sys_from = CRM_SYSTEM_MCP;
         }
@@ -805,7 +824,7 @@ pacemakerd_health_xml(pcmk__output_t *out, va_list args)
 
     if (sys_from == NULL) {
         if (state == pcmk_pacemakerd_state_remote) {
-            sys_from = "pacemaker-remoted";
+            sys_from = PCMK__SERVER_REMOTED;
         } else {
             sys_from = CRM_SYSTEM_MCP;
         }
@@ -984,8 +1003,8 @@ digests_text(pcmk__output_t *out, va_list args)
     if ((rsc != NULL) && (rsc->id != NULL)) {
         rsc_desc = rsc->id;
     }
-    if ((node != NULL) && (node->details->uname != NULL)) {
-        node_desc = node->details->uname;
+    if ((node != NULL) && (node->priv->name != NULL)) {
+        node_desc = node->priv->name;
     }
     out->begin_list(out, NULL, NULL, "Digests for %s %s on %s",
                     rsc_desc, action_desc, node_desc);
@@ -1042,7 +1061,7 @@ digests_xml(pcmk__output_t *out, va_list args)
     xml = pcmk__output_create_xml_node(out, PCMK_XE_DIGESTS,
                                        PCMK_XA_RESOURCE, pcmk__s(rsc->id, ""),
                                        PCMK_XA_NODE,
-                                       pcmk__s(node->details->uname, ""),
+                                       pcmk__s(node->priv->name, ""),
                                        PCMK_XA_TASK, pcmk__s(task, ""),
                                        PCMK_XA_INTERVAL, interval_s,
                                        NULL);
@@ -1058,18 +1077,18 @@ digests_xml(pcmk__output_t *out, va_list args)
     return pcmk_rc_ok;
 }
 
-#define STOP_SANITY_ASSERT(lineno) do {                                 \
-        if ((current != NULL) && current->details->unclean) {           \
-            /* It will be a pseudo op */                                \
-        } else if (stop == NULL) {                                      \
-            crm_err("%s:%d: No stop action exists for %s",              \
-                    __func__, lineno, rsc->id);                         \
-            CRM_ASSERT(stop != NULL);                                   \
-        } else if (pcmk_is_set(stop->flags, pcmk_action_optional)) {    \
-            crm_err("%s:%d: Action %s is still optional",               \
-                    __func__, lineno, stop->uuid);                      \
-            CRM_ASSERT(!pcmk_is_set(stop->flags, pcmk_action_optional));\
-        }                                                               \
+#define STOP_SANITY_ASSERT(lineno) do {                                     \
+        if ((current != NULL) && current->details->unclean) {               \
+            /* It will be a pseudo op */                                    \
+        } else if (stop == NULL) {                                          \
+            crm_err("%s:%d: No stop action exists for %s",                  \
+                    __func__, lineno, rsc->id);                             \
+            pcmk__assert(stop != NULL);                                     \
+        } else if (pcmk_is_set(stop->flags, pcmk__action_optional)) {       \
+            crm_err("%s:%d: Action %s is still optional",                   \
+                    __func__, lineno, stop->uuid);                          \
+            pcmk__assert(!pcmk_is_set(stop->flags, pcmk__action_optional)); \
+        }                                                                   \
     } while (0)
 
 PCMK__OUTPUT_ARGS("rsc-action", "pcmk_resource_t *", "pcmk_node_t *",
@@ -1093,12 +1112,12 @@ rsc_action_default(pcmk__output_t *out, va_list args)
     pcmk_action_t *demote = NULL;
     pcmk_action_t *reason_op = NULL;
 
-    if (!pcmk_is_set(rsc->flags, pcmk_rsc_managed)
+    if (!pcmk_is_set(rsc->flags, pcmk__rsc_managed)
         || (current == NULL && next == NULL)) {
-        const bool managed = pcmk_is_set(rsc->flags, pcmk_rsc_managed);
+        const bool managed = pcmk_is_set(rsc->flags, pcmk__rsc_managed);
 
         pcmk__rsc_info(rsc, "Leave   %s\t(%s%s)",
-                       rsc->id, pcmk_role_text(rsc->role),
+                       rsc->id, pcmk_role_text(rsc->priv->orig_role),
                        (managed? "" : " unmanaged"));
         return rc;
     }
@@ -1114,7 +1133,7 @@ rsc_action_default(pcmk__output_t *out, va_list args)
     }
 
     if ((start == NULL)
-        || !pcmk_is_set(start->flags, pcmk_action_runnable)) {
+        || !pcmk_is_set(start->flags, pcmk__action_runnable)) {
         start_node = NULL;
     } else {
         start_node = current;
@@ -1124,7 +1143,7 @@ rsc_action_default(pcmk__output_t *out, va_list args)
     if (possible_matches) {
         stop = possible_matches->data;
         g_list_free(possible_matches);
-    } else if (pcmk_is_set(rsc->flags, pcmk_rsc_stop_unexpected)) {
+    } else if (pcmk_is_set(rsc->flags, pcmk__rsc_stop_unexpected)) {
         /* The resource is multiply active with PCMK_META_MULTIPLE_ACTIVE set to
          * PCMK_VALUE_STOP_UNEXPECTED, and not stopping on its current node, but
          * it should be stopping elsewhere.
@@ -1151,7 +1170,7 @@ rsc_action_default(pcmk__output_t *out, va_list args)
         g_list_free(possible_matches);
     }
 
-    if (rsc->role == rsc->next_role) {
+    if (rsc->priv->orig_role == rsc->priv->next_role) {
         pcmk_action_t *migrate_op = NULL;
 
         CRM_CHECK(next != NULL, return rc);
@@ -1164,28 +1183,28 @@ rsc_action_default(pcmk__output_t *out, va_list args)
         }
 
         if ((migrate_op != NULL) && (current != NULL)
-            && pcmk_is_set(migrate_op->flags, pcmk_action_runnable)) {
+            && pcmk_is_set(migrate_op->flags, pcmk__action_runnable)) {
             rc = out->message(out, "rsc-action-item", "Migrate", rsc, current,
                               next, start, NULL);
 
-        } else if (pcmk_is_set(rsc->flags, pcmk_rsc_reload)) {
+        } else if (pcmk_is_set(rsc->flags, pcmk__rsc_reload)) {
             rc = out->message(out, "rsc-action-item", "Reload", rsc, current,
                               next, start, NULL);
 
         } else if ((start == NULL)
-                   || pcmk_is_set(start->flags, pcmk_action_optional)) {
+                   || pcmk_is_set(start->flags, pcmk__action_optional)) {
             if ((demote != NULL) && (promote != NULL)
-                && !pcmk_is_set(demote->flags, pcmk_action_optional)
-                && !pcmk_is_set(promote->flags, pcmk_action_optional)) {
+                && !pcmk_is_set(demote->flags, pcmk__action_optional)
+                && !pcmk_is_set(promote->flags, pcmk__action_optional)) {
                 rc = out->message(out, "rsc-action-item", "Re-promote", rsc,
                                   current, next, promote, demote);
             } else {
                 pcmk__rsc_info(rsc, "Leave   %s\t(%s %s)", rsc->id,
-                               pcmk_role_text(rsc->role),
+                               pcmk_role_text(rsc->priv->orig_role),
                                pcmk__node_name(next));
             }
 
-        } else if (!pcmk_is_set(start->flags, pcmk_action_runnable)) {
+        } else if (!pcmk_is_set(start->flags, pcmk__action_runnable)) {
             if ((stop == NULL) || (stop->reason == NULL)) {
                 reason_op = start;
             } else {
@@ -1196,13 +1215,13 @@ rsc_action_default(pcmk__output_t *out, va_list args)
             STOP_SANITY_ASSERT(__LINE__);
 
         } else if (moving && current) {
-            const bool failed = pcmk_is_set(rsc->flags, pcmk_rsc_failed);
+            const bool failed = pcmk_is_set(rsc->flags, pcmk__rsc_failed);
 
             rc = out->message(out, "rsc-action-item",
                               (failed? "Recover" : "Move"), rsc, current, next,
                               stop, NULL);
 
-        } else if (pcmk_is_set(rsc->flags, pcmk_rsc_failed)) {
+        } else if (pcmk_is_set(rsc->flags, pcmk__rsc_failed)) {
             rc = out->message(out, "rsc-action-item", "Recover", rsc, current,
                               NULL, stop, NULL);
             STOP_SANITY_ASSERT(__LINE__);
@@ -1223,24 +1242,26 @@ rsc_action_default(pcmk__output_t *out, va_list args)
     }
 
     if ((stop != NULL)
-        && ((rsc->next_role == pcmk_role_stopped)
+        && ((rsc->priv->next_role == pcmk_role_stopped)
             || ((start != NULL)
-                && !pcmk_is_set(start->flags, pcmk_action_runnable)))) {
+                && !pcmk_is_set(start->flags, pcmk__action_runnable)))) {
 
         key = stop_key(rsc);
-        for (GList *iter = rsc->running_on; iter != NULL; iter = iter->next) {
+        for (GList *iter = rsc->priv->active_nodes;
+             iter != NULL; iter = iter->next) {
+
             pcmk_node_t *node = iter->data;
             pcmk_action_t *stop_op = NULL;
 
             reason_op = start;
-            possible_matches = find_actions(rsc->actions, key, node);
+            possible_matches = find_actions(rsc->priv->actions, key, node);
             if (possible_matches) {
                 stop_op = possible_matches->data;
                 g_list_free(possible_matches);
             }
 
             if (stop_op != NULL) {
-                if (pcmk_is_set(stop_op->flags, pcmk_action_runnable)) {
+                if (pcmk_is_set(stop_op->flags, pcmk__action_runnable)) {
                     STOP_SANITY_ASSERT(__LINE__);
                 }
                 if (stop_op->reason != NULL) {
@@ -1258,7 +1279,8 @@ rsc_action_default(pcmk__output_t *out, va_list args)
 
     } else if ((stop != NULL)
                && pcmk_all_flags_set(rsc->flags,
-                                     pcmk_rsc_failed|pcmk_rsc_stop_if_failed)) {
+                                     pcmk__rsc_failed
+                                     |pcmk__rsc_stop_if_failed)) {
         /* 'stop' may be NULL if the failure was ignored */
         rc = out->message(out, "rsc-action-item", "Recover", rsc, current,
                           next, stop, start);
@@ -1269,28 +1291,28 @@ rsc_action_default(pcmk__output_t *out, va_list args)
                           stop, NULL);
         STOP_SANITY_ASSERT(__LINE__);
 
-    } else if (pcmk_is_set(rsc->flags, pcmk_rsc_reload)) {
+    } else if (pcmk_is_set(rsc->flags, pcmk__rsc_reload)) {
         rc = out->message(out, "rsc-action-item", "Reload", rsc, current, next,
                           start, NULL);
 
     } else if ((stop != NULL)
-               && !pcmk_is_set(stop->flags, pcmk_action_optional)) {
+               && !pcmk_is_set(stop->flags, pcmk__action_optional)) {
         rc = out->message(out, "rsc-action-item", "Restart", rsc, current,
                           next, start, NULL);
         STOP_SANITY_ASSERT(__LINE__);
 
-    } else if (rsc->role == pcmk_role_promoted) {
+    } else if (rsc->priv->orig_role == pcmk_role_promoted) {
         CRM_LOG_ASSERT(current != NULL);
         rc = out->message(out, "rsc-action-item", "Demote", rsc, current,
                           next, demote, NULL);
 
-    } else if (rsc->next_role == pcmk_role_promoted) {
+    } else if (rsc->priv->next_role == pcmk_role_promoted) {
         CRM_LOG_ASSERT(next);
         rc = out->message(out, "rsc-action-item", "Promote", rsc, current,
                           next, promote, NULL);
 
-    } else if ((rsc->role == pcmk_role_stopped)
-               && (rsc->next_role > pcmk_role_stopped)) {
+    } else if ((rsc->priv->orig_role == pcmk_role_stopped)
+               && (rsc->priv->next_role > pcmk_role_stopped)) {
         rc = out->message(out, "rsc-action-item", "Start", rsc, current, next,
                           start, NULL);
     }
@@ -1836,8 +1858,8 @@ pcmk__cluster_status_text(pcmk__output_t *out, va_list args)
 
     /* If there were any failed actions, print them */
     if (pcmk_is_set(section_opts, pcmk_section_failures)
-        && (scheduler->failed != NULL)
-        && (scheduler->failed->children != NULL)) {
+        && (scheduler->priv->failed != NULL)
+        && (scheduler->priv->failed->children != NULL)) {
 
         CHECK_RC(rc, out->message(out, "failed-action-list", scheduler, unames,
                                   resources, show_opts, rc == pcmk_rc_ok));
@@ -1870,7 +1892,8 @@ pcmk__cluster_status_text(pcmk__output_t *out, va_list args)
 
     /* Print tickets if requested */
     if (pcmk_is_set(section_opts, pcmk_section_tickets)) {
-        CHECK_RC(rc, out->message(out, "ticket-list", scheduler->tickets,
+        CHECK_RC(rc, out->message(out, "ticket-list",
+                                  scheduler->priv->ticket_constraints,
                                   (rc == pcmk_rc_ok), false, false));
     }
 
@@ -1973,8 +1996,8 @@ cluster_status_xml(pcmk__output_t *out, va_list args)
 
     /* If there were any failed actions, print them */
     if (pcmk_is_set(section_opts, pcmk_section_failures)
-        && (scheduler->failed != NULL)
-        && (scheduler->failed->children != NULL)) {
+        && (scheduler->priv->failed != NULL)
+        && (scheduler->priv->failed->children != NULL)) {
 
         out->message(out, "failed-action-list", scheduler, unames, resources,
                      show_opts, false);
@@ -1989,7 +2012,8 @@ cluster_status_xml(pcmk__output_t *out, va_list args)
 
     /* Print tickets if requested */
     if (pcmk_is_set(section_opts, pcmk_section_tickets)) {
-        out->message(out, "ticket-list", scheduler->tickets, false, false, false);
+        out->message(out, "ticket-list", scheduler->priv->ticket_constraints,
+                     false, false, false);
     }
 
     /* Print negative location constraints if requested */
@@ -2053,8 +2077,8 @@ cluster_status_html(pcmk__output_t *out, va_list args)
 
     /* If there were any failed actions, print them */
     if (pcmk_is_set(section_opts, pcmk_section_failures)
-        && (scheduler->failed != NULL)
-        && (scheduler->failed->children != NULL)) {
+        && (scheduler->priv->failed != NULL)
+        && (scheduler->priv->failed->children != NULL)) {
 
         out->message(out, "failed-action-list", scheduler, unames, resources,
                      show_opts, false);
@@ -2116,7 +2140,8 @@ cluster_status_html(pcmk__output_t *out, va_list args)
 
     /* Print tickets if requested */
     if (pcmk_is_set(section_opts, pcmk_section_tickets)) {
-        out->message(out, "ticket-list", scheduler->tickets, false, false, false);
+        out->message(out, "ticket-list", scheduler->priv->ticket_constraints,
+                     false, false, false);
     }
 
     /* Print negative location constraints if requested */
@@ -2454,7 +2479,7 @@ ticket_constraints_default(pcmk__output_t *out, va_list args)
             out->output_xml(out, PCMK_XE_CONSTRAINT, buf->str);
             g_string_free(buf, TRUE);
 
-            child = pcmk__xe_next(child);
+            child = pcmk__xe_next(child, NULL);
         } while (child != NULL);
     } else {
         GString *buf = g_string_sized_new(1024);
