@@ -41,15 +41,16 @@ best_op(const pcmk_resource_t *rsc, const pcmk_node_t *node)
     const char *best_digest = NULL;
 
     // Find node's resource history
-    xpath = crm_strdup_printf(XPATH_OP_HISTORY, node->details->uname, rsc->id);
-    history = get_xpath_object(xpath, rsc->cluster->input, LOG_NEVER);
+    xpath = crm_strdup_printf(XPATH_OP_HISTORY, node->priv->name, rsc->id);
+    history = get_xpath_object(xpath, rsc->priv->scheduler->input, LOG_NEVER);
     free(xpath);
 
     // Examine each history entry
     for (xmlNode *lrm_rsc_op = pcmk__xe_first_child(history,
                                                     PCMK__XE_LRM_RSC_OP, NULL,
                                                     NULL);
-         lrm_rsc_op != NULL; lrm_rsc_op = pcmk__xe_next_same(lrm_rsc_op)) {
+         lrm_rsc_op != NULL;
+         lrm_rsc_op = pcmk__xe_next(lrm_rsc_op, PCMK__XE_LRM_RSC_OP)) {
 
         const char *digest = crm_element_value(lrm_rsc_op,
                                                PCMK__XA_OP_RESTART_DIGEST);
@@ -94,7 +95,7 @@ best_op(const pcmk_resource_t *rsc, const pcmk_node_t *node)
         }
 
         // Do not use an older op if there's a newer one.
-        if (pe__is_newer_op(best, lrm_rsc_op, true) > 0) {
+        if (pe__is_newer_op(best, lrm_rsc_op) > 0) {
             continue;
         }
 
@@ -140,7 +141,7 @@ pcmk__resource_delete(cib_t *cib, uint32_t cib_opts, const char *rsc_id,
     rc = cib->cmds->remove(cib, PCMK_XE_RESOURCES, msg_data, cib_opts);
     rc = pcmk_legacy2rc(rc);
 
-    free_xml(msg_data);
+    pcmk__xml_free(msg_data);
     return rc;
 }
 
@@ -205,6 +206,7 @@ pcmk__resource_digests(pcmk__output_t *out, pcmk_resource_t *rsc,
     if ((out == NULL) || (rsc == NULL) || (node == NULL)) {
         return EINVAL;
     }
+
     if (!pcmk__is_primitive(rsc)) {
         // Only primitives get operation digests
         return EOPNOTSUPP;
@@ -225,7 +227,7 @@ pcmk__resource_digests(pcmk__output_t *out, pcmk_resource_t *rsc,
 
     // Calculate and show digests
     digests = pe__calculate_digests(rsc, task, &interval_ms, node, xml_op,
-                                    overrides, true, rsc->cluster);
+                                    overrides, true, rsc->priv->scheduler);
     rc = out->message(out, "digests", rsc, node, task, interval_ms, digests);
 
     pe__free_digests(digests);
