@@ -51,8 +51,7 @@ eval_rule(pcmk_scheduler_t *scheduler, const char *rule_id, const char **error)
      * to check. For the moment, these rules must only have a single
      * date_expression child and:
      * - Do not have a date_spec operation, or
-     * - Have a date_spec operation that contains years= but does not contain
-     *   moon=.
+     * - Have a date_spec operation that contains years=
      *
      * We do this in steps to provide better error messages. First, check that
      * there's any rule with the given ID.
@@ -111,9 +110,7 @@ eval_rule(pcmk_scheduler_t *scheduler, const char *rule_id, const char **error)
                                   "[@" PCMK_XA_OPERATION
                                       "='" PCMK_VALUE_DATE_SPEC "' "
                                   "and " PCMK_XE_DATE_SPEC
-                                      "/@" PCMK_XA_YEARS " "
-                                  "and not(" PCMK_XE_DATE_SPEC
-                                      "/@" PCMK__XA_MOON ")]",
+                                      "/@" PCMK_XA_YEARS "]",
                                   rule_id);
         xpath_obj = xpath_search(cib_constraints, xpath);
         num_results = numXpathResults(xpath_obj);
@@ -123,8 +120,7 @@ eval_rule(pcmk_scheduler_t *scheduler, const char *rule_id, const char **error)
         if (num_results == 0) {
             freeXpathObject(xpath_obj);
             *error = "Rule must either not use " PCMK_XE_DATE_SPEC ", or use "
-                     PCMK_XE_DATE_SPEC " with " PCMK_XA_YEARS "= but not "
-                     PCMK__XA_MOON "=";
+                     PCMK_XE_DATE_SPEC " with " PCMK_XA_YEARS "=";
             return EOPNOTSUPP;
         }
     }
@@ -134,11 +130,12 @@ eval_rule(pcmk_scheduler_t *scheduler, const char *rule_id, const char **error)
     /* We should have ensured this with the xpath query above, but double-
      * checking can't hurt.
      */
-    CRM_ASSERT(match != NULL);
-    CRM_ASSERT(pcmk__condition_type(match) == pcmk__condition_datetime);
+    pcmk__assert((match != NULL)
+                 && (pcmk__condition_type(match) == pcmk__condition_datetime));
 
-    rc = pcmk__evaluate_date_expression(match, scheduler->now, NULL);
-    if (rc == pcmk_rc_undetermined) { // Malformed or missing
+    rc = pcmk__evaluate_date_expression(match, scheduler->priv->now, NULL);
+    if ((rc != pcmk_rc_ok) && (rc != pcmk_rc_within_range)) {
+        // Malformed or missing
         *error = "Error parsing rule";
     }
 
@@ -166,7 +163,7 @@ pcmk__check_rules(pcmk__output_t *out, xmlNodePtr input, const crm_time_t *date,
     pcmk_scheduler_t *scheduler = NULL;
     int rc = pcmk_rc_ok;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     if (rule_ids == NULL) {
         // Trivial case; every rule specified is in effect
