@@ -45,10 +45,10 @@ assign_replica(pcmk__bundle_replica_t *replica, void *user_data)
     if (replica->ip != NULL) {
         pcmk__rsc_trace(bundle, "Assigning bundle %s IP %s",
                         bundle->id, replica->ip->id);
-        replica->ip->cmds->assign(replica->ip, prefer, stop_if_fail);
+        replica->ip->priv->cmds->assign(replica->ip, prefer, stop_if_fail);
     }
 
-    container_host = replica->container->allocated_to;
+    container_host = replica->container->priv->assigned_node;
     if (replica->remote != NULL) {
         if (pcmk__is_pacemaker_remote_node(container_host)) {
             /* REMOTE_CONTAINER_HACK: "Nested" connection resources must be on
@@ -57,33 +57,36 @@ assign_replica(pcmk__bundle_replica_t *replica, void *user_data)
              */
             pcmk__new_colocation("#replica-remote-with-host-remote", NULL,
                                  PCMK_SCORE_INFINITY, replica->remote,
-                                 container_host->details->remote_rsc, NULL,
+                                 container_host->priv->remote, NULL,
                                  NULL, pcmk__coloc_influence);
         }
         pcmk__rsc_trace(bundle, "Assigning bundle %s connection %s",
                         bundle->id, replica->remote->id);
-        replica->remote->cmds->assign(replica->remote, prefer, stop_if_fail);
+        replica->remote->priv->cmds->assign(replica->remote, prefer,
+                                            stop_if_fail);
     }
 
     if (replica->child != NULL) {
         pcmk_node_t *node = NULL;
         GHashTableIter iter;
 
-        g_hash_table_iter_init(&iter, replica->child->allowed_nodes);
+        g_hash_table_iter_init(&iter, replica->child->priv->allowed_nodes);
         while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &node)) {
             if (!pcmk__same_node(node, replica->node)) {
-                node->weight = -PCMK_SCORE_INFINITY;
+                node->assign->score = -PCMK_SCORE_INFINITY;
             } else if (!pcmk__threshold_reached(replica->child, node, NULL)) {
-                node->weight = PCMK_SCORE_INFINITY;
+                node->assign->score = PCMK_SCORE_INFINITY;
             }
         }
 
-        pcmk__set_rsc_flags(replica->child->parent, pcmk_rsc_assigning);
+        pcmk__set_rsc_flags(replica->child->priv->parent,
+                            pcmk__rsc_assigning);
         pcmk__rsc_trace(bundle, "Assigning bundle %s replica child %s",
                         bundle->id, replica->child->id);
-        replica->child->cmds->assign(replica->child, replica->node,
-                                     stop_if_fail);
-        pcmk__clear_rsc_flags(replica->child->parent, pcmk_rsc_assigning);
+        replica->child->priv->cmds->assign(replica->child, replica->node,
+                                           stop_if_fail);
+        pcmk__clear_rsc_flags(replica->child->priv->parent,
+                              pcmk__rsc_assigning);
     }
     return true;
 }
@@ -115,19 +118,20 @@ pcmk__bundle_assign(pcmk_resource_t *rsc, const pcmk_node_t *prefer,
     pcmk_resource_t *bundled_resource = NULL;
     struct assign_data assign_data = { prefer, stop_if_fail };
 
-    CRM_ASSERT(pcmk__is_bundle(rsc));
+    pcmk__assert(pcmk__is_bundle(rsc));
 
     pcmk__rsc_trace(rsc, "Assigning bundle %s", rsc->id);
-    pcmk__set_rsc_flags(rsc, pcmk_rsc_assigning);
+    pcmk__set_rsc_flags(rsc, pcmk__rsc_assigning);
 
-    pe__show_node_scores(!pcmk_is_set(rsc->cluster->flags,
-                                      pcmk_sched_output_scores),
-                         rsc, __func__, rsc->allowed_nodes, rsc->cluster);
+    pe__show_node_scores(!pcmk_is_set(rsc->priv->scheduler->flags,
+                                      pcmk__sched_output_scores),
+                         rsc, __func__, rsc->priv->allowed_nodes,
+                         rsc->priv->scheduler);
 
     // Assign all containers first, so we know what nodes the bundle will be on
     containers = g_list_sort(pe__bundle_containers(rsc), pcmk__cmp_instance);
     pcmk__assign_instances(rsc, containers, pe__bundle_max(rsc),
-                           rsc->fns->max_per_node(rsc));
+                           rsc->priv->fns->max_per_node(rsc));
     g_list_free(containers);
 
     // Then assign remaining replica resources
@@ -139,18 +143,19 @@ pcmk__bundle_assign(pcmk_resource_t *rsc, const pcmk_node_t *prefer,
         pcmk_node_t *node = NULL;
         GHashTableIter iter;
 
-        g_hash_table_iter_init(&iter, bundled_resource->allowed_nodes);
+        g_hash_table_iter_init(&iter, bundled_resource->priv->allowed_nodes);
         while (g_hash_table_iter_next(&iter, NULL, (gpointer *) & node)) {
             if (pe__node_is_bundle_instance(rsc, node)) {
-                node->weight = 0;
+                node->assign->score = 0;
             } else {
-                node->weight = -PCMK_SCORE_INFINITY;
+                node->assign->score = -PCMK_SCORE_INFINITY;
             }
         }
-        bundled_resource->cmds->assign(bundled_resource, prefer, stop_if_fail);
+        bundled_resource->priv->cmds->assign(bundled_resource, prefer,
+                                                stop_if_fail);
     }
 
-    pcmk__clear_rsc_flags(rsc, pcmk_rsc_assigning|pcmk_rsc_unassigned);
+    pcmk__clear_rsc_flags(rsc, pcmk__rsc_assigning|pcmk__rsc_unassigned);
     return NULL;
 }
 
@@ -167,13 +172,13 @@ static bool
 create_replica_actions(pcmk__bundle_replica_t *replica, void *user_data)
 {
     if (replica->ip != NULL) {
-        replica->ip->cmds->create_actions(replica->ip);
+        replica->ip->priv->cmds->create_actions(replica->ip);
     }
     if (replica->container != NULL) {
-        replica->container->cmds->create_actions(replica->container);
+        replica->container->priv->cmds->create_actions(replica->container);
     }
     if (replica->remote != NULL) {
-        replica->remote->cmds->create_actions(replica->remote);
+        replica->remote->priv->cmds->create_actions(replica->remote);
     }
     return true;
 }
@@ -191,7 +196,7 @@ pcmk__bundle_create_actions(pcmk_resource_t *rsc)
     GList *containers = NULL;
     pcmk_resource_t *bundled_resource = NULL;
 
-    CRM_ASSERT(pcmk__is_bundle(rsc));
+    pcmk__assert(pcmk__is_bundle(rsc));
 
     pe__foreach_bundle_replica(rsc, create_replica_actions, NULL);
 
@@ -201,9 +206,9 @@ pcmk__bundle_create_actions(pcmk_resource_t *rsc)
 
     bundled_resource = pe__bundled_resource(rsc);
     if (bundled_resource != NULL) {
-        bundled_resource->cmds->create_actions(bundled_resource);
+        bundled_resource->priv->cmds->create_actions(bundled_resource);
 
-        if (pcmk_is_set(bundled_resource->flags, pcmk_rsc_promotable)) {
+        if (pcmk_is_set(bundled_resource->flags, pcmk__rsc_promotable)) {
             pe__new_rsc_pseudo_action(rsc, PCMK_ACTION_PROMOTE, true, true);
             action = pe__new_rsc_pseudo_action(rsc, PCMK_ACTION_PROMOTED,
                                                true, true);
@@ -231,7 +236,7 @@ replica_internal_constraints(pcmk__bundle_replica_t *replica, void *user_data)
 {
     pcmk_resource_t *bundle = user_data;
 
-    replica->container->cmds->internal_constraints(replica->container);
+    replica->container->priv->cmds->internal_constraints(replica->container);
 
     // Start bundle -> start replica container
     pcmk__order_starts(bundle, replica->container,
@@ -257,7 +262,7 @@ replica_internal_constraints(pcmk__bundle_replica_t *replica, void *user_data)
                                  pcmk__ar_first_implies_then_graphed);
 
     if (replica->ip != NULL) {
-        replica->ip->cmds->internal_constraints(replica->ip);
+        replica->ip->priv->cmds->internal_constraints(replica->ip);
 
         // Replica IP address -> replica container (symmetric)
         pcmk__order_starts(replica->ip, replica->container,
@@ -277,11 +282,11 @@ replica_internal_constraints(pcmk__bundle_replica_t *replica, void *user_data)
          * colocated relative to the container, we don't need to do anything
          * explicit here with IP.
          */
-        replica->remote->cmds->internal_constraints(replica->remote);
+        replica->remote->priv->cmds->internal_constraints(replica->remote);
     }
 
     if (replica->child != NULL) {
-        CRM_ASSERT(replica->remote != NULL);
+        pcmk__assert(replica->remote != NULL);
         // "Start remote then child" is implicit in scheduler's remote logic
     }
     return true;
@@ -298,7 +303,7 @@ pcmk__bundle_internal_constraints(pcmk_resource_t *rsc)
 {
     pcmk_resource_t *bundled_resource = NULL;
 
-    CRM_ASSERT(pcmk__is_bundle(rsc));
+    pcmk__assert(pcmk__is_bundle(rsc));
 
     pe__foreach_bundle_replica(rsc, replica_internal_constraints, rsc);
 
@@ -327,9 +332,9 @@ pcmk__bundle_internal_constraints(pcmk_resource_t *rsc)
                                  rsc, PCMK_ACTION_STOPPED,
                                  pcmk__ar_first_implies_then_graphed);
 
-    bundled_resource->cmds->internal_constraints(bundled_resource);
+    bundled_resource->priv->cmds->internal_constraints(bundled_resource);
 
-    if (!pcmk_is_set(bundled_resource->flags, pcmk_rsc_promotable)) {
+    if (!pcmk_is_set(bundled_resource->flags, pcmk__rsc_promotable)) {
         return;
     }
     pcmk__promotable_restart_ordering(rsc);
@@ -396,9 +401,11 @@ static const pcmk_node_t *
 get_bundle_node_host(const pcmk_node_t *node)
 {
     if (pcmk__is_bundle_node(node)) {
-        const pcmk_resource_t *container = node->details->remote_rsc->container;
+        const pcmk_resource_t *container = NULL;
 
-        return container->fns->location(container, NULL, 0);
+        container = node->priv->remote->priv->launcher;
+        return container->priv->fns->location(container, NULL,
+                                              pcmk__rsc_node_assigned);
     }
     return node;
 }
@@ -422,7 +429,8 @@ compatible_container(const pcmk_resource_t *dependent,
     struct match_data match_data = { NULL, NULL };
 
     // If dependent is assigned, only check there
-    match_data.node = dependent->fns->location(dependent, NULL, 0);
+    match_data.node = dependent->priv->fns->location(dependent, NULL,
+                                                     pcmk__rsc_node_assigned);
     match_data.node = get_bundle_node_host(match_data.node);
     if (match_data.node != NULL) {
         pe__foreach_const_bundle_replica(bundle, match_replica_container,
@@ -431,7 +439,7 @@ compatible_container(const pcmk_resource_t *dependent,
     }
 
     // Otherwise, check for any of the dependent's allowed nodes
-    scratch = g_hash_table_get_values(dependent->allowed_nodes);
+    scratch = g_hash_table_get_values(dependent->priv->allowed_nodes);
     scratch = pcmk__sort_nodes(scratch, NULL);
     for (const GList *iter = scratch; iter != NULL; iter = iter->next) {
         match_data.node = iter->data;
@@ -472,35 +480,37 @@ replica_apply_coloc_score(const pcmk__bundle_replica_t *replica,
 {
     struct coloc_data *coloc_data = user_data;
     pcmk_node_t *chosen = NULL;
+    pcmk_resource_t *container = replica->container;
 
     if (coloc_data->colocation->score < PCMK_SCORE_INFINITY) {
         int priority_delta =
-            replica->container->cmds->apply_coloc_score(coloc_data->dependent,
-                                                        replica->container,
-                                                        coloc_data->colocation,
-                                                        false);
+            container->priv->cmds->apply_coloc_score(coloc_data->dependent,
+                                                     container,
+                                                     coloc_data->colocation,
+                                                     false);
 
         coloc_data->priority_delta =
             pcmk__add_scores(coloc_data->priority_delta, priority_delta);
         return true;
     }
 
-    chosen = replica->container->fns->location(replica->container, NULL, 0);
+    chosen = container->priv->fns->location(container, NULL,
+                                            pcmk__rsc_node_assigned);
     if ((chosen == NULL)
-        || is_set_recursive(replica->container, pcmk_rsc_blocked, true)) {
+        || is_set_recursive(container, pcmk__rsc_blocked, true)) {
         return true;
     }
 
     if ((coloc_data->colocation->primary_role >= pcmk_role_promoted)
         && ((replica->child == NULL)
-            || (replica->child->next_role < pcmk_role_promoted))) {
+            || (replica->child->priv->next_role < pcmk_role_promoted))) {
         return true;
     }
 
-    pcmk__rsc_trace(pe__const_top_resource(replica->container, true),
+    pcmk__rsc_trace(pe__const_top_resource(container, true),
                     "Allowing mandatory colocation %s using %s @%d",
                     coloc_data->colocation->id, pcmk__node_name(chosen),
-                    chosen->weight);
+                    chosen->assign->score);
     coloc_data->container_hosts = g_list_prepend(coloc_data->container_hosts,
                                                  chosen);
     return true;
@@ -533,10 +543,10 @@ pcmk__bundle_apply_coloc_score(pcmk_resource_t *dependent,
      * Instead, we add its colocation constraints to its containers and bundled
      * primitive and call the apply_coloc_score() method for them as dependents.
      */
-    CRM_ASSERT(pcmk__is_bundle(primary) && pcmk__is_primitive(dependent)
-               && (colocation != NULL) && !for_dependent);
+    pcmk__assert(pcmk__is_bundle(primary) && pcmk__is_primitive(dependent)
+                 && (colocation != NULL) && !for_dependent);
 
-    if (pcmk_is_set(primary->flags, pcmk_rsc_unassigned)) {
+    if (pcmk_is_set(primary->flags, pcmk__rsc_unassigned)) {
         pcmk__rsc_trace(primary,
                         "Skipping applying colocation %s "
                         "because %s is still provisional",
@@ -550,7 +560,7 @@ pcmk__bundle_apply_coloc_score(pcmk_resource_t *dependent,
     /* If the constraint dependent is a clone or bundle, "dependent" here is one
      * of its instances. Look for a compatible instance of this bundle.
      */
-    if (colocation->dependent->variant > pcmk_rsc_variant_group) {
+    if (colocation->dependent->priv->variant > pcmk__rsc_variant_group) {
         const pcmk_resource_t *primary_container = NULL;
 
         primary_container = compatible_container(dependent, primary);
@@ -558,9 +568,9 @@ pcmk__bundle_apply_coloc_score(pcmk_resource_t *dependent,
             pcmk__rsc_debug(primary, "Pairing %s with %s",
                             dependent->id, primary_container->id);
 
-            return dependent->cmds->apply_coloc_score(dependent,
-                                                      primary_container,
-                                                      colocation, true);
+            return dependent->priv->cmds->apply_coloc_score(dependent,
+                                                            primary_container,
+                                                            colocation, true);
         }
 
         if (colocation->score >= PCMK_SCORE_INFINITY) {
@@ -589,20 +599,21 @@ pcmk__bundle_apply_coloc_score(pcmk_resource_t *dependent,
     return coloc_data.priority_delta;
 }
 
-// Bundle implementation of pcmk_assignment_methods_t:with_this_colocations()
+// Bundle implementation of pcmk__assignment_methods_t:with_this_colocations()
 void
 pcmk__with_bundle_colocations(const pcmk_resource_t *rsc,
                               const pcmk_resource_t *orig_rsc, GList **list)
 {
     const pcmk_resource_t *bundled_rsc = NULL;
 
-    CRM_ASSERT(pcmk__is_bundle(rsc) && (orig_rsc != NULL) && (list != NULL));
+    pcmk__assert(pcmk__is_bundle(rsc) && (orig_rsc != NULL) && (list != NULL));
 
     // The bundle itself and its containers always get its colocations
     if ((orig_rsc == rsc)
-        || pcmk_is_set(orig_rsc->flags, pcmk_rsc_replica_container)) {
+        || pcmk_is_set(orig_rsc->flags, pcmk__rsc_replica_container)) {
 
-        pcmk__add_with_this_list(list, rsc->rsc_cons_lhs, orig_rsc);
+        pcmk__add_with_this_list(list, rsc->priv->with_this_colocations,
+                                 orig_rsc);
         return;
     }
 
@@ -611,7 +622,7 @@ pcmk__with_bundle_colocations(const pcmk_resource_t *rsc,
      */
     bundled_rsc = pe__bundled_resource(rsc);
     if ((bundled_rsc == NULL)
-        || !pcmk_is_set(bundled_rsc->flags, pcmk_rsc_promotable)
+        || !pcmk_is_set(bundled_rsc->flags, pcmk__rsc_promotable)
         || (pe__const_top_resource(orig_rsc, false) != bundled_rsc)) {
         return;
     }
@@ -622,31 +633,34 @@ pcmk__with_bundle_colocations(const pcmk_resource_t *rsc,
             /* orig_rsc is the clone and we're setting roles (or have already
              * done so)
              */
-            pcmk__add_with_this_list(list, rsc->rsc_cons_lhs, orig_rsc);
+            pcmk__add_with_this_list(list, rsc->priv->with_this_colocations,
+                                     orig_rsc);
         }
 
-    } else if (!pcmk_is_set(orig_rsc->flags, pcmk_rsc_unassigned)) {
+    } else if (!pcmk_is_set(orig_rsc->flags, pcmk__rsc_unassigned)) {
         /* orig_rsc is an instance and is already assigned. If something
          * requests colocations for orig_rsc now, it's for setting roles.
          */
-        pcmk__add_with_this_list(list, rsc->rsc_cons_lhs, orig_rsc);
+        pcmk__add_with_this_list(list, rsc->priv->with_this_colocations,
+                                 orig_rsc);
     }
 }
 
-// Bundle implementation of pcmk_assignment_methods_t:this_with_colocations()
+// Bundle implementation of pcmk__assignment_methods_t:this_with_colocations()
 void
 pcmk__bundle_with_colocations(const pcmk_resource_t *rsc,
                               const pcmk_resource_t *orig_rsc, GList **list)
 {
     const pcmk_resource_t *bundled_rsc = NULL;
 
-    CRM_ASSERT(pcmk__is_bundle(rsc) && (orig_rsc != NULL) && (list != NULL));
+    pcmk__assert(pcmk__is_bundle(rsc) && (orig_rsc != NULL) && (list != NULL));
 
     // The bundle itself and its containers always get its colocations
     if ((orig_rsc == rsc)
-        || pcmk_is_set(orig_rsc->flags, pcmk_rsc_replica_container)) {
+        || pcmk_is_set(orig_rsc->flags, pcmk__rsc_replica_container)) {
 
-        pcmk__add_this_with_list(list, rsc->rsc_cons, orig_rsc);
+        pcmk__add_this_with_list(list, rsc->priv->this_with_colocations,
+                                 orig_rsc);
         return;
     }
 
@@ -655,7 +669,7 @@ pcmk__bundle_with_colocations(const pcmk_resource_t *rsc,
      */
     bundled_rsc = pe__bundled_resource(rsc);
     if ((bundled_rsc == NULL)
-        || !pcmk_is_set(bundled_rsc->flags, pcmk_rsc_promotable)
+        || !pcmk_is_set(bundled_rsc->flags, pcmk__rsc_promotable)
         || (pe__const_top_resource(orig_rsc, false) != bundled_rsc)) {
         return;
     }
@@ -666,14 +680,16 @@ pcmk__bundle_with_colocations(const pcmk_resource_t *rsc,
             /* orig_rsc is the clone and we're setting roles (or have already
              * done so)
              */
-            pcmk__add_this_with_list(list, rsc->rsc_cons, orig_rsc);
+            pcmk__add_this_with_list(list, rsc->priv->this_with_colocations,
+                                     orig_rsc);
         }
 
-    } else if (!pcmk_is_set(orig_rsc->flags, pcmk_rsc_unassigned)) {
+    } else if (!pcmk_is_set(orig_rsc->flags, pcmk__rsc_unassigned)) {
         /* orig_rsc is an instance and is already assigned. If something
          * requests colocations for orig_rsc now, it's for setting roles.
          */
-        pcmk__add_this_with_list(list, rsc->rsc_cons, orig_rsc);
+        pcmk__add_this_with_list(list, rsc->priv->this_with_colocations,
+                                 orig_rsc);
     }
 }
 
@@ -693,22 +709,22 @@ pcmk__bundle_action_flags(pcmk_action_t *action, const pcmk_node_t *node)
     uint32_t flags = 0;
     pcmk_resource_t *bundled_resource = NULL;
 
-    CRM_ASSERT((action != NULL) && pcmk__is_bundle(action->rsc));
+    pcmk__assert((action != NULL) && pcmk__is_bundle(action->rsc));
 
     bundled_resource = pe__bundled_resource(action->rsc);
     if (bundled_resource != NULL) {
+        GList *children = bundled_resource->priv->children;
+
         // Clone actions are done on the bundled clone resource, not container
         switch (get_complex_task(bundled_resource, action->task)) {
-            case pcmk_action_unspecified:
-            case pcmk_action_notify:
-            case pcmk_action_notified:
-            case pcmk_action_promote:
-            case pcmk_action_promoted:
-            case pcmk_action_demote:
-            case pcmk_action_demoted:
-                return pcmk__collective_action_flags(action,
-                                                     bundled_resource->children,
-                                                     node);
+            case pcmk__action_unspecified:
+            case pcmk__action_notify:
+            case pcmk__action_notified:
+            case pcmk__action_promote:
+            case pcmk__action_promoted:
+            case pcmk__action_demote:
+            case pcmk__action_demoted:
+                return pcmk__collective_action_flags(action, children, node);
             default:
                 break;
         }
@@ -734,11 +750,10 @@ apply_location_to_replica(pcmk__bundle_replica_t *replica, void *user_data)
 {
     pcmk__location_t *location = user_data;
 
-    if (replica->container != NULL) {
-        replica->container->cmds->apply_location(replica->container, location);
-    }
+    replica->container->priv->cmds->apply_location(replica->container,
+                                                   location);
     if (replica->ip != NULL) {
-        replica->ip->cmds->apply_location(replica->ip, location);
+        replica->ip->priv->cmds->apply_location(replica->ip, location);
     }
     return true;
 }
@@ -755,7 +770,7 @@ pcmk__bundle_apply_location(pcmk_resource_t *rsc, pcmk__location_t *location)
 {
     pcmk_resource_t *bundled_resource = NULL;
 
-    CRM_ASSERT((location != NULL) && pcmk__is_bundle(rsc));
+    pcmk__assert((location != NULL) && pcmk__is_bundle(rsc));
 
     pcmk__apply_location(rsc, location);
     pe__foreach_bundle_replica(rsc, apply_location_to_replica, location);
@@ -764,9 +779,12 @@ pcmk__bundle_apply_location(pcmk_resource_t *rsc, pcmk__location_t *location)
     if ((bundled_resource != NULL)
         && ((location->role_filter == pcmk_role_unpromoted)
             || (location->role_filter == pcmk_role_promoted))) {
-        bundled_resource->cmds->apply_location(bundled_resource, location);
-        bundled_resource->rsc_location = g_list_prepend(
-            bundled_resource->rsc_location, location);
+
+        bundled_resource->priv->cmds->apply_location(bundled_resource,
+                                                     location);
+        bundled_resource->priv->location_constraints =
+            g_list_prepend(bundled_resource->priv->location_constraints,
+                           location);
     }
 }
 
@@ -784,17 +802,17 @@ pcmk__bundle_apply_location(pcmk_resource_t *rsc, pcmk__location_t *location)
 static bool
 add_replica_actions_to_graph(pcmk__bundle_replica_t *replica, void *user_data)
 {
-    if ((replica->remote != NULL) && (replica->container != NULL)
+    if ((replica->remote != NULL)
         && pe__bundle_needs_remote_name(replica->remote)) {
 
         /* REMOTE_CONTAINER_HACK: Allow remote nodes to run containers that
-         * run pacemaker-remoted inside, without needing a separate IP for
+         * run the remote executor inside, without needing a separate IP for
          * the container. This is done by configuring the inner remote's
          * connection host as the magic string "#uname", then
          * replacing it with the underlying host when needed.
          */
-        xmlNode *nvpair = get_xpath_object(XPATH_REMOTE, replica->remote->xml,
-                                           LOG_ERR);
+        xmlNode *nvpair = get_xpath_object(XPATH_REMOTE,
+                                           replica->remote->priv->xml, LOG_ERR);
         const char *calculated_addr = NULL;
 
         // Replace the value in replica->remote->xml (if appropriate)
@@ -807,9 +825,10 @@ add_replica_actions_to_graph(pcmk__bundle_replica_t *replica, void *user_data)
              * will grab it from there to replace it in node-evaluated
              * parameters.
              */
-            GHashTable *params = pe_rsc_params(replica->remote,
-                                               NULL, replica->remote->cluster);
+            GHashTable *params = NULL;
 
+            params = pe_rsc_params(replica->remote, NULL,
+                                   replica->remote->priv->scheduler);
             pcmk__insert_dup(params, PCMK_REMOTE_RA_ADDR, calculated_addr);
         } else {
             pcmk_resource_t *bundle = user_data;
@@ -827,13 +846,11 @@ add_replica_actions_to_graph(pcmk__bundle_replica_t *replica, void *user_data)
         }
     }
     if (replica->ip != NULL) {
-        replica->ip->cmds->add_actions_to_graph(replica->ip);
-    }
-    if (replica->container != NULL) {
-        replica->container->cmds->add_actions_to_graph(replica->container);
+        replica->ip->priv->cmds->add_actions_to_graph(replica->ip);
     }
+    replica->container->priv->cmds->add_actions_to_graph(replica->container);
     if (replica->remote != NULL) {
-        replica->remote->cmds->add_actions_to_graph(replica->remote);
+        replica->remote->priv->cmds->add_actions_to_graph(replica->remote);
     }
     return true;
 }
@@ -849,11 +866,11 @@ pcmk__bundle_add_actions_to_graph(pcmk_resource_t *rsc)
 {
     pcmk_resource_t *bundled_resource = NULL;
 
-    CRM_ASSERT(pcmk__is_bundle(rsc));
+    pcmk__assert(pcmk__is_bundle(rsc));
 
     bundled_resource = pe__bundled_resource(rsc);
     if (bundled_resource != NULL) {
-        bundled_resource->cmds->add_actions_to_graph(bundled_resource);
+        bundled_resource->priv->cmds->add_actions_to_graph(bundled_resource);
     }
     pe__foreach_bundle_replica(rsc, add_replica_actions_to_graph, rsc);
 }
@@ -888,7 +905,7 @@ order_replica_start_after(pcmk__bundle_replica_t *replica, void *user_data)
                        pcmk__op_key(replica->container->id, PCMK_ACTION_START,
                                     0),
                        NULL, pcmk__ar_ordered|pcmk__ar_if_on_same_node,
-                       replica->container->cluster);
+                       replica->container->priv->scheduler);
     return true;
 }
 
@@ -905,20 +922,21 @@ static bool
 create_replica_probes(pcmk__bundle_replica_t *replica, void *user_data)
 {
     struct probe_data *probe_data = user_data;
+    pcmk_resource_t *bundle = probe_data->bundle;
 
     if ((replica->ip != NULL)
-        && replica->ip->cmds->create_probe(replica->ip, probe_data->node)) {
+        && replica->ip->priv->cmds->create_probe(replica->ip,
+                                                 probe_data->node)) {
         probe_data->any_created = true;
     }
     if ((replica->child != NULL)
         && pcmk__same_node(probe_data->node, replica->node)
-        && replica->child->cmds->create_probe(replica->child,
-                                              probe_data->node)) {
+        && replica->child->priv->cmds->create_probe(replica->child,
+                                                    probe_data->node)) {
         probe_data->any_created = true;
     }
-    if ((replica->container != NULL)
-        && replica->container->cmds->create_probe(replica->container,
-                                                  probe_data->node)) {
+    if (replica->container->priv->cmds->create_probe(replica->container,
+                                                     probe_data->node)) {
         probe_data->any_created = true;
 
         /* If we're limited to one replica per host (due to
@@ -933,28 +951,28 @@ create_replica_probes(pcmk__bundle_replica_t *replica, void *user_data)
          * mappings (which won't include an IP for uniqueness)
          * are already taken
          */
-        if (probe_data->bundle->fns->max_per_node(probe_data->bundle) == 1) {
-            pe__foreach_bundle_replica(probe_data->bundle,
-                                       order_replica_start_after, replica);
+        if (bundle->priv->fns->max_per_node(bundle) == 1) {
+            pe__foreach_bundle_replica(bundle, order_replica_start_after,
+                                       replica);
         }
     }
-    if ((replica->container != NULL) && (replica->remote != NULL)
-        && replica->remote->cmds->create_probe(replica->remote,
-                                               probe_data->node)) {
+    if ((replica->remote != NULL)
+        && replica->remote->priv->cmds->create_probe(replica->remote,
+                                                     probe_data->node)) {
         /* Do not probe the remote resource until we know where the container is
          * running. This is required for REMOTE_CONTAINER_HACK to correctly
          * probe remote resources.
          */
         char *probe_uuid = pcmk__op_key(replica->remote->id,
                                         PCMK_ACTION_MONITOR, 0);
-        pcmk_action_t *probe = find_first_action(replica->remote->actions,
-                                                 probe_uuid, NULL,
-                                                 probe_data->node);
+        pcmk_action_t *probe = NULL;
 
+        probe = find_first_action(replica->remote->priv->actions, probe_uuid,
+                                  NULL, probe_data->node);
         free(probe_uuid);
         if (probe != NULL) {
             probe_data->any_created = true;
-            pcmk__rsc_trace(probe_data->bundle, "Ordering %s probe on %s",
+            pcmk__rsc_trace(bundle, "Ordering %s probe on %s",
                             replica->remote->id,
                             pcmk__node_name(probe_data->node));
             pcmk__new_ordering(replica->container,
@@ -962,7 +980,7 @@ create_replica_probes(pcmk__bundle_replica_t *replica, void *user_data)
                                             PCMK_ACTION_START, 0),
                                NULL, replica->remote, NULL, probe,
                                pcmk__ar_nested_remote_probe,
-                               probe_data->bundle->cluster);
+                               bundle->priv->scheduler);
         }
     }
     return true;
@@ -983,7 +1001,7 @@ pcmk__bundle_create_probe(pcmk_resource_t *rsc, pcmk_node_t *node)
 {
     struct probe_data probe_data = { rsc, node, false };
 
-    CRM_ASSERT(pcmk__is_bundle(rsc));
+    pcmk__assert(pcmk__is_bundle(rsc));
     pe__foreach_bundle_replica(rsc, create_replica_probes, &probe_data);
     return probe_data.any_created;
 }
@@ -1001,16 +1019,14 @@ static bool
 output_replica_actions(pcmk__bundle_replica_t *replica, void *user_data)
 {
     if (replica->ip != NULL) {
-        replica->ip->cmds->output_actions(replica->ip);
-    }
-    if (replica->container != NULL) {
-        replica->container->cmds->output_actions(replica->container);
+        replica->ip->priv->cmds->output_actions(replica->ip);
     }
+    replica->container->priv->cmds->output_actions(replica->container);
     if (replica->remote != NULL) {
-        replica->remote->cmds->output_actions(replica->remote);
+        replica->remote->priv->cmds->output_actions(replica->remote);
     }
     if (replica->child != NULL) {
-        replica->child->cmds->output_actions(replica->child);
+        replica->child->priv->cmds->output_actions(replica->child);
     }
     return true;
 }
@@ -1024,11 +1040,11 @@ output_replica_actions(pcmk__bundle_replica_t *replica, void *user_data)
 void
 pcmk__output_bundle_actions(pcmk_resource_t *rsc)
 {
-    CRM_ASSERT(pcmk__is_bundle(rsc));
+    pcmk__assert(pcmk__is_bundle(rsc));
     pe__foreach_bundle_replica(rsc, output_replica_actions, NULL);
 }
 
-// Bundle implementation of pcmk_assignment_methods_t:add_utilization()
+// Bundle implementation of pcmk__assignment_methods_t:add_utilization()
 void
 pcmk__bundle_add_utilization(const pcmk_resource_t *rsc,
                              const pcmk_resource_t *orig_rsc, GList *all_rscs,
@@ -1036,9 +1052,9 @@ pcmk__bundle_add_utilization(const pcmk_resource_t *rsc,
 {
     pcmk_resource_t *container = NULL;
 
-    CRM_ASSERT(pcmk__is_bundle(rsc));
+    pcmk__assert(pcmk__is_bundle(rsc));
 
-    if (!pcmk_is_set(rsc->flags, pcmk_rsc_unassigned)) {
+    if (!pcmk_is_set(rsc->flags, pcmk__rsc_unassigned)) {
         return;
     }
 
@@ -1048,15 +1064,15 @@ pcmk__bundle_add_utilization(const pcmk_resource_t *rsc,
      */
     container = pe__first_container(rsc);
     if (container != NULL) {
-        container->cmds->add_utilization(container, orig_rsc, all_rscs,
-                                         utilization);
+        container->priv->cmds->add_utilization(container, orig_rsc, all_rscs,
+                                               utilization);
     }
 }
 
-// Bundle implementation of pcmk_assignment_methods_t:shutdown_lock()
+// Bundle implementation of pcmk__assignment_methods_t:shutdown_lock()
 void
 pcmk__bundle_shutdown_lock(pcmk_resource_t *rsc)
 {
-    CRM_ASSERT(pcmk__is_bundle(rsc));
+    pcmk__assert(pcmk__is_bundle(rsc));
     // Bundles currently don't support shutdown locks
 }
