@@ -39,21 +39,21 @@ pcmk__clone_assign(pcmk_resource_t *rsc, const pcmk_node_t *prefer,
 {
     GList *colocations = NULL;
 
-    CRM_ASSERT(pcmk__is_clone(rsc));
+    pcmk__assert(pcmk__is_clone(rsc));
 
-    if (!pcmk_is_set(rsc->flags, pcmk_rsc_unassigned)) {
+    if (!pcmk_is_set(rsc->flags, pcmk__rsc_unassigned)) {
         return NULL; // Assignment has already been done
     }
 
     // Detect assignment loops
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_assigning)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_assigning)) {
         pcmk__rsc_debug(rsc, "Breaking assignment loop involving %s", rsc->id);
         return NULL;
     }
-    pcmk__set_rsc_flags(rsc, pcmk_rsc_assigning);
+    pcmk__set_rsc_flags(rsc, pcmk__rsc_assigning);
 
     // If this clone is promotable, consider nodes' promotion scores
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_promotable)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_promotable)) {
         pcmk__add_promotion_scores(rsc);
     }
 
@@ -61,11 +61,11 @@ pcmk__clone_assign(pcmk_resource_t *rsc, const pcmk_node_t *prefer,
     colocations = pcmk__this_with_colocations(rsc);
     for (GList *iter = colocations; iter != NULL; iter = iter->next) {
         pcmk__colocation_t *constraint = (pcmk__colocation_t *) iter->data;
+        pcmk_resource_t *primary = constraint->primary;
 
         pcmk__rsc_trace(rsc, "%s: Assigning colocation %s primary %s first",
-                        rsc->id, constraint->id, constraint->primary->id);
-        constraint->primary->cmds->assign(constraint->primary, prefer,
-                                          stop_if_fail);
+                        rsc->id, constraint->id, primary->id);
+        primary->priv->cmds->assign(primary, prefer, stop_if_fail);
     }
     g_list_free(colocations);
 
@@ -74,19 +74,20 @@ pcmk__clone_assign(pcmk_resource_t *rsc, const pcmk_node_t *prefer,
     g_list_foreach(colocations, pcmk__add_dependent_scores, rsc);
     g_list_free(colocations);
 
-    pe__show_node_scores(!pcmk_is_set(rsc->cluster->flags,
-                                      pcmk_sched_output_scores),
-                         rsc, __func__, rsc->allowed_nodes, rsc->cluster);
+    pe__show_node_scores(!pcmk_is_set(rsc->priv->scheduler->flags,
+                                      pcmk__sched_output_scores),
+                         rsc, __func__, rsc->priv->allowed_nodes,
+                         rsc->priv->scheduler);
 
-    rsc->children = g_list_sort(rsc->children, pcmk__cmp_instance);
-    pcmk__assign_instances(rsc, rsc->children, pe__clone_max(rsc),
+    rsc->priv->children = g_list_sort(rsc->priv->children, pcmk__cmp_instance);
+    pcmk__assign_instances(rsc, rsc->priv->children, pe__clone_max(rsc),
                            pe__clone_node_max(rsc));
 
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_promotable)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_promotable)) {
         pcmk__set_instance_roles(rsc);
     }
 
-    pcmk__clear_rsc_flags(rsc, pcmk_rsc_unassigned|pcmk_rsc_assigning);
+    pcmk__clear_rsc_flags(rsc, pcmk__rsc_unassigned|pcmk__rsc_assigning);
     pcmk__rsc_trace(rsc, "Assigned clone %s", rsc->id);
     return NULL;
 }
@@ -100,11 +101,11 @@ pcmk__clone_assign(pcmk_resource_t *rsc, const pcmk_node_t *prefer,
 void
 pcmk__clone_create_actions(pcmk_resource_t *rsc)
 {
-    CRM_ASSERT(pcmk__is_clone(rsc));
+    pcmk__assert(pcmk__is_clone(rsc));
 
     pcmk__rsc_trace(rsc, "Creating actions for clone %s", rsc->id);
-    pcmk__create_instance_actions(rsc, rsc->children);
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_promotable)) {
+    pcmk__create_instance_actions(rsc, rsc->priv->children);
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_promotable)) {
         pcmk__create_promotable_actions(rsc);
     }
 }
@@ -120,7 +121,7 @@ pcmk__clone_internal_constraints(pcmk_resource_t *rsc)
 {
     bool ordered = false;
 
-    CRM_ASSERT(pcmk__is_clone(rsc));
+    pcmk__assert(pcmk__is_clone(rsc));
 
     pcmk__rsc_trace(rsc, "Creating internal constraints for clone %s", rsc->id);
 
@@ -136,7 +137,7 @@ pcmk__clone_internal_constraints(pcmk_resource_t *rsc)
                                  pcmk__ar_unrunnable_first_blocks);
 
     // Demoted -> stop and started -> promote
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_promotable)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_promotable)) {
         pcmk__order_resource_actions(rsc, PCMK_ACTION_DEMOTED,
                                      rsc, PCMK_ACTION_STOP,
                                      pcmk__ar_ordered);
@@ -151,12 +152,15 @@ pcmk__clone_internal_constraints(pcmk_resource_t *rsc)
          * instances might have been previously shuffled for assignment or
          * promotion purposes, so re-sort them.
          */
-        rsc->children = g_list_sort(rsc->children, pcmk__cmp_instance_number);
+        rsc->priv->children = g_list_sort(rsc->priv->children,
+                                          pcmk__cmp_instance_number);
     }
-    for (GList *iter = rsc->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = rsc->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *instance = (pcmk_resource_t *) iter->data;
 
-        instance->cmds->internal_constraints(instance);
+        instance->priv->cmds->internal_constraints(instance);
 
         // Start clone -> start instance -> clone started
         pcmk__order_starts(rsc, instance, pcmk__ar_unrunnable_first_blocks
@@ -185,7 +189,7 @@ pcmk__clone_internal_constraints(pcmk_resource_t *rsc)
             }
         }
     }
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_promotable)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_promotable)) {
         pcmk__order_promotable_instances(rsc);
     }
 }
@@ -201,15 +205,16 @@ pcmk__clone_internal_constraints(pcmk_resource_t *rsc)
 static bool
 can_interleave(const pcmk__colocation_t *colocation)
 {
+    const pcmk_resource_t *primary = colocation->primary;
     const pcmk_resource_t *dependent = colocation->dependent;
 
     // Only colocations between clone or bundle resources use interleaving
-    if (dependent->variant <= pcmk_rsc_variant_group) {
+    if (dependent->priv->variant <= pcmk__rsc_variant_group) {
         return false;
     }
 
     // Only the dependent needs to be marked for interleaving
-    if (!crm_is_true(g_hash_table_lookup(dependent->meta,
+    if (!crm_is_true(g_hash_table_lookup(dependent->priv->meta,
                                          PCMK_META_INTERLEAVE))) {
         return false;
     }
@@ -217,11 +222,11 @@ can_interleave(const pcmk__colocation_t *colocation)
     /* @TODO Do we actually care about multiple primary instances sharing a
      * dependent instance?
      */
-    if (dependent->fns->max_per_node(dependent)
-        != colocation->primary->fns->max_per_node(colocation->primary)) {
+    if (dependent->priv->fns->max_per_node(dependent)
+        != primary->priv->fns->max_per_node(primary)) {
         pcmk__config_err("Cannot interleave %s and %s because they do not "
                          "support the same number of instances per node",
-                         dependent->id, colocation->primary->id);
+                         dependent->id, primary->id);
         return false;
     }
 
@@ -256,12 +261,10 @@ pcmk__clone_apply_coloc_score(pcmk_resource_t *dependent,
      * we add its colocation constraints to its instances and call the
      * apply_coloc_score() method for the instances as dependents.
      */
-    CRM_ASSERT(!for_dependent);
+    pcmk__assert(!for_dependent && (colocation != NULL)
+                 && pcmk__is_clone(primary) && pcmk__is_primitive(dependent));
 
-    CRM_ASSERT((colocation != NULL) && pcmk__is_clone(primary)
-               && pcmk__is_primitive(dependent));
-
-    if (pcmk_is_set(primary->flags, pcmk_rsc_unassigned)) {
+    if (pcmk_is_set(primary->flags, pcmk__rsc_unassigned)) {
         pcmk__rsc_trace(primary,
                         "Delaying processing colocation %s "
                         "because cloned primary %s is still provisional",
@@ -274,10 +277,10 @@ pcmk__clone_apply_coloc_score(pcmk_resource_t *dependent,
                     pcmk_readable_score(colocation->score));
 
     // Apply role-specific colocations
-    if (pcmk_is_set(primary->flags, pcmk_rsc_promotable)
+    if (pcmk_is_set(primary->flags, pcmk__rsc_promotable)
         && (colocation->primary_role != pcmk_role_unknown)) {
 
-        if (pcmk_is_set(dependent->flags, pcmk_rsc_unassigned)) {
+        if (pcmk_is_set(dependent->flags, pcmk__rsc_unassigned)) {
             // We're assigning the dependent to a node
             pcmk__update_dependent_with_promotable(primary, dependent,
                                                    colocation);
@@ -303,9 +306,9 @@ pcmk__clone_apply_coloc_score(pcmk_resource_t *dependent,
             pcmk__rsc_debug(primary, "Interleaving %s with %s",
                             dependent->id, primary_instance->id);
 
-            return dependent->cmds->apply_coloc_score(dependent,
-                                                      primary_instance,
-                                                      colocation, true);
+            return dependent->priv->cmds->apply_coloc_score(dependent,
+                                                            primary_instance,
+                                                            colocation, true);
         }
 
         if (colocation->score >= PCMK_SCORE_INFINITY) {
@@ -328,15 +331,19 @@ pcmk__clone_apply_coloc_score(pcmk_resource_t *dependent,
         GList *primary_nodes = NULL;
 
         // Dependent can run only where primary will have unblocked instances
-        for (iter = primary->children; iter != NULL; iter = iter->next) {
-            const pcmk_resource_t *instance = iter->data;
-            pcmk_node_t *chosen = instance->fns->location(instance, NULL, 0);
+        for (iter = primary->priv->children;
+             iter != NULL; iter = iter->next) {
 
+            const pcmk_resource_t *instance = iter->data;
+            pcmk_node_t *chosen = NULL;
+
+            chosen = instance->priv->fns->location(instance, NULL,
+                                                   pcmk__rsc_node_assigned);
             if ((chosen != NULL)
-                && !is_set_recursive(instance, pcmk_rsc_blocked, TRUE)) {
+                && !is_set_recursive(instance, pcmk__rsc_blocked, TRUE)) {
                 pcmk__rsc_trace(primary, "Allowing %s: %s %d",
                                 colocation->id, pcmk__node_name(chosen),
-                                chosen->weight);
+                                chosen->assign->score);
                 primary_nodes = g_list_prepend(primary_nodes, chosen);
             }
         }
@@ -347,43 +354,51 @@ pcmk__clone_apply_coloc_score(pcmk_resource_t *dependent,
     }
 
     // Apply optional colocations
-    for (iter = primary->children; iter != NULL; iter = iter->next) {
+    for (iter = primary->priv->children; iter != NULL; iter = iter->next) {
         const pcmk_resource_t *instance = iter->data;
-        int instance_delta = instance->cmds->apply_coloc_score(dependent,
-                                                               instance,
-                                                               colocation,
-                                                               false);
+        int instance_delta = instance->priv->cmds->apply_coloc_score(dependent,
+                                                                     instance,
+                                                                     colocation,
+                                                                     false);
 
         priority_delta = pcmk__add_scores(priority_delta, instance_delta);
     }
     return priority_delta;
 }
 
-// Clone implementation of pcmk_assignment_methods_t:with_this_colocations()
+// Clone implementation of pcmk__assignment_methods_t:with_this_colocations()
 void
 pcmk__with_clone_colocations(const pcmk_resource_t *rsc,
                              const pcmk_resource_t *orig_rsc, GList **list)
 {
+    const pcmk_resource_t *parent = NULL;
+
     CRM_CHECK((rsc != NULL) && (orig_rsc != NULL) && (list != NULL), return);
+    parent = rsc->priv->parent;
 
-    pcmk__add_with_this_list(list, rsc->rsc_cons_lhs, orig_rsc);
+    pcmk__add_with_this_list(list, rsc->priv->with_this_colocations,
+                             orig_rsc);
 
-    if (rsc->parent != NULL) {
-        rsc->parent->cmds->with_this_colocations(rsc->parent, orig_rsc, list);
+    if (parent != NULL) {
+        parent->priv->cmds->with_this_colocations(parent, orig_rsc, list);
     }
 }
 
-// Clone implementation of pcmk_assignment_methods_t:this_with_colocations()
+// Clone implementation of pcmk__assignment_methods_t:this_with_colocations()
 void
 pcmk__clone_with_colocations(const pcmk_resource_t *rsc,
                              const pcmk_resource_t *orig_rsc, GList **list)
 {
+    const pcmk_resource_t *parent = NULL;
+
     CRM_CHECK((rsc != NULL) && (orig_rsc != NULL) && (list != NULL), return);
+    parent = rsc->priv->parent;
 
-    pcmk__add_this_with_list(list, rsc->rsc_cons, orig_rsc);
+    pcmk__add_this_with_list(list, rsc->priv->this_with_colocations,
+                             orig_rsc);
 
-    if (rsc->parent != NULL) {
-        rsc->parent->cmds->this_with_colocations(rsc->parent, orig_rsc, list);
+    if (parent != NULL) {
+        parent->priv->cmds->this_with_colocations(parent, orig_rsc, list);
     }
 }
 
@@ -399,9 +414,10 @@ pcmk__clone_with_colocations(const pcmk_resource_t *rsc,
 uint32_t
 pcmk__clone_action_flags(pcmk_action_t *action, const pcmk_node_t *node)
 {
-    CRM_ASSERT((action != NULL) && pcmk__is_clone(action->rsc));
+    pcmk__assert((action != NULL) && pcmk__is_clone(action->rsc));
 
-    return pcmk__collective_action_flags(action, action->rsc->children, node);
+    return pcmk__collective_action_flags(action, action->rsc->priv->children,
+                                         node);
 }
 
 /*!
@@ -418,10 +434,12 @@ pcmk__clone_apply_location(pcmk_resource_t *rsc, pcmk__location_t *location)
 
     pcmk__apply_location(rsc, location);
 
-    for (GList *iter = rsc->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = rsc->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *instance = (pcmk_resource_t *) iter->data;
 
-        instance->cmds->apply_location(instance, location);
+        instance->priv->cmds->apply_location(instance, location);
     }
 }
 
@@ -431,7 +449,7 @@ call_action_flags(gpointer data, gpointer user_data)
 {
     pcmk_resource_t *rsc = user_data;
 
-    rsc->cmds->action_flags((pcmk_action_t *) data, NULL);
+    rsc->priv->cmds->action_flags((pcmk_action_t *) data, NULL);
 }
 
 /*!
@@ -443,15 +461,17 @@ call_action_flags(gpointer data, gpointer user_data)
 void
 pcmk__clone_add_actions_to_graph(pcmk_resource_t *rsc)
 {
-    CRM_ASSERT(pcmk__is_clone(rsc));
+    pcmk__assert(pcmk__is_clone(rsc));
 
-    g_list_foreach(rsc->actions, call_action_flags, rsc);
+    g_list_foreach(rsc->priv->actions, call_action_flags, rsc);
     pe__create_clone_notifications(rsc);
 
-    for (GList *iter = rsc->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = rsc->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *child_rsc = (pcmk_resource_t *) iter->data;
 
-        child_rsc->cmds->add_actions_to_graph(child_rsc);
+        child_rsc->priv->cmds->add_actions_to_graph(child_rsc);
     }
 
     pcmk__add_rsc_actions_to_graph(rsc);
@@ -465,15 +485,15 @@ pcmk__clone_add_actions_to_graph(pcmk_resource_t *rsc)
  * \param[in] rsc   Resource to check
  * \param[in] node  Node to check
  *
- * \return true if \p node is in the known_on table of \p rsc or any of its
+ * \return true if \p node is in the probed_nodes table of \p rsc or any of its
  *         children, otherwise false
  */
 static bool
 rsc_probed_on(const pcmk_resource_t *rsc, const pcmk_node_t *node)
 {
-    if (rsc->children != NULL) {
-        for (GList *child_iter = rsc->children; child_iter != NULL;
-             child_iter = child_iter->next) {
+    if (rsc->priv->children != NULL) {
+        for (GList *child_iter = rsc->priv->children;
+             child_iter != NULL; child_iter = child_iter->next) {
 
             pcmk_resource_t *child = (pcmk_resource_t *) child_iter->data;
 
@@ -484,11 +504,11 @@ rsc_probed_on(const pcmk_resource_t *rsc, const pcmk_node_t *node)
         return false;
     }
 
-    if (rsc->known_on != NULL) {
+    if (rsc->priv->probed_nodes != NULL) {
         GHashTableIter iter;
         pcmk_node_t *known_node = NULL;
 
-        g_hash_table_iter_init(&iter, rsc->known_on);
+        g_hash_table_iter_init(&iter, rsc->priv->probed_nodes);
         while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &known_node)) {
             if (pcmk__same_node(node, known_node)) {
                 return true;
@@ -511,7 +531,9 @@ rsc_probed_on(const pcmk_resource_t *rsc, const pcmk_node_t *node)
 static pcmk_resource_t *
 find_probed_instance_on(const pcmk_resource_t *clone, const pcmk_node_t *node)
 {
-    for (GList *iter = clone->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = clone->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *instance = (pcmk_resource_t *) iter->data;
 
         if (rsc_probed_on(instance, node)) {
@@ -535,12 +557,14 @@ probe_anonymous_clone(pcmk_resource_t *clone, pcmk_node_t *node)
     pcmk_resource_t *child = find_probed_instance_on(clone, node);
 
     // Otherwise, check if we plan to start an instance on this node
-    for (GList *iter = clone->children; (iter != NULL) && (child == NULL);
-         iter = iter->next) {
+    for (GList *iter = clone->priv->children;
+         (iter != NULL) && (child == NULL); iter = iter->next) {
+
         pcmk_resource_t *instance = (pcmk_resource_t *) iter->data;
         const pcmk_node_t *instance_node = NULL;
 
-        instance_node = instance->fns->location(instance, NULL, 0);
+        instance_node = instance->priv->fns->location(instance, NULL,
+                                                      pcmk__rsc_node_assigned);
         if (pcmk__same_node(instance_node, node)) {
             child = instance;
         }
@@ -548,11 +572,11 @@ probe_anonymous_clone(pcmk_resource_t *clone, pcmk_node_t *node)
 
     // Otherwise, use the first clone instance
     if (child == NULL) {
-        child = clone->children->data;
+        child = clone->priv->children->data;
     }
 
     // Anonymous clones only need to probe a single instance
-    return child->cmds->create_probe(child, node);
+    return child->priv->cmds->create_probe(child, node);
 }
 
 /*!
@@ -567,9 +591,9 @@ probe_anonymous_clone(pcmk_resource_t *clone, pcmk_node_t *node)
 bool
 pcmk__clone_create_probe(pcmk_resource_t *rsc, pcmk_node_t *node)
 {
-    CRM_ASSERT((node != NULL) && pcmk__is_clone(rsc));
+    pcmk__assert((node != NULL) && pcmk__is_clone(rsc));
 
-    if (rsc->exclusive_discover) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_exclusive_probes)) {
         /* The clone is configured to be probed only where a location constraint
          * exists with PCMK_XA_RESOURCE_DISCOVERY set to exclusive.
          *
@@ -579,27 +603,28 @@ pcmk__clone_create_probe(pcmk_resource_t *rsc, pcmk_node_t *node)
          * instances), and affects the CRM_meta_notify_available_uname variable
          * passed with notify actions.
          */
-        pcmk_node_t *allowed = g_hash_table_lookup(rsc->allowed_nodes,
-                                                   node->details->id);
+        pcmk_node_t *allowed = g_hash_table_lookup(rsc->priv->allowed_nodes,
+                                                   node->priv->id);
 
         if ((allowed == NULL)
-            || (allowed->rsc_discover_mode != pcmk_probe_exclusive)) {
+            || (allowed->assign->probe_mode != pcmk__probe_exclusive)) {
             /* This node is not marked for resource discovery. Remove it from
-             * allowed_nodes so that notifications contain only nodes that the
+             * allowed nodes so that notifications contain only nodes that the
              * clone can possibly run on.
              */
             pcmk__rsc_trace(rsc,
                             "Skipping probe for %s on %s because resource has "
                             "exclusive discovery but is not allowed on node",
                             rsc->id, pcmk__node_name(node));
-            g_hash_table_remove(rsc->allowed_nodes, node->details->id);
+            g_hash_table_remove(rsc->priv->allowed_nodes, node->priv->id);
             return false;
         }
     }
 
-    rsc->children = g_list_sort(rsc->children, pcmk__cmp_instance_number);
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_unique)) {
-        return pcmk__probe_resource_list(rsc->children, node);
+    rsc->priv->children = g_list_sort(rsc->priv->children,
+                                      pcmk__cmp_instance_number);
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_unique)) {
+        return pcmk__probe_resource_list(rsc->priv->children, node);
     } else {
         return probe_anonymous_clone(rsc, node);
     }
@@ -619,14 +644,14 @@ pcmk__clone_add_graph_meta(const pcmk_resource_t *rsc, xmlNode *xml)
 {
     char *name = NULL;
 
-    CRM_ASSERT(pcmk__is_clone(rsc) && (xml != NULL));
+    pcmk__assert(pcmk__is_clone(rsc) && (xml != NULL));
 
     name = crm_meta_name(PCMK_META_GLOBALLY_UNIQUE);
-    crm_xml_add(xml, name, pcmk__flag_text(rsc->flags, pcmk_rsc_unique));
+    crm_xml_add(xml, name, pcmk__flag_text(rsc->flags, pcmk__rsc_unique));
     free(name);
 
     name = crm_meta_name(PCMK_META_NOTIFY);
-    crm_xml_add(xml, name, pcmk__flag_text(rsc->flags, pcmk_rsc_notify));
+    crm_xml_add(xml, name, pcmk__flag_text(rsc->flags, pcmk__rsc_notify));
     free(name);
 
     name = crm_meta_name(PCMK_META_CLONE_MAX);
@@ -637,7 +662,7 @@ pcmk__clone_add_graph_meta(const pcmk_resource_t *rsc, xmlNode *xml)
     crm_xml_add_int(xml, name, pe__clone_node_max(rsc));
     free(name);
 
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_promotable)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_promotable)) {
         int promoted_max = pe__clone_promoted_max(rsc);
         int promoted_node_max = pe__clone_promoted_node_max(rsc);
 
@@ -662,7 +687,7 @@ pcmk__clone_add_graph_meta(const pcmk_resource_t *rsc, xmlNode *xml)
     }
 }
 
-// Clone implementation of pcmk_assignment_methods_t:add_utilization()
+// Clone implementation of pcmk__assignment_methods_t:add_utilization()
 void
 pcmk__clone_add_utilization(const pcmk_resource_t *rsc,
                             const pcmk_resource_t *orig_rsc, GList *all_rscs,
@@ -671,29 +696,31 @@ pcmk__clone_add_utilization(const pcmk_resource_t *rsc,
     bool existing = false;
     pcmk_resource_t *child = NULL;
 
-    CRM_ASSERT(pcmk__is_clone(rsc) && (orig_rsc != NULL)
-               && (utilization != NULL));
+    pcmk__assert(pcmk__is_clone(rsc) && (orig_rsc != NULL)
+                 && (utilization != NULL));
 
-    if (!pcmk_is_set(rsc->flags, pcmk_rsc_unassigned)) {
+    if (!pcmk_is_set(rsc->flags, pcmk__rsc_unassigned)) {
         return;
     }
 
     // Look for any child already existing in the list
-    for (GList *iter = rsc->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = rsc->priv->children;
+         iter != NULL; iter = iter->next) {
+
         child = (pcmk_resource_t *) iter->data;
         if (g_list_find(all_rscs, child)) {
             existing = true; // Keep checking remaining children
         } else {
             // If this is a clone of a group, look for group's members
-            for (GList *member_iter = child->children; member_iter != NULL;
-                 member_iter = member_iter->next) {
+            for (GList *member_iter = child->priv->children;
+                 member_iter != NULL; member_iter = member_iter->next) {
 
                 pcmk_resource_t *member = (pcmk_resource_t *) member_iter->data;
 
                 if (g_list_find(all_rscs, member) != NULL) {
                     // Add *child's* utilization, not group member's
-                    child->cmds->add_utilization(child, orig_rsc, all_rscs,
-                                                 utilization);
+                    child->priv->cmds->add_utilization(child, orig_rsc,
+                                                       all_rscs, utilization);
                     existing = true;
                     break;
                 }
@@ -701,18 +728,19 @@ pcmk__clone_add_utilization(const pcmk_resource_t *rsc,
         }
     }
 
-    if (!existing && (rsc->children != NULL)) {
+    if (!existing && (rsc->priv->children != NULL)) {
         // If nothing was found, still add first child's utilization
-        child = (pcmk_resource_t *) rsc->children->data;
+        child = (pcmk_resource_t *) rsc->priv->children->data;
 
-        child->cmds->add_utilization(child, orig_rsc, all_rscs, utilization);
+        child->priv->cmds->add_utilization(child, orig_rsc, all_rscs,
+                                           utilization);
     }
 }
 
-// Clone implementation of pcmk_assignment_methods_t:shutdown_lock()
+// Clone implementation of pcmk__assignment_methods_t:shutdown_lock()
 void
 pcmk__clone_shutdown_lock(pcmk_resource_t *rsc)
 {
-    CRM_ASSERT(pcmk__is_clone(rsc));
+    pcmk__assert(pcmk__is_clone(rsc));
     return; // Clones currently don't support shutdown locks
 }
