@@ -17,6 +17,7 @@
 
 #include <crm/crm.h>
 #include <crm/cib.h>
+#include <crm/common/scheduler.h>
 #include <crm/common/xml.h>
 #include <crm/common/xml_internal.h>
 #include <crm/common/iso8601.h>
@@ -26,27 +27,6 @@
 #include <pacemaker-internal.h>
 #include "libpacemaker_private.h"
 
-static bool
-evaluate_lifetime(xmlNode *lifetime, pcmk_scheduler_t *scheduler)
-{
-    bool result = false;
-    crm_time_t *next_change = crm_time_new_undefined();
-    pcmk_rule_input_t rule_input = {
-        .now = scheduler->now,
-    };
-
-    result = (pcmk__evaluate_rules(lifetime, &rule_input,
-                                   next_change) == pcmk_rc_ok);
-
-    if (crm_time_is_defined(next_change)) {
-        time_t recheck = (time_t) crm_time_get_seconds_since_epoch(next_change);
-
-        pe__update_recheck_time(recheck, scheduler, "constraint lifetime");
-    }
-    crm_time_free(next_change);
-    return result;
-}
-
 /*!
  * \internal
  * \brief Unpack constraints from XML
@@ -64,9 +44,8 @@ pcmk__unpack_constraints(pcmk_scheduler_t *scheduler)
 
     for (xmlNode *xml_obj = pcmk__xe_first_child(xml_constraints, NULL, NULL,
                                                  NULL);
-         xml_obj != NULL; xml_obj = pcmk__xe_next(xml_obj)) {
+         xml_obj != NULL; xml_obj = pcmk__xe_next(xml_obj, NULL)) {
 
-        xmlNode *lifetime = NULL;
         const char *id = crm_element_value(xml_obj, PCMK_XA_ID);
         const char *tag = (const char *) xml_obj->name;
 
@@ -78,17 +57,7 @@ pcmk__unpack_constraints(pcmk_scheduler_t *scheduler)
 
         crm_trace("Unpacking %s constraint '%s'", tag, id);
 
-        lifetime = pcmk__xe_first_child(xml_obj, PCMK__XE_LIFETIME, NULL, NULL);
-        if (lifetime != NULL) {
-            pcmk__config_warn("Support for '" PCMK__XE_LIFETIME "' element "
-                              "(in %s) is deprecated and will be dropped "
-                              "in a later release", id);
-        }
-
-        if ((lifetime != NULL) && !evaluate_lifetime(lifetime, scheduler)) {
-            crm_info("Constraint %s %s is not active", tag, id);
-
-        } else if (pcmk__str_eq(PCMK_XE_RSC_ORDER, tag, pcmk__str_none)) {
+        if (pcmk__str_eq(PCMK_XE_RSC_ORDER, tag, pcmk__str_none)) {
             pcmk__unpack_ordering(xml_obj, scheduler);
 
         } else if (pcmk__str_eq(PCMK_XE_RSC_COLOCATION, tag, pcmk__str_none)) {
@@ -114,9 +83,10 @@ pcmk__find_constraint_resource(GList *rsc_list, const char *id)
     }
     for (GList *iter = rsc_list; iter != NULL; iter = iter->next) {
         pcmk_resource_t *parent = iter->data;
-        pcmk_resource_t *match = parent->fns->find_rsc(parent, id, NULL,
-                                                       pcmk_rsc_match_history);
+        pcmk_resource_t *match = NULL;
 
+        match = parent->priv->fns->find_rsc(parent, id, NULL,
+                                            pcmk_rsc_match_history);
         if (match != NULL) {
             if (!pcmk__str_eq(match->id, id, pcmk__str_none)) {
                 /* We found an instance of a clone instead */
@@ -143,12 +113,12 @@ pcmk__find_constraint_resource(GList *rsc_list, const char *id)
  */
 static bool
 find_constraint_tag(const pcmk_scheduler_t *scheduler, const char *id,
-                    pcmk_tag_t **tag)
+                    pcmk__idref_t **tag)
 {
     *tag = NULL;
 
     // Check whether id refers to a resource set template
-    if (g_hash_table_lookup_extended(scheduler->template_rsc_sets, id,
+    if (g_hash_table_lookup_extended(scheduler->priv->templates, id,
                                      NULL, (gpointer *) tag)) {
         if (*tag == NULL) {
             crm_notice("No resource is derived from template '%s'", id);
@@ -158,7 +128,7 @@ find_constraint_tag(const pcmk_scheduler_t *scheduler, const char *id,
     }
 
     // If not, check whether id refers to a tag
-    if (g_hash_table_lookup_extended(scheduler->tags, id,
+    if (g_hash_table_lookup_extended(scheduler->priv->tags, id,
                                      NULL, (gpointer *) tag)) {
         if (*tag == NULL) {
             crm_notice("No resource is tagged with '%s'", id);
@@ -171,6 +141,44 @@ find_constraint_tag(const pcmk_scheduler_t *scheduler, const char *id,
     return false;
 }
 
+/*!
+ * \internal
+ * \brief Parse a role attribute from a constraint
+ *
+ * This is like pcmk_parse_role() except that started is treated as
+ * pcmk_role_unknown (indicating any role), and the return value is
+ * pcmk_rc_unpack_error for invalid specifications.
+ *
+ * \param[in] id         ID of constraint being parsed (for logging only)
+ * \param[in] role_spec  Role specification
+ * \param[in] role       Where to store parsed role
+ *
+ * \return Standard Pacemaker return code
+ */
+int
+pcmk__parse_constraint_role(const char *id, const char *role_spec,
+                            enum rsc_role_e *role)
+{
+    *role = pcmk_parse_role(role_spec);
+    switch (*role) {
+        case pcmk_role_unknown:
+            if (role_spec != NULL) {
+                pcmk__config_err("Ignoring constraint %s: Invalid role '%s'",
+                                 id, role_spec);
+                return pcmk_rc_unpack_error;
+            }
+            break;
+
+        case pcmk_role_started:
+            *role = pcmk_role_unknown;
+            break;
+
+        default:
+            break;
+    }
+    return pcmk_rc_ok;
+}
+
 /*!
  * \brief
  * \internal Check whether an ID refers to a valid resource or tag
@@ -186,10 +194,10 @@ find_constraint_tag(const pcmk_scheduler_t *scheduler, const char *id,
  */
 bool
 pcmk__valid_resource_or_tag(const pcmk_scheduler_t *scheduler, const char *id,
-                            pcmk_resource_t **rsc, pcmk_tag_t **tag)
+                            pcmk_resource_t **rsc, pcmk__idref_t **tag)
 {
     if (rsc != NULL) {
-        *rsc = pcmk__find_constraint_resource(scheduler->resources, id);
+        *rsc = pcmk__find_constraint_resource(scheduler->priv->resources, id);
         if (*rsc != NULL) {
             return true;
         }
@@ -216,7 +224,8 @@ pcmk__valid_resource_or_tag(const pcmk_scheduler_t *scheduler, const char *id,
  * \param[in]     scheduler  Scheduler data
  *
  * \return Equivalent XML with resource tags replaced (or NULL if none)
- * \note It is the caller's responsibility to free the result with free_xml().
+ * \note It is the caller's responsibility to free the return value with
+ *       \c pcmk__xml_free().
  */
 xmlNode *
 pcmk__expand_tags_in_sets(xmlNode *xml_obj, const pcmk_scheduler_t *scheduler)
@@ -234,24 +243,25 @@ pcmk__expand_tags_in_sets(xmlNode *xml_obj, const pcmk_scheduler_t *scheduler)
 
     for (xmlNode *set = pcmk__xe_first_child(new_xml, PCMK_XE_RESOURCE_SET,
                                              NULL, NULL);
-         set != NULL; set = pcmk__xe_next_same(set)) {
+         set != NULL; set = pcmk__xe_next(set, PCMK_XE_RESOURCE_SET)) {
 
         GList *tag_refs = NULL;
         GList *iter = NULL;
 
         for (xmlNode *xml_rsc = pcmk__xe_first_child(set, PCMK_XE_RESOURCE_REF,
                                                      NULL, NULL);
-             xml_rsc != NULL; xml_rsc = pcmk__xe_next_same(xml_rsc)) {
+             xml_rsc != NULL;
+             xml_rsc = pcmk__xe_next(xml_rsc, PCMK_XE_RESOURCE_REF)) {
 
             pcmk_resource_t *rsc = NULL;
-            pcmk_tag_t *tag = NULL;
+            pcmk__idref_t *tag = NULL;
 
             if (!pcmk__valid_resource_or_tag(scheduler, pcmk__xe_id(xml_rsc),
                                              &rsc, &tag)) {
                 pcmk__config_err("Ignoring resource sets for constraint '%s' "
                                  "because '%s' is not a valid resource or tag",
                                  pcmk__xe_id(xml_obj), pcmk__xe_id(xml_rsc));
-                free_xml(new_xml);
+                pcmk__xml_free(new_xml);
                 return NULL;
 
             } else if (rsc) {
@@ -283,17 +293,14 @@ pcmk__expand_tags_in_sets(xmlNode *xml_obj, const pcmk_scheduler_t *scheduler)
                  */
 
                 for (iter = tag->refs; iter != NULL; iter = iter->next) {
-                    const char *obj_ref = iter->data;
-                    xmlNode *new_rsc_ref = NULL;
+                    const char *ref_id = iter->data;
+                    xmlNode *new_ref = pcmk__xe_create(set,
+                                                       PCMK_XE_RESOURCE_REF);
 
-                    new_rsc_ref = xmlNewDocRawNode(set->doc, NULL,
-                                                   (pcmkXmlStr)
-                                                   PCMK_XE_RESOURCE_REF,
-                                                   NULL);
-                    crm_xml_add(new_rsc_ref, PCMK_XA_ID, obj_ref);
-                    xmlAddNextSibling(last_ref, new_rsc_ref);
+                    crm_xml_add(new_ref, PCMK_XA_ID, ref_id);
+                    xmlAddNextSibling(last_ref, new_ref);
 
-                    last_ref = new_rsc_ref;
+                    last_ref = new_ref;
                 }
 
                 any_refs = true;
@@ -318,13 +325,13 @@ pcmk__expand_tags_in_sets(xmlNode *xml_obj, const pcmk_scheduler_t *scheduler)
         for (iter = tag_refs; iter != NULL; iter = iter->next) {
             xmlNode *tag_ref = iter->data;
 
-            free_xml(tag_ref);
+            pcmk__xml_free(tag_ref);
         }
         g_list_free(tag_refs);
     }
 
     if (!any_refs) {
-        free_xml(new_xml);
+        pcmk__xml_free(new_xml);
         new_xml = NULL;
     }
     return new_xml;
@@ -349,7 +356,7 @@ pcmk__tag_to_set(xmlNode *xml_obj, xmlNode **rsc_set, const char *attr,
     const char *id = NULL;
 
     pcmk_resource_t *rsc = NULL;
-    pcmk_tag_t *tag = NULL;
+    pcmk__idref_t *tag = NULL;
 
     *rsc_set = NULL;
 
@@ -426,9 +433,11 @@ void
 pcmk__create_internal_constraints(pcmk_scheduler_t *scheduler)
 {
     crm_trace("Create internal constraints");
-    for (GList *iter = scheduler->resources; iter != NULL; iter = iter->next) {
+    for (GList *iter = scheduler->priv->resources;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *rsc = (pcmk_resource_t *) iter->data;
 
-        rsc->cmds->internal_constraints(rsc);
+        rsc->priv->cmds->internal_constraints(rsc);
     }
 }
