@@ -20,41 +20,6 @@
 
 #include "libpacemaker_private.h"
 
-static int
-get_node_score(const char *rule, const char *score, bool raw,
-               pcmk_node_t *node, pcmk_resource_t *rsc)
-{
-    int score_f = 0;
-
-    if (score == NULL) {
-        pcmk__config_warn("Rule %s: no score specified (assuming 0)", rule);
-
-    } else if (raw) {
-        score_f = char2score(score);
-
-    } else {
-        const char *target = NULL;
-        const char *attr_score = NULL;
-
-        target = g_hash_table_lookup(rsc->meta,
-                                     PCMK_META_CONTAINER_ATTRIBUTE_TARGET);
-
-        attr_score = pcmk__node_attr(node, score, target,
-                                     pcmk__rsc_node_current);
-        if (attr_score == NULL) {
-            crm_debug("Rule %s: %s did not have a value for %s",
-                      rule, pcmk__node_name(node), score);
-            score_f = -PCMK_SCORE_INFINITY;
-
-        } else {
-            crm_debug("Rule %s: %s had value %s for %s",
-                      rule, pcmk__node_name(node), attr_score, score);
-            score_f = char2score(attr_score);
-        }
-    }
-    return score_f;
-}
-
 /*!
  * \internal
  * \brief Parse a role configuration for a location constraint
@@ -92,6 +57,123 @@ parse_location_role(const char *role_spec, enum rsc_role_e *role)
     return true;
 }
 
+/*!
+ * \internal
+ * \brief Get the score attribute name (if any) used for a rule
+ *
+ * \param[in]  rule_xml    Rule XML
+ * \param[out] allocated   If the score attribute name needs to be allocated,
+ *                         this will be set to the non-const equivalent of the
+ *                         return value (should be set to NULL when passed)
+ * \param[in]  rule_input  Values used to evaluate rule criteria
+ *
+ * \return Score attribute name used for rule, or NULL if none
+ * \note The caller is responsible for freeing \p *allocated if it is non-NULL.
+ */
+static const char *
+score_attribute_name(const xmlNode *rule_xml, char **allocated,
+                     const pcmk_rule_input_t *rule_input)
+{
+    const char *name = NULL;
+
+    name = crm_element_value(rule_xml, PCMK_XA_SCORE_ATTRIBUTE);
+    if (name == NULL) {
+        return NULL;
+    }
+
+    /* A score attribute name may use submatches extracted from a
+     * resource ID regular expression. For example, if score-attribute is
+     * "loc-\1", rsc-pattern is "ip-(.*)", and the resource ID is "ip-db", then
+     * the score attribute name is "loc-db".
+     */
+    if ((rule_input->rsc_id != NULL) && (rule_input->rsc_id_nmatches > 0)) {
+        *allocated = pcmk__replace_submatches(name, rule_input->rsc_id,
+                                              rule_input->rsc_id_submatches,
+                                              rule_input->rsc_id_nmatches);
+        if (*allocated != NULL) {
+            name = *allocated;
+        }
+    }
+    return name;
+}
+
+/*!
+ * \internal
+ * \brief Parse a score from a rule without a score attribute
+ *
+ * \param[in]  rule_xml    Rule XML
+ * \param[out] score       Where to store parsed score
+ *
+ * \return Standard Pacemaker return code
+ */
+static int
+score_from_rule(const xmlNode *rule_xml, int *score)
+{
+    int rc = pcmk_rc_ok;
+    const char *score_s = crm_element_value(rule_xml, PCMK_XA_SCORE);
+
+    if (score_s == NULL) { // Not possible with schema validation enabled
+        pcmk__config_err("Ignoring location constraint rule %s because "
+                         "neither " PCMK_XA_SCORE " nor "
+                         PCMK_XA_SCORE_ATTRIBUTE " was specified",
+                         pcmk__xe_id(rule_xml));
+        return pcmk_rc_unpack_error;
+    }
+
+    rc = pcmk_parse_score(score_s, score, 0);
+    if (rc != pcmk_rc_ok) { // Not possible with schema validation enabled
+        pcmk__config_err("Ignoring location constraint rule %s because "
+                         "'%s' is not a valid " PCMK_XA_SCORE ": %s",
+                         pcmk__xe_id(rule_xml), score_s, pcmk_rc_str(rc));
+        return pcmk_rc_unpack_error;
+    }
+
+    return pcmk_rc_ok;
+}
+
+/*!
+ * \internal
+ * \brief Get a rule score from a node attribute
+ *
+ * \param[in]  constraint_id  Location constraint ID (for logging only)
+ * \param[in]  attr_name      Name of node attribute with score
+ * \param[in]  node           Node to get attribute for
+ * \param[in]  rsc            Resource being located
+ * \param[out] score          Where to store parsed score
+ *
+ * \return Standard Pacemaker return code (pcmk_rc_ok if a valid score was
+ *         parsed, ENXIO if the node attribute was unset, and some other value
+ *         if the node attribute value was invalid)
+ */
+static int
+score_from_attr(const char *constraint_id, const char *attr_name,
+                const pcmk_node_t *node, const pcmk_resource_t *rsc, int *score)
+{
+    int rc = pcmk_rc_ok;
+    const char *target = NULL;
+    const char *score_s = NULL;
+
+    target = g_hash_table_lookup(rsc->priv->meta,
+                                 PCMK_META_CONTAINER_ATTRIBUTE_TARGET);
+    score_s = pcmk__node_attr(node, attr_name, target, pcmk__rsc_node_current);
+    if (pcmk__str_empty(score_s)) {
+        crm_info("Ignoring location %s for %s on %s "
+                 "because it has no node attribute %s",
+                 constraint_id, rsc->id, pcmk__node_name(node), attr_name);
+        return ENXIO;
+    }
+
+    rc = pcmk_parse_score(score_s, score, 0);
+    if (rc != pcmk_rc_ok) {
+        crm_warn("Ignoring location %s for node %s because node "
+                 "attribute %s value '%s' is not a valid score: %s",
+                 constraint_id, pcmk__node_name(node), attr_name,
+                 score_s, pcmk_rc_str(rc));
+        return rc;
+    }
+    return pcmk_rc_ok;
+}
+
 /*!
  * \internal
  * \brief Generate a location constraint from a rule
@@ -104,37 +186,37 @@ parse_location_role(const char *role_spec, enum rsc_role_e *role)
  * \param[in,out] rule_input     Values used to evaluate rule criteria
  *                               (node-specific values will be overwritten by
  *                               this function)
+ * \param[in]     constraint_id  ID of location constraint (for logging only)
  *
  * \return true if rule is valid, otherwise false
  */
 static bool
 generate_location_rule(pcmk_resource_t *rsc, xmlNode *rule_xml,
                        const char *discovery, crm_time_t *next_change,
-                       pcmk_rule_input_t *rule_input)
+                       pcmk_rule_input_t *rule_input, const char *constraint_id)
 {
     const char *rule_id = NULL;
-    const char *score = NULL;
+    const char *score_attr = NULL;
     const char *boolean = NULL;
     const char *role_spec = NULL;
 
     GList *iter = NULL;
-
-    bool raw_score = true;
-    bool score_allocated = false;
-
+    int score = 0;
+    char *local_score_attr = NULL;
     pcmk__location_t *location_rule = NULL;
     enum rsc_role_e role = pcmk_role_unknown;
     enum pcmk__combine combine = pcmk__combine_unknown;
 
-    rule_xml = expand_idref(rule_xml, rsc->cluster->input);
+    rule_xml = pcmk__xe_resolve_idref(rule_xml, rsc->priv->scheduler->input);
     if (rule_xml == NULL) {
         return false; // Error already logged
     }
 
     rule_id = crm_element_value(rule_xml, PCMK_XA_ID);
     if (rule_id == NULL) {
-        pcmk__config_err("Ignoring " PCMK_XE_RULE " without " PCMK_XA_ID
-                         " in location constraint");
+        pcmk__config_err("Ignoring location constraint '%s' because its rule "
+                         "has no " PCMK_XA_ID,
+                         constraint_id);
         return false;
     }
 
@@ -144,37 +226,33 @@ generate_location_rule(pcmk_resource_t *rsc, xmlNode *rule_xml,
     if (parse_location_role(role_spec, &role)) {
         crm_trace("Setting rule %s role filter to %s", rule_id, role_spec);
     } else {
-        pcmk__config_err("Ignoring rule %s: Invalid " PCMK_XA_ROLE " '%s'",
-                         rule_id, role_spec);
+        pcmk__config_err("Ignoring location constraint '%s' because rule '%s' "
+                         "has invalid " PCMK_XA_ROLE " '%s'",
+                         constraint_id, rule_id, role_spec);
         return false;
     }
 
-    crm_trace("Processing location constraint rule %s", rule_id);
-
-    score = crm_element_value(rule_xml, PCMK_XA_SCORE);
-    if (score == NULL) {
-        score = crm_element_value(rule_xml, PCMK_XA_SCORE_ATTRIBUTE);
-        if (score != NULL) {
-            raw_score = false;
-        }
-    }
-
     combine = pcmk__parse_combine(boolean);
     switch (combine) {
         case pcmk__combine_and:
         case pcmk__combine_or:
             break;
 
-        default:
-            /* @COMPAT When we can break behavioral backward compatibility,
-             * return false
-             */
-            pcmk__config_warn("Location constraint rule %s has invalid "
-                              PCMK_XA_BOOLEAN_OP " value '%s', using default "
-                              "'" PCMK_VALUE_AND "'",
-                              rule_id, boolean);
-            combine = pcmk__combine_and;
-            break;
+        default: // Not possible with schema validation enabled
+            pcmk__config_err("Ignoring location constraint '%s' because rule "
+                             "'%s' has invalid " PCMK_XA_BOOLEAN_OP " '%s'",
+                             constraint_id, rule_id, boolean);
+            return false;
+    }
+
+    /* Users may configure the rule with either a score or the name of a
+     * node attribute whose value should be used as the constraint score for
+     * that node.
+     */
+    score_attr = score_attribute_name(rule_xml, &local_score_attr, rule_input);
+    if ((score_attr == NULL)
+        && (score_from_rule(rule_xml, &score) != pcmk_rc_ok)) {
+        return false;
     }
 
     location_rule = pcmk__new_location(rule_id, rsc, 0, discovery, NULL);
@@ -182,40 +260,37 @@ generate_location_rule(pcmk_resource_t *rsc, xmlNode *rule_xml,
 
     location_rule->role_filter = role;
 
-    if ((rule_input->rsc_id != NULL) && (rule_input->rsc_id_nmatches > 0)
-        && !raw_score) {
+    for (iter = rsc->priv->scheduler->nodes;
+         iter != NULL; iter = iter->next) {
 
-        char *result = pcmk__replace_submatches(score, rule_input->rsc_id,
-                                                rule_input->rsc_id_submatches,
-                                                rule_input->rsc_id_nmatches);
-
-        if (result != NULL) {
-            score = result;
-            score_allocated = true;
-        }
-    }
-
-    for (iter = rsc->cluster->nodes; iter != NULL; iter = iter->next) {
         pcmk_node_t *node = iter->data;
+        pcmk_node_t *local = NULL;
 
-        rule_input->node_attrs = node->details->attrs;
-        rule_input->rsc_params = pe_rsc_params(rsc, node, rsc->cluster);
+        rule_input->node_attrs = node->priv->attrs;
+        rule_input->rsc_params = pe_rsc_params(rsc, node,
+                                               rsc->priv->scheduler);
 
         if (pcmk_evaluate_rule(rule_xml, rule_input,
-                               next_change) == pcmk_rc_ok) {
-            pcmk_node_t *local = pe__copy_node(node);
-
-            location_rule->nodes = g_list_prepend(location_rule->nodes, local);
-            local->weight = get_node_score(rule_id, score, raw_score, node,
-                                           rsc);
-            crm_trace("%s has score %s after %s", pcmk__node_name(node),
-                      pcmk_readable_score(local->weight), rule_id);
+                               next_change) != pcmk_rc_ok) {
+            continue;
         }
+
+        if ((score_attr != NULL)
+            && (score_from_attr(constraint_id, score_attr, node, rsc,
+                                &score) != pcmk_rc_ok)) {
+            continue; // Message already logged
+        }
+
+        local = pe__copy_node(node);
+        location_rule->nodes = g_list_prepend(location_rule->nodes, local);
+        local->assign->score = score;
+        pcmk__rsc_trace(rsc,
+                        "Location %s score for %s on %s is %s via rule %s",
+                        constraint_id, rsc->id, pcmk__node_name(node),
+                        pcmk_readable_score(score), rule_id);
     }
 
-    if (score_allocated) {
-        free((char *)score);
-    }
+    free(local_score_attr);
 
     if (location_rule->nodes == NULL) {
         crm_trace("No matching nodes for location constraint rule %s", rule_id);
@@ -234,7 +309,7 @@ unpack_rsc_location(xmlNode *xml_obj, pcmk_resource_t *rsc,
 {
     const char *rsc_id = crm_element_value(xml_obj, PCMK_XA_RSC);
     const char *id = crm_element_value(xml_obj, PCMK_XA_ID);
-    const char *node = crm_element_value(xml_obj, PCMK_XE_NODE);
+    const char *node = crm_element_value(xml_obj, PCMK_XA_NODE);
     const char *discovery = crm_element_value(xml_obj,
                                               PCMK_XA_RESOURCE_DISCOVERY);
 
@@ -249,8 +324,9 @@ unpack_rsc_location(xmlNode *xml_obj, pcmk_resource_t *rsc,
     }
 
     if ((node != NULL) && (score != NULL)) {
-        int score_i = char2score(score);
-        pcmk_node_t *match = pcmk_find_node(rsc->cluster, node);
+        int score_i = 0;
+        int rc = pcmk_rc_ok;
+        pcmk_node_t *match = pcmk_find_node(rsc->priv->scheduler, node);
         enum rsc_role_e role = pcmk_role_unknown;
         pcmk__location_t *location = NULL;
 
@@ -261,19 +337,24 @@ unpack_rsc_location(xmlNode *xml_obj, pcmk_resource_t *rsc,
             return;
         }
 
+        rc = pcmk_parse_score(score, &score_i, 0);
+        if (rc != pcmk_rc_ok) { // Not possible with schema validation enabled
+            pcmk__config_err("Ignoring location constraint %s "
+                             "because '%s' is not a valid score", id, score);
+            return;
+        }
+
         if (role_spec == NULL) {
             role_spec = crm_element_value(xml_obj, PCMK_XA_ROLE);
         }
         if (parse_location_role(role_spec, &role)) {
             crm_trace("Setting location constraint %s role filter: %s",
                       id, role_spec);
-        } else {
-            /* @COMPAT The previous behavior of creating the constraint ignoring
-             * the role is retained for now, but we should ignore the entire
-             * constraint when we can break backward compatibility.
-             */
-            pcmk__config_err("Ignoring role in constraint %s: "
-                             "Invalid value '%s'", id, role_spec);
+        } else { // Not possible with schema validation enabled
+            pcmk__config_err("Ignoring location constraint %s "
+                             "because '%s' is not a valid " PCMK_XA_ROLE,
+                             id, role_spec);
+            return;
         }
 
         location = pcmk__new_location(id, rsc, score_i, discovery, match);
@@ -283,49 +364,19 @@ unpack_rsc_location(xmlNode *xml_obj, pcmk_resource_t *rsc,
         location->role_filter = role;
 
     } else {
-        bool empty = true;
         crm_time_t *next_change = crm_time_new_undefined();
+        xmlNode *rule_xml = pcmk__xe_first_child(xml_obj, PCMK_XE_RULE, NULL,
+                                                 NULL);
         pcmk_rule_input_t rule_input = {
-            .now = rsc->cluster->now,
-            .rsc_meta = rsc->meta,
+            .now = rsc->priv->scheduler->priv->now,
+            .rsc_meta = rsc->priv->meta,
             .rsc_id = rsc_id_match,
             .rsc_id_submatches = rsc_id_submatches,
             .rsc_id_nmatches = rsc_id_nmatches,
         };
 
-        /* This loop is logically parallel to pcmk__evaluate_rules(), except
-         * instead of checking whether any rule is active, we set up location
-         * constraints for each active rule.
-         *
-         * @COMPAT When we can break backward compatibility, limit location
-         * constraints to a single rule, for consistency with other contexts.
-         * Since a rule may contain other rules, this does not prohibit any
-         * existing use cases.
-         */
-        for (xmlNode *rule_xml = pcmk__xe_first_child(xml_obj, PCMK_XE_RULE,
-                                                      NULL, NULL);
-             rule_xml != NULL; rule_xml = pcmk__xe_next_same(rule_xml)) {
-
-            if (generate_location_rule(rsc, rule_xml, discovery, next_change,
-                                       &rule_input)) {
-                if (empty) {
-                    empty = false;
-                    continue;
-                }
-                pcmk__warn_once(pcmk__wo_location_rules,
-                                "Support for multiple " PCMK_XE_RULE
-                                " elements in a location constraint is "
-                                "deprecated and will be removed in a future "
-                                "release (use a single new rule combining the "
-                                "previous rules with " PCMK_XA_BOOLEAN_OP
-                                " set to '" PCMK_VALUE_OR "' instead)");
-            }
-        }
-
-        if (empty) {
-            pcmk__config_err("Ignoring constraint '%s' because it contains "
-                             "no valid rules", id);
-        }
+        generate_location_rule(rsc, rule_xml, discovery, next_change,
+                               &rule_input, id);
 
         /* If there is a point in the future when the evaluation of a rule will
          * change, make sure the scheduler is re-run by that time.
@@ -333,7 +384,7 @@ unpack_rsc_location(xmlNode *xml_obj, pcmk_resource_t *rsc,
         if (crm_time_is_defined(next_change)) {
             time_t t = (time_t) crm_time_get_seconds_since_epoch(next_change);
 
-            pe__update_recheck_time(t, rsc->cluster,
+            pe__update_recheck_time(t, rsc->priv->scheduler,
                                     "location rule evaluation");
         }
         crm_time_free(next_change);
@@ -349,7 +400,7 @@ unpack_simple_location(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
     if (value) {
         pcmk_resource_t *rsc;
 
-        rsc = pcmk__find_constraint_resource(scheduler->resources, value);
+        rsc = pcmk__find_constraint_resource(scheduler->priv->resources, value);
         unpack_rsc_location(xml_obj, rsc, NULL, NULL, NULL, 0, NULL);
     }
 
@@ -370,8 +421,8 @@ unpack_simple_location(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
             return;
         }
 
-        for (GList *iter = scheduler->resources; iter != NULL;
-             iter = iter->next) {
+        for (GList *iter = scheduler->priv->resources;
+             iter != NULL; iter = iter->next) {
 
             pcmk_resource_t *r = iter->data;
             int nregs = 0;
@@ -417,7 +468,7 @@ unpack_location_tags(xmlNode *xml_obj, xmlNode **expanded_xml,
     const char *rsc_id = NULL;
     const char *state = NULL;
     pcmk_resource_t *rsc = NULL;
-    pcmk_tag_t *tag = NULL;
+    pcmk__idref_t *tag = NULL;
     xmlNode *rsc_set = NULL;
 
     *expanded_xml = NULL;
@@ -462,7 +513,7 @@ unpack_location_tags(xmlNode *xml_obj, xmlNode **expanded_xml,
      */
     if (!pcmk__tag_to_set(*expanded_xml, &rsc_set, PCMK_XA_RSC,
                           false, scheduler)) {
-        free_xml(*expanded_xml);
+        pcmk__xml_free(*expanded_xml);
         *expanded_xml = NULL;
         return pcmk_rc_unpack_error;
     }
@@ -479,7 +530,7 @@ unpack_location_tags(xmlNode *xml_obj, xmlNode **expanded_xml,
 
     } else {
         // No sets
-        free_xml(*expanded_xml);
+        pcmk__xml_free(*expanded_xml);
         *expanded_xml = NULL;
     }
 
@@ -511,9 +562,10 @@ unpack_location_set(xmlNode *location, xmlNode *set,
     local_score = crm_element_value(set, PCMK_XA_SCORE);
 
     for (xml_rsc = pcmk__xe_first_child(set, PCMK_XE_RESOURCE_REF, NULL, NULL);
-         xml_rsc != NULL; xml_rsc = pcmk__xe_next_same(xml_rsc)) {
+         xml_rsc != NULL;
+         xml_rsc = pcmk__xe_next(xml_rsc, PCMK_XE_RESOURCE_REF)) {
 
-        resource = pcmk__find_constraint_resource(scheduler->resources,
+        resource = pcmk__find_constraint_resource(scheduler->priv->resources,
                                                   pcmk__xe_id(xml_rsc));
         if (resource == NULL) {
             pcmk__config_err("%s: No resource found for %s",
@@ -547,22 +599,22 @@ pcmk__unpack_location(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
     }
 
     for (set = pcmk__xe_first_child(xml_obj, PCMK_XE_RESOURCE_SET, NULL, NULL);
-         set != NULL; set = pcmk__xe_next_same(set)) {
+         set != NULL; set = pcmk__xe_next(set, PCMK_XE_RESOURCE_SET)) {
 
         any_sets = true;
-        set = expand_idref(set, scheduler->input);
+        set = pcmk__xe_resolve_idref(set, scheduler->input);
         if ((set == NULL) // Configuration error, message already logged
             || (unpack_location_set(xml_obj, set, scheduler) != pcmk_rc_ok)) {
 
             if (expanded_xml) {
-                free_xml(expanded_xml);
+                pcmk__xml_free(expanded_xml);
             }
             return;
         }
     }
 
     if (expanded_xml) {
-        free_xml(expanded_xml);
+        pcmk__xml_free(expanded_xml);
         xml_obj = orig_xml;
     }
 
@@ -578,7 +630,7 @@ pcmk__unpack_location(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
  * \param[in]     id             XML ID of location constraint
  * \param[in,out] rsc            Resource in location constraint
  * \param[in]     node_score     Constraint score
- * \param[in]     discover_mode  Resource discovery option for constraint
+ * \param[in]     probe_mode     When resource should be probed on node
  * \param[in]     node           Node in constraint (or NULL if rule-based)
  *
  * \return Newly allocated location constraint on success, otherwise NULL
@@ -587,7 +639,7 @@ pcmk__unpack_location(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
  */
 pcmk__location_t *
 pcmk__new_location(const char *id, pcmk_resource_t *rsc,
-                   int node_score, const char *discover_mode, pcmk_node_t *node)
+                   int node_score, const char *probe_mode, pcmk_node_t *node)
 {
     pcmk__location_t *new_con = NULL;
 
@@ -609,34 +661,35 @@ pcmk__new_location(const char *id, pcmk_resource_t *rsc,
     new_con->nodes = NULL;
     new_con->role_filter = pcmk_role_unknown;
 
-    if (pcmk__str_eq(discover_mode, PCMK_VALUE_ALWAYS,
+    if (pcmk__str_eq(probe_mode, PCMK_VALUE_ALWAYS,
                      pcmk__str_null_matches|pcmk__str_casei)) {
-        new_con->discover_mode = pcmk_probe_always;
+        new_con->probe_mode = pcmk__probe_always;
 
-    } else if (pcmk__str_eq(discover_mode, PCMK_VALUE_NEVER,
-                            pcmk__str_casei)) {
-        new_con->discover_mode = pcmk_probe_never;
+    } else if (pcmk__str_eq(probe_mode, PCMK_VALUE_NEVER, pcmk__str_casei)) {
+        new_con->probe_mode = pcmk__probe_never;
 
-    } else if (pcmk__str_eq(discover_mode, PCMK_VALUE_EXCLUSIVE,
+    } else if (pcmk__str_eq(probe_mode, PCMK_VALUE_EXCLUSIVE,
                             pcmk__str_casei)) {
-        new_con->discover_mode = pcmk_probe_exclusive;
-        rsc->exclusive_discover = TRUE;
+        new_con->probe_mode = pcmk__probe_exclusive;
+        pcmk__set_rsc_flags(rsc, pcmk__rsc_exclusive_probes);
 
     } else {
         pcmk__config_err("Invalid " PCMK_XA_RESOURCE_DISCOVERY " value %s "
-                         "in location constraint", discover_mode);
+                         "in location constraint", probe_mode);
     }
 
     if (node != NULL) {
         pcmk_node_t *copy = pe__copy_node(node);
 
-        copy->weight = node_score;
+        copy->assign->score = node_score;
         new_con->nodes = g_list_prepend(NULL, copy);
     }
 
-    rsc->cluster->placement_constraints = g_list_prepend(
-        rsc->cluster->placement_constraints, new_con);
-    rsc->rsc_location = g_list_prepend(rsc->rsc_location, new_con);
+    rsc->priv->scheduler->priv->location_constraints =
+        g_list_prepend(rsc->priv->scheduler->priv->location_constraints,
+                       new_con);
+    rsc->priv->location_constraints =
+        g_list_prepend(rsc->priv->location_constraints, new_con);
 
     return new_con;
 }
@@ -650,11 +703,11 @@ pcmk__new_location(const char *id, pcmk_resource_t *rsc,
 void
 pcmk__apply_locations(pcmk_scheduler_t *scheduler)
 {
-    for (GList *iter = scheduler->placement_constraints;
+    for (GList *iter = scheduler->priv->location_constraints;
          iter != NULL; iter = iter->next) {
         pcmk__location_t *location = iter->data;
 
-        location->rsc->cmds->apply_location(location->rsc, location);
+        location->rsc->priv->cmds->apply_location(location->rsc, location);
     }
 }
 
@@ -673,14 +726,15 @@ pcmk__apply_location(pcmk_resource_t *rsc, pcmk__location_t *location)
 {
     bool need_role = false;
 
-    CRM_ASSERT((rsc != NULL) && (location != NULL));
+    pcmk__assert((rsc != NULL) && (location != NULL));
 
     // If a role was specified, ensure constraint is applicable
     need_role = (location->role_filter > pcmk_role_unknown);
-    if (need_role && (location->role_filter != rsc->next_role)) {
+    if (need_role && (location->role_filter != rsc->priv->next_role)) {
         pcmk__rsc_trace(rsc,
                         "Not applying %s to %s because role will be %s not %s",
-                        location->id, rsc->id, pcmk_role_text(rsc->next_role),
+                        location->id, rsc->id,
+                        pcmk_role_text(rsc->priv->next_role),
                         pcmk_role_text(location->role_filter));
         return;
     }
@@ -693,8 +747,10 @@ pcmk__apply_location(pcmk_resource_t *rsc, pcmk__location_t *location)
 
     for (GList *iter = location->nodes; iter != NULL; iter = iter->next) {
         pcmk_node_t *node = iter->data;
-        pcmk_node_t *allowed_node = g_hash_table_lookup(rsc->allowed_nodes,
-                                                        node->details->id);
+        pcmk_node_t *allowed_node = NULL;
+
+        allowed_node = g_hash_table_lookup(rsc->priv->allowed_nodes,
+                                           node->priv->id);
 
         pcmk__rsc_trace(rsc, "Applying %s%s%s to %s score on %s: %c %s",
                         location->id,
@@ -702,24 +758,25 @@ pcmk__apply_location(pcmk_resource_t *rsc, pcmk__location_t *location)
                         (need_role? pcmk_role_text(location->role_filter) : ""),
                         rsc->id, pcmk__node_name(node),
                         ((allowed_node == NULL)? '=' : '+'),
-                        pcmk_readable_score(node->weight));
+                        pcmk_readable_score(node->assign->score));
 
         if (allowed_node == NULL) {
             allowed_node = pe__copy_node(node);
-            g_hash_table_insert(rsc->allowed_nodes,
-                                (gpointer) allowed_node->details->id,
+            g_hash_table_insert(rsc->priv->allowed_nodes,
+                                (gpointer) allowed_node->priv->id,
                                 allowed_node);
         } else {
-            allowed_node->weight = pcmk__add_scores(allowed_node->weight,
-                                                    node->weight);
+            allowed_node->assign->score =
+                pcmk__add_scores(allowed_node->assign->score,
+                                 node->assign->score);
         }
 
-        if (allowed_node->rsc_discover_mode < location->discover_mode) {
-            if (location->discover_mode == pcmk_probe_exclusive) {
-                rsc->exclusive_discover = TRUE;
+        if (allowed_node->assign->probe_mode < location->probe_mode) {
+            if (location->probe_mode == pcmk__probe_exclusive) {
+                pcmk__set_rsc_flags(rsc, pcmk__rsc_exclusive_probes);
             }
             /* exclusive > never > always... always is default */
-            allowed_node->rsc_discover_mode = location->discover_mode;
+            allowed_node->assign->probe_mode = location->probe_mode;
         }
     }
 }
