@@ -32,19 +32,21 @@ pcmk__node_available(const pcmk_node_t *node, bool consider_score,
 {
     if ((node == NULL) || (node->details == NULL) || !node->details->online
             || node->details->shutdown || node->details->unclean
-            || node->details->standby || node->details->maintenance) {
+            || pcmk_is_set(node->priv->flags, pcmk__node_standby)
+            || node->details->maintenance) {
         return false;
     }
 
-    if (consider_score && (node->weight < 0)) {
+    if (consider_score && (node->assign->score < 0)) {
         return false;
     }
 
     // @TODO Go through all callers to see which should set consider_guest
     if (consider_guest && pcmk__is_guest_or_bundle_node(node)) {
-        pcmk_resource_t *guest = node->details->remote_rsc->container;
+        pcmk_resource_t *guest = node->priv->remote->priv->launcher;
 
-        if (guest->fns->location(guest, NULL, FALSE) == NULL) {
+        if (guest->priv->fns->location(guest, NULL,
+                                       pcmk__rsc_node_assigned) == NULL) {
             return false;
         }
     }
@@ -54,11 +56,12 @@ pcmk__node_available(const pcmk_node_t *node, bool consider_score,
 
 /*!
  * \internal
- * \brief Copy a hash table of node objects
+ * \brief Create a hash table with copies of another table's nodes
  *
  * \param[in] nodes  Hash table to copy
  *
- * \return New copy of nodes (or NULL if nodes is NULL)
+ * \return New table with copies of nodes in \p nodes, or \c NULL if \p nodes is
+ *         \c NULL
  */
 GHashTable *
 pcmk__copy_node_table(GHashTable *nodes)
@@ -70,12 +73,12 @@ pcmk__copy_node_table(GHashTable *nodes)
     if (nodes == NULL) {
         return NULL;
     }
-    new_table = pcmk__strkey_table(NULL, free);
+    new_table = pcmk__strkey_table(NULL, pcmk__free_node_copy);
     g_hash_table_iter_init(&iter, nodes);
     while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &node)) {
         pcmk_node_t *new_node = pe__copy_node(node);
 
-        g_hash_table_insert(new_table, (gpointer) new_node->details->id,
+        g_hash_table_insert(new_table, (gpointer) new_node->priv->id,
                             new_node);
     }
     return new_table;
@@ -113,16 +116,18 @@ destroy_node_tables(gpointer data)
 void
 pcmk__copy_node_tables(const pcmk_resource_t *rsc, GHashTable **copy)
 {
-    CRM_ASSERT((rsc != NULL) && (copy != NULL));
+    pcmk__assert((rsc != NULL) && (copy != NULL));
 
     if (*copy == NULL) {
         *copy = pcmk__strkey_table(NULL, destroy_node_tables);
     }
 
     g_hash_table_insert(*copy, rsc->id,
-                        pcmk__copy_node_table(rsc->allowed_nodes));
+                        pcmk__copy_node_table(rsc->priv->allowed_nodes));
+
+    for (const GList *iter = rsc->priv->children;
+         iter != NULL; iter = iter->next) {
 
-    for (const GList *iter = rsc->children; iter != NULL; iter = iter->next) {
         pcmk__copy_node_tables((const pcmk_resource_t *) iter->data, copy);
     }
 }
@@ -144,15 +149,17 @@ pcmk__copy_node_tables(const pcmk_resource_t *rsc, GHashTable **copy)
 void
 pcmk__restore_node_tables(pcmk_resource_t *rsc, GHashTable *backup)
 {
-    CRM_ASSERT((rsc != NULL) && (backup != NULL));
+    pcmk__assert((rsc != NULL) && (backup != NULL));
 
-    g_hash_table_destroy(rsc->allowed_nodes);
+    g_hash_table_destroy(rsc->priv->allowed_nodes);
 
     // Copy to avoid danger with multiple restores
-    rsc->allowed_nodes = g_hash_table_lookup(backup, rsc->id);
-    rsc->allowed_nodes = pcmk__copy_node_table(rsc->allowed_nodes);
+    rsc->priv->allowed_nodes =
+        pcmk__copy_node_table(g_hash_table_lookup(backup, rsc->id));
+
+    for (GList *iter = rsc->priv->children;
+         iter != NULL; iter = iter->next) {
 
-    for (GList *iter = rsc->children; iter != NULL; iter = iter->next) {
         pcmk__restore_node_tables((pcmk_resource_t *) iter->data, backup);
     }
 }
@@ -177,7 +184,7 @@ pcmk__copy_node_list(const GList *list, bool reset)
 
         new_node = pe__copy_node(this_node);
         if (reset) {
-            new_node->weight = 0;
+            new_node->assign->score = 0;
         }
         result = g_list_prepend(result, new_node);
     }
@@ -220,10 +227,10 @@ compare_nodes(gconstpointer a, gconstpointer b, gpointer data)
     // Compare node scores
 
     if (pcmk__node_available(node1, false, false)) {
-        node1_score = node1->weight;
+        node1_score = node1->assign->score;
     }
     if (pcmk__node_available(node2, false, false)) {
-        node2_score = node2->weight;
+        node2_score = node2->assign->score;
     }
 
     if (node1_score > node2_score) {
@@ -242,12 +249,12 @@ compare_nodes(gconstpointer a, gconstpointer b, gpointer data)
 
     // If appropriate, compare node utilization
 
-    if (pcmk__str_eq(node1->details->data_set->placement_strategy,
+    if (pcmk__str_eq(node1->priv->scheduler->priv->placement_strategy,
                      PCMK_VALUE_MINIMAL, pcmk__str_casei)) {
         goto equal;
     }
 
-    if (pcmk__str_eq(node1->details->data_set->placement_strategy,
+    if (pcmk__str_eq(node1->priv->scheduler->priv->placement_strategy,
                      PCMK_VALUE_BALANCED, pcmk__str_casei)) {
 
         result = pcmk__compare_node_capacities(node1, node2);
@@ -265,16 +272,16 @@ compare_nodes(gconstpointer a, gconstpointer b, gpointer data)
 
     // Compare number of resources already assigned to node
 
-    if (node1->details->num_resources < node2->details->num_resources) {
+    if (node1->priv->num_resources < node2->priv->num_resources) {
         crm_trace("%s before %s (%d resources < %d)",
                   pcmk__node_name(node1), pcmk__node_name(node2),
-                  node1->details->num_resources, node2->details->num_resources);
+                  node1->priv->num_resources, node2->priv->num_resources);
         return -1;
 
-    } else if (node1->details->num_resources > node2->details->num_resources) {
+    } else if (node1->priv->num_resources > node2->priv->num_resources) {
         crm_trace("%s after %s (%d resources > %d)",
                   pcmk__node_name(node1), pcmk__node_name(node2),
-                  node1->details->num_resources, node2->details->num_resources);
+                  node1->priv->num_resources, node2->priv->num_resources);
         return 1;
     }
 
@@ -294,7 +301,7 @@ compare_nodes(gconstpointer a, gconstpointer b, gpointer data)
 
     // If all else is equal, prefer node with lowest-sorting name
 equal:
-    result = strcmp(node1->details->uname, node2->details->uname);
+    result = strcmp(node1->priv->name, node2->priv->name);
     if (result < 0) {
         crm_trace("%s before %s (name)",
                   pcmk__node_name(node1), pcmk__node_name(node2));
@@ -363,7 +370,7 @@ pcmk__apply_node_health(pcmk_scheduler_t *scheduler)
     int base_health = 0;
     enum pcmk__health_strategy strategy;
     const char *strategy_str =
-        pcmk__cluster_option(scheduler->config_hash,
+        pcmk__cluster_option(scheduler->priv->options,
                              PCMK_OPT_NODE_HEALTH_STRATEGY);
 
     strategy = pcmk__parse_health_strategy(strategy_str);
@@ -374,7 +381,7 @@ pcmk__apply_node_health(pcmk_scheduler_t *scheduler)
 
     // The progressive strategy can use a base health score
     if (strategy == pcmk__health_strategy_progressive) {
-        base_health = pe__health_score(PCMK_OPT_NODE_HEALTH_BASE, scheduler);
+        base_health = pcmk__health_score(PCMK_OPT_NODE_HEALTH_BASE, scheduler);
     }
 
     for (GList *iter = scheduler->nodes; iter != NULL; iter = iter->next) {
@@ -389,7 +396,7 @@ pcmk__apply_node_health(pcmk_scheduler_t *scheduler)
                  pcmk__node_name(node), health);
 
         // Use node health as a location score for each resource on the node
-        for (GList *r = scheduler->resources; r != NULL; r = r->next) {
+        for (GList *r = scheduler->priv->resources; r != NULL; r = r->next) {
             pcmk_resource_t *rsc = (pcmk_resource_t *) r->data;
 
             bool constrain = true;
@@ -398,8 +405,8 @@ pcmk__apply_node_health(pcmk_scheduler_t *scheduler)
                 /* Negative health scores do not apply to resources with
                  * PCMK_META_ALLOW_UNHEALTHY_NODES=true.
                  */
-                constrain = !crm_is_true(g_hash_table_lookup(rsc->meta,
-                                         PCMK_META_ALLOW_UNHEALTHY_NODES));
+                constrain = !crm_is_true(g_hash_table_lookup(rsc->priv->meta,
+                                                             PCMK_META_ALLOW_UNHEALTHY_NODES));
             }
             if (constrain) {
                 pcmk__new_location(strategy_str, rsc, health, NULL, node);
@@ -428,10 +435,12 @@ pcmk__top_allowed_node(const pcmk_resource_t *rsc, const pcmk_node_t *node)
 
     if ((rsc == NULL) || (node == NULL)) {
         return NULL;
-    } else if (rsc->parent == NULL) {
-        allowed_nodes = rsc->allowed_nodes;
-    } else {
-        allowed_nodes = rsc->parent->allowed_nodes;
     }
-    return g_hash_table_lookup(allowed_nodes, node->details->id);
+
+    if (rsc->priv->parent == NULL) {
+        allowed_nodes = rsc->priv->allowed_nodes;
+    } else {
+        allowed_nodes = rsc->priv->parent->priv->allowed_nodes;
+    }
+    return g_hash_table_lookup(allowed_nodes, node->priv->id);
 }
