@@ -30,7 +30,7 @@ enum ordering_symmetry {
 };
 
 #define EXPAND_CONSTRAINT_IDREF(__set, __rsc, __name) do {                  \
-        __rsc = pcmk__find_constraint_resource(scheduler->resources,        \
+        __rsc = pcmk__find_constraint_resource(scheduler->priv->resources,  \
                                                __name);                     \
         if (__rsc == NULL) {                                                \
             pcmk__config_err("%s: No resource found for %s", __set, __name);\
@@ -83,8 +83,9 @@ get_ordering_type(const xmlNode *xml_obj)
 
         if (score) {
             // @COMPAT deprecated informally since 1.0.7, formally since 2.0.1
-            int score_i = char2score(score);
+            int score_i = 0;
 
+            (void) pcmk_parse_score(score, &score_i, 0);
             if (score_i == 0) {
                 kind_e = pe_order_kind_optional;
             }
@@ -229,22 +230,16 @@ ordering_flags_for_kind(enum pe_order_kind kind, const char *first,
  *
  * \param[in] xml            Ordering XML
  * \param[in] resource_attr  XML attribute name for resource ID
- * \param[in] instance_attr  XML attribute name for instance number.
- *                           This option is deprecated and will be removed in a
- *                           future release.
  * \param[in] scheduler      Scheduler data
  *
  * \return Resource corresponding to \p id, or NULL if none
  */
 static pcmk_resource_t *
 get_ordering_resource(const xmlNode *xml, const char *resource_attr,
-                      const char *instance_attr,
                       const pcmk_scheduler_t *scheduler)
 {
-    // @COMPAT: instance_attr and instance_id variables deprecated since 2.1.5
     pcmk_resource_t *rsc = NULL;
     const char *rsc_id = crm_element_value(xml, resource_attr);
-    const char *instance_id = crm_element_value(xml, instance_attr);
 
     if (rsc_id == NULL) {
         pcmk__config_err("Ignoring constraint '%s' without %s",
@@ -252,33 +247,13 @@ get_ordering_resource(const xmlNode *xml, const char *resource_attr,
         return NULL;
     }
 
-    rsc = pcmk__find_constraint_resource(scheduler->resources, rsc_id);
+    rsc = pcmk__find_constraint_resource(scheduler->priv->resources, rsc_id);
     if (rsc == NULL) {
         pcmk__config_err("Ignoring constraint '%s' because resource '%s' "
                          "does not exist", pcmk__xe_id(xml), rsc_id);
         return NULL;
     }
 
-    if (instance_id != NULL) {
-        pcmk__warn_once(pcmk__wo_order_inst,
-                        "Support for " PCMK__XA_FIRST_INSTANCE " and "
-                        PCMK__XA_THEN_INSTANCE " is deprecated and will be "
-                        "removed in a future release.");
-
-        if (!pcmk__is_clone(rsc)) {
-            pcmk__config_err("Ignoring constraint '%s' because resource '%s' "
-                             "is not a clone but instance '%s' was requested",
-                             pcmk__xe_id(xml), rsc_id, instance_id);
-            return NULL;
-        }
-        rsc = find_clone_instance(rsc, instance_id);
-        if (rsc == NULL) {
-            pcmk__config_err("Ignoring constraint '%s' because resource '%s' "
-                             "does not have an instance '%s'",
-                             pcmk__xe_id(xml), rsc_id, instance_id);
-            return NULL;
-        }
-    }
     return rsc;
 }
 
@@ -301,7 +276,7 @@ get_minimum_first_instances(const pcmk_resource_t *rsc, const xmlNode *xml)
         return 0;
     }
 
-    clone_min = g_hash_table_lookup(rsc->meta, PCMK_META_CLONE_MIN);
+    clone_min = g_hash_table_lookup(rsc->priv->meta, PCMK_META_CLONE_MIN);
     if (clone_min != NULL) {
         int clone_min_int = 0;
 
@@ -348,7 +323,8 @@ clone_min_ordering(const char *id,
 {
     // Create a pseudo-action for when the minimum instances are active
     char *task = crm_strdup_printf(PCMK_ACTION_CLONE_ONE_OR_MORE ":%s", id);
-    pcmk_action_t *clone_min_met = get_pseudo_op(task, rsc_first->cluster);
+    pcmk_action_t *clone_min_met = get_pseudo_op(task,
+                                                 rsc_first->priv->scheduler);
 
     free(task);
 
@@ -356,46 +332,27 @@ clone_min_ordering(const char *id,
      * considered runnable before allowing the pseudo-action to be runnable.
      */
     clone_min_met->required_runnable_before = clone_min;
-    pcmk__set_action_flags(clone_min_met, pcmk_action_min_runnable);
 
     // Order the actions for each clone instance before the pseudo-action
-    for (GList *iter = rsc_first->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = rsc_first->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *child = iter->data;
 
         pcmk__new_ordering(child, pcmk__op_key(child->id, action_first, 0),
                            NULL, NULL, NULL, clone_min_met,
                            pcmk__ar_min_runnable
                            |pcmk__ar_first_implies_then_graphed,
-                           rsc_first->cluster);
+                           rsc_first->priv->scheduler);
     }
 
     // Order "then" action after the pseudo-action (if runnable)
     pcmk__new_ordering(NULL, NULL, clone_min_met, rsc_then,
                        pcmk__op_key(rsc_then->id, action_then, 0),
                        NULL, flags|pcmk__ar_unrunnable_first_blocks,
-                       rsc_first->cluster);
+                       rsc_first->priv->scheduler);
 }
 
-/*!
- * \internal
- * \brief Update ordering flags for restart-type=restart
- *
- * \param[in]     rsc    'Then' resource in ordering
- * \param[in]     kind   Ordering kind
- * \param[in]     flag   Ordering flag to set (when applicable)
- * \param[in,out] flags  Ordering flag set to update
- *
- * \compat The \c PCMK__META_RESTART_TYPE resource meta-attribute is deprecated.
- *         Eventually, it will be removed, and \c pe_restart_ignore will be the
- *         only behavior, at which time this can just be removed entirely.
- */
-#define handle_restart_type(rsc, kind, flag, flags) do {        \
-        if (((kind) == pe_order_kind_optional)                  \
-            && ((rsc)->restart_type == pe_restart_restart)) {   \
-            pcmk__set_relation_flags((flags), (flag));          \
-        }                                                       \
-    } while (0)
-
 /*!
  * \internal
  * \brief Create new ordering for inverse of symmetric constraint
@@ -421,7 +378,6 @@ inverse_ordering(const char *id, enum pe_order_kind kind,
         uint32_t flags = ordering_flags_for_kind(kind, action_first,
                                                  ordering_symmetric_inverse);
 
-        handle_restart_type(rsc_then, kind, pcmk__ar_then_implies_first, flags);
         pcmk__order_resource_actions(rsc_then, action_then, rsc_first,
                                      action_first, flags);
     }
@@ -450,14 +406,12 @@ unpack_simple_rsc_order(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
         return;
     }
 
-    rsc_first = get_ordering_resource(xml_obj, PCMK_XA_FIRST,
-                                      PCMK__XA_FIRST_INSTANCE, scheduler);
+    rsc_first = get_ordering_resource(xml_obj, PCMK_XA_FIRST, scheduler);
     if (rsc_first == NULL) {
         return;
     }
 
-    rsc_then = get_ordering_resource(xml_obj, PCMK_XA_THEN,
-                                     PCMK__XA_THEN_INSTANCE, scheduler);
+    rsc_then = get_ordering_resource(xml_obj, PCMK_XA_THEN, scheduler);
     if (rsc_then == NULL) {
         return;
     }
@@ -477,8 +431,6 @@ unpack_simple_rsc_order(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
     symmetry = get_ordering_symmetry(xml_obj, kind, NULL);
     flags = ordering_flags_for_kind(kind, action_first, symmetry);
 
-    handle_restart_type(rsc_then, kind, pcmk__ar_first_implies_then, flags);
-
     /* If there is a minimum number of instances that must be runnable before
      * the 'then' action is runnable, we use a pseudo-action for convenience:
      * minimum number of clone instances have runnable actions ->
@@ -549,7 +501,7 @@ pcmk__new_ordering(pcmk_resource_t *first_rsc, char *first_action_task,
 
     order = pcmk__assert_alloc(1, sizeof(pcmk__action_relation_t));
 
-    order->id = sched->order_id++;
+    order->id = sched->priv->next_ordering_id++;
     order->flags = flags;
     order->rsc1 = first_rsc;
     order->rsc2 = then_rsc;
@@ -575,12 +527,12 @@ pcmk__new_ordering(pcmk_resource_t *first_rsc, char *first_action_task,
     }
 
     pcmk__rsc_trace(first_rsc, "Created ordering %d for %s then %s",
-                    (sched->order_id - 1),
+                    (sched->priv->next_ordering_id - 1),
                     pcmk__s(order->task1, "an underspecified action"),
                     pcmk__s(order->task2, "an underspecified action"));
 
-    sched->ordering_constraints = g_list_prepend(sched->ordering_constraints,
-                                                 order);
+    sched->priv->ordering_constraints =
+        g_list_prepend(sched->priv->ordering_constraints, order);
     pcmk__order_migration_equivalents(order);
 }
 
@@ -636,7 +588,8 @@ unpack_order_set(const xmlNode *set, enum pe_order_kind parent_kind,
     for (const xmlNode *xml_rsc = pcmk__xe_first_child(set,
                                                        PCMK_XE_RESOURCE_REF,
                                                        NULL, NULL);
-         xml_rsc != NULL; xml_rsc = pcmk__xe_next_same(xml_rsc)) {
+         xml_rsc != NULL;
+         xml_rsc = pcmk__xe_next(xml_rsc, PCMK_XE_RESOURCE_REF)) {
 
         EXPAND_CONSTRAINT_IDREF(id, resource, pcmk__xe_id(xml_rsc));
         resources = g_list_append(resources, resource);
@@ -771,11 +724,12 @@ order_rsc_sets(const char *id, const xmlNode *set1, const xmlNode *set2,
         pcmk_action_t *unordered_action = get_pseudo_op(task, scheduler);
 
         free(task);
-        pcmk__set_action_flags(unordered_action, pcmk_action_min_runnable);
+        unordered_action->required_runnable_before = 1;
 
         for (xml_rsc = pcmk__xe_first_child(set1, PCMK_XE_RESOURCE_REF, NULL,
                                             NULL);
-             xml_rsc != NULL; xml_rsc = pcmk__xe_next_same(xml_rsc)) {
+             xml_rsc != NULL;
+             xml_rsc = pcmk__xe_next(xml_rsc, PCMK_XE_RESOURCE_REF)) {
 
             EXPAND_CONSTRAINT_IDREF(id, rsc_1, pcmk__xe_id(xml_rsc));
 
@@ -791,7 +745,8 @@ order_rsc_sets(const char *id, const xmlNode *set1, const xmlNode *set2,
         }
         for (xml_rsc_2 = pcmk__xe_first_child(set2, PCMK_XE_RESOURCE_REF, NULL,
                                               NULL);
-             xml_rsc_2 != NULL; xml_rsc_2 = pcmk__xe_next_same(xml_rsc_2)) {
+             xml_rsc_2 != NULL;
+             xml_rsc_2 = pcmk__xe_next(xml_rsc_2, PCMK_XE_RESOURCE_REF)) {
 
             EXPAND_CONSTRAINT_IDREF(id, rsc_2, pcmk__xe_id(xml_rsc_2));
 
@@ -823,7 +778,8 @@ order_rsc_sets(const char *id, const xmlNode *set1, const xmlNode *set2,
 
             for (xml_rsc = pcmk__xe_first_child(set1, PCMK_XE_RESOURCE_REF,
                                                 NULL, NULL);
-                 xml_rsc != NULL; xml_rsc = pcmk__xe_next_same(xml_rsc)) {
+                 xml_rsc != NULL;
+                 xml_rsc = pcmk__xe_next(xml_rsc, PCMK_XE_RESOURCE_REF)) {
 
                 rid = pcmk__xe_id(xml_rsc);
             }
@@ -838,7 +794,8 @@ order_rsc_sets(const char *id, const xmlNode *set1, const xmlNode *set2,
 
             for (xml_rsc = pcmk__xe_first_child(set2, PCMK_XE_RESOURCE_REF,
                                                 NULL, NULL);
-                 xml_rsc != NULL; xml_rsc = pcmk__xe_next_same(xml_rsc)) {
+                 xml_rsc != NULL;
+                 xml_rsc = pcmk__xe_next(xml_rsc, PCMK_XE_RESOURCE_REF)) {
 
                 rid = pcmk__xe_id(xml_rsc);
             }
@@ -860,7 +817,8 @@ order_rsc_sets(const char *id, const xmlNode *set1, const xmlNode *set2,
     } else if (rsc_1 != NULL) {
         for (xml_rsc = pcmk__xe_first_child(set2, PCMK_XE_RESOURCE_REF, NULL,
                                             NULL);
-             xml_rsc != NULL; xml_rsc = pcmk__xe_next_same(xml_rsc)) {
+             xml_rsc != NULL;
+             xml_rsc = pcmk__xe_next(xml_rsc, PCMK_XE_RESOURCE_REF)) {
 
             EXPAND_CONSTRAINT_IDREF(id, rsc_2, pcmk__xe_id(xml_rsc));
             pcmk__order_resource_actions(rsc_1, action_1, rsc_2, action_2,
@@ -870,7 +828,8 @@ order_rsc_sets(const char *id, const xmlNode *set1, const xmlNode *set2,
     } else if (rsc_2 != NULL) {
         for (xml_rsc = pcmk__xe_first_child(set1, PCMK_XE_RESOURCE_REF, NULL,
                                             NULL);
-             xml_rsc != NULL; xml_rsc = pcmk__xe_next_same(xml_rsc)) {
+             xml_rsc != NULL;
+             xml_rsc = pcmk__xe_next(xml_rsc, PCMK_XE_RESOURCE_REF)) {
 
             EXPAND_CONSTRAINT_IDREF(id, rsc_1, pcmk__xe_id(xml_rsc));
             pcmk__order_resource_actions(rsc_1, action_1, rsc_2, action_2,
@@ -880,14 +839,16 @@ order_rsc_sets(const char *id, const xmlNode *set1, const xmlNode *set2,
     } else {
         for (xml_rsc = pcmk__xe_first_child(set1, PCMK_XE_RESOURCE_REF, NULL,
                                             NULL);
-             xml_rsc != NULL; xml_rsc = pcmk__xe_next_same(xml_rsc)) {
+             xml_rsc != NULL;
+             xml_rsc = pcmk__xe_next(xml_rsc, PCMK_XE_RESOURCE_REF)) {
 
             EXPAND_CONSTRAINT_IDREF(id, rsc_1, pcmk__xe_id(xml_rsc));
 
             for (xmlNode *xml_rsc_2 = pcmk__xe_first_child(set2,
                                                            PCMK_XE_RESOURCE_REF,
                                                            NULL, NULL);
-                 xml_rsc_2 != NULL; xml_rsc_2 = pcmk__xe_next_same(xml_rsc_2)) {
+                 xml_rsc_2 != NULL;
+                 xml_rsc_2 = pcmk__xe_next(xml_rsc_2, PCMK_XE_RESOURCE_REF)) {
 
                 EXPAND_CONSTRAINT_IDREF(id, rsc_2, pcmk__xe_id(xml_rsc_2));
                 pcmk__order_resource_actions(rsc_1, action_1, rsc_2,
@@ -921,8 +882,8 @@ unpack_order_tags(xmlNode *xml_obj, xmlNode **expanded_xml,
 
     pcmk_resource_t *rsc_first = NULL;
     pcmk_resource_t *rsc_then = NULL;
-    pcmk_tag_t *tag_first = NULL;
-    pcmk_tag_t *tag_then = NULL;
+    pcmk__idref_t *tag_first = NULL;
+    pcmk__idref_t *tag_then = NULL;
 
     xmlNode *rsc_set_first = NULL;
     xmlNode *rsc_set_then = NULL;
@@ -972,7 +933,7 @@ unpack_order_tags(xmlNode *xml_obj, xmlNode **expanded_xml,
      */
     if (!pcmk__tag_to_set(*expanded_xml, &rsc_set_first, PCMK_XA_FIRST, true,
                           scheduler)) {
-        free_xml(*expanded_xml);
+        pcmk__xml_free(*expanded_xml);
         *expanded_xml = NULL;
         return pcmk_rc_unpack_error;
     }
@@ -993,7 +954,7 @@ unpack_order_tags(xmlNode *xml_obj, xmlNode **expanded_xml,
      */
     if (!pcmk__tag_to_set(*expanded_xml, &rsc_set_then, PCMK_XA_THEN, true,
                           scheduler)) {
-        free_xml(*expanded_xml);
+        pcmk__xml_free(*expanded_xml);
         *expanded_xml = NULL;
         return pcmk_rc_unpack_error;
     }
@@ -1012,7 +973,7 @@ unpack_order_tags(xmlNode *xml_obj, xmlNode **expanded_xml,
     if (any_sets) {
         crm_log_xml_trace(*expanded_xml, "Expanded " PCMK_XE_RSC_ORDER);
     } else {
-        free_xml(*expanded_xml);
+        pcmk__xml_free(*expanded_xml);
         *expanded_xml = NULL;
     }
 
@@ -1053,14 +1014,14 @@ pcmk__unpack_ordering(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
 
     // If the constraint has resource sets, unpack them
     for (set = pcmk__xe_first_child(xml_obj, PCMK_XE_RESOURCE_SET, NULL, NULL);
-         set != NULL; set = pcmk__xe_next_same(set)) {
+         set != NULL; set = pcmk__xe_next(set, PCMK_XE_RESOURCE_SET)) {
 
-        set = expand_idref(set, scheduler->input);
+        set = pcmk__xe_resolve_idref(set, scheduler->input);
         if ((set == NULL) // Configuration error, message already logged
             || (unpack_order_set(set, kind, invert, scheduler) != pcmk_rc_ok)) {
 
             if (expanded_xml != NULL) {
-                free_xml(expanded_xml);
+                pcmk__xml_free(expanded_xml);
             }
             return;
         }
@@ -1070,7 +1031,7 @@ pcmk__unpack_ordering(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
             if (order_rsc_sets(id, last, set, kind, scheduler,
                                symmetry) != pcmk_rc_ok) {
                 if (expanded_xml != NULL) {
-                    free_xml(expanded_xml);
+                    pcmk__xml_free(expanded_xml);
                 }
                 return;
             }
@@ -1079,7 +1040,7 @@ pcmk__unpack_ordering(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
                 && (order_rsc_sets(id, set, last, kind, scheduler,
                                    ordering_symmetric_inverse) != pcmk_rc_ok)) {
                 if (expanded_xml != NULL) {
-                    free_xml(expanded_xml);
+                    pcmk__xml_free(expanded_xml);
                 }
                 return;
             }
@@ -1089,7 +1050,7 @@ pcmk__unpack_ordering(xmlNode *xml_obj, pcmk_scheduler_t *scheduler)
     }
 
     if (expanded_xml) {
-        free_xml(expanded_xml);
+        pcmk__xml_free(expanded_xml);
         xml_obj = orig_xml;
     }
 
@@ -1105,7 +1066,7 @@ ordering_is_invalid(pcmk_action_t *action, pcmk__related_action_t *input)
     /* Prevent user-defined ordering constraints between resources
      * running in a guest node and the resource that defines that node.
      */
-    if (!pcmk_is_set(input->type, pcmk__ar_guest_allowed)
+    if (!pcmk_is_set(input->flags, pcmk__ar_guest_allowed)
         && (input->action->rsc != NULL)
         && pcmk__rsc_corresponds_to_guest(action->rsc, input->action->node)) {
 
@@ -1121,7 +1082,7 @@ ordering_is_invalid(pcmk_action_t *action, pcmk__related_action_t *input)
      * migrated from node2 to node1. If there would be a graph loop,
      * break the order "load_stopped_node2" -> "rscA_migrate_to node1".
      */
-    if (((uint32_t) input->type == pcmk__ar_if_on_same_node_or_target)
+    if ((input->flags == pcmk__ar_if_on_same_node_or_target)
         && (action->rsc != NULL)
         && pcmk__str_eq(action->task, PCMK_ACTION_MIGRATE_TO, pcmk__str_none)
         && pcmk__graph_has_loop(action, action, input)) {
@@ -1134,7 +1095,9 @@ ordering_is_invalid(pcmk_action_t *action, pcmk__related_action_t *input)
 void
 pcmk__disable_invalid_orderings(pcmk_scheduler_t *scheduler)
 {
-    for (GList *iter = scheduler->actions; iter != NULL; iter = iter->next) {
+    for (GList *iter = scheduler->priv->actions;
+         iter != NULL; iter = iter->next) {
+
         pcmk_action_t *action = (pcmk_action_t *) iter->data;
         pcmk__related_action_t *input = NULL;
 
@@ -1143,7 +1106,7 @@ pcmk__disable_invalid_orderings(pcmk_scheduler_t *scheduler)
 
             input = input_iter->data;
             if (ordering_is_invalid(action, input)) {
-                input->type = (enum pe_ordering) pcmk__ar_none;
+                input->flags = pcmk__ar_none;
             }
         }
     }
@@ -1159,7 +1122,7 @@ pcmk__disable_invalid_orderings(pcmk_scheduler_t *scheduler)
 void
 pcmk__order_stops_before_shutdown(pcmk_node_t *node, pcmk_action_t *shutdown_op)
 {
-    for (GList *iter = node->details->data_set->actions;
+    for (GList *iter = node->priv->scheduler->priv->actions;
          iter != NULL; iter = iter->next) {
 
         pcmk_action_t *action = (pcmk_action_t *) iter->data;
@@ -1172,7 +1135,7 @@ pcmk__order_stops_before_shutdown(pcmk_node_t *node, pcmk_action_t *shutdown_op)
 
         // Resources and nodes in maintenance mode won't be touched
 
-        if (pcmk_is_set(action->rsc->flags, pcmk_rsc_maintenance)) {
+        if (pcmk_is_set(action->rsc->flags, pcmk__rsc_maintenance)) {
             pcmk__rsc_trace(action->rsc,
                             "Not ordering %s before shutdown of %s because "
                             "resource in maintenance mode",
@@ -1192,7 +1155,7 @@ pcmk__order_stops_before_shutdown(pcmk_node_t *node, pcmk_action_t *shutdown_op)
          * we may still end up blocking)
          */
         if (!pcmk_any_flags_set(action->rsc->flags,
-                                pcmk_rsc_managed|pcmk_rsc_blocked)) {
+                                pcmk__rsc_managed|pcmk__rsc_blocked)) {
             pcmk__rsc_trace(action->rsc,
                             "Not ordering %s before shutdown of %s because "
                             "resource is unmanaged or blocked",
@@ -1202,11 +1165,11 @@ pcmk__order_stops_before_shutdown(pcmk_node_t *node, pcmk_action_t *shutdown_op)
 
         pcmk__rsc_trace(action->rsc, "Ordering %s before shutdown of %s",
                         action->uuid, pcmk__node_name(node));
-        pcmk__clear_action_flags(action, pcmk_action_optional);
+        pcmk__clear_action_flags(action, pcmk__action_optional);
         pcmk__new_ordering(action->rsc, NULL, action, NULL,
                            strdup(PCMK_ACTION_DO_SHUTDOWN), shutdown_op,
                            pcmk__ar_ordered|pcmk__ar_unrunnable_first_blocks,
-                           node->details->data_set);
+                           node->priv->scheduler);
     }
 }
 
@@ -1224,7 +1187,7 @@ static GList *
 find_actions_by_task(const pcmk_resource_t *rsc, const char *original_key)
 {
     // Search under given task key directly
-    GList *list = find_actions(rsc->actions, original_key, NULL);
+    GList *list = find_actions(rsc->priv->actions, original_key, NULL);
 
     if (list == NULL) {
         // Search again using this resource's ID
@@ -1235,7 +1198,7 @@ find_actions_by_task(const pcmk_resource_t *rsc, const char *original_key)
         CRM_CHECK(parse_op_key(original_key, NULL, &task, &interval_ms),
                   return NULL);
         key = pcmk__op_key(rsc->id, task, interval_ms);
-        list = find_actions(rsc->actions, key, NULL);
+        list = find_actions(rsc->priv->actions, key, NULL);
         free(key);
         free(task);
     }
@@ -1278,7 +1241,7 @@ order_resource_actions_after(pcmk_action_t *first_action,
     }
 
     if ((first_action != NULL) && (first_action->rsc == rsc)
-        && pcmk_is_set(first_action->flags, pcmk_action_migration_abort)) {
+        && pcmk_is_set(first_action->flags, pcmk__action_migration_abort)) {
 
         pcmk__rsc_trace(rsc,
                         "Detected dangling migration ordering (%s then %s %s)",
@@ -1302,7 +1265,7 @@ order_resource_actions_after(pcmk_action_t *first_action,
         if (first_action != NULL) {
             order_actions(first_action, then_action_iter, flags);
         } else {
-            pcmk__clear_action_flags(then_action_iter, pcmk_action_runnable);
+            pcmk__clear_action_flags(then_action_iter, pcmk__action_runnable);
             crm_warn("%s of %s is unrunnable because there is no %s of %s "
                      "to order it after", then_action_iter->task, rsc->id,
                      order->task1, order->rsc1->id);
@@ -1319,7 +1282,7 @@ rsc_order_first(pcmk_resource_t *first_rsc, pcmk__action_relation_t *order)
     pcmk_action_t *first_action = order->action1;
     pcmk_resource_t *then_rsc = order->rsc2;
 
-    CRM_ASSERT(first_rsc != NULL);
+    pcmk__assert(first_rsc != NULL);
     pcmk__rsc_trace(first_rsc, "Applying ordering constraint %d (first: %s)",
                     order->id, first_rsc->id);
 
@@ -1339,11 +1302,13 @@ rsc_order_first(pcmk_resource_t *first_rsc, pcmk__action_relation_t *order)
         char *key = NULL;
         char *op_type = NULL;
         guint interval_ms = 0;
+        enum rsc_role_e first_role;
 
         parse_op_key(order->task1, NULL, &op_type, &interval_ms);
         key = pcmk__op_key(first_rsc->id, op_type, interval_ms);
 
-        if ((first_rsc->fns->state(first_rsc, TRUE) == pcmk_role_stopped)
+        first_role = first_rsc->priv->fns->state(first_rsc, TRUE);
+        if ((first_role == pcmk_role_stopped)
             && pcmk__str_eq(op_type, PCMK_ACTION_STOP, pcmk__str_none)) {
             free(key);
             pcmk__rsc_trace(first_rsc,
@@ -1351,8 +1316,7 @@ rsc_order_first(pcmk_resource_t *first_rsc, pcmk__action_relation_t *order)
                             "not found",
                             order->id, order->task1, first_rsc->id);
 
-        } else if ((first_rsc->fns->state(first_rsc,
-                                          TRUE) == pcmk_role_unpromoted)
+        } else if ((first_role == pcmk_role_unpromoted)
                    && pcmk__str_eq(op_type, PCMK_ACTION_DEMOTE,
                                    pcmk__str_none)) {
             free(key);
@@ -1366,7 +1330,7 @@ rsc_order_first(pcmk_resource_t *first_rsc, pcmk__action_relation_t *order)
                             "Creating first (%s for %s) for constraint %d ",
                             order->task1, first_rsc->id, order->id);
             first_action = custom_action(first_rsc, key, op_type, NULL, TRUE,
-                                         first_rsc->cluster);
+                                         first_rsc->priv->scheduler);
             first_actions = g_list_prepend(NULL, first_action);
         }
 
@@ -1433,9 +1397,10 @@ pcmk__apply_orderings(pcmk_scheduler_t *sched)
      * @TODO This is brittle and should be carefully redesigned so that the
      * order of creation doesn't matter, and the reverse becomes unneeded.
      */
-    sched->ordering_constraints = g_list_reverse(sched->ordering_constraints);
+    sched->priv->ordering_constraints =
+        g_list_reverse(sched->priv->ordering_constraints);
 
-    for (GList *iter = sched->ordering_constraints;
+    for (GList *iter = sched->priv->ordering_constraints;
          iter != NULL; iter = iter->next) {
 
         pcmk__action_relation_t *order = iter->data;
@@ -1457,13 +1422,13 @@ pcmk__apply_orderings(pcmk_scheduler_t *sched)
         }
     }
 
-    g_list_foreach(sched->actions, block_colocation_dependents, NULL);
+    g_list_foreach(sched->priv->actions, block_colocation_dependents, NULL);
 
     crm_trace("Ordering probes");
     pcmk__order_probes(sched);
 
-    crm_trace("Updating %d actions", g_list_length(sched->actions));
-    g_list_foreach(sched->actions, update_action_for_orderings, sched);
+    crm_trace("Updating %d actions", g_list_length(sched->priv->actions));
+    g_list_foreach(sched->priv->actions, update_action_for_orderings, sched);
 
     pcmk__disable_invalid_orderings(sched);
 }
