@@ -46,12 +46,12 @@ create_action_name(const pcmk_action_t *action, bool verbose)
     char *action_name = NULL;
     const char *prefix = "";
     const char *action_host = NULL;
-    const char *clone_name = NULL;
+    const char *history_id = NULL;
     const char *task = action->task;
 
     if (action->node != NULL) {
-        action_host = action->node->details->uname;
-    } else if (!pcmk_is_set(action->flags, pcmk_action_pseudo)) {
+        action_host = action->node->priv->name;
+    } else if (!pcmk_is_set(action->flags, pcmk__action_pseudo)) {
         action_host = "<none>";
     }
 
@@ -61,10 +61,10 @@ create_action_name(const pcmk_action_t *action, bool verbose)
     }
 
     if (action->rsc != NULL) {
-        clone_name = action->rsc->clone_name;
+        history_id = action->rsc->priv->history_id;
     }
 
-    if (clone_name != NULL) {
+    if (history_id != NULL) {
         char *key = NULL;
         guint interval_ms = 0;
 
@@ -80,11 +80,10 @@ create_action_name(const pcmk_action_t *action, bool verbose)
             const char *n_task = g_hash_table_lookup(action->meta,
                                                      "notify_key_operation");
 
-            CRM_ASSERT(n_type != NULL);
-            CRM_ASSERT(n_task != NULL);
-            key = pcmk__notify_key(clone_name, n_type, n_task);
+            pcmk__assert((n_type != NULL) && (n_task != NULL));
+            key = pcmk__notify_key(history_id, n_type, n_task);
         } else {
-            key = pcmk__op_key(clone_name, task, interval_ms);
+            key = pcmk__op_key(history_id, task, interval_ms);
         }
 
         if (action_host != NULL) {
@@ -139,7 +138,7 @@ print_cluster_status(pcmk_scheduler_t *scheduler, uint32_t show_opts,
                      uint32_t section_opts, const char *title,
                      bool print_spacer)
 {
-    pcmk__output_t *out = scheduler->priv;
+    pcmk__output_t *out = scheduler->priv->out;
     GList *all = NULL;
     crm_exit_t stonith_rc = 0;
     enum pcmk_pacemakerd_state state = pcmk_pacemakerd_state_invalid;
@@ -170,7 +169,7 @@ print_cluster_status(pcmk_scheduler_t *scheduler, uint32_t show_opts,
 static void
 print_transition_summary(pcmk_scheduler_t *scheduler, bool print_spacer)
 {
-    pcmk__output_t *out = scheduler->priv;
+    pcmk__output_t *out = scheduler->priv->out;
 
     PCMK__OUTPUT_SPACER_IF(out, print_spacer);
     out->begin_list(out, NULL, NULL, "Transition Summary");
@@ -186,23 +185,23 @@ print_transition_summary(pcmk_scheduler_t *scheduler, bool print_spacer)
  * \param[in]     input      What to set as cluster input
  * \param[in]     out        What to set as cluster output object
  * \param[in]     use_date   What to set as cluster's current timestamp
- * \param[in]     flags      Group of enum pcmk_scheduler_flags to set
+ * \param[in]     flags      Group of enum pcmk__scheduler_flags to set
  */
 static void
 reset(pcmk_scheduler_t *scheduler, xmlNodePtr input, pcmk__output_t *out,
       const char *use_date, unsigned int flags)
 {
     scheduler->input = input;
-    scheduler->priv = out;
+    scheduler->priv->out = out;
     set_effective_date(scheduler, true, use_date);
     if (pcmk_is_set(flags, pcmk_sim_sanitized)) {
-        pcmk__set_scheduler_flags(scheduler, pcmk_sched_sanitized);
+        pcmk__set_scheduler_flags(scheduler, pcmk__sched_sanitized);
     }
     if (pcmk_is_set(flags, pcmk_sim_show_scores)) {
-        pcmk__set_scheduler_flags(scheduler, pcmk_sched_output_scores);
+        pcmk__set_scheduler_flags(scheduler, pcmk__sched_output_scores);
     }
     if (pcmk_is_set(flags, pcmk_sim_show_utilization)) {
-        pcmk__set_scheduler_flags(scheduler, pcmk_sched_show_utilization);
+        pcmk__set_scheduler_flags(scheduler, pcmk__sched_show_utilization);
     }
 }
 
@@ -231,30 +230,30 @@ write_sim_dotfile(pcmk_scheduler_t *scheduler, const char *dot_file,
     }
 
     fprintf(dot_strm, " digraph \"g\" {\n");
-    for (iter = scheduler->actions; iter != NULL; iter = iter->next) {
+    for (iter = scheduler->priv->actions; iter != NULL; iter = iter->next) {
         pcmk_action_t *action = (pcmk_action_t *) iter->data;
         const char *style = "dashed";
         const char *font = "black";
         const char *color = "black";
         char *action_name = create_action_name(action, verbose);
 
-        if (pcmk_is_set(action->flags, pcmk_action_pseudo)) {
+        if (pcmk_is_set(action->flags, pcmk__action_pseudo)) {
             font = "orange";
         }
 
-        if (pcmk_is_set(action->flags, pcmk_action_added_to_graph)) {
+        if (pcmk_is_set(action->flags, pcmk__action_added_to_graph)) {
             style = PCMK__VALUE_BOLD;
             color = "green";
 
         } else if ((action->rsc != NULL)
-                   && !pcmk_is_set(action->rsc->flags, pcmk_rsc_managed)) {
+                   && !pcmk_is_set(action->rsc->flags, pcmk__rsc_managed)) {
             color = "red";
             font = "purple";
             if (!all_actions) {
                 goto do_not_write;
             }
 
-        } else if (pcmk_is_set(action->flags, pcmk_action_optional)) {
+        } else if (pcmk_is_set(action->flags, pcmk__action_optional)) {
             color = "blue";
             if (!all_actions) {
                 goto do_not_write;
@@ -262,17 +261,17 @@ write_sim_dotfile(pcmk_scheduler_t *scheduler, const char *dot_file,
 
         } else {
             color = "red";
-            CRM_LOG_ASSERT(!pcmk_is_set(action->flags, pcmk_action_runnable));
+            CRM_LOG_ASSERT(!pcmk_is_set(action->flags, pcmk__action_runnable));
         }
 
-        pcmk__set_action_flags(action, pcmk_action_added_to_graph);
+        pcmk__set_action_flags(action, pcmk__action_added_to_graph);
         fprintf(dot_strm, "\"%s\" [ style=%s color=\"%s\" fontcolor=\"%s\"]\n",
                 action_name, style, color, font);
   do_not_write:
         free(action_name);
     }
 
-    for (iter = scheduler->actions; iter != NULL; iter = iter->next) {
+    for (iter = scheduler->priv->actions; iter != NULL; iter = iter->next) {
         pcmk_action_t *action = (pcmk_action_t *) iter->data;
 
         for (GList *before_iter = action->actions_before;
@@ -285,15 +284,15 @@ write_sim_dotfile(pcmk_scheduler_t *scheduler, const char *dot_file,
             const char *style = "dashed";
             bool optional = true;
 
-            if (before->state == pe_link_dumped) {
+            if (before->graphed) {
                 optional = false;
                 style = PCMK__VALUE_BOLD;
-            } else if ((uint32_t) before->type == pcmk__ar_none) {
+            } else if (before->flags == pcmk__ar_none) {
                 continue;
             } else if (pcmk_is_set(before->action->flags,
-                                   pcmk_action_added_to_graph)
-                       && pcmk_is_set(action->flags, pcmk_action_added_to_graph)
-                       && (uint32_t) before->type != pcmk__ar_if_on_same_node_or_target) {
+                                   pcmk__action_added_to_graph)
+                       && pcmk_is_set(action->flags, pcmk__action_added_to_graph)
+                       && before->flags != pcmk__ar_if_on_same_node_or_target) {
                 optional = false;
             }
 
@@ -318,7 +317,7 @@ write_sim_dotfile(pcmk_scheduler_t *scheduler, const char *dot_file,
  * \brief Profile the configuration updates and scheduler actions in a single
  *        CIB file, printing the profiling timings.
  *
- * \note \p scheduler->priv must have been set to a valid \p pcmk__output_t
+ * \note \p scheduler->priv->out must have been set to a valid \p pcmk__output_t
  *       object before this function is called.
  *
  * \param[in]     xml_file   The CIB file to profile
@@ -330,13 +329,13 @@ static void
 profile_file(const char *xml_file, long long repeat,
              pcmk_scheduler_t *scheduler, const char *use_date)
 {
-    pcmk__output_t *out = scheduler->priv;
+    pcmk__output_t *out = scheduler->priv->out;
     xmlNode *cib_object = NULL;
     clock_t start = 0;
     clock_t end;
-    unsigned long long scheduler_flags = pcmk_sched_no_compat;
+    unsigned long long scheduler_flags = pcmk__sched_none;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     cib_object = pcmk__xml_read(xml_file);
     start = clock();
@@ -346,20 +345,20 @@ profile_file(const char *xml_file, long long repeat,
     }
 
     if (pcmk__update_configured_schema(&cib_object, false) != pcmk_rc_ok) {
-        free_xml(cib_object);
+        pcmk__xml_free(cib_object);
         return;
     }
 
     if (!pcmk__validate_xml(cib_object, NULL, NULL, NULL)) {
-        free_xml(cib_object);
+        pcmk__xml_free(cib_object);
         return;
     }
 
-    if (pcmk_is_set(scheduler->flags, pcmk_sched_output_scores)) {
-        scheduler_flags |= pcmk_sched_output_scores;
+    if (pcmk_is_set(scheduler->flags, pcmk__sched_output_scores)) {
+        scheduler_flags |= pcmk__sched_output_scores;
     }
-    if (pcmk_is_set(scheduler->flags, pcmk_sched_show_utilization)) {
-        scheduler_flags |= pcmk_sched_show_utilization;
+    if (pcmk_is_set(scheduler->flags, pcmk__sched_show_utilization)) {
+        scheduler_flags |= pcmk__sched_show_utilization;
     }
 
     for (int i = 0; i < repeat; ++i) {
@@ -382,12 +381,12 @@ void
 pcmk__profile_dir(const char *dir, long long repeat,
                   pcmk_scheduler_t *scheduler, const char *use_date)
 {
-    pcmk__output_t *out = scheduler->priv;
+    pcmk__output_t *out = scheduler->priv->out;
     struct dirent **namelist;
 
     int file_num = scandir(dir, &namelist, 0, alphasort);
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     if (file_num > 0) {
         struct stat prop;
@@ -422,7 +421,7 @@ pcmk__profile_dir(const char *dir, long long repeat,
  * \brief Set the date of the cluster, either to the value given by
  *        \p use_date, or to the \c PCMK_XA_EXECUTION_DATE value in the CIB.
  *
- * \note \p scheduler->priv must have been set to a valid \p pcmk__output_t
+ * \note \p scheduler->priv->out must have been set to a valid \p pcmk__output_t
  *       object before this function is called.
  *
  * \param[in,out] scheduler       Scheduler data
@@ -435,26 +434,27 @@ static void
 set_effective_date(pcmk_scheduler_t *scheduler, bool print_original,
                    const char *use_date)
 {
-    pcmk__output_t *out = scheduler->priv;
+    pcmk__output_t *out = scheduler->priv->out;
     time_t original_date = 0;
 
-    CRM_ASSERT(out != NULL);
+    pcmk__assert(out != NULL);
 
     crm_element_value_epoch(scheduler->input, PCMK_XA_EXECUTION_DATE,
                             &original_date);
 
     if (use_date) {
-        scheduler->now = crm_time_new(use_date);
+        scheduler->priv->now = crm_time_new(use_date);
         out->info(out, "Setting effective cluster time: %s", use_date);
-        crm_time_log(LOG_NOTICE, "Pretending 'now' is", scheduler->now,
+        crm_time_log(LOG_NOTICE, "Pretending 'now' is", scheduler->priv->now,
                      crm_time_log_date | crm_time_log_timeofday);
 
     } else if (original_date != 0) {
-        scheduler->now = pcmk__copy_timet(original_date);
+        scheduler->priv->now = pcmk__copy_timet(original_date);
 
         if (print_original) {
-            char *when = crm_time_as_string(scheduler->now,
-                            crm_time_log_date|crm_time_log_timeofday);
+            char *when = crm_time_as_string(scheduler->priv->now,
+                                            crm_time_log_date
+                                            |crm_time_log_timeofday);
 
             out->info(out, "Using the original execution date of: %s", when);
             free(when);
@@ -566,15 +566,15 @@ simulate_resource_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
 
     pcmk__scan_min_int(target_rc_s, &target_outcome, 0);
 
-    CRM_ASSERT(fake_cib->cmds->query(fake_cib, NULL, NULL,
-                                     cib_sync_call|cib_scope_local) == pcmk_ok);
+    pcmk__assert(fake_cib->cmds->query(fake_cib, NULL, NULL,
+                                       cib_sync_call) == pcmk_ok);
 
     // Ensure the action node is in the CIB
     uuid = crm_element_value_copy(action->xml, PCMK__META_ON_NODE_UUID);
     cib_node = pcmk__inject_node(fake_cib, node,
                                  ((router_node == NULL)? uuid: node));
     free(uuid);
-    CRM_ASSERT(cib_node != NULL);
+    pcmk__assert(cib_node != NULL);
 
     // Add a history entry for the action
     cib_resource = pcmk__inject_resource_history(out, cib_node, resource,
@@ -584,7 +584,7 @@ simulate_resource_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
         crm_err("Could not simulate action %d history for resource %s",
                 action->id, resource);
         free(node);
-        free_xml(cib_node);
+        pcmk__xml_free(cib_node);
         return EINVAL;
     }
 
@@ -600,6 +600,7 @@ simulate_resource_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
         const char *spec = (const char *) iter->data;
         char *key = NULL;
         const char *match_name = NULL;
+        const char *offset = NULL;
 
         // Allow user to specify anonymous clone with or without instance number
         key = crm_strdup_printf(PCMK__OP_FMT "@%s=", resource, op->op_type,
@@ -637,20 +638,30 @@ simulate_resource_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
                   action->id, op->rc);
         pcmk__set_graph_action_flags(action, pcmk__graph_action_failed);
         graph->abort_priority = PCMK_SCORE_INFINITY;
+
+        if (pcmk__str_eq(op->op_type, PCMK_ACTION_START, pcmk__str_none)) {
+            offset = pcmk__s(graph->failed_start_offset, PCMK_VALUE_INFINITY);
+
+        } else if (pcmk__str_eq(op->op_type, PCMK_ACTION_STOP,
+                                pcmk__str_none)) {
+            offset = pcmk__s(graph->failed_stop_offset, PCMK_VALUE_INFINITY);
+        }
+
         pcmk__inject_failcount(out, fake_cib, cib_node, match_name, op->op_type,
-                               op->interval_ms, op->rc);
+                               op->interval_ms, op->rc,
+                               pcmk_str_is_infinity(offset));
         break;
     }
 
-    pcmk__inject_action_result(cib_resource, op, target_outcome);
+    pcmk__inject_action_result(cib_resource, op, node, target_outcome);
     lrmd_free_event(op);
     rc = fake_cib->cmds->modify(fake_cib, PCMK_XE_STATUS, cib_node,
-                                cib_sync_call|cib_scope_local);
-    CRM_ASSERT(rc == pcmk_ok);
+                                cib_sync_call);
+    pcmk__assert(rc == pcmk_ok);
 
   done:
     free(node);
-    free_xml(cib_node);
+    pcmk__xml_free(cib_node);
     pcmk__set_graph_action_flags(action, pcmk__graph_action_confirmed);
     pcmk__update_graph(graph, action);
     return pcmk_rc_ok;
@@ -704,11 +715,11 @@ simulate_fencing_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
         xmlNode *cib_node = pcmk__inject_node_state_change(fake_cib, target,
                                                            false);
 
-        CRM_ASSERT(cib_node != NULL);
+        pcmk__assert(cib_node != NULL);
         crm_xml_add(cib_node, PCMK_XA_CRM_DEBUG_ORIGIN, __func__);
         rc = fake_cib->cmds->replace(fake_cib, PCMK_XE_STATUS, cib_node,
-                                     cib_sync_call|cib_scope_local);
-        CRM_ASSERT(rc == pcmk_ok);
+                                     cib_sync_call);
+        pcmk__assert(rc == pcmk_ok);
 
         // Simulate controller clearing node's resource history and attributes
         pcmk__g_strcat(xpath,
@@ -716,7 +727,7 @@ simulate_fencing_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
                        "[@" PCMK_XA_UNAME "='", target, "']/" PCMK__XE_LRM,
                        NULL);
         fake_cib->cmds->remove(fake_cib, (const char *) xpath->str, NULL,
-                               cib_xpath|cib_sync_call|cib_scope_local);
+                               cib_xpath|cib_sync_call);
 
         g_string_truncate(xpath, 0);
         pcmk__g_strcat(xpath,
@@ -724,9 +735,9 @@ simulate_fencing_action(pcmk__graph_t *graph, pcmk__graph_action_t *action)
                        "[@" PCMK_XA_UNAME "='", target, "']"
                        "/" PCMK__XE_TRANSIENT_ATTRIBUTES, NULL);
         fake_cib->cmds->remove(fake_cib, (const char *) xpath->str, NULL,
-                               cib_xpath|cib_sync_call|cib_scope_local);
+                               cib_xpath|cib_sync_call);
 
-        free_xml(cib_node);
+        pcmk__xml_free(cib_node);
         g_string_free(xpath, TRUE);
     }
 
@@ -750,7 +761,7 @@ pcmk__simulate_transition(pcmk_scheduler_t *scheduler, cib_t *cib,
         simulate_fencing_action,
     };
 
-    out = scheduler->priv;
+    out = scheduler->priv->out;
 
     fake_cib = cib;
     fake_op_fail_list = op_fail_list;
@@ -760,10 +771,10 @@ pcmk__simulate_transition(pcmk_scheduler_t *scheduler, cib_t *cib,
     }
 
     pcmk__set_graph_functions(&simulation_fns);
-    transition = pcmk__unpack_graph(scheduler->graph, crm_system_name);
+    transition = pcmk__unpack_graph(scheduler->priv->graph, crm_system_name);
     pcmk__log_graph(LOG_DEBUG, transition);
 
-    fake_resource_list = scheduler->resources;
+    fake_resource_list = scheduler->priv->resources;
     do {
         graph_rc = pcmk__execute_graph(transition);
     } while (graph_rc == pcmk__graph_active);
@@ -781,9 +792,9 @@ pcmk__simulate_transition(pcmk_scheduler_t *scheduler, cib_t *cib,
         // If not quiet, we'll need the resulting CIB for later display
         xmlNode *cib_object = NULL;
         int rc = fake_cib->cmds->query(fake_cib, NULL, &cib_object,
-                                       cib_sync_call|cib_scope_local);
+                                       cib_sync_call);
 
-        CRM_ASSERT(rc == pcmk_ok);
+        pcmk__assert(rc == pcmk_ok);
         pe_reset_working_set(scheduler);
         scheduler->input = cib_object;
         out->end_list(out);
@@ -811,30 +822,23 @@ pcmk__simulate(pcmk_scheduler_t *scheduler, pcmk__output_t *out,
     reset(scheduler, input, out, use_date, flags);
     cluster_status(scheduler);
 
-    if ((cib->variant == cib_native)
-        && pcmk_is_set(section_opts, pcmk_section_times)) {
-        if (pcmk__our_nodename == NULL) {
-            // Currently used only in the times section
-            pcmk__query_node_name(out, 0, &pcmk__our_nodename, 0);
-        }
-        scheduler->localhost = pcmk__our_nodename;
-    }
-
     if (!out->is_quiet(out)) {
         const bool show_pending = pcmk_is_set(flags, pcmk_sim_show_pending);
 
-        if (pcmk_is_set(scheduler->flags, pcmk_sched_in_maintenance)) {
+        if (pcmk_is_set(scheduler->flags, pcmk__sched_in_maintenance)) {
             printed = out->message(out, "maint-mode", scheduler->flags);
         }
 
-        if (scheduler->disabled_resources || scheduler->blocked_resources) {
+        if ((scheduler->priv->disabled_resources > 0)
+            || (scheduler->priv->blocked_resources > 0)) {
+
             PCMK__OUTPUT_SPACER_IF(out, printed == pcmk_rc_ok);
             printed = out->info(out,
                                 "%d of %d resource instances DISABLED and "
                                 "%d BLOCKED from further action due to failure",
-                                scheduler->disabled_resources,
-                                scheduler->ninstances,
-                                scheduler->blocked_resources);
+                                scheduler->priv->disabled_resources,
+                                scheduler->priv->ninstances,
+                                scheduler->priv->blocked_resources);
         }
 
         /* Most formatted output headers use caps for each word, but this one
@@ -873,7 +877,7 @@ pcmk__simulate(pcmk_scheduler_t *scheduler, pcmk__output_t *out,
     }
 
     if (input_file != NULL) {
-        rc = pcmk__xml_write_file(input, input_file, false, NULL);
+        rc = pcmk__xml_write_file(input, input_file, false);
         if (rc != pcmk_rc_ok) {
             goto simulate_done;
         }
@@ -881,29 +885,29 @@ pcmk__simulate(pcmk_scheduler_t *scheduler, pcmk__output_t *out,
 
     if (pcmk_any_flags_set(flags, pcmk_sim_process | pcmk_sim_simulate)) {
         pcmk__output_t *logger_out = NULL;
-        unsigned long long scheduler_flags = pcmk_sched_no_compat;
+        unsigned long long scheduler_flags = pcmk__sched_none;
 
-        if (pcmk_is_set(scheduler->flags, pcmk_sched_output_scores)) {
-            scheduler_flags |= pcmk_sched_output_scores;
+        if (pcmk_is_set(scheduler->flags, pcmk__sched_output_scores)) {
+            scheduler_flags |= pcmk__sched_output_scores;
         }
-        if (pcmk_is_set(scheduler->flags, pcmk_sched_show_utilization)) {
-            scheduler_flags |= pcmk_sched_show_utilization;
+        if (pcmk_is_set(scheduler->flags, pcmk__sched_show_utilization)) {
+            scheduler_flags |= pcmk__sched_show_utilization;
         }
 
         if (pcmk_all_flags_set(scheduler->flags,
-                               pcmk_sched_output_scores
-                               |pcmk_sched_show_utilization)) {
+                               pcmk__sched_output_scores
+                               |pcmk__sched_show_utilization)) {
             PCMK__OUTPUT_SPACER_IF(out, printed == pcmk_rc_ok);
             out->begin_list(out, NULL, NULL,
                             "Assignment Scores and Utilization Information");
             printed = pcmk_rc_ok;
 
-        } else if (pcmk_is_set(scheduler->flags, pcmk_sched_output_scores)) {
+        } else if (pcmk_is_set(scheduler->flags, pcmk__sched_output_scores)) {
             PCMK__OUTPUT_SPACER_IF(out, printed == pcmk_rc_ok);
             out->begin_list(out, NULL, NULL, "Assignment Scores");
             printed = pcmk_rc_ok;
 
-        } else if (pcmk_is_set(scheduler->flags, pcmk_sched_show_utilization)) {
+        } else if (pcmk_is_set(scheduler->flags, pcmk__sched_show_utilization)) {
             PCMK__OUTPUT_SPACER_IF(out, printed == pcmk_rc_ok);
             out->begin_list(out, NULL, NULL, "Utilization Information");
             printed = pcmk_rc_ok;
@@ -915,7 +919,7 @@ pcmk__simulate(pcmk_scheduler_t *scheduler, pcmk__output_t *out,
             }
             pe__register_messages(logger_out);
             pcmk__register_lib_messages(logger_out);
-            scheduler->priv = logger_out;
+            scheduler->priv->out = logger_out;
         }
 
         pcmk__schedule_actions(input, scheduler_flags, scheduler);
@@ -925,14 +929,14 @@ pcmk__simulate(pcmk_scheduler_t *scheduler, pcmk__output_t *out,
         } else {
             logger_out->finish(logger_out, CRM_EX_OK, true, NULL);
             pcmk__output_free(logger_out);
-            scheduler->priv = out;
+            scheduler->priv->out = out;
         }
 
         input = NULL;           /* Don't try and free it twice */
 
         if (graph_file != NULL) {
-            rc = pcmk__xml_write_file(scheduler->graph, graph_file, false,
-                                      NULL);
+            rc = pcmk__xml_write_file(scheduler->priv->graph, graph_file,
+                                      false);
             if (rc != pcmk_rc_ok) {
                 rc = pcmk_rc_graph_error;
                 goto simulate_done;
@@ -973,10 +977,10 @@ pcmk__simulate(pcmk_scheduler_t *scheduler, pcmk__output_t *out,
     set_effective_date(scheduler, true, use_date);
 
     if (pcmk_is_set(flags, pcmk_sim_show_scores)) {
-        pcmk__set_scheduler_flags(scheduler, pcmk_sched_output_scores);
+        pcmk__set_scheduler_flags(scheduler, pcmk__sched_output_scores);
     }
     if (pcmk_is_set(flags, pcmk_sim_show_utilization)) {
-        pcmk__set_scheduler_flags(scheduler, pcmk_sched_show_utilization);
+        pcmk__set_scheduler_flags(scheduler, pcmk__sched_show_utilization);
     }
 
     cluster_status(scheduler);
