@@ -96,7 +96,7 @@ block_failure(const pcmk_node_t *node, pcmk_resource_t *rsc,
                                         "='" PCMK_VALUE_BLOCK "']",
                                     xml_name);
 
-    xmlXPathObject *xpathObj = xpath_search(rsc->xml, xpath);
+    xmlXPathObject *xpathObj = xpath_search(rsc->priv->xml, xpath);
     gboolean should_block = FALSE;
 
     free(xpath);
@@ -134,10 +134,11 @@ block_failure(const pcmk_node_t *node, pcmk_resource_t *rsc,
                   "[@" PCMK_META_INTERVAL "='%u']"
 
                 lrm_op_xpath = crm_strdup_printf(XPATH_FMT,
-                                                 node->details->uname, xml_name,
+                                                 node->priv->name, xml_name,
                                                  conf_op_name,
                                                  conf_op_interval_ms);
-                lrm_op_xpathObj = xpath_search(rsc->cluster->input, lrm_op_xpath);
+                lrm_op_xpathObj = xpath_search(rsc->priv->scheduler->input,
+                                               lrm_op_xpath);
 
                 free(lrm_op_xpath);
 
@@ -183,9 +184,9 @@ block_failure(const pcmk_node_t *node, pcmk_resource_t *rsc,
 static inline char *
 rsc_fail_name(const pcmk_resource_t *rsc)
 {
-    const char *name = (rsc->clone_name? rsc->clone_name : rsc->id);
+    const char *name = pcmk__s(rsc->priv->history_id, rsc->id);
 
-    return pcmk_is_set(rsc->flags, pcmk_rsc_unique)? strdup(name) : clone_strip(name);
+    return pcmk_is_set(rsc->flags, pcmk__rsc_unique)? strdup(name) : clone_strip(name);
 }
 
 /*!
@@ -194,7 +195,6 @@ rsc_fail_name(const pcmk_resource_t *rsc)
  *
  * \param[in]  prefix    Attribute prefix to match
  * \param[in]  rsc_name  Resource name to match as used in failure attributes
- * \param[in]  is_legacy Whether DC uses per-resource fail counts
  * \param[in]  is_unique Whether the resource is a globally unique clone
  * \param[out] re        Where to store resulting regular expression
  *
@@ -203,21 +203,15 @@ rsc_fail_name(const pcmk_resource_t *rsc)
  *       The caller is responsible for freeing re with regfree().
  */
 static int
-generate_fail_regex(const char *prefix, const char *rsc_name,
-                    gboolean is_legacy, gboolean is_unique, regex_t *re)
+generate_fail_regex(const char *prefix, const char *rsc_name, bool is_unique,
+                    regex_t *re)
 {
-    char *pattern;
-
-    /* @COMPAT DC < 1.1.17: Fail counts used to be per-resource rather than
-     * per-operation.
-     */
-    const char *op_pattern = (is_legacy? "" : "#.+_[0-9]+");
+    char *pattern = NULL;
+    const char *op_pattern = "#.+_[0-9]+";
 
     /* Ignore instance numbers for anything other than globally unique clones.
      * Anonymous clone fail counts could contain an instance number if the
      * clone was initially unique, failed, then was converted to anonymous.
-     * @COMPAT Also, before 1.1.8, anonymous clone fail counts always contained
-     * clone instance numbers.
      */
     const char *instance_pattern = (is_unique? "" : "(:[0-9]+)?");
 
@@ -245,25 +239,19 @@ generate_fail_regex(const char *prefix, const char *rsc_name,
  *       regfree().
  */
 static int
-generate_fail_regexes(const pcmk_resource_t *rsc,
-                      regex_t *failcount_re, regex_t *lastfailure_re)
+generate_fail_regexes(const pcmk_resource_t *rsc, regex_t *failcount_re,
+                      regex_t *lastfailure_re)
 {
     int rc = pcmk_rc_ok;
     char *rsc_name = rsc_fail_name(rsc);
-    const char *version = crm_element_value(rsc->cluster->input,
-                                            PCMK_XA_CRM_FEATURE_SET);
 
-    // @COMPAT Pacemaker <= 1.1.16 used a single fail count per resource
-    gboolean is_legacy = (compare_version(version, "3.0.13") < 0);
-
-    if (generate_fail_regex(PCMK__FAIL_COUNT_PREFIX, rsc_name, is_legacy,
-                            pcmk_is_set(rsc->flags, pcmk_rsc_unique),
+    if (generate_fail_regex(PCMK__FAIL_COUNT_PREFIX, rsc_name,
+                            pcmk_is_set(rsc->flags, pcmk__rsc_unique),
                             failcount_re) != pcmk_rc_ok) {
         rc = EINVAL;
 
     } else if (generate_fail_regex(PCMK__LAST_FAILURE_PREFIX, rsc_name,
-                                   is_legacy,
-                                   pcmk_is_set(rsc->flags, pcmk_rsc_unique),
+                                   pcmk_is_set(rsc->flags, pcmk__rsc_unique),
                                    lastfailure_re) != pcmk_rc_ok) {
         rc = EINVAL;
         regfree(failcount_re);
@@ -300,8 +288,17 @@ update_failcount_for_attr(gpointer key, gpointer value, gpointer user_data)
 
     // If this is a matching fail count attribute, update fail count
     if (regexec(&(fc_data->failcount_re), (const char *) key, 0, NULL, 0) == 0) {
-        fc_data->failcount = pcmk__add_scores(fc_data->failcount,
-                                              char2score(value));
+        int score = 0;
+        int rc = pcmk_parse_score(value, &score, 0);
+
+        if (rc != pcmk_rc_ok) {
+            crm_warn("Ignoring %s for %s "
+                     "because '%s' is not a valid fail count: %s",
+                     (const char *) key, pcmk__node_name(fc_data->node),
+                     value, pcmk_rc_str(rc));
+            return;
+        }
+        fc_data->failcount = pcmk__add_scores(fc_data->failcount, score);
         pcmk__rsc_trace(fc_data->rsc, "Added %s (%s) to %s fail count (now %s)",
                         (const char *) key, (const char *) value,
                         fc_data->rsc->id,
@@ -313,34 +310,40 @@ update_failcount_for_attr(gpointer key, gpointer value, gpointer user_data)
     if (regexec(&(fc_data->lastfailure_re), (const char *) key, 0, NULL,
                 0) == 0) {
         long long last_ll;
+        int rc = pcmk__scan_ll(value, &last_ll, 0LL);
 
-        if (pcmk__scan_ll(value, &last_ll, 0LL) == pcmk_rc_ok) {
-            fc_data->last_failure = (time_t) QB_MAX(fc_data->last_failure,
-                                                    last_ll);
+        if (rc != pcmk_rc_ok) {
+            crm_info("Ignoring invalid value '%s' for %s: %s",
+                     (const char *) value, (const char *) key, pcmk_rc_str(rc));
+            return;
         }
+        fc_data->last_failure = (time_t) QB_MAX(fc_data->last_failure, last_ll);
     }
 }
 
 /*!
  * \internal
- * \brief Update fail count and last failure appropriately for a filler resource
+ * \brief Update fail count and last failure appropriately for launched resource
  *
- * \param[in] data       Filler resource
+ * \param[in] data       Launched resource
  * \param[in] user_data  Fail count data to update
  */
 static void
-update_failcount_for_filler(gpointer data, gpointer user_data)
+update_launched_failcount(gpointer data, gpointer user_data)
 {
-    pcmk_resource_t *filler = data;
+    pcmk_resource_t *launched = data;
     struct failcount_data *fc_data = user_data;
-    time_t filler_last_failure = 0;
+    time_t launched_last_failure = 0;
 
-    fc_data->failcount += pe_get_failcount(fc_data->node, filler,
-                                           &filler_last_failure, fc_data->flags,
-                                           fc_data->xml_op);
-    fc_data->last_failure = QB_MAX(fc_data->last_failure, filler_last_failure);
+    fc_data->failcount += pe_get_failcount(fc_data->node, launched,
+                                           &launched_last_failure,
+                                           fc_data->flags, fc_data->xml_op);
+    fc_data->last_failure = QB_MAX(fc_data->last_failure, launched_last_failure);
 }
 
+#define readable_expiration(rsc)    \
+    pcmk__readable_interval((rsc)->priv->failure_expiration_ms)
+
 /*!
  * \internal
  * \brief Get a resource's fail count on a node
@@ -374,50 +377,53 @@ pe_get_failcount(const pcmk_node_t *node, pcmk_resource_t *rsc,
     CRM_CHECK(generate_fail_regexes(rsc, &fc_data.failcount_re,
                                     &fc_data.lastfailure_re) == pcmk_rc_ok,
               return 0);
-    g_hash_table_foreach(node->details->attrs, update_failcount_for_attr,
+    g_hash_table_foreach(node->priv->attrs, update_failcount_for_attr,
                          &fc_data);
     regfree(&(fc_data.failcount_re));
     regfree(&(fc_data.lastfailure_re));
 
     // If failure blocks the resource, disregard any failure timeout
-    if ((fc_data.failcount > 0) && (rsc->failure_timeout > 0)
+    if ((fc_data.failcount > 0) && (rsc->priv->failure_expiration_ms > 0)
         && block_failure(node, rsc, xml_op)) {
 
-        pcmk__config_warn("Ignoring failure timeout %d for %s "
+        pcmk__config_warn("Ignoring failure timeout (%s) for %s "
                           "because it conflicts with "
                           PCMK_META_ON_FAIL "=" PCMK_VALUE_BLOCK,
-                          rsc->failure_timeout, rsc->id);
-        rsc->failure_timeout = 0;
+                          readable_expiration(rsc), rsc->id);
+        rsc->priv->failure_expiration_ms = 0;
     }
 
     // If all failures have expired, ignore fail count
     if (pcmk_is_set(flags, pcmk__fc_effective) && (fc_data.failcount > 0)
-        && (fc_data.last_failure > 0) && (rsc->failure_timeout != 0)) {
+        && (fc_data.last_failure > 0)
+        && (rsc->priv->failure_expiration_ms > 0)) {
 
-        time_t now = get_effective_time(rsc->cluster);
+        time_t now = get_effective_time(rsc->priv->scheduler);
+        const guint expiration = pcmk__timeout_ms2s(rsc->priv->failure_expiration_ms);
 
-        if (now > (fc_data.last_failure + rsc->failure_timeout)) {
-            pcmk__rsc_debug(rsc, "Failcount for %s on %s expired after %ds",
+        if (now > (fc_data.last_failure + expiration)) {
+            pcmk__rsc_debug(rsc, "Failcount for %s on %s expired after %s",
                             rsc->id, pcmk__node_name(node),
-                            rsc->failure_timeout);
+                            readable_expiration(rsc));
             fc_data.failcount = 0;
         }
     }
 
-    /* Add the fail count of any filler resources, except that we never want the
-     * fail counts of a bundle container's fillers to count towards the
-     * container's fail count.
+    /* Add the fail count of any launched resources, except that we never want
+     * the fail counts of a bundle container's launched resources to count
+     * towards the container's fail count.
      *
      * Most importantly, a Pacemaker Remote connection to a bundle container
-     * is a filler of the container, but can reside on a different node than the
+     * is launched by the container, but can reside on a different node than the
      * container itself. Counting its fail count on its node towards the
      * container's fail count on that node could lead to attempting to stop the
      * container on the wrong node.
      */
-    if (pcmk_is_set(flags, pcmk__fc_fillers) && (rsc->fillers != NULL)
-        && !pcmk__is_bundled(rsc)) {
+    if (pcmk_is_set(flags, pcmk__fc_launched)
+        && (rsc->priv->launched != NULL) && !pcmk__is_bundled(rsc)) {
 
-        g_list_foreach(rsc->fillers, update_failcount_for_filler, &fc_data);
+        g_list_foreach(rsc->priv->launched, update_launched_failcount,
+                       &fc_data);
         if (fc_data.failcount > 0) {
             pcmk__rsc_info(rsc,
                            "Container %s and the resources within it "
@@ -467,7 +473,7 @@ pe__clear_failcount(pcmk_resource_t *rsc, const pcmk_node_t *node,
     clear = custom_action(rsc, key, PCMK_ACTION_CLEAR_FAILCOUNT, node, FALSE,
                           scheduler);
     pcmk__insert_meta(clear, PCMK__META_OP_NO_WAIT, PCMK_VALUE_TRUE);
-    crm_notice("Clearing failure of %s on %s because %s " CRM_XS " %s",
+    crm_notice("Clearing failure of %s on %s because %s " QB_XS " %s",
                rsc->id, pcmk__node_name(node), reason, clear->uuid);
     return clear;
 }
