@@ -37,9 +37,11 @@ pcmk_resource_t *
 pe__last_group_member(const pcmk_resource_t *group)
 {
     if (group != NULL) {
-        CRM_CHECK(pcmk__is_group(group) && (group->variant_opaque != NULL),
-                  return NULL);
-        return ((group_variant_data_t *) group->variant_opaque)->last_child;
+        const group_variant_data_t *group_data = NULL;
+
+        CRM_CHECK(pcmk__is_group(group), return NULL);
+        group_data = group->priv->variant_opaque;
+        return group_data->last_child;
     }
     return NULL;
 }
@@ -56,11 +58,10 @@ pe__last_group_member(const pcmk_resource_t *group)
 bool
 pe__group_flag_is_set(const pcmk_resource_t *group, uint32_t flags)
 {
-    group_variant_data_t *group_data = NULL;
+    const group_variant_data_t *group_data = NULL;
 
-    CRM_CHECK(pcmk__is_group(group) && (group->variant_opaque != NULL),
-              return false);
-    group_data = (group_variant_data_t *) group->variant_opaque;
+    CRM_CHECK(pcmk__is_group(group), return false);
+    group_data = group->priv->variant_opaque;
     return pcmk_all_flags_set(group_data->flags, flags);
 }
 
@@ -80,13 +81,14 @@ set_group_flag(pcmk_resource_t *group, const char *option, uint32_t flag,
     const char *value_s = NULL;
     int value = 0;
 
-    value_s = g_hash_table_lookup(group->meta, option);
+    value_s = g_hash_table_lookup(group->priv->meta, option);
 
     // We don't actually need the null check but it speeds up the common case
     if ((value_s == NULL) || (crm_str_to_boolean(value_s, &value) < 0)
         || (value != 0)) {
+        group_variant_data_t *group_data = group->priv->variant_opaque;
 
-        ((group_variant_data_t *) group->variant_opaque)->flags |= flag;
+        group_data->flags |= flag;
 
     } else {
         pcmk__warn_once(wo_bit,
@@ -101,10 +103,12 @@ inactive_resources(pcmk_resource_t *rsc)
 {
     int retval = 0;
 
-    for (GList *gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
+    for (GList *gIter = rsc->priv->children;
+         gIter != NULL; gIter = gIter->next) {
+
         pcmk_resource_t *child_rsc = (pcmk_resource_t *) gIter->data;
 
-        if (!child_rsc->fns->active(child_rsc, TRUE)) {
+        if (!child_rsc->priv->fns->active(child_rsc, TRUE)) {
             retval++;
         }
     }
@@ -128,10 +132,10 @@ group_header(pcmk__output_t *out, int *rc, const pcmk_resource_t *rsc,
         pcmk__add_separated_word(&attrs, 64, "disabled", ", ");
     }
 
-    if (pcmk_is_set(rsc->flags, pcmk_rsc_maintenance)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_maintenance)) {
         pcmk__add_separated_word(&attrs, 64, "maintenance", ", ");
 
-    } else if (!pcmk_is_set(rsc->flags, pcmk_rsc_managed)) {
+    } else if (!pcmk_is_set(rsc->flags, pcmk__rsc_managed)) {
         pcmk__add_separated_word(&attrs, 64, "unmanaged", ", ");
     }
 
@@ -155,8 +159,8 @@ skip_child_rsc(pcmk_resource_t *rsc, pcmk_resource_t *child,
 {
     bool star_list = pcmk__list_of_1(only_rsc) &&
                      pcmk__str_eq("*", g_list_first(only_rsc)->data, pcmk__str_none);
-    bool child_filtered = child->fns->is_filtered(child, only_rsc, FALSE);
-    bool child_active = child->fns->active(child, FALSE);
+    bool child_filtered = child->priv->fns->is_filtered(child, only_rsc, FALSE);
+    bool child_active = child->priv->fns->active(child, FALSE);
     bool show_inactive = pcmk_is_set(show_opts, pcmk_show_inactive_rscs);
 
     /* If the resource is in only_rsc by name (so, ignoring "*") then allow
@@ -179,7 +183,7 @@ skip_child_rsc(pcmk_resource_t *rsc, pcmk_resource_t *child,
 gboolean
 group_unpack(pcmk_resource_t *rsc, pcmk_scheduler_t *scheduler)
 {
-    xmlNode *xml_obj = rsc->xml;
+    xmlNode *xml_obj = rsc->priv->xml;
     xmlNode *xml_native_rsc = NULL;
     group_variant_data_t *group_data = NULL;
     const char *clone_id = NULL;
@@ -188,7 +192,7 @@ group_unpack(pcmk_resource_t *rsc, pcmk_scheduler_t *scheduler)
 
     group_data = pcmk__assert_alloc(1, sizeof(group_variant_data_t));
     group_data->last_child = NULL;
-    rsc->variant_opaque = group_data;
+    rsc->priv->variant_opaque = group_data;
 
     // @COMPAT These are deprecated since 2.1.5
     set_group_flag(rsc, PCMK_META_ORDERED, pcmk__group_ordered,
@@ -196,39 +200,34 @@ group_unpack(pcmk_resource_t *rsc, pcmk_scheduler_t *scheduler)
     set_group_flag(rsc, "collocated", pcmk__group_colocated,
                    pcmk__wo_group_coloc);
 
-    clone_id = crm_element_value(rsc->xml, PCMK__META_CLONE);
+    clone_id = crm_element_value(rsc->priv->xml, PCMK__META_CLONE);
 
-    for (xml_native_rsc = pcmk__xe_first_child(xml_obj, NULL, NULL, NULL);
+    for (xml_native_rsc = pcmk__xe_first_child(xml_obj, PCMK_XE_PRIMITIVE,
+                                               NULL, NULL);
          xml_native_rsc != NULL;
-         xml_native_rsc = pcmk__xe_next(xml_native_rsc)) {
+         xml_native_rsc = pcmk__xe_next(xml_native_rsc, PCMK_XE_PRIMITIVE)) {
 
-        if (pcmk__xe_is(xml_native_rsc, PCMK_XE_PRIMITIVE)) {
-            pcmk_resource_t *new_rsc = NULL;
+        pcmk_resource_t *new_rsc = NULL;
 
-            crm_xml_add(xml_native_rsc, PCMK__META_CLONE, clone_id);
-            if (pe__unpack_resource(xml_native_rsc, &new_rsc, rsc,
-                                    scheduler) != pcmk_rc_ok) {
-                continue;
-            }
-
-            rsc->children = g_list_append(rsc->children, new_rsc);
-            group_data->last_child = new_rsc;
-            pcmk__rsc_trace(rsc, "Added %s member %s", rsc->id, new_rsc->id);
+        crm_xml_add(xml_native_rsc, PCMK__META_CLONE, clone_id);
+        if (pe__unpack_resource(xml_native_rsc, &new_rsc, rsc,
+                                scheduler) != pcmk_rc_ok) {
+            continue;
         }
+
+        rsc->priv->children = g_list_append(rsc->priv->children, new_rsc);
+        group_data->last_child = new_rsc;
+        pcmk__rsc_trace(rsc, "Added %s member %s", rsc->id, new_rsc->id);
     }
 
-    if (rsc->children == NULL) {
-        /* The schema does not allow empty groups, but if validation is
-         * disabled, we allow them (members can be added later).
-         *
-         * @COMPAT At a major release bump, we should consider this a failure so
-         *         that group methods can assume children is not NULL, and there
-         *         are no strange effects from phantom groups due to their
-         *         presence or meta-attributes.
-         */
-        pcmk__config_warn("Group %s will be ignored because it does not have "
-                          "any members", rsc->id);
+    if (rsc->priv->children == NULL) {
+        // Not possible with schema validation enabled
+        free(group_data);
+        rsc->priv->variant_opaque = NULL;
+        pcmk__config_err("Group %s has no members", rsc->id);
+        return FALSE;
     }
+
     return TRUE;
 }
 
@@ -237,12 +236,13 @@ group_active(pcmk_resource_t *rsc, gboolean all)
 {
     gboolean c_all = TRUE;
     gboolean c_any = FALSE;
-    GList *gIter = rsc->children;
 
-    for (; gIter != NULL; gIter = gIter->next) {
+    for (GList *gIter = rsc->priv->children;
+         gIter != NULL; gIter = gIter->next) {
+
         pcmk_resource_t *child_rsc = (pcmk_resource_t *) gIter->data;
 
-        if (child_rsc->fns->active(child_rsc, all)) {
+        if (child_rsc->priv->fns->active(child_rsc, all)) {
             c_any = TRUE;
         } else {
             c_all = FALSE;
@@ -257,85 +257,6 @@ group_active(pcmk_resource_t *rsc, gboolean all)
     return TRUE;
 }
 
-/*!
- * \internal
- * \deprecated This function will be removed in a future release
- */
-static void
-group_print_xml(pcmk_resource_t *rsc, const char *pre_text, long options,
-                void *print_data)
-{
-    GList *gIter = rsc->children;
-    char *child_text = crm_strdup_printf("%s     ", pre_text);
-
-    status_print("%s<group " PCMK_XA_ID "=\"%s\" ", pre_text, rsc->id);
-    status_print("number_resources=\"%d\" ", g_list_length(rsc->children));
-    status_print(">\n");
-
-    for (; gIter != NULL; gIter = gIter->next) {
-        pcmk_resource_t *child_rsc = (pcmk_resource_t *) gIter->data;
-
-        child_rsc->fns->print(child_rsc, child_text, options, print_data);
-    }
-
-    status_print("%s</group>\n", pre_text);
-    free(child_text);
-}
-
-/*!
- * \internal
- * \deprecated This function will be removed in a future release
- */
-void
-group_print(pcmk_resource_t *rsc, const char *pre_text, long options,
-            void *print_data)
-{
-    char *child_text = NULL;
-    GList *gIter = rsc->children;
-
-    if (pre_text == NULL) {
-        pre_text = " ";
-    }
-
-    if (options & pe_print_xml) {
-        group_print_xml(rsc, pre_text, options, print_data);
-        return;
-    }
-
-    child_text = crm_strdup_printf("%s    ", pre_text);
-
-    status_print("%sResource Group: %s", pre_text ? pre_text : "", rsc->id);
-
-    if (options & pe_print_html) {
-        status_print("\n<ul>\n");
-
-    } else if ((options & pe_print_log) == 0) {
-        status_print("\n");
-    }
-
-    if (options & pe_print_brief) {
-        print_rscs_brief(rsc->children, child_text, options, print_data, TRUE);
-
-    } else {
-        for (; gIter != NULL; gIter = gIter->next) {
-            pcmk_resource_t *child_rsc = (pcmk_resource_t *) gIter->data;
-
-            if (options & pe_print_html) {
-                status_print("<li>\n");
-            }
-            child_rsc->fns->print(child_rsc, child_text, options, print_data);
-            if (options & pe_print_html) {
-                status_print("</li>\n");
-            }
-        }
-    }
-
-    if (options & pe_print_html) {
-        status_print("</ul>\n");
-    }
-    free(child_text);
-}
-
 PCMK__OUTPUT_ARGS("group", "uint32_t", "pcmk_resource_t *", "GList *",
                   "GList *")
 int
@@ -345,9 +266,8 @@ pe__group_xml(pcmk__output_t *out, va_list args)
     pcmk_resource_t *rsc = va_arg(args, pcmk_resource_t *);
     GList *only_node = va_arg(args, GList *);
     GList *only_rsc = va_arg(args, GList *);
-    
+
     const char *desc = NULL;
-    GList *gIter = rsc->children;
 
     int rc = pcmk_rc_no_output;
 
@@ -356,11 +276,13 @@ pe__group_xml(pcmk__output_t *out, va_list args)
 
     desc = pe__resource_description(rsc, show_opts);
 
-    if (rsc->fns->is_filtered(rsc, only_rsc, TRUE)) {
+    if (rsc->priv->fns->is_filtered(rsc, only_rsc, TRUE)) {
         return rc;
     }
 
-    for (; gIter != NULL; gIter = gIter->next) {
+    for (GList *gIter = rsc->priv->children;
+         gIter != NULL; gIter = gIter->next) {
+
         pcmk_resource_t *child_rsc = (pcmk_resource_t *) gIter->data;
 
         if (skip_child_rsc(rsc, child_rsc, parent_passes, only_rsc, show_opts)) {
@@ -370,8 +292,9 @@ pe__group_xml(pcmk__output_t *out, va_list args)
         if (rc == pcmk_rc_no_output) {
             char *count = pcmk__itoa(g_list_length(gIter));
             const char *maintenance = pcmk__flag_text(rsc->flags,
-                                                      pcmk_rsc_maintenance);
-            const char *managed = pcmk__flag_text(rsc->flags, pcmk_rsc_managed);
+                                                      pcmk__rsc_maintenance);
+            const char *managed = pcmk__flag_text(rsc->flags,
+                                                  pcmk__rsc_managed);
             const char *disabled = pcmk__btoa(pe__resource_is_disabled(rsc));
 
             rc = pe__name_and_nvpairs_xml(out, true, PCMK_XE_GROUP,
@@ -383,11 +306,11 @@ pe__group_xml(pcmk__output_t *out, va_list args)
                                           PCMK_XA_DESCRIPTION, desc,
                                           NULL);
             free(count);
-            CRM_ASSERT(rc == pcmk_rc_ok);
+            pcmk__assert(rc == pcmk_rc_ok);
         }
 
-        out->message(out, (const char *) child_rsc->xml->name, show_opts,
-                     child_rsc, only_node, only_rsc);
+        out->message(out, (const char *) child_rsc->priv->xml->name,
+                     show_opts, child_rsc, only_node, only_rsc);
     }
 
     if (rc == pcmk_rc_ok) {
@@ -413,17 +336,17 @@ pe__group_default(pcmk__output_t *out, va_list args)
     gboolean parent_passes = pcmk__str_in_list(rsc_printable_id(rsc), only_rsc, pcmk__str_star_matches) ||
                              (strstr(rsc->id, ":") != NULL && pcmk__str_in_list(rsc->id, only_rsc, pcmk__str_star_matches));
 
-    gboolean active = rsc->fns->active(rsc, TRUE);
-    gboolean partially_active = rsc->fns->active(rsc, FALSE);
+    gboolean active = rsc->priv->fns->active(rsc, TRUE);
+    gboolean partially_active = rsc->priv->fns->active(rsc, FALSE);
 
     desc = pe__resource_description(rsc, show_opts);
 
-    if (rsc->fns->is_filtered(rsc, only_rsc, TRUE)) {
+    if (rsc->priv->fns->is_filtered(rsc, only_rsc, TRUE)) {
         return rc;
     }
 
     if (pcmk_is_set(show_opts, pcmk_show_brief)) {
-        GList *rscs = pe__filter_rsc_list(rsc->children, only_rsc);
+        GList *rscs = pe__filter_rsc_list(rsc->priv->children, only_rsc);
 
         if (rscs != NULL) {
             group_header(out, &rc, rsc, !active && partially_active ? inactive_resources(rsc) : 0,
@@ -435,7 +358,8 @@ pe__group_default(pcmk__output_t *out, va_list args)
         }
 
     } else {
-        for (GList *gIter = rsc->children; gIter; gIter = gIter->next) {
+        for (GList *gIter = rsc->priv->children;
+             gIter != NULL; gIter = gIter->next) {
             pcmk_resource_t *child_rsc = (pcmk_resource_t *) gIter->data;
 
             if (skip_child_rsc(rsc, child_rsc, parent_passes, only_rsc, show_opts)) {
@@ -444,8 +368,8 @@ pe__group_default(pcmk__output_t *out, va_list args)
 
             group_header(out, &rc, rsc, !active && partially_active ? inactive_resources(rsc) : 0,
                          pcmk_is_set(show_opts, pcmk_show_inactive_rscs), desc);
-            out->message(out, (const char *) child_rsc->xml->name, show_opts,
-                         child_rsc, only_node, only_rsc);
+            out->message(out, (const char *) child_rsc->priv->xml->name,
+                         show_opts, child_rsc, only_node, only_rsc);
         }
     }
 
@@ -461,16 +385,18 @@ group_free(pcmk_resource_t * rsc)
 
     pcmk__rsc_trace(rsc, "Freeing %s", rsc->id);
 
-    for (GList *gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
+    for (GList *gIter = rsc->priv->children;
+         gIter != NULL; gIter = gIter->next) {
+
         pcmk_resource_t *child_rsc = (pcmk_resource_t *) gIter->data;
 
-        CRM_ASSERT(child_rsc);
+        pcmk__assert(child_rsc != NULL);
         pcmk__rsc_trace(child_rsc, "Freeing child %s", child_rsc->id);
-        child_rsc->fns->free(child_rsc);
+        child_rsc->priv->fns->free(child_rsc);
     }
 
     pcmk__rsc_trace(rsc, "Freeing child list");
-    g_list_free(rsc->children);
+    g_list_free(rsc->priv->children);
 
     common_free(rsc);
 }
@@ -479,11 +405,13 @@ enum rsc_role_e
 group_resource_state(const pcmk_resource_t * rsc, gboolean current)
 {
     enum rsc_role_e group_role = pcmk_role_unknown;
-    GList *gIter = rsc->children;
 
-    for (; gIter != NULL; gIter = gIter->next) {
+    for (GList *gIter = rsc->priv->children;
+         gIter != NULL; gIter = gIter->next) {
+
         pcmk_resource_t *child_rsc = (pcmk_resource_t *) gIter->data;
-        enum rsc_role_e role = child_rsc->fns->state(child_rsc, current);
+        enum rsc_role_e role = child_rsc->priv->fns->state(child_rsc,
+                                                           current);
 
         if (role > group_role) {
             group_role = role;
@@ -510,12 +438,13 @@ pe__group_is_filtered(const pcmk_resource_t *rsc, GList *only_rsc,
     } else if (strstr(rsc->id, ":") != NULL && pcmk__str_in_list(rsc->id, only_rsc, pcmk__str_star_matches)) {
         passes = TRUE;
     } else {
-        for (const GList *iter = rsc->children;
+        for (const GList *iter = rsc->priv->children;
              iter != NULL; iter = iter->next) {
 
             const pcmk_resource_t *child_rsc = iter->data;
 
-            if (!child_rsc->fns->is_filtered(child_rsc, only_rsc, FALSE)) {
+            if (!child_rsc->priv->fns->is_filtered(child_rsc, only_rsc,
+                                                   FALSE)) {
                 passes = TRUE;
                 break;
             }
@@ -536,6 +465,6 @@ pe__group_is_filtered(const pcmk_resource_t *rsc, GList *only_rsc,
 unsigned int
 pe__group_max_per_node(const pcmk_resource_t *rsc)
 {
-    CRM_ASSERT(pcmk__is_group(rsc));
+    pcmk__assert(pcmk__is_group(rsc));
     return 1U;
 }
