@@ -35,9 +35,9 @@ pe__free_digests(gpointer ptr)
     pcmk__op_digest_t *data = ptr;
 
     if (data != NULL) {
-        free_xml(data->params_all);
-        free_xml(data->params_secure);
-        free_xml(data->params_restart);
+        pcmk__xml_free(data->params_all);
+        pcmk__xml_free(data->params_secure);
+        pcmk__xml_free(data->params_restart);
 
         free(data->digest_all_calc);
         free(data->digest_restart_calc);
@@ -145,7 +145,7 @@ calculate_main_digest(pcmk__op_digest_t *data, pcmk_resource_t *rsc,
      * result to RSC_DIGEST_RESTART for the case where the main digest doesn't
      * match.
      */
-    params = pcmk__unpack_action_rsc_params(action_config, node->details->attrs,
+    params = pcmk__unpack_action_rsc_params(action_config, node->priv->attrs,
                                             scheduler);
     if ((*interval_ms == 0) && (g_hash_table_size(params) > 0)) {
         data->rc = pcmk__digest_restart;
@@ -161,8 +161,7 @@ calculate_main_digest(pcmk__op_digest_t *data, pcmk_resource_t *rsc,
 
     pcmk__filter_op_for_digest(data->params_all);
 
-    data->digest_all_calc = calculate_operation_digest(data->params_all,
-                                                       op_version);
+    data->digest_all_calc = pcmk__digest_operation(data->params_all);
 }
 
 // Return true if XML attribute name is a Pacemaker-defined fencing parameter
@@ -188,7 +187,7 @@ calculate_secure_digest(pcmk__op_digest_t *data, const pcmk_resource_t *rsc,
                         GHashTable *params, const xmlNode *xml_op,
                         const char *op_version, GHashTable *overrides)
 {
-    const char *class = crm_element_value(rsc->xml, PCMK_XA_CLASS);
+    const char *class = crm_element_value(rsc->priv->xml, PCMK_XA_CLASS);
     const char *secure_list = NULL;
     bool old_version = (compare_version(op_version, "3.16.0") < 0);
 
@@ -239,8 +238,7 @@ calculate_secure_digest(pcmk__op_digest_t *data, const pcmk_resource_t *rsc,
                              CRM_META "_" PCMK_META_TIMEOUT);
     }
 
-    data->digest_secure_calc = calculate_operation_digest(data->params_secure,
-                                                          op_version);
+    data->digest_secure_calc = pcmk__digest_operation(data->params_secure);
 }
 
 /*!
@@ -281,8 +279,7 @@ calculate_restart_digest(pcmk__op_digest_t *data, const xmlNode *xml_op,
     }
 
     value = crm_element_value(xml_op, PCMK_XA_CRM_FEATURE_SET);
-    data->digest_restart_calc = calculate_operation_digest(data->params_restart,
-                                                           value);
+    data->digest_restart_calc = pcmk__digest_operation(data->params_restart);
 }
 
 /*!
@@ -316,7 +313,8 @@ pe__calculate_digests(pcmk_resource_t *rsc, const char *task,
 
     data = calloc(1, sizeof(pcmk__op_digest_t));
     if (data == NULL) {
-        pcmk__sched_err("Could not allocate memory for operation digest");
+        pcmk__sched_err(scheduler,
+                        "Could not allocate memory for operation digest");
         return NULL;
     }
 
@@ -368,12 +366,12 @@ rsc_action_digest(pcmk_resource_t *rsc, const char *task, guint interval_ms,
     pcmk__op_digest_t *data = NULL;
     char *key = pcmk__op_key(rsc->id, task, interval_ms);
 
-    data = g_hash_table_lookup(node->details->digest_cache, key);
+    data = g_hash_table_lookup(node->priv->digest_cache, key);
     if (data == NULL) {
         data = pe__calculate_digests(rsc, task, &interval_ms, node, xml_op,
                                      NULL, calc_secure, scheduler);
-        CRM_ASSERT(data != NULL);
-        g_hash_table_insert(node->details->digest_cache, strdup(key), data);
+        pcmk__assert(data != NULL);
+        g_hash_table_insert(node->priv->digest_cache, strdup(key), data);
     }
     free(key);
     return data;
@@ -402,7 +400,7 @@ rsc_action_digest_cmp(pcmk_resource_t *rsc, const xmlNode *xml_op,
     const char *digest_all;
     const char *digest_restart;
 
-    CRM_ASSERT(node != NULL);
+    pcmk__assert(node != NULL);
 
     op_version = crm_element_value(xml_op, PCMK_XA_CRM_FEATURE_SET);
     digest_all = crm_element_value(xml_op, PCMK__XA_OP_DIGEST);
@@ -411,7 +409,7 @@ rsc_action_digest_cmp(pcmk_resource_t *rsc, const xmlNode *xml_op,
     crm_element_value_ms(xml_op, PCMK_META_INTERVAL, &interval_ms);
     data = rsc_action_digest(rsc, task, interval_ms, node, xml_op,
                              pcmk_is_set(scheduler->flags,
-                                         pcmk_sched_sanitized),
+                                         pcmk__sched_sanitized),
                              scheduler);
 
     if (digest_restart && data->digest_restart_calc && strcmp(data->digest_restart_calc, digest_restart) != 0) {
@@ -578,8 +576,9 @@ pe__compare_fencing_digest(pcmk_resource_t *rsc, const char *agent,
     if (unfencing_digest_matches(rsc->id, agent, data->digest_secure_calc,
                                  node_summary)) {
         data->rc = pcmk__digest_match;
-        if (!pcmk__is_daemon && scheduler->priv != NULL) {
-            pcmk__output_t *out = scheduler->priv;
+        if (!pcmk__is_daemon && (scheduler->priv->out != NULL)) {
+            pcmk__output_t *out = scheduler->priv->out;
+
             out->info(out, "Only 'private' parameters to %s "
                       "for unfencing %s changed", rsc->id,
                       pcmk__node_name(node));
@@ -589,11 +588,11 @@ pe__compare_fencing_digest(pcmk_resource_t *rsc, const char *agent,
 
     // Parameters don't match
     data->rc = pcmk__digest_mismatch;
-    if (pcmk_is_set(scheduler->flags, pcmk_sched_sanitized)
+    if (pcmk_is_set(scheduler->flags, pcmk__sched_sanitized)
         && (data->digest_secure_calc != NULL)) {
 
-        if (scheduler->priv != NULL) {
-            pcmk__output_t *out = scheduler->priv;
+        if (scheduler->priv->out != NULL) {
+            pcmk__output_t *out = scheduler->priv->out;
             char *digest = create_unfencing_summary(rsc->id, agent,
                                                     data->digest_secure_calc);
 
