@@ -17,26 +17,28 @@
  * \internal
  * \brief Check whether a resource creates a guest node
  *
- * If a given resource contains a filler resource that is a remote connection,
- * return that filler resource (or NULL if none is found).
+ * If a given resource contains a launched resource that is a remote connection,
+ * return that launched resource (or NULL if none is found).
  *
  * \param[in] scheduler  Scheduler data
  * \param[in] rsc        Resource to check
  *
- * \return Filler resource with remote connection, or NULL if none found
+ * \return Launched remote connection, or NULL if none found
  */
 pcmk_resource_t *
 pe__resource_contains_guest_node(const pcmk_scheduler_t *scheduler,
                                  const pcmk_resource_t *rsc)
 {
     if ((rsc != NULL) && (scheduler != NULL)
-        && pcmk_is_set(scheduler->flags, pcmk_sched_have_remote_nodes)) {
+        && pcmk_is_set(scheduler->flags, pcmk__sched_have_remote_nodes)) {
 
-        for (GList *gIter = rsc->fillers; gIter != NULL; gIter = gIter->next) {
-            pcmk_resource_t *filler = gIter->data;
+        for (GList *gIter = rsc->priv->launched;
+             gIter != NULL; gIter = gIter->next) {
 
-            if (filler->is_remote_node) {
-                return filler;
+            pcmk_resource_t *launched = gIter->data;
+
+            if (pcmk_is_set(launched->flags, pcmk__rsc_is_remote_connection)) {
+                return launched;
             }
         }
     }
@@ -88,13 +90,14 @@ pe_foreach_guest_node(const pcmk_scheduler_t *scheduler,
     GList *iter;
 
     CRM_CHECK(scheduler && host && host->details && helper, return);
-    if (!pcmk_is_set(scheduler->flags, pcmk_sched_have_remote_nodes)) {
+    if (!pcmk_is_set(scheduler->flags, pcmk__sched_have_remote_nodes)) {
         return;
     }
     for (iter = host->details->running_rsc; iter != NULL; iter = iter->next) {
         pcmk_resource_t *rsc = (pcmk_resource_t *) iter->data;
 
-        if (rsc->is_remote_node && (rsc->container != NULL)) {
+        if (pcmk_is_set(rsc->flags, pcmk__rsc_is_remote_connection)
+            && (rsc->priv->launcher != NULL)) {
             pcmk_node_t *guest_node = pcmk_find_node(scheduler, rsc->id);
 
             if (guest_node) {
@@ -140,7 +143,7 @@ pe_create_remote_xml(xmlNode *parent, const char *uname,
 
     // Add meta-attributes
     xml_sub = pcmk__xe_create(remote, PCMK_XE_META_ATTRIBUTES);
-    crm_xml_set_id(xml_sub, "%s-%s", uname, PCMK_XE_META_ATTRIBUTES);
+    pcmk__xe_set_id(xml_sub, "%s-%s", uname, PCMK_XE_META_ATTRIBUTES);
     crm_create_nvpair_xml(xml_sub, NULL,
                           PCMK__META_INTERNAL_RSC, PCMK_VALUE_TRUE);
     if (container_id) {
@@ -158,7 +161,7 @@ pe_create_remote_xml(xmlNode *parent, const char *uname,
     // Add instance attributes
     if (port || server) {
         xml_sub = pcmk__xe_create(remote, PCMK_XE_INSTANCE_ATTRIBUTES);
-        crm_xml_set_id(xml_sub, "%s-%s", uname, PCMK_XE_INSTANCE_ATTRIBUTES);
+        pcmk__xe_set_id(xml_sub, "%s-%s", uname, PCMK_XE_INSTANCE_ATTRIBUTES);
         if (server) {
             crm_create_nvpair_xml(xml_sub, NULL, PCMK_REMOTE_RA_ADDR, server);
         }
@@ -202,7 +205,8 @@ pe__add_param_check(const xmlNode *rsc_op, pcmk_resource_t *rsc,
     check_op->rsc = rsc;
     check_op->node = node;
     check_op->check_type = flag;
-    scheduler->param_check = g_list_prepend(scheduler->param_check, check_op);
+    scheduler->priv->param_check = g_list_prepend(scheduler->priv->param_check,
+                                                  check_op);
 }
 
 /*!
@@ -219,7 +223,7 @@ pe__foreach_param_check(pcmk_scheduler_t *scheduler,
 {
     CRM_CHECK(scheduler && cb, return);
 
-    for (GList *item = scheduler->param_check;
+    for (GList *item = scheduler->priv->param_check;
          item != NULL; item = item->next) {
         struct check_op *check_op = item->data;
 
@@ -231,8 +235,8 @@ pe__foreach_param_check(pcmk_scheduler_t *scheduler,
 void
 pe__free_param_checks(pcmk_scheduler_t *scheduler)
 {
-    if (scheduler && scheduler->param_check) {
-        g_list_free_full(scheduler->param_check, free);
-        scheduler->param_check = NULL;
+    if ((scheduler != NULL) && (scheduler->priv->param_check != NULL)) {
+        g_list_free_full(scheduler->priv->param_check, free);
+        scheduler->priv->param_check = NULL;
     }
 }
