@@ -27,7 +27,7 @@ static int
 setup(void **state) {
     char *path = NULL;
 
-    crm_xml_init();
+    pcmk__xml_init();
 
     path = crm_strdup_printf("%s/crm_mon.xml", getenv("PCMK_CTS_CLI_DIR"));
     input = pcmk__xml_read(path);
@@ -43,8 +43,7 @@ setup(void **state) {
         return 1;
     }
 
-    pcmk__set_scheduler_flags(scheduler,
-                              pcmk_sched_no_counts|pcmk_sched_no_compat);
+    pcmk__set_scheduler_flags(scheduler, pcmk__sched_no_counts);
     scheduler->input = input;
 
     cluster_status(scheduler);
@@ -55,7 +54,9 @@ setup(void **state) {
     httpd_bundle_0 = pcmk_find_node(scheduler, "httpd-bundle-0");
 
     /* Get references to several resources we use frequently. */
-    for (GList *iter = scheduler->resources; iter != NULL; iter = iter->next) {
+    for (GList *iter = scheduler->priv->resources;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *rsc = (pcmk_resource_t *) iter->data;
 
         if (strcmp(rsc->id, "exim-group") == 0) {
@@ -79,13 +80,13 @@ setup(void **state) {
 static int
 teardown(void **state) {
     pe_free_working_set(scheduler);
-
+    pcmk__xml_cleanup();
     return 0;
 }
 
 static void
 bad_args(void **state) {
-    pcmk_resource_t *rsc = g_list_first(scheduler->resources)->data;
+    pcmk_resource_t *rsc = g_list_first(scheduler->priv->resources)->data;
     char *id = rsc->id;
     char *name = NULL;
 
@@ -118,7 +119,9 @@ primitive_rsc(void **state) {
     pcmk_resource_t *dummy = NULL;
 
     /* Find the "dummy" resource, which is the only one with that ID in the set. */
-    for (GList *iter = scheduler->resources; iter != NULL; iter = iter->next) {
+    for (GList *iter = scheduler->priv->resources;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *rsc = (pcmk_resource_t *) iter->data;
 
         if (strcmp(rsc->id, "dummy") == 0) {
@@ -211,7 +214,9 @@ group_member_rsc(void **state) {
     pcmk_resource_t *public_ip = NULL;
 
     /* Find the "Public-IP" resource, a member of "exim-group". */
-    for (GList *iter = exim_group->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = exim_group->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *rsc = (pcmk_resource_t *) iter->data;
 
         if (strcmp(rsc->id, "Public-IP") == 0) {
@@ -252,7 +257,9 @@ inactive_group_member_rsc(void **state) {
     pcmk_resource_t *inactive_dummy_1 = NULL;
 
     /* Find the "inactive-dummy-1" resource, a member of "inactive-group". */
-    for (GList *iter = inactive_group->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = inactive_group->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *rsc = (pcmk_resource_t *) iter->data;
 
         if (strcmp(rsc->id, "inactive-dummy-1") == 0) {
@@ -352,7 +359,9 @@ clone_instance_rsc(void **state) {
     pcmk_resource_t *promotable_1 = NULL;
 
     /* Find the "promotable-rsc:0" and "promotable-rsc:1" resources, members of "promotable-clone". */
-    for (GList *iter = promotable_clone->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = promotable_clone->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *rsc = (pcmk_resource_t *) iter->data;
 
         if (strcmp(rsc->id, "promotable-rsc:0") == 0) {
@@ -491,7 +500,9 @@ renamed_rsc(void **state) {
     pcmk_resource_t *promotable_1 = NULL;
 
     /* Find the "promotable-rsc:0" and "promotable-rsc:1" resources, members of "promotable-clone". */
-    for (GList *iter = promotable_clone->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = promotable_clone->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *rsc = (pcmk_resource_t *) iter->data;
 
         if (strcmp(rsc->id, "promotable-rsc:0") == 0) {
@@ -504,7 +515,7 @@ renamed_rsc(void **state) {
     assert_non_null(promotable_0);
     assert_non_null(promotable_1);
 
-    // Passes because pcmk_rsc_match_history means base name matches clone_name
+    // Passes because pcmk_rsc_match_history means base name matches history_id
     assert_ptr_equal(promotable_0,
                      native_find_rsc(promotable_0, "promotable-rsc", NULL,
                                      pcmk_rsc_match_history));
@@ -710,7 +721,9 @@ clone_group_instance_rsc(void **rsc) {
     pcmk_resource_t *mysql_group_1 = NULL;
 
     /* Find the "mysql-group:0" and "mysql-group:1" resources, members of "mysql-clone-group". */
-    for (GList *iter = mysql_clone_group->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = mysql_clone_group->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *rsc = (pcmk_resource_t *) iter->data;
 
         if (strcmp(rsc->id, "mysql-group:0") == 0) {
@@ -848,11 +861,14 @@ clone_group_member_rsc(void **state) {
     pcmk_resource_t *mysql_proxy = NULL;
 
     /* Find the "mysql-proxy" resource, a member of "mysql-group". */
-    for (GList *iter = mysql_clone_group->children; iter != NULL; iter = iter->next) {
+    for (GList *iter = mysql_clone_group->priv->children;
+         iter != NULL; iter = iter->next) {
+
         pcmk_resource_t *rsc = (pcmk_resource_t *) iter->data;
 
         if (strcmp(rsc->id, "mysql-group:0") == 0) {
-            for (GList *iter2 = rsc->children; iter2 != NULL; iter2 = iter2->next) {
+            for (GList *iter2 = rsc->priv->children;
+                 iter2 != NULL; iter2 = iter2->next) {
                 pcmk_resource_t *child = (pcmk_resource_t *) iter2->data;
 
                 if (strcmp(child->id, "mysql-proxy:0") == 0) {
