@@ -23,26 +23,26 @@ check_defaults(void **state) {
     pcmk_scheduler_t *scheduler = pcmk__assert_alloc(1,
                                                      sizeof(pcmk_scheduler_t));
 
+    scheduler->priv = pcmk__assert_alloc(1, sizeof(pcmk__scheduler_private_t));
     set_working_set_defaults(scheduler);
 
-    flags = pcmk_sched_symmetric_cluster
-            |pcmk_sched_stop_removed_resources
-            |pcmk_sched_cancel_removed_actions;
+    flags = pcmk__sched_symmetric_cluster
+#if PCMK__CONCURRENT_FENCING_DEFAULT_TRUE
+            |pcmk__sched_concurrent_fencing
+#endif
+            |pcmk__sched_stop_removed_resources
+            |pcmk__sched_cancel_removed_actions;
 
-    if (!strcmp(PCMK__CONCURRENT_FENCING_DEFAULT, PCMK_VALUE_TRUE)) {
-        flags |= pcmk_sched_concurrent_fencing;
-    }
-
-
-    assert_null(scheduler->priv);
-    assert_int_equal(scheduler->order_id, 1);
-    assert_int_equal(scheduler->action_id, 1);
+    assert_null(scheduler->priv->out);
+    assert_int_equal(scheduler->priv->next_ordering_id, 1);
+    assert_int_equal(scheduler->priv->next_action_id, 1);
     assert_int_equal(scheduler->no_quorum_policy, pcmk_no_quorum_stop);
     assert_int_equal(scheduler->flags, flags);
 
     /* Avoid calling pe_free_working_set here so we don't artificially
      * inflate the coverage numbers.
      */
+    free(scheduler->priv);
     free(scheduler);
 }
 
