@@ -9,10 +9,6 @@
 
 #include <crm_internal.h>
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
@@ -29,20 +25,15 @@
 #include <crm/common/xml.h>
 #include "services_private.h"
 #include "services_ocf.h"
-#include "services_lsb.h"
 
-#if SUPPORT_UPSTART
-#  include <upstart.h>
+#if PCMK__ENABLE_LSB
+#include "services_lsb.h"
 #endif
 
 #if SUPPORT_SYSTEMD
 #  include <systemd.h>
 #endif
 
-#if SUPPORT_NAGIOS
-#  include <services_nagios.h>
-#endif
-
 /* TODO: Develop a rollover strategy */
 
 static int operations = 0;
@@ -64,16 +55,18 @@ static void handle_blocked_ops(void);
  *
  * \return Service class if found, NULL otherwise
  *
- * \note The priority is LSB, then systemd, then upstart. It would be preferable
- *       to put systemd first, but LSB merely requires a file existence check,
- *       while systemd requires contacting D-Bus.
+ * \note The priority is LSB then systemd. It would be preferable to put systemd
+ *       first, but LSB merely requires a file existence check, while systemd
+ *       requires contacting DBus.
  */
 const char *
 resources_find_service_class(const char *agent)
 {
+#if PCMK__ENABLE_LSB
     if (services__lsb_agent_exists(agent)) {
         return PCMK_RESOURCE_CLASS_LSB;
     }
+#endif
 
 #if SUPPORT_SYSTEMD
     if (systemd_unit_exists(agent)) {
@@ -81,11 +74,6 @@ resources_find_service_class(const char *agent)
     }
 #endif
 
-#if SUPPORT_UPSTART
-    if (upstart_job_exists(agent)) {
-        return PCMK_RESOURCE_CLASS_UPSTART;
-    }
-#endif
     return NULL;
 }
 
@@ -99,18 +87,18 @@ init_recurring_actions(void)
 
 /*!
  * \internal
- * \brief Check whether op is in-flight systemd or upstart op
+ * \brief Check whether op is in-flight systemd op
  *
  * \param[in] op  Operation to check
  *
- * \return TRUE if op is in-flight systemd or upstart op
+ * \return TRUE if op is in-flight systemd op
  */
 static inline gboolean
-inflight_systemd_or_upstart(const svc_action_t *op)
+inflight_systemd(const svc_action_t *op)
 {
-    return pcmk__strcase_any_of(op->standard, PCMK_RESOURCE_CLASS_SYSTEMD,
-                           PCMK_RESOURCE_CLASS_UPSTART, NULL) &&
-           g_list_find(inflight_ops, op) != NULL;
+    return pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_SYSTEMD,
+                        pcmk__str_casei)
+           && (g_list_find(inflight_ops, op) != NULL);
 }
 
 /*!
@@ -130,21 +118,33 @@ expand_resource_class(const char *rsc, const char *standard, const char *agent)
 {
     char *expanded_class = NULL;
 
+#if PCMK__ENABLE_SERVICE
     if (strcasecmp(standard, PCMK_RESOURCE_CLASS_SERVICE) == 0) {
         const char *found_class = resources_find_service_class(agent);
 
-        if (found_class) {
+        if (found_class != NULL) {
             crm_debug("Found %s agent %s for %s", found_class, agent, rsc);
-            expanded_class = strdup(found_class);
+            expanded_class = pcmk__str_copy(found_class);
         } else {
-            crm_info("Assuming resource class lsb for agent %s for %s",
-                     agent, rsc);
-            expanded_class = strdup(PCMK_RESOURCE_CLASS_LSB);
+            const char *default_standard = NULL;
+
+#if PCMK__ENABLE_LSB
+            default_standard = PCMK_RESOURCE_CLASS_LSB;
+#elif SUPPORT_SYSTEMD
+            default_standard = PCMK_RESOURCE_CLASS_SYSTEMD;
+#else
+#error No standards supported for service alias (configure script bug)
+#endif
+            crm_info("Assuming resource class %s for agent %s for %s",
+                     default_standard, agent, rsc);
+            expanded_class = pcmk__str_copy(default_standard);
         }
-    } else {
-        expanded_class = strdup(standard);
     }
-    CRM_ASSERT(expanded_class);
+#endif
+
+    if (expanded_class == NULL) {
+        expanded_class = pcmk__str_copy(standard);
+    }
     return expanded_class;
 }
 
@@ -303,20 +303,13 @@ services__create_resource_action(const char *name, const char *standard,
     if (strcasecmp(op->standard, PCMK_RESOURCE_CLASS_OCF) == 0) {
         rc = services__ocf_prepare(op);
 
+#if PCMK__ENABLE_LSB
     } else if (strcasecmp(op->standard, PCMK_RESOURCE_CLASS_LSB) == 0) {
         rc = services__lsb_prepare(op);
-
+#endif
 #if SUPPORT_SYSTEMD
     } else if (strcasecmp(op->standard, PCMK_RESOURCE_CLASS_SYSTEMD) == 0) {
         rc = services__systemd_prepare(op);
-#endif
-#if SUPPORT_UPSTART
-    } else if (strcasecmp(op->standard, PCMK_RESOURCE_CLASS_UPSTART) == 0) {
-        rc = services__upstart_prepare(op);
-#endif
-#if SUPPORT_NAGIOS
-    } else if (strcasecmp(op->standard, PCMK_RESOURCE_CLASS_NAGIOS) == 0) {
-        rc = services__nagios_prepare(op);
 #endif
     } else {
         crm_info("Unknown resource standard: %s", op->standard);
@@ -434,10 +427,9 @@ services_alert_create(const char *id, const char *exec, int timeout,
  * \return pcmk_ok on success, -errno otherwise
  *
  * \note This will have no effect unless the process executing the action runs
- *       as root, and the action is not a systemd or upstart action.
- *       We could implement this for systemd by adding User= and Group= to
- *       [Service] in the override file, but that seems more likely to cause
- *       problems than be useful.
+ *       as root and the action is not a systemd action. We could implement this
+ *       for systemd by adding User= and Group= to [Service] in the override
+ *       file, but that seems more likely to cause problems than be useful.
  */
 int
 services_action_user(svc_action_t *op, const char *user)
@@ -556,21 +548,11 @@ services_result2ocf(const char *standard, const char *action, int exit_status)
         return services__systemd2ocf(exit_status);
 #endif
 
-#if SUPPORT_UPSTART
-    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_UPSTART,
-                            pcmk__str_casei)) {
-        return services__upstart2ocf(exit_status);
-#endif
-
-#if SUPPORT_NAGIOS
-    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_NAGIOS,
-                            pcmk__str_casei)) {
-        return services__nagios2ocf(exit_status);
-#endif
-
+#if PCMK__ENABLE_LSB
     } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_LSB,
                             pcmk__str_casei)) {
         return services__lsb2ocf(action, exit_status);
+#endif
 
     } else {
         crm_warn("Treating result from unknown standard '%s' as OCF",
@@ -695,8 +677,8 @@ services_action_cancel(const char *name, const char *action, guint interval_ms)
     }
 
 #if HAVE_DBUS
-    // In-flight systemd and upstart ops don't have a pid
-    if (inflight_systemd_or_upstart(op)) {
+    // In-flight systemd ops don't have a pid
+    if (inflight_systemd(op)) {
         inflight_ops = g_list_remove(inflight_ops, op);
 
         /* This will cause any result that comes in later to be discarded, so we
@@ -741,7 +723,7 @@ services_action_kick(const char *name, const char *action, guint interval_ms)
     }
 
 
-    if (op->pid || inflight_systemd_or_upstart(op)) {
+    if (op->pid || inflight_systemd(op)) {
         return TRUE;
     } else {
         if (op->opaque->repeat_timer) {
@@ -812,13 +794,6 @@ handle_duplicate_recurring(svc_action_t *op)
 static int
 execute_action(svc_action_t *op)
 {
-#if SUPPORT_UPSTART
-    if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_UPSTART,
-                     pcmk__str_casei)) {
-        return services__execute_upstart(op);
-    }
-#endif
-
 #if SUPPORT_SYSTEMD
     if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_SYSTEMD,
                      pcmk__str_casei)) {
@@ -836,7 +811,7 @@ services_add_inflight_op(svc_action_t * op)
         return;
     }
 
-    CRM_ASSERT(op->synchronous == FALSE);
+    pcmk__assert(op->synchronous == FALSE);
 
     /* keep track of ops that are in-flight to avoid collisions in the same namespace */
     if (op->rsc) {
@@ -912,7 +887,7 @@ is_op_blocked(const char *rsc)
 
     for (gIter = inflight_ops; gIter != NULL; gIter = gIter->next) {
         op = gIter->data;
-        if (pcmk__str_eq(op->rsc, rsc, pcmk__str_casei)) {
+        if (pcmk__str_eq(op->rsc, rsc, pcmk__str_none)) {
             return TRUE;
         }
     }
@@ -987,6 +962,7 @@ execute_metadata_action(svc_action_t *op)
         return EINVAL;
     }
 
+#if PCMK__ENABLE_SERVICE
     if (!strcmp(class, PCMK_RESOURCE_CLASS_SERVICE)) {
         class = resources_find_service_class(op->agent);
     }
@@ -998,17 +974,13 @@ execute_metadata_action(svc_action_t *op)
                              "Agent standard could not be determined");
         return EINVAL;
     }
+#endif
 
+#if PCMK__ENABLE_LSB
     if (pcmk__str_eq(class, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)) {
         return pcmk_legacy2rc(services__get_lsb_metadata(op->agent,
                                                          &op->stdout_data));
     }
-
-#if SUPPORT_NAGIOS
-    if (pcmk__str_eq(class, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
-        return pcmk_legacy2rc(services__get_nagios_metadata(op->agent,
-                                                            &op->stdout_data));
-    }
 #endif
 
     return execute_action(op);
@@ -1061,8 +1033,14 @@ resources_list_standards(void)
     GList *standards = NULL;
 
     standards = g_list_append(standards, strdup(PCMK_RESOURCE_CLASS_OCF));
-    standards = g_list_append(standards, strdup(PCMK_RESOURCE_CLASS_LSB));
+
+#if PCMK__ENABLE_SERVICE
     standards = g_list_append(standards, strdup(PCMK_RESOURCE_CLASS_SERVICE));
+#endif
+
+#if PCMK__ENABLE_LSB
+    standards = g_list_append(standards, strdup(PCMK_RESOURCE_CLASS_LSB));
+#endif
 
 #if SUPPORT_SYSTEMD
     {
@@ -1076,30 +1054,6 @@ resources_list_standards(void)
     }
 #endif
 
-#if SUPPORT_UPSTART
-    {
-        GList *agents = upstart_job_listall();
-
-        if (agents != NULL) {
-            standards = g_list_append(standards,
-                                      strdup(PCMK_RESOURCE_CLASS_UPSTART));
-            g_list_free_full(agents, free);
-        }
-    }
-#endif
-
-#if SUPPORT_NAGIOS
-    {
-        GList *agents = services__list_nagios_agents();
-
-        if (agents != NULL) {
-            standards = g_list_append(standards,
-                                      strdup(PCMK_RESOURCE_CLASS_NAGIOS));
-            g_list_free_full(agents, free);
-        }
-    }
-#endif
-
     return standards;
 }
 
@@ -1117,11 +1071,14 @@ GList *
 resources_list_agents(const char *standard, const char *provider)
 {
     if ((standard == NULL)
-        || (strcasecmp(standard, PCMK_RESOURCE_CLASS_SERVICE) == 0)) {
+#if PCMK__ENABLE_SERVICE
+        || (strcasecmp(standard, PCMK_RESOURCE_CLASS_SERVICE) == 0)
+#endif
+        ) {
 
         GList *tmp1;
         GList *tmp2;
-        GList *result = services__list_lsb_agents();
+        GList *result = NULL;
 
         if (standard == NULL) {
             tmp1 = result;
@@ -1130,6 +1087,11 @@ resources_list_agents(const char *standard, const char *provider)
                 result = g_list_concat(tmp1, tmp2);
             }
         }
+
+#if PCMK__ENABLE_LSB
+        result = g_list_concat(result, services__list_lsb_agents());
+#endif
+
 #if SUPPORT_SYSTEMD
         tmp1 = result;
         tmp2 = systemd_unit_listall();
@@ -1138,31 +1100,17 @@ resources_list_agents(const char *standard, const char *provider)
         }
 #endif
 
-#if SUPPORT_UPSTART
-        tmp1 = result;
-        tmp2 = upstart_job_listall();
-        if (tmp2) {
-            result = g_list_concat(tmp1, tmp2);
-        }
-#endif
-
         return result;
 
     } else if (strcasecmp(standard, PCMK_RESOURCE_CLASS_OCF) == 0) {
         return resources_os_list_ocf_agents(provider);
+#if PCMK__ENABLE_LSB
     } else if (strcasecmp(standard, PCMK_RESOURCE_CLASS_LSB) == 0) {
         return services__list_lsb_agents();
+#endif
 #if SUPPORT_SYSTEMD
     } else if (strcasecmp(standard, PCMK_RESOURCE_CLASS_SYSTEMD) == 0) {
         return systemd_unit_listall();
-#endif
-#if SUPPORT_UPSTART
-    } else if (strcasecmp(standard, PCMK_RESOURCE_CLASS_UPSTART) == 0) {
-        return upstart_job_listall();
-#endif
-#if SUPPORT_NAGIOS
-    } else if (strcasecmp(standard, PCMK_RESOURCE_CLASS_NAGIOS) == 0) {
-        return services__list_nagios_agents();
 #endif
     }
 
@@ -1209,43 +1157,38 @@ resources_agent_exists(const char *standard, const char *provider, const char *a
         goto done;
     }
 
+#if PCMK__ENABLE_SERVICE
     if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_SERVICE, pcmk__str_casei)) {
+#if PCMK__ENABLE_LSB
         if (services__lsb_agent_exists(agent)) {
             rc = TRUE;
-#if SUPPORT_SYSTEMD
-        } else if (systemd_unit_exists(agent)) {
-            rc = TRUE;
-#endif
-
-#if SUPPORT_UPSTART
-        } else if (upstart_job_exists(agent)) {
-            rc = TRUE;
-#endif
-        } else {
-            rc = FALSE;
+            goto done;
         }
+#endif
+#if SUPPORT_SYSTEMD
+        if (systemd_unit_exists(agent)) {
+            rc = TRUE;
+            goto done;
+        }
+#endif
+        rc = FALSE;
+        goto done;
+    }
+#endif
 
-    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_OCF, pcmk__str_casei)) {
+    if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_OCF, pcmk__str_casei)) {
         rc = services__ocf_agent_exists(provider, agent);
 
+#if PCMK__ENABLE_LSB
     } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)) {
         rc = services__lsb_agent_exists(agent);
+#endif
 
 #if SUPPORT_SYSTEMD
     } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_SYSTEMD, pcmk__str_casei)) {
         rc = systemd_unit_exists(agent);
 #endif
 
-#if SUPPORT_UPSTART
-    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_UPSTART, pcmk__str_casei)) {
-        rc = upstart_job_exists(agent);
-#endif
-
-#if SUPPORT_NAGIOS
-    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
-        rc = services__nagios_agent_exists(agent);
-#endif
-
     } else {
         rc = FALSE;
     }
@@ -1313,7 +1256,7 @@ services__format_result(svc_action_t *action, int agent_status,
     if (format != NULL) {
         va_start(ap, format);
         len = vasprintf(&reason, format, ap);
-        CRM_ASSERT(len > 0);
+        pcmk__assert(len > 0);
         va_end(ap);
     }
     free(action->opaque->exit_reason);
