@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2023 the Pacemaker project contributors
+ * Copyright 2010-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -9,10 +9,6 @@
 
 #include <crm_internal.h>
 
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
-#endif
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -66,7 +62,7 @@ sigchld_setup(struct sigchld_data_s *data)
     // Block SIGCHLD (saving previous set of blocked signals to restore later)
     if (sigprocmask(SIG_BLOCK, &(data->mask), &(data->old_mask)) < 0) {
         crm_info("Wait for child process completion failed: %s "
-                 CRM_XS " source=sigprocmask", pcmk_rc_str(errno));
+                 QB_XS " source=sigprocmask", pcmk_rc_str(errno));
         return false;
     }
 
@@ -86,7 +82,7 @@ sigchld_open(struct sigchld_data_s *data)
     fd = signalfd(-1, &(data->mask), SFD_NONBLOCK);
     if (fd < 0) {
         crm_info("Wait for child process completion failed: %s "
-                 CRM_XS " source=signalfd", pcmk_rc_str(errno));
+                 QB_XS " source=signalfd", pcmk_rc_str(errno));
     }
     return fd;
 }
@@ -113,7 +109,7 @@ sigchld_received(int fd, int pid, struct sigchld_data_s *data)
     s = read(fd, &fdsi, sizeof(struct signalfd_siginfo));
     if (s != sizeof(struct signalfd_siginfo)) {
         crm_info("Wait for child process completion failed: %s "
-                 CRM_XS " source=read", pcmk_rc_str(errno));
+                 QB_XS " source=read", pcmk_rc_str(errno));
 
     } else if (fdsi.ssi_signo == SIGCHLD) {
         if (fdsi.ssi_pid == pid) {
@@ -172,7 +168,7 @@ sigchld_handler(void)
         && (last_sigchld_data->pipe_fd[1] >= 0)
         && (write(last_sigchld_data->pipe_fd[1], "", 1) == -1)) {
         crm_info("Wait for child process completion failed: %s "
-                 CRM_XS " source=write", pcmk_rc_str(errno));
+                 QB_XS " source=write", pcmk_rc_str(errno));
     }
 }
 
@@ -185,18 +181,18 @@ sigchld_setup(struct sigchld_data_s *data)
 
     if (pipe(data->pipe_fd) == -1) {
         crm_info("Wait for child process completion failed: %s "
-                 CRM_XS " source=pipe", pcmk_rc_str(errno));
+                 QB_XS " source=pipe", pcmk_rc_str(errno));
         return false;
     }
 
     rc = pcmk__set_nonblocking(data->pipe_fd[0]);
     if (rc != pcmk_rc_ok) {
-        crm_info("Could not set pipe input non-blocking: %s " CRM_XS " rc=%d",
+        crm_info("Could not set pipe input non-blocking: %s " QB_XS " rc=%d",
                  pcmk_rc_str(rc), rc);
     }
     rc = pcmk__set_nonblocking(data->pipe_fd[1]);
     if (rc != pcmk_rc_ok) {
-        crm_info("Could not set pipe output non-blocking: %s " CRM_XS " rc=%d",
+        crm_info("Could not set pipe output non-blocking: %s " QB_XS " rc=%d",
                  pcmk_rc_str(rc), rc);
     }
 
@@ -206,7 +202,7 @@ sigchld_setup(struct sigchld_data_s *data)
     sigemptyset(&(data->sa.sa_mask));
     if (sigaction(SIGCHLD, &(data->sa), &(data->old_sa)) < 0) {
         crm_info("Wait for child process completion failed: %s "
-                 CRM_XS " source=sigaction", pcmk_rc_str(errno));
+                 QB_XS " source=sigaction", pcmk_rc_str(errno));
     }
 
     data->ignored = false;
@@ -283,15 +279,21 @@ close_pipe(int fildes[])
     }
 }
 
+#define out_type(is_stderr) ((is_stderr)? "stderr" : "stdout")
+
+// Maximum number of bytes of stdout or stderr we'll accept
+#define MAX_OUTPUT (10 * 1024 * 1024)
+
 static gboolean
 svc_read_output(int fd, svc_action_t * op, bool is_stderr)
 {
     char *data = NULL;
-    int rc = 0, len = 0;
+    ssize_t rc = 0;
+    size_t len = 0;
+    size_t discarded = 0;
     char buf[500];
     static const size_t buf_read_len = sizeof(buf) - 1;
 
-
     if (fd < 0) {
         crm_trace("No fd for %s", op->id);
         return FALSE;
@@ -300,34 +302,45 @@ svc_read_output(int fd, svc_action_t * op, bool is_stderr)
     if (is_stderr && op->stderr_data) {
         len = strlen(op->stderr_data);
         data = op->stderr_data;
-        crm_trace("Reading %s stderr into offset %d", op->id, len);
+        crm_trace("Reading %s stderr into offset %lld",
+                  op->id, (long long) len);
 
     } else if (is_stderr == FALSE && op->stdout_data) {
         len = strlen(op->stdout_data);
         data = op->stdout_data;
-        crm_trace("Reading %s stdout into offset %d", op->id, len);
+        crm_trace("Reading %s stdout into offset %lld",
+                  op->id, (long long) len);
 
     } else {
-        crm_trace("Reading %s %s into offset %d", op->id, is_stderr?"stderr":"stdout", len);
+        crm_trace("Reading %s %s", op->id, out_type(is_stderr));
     }
 
     do {
+        errno = 0;
         rc = read(fd, buf, buf_read_len);
         if (rc > 0) {
-            buf[rc] = 0;
-            crm_trace("Got %d chars: %.80s", rc, buf);
-            data = pcmk__realloc(data, len + rc + 1);
-            len += sprintf(data + len, "%s", buf);
+            if (len < MAX_OUTPUT) {
+                buf[rc] = 0;
+                crm_trace("Received %lld bytes of %s %s: %.80s",
+                          (long long) rc, op->id, out_type(is_stderr), buf);
+                data = pcmk__realloc(data, len + rc + 1);
+                strcpy(data + len, buf);
+                len += rc;
+            } else {
+                discarded += rc;
+            }
 
-        } else if (errno != EINTR) {
-            /* error or EOF
-             * Cleanup happens in pipe_done()
-             */
-            rc = FALSE;
+        } else if (errno != EINTR) { // Fatal error or EOF
+            rc = 0;
             break;
         }
+    } while ((rc == buf_read_len) || (rc < 0));
 
-    } while (rc == buf_read_len || rc < 0);
+    if (discarded > 0) {
+        crm_warn("Truncated %s %s to %lld bytes (discarded %lld)",
+                 op->id, out_type(is_stderr), (long long) len,
+                 (long long) discarded);
+    }
 
     if (is_stderr) {
         op->stderr_data = data;
@@ -335,7 +348,7 @@ svc_read_output(int fd, svc_action_t * op, bool is_stderr)
         op->stdout_data = data;
     }
 
-    return rc;
+    return rc != 0;
 }
 
 static int
@@ -453,7 +466,7 @@ add_action_env_vars(const svc_action_t *op)
 
     set_ocf_env("OCF_RA_VERSION_MAJOR", PCMK_OCF_MAJOR_VERSION, NULL);
     set_ocf_env("OCF_RA_VERSION_MINOR", PCMK_OCF_MINOR_VERSION, NULL);
-    set_ocf_env("OCF_ROOT", OCF_ROOT_DIR, NULL);
+    set_ocf_env("OCF_ROOT", PCMK_OCF_ROOT, NULL);
     set_ocf_env("OCF_EXIT_REASON_PREFIX", PCMK_OCF_REASON_PREFIX, NULL);
 
     if (op->rsc) {
@@ -475,7 +488,9 @@ pipe_in_single_parameter(gpointer key, gpointer value, gpointer user_data)
 {
     svc_action_t *op = user_data;
     char *buffer = crm_strdup_printf("%s=%s\n", (char *)key, (char *) value);
-    int ret, total = 0, len = strlen(buffer);
+    size_t len = strlen(buffer);
+    size_t total = 0;
+    ssize_t ret = 0;
 
     do {
         errno = 0;
@@ -483,7 +498,6 @@ pipe_in_single_parameter(gpointer key, gpointer value, gpointer user_data)
         if (ret > 0) {
             total += ret;
         }
-
     } while ((errno == EINTR) && (total < len));
     free(buffer);
 }
@@ -549,9 +563,9 @@ services__finalize_async_op(svc_action_t *op)
             services__set_cancelled(op);
             cancel_recurring_action(op);
         } else {
-            op->opaque->repeat_timer = g_timeout_add(op->interval_ms,
-                                                     recurring_action_timer,
-                                                     (void *) op);
+            op->opaque->repeat_timer = pcmk__create_timer(op->interval_ms,
+                                                          recurring_action_timer,
+                                                          op);
         }
     }
 
@@ -754,16 +768,12 @@ services__generic_error(const svc_action_t *op)
         return PCMK_OCF_UNKNOWN_ERROR;
     }
 
+#if PCMK__ENABLE_LSB
     if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)
         && pcmk__str_eq(op->action, PCMK_ACTION_STATUS, pcmk__str_casei)) {
 
         return PCMK_LSB_STATUS_UNKNOWN;
     }
-
-#if SUPPORT_NAGIOS
-    if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
-        return NAGIOS_STATE_UNKNOWN;
-    }
 #endif
 
     return PCMK_OCF_UNKNOWN_ERROR;
@@ -789,16 +799,12 @@ services__not_installed_error(const svc_action_t *op)
         return PCMK_OCF_UNKNOWN_ERROR;
     }
 
+#if PCMK__ENABLE_LSB
     if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)
         && pcmk__str_eq(op->action, PCMK_ACTION_STATUS, pcmk__str_casei)) {
 
         return PCMK_LSB_STATUS_NOT_INSTALLED;
     }
-
-#if SUPPORT_NAGIOS
-    if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
-        return NAGIOS_STATE_UNKNOWN;
-    }
 #endif
 
     return PCMK_OCF_NOT_INSTALLED;
@@ -824,16 +830,12 @@ services__authorization_error(const svc_action_t *op)
         return PCMK_OCF_UNKNOWN_ERROR;
     }
 
+#if PCMK__ENABLE_LSB
     if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)
         && pcmk__str_eq(op->action, PCMK_ACTION_STATUS, pcmk__str_casei)) {
 
         return PCMK_LSB_STATUS_INSUFFICIENT_PRIV;
     }
-
-#if SUPPORT_NAGIOS
-    if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
-        return NAGIOS_INSUFFICIENT_PRIV;
-    }
 #endif
 
     return PCMK_OCF_INSUFFICIENT_PRIV;
@@ -860,16 +862,12 @@ services__configuration_error(const svc_action_t *op, bool is_fatal)
         return PCMK_OCF_UNKNOWN_ERROR;
     }
 
+#if PCMK__ENABLE_LSB
     if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_LSB, pcmk__str_casei)
         && pcmk__str_eq(op->action, PCMK_ACTION_STATUS, pcmk__str_casei)) {
 
         return PCMK_LSB_NOT_CONFIGURED;
     }
-
-#if SUPPORT_NAGIOS
-    if (pcmk__str_eq(op->standard, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
-        return NAGIOS_STATE_UNKNOWN;
-    }
 #endif
 
     return is_fatal? PCMK_OCF_NOT_CONFIGURED : PCMK_OCF_INVALID_PARAM;
@@ -935,6 +933,7 @@ exit_child(const svc_action_t *op, int exit_status, const char *exit_reason)
                         pcmk__str_none)) {
         fprintf(stderr, PCMK_OCF_REASON_PREFIX "%s\n", exit_reason);
     }
+    pcmk_common_cleanup();
     _exit(exit_status);
 }
 
@@ -950,7 +949,6 @@ action_launch_child(svc_action_t *op)
      * scripts when SIGPIPE is ignored by the environment. */
     signal(SIGPIPE, SIG_DFL);
 
-#if defined(HAVE_SCHED_SETSCHEDULER)
     if (sched_getscheduler(0) != SCHED_OTHER) {
         struct sched_param sp;
 
@@ -961,7 +959,7 @@ action_launch_child(svc_action_t *op)
             crm_info("Could not reset scheduling policy for %s", op->id);
         }
     }
-#endif
+
     if (setpriority(PRIO_PROCESS, 0, 0) == -1) {
         crm_info("Could not reset process priority for %s", op->id);
     }
@@ -981,7 +979,7 @@ action_launch_child(svc_action_t *op)
      * exit reason for the parent to parse.
      */
 
-#if SUPPORT_CIBSECRETS
+#if PCMK__ENABLE_CIBSECRETS
     rc = pcmk__substitute_secrets(op->rsc, op->params);
     if (rc != pcmk_rc_ok) {
         if (pcmk__str_eq(op->action, PCMK_ACTION_STOP, pcmk__str_casei)) {
@@ -1095,7 +1093,7 @@ wait_for_sync_result(svc_action_t *op, struct sigchld_data_s *data)
                 } else if (wait_rc < 0) {
                     wait_reason = pcmk_rc_str(errno);
                     crm_info("Wait for completion of %s[%d] failed: %s "
-                             CRM_XS " source=waitpid",
+                             QB_XS " source=waitpid",
                              op->id, op->pid, wait_reason);
                     wait_rc = 0; // Act as if process is still running
 
@@ -1120,7 +1118,7 @@ wait_for_sync_result(svc_action_t *op, struct sigchld_data_s *data)
         } else if ((poll_rc < 0) && (errno != EINTR)) {
             wait_reason = pcmk_rc_str(errno);
             crm_info("Wait for completion of %s[%d] failed: %s "
-                     CRM_XS " source=poll", op->id, op->pid, wait_reason);
+                     QB_XS " source=poll", op->id, op->pid, wait_reason);
             break;
         }
 
@@ -1214,12 +1212,12 @@ services__execute_file(svc_action_t *op)
     int stdin_fd[2] = {-1, -1};
     int rc;
     struct stat st;
-    struct sigchld_data_s data;
+    struct sigchld_data_s data = { .ignored = false };
 
     // Catch common failure conditions early
     if (stat(op->opaque->exec, &st) != 0) {
         rc = errno;
-        crm_info("Cannot execute '%s': %s " CRM_XS " stat rc=%d",
+        crm_info("Cannot execute '%s': %s " QB_XS " stat rc=%d",
                  op->opaque->exec, pcmk_rc_str(rc), rc);
         services__handle_exec_error(op, rc);
         goto done;
@@ -1227,7 +1225,7 @@ services__execute_file(svc_action_t *op)
 
     if (pipe(stdout_fd) < 0) {
         rc = errno;
-        crm_info("Cannot execute '%s': %s " CRM_XS " pipe(stdout) rc=%d",
+        crm_info("Cannot execute '%s': %s " QB_XS " pipe(stdout) rc=%d",
                  op->opaque->exec, pcmk_rc_str(rc), rc);
         services__handle_exec_error(op, rc);
         goto done;
@@ -1238,7 +1236,7 @@ services__execute_file(svc_action_t *op)
 
         close_pipe(stdout_fd);
 
-        crm_info("Cannot execute '%s': %s " CRM_XS " pipe(stderr) rc=%d",
+        crm_info("Cannot execute '%s': %s " QB_XS " pipe(stderr) rc=%d",
                  op->opaque->exec, pcmk_rc_str(rc), rc);
         services__handle_exec_error(op, rc);
         goto done;
@@ -1251,7 +1249,7 @@ services__execute_file(svc_action_t *op)
             close_pipe(stdout_fd);
             close_pipe(stderr_fd);
 
-            crm_info("Cannot execute '%s': %s " CRM_XS " pipe(stdin) rc=%d",
+            crm_info("Cannot execute '%s': %s " QB_XS " pipe(stdin) rc=%d",
                      op->opaque->exec, pcmk_rc_str(rc), rc);
             services__handle_exec_error(op, rc);
             goto done;
@@ -1276,7 +1274,7 @@ services__execute_file(svc_action_t *op)
             close_pipe(stdout_fd);
             close_pipe(stderr_fd);
 
-            crm_info("Cannot execute '%s': %s " CRM_XS " fork rc=%d",
+            crm_info("Cannot execute '%s': %s " QB_XS " fork rc=%d",
                      op->opaque->exec, pcmk_rc_str(rc), rc);
             services__handle_exec_error(op, rc);
             if (op->synchronous) {
@@ -1294,7 +1292,7 @@ services__execute_file(svc_action_t *op)
             if (STDOUT_FILENO != stdout_fd[1]) {
                 if (dup2(stdout_fd[1], STDOUT_FILENO) != STDOUT_FILENO) {
                     crm_warn("Can't redirect output from '%s': %s "
-                             CRM_XS " errno=%d",
+                             QB_XS " errno=%d",
                              op->opaque->exec, pcmk_rc_str(errno), errno);
                 }
                 close(stdout_fd[1]);
@@ -1302,7 +1300,7 @@ services__execute_file(svc_action_t *op)
             if (STDERR_FILENO != stderr_fd[1]) {
                 if (dup2(stderr_fd[1], STDERR_FILENO) != STDERR_FILENO) {
                     crm_warn("Can't redirect error output from '%s': %s "
-                             CRM_XS " errno=%d",
+                             QB_XS " errno=%d",
                              op->opaque->exec, pcmk_rc_str(errno), errno);
                 }
                 close(stderr_fd[1]);
@@ -1311,7 +1309,7 @@ services__execute_file(svc_action_t *op)
                 (STDIN_FILENO != stdin_fd[0])) {
                 if (dup2(stdin_fd[0], STDIN_FILENO) != STDIN_FILENO) {
                     crm_warn("Can't redirect input to '%s': %s "
-                             CRM_XS " errno=%d",
+                             QB_XS " errno=%d",
                              op->opaque->exec, pcmk_rc_str(errno), errno);
                 }
                 close(stdin_fd[0]);
@@ -1322,7 +1320,7 @@ services__execute_file(svc_action_t *op)
             }
 
             action_launch_child(op);
-            CRM_ASSERT(0);  /* action_launch_child is effectively noreturn */
+            pcmk__assert(false); // action_launch_child() should not return
     }
 
     /* Only the parent reaches here */
@@ -1336,7 +1334,7 @@ services__execute_file(svc_action_t *op)
     rc = pcmk__set_nonblocking(op->opaque->stdout_fd);
     if (rc != pcmk_rc_ok) {
         crm_info("Could not set '%s' output non-blocking: %s "
-                 CRM_XS " rc=%d",
+                 QB_XS " rc=%d",
                  op->opaque->exec, pcmk_rc_str(rc), rc);
     }
 
@@ -1344,7 +1342,7 @@ services__execute_file(svc_action_t *op)
     rc = pcmk__set_nonblocking(op->opaque->stderr_fd);
     if (rc != pcmk_rc_ok) {
         crm_info("Could not set '%s' error output non-blocking: %s "
-                 CRM_XS " rc=%d",
+                 QB_XS " rc=%d",
                  op->opaque->exec, pcmk_rc_str(rc), rc);
     }
 
@@ -1355,7 +1353,7 @@ services__execute_file(svc_action_t *op)
         rc = pcmk__set_nonblocking(op->opaque->stdin_fd);
         if (rc != pcmk_rc_ok) {
             crm_info("Could not set '%s' input non-blocking: %s "
-                    CRM_XS " fd=%d,rc=%d", op->opaque->exec,
+                    QB_XS " fd=%d,rc=%d", op->opaque->exec,
                     pcmk_rc_str(rc), op->opaque->stdin_fd, rc);
         }
         pipe_in_action_stdin_parameters(op);
