@@ -90,9 +90,10 @@ class LogAudit(ClusterAudit):
                 if rc != 0:
                     self._cm.log("ERROR: Cannot start 'systemd-journald' on %s" % node)
 
-            (rc, _) = self._cm.rsh(node, "service %s restart" % self._cm.env["syslogd"])
-            if rc != 0:
-                self._cm.log("ERROR: Cannot restart '%s' on %s" % (self._cm.env["syslogd"], node))
+            if "syslogd" in self._cm.env:
+                (rc, _) = self._cm.rsh(node, "service %s restart" % self._cm.env["syslogd"])
+                if rc != 0:
+                    self._cm.log("ERROR: Cannot restart '%s' on %s" % (self._cm.env["syslogd"], node))
 
     def _create_watcher(self, patterns, kind):
         """Create a new LogWatcher instance for the given patterns."""
@@ -490,7 +491,8 @@ class PrimitiveAudit(ClusterAudit):
             self.debug("No nodes active - skipping %s" % self.name)
             return False
 
-        (_, lines) = self._cm.rsh(self._target, "crm_resource -c", verbose=1)
+        (_, lines) = self._cm.rsh(self._target, "crm_resource --list-cts",
+                                  verbose=1)
 
         for line in lines:
             if re.search("^Resource", line):
@@ -649,7 +651,9 @@ class ColocationAudit(PrimitiveAudit):
 
     def _crm_location(self, resource):
         """Return a list of cluster nodes where a given resource is running."""
-        (rc, lines) = self._cm.rsh(self._target, "crm_resource -W -r %s -Q" % resource, verbose=1)
+        (rc, lines) = self._cm.rsh(self._target,
+                                   "crm_resource --locate -r %s -Q" % resource,
+                                   verbose=1)
         hosts = []
 
         if rc == 0:
