@@ -9,14 +9,45 @@ import re
 import time
 import threading
 
-from dateutil.parser import isoparser
-
 from pacemaker.buildoptions import BuildOptions
 from pacemaker._cts.errors import OutputNotFoundError
 from pacemaker._cts.logging import LogFactory
 from pacemaker._cts.remote import RemoteFactory
 
-LOG_WATCHER_BIN = "%s/cts-log-watcher" % BuildOptions.DAEMON_DIR
+# dateutil >=2.7.0 provides isoparser, but we support older versions
+try:
+    from dateutil.parser import isoparser
+except ImportError:
+    import datetime
+    import subprocess
+    # pylint: disable=ungrouped-imports
+    from pacemaker._cts.process import pipe_communicate
+
+    # pylint: disable=invalid-name
+    class isoparser():
+        """Mimic isoparser.isoparse when not available."""
+
+        def isoparse(self, timestamp):
+            """Mimic isoparser.isoparse when not available."""
+            with subprocess.Popen(["iso8601", "--output-as", "xml", "-d", timestamp],
+                                  stdout=subprocess.PIPE) as result:
+
+                output = pipe_communicate(result)
+                if result.returncode != 0:
+                    raise RuntimeError("Unable to run iso8601 command")
+                match = re.search(r".*<date>\s*(\d+)-(\d+)-(\d+) (\d+):(\d+):(\d+)Z",
+                                  output)
+                if match is None:
+                    raise RuntimeError("Unable to parse iso8601 command output")
+                return datetime.datetime(int(match.group(1)),
+                                         int(match.group(2)),
+                                         int(match.group(3)),
+                                         int(match.group(4)),
+                                         int(match.group(5)),
+                                         int(match.group(6)))
+
+
+CTS_SUPPORT_BIN = "%s/cts-support" % BuildOptions.DAEMON_DIR
 
 
 @unique
@@ -92,9 +123,7 @@ class SearchObj:
         raise NotImplementedError
 
     def harvest_cached(self):
-        """
-        Return cached logs from before the limit timestamp.
-        """
+        """Return cached logs from before the limit timestamp."""
         raise NotImplementedError
 
     def end(self):
@@ -171,15 +200,13 @@ class FileObj(SearchObj):
 
             return None
 
-        cmd = ("%s -p CTSwatcher: -l 200 -f %s -o %s"
-               % (LOG_WATCHER_BIN, self.filename, self.offset))
+        cmd = ("%s watch -p CTSwatcher: -l 200 -f %s -o %s"
+               % (CTS_SUPPORT_BIN, self.filename, self.offset))
 
         return self.rsh.call_async(self.host, cmd, delegate=self)
 
     def harvest_cached(self):
-        """
-        Return cached logs from before the limit timestamp.
-        """
+        """Return cached logs from before the limit timestamp."""
         # cts-log-watcher script renders caching unnecessary for FileObj.
         # @TODO Caching might be slightly more efficient, if not too complex.
         return []
@@ -194,8 +221,8 @@ class FileObj(SearchObj):
         if self.limit:
             return
 
-        cmd = ("%s -p CTSwatcher: -l 2 -f %s -o EOF"
-               % (LOG_WATCHER_BIN, self.filename))
+        cmd = ("%s watch -p CTSwatcher: -l 2 -f %s -o EOF"
+               % (CTS_SUPPORT_BIN, self.filename))
 
         # pylint: disable=not-callable
         (_, lines) = self.rsh(self.host, cmd, verbose=0)
@@ -320,9 +347,7 @@ class JournalObj(SearchObj):
         return self.rsh.call_async(self.host, command, delegate=self)
 
     def harvest_cached(self):
-        """
-        Return cached logs from before the limit timestamp.
-        """
+        """Return cached logs from before the limit timestamp."""
         before, self._cache = self._split_msgs_by_limit(self._cache)
         return before
 
