@@ -50,7 +50,6 @@ static struct {
     gboolean allow_create;
     gboolean force;
     gboolean get_node_path;
-    gboolean local;
     gboolean no_children;
     gboolean score_update;
     gboolean sync_call;
@@ -60,8 +59,8 @@ static struct {
      */
     gboolean extended_version;
 
-    //! \deprecated
-    gboolean no_bcast;
+    // @COMPAT Deprecated since 3.0.0
+    gboolean local;
 } options;
 
 int do_init(void);
@@ -121,7 +120,7 @@ report_schema_unchanged(void)
 static inline bool
 cib_action_is_dangerous(void)
 {
-    return options.no_bcast || options.delete_all
+    return options.delete_all
            || pcmk__str_any_of(options.cib_action,
                                PCMK__CIB_REQUEST_UPGRADE,
                                PCMK__CIB_REQUEST_ERASE,
@@ -348,9 +347,6 @@ static GOptionEntry addl_entries[] = {
     { "sync-call", 's', G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE,
       &options.sync_call, "Wait for call to complete before returning", NULL },
 
-    { "local", 'l', G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE, &options.local,
-      "Command takes effect locally (should be used only for queries)", NULL },
-
     { "scope", 'o', G_OPTION_FLAG_NONE, G_OPTION_ARG_CALLBACK, section_cb,
       "Limit scope of operation to specific section of CIB\n"
       INDENT "Valid values: " PCMK_XE_CONFIGURATION ", " PCMK_XE_NODES
@@ -410,9 +406,8 @@ static GOptionEntry addl_entries[] = {
       INDENT "existing value and X are parsed and added as scores.\n"
 
       INDENT "Scores are integer values capped at INFINITY and -INFINITY.\n"
-      INDENT "Refer to Pacemaker Explained and to the char2score() function\n"
-      INDENT "for more details on scores, including how they're parsed and\n"
-      INDENT "added.",
+      INDENT "Refer to Pacemaker Explained for more details on scores,\n"
+      INDENT "including how they are parsed and added.",
       NULL },
 
     { "allow-create", 'c', G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE,
@@ -430,13 +425,9 @@ static GOptionEntry addl_entries[] = {
     { "node", 'N', G_OPTION_FLAG_NONE, G_OPTION_ARG_STRING, &options.dest_node,
       "(Advanced) Send command to the specified host", "value" },
 
-    // @COMPAT: Deprecated
-    { "no-bcast", 'b', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE,
-      &options.no_bcast, "deprecated", NULL },
-
-    // @COMPAT: Deprecated
-    { "host", 'h', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING,
-      &options.dest_node, "deprecated", NULL },
+    // @COMPAT Deprecated since 3.0.0
+    { "local", 'l', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &options.local,
+      "(deprecated)", NULL },
 
     { NULL }
 };
@@ -456,8 +447,8 @@ build_arg_context(pcmk__common_args_t *args)
     };
 
     desc = "Examples:\n\n"
-           "Query the configuration from the local node:\n\n"
-           "\t# cibadmin --query --local\n\n"
+           "Query the configuration:\n\n"
+           "\t# cibadmin --query\n\n"
            "Query just the cluster options configuration:\n\n"
            "\t# cibadmin --query --scope " PCMK_XE_CRM_CONFIG "\n\n"
            "Query all '" PCMK_META_TARGET_ROLE "' settings:\n\n"
@@ -654,19 +645,6 @@ main(int argc, char **argv)
                               cib_xpath_address);
     }
 
-    if (options.local) {
-        // Configure command to take effect only locally
-        cib__set_call_options(options.cmd_options, crm_system_name,
-                              cib_scope_local);
-    }
-
-    // @COMPAT: Deprecated option
-    if (options.no_bcast) {
-        // Configure command to take effect only locally and not to broadcast
-        cib__set_call_options(options.cmd_options, crm_system_name,
-                              cib_inhibit_bcast|cib_scope_local);
-    }
-
     if (options.no_children) {
         // When querying an object, don't include its children in the result
         cib__set_call_options(options.cmd_options, crm_system_name,
@@ -763,7 +741,7 @@ main(int argc, char **argv)
             goto done;
         }
 
-        digest = calculate_on_disk_digest(input);
+        digest = pcmk__digest_on_disk_cib(input);
         fprintf(stderr, "Digest: ");
         fprintf(stdout, "%s\n", pcmk__s(digest, "<null>"));
         free(digest);
@@ -771,7 +749,6 @@ main(int argc, char **argv)
 
     } else if (strcmp(options.cib_action, "md5-sum-versioned") == 0) {
         char *digest = NULL;
-        const char *version = NULL;
 
         if (input == NULL) {
             exit_code = CRM_EX_USAGE;
@@ -780,9 +757,7 @@ main(int argc, char **argv)
             goto done;
         }
 
-        version = crm_element_value(input, PCMK_XA_CRM_FEATURE_SET);
-        digest = calculate_xml_versioned_digest(input, FALSE, TRUE, version);
-        fprintf(stderr, "Versioned (%s) digest: ", version);
+        digest = pcmk__digest_xml(input, true);
         fprintf(stdout, "%s\n", pcmk__s(digest, "<null>"));
         free(digest);
         goto done;
@@ -854,7 +829,7 @@ main(int argc, char **argv)
                                              options.cmd_options) == pcmk_ok) {
                         pcmk__update_schema(&obj, NULL, true, false);
                     }
-                    free_xml(obj);
+                    pcmk__xml_free(obj);
 
                 } else if (output != NULL) {
                     // Show validation errors to stderr
@@ -910,8 +885,8 @@ done:
     free(options.validate_with);
 
     g_free(acl_cred);
-    free_xml(input);
-    free_xml(output);
+    pcmk__xml_free(input);
+    pcmk__xml_free(output);
 
     rc = cib__clean_up_connection(&the_cib);
     if (exit_code == CRM_EX_OK) {
@@ -948,7 +923,7 @@ do_init(void)
     int rc = pcmk_ok;
 
     the_cib = cib_new();
-    rc = cib__signon_attempts(the_cib, crm_system_name, cib_command, 5);
+    rc = cib__signon_attempts(the_cib, cib_command, 5);
     if (rc != pcmk_ok) {
         crm_err("Could not connect to the CIB: %s", pcmk_strerror(rc));
         fprintf(stderr, "Could not connect to the CIB: %s\n",
@@ -968,7 +943,7 @@ cibadmin_op_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
         report_schema_unchanged();
 
     } else if (rc != pcmk_rc_ok) {
-        crm_warn("Call %s failed: %s " CRM_XS " rc=%d",
+        crm_warn("Call %s failed: %s " QB_XS " rc=%d",
                  options.cib_action, pcmk_rc_str(rc), rc);
         fprintf(stderr, "Call %s failed: %s\n",
                 options.cib_action, pcmk_rc_str(rc));
