@@ -124,21 +124,19 @@ apply_patch(xmlNode *input, xmlNode *patch, gboolean as_cib)
     rc = pcmk_legacy2rc(rc);
     if (rc != pcmk_rc_ok) {
         fprintf(stderr, "Could not apply patch: %s\n", pcmk_rc_str(rc));
-        free_xml(output);
+        pcmk__xml_free(output);
         return rc;
     }
 
     if (output != NULL) {
-        const char *version;
         char *buffer;
 
         print_patch(output);
 
-        version = crm_element_value(output, PCMK_XA_CRM_FEATURE_SET);
-        buffer = calculate_xml_versioned_digest(output, FALSE, TRUE, version);
+        buffer = pcmk__digest_xml(output, true);
         crm_trace("Digest: %s", pcmk__s(buffer, "<null>\n"));
         free(buffer);
-        free_xml(output);
+        pcmk__xml_free(output);
     }
     return pcmk_rc_ok;
 }
@@ -162,49 +160,6 @@ log_patch_cib_versions(xmlNode *patch)
     }
 }
 
-static void
-strip_patch_cib_version(xmlNode *patch, const char **vfields, size_t nvfields)
-{
-    int format = 1;
-
-    crm_element_value_int(patch, PCMK_XA_FORMAT, &format);
-    if (format == 2) {
-        xmlNode *version_xml = pcmk__xe_first_child(patch, PCMK_XE_VERSION,
-                                                    NULL, NULL);
-
-        if (version_xml) {
-            free_xml(version_xml);
-        }
-
-    } else {
-        int i = 0;
-
-        const char *tags[] = {
-            PCMK__XE_DIFF_REMOVED,
-            PCMK__XE_DIFF_ADDED,
-        };
-
-        for (i = 0; i < PCMK__NELEM(tags); i++) {
-            xmlNode *tmp = NULL;
-            int lpc;
-
-            tmp = pcmk__xe_first_child(patch, tags[i], NULL, NULL);
-            if (tmp) {
-                for (lpc = 0; lpc < nvfields; lpc++) {
-                    pcmk__xe_remove_attr(tmp, vfields[lpc]);
-                }
-
-                tmp = pcmk__xe_first_child(tmp, PCMK_XE_CIB, NULL, NULL);
-                if (tmp) {
-                    for (lpc = 0; lpc < nvfields; lpc++) {
-                        pcmk__xe_remove_attr(tmp, vfields[lpc]);
-                    }
-                }
-            }
-        }
-    }
-}
-
 // \return Standard Pacemaker return code
 static int
 generate_patch(xmlNode *object_1, xmlNode *object_2, const char *xml_file_2,
@@ -251,12 +206,13 @@ generate_patch(xmlNode *object_1, xmlNode *object_2, const char *xml_file_2,
         log_patch_cib_versions(output);
 
     } else if (no_version) {
-        strip_patch_cib_version(output, vfields, PCMK__NELEM(vfields));
+        pcmk__xml_free(pcmk__xe_first_child(output, PCMK_XE_VERSION, NULL,
+                                            NULL));
     }
 
     pcmk__log_xml_patchset(LOG_NOTICE, output);
     print_patch(output);
-    free_xml(output);
+    pcmk__xml_free(output);
 
     /* pcmk_rc_error means there's a non-empty diff.
      * @COMPAT Choose a more descriptive return code, like one that maps to
@@ -374,8 +330,8 @@ done:
     pcmk__free_arg_context(context);
     free(options.xml_file_1);
     free(options.xml_file_2);
-    free_xml(object_1);
-    free_xml(object_2);
+    pcmk__xml_free(object_1);
+    pcmk__xml_free(object_2);
 
     pcmk__output_and_clear_error(&error, NULL);
     crm_exit(exit_code);
