@@ -84,12 +84,13 @@ static gchar **processed_args = NULL;
 static time_t last_refresh = 0;
 volatile crm_trigger_t *refresh_trigger = NULL;
 
+static pcmk_scheduler_t *scheduler = NULL;
 static enum pcmk__fence_history fence_history = pcmk__fence_history_none;
 
 int interactive_fence_level = 0;
 
 static pcmk__supported_format_t formats[] = {
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
     CRM_MON_SUPPORTED_FORMAT_CURSES,
 #endif
     PCMK__SUPPORTED_FORMAT_HTML,
@@ -202,14 +203,6 @@ static pcmk__message_entry_t fmt_functions[] = {
     { NULL, NULL, NULL },
 };
 
-/* Define exit codes for monitoring-compatible output
- * For nagios plugins, the possibilities are
- * OK=0, WARN=1, CRIT=2, and UNKNOWN=3
- */
-#define MON_STATUS_WARN    CRM_EX_ERROR
-#define MON_STATUS_CRIT    CRM_EX_INVALID_PARAM
-#define MON_STATUS_UNKNOWN CRM_EX_UNIMPLEMENT_FEATURE
-
 #define RECONNECT_MSECS 5000
 
 struct {
@@ -246,7 +239,7 @@ static void refresh_after_event(gboolean data_updated, gboolean enforce);
 
 static uint32_t
 all_includes(mon_output_format_t fmt) {
-    if (fmt == mon_output_monitor || fmt == mon_output_plain || fmt == mon_output_console) {
+    if ((fmt == mon_output_plain) || (fmt == mon_output_console)) {
         return ~pcmk_section_options;
     } else {
         return pcmk_section_all;
@@ -256,11 +249,9 @@ all_includes(mon_output_format_t fmt) {
 static uint32_t
 default_includes(mon_output_format_t fmt) {
     switch (fmt) {
-        case mon_output_monitor:
         case mon_output_plain:
         case mon_output_console:
         case mon_output_html:
-        case mon_output_cgi:
             return pcmk_section_summary
                    |pcmk_section_nodes
                    |pcmk_section_resources
@@ -427,31 +418,6 @@ include_exclude_cb(const gchar *option_name, const gchar *optarg, gpointer data,
     return TRUE;
 }
 
-static gboolean
-as_cgi_cb(const gchar *option_name, const gchar *optarg, gpointer data, GError **err) {
-    pcmk__str_update(&args->output_ty, "html");
-    output_format = mon_output_cgi;
-    options.exec_mode = mon_exec_one_shot;
-    return TRUE;
-}
-
-static gboolean
-as_html_cb(const gchar *option_name, const gchar *optarg, gpointer data, GError **err) {
-    pcmk__str_update(&args->output_dest, optarg);
-    pcmk__str_update(&args->output_ty, "html");
-    output_format = mon_output_html;
-    umask(S_IWGRP | S_IWOTH);   // World-readable HTML
-    return TRUE;
-}
-
-static gboolean
-as_simple_cb(const gchar *option_name, const gchar *optarg, gpointer data, GError **err) {
-    pcmk__str_update(&args->output_ty, "text");
-    output_format = mon_output_monitor;
-    options.exec_mode = mon_exec_one_shot;
-    return TRUE;
-}
-
 static gboolean
 as_xml_cb(const gchar *option_name, const gchar *optarg, gpointer data, GError **err) {
     pcmk__str_update(&args->output_ty, "xml");
@@ -514,13 +480,6 @@ inactive_resources_cb(const gchar *option_name, const gchar *optarg, gpointer da
     return TRUE;
 }
 
-static gboolean
-no_curses_cb(const gchar *option_name, const gchar *optarg, gpointer data, GError **err) {
-    pcmk__str_update(&args->output_ty, "text");
-    output_format = mon_output_plain;
-    return TRUE;
-}
-
 static gboolean
 print_brief_cb(const gchar *option_name, const gchar *optarg, gpointer data, GError **err) {
     show_opts |= pcmk_show_brief;
@@ -763,34 +722,18 @@ static GOptionEntry display_entries[] = {
 };
 
 static GOptionEntry deprecated_entries[] = {
-    { "as-html", 'h', G_OPTION_FLAG_FILENAME, G_OPTION_ARG_CALLBACK, as_html_cb,
-      "Write cluster status to the named HTML file.\n"
-      INDENT "Use --output-as=html --output-to=FILE instead.",
-      "FILE" },
-
+    /* @COMPAT resource-agents <4.15.0 uses --as-xml, so removing this option
+     * must wait until we no longer support building on any platforms that ship
+     * the older agents.
+     */
     { "as-xml", 'X', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, as_xml_cb,
       "Write cluster status as XML to stdout. This will enable one-shot mode.\n"
       INDENT "Use --output-as=xml instead.",
       NULL },
 
-    { "simple-status", 's', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-      as_simple_cb,
-      "Display the cluster status once as a simple one line output\n"
-      INDENT "(suitable for nagios)",
-      NULL },
-
-    { "disable-ncurses", 'N', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, no_curses_cb,
-      "Disable the use of ncurses.\n"
-      INDENT "Use --output-as=text instead.",
-      NULL },
-
-    { "web-cgi", 'w', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, as_cgi_cb,
-      "Web mode with output suitable for CGI (preselected when run as *.cgi).\n"
-      INDENT "Use --output-as=html --html-cgi instead.",
-      NULL },
-
     { NULL }
 };
+
 /* *INDENT-ON* */
 
 /* Reconnect to the CIB and fencing agent after reconnect_ms has passed.  This sounds
@@ -800,7 +743,7 @@ static GOptionEntry deprecated_entries[] = {
 static gboolean
 reconnect_after_timeout(gpointer data)
 {
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
     if (output_format == mon_output_console) {
         clear();
         refresh();
@@ -818,8 +761,8 @@ reconnect_after_timeout(gpointer data)
     out->message(out, "crm-mon-disconnected",
                  "Latest connection attempt failed", pcmkd_state);
 
-    reconnect_timer = g_timeout_add(options.reconnect_ms,
-                                    reconnect_after_timeout, NULL);
+    reconnect_timer = pcmk__create_timer(options.reconnect_ms,
+                                         reconnect_after_timeout, NULL);
     return G_SOURCE_REMOVE;
 }
 
@@ -863,8 +806,8 @@ mon_cib_connection_destroy(gpointer user_data)
 
     if (cib) {
         cib->cmds->signoff(cib);
-        reconnect_timer = g_timeout_add(options.reconnect_ms,
-                                        reconnect_after_timeout, NULL);
+        reconnect_timer = pcmk__create_timer(options.reconnect_ms,
+                                             reconnect_after_timeout, NULL);
     }
 }
 
@@ -875,7 +818,7 @@ mon_shutdown(int nsig)
     clean_up(CRM_EX_OK);
 }
 
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
 static volatile sighandler_t ncurses_winch_handler;
 
 /* Signal handler installed the regular way (not into the main loop) for when
@@ -1068,7 +1011,7 @@ setup_api_connections(void)
     return rc;
 }
 
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
 static const char *
 get_option_desc(char c)
 {
@@ -1154,10 +1097,6 @@ detect_user_input(GIOChannel *channel, GIOCondition condition, gpointer user_dat
                 break;
             case 'R':
                 show_opts ^= pcmk_show_details;
-#ifdef PCMK__COMPAT_2_0
-                // Keep failed action output the same as 2.0.x
-                show_opts |= pcmk_show_failed_detail;
-#endif
                 break;
             case 't':
                 show_opts ^= pcmk_show_timing;
@@ -1211,11 +1150,7 @@ detect_user_input(GIOChannel *channel, GIOCondition condition, gpointer user_dat
         print_option_help(out, 'A', pcmk_is_set(show, pcmk_section_attributes));
         print_option_help(out, 'L', pcmk_is_set(show, pcmk_section_bans));
         print_option_help(out, 'D', !pcmk_is_set(show, pcmk_section_summary));
-#ifdef PCMK__COMPAT_2_0
-        print_option_help(out, 'R', pcmk_any_flags_set(show_opts, pcmk_show_details & ~pcmk_show_failed_detail));
-#else
         print_option_help(out, 'R', pcmk_any_flags_set(show_opts, pcmk_show_details));
-#endif
         print_option_help(out, 'b', pcmk_is_set(show_opts, pcmk_show_brief));
         print_option_help(out, 'j', pcmk_is_set(show_opts, pcmk_show_pending));
         curses_formatted_printf(out, "%d m: \t%s\n", interactive_fence_level, get_option_desc('m'));
@@ -1227,7 +1162,7 @@ refresh:
 
     return rc;
 }
-#endif  // CURSES_ENABLED
+#endif  // PCMK__ENABLE_CURSES
 
 // Basically crm_signal_handler(SIGCHLD, SIG_IGN) plus the SA_NOCLDWAIT flag
 static void
@@ -1259,17 +1194,14 @@ build_arg_context(pcmk__common_args_t *args, GOptionGroup **group) {
         { NULL }
     };
 
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
     const char *fmts = "console (default), html, text, xml, none";
 #else
     const char *fmts = "text (default), html, xml, none";
-#endif // CURSES_ENABLED
+#endif // PCMK__ENABLE_CURSES
     const char *desc = NULL;
 
     desc = "Notes:\n\n"
-           "If this program is called as crm_mon.cgi, --output-as=html and\n"
-           "--html-cgi are automatically added to the command line\n"
-           "arguments.\n\n"
 
            "Time Specification:\n\n"
            "The TIMESPEC in any command line option can be specified in many\n"
@@ -1289,7 +1221,7 @@ build_arg_context(pcmk__common_args_t *args, GOptionGroup **group) {
            "list of valid sections, pass --include=list or --exclude=list.\n\n"
 
            "Interactive Use:\n\n"
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
            "When run interactively, crm_mon can be told to hide and show\n"
            "various sections of output. To see a help screen explaining the\n"
            "options, press '?'. Any key stroke aside from those listed will\n"
@@ -1298,14 +1230,14 @@ build_arg_context(pcmk__common_args_t *args, GOptionGroup **group) {
            "The local installation of Pacemaker was built without support for\n"
            "interactive (console) mode. A curses library must be available at\n"
            "build time to support interactive mode.\n\n"
-#endif // CURSES_ENABLED
+#endif // PCMK__ENABLE_CURSES
 
            "Examples:\n\n"
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
            "Display the cluster status on the console with updates as they\n"
            "occur:\n\n"
            "\tcrm_mon\n\n"
-#endif // CURSES_ENABLED
+#endif // PCMK__ENABLE_CURSES
 
            "Display the cluster status once and exit:\n\n"
            "\tcrm_mon -1\n\n"
@@ -1336,31 +1268,13 @@ build_arg_context(pcmk__common_args_t *args, GOptionGroup **group) {
     return context;
 }
 
-/* If certain format options were specified, we want to set some extra
- * options.  We can just process these like they were given on the
- * command line.
- */
-static void
-add_output_args(void) {
-    GError *err = NULL;
-
-    if (output_format == mon_output_cgi) {
-        if (!pcmk__force_args(context, &err, "%s --html-cgi", g_get_prgname())) {
-            g_propagate_error(&error, err);
-            clean_up(CRM_EX_USAGE);
-        }
-    }
-}
-
 /*!
  * \internal
- * \brief Set output format based on \p --output-as arguments and mode arguments
+ * \brief Set output format based on \c --output-as arguments and mode arguments
  *
- * When the deprecated output format arguments (\p --as-cgi, \p --as-html,
- * \p --simple-status, \p --as-xml) are parsed, callback functions set
- * \p output_format (and the umask if appropriate). If none of the deprecated
- * arguments were specified, this function does the same based on the current
- * \p --output-as arguments and the \p --one-shot and \p --daemonize arguments.
+ * When the deprecated \c --as-xml argument is parsed, a callback function sets
+ * \c output_format. Otherwise, this function does the same based on the current
+ * \c --output-as arguments and the \c --one-shot and \c --daemonize arguments.
  *
  * \param[in,out] args  Command line arguments
  */
@@ -1368,8 +1282,8 @@ static void
 reconcile_output_format(pcmk__common_args_t *args)
 {
     if (output_format != mon_output_unset) {
-        /* One of the deprecated arguments was used, and we're finished. Note
-         * that this means the deprecated arguments take precedence.
+        /* The deprecated --as-xml argument was used, and we're finished. Note
+         * that this means the deprecated argument takes precedence.
          */
         return;
     }
@@ -1384,7 +1298,7 @@ reconcile_output_format(pcmk__common_args_t *args)
     } else if (pcmk__str_eq(args->output_ty, "xml", pcmk__str_none)) {
         output_format = mon_output_xml;
 
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
     } else if (pcmk__str_eq(args->output_ty, "console",
                             pcmk__str_null_matches)) {
         /* Console is the default format if no conflicting options are given.
@@ -1408,7 +1322,7 @@ reconcile_output_format(pcmk__common_args_t *args)
             output_format = mon_output_console;
             crm_enable_stderr(FALSE);
         }
-#endif // CURSES_ENABLED
+#endif // PCMK__ENABLE_CURSES
 
     } else if (pcmk__str_eq(args->output_ty, "text", pcmk__str_null_matches)) {
         /* Text output was explicitly requested, or it's the default because
@@ -1451,11 +1365,7 @@ set_default_exec_mode(const pcmk__common_args_t *args)
 static void
 clean_up_on_connection_failure(int rc)
 {
-    if (output_format == mon_output_monitor) {
-        g_set_error(&error, PCMK__EXITC_ERROR, CRM_EX_ERROR, "CLUSTER CRIT: Connection to cluster failed: %s",
-                    pcmk_rc_str(rc));
-        clean_up(MON_STATUS_CRIT);
-    } else if (rc == ENOTCONN) {
+    if (rc == ENOTCONN) {
         if (pcmkd_state == pcmk_pacemakerd_state_remote) {
             g_set_error(&error, PCMK__EXITC_ERROR, CRM_EX_ERROR, "Error: remote-node not connected to cluster");
         } else {
@@ -1473,8 +1383,7 @@ one_shot(void)
 {
     int rc = pcmk__status(out, cib, fence_history, show, show_opts,
                           options.only_node, options.only_rsc,
-                          options.neg_location_prefix,
-                          output_format == mon_output_monitor, 0);
+                          options.neg_location_prefix, 0);
 
     if (rc == pcmk_rc_ok) {
         clean_up(pcmk_rc2exitc(rc));
@@ -1511,20 +1420,13 @@ main(int argc, char **argv)
     // Avoid needing to wait for subprocesses forked for -E/--external-agent
     avoid_zombies();
 
-    if (pcmk__ends_with_ext(argv[0], ".cgi")) {
-        output_format = mon_output_cgi;
-        options.exec_mode = mon_exec_one_shot;
-    }
-
-    processed_args = pcmk__cmdline_preproc(argv, "ehimpxEILU");
+    processed_args = pcmk__cmdline_preproc(argv, "eimpxEILU");
 
     fence_history_cb("--fence-history", "1", NULL, NULL);
 
-    /* Set an HTML title regardless of what format we will eventually use.  This can't
-     * be done in add_output_args.  That function is called after command line
-     * arguments are processed in the next block, which means it'll override whatever
-     * title the user provides.  Doing this here means the user can give their own
-     * title on the command line.
+    /* Set an HTML title regardless of what format we will eventually use.
+     * Doing this here means the user can give their own title on the command
+     * line.
      */
     if (!pcmk__force_args(context, &error, "%s --html-title \"Cluster Status\"",
                           g_get_prgname())) {
@@ -1595,7 +1497,6 @@ main(int argc, char **argv)
 
     reconcile_output_format(args);
     set_default_exec_mode(args);
-    add_output_args();
 
     rc = pcmk__output_new(&out, args->output_ty, args->output_dest, argv);
     if (rc != pcmk_rc_ok) {
@@ -1614,7 +1515,7 @@ main(int argc, char **argv)
     /* If we had a valid format for pcmk__output_new(), output_format should be
      * set by now.
      */
-    CRM_ASSERT(output_format != mon_output_unset);
+    pcmk__assert(output_format != mon_output_unset);
 
     if (output_format == mon_output_plain) {
         pcmk__output_text_set_fancy(out, true);
@@ -1676,28 +1577,12 @@ main(int argc, char **argv)
         return clean_up(CRM_EX_OK);
     }
 
-    /* Extra sanity checks when in CGI mode */
-    if (output_format == mon_output_cgi) {
-        if (cib->variant == cib_file) {
-            g_set_error(&error, PCMK__EXITC_ERROR, CRM_EX_USAGE, "CGI mode used with CIB file");
-            return clean_up(CRM_EX_USAGE);
-        } else if (options.external_agent != NULL) {
-            g_set_error(&error, PCMK__EXITC_ERROR, CRM_EX_USAGE, "CGI mode cannot be used with --external-agent");
-            return clean_up(CRM_EX_USAGE);
-        } else if (options.exec_mode == mon_exec_daemonized) {
-            g_set_error(&error, PCMK__EXITC_ERROR, CRM_EX_USAGE, "CGI mode cannot be used with -d");
-            return clean_up(CRM_EX_USAGE);
-        }
-    }
-
     if (output_format == mon_output_xml) {
         show_opts |= pcmk_show_inactive_rscs | pcmk_show_timing;
     }
 
-    if ((output_format == mon_output_html || output_format == mon_output_cgi)
-        && (out->dest != stdout)) {
-
-        char *content = pcmk__itoa(options.reconnect_ms / 1000);
+    if ((output_format == mon_output_html) && (out->dest != stdout)) {
+        char *content = pcmk__itoa(pcmk__timeout_ms2s(options.reconnect_ms));
 
         pcmk__html_add_header(PCMK__XE_META,
                               PCMK__XA_HTTP_EQUIV, PCMK__VALUE_REFRESH,
@@ -1706,11 +1591,6 @@ main(int argc, char **argv)
         free(content);
     }
 
-#ifdef PCMK__COMPAT_2_0
-    // Keep failed action output the same as 2.0.x
-    show_opts |= pcmk_show_failed_detail;
-#endif
-
     crm_info("Starting %s", crm_system_name);
 
     cib__set_output(cib, out);
@@ -1719,6 +1599,14 @@ main(int argc, char **argv)
         one_shot();
     }
 
+    scheduler = pe_new_working_set();
+    pcmk__mem_assert(scheduler);
+    scheduler->priv->out = out;
+    if ((cib->variant == cib_native) && pcmk_is_set(show, pcmk_section_times)) {
+        // Currently used only in the times section
+        pcmk__query_node_name(out, 0, &(scheduler->priv->local_node_name), 0);
+    }
+
     out->message(out, "crm-mon-disconnected",
                  "Waiting for initial connection", pcmkd_state);
     do {
@@ -1727,13 +1615,13 @@ main(int argc, char **argv)
 
         if (rc != pcmk_rc_ok) {
             if ((rc == ENOTCONN) || (rc == ECONNREFUSED)) {
-                out->transient(out, "Connection failed. Retrying in %ums...",
-                               options.reconnect_ms);
+                out->transient(out, "Connection failed. Retrying in %s...",
+                               pcmk__readable_interval(options.reconnect_ms));
             }
 
             // Give some time to view all output even if we won't retry
             pcmk__sleep_ms(options.reconnect_ms);
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
             if (output_format == mon_output_console) {
                 clear();
                 refresh();
@@ -1753,7 +1641,7 @@ main(int argc, char **argv)
 
     mainloop_add_signal(SIGTERM, mon_shutdown);
     mainloop_add_signal(SIGINT, mon_shutdown);
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
     if (output_format == mon_output_console) {
         ncurses_winch_handler = crm_signal_handler(SIGWINCH, mon_winresize);
         if (ncurses_winch_handler == SIG_DFL ||
@@ -1897,7 +1785,7 @@ handle_rsc_op(xmlNode *xml, void *userdata)
         }
 
     } else if (status == PCMK_EXEC_DONE) {
-        desc = services_ocf_exitcode_str(rc);
+        desc = crm_exit_str(rc);
         crm_warn("%s of %s on %s failed: %s", task, rsc, node, desc);
 
     } else {
@@ -1940,7 +1828,7 @@ handle_op_for_node(xmlNode *xml, void *userdata)
 }
 
 static int
-crm_diff_update_element_v2(xmlNode *change, void *userdata)
+crm_diff_update_element(xmlNode *change, void *userdata)
 {
     const char *name = NULL;
     const char *op = crm_element_value(change, PCMK_XA_OPERATION);
@@ -1975,8 +1863,8 @@ crm_diff_update_element_v2(xmlNode *change, void *userdata)
 
     } else if(name == NULL) {
         crm_debug("No result for %s operation to %s", op, xpath);
-        CRM_ASSERT(pcmk__str_any_of(op, PCMK_VALUE_MOVE, PCMK_VALUE_DELETE,
-                                    NULL));
+        pcmk__assert(pcmk__str_any_of(op, PCMK_VALUE_MOVE, PCMK_VALUE_DELETE,
+                                      NULL));
 
     } else if (strcmp(name, PCMK_XE_CIB) == 0) {
         pcmk__xe_foreach_child(pcmk__xe_first_child(match, PCMK_XE_STATUS, NULL,
@@ -2022,34 +1910,6 @@ crm_diff_update_element_v2(xmlNode *change, void *userdata)
     return pcmk_rc_ok;
 }
 
-static void
-crm_diff_update_v2(const char *event, xmlNode * msg)
-{
-    xmlNode *wrapper = pcmk__xe_first_child(msg, PCMK__XE_CIB_UPDATE_RESULT,
-                                            NULL, NULL);
-    xmlNode *diff = pcmk__xe_first_child(wrapper, NULL, NULL, NULL);
-
-    pcmk__xe_foreach_child(diff, NULL, crm_diff_update_element_v2, NULL);
-}
-
-static void
-crm_diff_update_v1(const char *event, xmlNode * msg)
-{
-    /* Process operation updates */
-    xmlXPathObject *xpathObj = xpath_search(msg,
-                                            "//" PCMK__XE_CIB_UPDATE_RESULT
-                                            "//" PCMK__XE_DIFF_ADDED
-                                            "//" PCMK__XE_LRM_RSC_OP);
-    int lpc = 0, max = numXpathResults(xpathObj);
-
-    for (lpc = 0; lpc < max; lpc++) {
-        xmlNode *rsc_op = getXpathResult(xpathObj, lpc);
-
-        handle_rsc_op(rsc_op, NULL);
-    }
-    freeXpathObject(xpathObj);
-}
-
 static void
 crm_diff_update(const char *event, xmlNode * msg)
 {
@@ -2069,34 +1929,36 @@ crm_diff_update(const char *event, xmlNode * msg)
             case -pcmk_err_diff_resync:
             case -pcmk_err_diff_failed:
                 crm_notice("[%s] Patch aborted: %s (%d)", event, pcmk_strerror(rc), rc);
-                free_xml(current_cib); current_cib = NULL;
+                pcmk__xml_free(current_cib); current_cib = NULL;
                 break;
             case pcmk_ok:
                 cib_updated = TRUE;
                 break;
             default:
                 crm_notice("[%s] ABORTED: %s (%d)", event, pcmk_strerror(rc), rc);
-                free_xml(current_cib); current_cib = NULL;
+                pcmk__xml_free(current_cib); current_cib = NULL;
         }
     }
 
     if (current_cib == NULL) {
         crm_trace("Re-requesting the full cib");
-        cib->cmds->query(cib, NULL, &current_cib, cib_scope_local | cib_sync_call);
+        cib->cmds->query(cib, NULL, &current_cib, cib_sync_call);
     }
 
     if (options.external_agent) {
         int format = 0;
         crm_element_value_int(diff, PCMK_XA_FORMAT, &format);
-        switch(format) {
-            case 1:
-                crm_diff_update_v1(event, msg);
-                break;
-            case 2:
-                crm_diff_update_v2(event, msg);
-                break;
-            default:
-                crm_err("Unknown patch format: %d", format);
+
+        if (format == 2) {
+            xmlNode *wrapper = pcmk__xe_first_child(msg,
+                                                    PCMK__XE_CIB_UPDATE_RESULT,
+                                                    NULL, NULL);
+            xmlNode *diff = pcmk__xe_first_child(wrapper, NULL, NULL, NULL);
+
+            pcmk__xe_foreach_child(diff, NULL, crm_diff_update_element, NULL);
+
+        } else {
+            crm_err("Unknown patch format: %d", format);
         }
     }
 
@@ -2139,16 +2001,13 @@ mon_refresh_display(gpointer user_data)
         out->reset(out);
     }
 
-    rc = pcmk__output_cluster_status(out, st, cib, current_cib, pcmkd_state,
-                                     fence_history, show, show_opts,
+    rc = pcmk__output_cluster_status(scheduler, st, cib, current_cib,
+                                     pcmkd_state, fence_history, show,
+                                     show_opts,
                                      options.only_node,options.only_rsc,
-                                     options.neg_location_prefix,
-                                     output_format == mon_output_monitor);
+                                     options.neg_location_prefix);
 
-    if (output_format == mon_output_monitor && rc != pcmk_rc_ok) {
-        clean_up(MON_STATUS_WARN);
-        return G_SOURCE_REMOVE;
-    } else if (rc == pcmk_rc_schema_validation) {
+    if (rc == pcmk_rc_schema_validation) {
         clean_up(CRM_EX_CONFIG);
         return G_SOURCE_REMOVE;
     }
@@ -2214,7 +2073,7 @@ refresh_after_event(gboolean data_updated, gboolean enforce)
     setup_fencer_connection();
 
     if (enforce ||
-        ((now - last_refresh) > (options.reconnect_ms / 1000)) ||
+        ((now - last_refresh) > pcmk__timeout_ms2s(options.reconnect_ms)) ||
         updates >= 10) {
         mainloop_set_trigger((crm_trigger_t *) refresh_trigger);
         mainloop_timer_stop(refresh_timer);
@@ -2268,6 +2127,8 @@ clean_up(crm_exit_t exit_code)
 
     g_strfreev(processed_args);
 
+    pe_free_working_set(scheduler);
+
     /* (2) If this is abnormal termination and we're in curses mode, shut down
      * curses first.  Any messages displayed to the screen before curses is shut
      * down will be lost because doing the shut down will also restore the
