@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2023 the Pacemaker project contributors
+ * Copyright 2019-2024 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -23,36 +23,26 @@
  * We believe the following to be acceptable and portable.
  */
 
-#  if defined(HAVE_LIBNCURSES) || defined(HAVE_LIBCURSES)
-#    if defined(HAVE_NCURSES_H) && !defined(HAVE_INCOMPATIBLE_PRINTW)
+#  if PCMK__ENABLE_CURSES
+#    if defined(HAVE_NCURSES_H)
 #      include <ncurses.h>
-#      define CURSES_ENABLED 1
-#    elif defined(HAVE_NCURSES_NCURSES_H) && !defined(HAVE_INCOMPATIBLE_PRINTW)
+#    elif defined(HAVE_NCURSES_NCURSES_H)
 #      include <ncurses/ncurses.h>
-#      define CURSES_ENABLED 1
-#    elif defined(HAVE_CURSES_H) && !defined(HAVE_INCOMPATIBLE_PRINTW)
+#    elif defined(HAVE_CURSES_H)
 #      include <curses.h>
-#      define CURSES_ENABLED 1
-#    elif defined(HAVE_CURSES_CURSES_H) && !defined(HAVE_INCOMPATIBLE_PRINTW)
+#    elif defined(HAVE_CURSES_CURSES_H)
 #      include <curses/curses.h>
-#      define CURSES_ENABLED 1
-#    else
-#      define CURSES_ENABLED 0
 #    endif
-#  else
-#    define CURSES_ENABLED 0
 #  endif
 
 typedef enum mon_output_format_e {
     mon_output_unset,
     mon_output_none,
-    mon_output_monitor,
     mon_output_plain,
     mon_output_console,
     mon_output_xml,
     mon_output_legacy_xml,
     mon_output_html,
-    mon_output_cgi
 } mon_output_format_t;
 
 enum mon_exec_mode {
@@ -64,7 +54,7 @@ enum mon_exec_mode {
 
 void crm_mon_register_messages(pcmk__output_t *out);
 
-#if CURSES_ENABLED
+#if PCMK__ENABLE_CURSES
 pcmk__output_t *crm_mon_mk_curses_output(char **argv);
 void curses_formatted_printf(pcmk__output_t *out, const char *format, ...) G_GNUC_PRINTF(2, 3);
 void curses_formatted_vprintf(pcmk__output_t *out, const char *format, va_list args) G_GNUC_PRINTF(2, 0);
