@@ -48,7 +48,6 @@ enum rsc_command {
     cmd_execute_agent,
     cmd_fail,
     cmd_get_param,
-    cmd_get_property,
     cmd_list_active_ops,
     cmd_list_agents,
     cmd_list_all_ops,
@@ -66,7 +65,6 @@ enum rsc_command {
     cmd_refresh,
     cmd_restart,
     cmd_set_param,
-    cmd_set_property,
     cmd_wait,
     cmd_why,
 };
@@ -94,7 +92,6 @@ struct {
     gchar *prop_value;            // --parameter-value (attribute value)
     guint timeout_ms;             // Parsed from --timeout value
     char *agent_spec;             // Standard and/or provider and/or agent
-    gchar *xml_file;              // Value of (deprecated) --xml-file
     int check_level;              // Optional value of --validate or --force-check
 
     // Resource configuration specified via command-line arguments
@@ -253,7 +250,7 @@ start_mainloop(pcmk_ipc_api_t *capi)
                   count, pcmk__plural_alt(count, "reply", "replies"));
         exit_code = CRM_EX_DISCONNECT; // For unexpected disconnects
         mainloop = g_main_loop_new(NULL, FALSE);
-        g_timeout_add(MESSAGE_TIMEOUT_S * 1000, resource_ipc_timeout, NULL);
+        pcmk__create_timer(MESSAGE_TIMEOUT_S * 1000, resource_ipc_timeout, NULL);
         g_main_loop_run(mainloop);
     }
 }
@@ -383,10 +380,6 @@ command_cb(const gchar *option_name, const gchar *optarg, gpointer data,
         options.rsc_cmd = cmd_get_param;
         pcmk__str_update(&options.prop_name, optarg);
 
-    } else if (pcmk__str_any_of(option_name, "-G", "--get-property", NULL)) {
-        options.rsc_cmd = cmd_get_property;
-        pcmk__str_update(&options.prop_name, optarg);
-
     } else if (pcmk__str_any_of(option_name, "-O", "--list-operations", NULL)) {
         options.rsc_cmd = cmd_list_active_ops;
 
@@ -447,10 +440,6 @@ command_cb(const gchar *option_name, const gchar *optarg, gpointer data,
         options.rsc_cmd = cmd_set_param;
         pcmk__str_update(&options.prop_name, optarg);
 
-    } else if (pcmk__str_any_of(option_name, "-S", "--set-property", NULL)) {
-        options.rsc_cmd = cmd_set_property;
-        pcmk__str_update(&options.prop_name, optarg);
-
     } else if (pcmk__str_eq(option_name, "--wait", pcmk__str_none)) {
         options.rsc_cmd = cmd_wait;
 
@@ -486,9 +475,9 @@ static GOptionEntry query_entries[] = {
       INDENT "--resource and/or --node",
       NULL },
     { "list-options", 0, G_OPTION_FLAG_NONE, G_OPTION_ARG_CALLBACK, command_cb,
-      "List all available options of the given type\n"
+      "List all available options of the given type.\n"
       INDENT "Allowed values:\n"
-      INDENT PCMK__VALUE_PRIMITIVE "(primitive resource meta-attributes), "
+      INDENT PCMK__VALUE_PRIMITIVE " (primitive resource meta-attributes),\n"
       INDENT PCMK_VALUE_FENCING " (parameters common to all fencing resources)",
       "TYPE" },
     { "list-standards", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
@@ -522,11 +511,6 @@ static GOptionEntry query_entries[] = {
       "Display named parameter for resource (use instance attribute\n"
       INDENT "unless --element, --meta, or --utilization is specified)",
       "PARAM" },
-    { "get-property", 'G', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_CALLBACK,
-          command_cb,
-      "Display named property of resource ('class', 'type', or 'provider') "
-      "(requires --resource)",
-      "PROPERTY" },
     { "locate", 'W', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, command_cb,
       "Show node(s) currently running resource",
       NULL },
@@ -588,11 +572,6 @@ static GOptionEntry command_entries[] = {
       "Delete named parameter for resource. Use instance attribute\n"
       INDENT "unless --element, --meta or, --utilization is specified.",
       "PARAM" },
-    { "set-property", 'S', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_CALLBACK,
-          command_cb,
-      "Set named property of resource ('class', 'type', or 'provider') "
-      "(requires -r, -t, -v)",
-      "PROPERTY" },
 
     { NULL }
 };
@@ -776,9 +755,8 @@ static GOptionEntry addl_entries[] = {
       "Force the action to be performed. See help for individual commands for\n"
       INDENT "additional behavior.",
       NULL },
-    { "xml-file", 'x', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME, &options.xml_file,
-      NULL,
-      "FILE" },
+
+    // @COMPAT Used in resource-agents prior to v4.2.0
     { "host-uname", 'H', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &options.host_uname,
       NULL,
       "HOST" },
@@ -838,12 +816,9 @@ timeout_cb(const gchar *option_name, const gchar *optarg, gpointer data, GError
     long long timeout_ms = crm_get_msec(optarg);
 
     if (timeout_ms < 0) {
-        // @COMPAT When we can break backward compatibilty, return FALSE
-        crm_warn("Ignoring invalid timeout '%s'", optarg);
-        options.timeout_ms = 0U;
-    } else {
-        options.timeout_ms = (guint) QB_MIN(timeout_ms, UINT_MAX);
+        return FALSE;
     }
+    options.timeout_ms = (guint) QB_MIN(timeout_ms, UINT_MAX);
     return TRUE;
 }
 
@@ -860,18 +835,18 @@ ban_or_move(pcmk__output_t *out, pcmk_resource_t *rsc,
     current = pe__find_active_requires(rsc, &nactive);
 
     if (nactive == 1) {
-        rc = cli_resource_ban(out, options.rsc_id, current->details->uname, move_lifetime,
-                              cib_conn, cib_sync_call,
+        rc = cli_resource_ban(out, options.rsc_id, current->priv->name,
+                              move_lifetime, cib_conn,
                               options.promoted_role_only, PCMK_ROLE_PROMOTED);
 
-    } else if (pcmk_is_set(rsc->flags, pcmk_rsc_promotable)) {
+    } else if (pcmk_is_set(rsc->flags, pcmk__rsc_promotable)) {
         int count = 0;
         GList *iter = NULL;
 
         current = NULL;
-        for(iter = rsc->children; iter; iter = iter->next) {
+        for (iter = rsc->priv->children; iter != NULL; iter = iter->next) {
             pcmk_resource_t *child = (pcmk_resource_t *)iter->data;
-            enum rsc_role_e child_role = child->fns->state(child, TRUE);
+            enum rsc_role_e child_role = child->priv->fns->state(child, TRUE);
 
             if (child_role == pcmk_role_promoted) {
                 count++;
@@ -880,8 +855,8 @@ ban_or_move(pcmk__output_t *out, pcmk_resource_t *rsc,
         }
 
         if(count == 1 && current) {
-            rc = cli_resource_ban(out, options.rsc_id, current->details->uname, move_lifetime,
-                                  cib_conn, cib_sync_call,
+            rc = cli_resource_ban(out, options.rsc_id, current->priv->name,
+                                  move_lifetime, cib_conn,
                                   options.promoted_role_only,
                                   PCMK_ROLE_PROMOTED);
 
@@ -934,7 +909,7 @@ cleanup(pcmk__output_t *out, pcmk_resource_t *rsc, pcmk_node_t *node)
 }
 
 static int
-clear_constraints(pcmk__output_t *out, xmlNodePtr *cib_xml_copy)
+clear_constraints(pcmk__output_t *out)
 {
     GList *before = NULL;
     GList *after = NULL;
@@ -949,8 +924,7 @@ clear_constraints(pcmk__output_t *out, xmlNodePtr *cib_xml_copy)
 
     if (options.clear_expired) {
         rc = cli_resource_clear_all_expired(scheduler->input, cib_conn,
-                                            cib_sync_call, options.rsc_id,
-                                            options.host_uname,
+                                            options.rsc_id, options.host_uname,
                                             options.promoted_role_only);
 
     } else if (options.host_uname) {
@@ -962,28 +936,29 @@ clear_constraints(pcmk__output_t *out, xmlNodePtr *cib_xml_copy)
             }
             return rc;
         }
-        rc = cli_resource_clear(options.rsc_id, dest->details->uname, NULL,
-                                cib_conn, cib_sync_call, true, options.force);
+        rc = cli_resource_clear(options.rsc_id, dest->priv->name, NULL,
+                                cib_conn, true, options.force);
 
     } else {
         rc = cli_resource_clear(options.rsc_id, NULL, scheduler->nodes,
-                                cib_conn, cib_sync_call, true, options.force);
+                                cib_conn, true, options.force);
     }
 
     if (!out->is_quiet(out)) {
-        rc = cib_conn->cmds->query(cib_conn, NULL, cib_xml_copy, cib_scope_local | cib_sync_call);
+        xmlNode *cib_xml = NULL;
+
+        rc = cib_conn->cmds->query(cib_conn, NULL, &cib_xml, cib_sync_call);
         rc = pcmk_legacy2rc(rc);
 
         if (rc != pcmk_rc_ok) {
             g_set_error(&error, PCMK__RC_ERROR, rc,
                         _("Could not get modified CIB: %s\n"), pcmk_rc_str(rc));
             g_list_free(before);
-            free_xml(*cib_xml_copy);
-            *cib_xml_copy = NULL;
+            pcmk__xml_free(cib_xml);
             return rc;
         }
 
-        scheduler->input = *cib_xml_copy;
+        scheduler->input = cib_xml;
         cluster_status(scheduler);
 
         after = build_constraint_list(scheduler->input);
@@ -1002,36 +977,19 @@ clear_constraints(pcmk__output_t *out, xmlNodePtr *cib_xml_copy)
 }
 
 static int
-initialize_scheduler_data(xmlNodePtr *cib_xml_copy)
+initialize_scheduler_data(xmlNode **cib_xml_orig)
 {
     int rc = pcmk_rc_ok;
 
-    if (options.xml_file != NULL) {
-        *cib_xml_copy = pcmk__xml_read(options.xml_file);
-        if (*cib_xml_copy == NULL) {
-            rc = pcmk_rc_cib_corrupt;
-        }
-    } else {
-        rc = cib_conn->cmds->query(cib_conn, NULL, cib_xml_copy, cib_scope_local | cib_sync_call);
-        rc = pcmk_legacy2rc(rc);
-    }
-
-    if (rc == pcmk_rc_ok) {
-        scheduler = pe_new_working_set();
-        if (scheduler == NULL) {
-            rc = ENOMEM;
-        } else {
-            pcmk__set_scheduler_flags(scheduler,
-                                      pcmk_sched_no_counts
-                                      |pcmk_sched_no_compat);
-            scheduler->priv = out;
-            rc = update_scheduler_input(scheduler, cib_xml_copy);
-        }
+    scheduler = pe_new_working_set();
+    if (scheduler == NULL) {
+        return ENOMEM;
     }
 
+    pcmk__set_scheduler_flags(scheduler, pcmk__sched_no_counts);
+    scheduler->priv->out = out;
+    rc = update_scheduler_input(out, scheduler, cib_conn, cib_xml_orig);
     if (rc != pcmk_rc_ok) {
-        free_xml(*cib_xml_copy);
-        *cib_xml_copy = NULL;
         return rc;
     }
 
@@ -1070,7 +1028,7 @@ refresh(pcmk__output_t *out)
         pcmk_node_t *node = pcmk_find_node(scheduler, options.host_uname);
 
         if (pcmk__is_pacemaker_remote_node(node)) {
-            node = pcmk__current_node(node->details->remote_rsc);
+            node = pcmk__current_node(node->priv->remote);
             if (node == NULL) {
                 rc = ENXIO;
                 g_set_error(&error, PCMK__RC_ERROR, rc,
@@ -1078,7 +1036,7 @@ refresh(pcmk__output_t *out)
                             options.host_uname);
                 return rc;
             }
-            router_node = node->details->uname;
+            router_node = node->priv->name;
             attr_options |= pcmk__node_attr_remote;
         }
     }
@@ -1127,39 +1085,6 @@ refresh_resource(pcmk__output_t *out, pcmk_resource_t *rsc, pcmk_node_t *node)
     }
 }
 
-static int
-set_property(void)
-{
-    int rc = pcmk_rc_ok;
-    xmlNode *msg_data = NULL;
-
-    if (pcmk__str_empty(options.rsc_type)) {
-        g_set_error(&error, PCMK__EXITC_ERROR, CRM_EX_USAGE,
-                    _("Must specify -t with resource type"));
-        rc = ENXIO;
-        return rc;
-
-    } else if (pcmk__str_empty(options.prop_value)) {
-        g_set_error(&error, PCMK__EXITC_ERROR, CRM_EX_USAGE,
-                    _("Must supply -v with new value"));
-        rc = ENXIO;
-        return rc;
-    }
-
-    CRM_LOG_ASSERT(options.prop_name != NULL);
-
-    msg_data = pcmk__xe_create(NULL, options.rsc_type);
-    crm_xml_add(msg_data, PCMK_XA_ID, options.rsc_id);
-    crm_xml_add(msg_data, options.prop_name, options.prop_value);
-
-    rc = cib_conn->cmds->modify(cib_conn, PCMK_XE_RESOURCES, msg_data,
-                                cib_sync_call);
-    rc = pcmk_legacy2rc(rc);
-    free_xml(msg_data);
-
-    return rc;
-}
-
 static int
 show_metadata(pcmk__output_t *out, const char *agent_spec)
 {
@@ -1274,15 +1199,11 @@ get_find_flags(void)
         case cmd_why:
             return pcmk_rsc_match_history|pcmk_rsc_match_anon_basename;
 
-        // @COMPAT See note in is_scheduler_required()
-        case cmd_delete:
         case cmd_delete_param:
         case cmd_get_param:
-        case cmd_get_property:
         case cmd_query_xml_raw:
         case cmd_query_xml:
         case cmd_set_param:
-        case cmd_set_property:
             return pcmk_rsc_match_history|pcmk_rsc_match_basename;
 
         default:
@@ -1409,13 +1330,8 @@ is_scheduler_required(void)
         return false;
     }
 
-    /* @COMPAT cmd_delete does not actually need the scheduler and should not
-     * set find_flags. However, crm_resource --delete currently throws a
-     * "resource not found" error if the resource doesn't exist. This is
-     * incorrect behavior (deleting a nonexistent resource should be considered
-     * success); however, we shouldn't change it until 3.0.0.
-     */
     switch (options.rsc_cmd) {
+        case cmd_delete:
         case cmd_list_agents:
         case cmd_list_alternatives:
         case cmd_list_options:
@@ -1439,10 +1355,10 @@ is_scheduler_required(void)
 static bool
 accept_clone_instance(void)
 {
-    // @COMPAT At 3.0.0, add cmd_delete; for now, don't throw error
     switch (options.rsc_cmd) {
         case cmd_ban:
         case cmd_clear:
+        case cmd_delete:
         case cmd_move:
         case cmd_restart:
             return false;
@@ -1519,7 +1435,7 @@ build_arg_context(pcmk__common_args_t *args, GOptionGroup **group) {
 int
 main(int argc, char **argv)
 {
-    xmlNode *cib_xml_copy = NULL;
+    xmlNode *cib_xml_orig = NULL;
     pcmk_resource_t *rsc = NULL;
     pcmk_node_t *node = NULL;
     uint32_t find_flags = 0;
@@ -1604,7 +1520,7 @@ main(int argc, char **argv)
             len++;
         }
 
-        CRM_ASSERT(len > 0);
+        pcmk__assert(len > 0);
 
         /* Add 1 for the strv[0] string below, and add another 1 for the NULL
          * at the end of the array so g_strjoinv knows when to stop.
@@ -1640,7 +1556,6 @@ main(int argc, char **argv)
              * argument.
              */
             case cmd_get_param:
-            case cmd_get_property:
             case cmd_list_instances:
             case cmd_list_standards:
                 pcmk__output_enable_list_element(out);
@@ -1677,9 +1592,12 @@ main(int argc, char **argv)
         }
 
     } else if (options.cmdline_params != NULL) {
-        // @COMPAT @TODO error out here when we can break backward compatibility
+        exit_code = CRM_EX_USAGE;
+        g_set_error(&error, PCMK__EXITC_ERROR, exit_code,
+                    _("--option must be used with --validate and without -r"));
         g_hash_table_destroy(options.cmdline_params);
         options.cmdline_params = NULL;
+        goto done;
     }
 
     if (is_resource_required() && (options.rsc_id == NULL)) {
@@ -1708,7 +1626,7 @@ main(int argc, char **argv)
                         _("Could not create CIB connection"));
             goto done;
         }
-        rc = cib__signon_attempts(cib_conn, crm_system_name, cib_command, 5);
+        rc = cib__signon_attempts(cib_conn, cib_command, 5);
         rc = pcmk_legacy2rc(rc);
         if (rc != pcmk_rc_ok) {
             exit_code = pcmk_rc2exitc(rc);
@@ -1720,7 +1638,7 @@ main(int argc, char **argv)
 
     // Populate scheduler data from XML file if specified or CIB query otherwise
     if (is_scheduler_required()) {
-        rc = initialize_scheduler_data(&cib_xml_copy);
+        rc = initialize_scheduler_data(&cib_xml_orig);
         if (rc != pcmk_rc_ok) {
             exit_code = pcmk_rc2exitc(rc);
             goto done;
@@ -1731,8 +1649,8 @@ main(int argc, char **argv)
 
     // If command requires that resource exist if specified, find it
     if ((find_flags != 0) && (options.rsc_id != NULL)) {
-        rsc = pe_find_resource_with_flags(scheduler->resources, options.rsc_id,
-                                          find_flags);
+        rsc = pe_find_resource_with_flags(scheduler->priv->resources,
+                                          options.rsc_id, find_flags);
         if (rsc == NULL) {
             exit_code = CRM_EX_NOSUCH;
             g_set_error(&error, PCMK__EXITC_ERROR, exit_code,
@@ -1743,7 +1661,8 @@ main(int argc, char **argv)
         /* The --ban, --clear, --move, and --restart commands do not work with
          * instances of clone resourcs.
          */
-        if (pcmk__is_clone(rsc->parent) && (strchr(options.rsc_id, ':') != NULL)
+        if (pcmk__is_clone(rsc->priv->parent)
+            && (strchr(options.rsc_id, ':') != NULL)
             && !accept_clone_instance()) {
 
             exit_code = CRM_EX_INVALID_PARAM;
@@ -1807,7 +1726,8 @@ main(int argc, char **argv)
         }
 
         case cmd_list_instances:
-            rc = out->message(out, "resource-names-list", scheduler->resources);
+            rc = out->message(out, "resource-names-list",
+                              scheduler->priv->resources);
 
             if (rc != pcmk_rc_ok) {
                 rc = ENXIO;
@@ -1847,7 +1767,7 @@ main(int argc, char **argv)
              */
             rc = cli_resource_restart(out, rsc, node, options.move_lifetime,
                                       options.timeout_ms, cib_conn,
-                                      cib_sync_call, options.promoted_role_only,
+                                      options.promoted_role_only,
                                       options.force);
             break;
 
@@ -1887,8 +1807,8 @@ main(int argc, char **argv)
 
         case cmd_cts:
             rc = pcmk_rc_ok;
-            g_list_foreach(scheduler->resources, (GFunc) cli_resource_print_cts,
-                           out);
+            g_list_foreach(scheduler->priv->resources,
+                           (GFunc) cli_resource_print_cts, out);
             cli_resource_print_cts_constraints(scheduler);
             break;
 
@@ -1932,12 +1852,12 @@ main(int argc, char **argv)
                 rc = pcmk_rc_node_unknown;
             } else {
                 rc = out->message(out, "resource-reasons-list",
-                                  scheduler->resources, rsc, node);
+                                  scheduler->priv->resources, rsc, node);
             }
             break;
 
         case cmd_clear:
-            rc = clear_constraints(out, &cib_xml_copy);
+            rc = clear_constraints(out);
             break;
 
         case cmd_move:
@@ -1946,8 +1866,7 @@ main(int argc, char **argv)
             } else {
                 rc = cli_resource_move(rsc, options.rsc_id, options.host_uname,
                                        options.move_lifetime, cib_conn,
-                                       cib_sync_call, scheduler,
-                                       options.promoted_role_only,
+                                       scheduler, options.promoted_role_only,
                                        options.force);
             }
 
@@ -1964,9 +1883,9 @@ main(int argc, char **argv)
             } else if (node == NULL) {
                 rc = pcmk_rc_node_unknown;
             } else {
-                rc = cli_resource_ban(out, options.rsc_id, node->details->uname,
+                rc = cli_resource_ban(out, options.rsc_id, node->priv->name,
                                       options.move_lifetime, cib_conn,
-                                      cib_sync_call, options.promoted_role_only,
+                                      options.promoted_role_only,
                                       PCMK_ROLE_PROMOTED);
             }
 
@@ -1977,22 +1896,11 @@ main(int argc, char **argv)
 
             break;
 
-        case cmd_get_property:
-            rc = out->message(out, "property-list", rsc, options.prop_name);
-            if (rc == pcmk_rc_no_output) {
-                rc = ENXIO;
-            }
-
-            break;
-
-        case cmd_set_property:
-            rc = set_property();
-            break;
-
         case cmd_get_param: {
             unsigned int count = 0;
             GHashTable *params = NULL;
-            pcmk_node_t *current = rsc->fns->active_node(rsc, &count, NULL);
+            pcmk_node_t *current = rsc->priv->fns->active_node(rsc, &count,
+                                                               NULL);
             bool free_params = true;
             const char* value = NULL;
 
@@ -2021,18 +1929,17 @@ main(int argc, char **argv)
 
             } else if (pcmk__str_eq(options.attr_set_type, ATTR_SET_ELEMENT, pcmk__str_none)) {
 
-                value = crm_element_value(rsc->xml, options.prop_name);
+                value = crm_element_value(rsc->priv->xml, options.prop_name);
                 free_params = false;
 
             } else {
                 pe_rule_eval_data_t rule_data = {
-                    .now = scheduler->now,
+                    .now = scheduler->priv->now,
                 };
 
                 params = pcmk__strkey_table(free, free);
-                pe__unpack_dataset_nvpairs(rsc->xml, PCMK_XE_UTILIZATION,
-                                           &rule_data, params, NULL, FALSE,
-                                           scheduler);
+                pe__unpack_dataset_nvpairs(rsc->priv->xml, PCMK_XE_UTILIZATION,
+                                           &rule_data, params, NULL, scheduler);
 
                 value = g_hash_table_lookup(params, options.prop_name);
             }
@@ -2061,7 +1968,7 @@ main(int argc, char **argv)
                                                options.prop_name,
                                                options.prop_value,
                                                options.recursive, cib_conn,
-                                               options.force);
+                                               cib_xml_orig, options.force);
             break;
 
         case cmd_delete_param:
@@ -2071,7 +1978,7 @@ main(int argc, char **argv)
                                                options.attr_set_type,
                                                options.prop_id,
                                                options.prop_name, cib_conn,
-                                               cib_sync_call, options.force);
+                                               cib_xml_orig, options.force);
             break;
 
         case cmd_cleanup:
@@ -2100,9 +2007,8 @@ main(int argc, char **argv)
              * command line arguments.
              */
             if (options.rsc_type == NULL) {
-                // @COMPAT @TODO change this to exit_code = CRM_EX_USAGE
-                rc = ENXIO;
-                g_set_error(&error, PCMK__RC_ERROR, rc,
+                exit_code = CRM_EX_USAGE;
+                g_set_error(&error, PCMK__EXITC_ERROR, CRM_EX_USAGE,
                             _("You need to specify a resource type with -t"));
             } else {
                 rc = pcmk__resource_delete(cib_conn, cib_sync_call,
@@ -2154,6 +2060,8 @@ done:
         }
     }
 
+    pcmk__xml_free(cib_xml_orig);
+
     g_free(options.host_uname);
     g_free(options.interval_spec);
     g_free(options.move_lifetime);
@@ -2168,7 +2076,6 @@ done:
     free(options.v_agent);
     free(options.v_class);
     free(options.v_provider);
-    g_free(options.xml_file);
     g_strfreev(options.remainder);
 
     if (options.override_params != NULL) {
