@@ -59,7 +59,7 @@ parse_cli_lifetime(pcmk__output_t *out, const char *move_lifetime)
 // \return Standard Pacemaker return code
 int
 cli_resource_ban(pcmk__output_t *out, const char *rsc_id, const char *host,
-                 const char *move_lifetime, cib_t * cib_conn, int cib_options,
+                 const char *move_lifetime, cib_t *cib_conn,
                  gboolean promoted_role_only, const char *promoted_role)
 {
     char *later_s = NULL;
@@ -75,7 +75,7 @@ cli_resource_ban(pcmk__output_t *out, const char *rsc_id, const char *host,
     fragment = pcmk__xe_create(NULL, PCMK_XE_CONSTRAINTS);
 
     location = pcmk__xe_create(fragment, PCMK_XE_RSC_LOCATION);
-    crm_xml_set_id(location, "cli-ban-%s-on-%s", rsc_id, host);
+    pcmk__xe_set_id(location, "cli-ban-%s-on-%s", rsc_id, host);
 
     out->info(out,
               "WARNING: Creating " PCMK_XE_RSC_LOCATION " constraint '%s' with "
@@ -104,28 +104,28 @@ cli_resource_ban(pcmk__output_t *out, const char *rsc_id, const char *host,
         xmlNode *rule = pcmk__xe_create(location, PCMK_XE_RULE);
         xmlNode *expr = pcmk__xe_create(rule, PCMK_XE_EXPRESSION);
 
-        crm_xml_set_id(rule, "cli-ban-%s-on-%s-rule", rsc_id, host);
+        pcmk__xe_set_id(rule, "cli-ban-%s-on-%s-rule", rsc_id, host);
         crm_xml_add(rule, PCMK_XA_SCORE, PCMK_VALUE_MINUS_INFINITY);
         crm_xml_add(rule, PCMK_XA_BOOLEAN_OP, PCMK_VALUE_AND);
 
-        crm_xml_set_id(expr, "cli-ban-%s-on-%s-expr", rsc_id, host);
+        pcmk__xe_set_id(expr, "cli-ban-%s-on-%s-expr", rsc_id, host);
         crm_xml_add(expr, PCMK_XA_ATTRIBUTE, CRM_ATTR_UNAME);
         crm_xml_add(expr, PCMK_XA_OPERATION, PCMK_VALUE_EQ);
         crm_xml_add(expr, PCMK_XA_VALUE, host);
         crm_xml_add(expr, PCMK_XA_TYPE, PCMK_VALUE_STRING);
 
         expr = pcmk__xe_create(rule, PCMK_XE_DATE_EXPRESSION);
-        crm_xml_set_id(expr, "cli-ban-%s-on-%s-lifetime", rsc_id, host);
+        pcmk__xe_set_id(expr, "cli-ban-%s-on-%s-lifetime", rsc_id, host);
         crm_xml_add(expr, PCMK_XA_OPERATION, PCMK_VALUE_LT);
         crm_xml_add(expr, PCMK_XA_END, later_s);
     }
 
     crm_log_xml_notice(fragment, "Modify");
     rc = cib_conn->cmds->modify(cib_conn, PCMK_XE_CONSTRAINTS, fragment,
-                                cib_options);
+                                cib_sync_call);
     rc = pcmk_legacy2rc(rc);
 
-    free_xml(fragment);
+    pcmk__xml_free(fragment);
     free(later_s);
 
     if ((rc != pcmk_rc_ok)
@@ -133,8 +133,8 @@ cli_resource_ban(pcmk__output_t *out, const char *rsc_id, const char *host,
         && (strcmp(promoted_role, PCMK_ROLE_PROMOTED) == 0)) {
 
         int banrc = cli_resource_ban(out, rsc_id, host, move_lifetime,
-                              cib_conn, cib_options, promoted_role_only,
-                              PCMK__ROLE_PROMOTED_LEGACY);
+                                     cib_conn, promoted_role_only,
+                                     PCMK__ROLE_PROMOTED_LEGACY);
         if (banrc == pcmk_rc_ok) {
             rc = banrc;
         }
@@ -146,7 +146,7 @@ cli_resource_ban(pcmk__output_t *out, const char *rsc_id, const char *host,
 // \return Standard Pacemaker return code
 int
 cli_resource_prefer(pcmk__output_t *out,const char *rsc_id, const char *host,
-                    const char *move_lifetime, cib_t *cib_conn, int cib_options,
+                    const char *move_lifetime, cib_t *cib_conn,
                     gboolean promoted_role_only, const char *promoted_role)
 {
     char *later_s = parse_cli_lifetime(out, move_lifetime);
@@ -166,7 +166,7 @@ cli_resource_prefer(pcmk__output_t *out,const char *rsc_id, const char *host,
     fragment = pcmk__xe_create(NULL, PCMK_XE_CONSTRAINTS);
 
     location = pcmk__xe_create(fragment, PCMK_XE_RSC_LOCATION);
-    crm_xml_set_id(location, "cli-prefer-%s", rsc_id);
+    pcmk__xe_set_id(location, "cli-prefer-%s", rsc_id);
 
     crm_xml_add(location, PCMK_XA_RSC, rsc_id);
     if(promoted_role_only) {
@@ -184,28 +184,28 @@ cli_resource_prefer(pcmk__output_t *out,const char *rsc_id, const char *host,
         xmlNode *rule = pcmk__xe_create(location, PCMK_XE_RULE);
         xmlNode *expr = pcmk__xe_create(rule, PCMK_XE_EXPRESSION);
 
-        crm_xml_set_id(rule, "cli-prefer-rule-%s", rsc_id);
+        pcmk__xe_set_id(rule, "cli-prefer-rule-%s", rsc_id);
         crm_xml_add(rule, PCMK_XA_SCORE, PCMK_VALUE_INFINITY);
         crm_xml_add(rule, PCMK_XA_BOOLEAN_OP, PCMK_VALUE_AND);
 
-        crm_xml_set_id(expr, "cli-prefer-expr-%s", rsc_id);
+        pcmk__xe_set_id(expr, "cli-prefer-expr-%s", rsc_id);
         crm_xml_add(expr, PCMK_XA_ATTRIBUTE, CRM_ATTR_UNAME);
         crm_xml_add(expr, PCMK_XA_OPERATION, PCMK_VALUE_EQ);
         crm_xml_add(expr, PCMK_XA_VALUE, host);
         crm_xml_add(expr, PCMK_XA_TYPE, PCMK_VALUE_STRING);
 
         expr = pcmk__xe_create(rule, PCMK_XE_DATE_EXPRESSION);
-        crm_xml_set_id(expr, "cli-prefer-lifetime-end-%s", rsc_id);
+        pcmk__xe_set_id(expr, "cli-prefer-lifetime-end-%s", rsc_id);
         crm_xml_add(expr, PCMK_XA_OPERATION, PCMK_VALUE_LT);
         crm_xml_add(expr, PCMK_XA_END, later_s);
     }
 
     crm_log_xml_info(fragment, "Modify");
     rc = cib_conn->cmds->modify(cib_conn, PCMK_XE_CONSTRAINTS, fragment,
-                                cib_options);
+                                cib_sync_call);
     rc = pcmk_legacy2rc(rc);
 
-    free_xml(fragment);
+    pcmk__xml_free(fragment);
     free(later_s);
 
     if ((rc != pcmk_rc_ok)
@@ -213,8 +213,8 @@ cli_resource_prefer(pcmk__output_t *out,const char *rsc_id, const char *host,
         && (strcmp(promoted_role, PCMK_ROLE_PROMOTED) == 0)) {
 
         int preferrc = cli_resource_prefer(out, rsc_id, host, move_lifetime,
-                                 cib_conn, cib_options, promoted_role_only,
-                                 PCMK__ROLE_PROMOTED_LEGACY);
+                                           cib_conn, promoted_role_only,
+                                           PCMK__ROLE_PROMOTED_LEGACY);
         if (preferrc == pcmk_rc_ok) {
             rc = preferrc;
         }
@@ -245,8 +245,8 @@ cli_resource_prefer(pcmk__output_t *out,const char *rsc_id, const char *host,
  * \return Standard Pacemaker return code
  */
 static int
-resource_clear_node_in_expr(const char *rsc_id, const char *host, cib_t * cib_conn,
-                            int cib_options)
+resource_clear_node_in_expr(const char *rsc_id, const char *host,
+                            cib_t *cib_conn)
 {
     int rc = pcmk_rc_ok;
     char *xpath_string = NULL;
@@ -262,7 +262,8 @@ resource_clear_node_in_expr(const char *rsc_id, const char *host, cib_t * cib_co
 
     xpath_string = crm_strdup_printf(XPATH_FMT, rsc_id, rsc_id, host);
 
-    rc = cib_conn->cmds->remove(cib_conn, xpath_string, NULL, cib_xpath | cib_options);
+    rc = cib_conn->cmds->remove(cib_conn, xpath_string, NULL,
+                                cib_xpath|cib_sync_call);
     if (rc == -ENXIO) {
         rc = pcmk_rc_ok;
     } else {
@@ -276,7 +277,7 @@ resource_clear_node_in_expr(const char *rsc_id, const char *host, cib_t * cib_co
 // \return Standard Pacemaker return code
 static int
 resource_clear_node_in_location(const char *rsc_id, const char *host, cib_t * cib_conn,
-                                int cib_options, bool clear_ban_constraints, gboolean force)
+                                bool clear_ban_constraints, gboolean force)
 {
     int rc = pcmk_rc_ok;
     xmlNode *fragment = NULL;
@@ -286,32 +287,32 @@ resource_clear_node_in_location(const char *rsc_id, const char *host, cib_t * ci
 
     if (clear_ban_constraints == TRUE) {
         location = pcmk__xe_create(fragment, PCMK_XE_RSC_LOCATION);
-        crm_xml_set_id(location, "cli-ban-%s-on-%s", rsc_id, host);
+        pcmk__xe_set_id(location, "cli-ban-%s-on-%s", rsc_id, host);
     }
 
     location = pcmk__xe_create(fragment, PCMK_XE_RSC_LOCATION);
-    crm_xml_set_id(location, "cli-prefer-%s", rsc_id);
+    pcmk__xe_set_id(location, "cli-prefer-%s", rsc_id);
     if (force == FALSE) {
         crm_xml_add(location, PCMK_XE_NODE, host);
     }
 
     crm_log_xml_info(fragment, "Delete");
     rc = cib_conn->cmds->remove(cib_conn, PCMK_XE_CONSTRAINTS, fragment,
-                                cib_options);
+                                cib_sync_call);
     if (rc == -ENXIO) {
         rc = pcmk_rc_ok;
     } else {
         rc = pcmk_legacy2rc(rc);
     }
 
-    free_xml(fragment);
+    pcmk__xml_free(fragment);
     return rc;
 }
 
 // \return Standard Pacemaker return code
 int
 cli_resource_clear(const char *rsc_id, const char *host, GList *allnodes, cib_t * cib_conn,
-                   int cib_options, bool clear_ban_constraints, gboolean force)
+                   bool clear_ban_constraints, gboolean force)
 {
     int rc = pcmk_rc_ok;
 
@@ -320,7 +321,7 @@ cli_resource_clear(const char *rsc_id, const char *host, GList *allnodes, cib_t
     }
 
     if (host) {
-        rc = resource_clear_node_in_expr(rsc_id, host, cib_conn, cib_options);
+        rc = resource_clear_node_in_expr(rsc_id, host, cib_conn);
 
         /* rc does not tell us whether the previous operation did anything, only
          * whether it failed or not.  Thus, as long as it did not fail, we need
@@ -328,8 +329,7 @@ cli_resource_clear(const char *rsc_id, const char *host, GList *allnodes, cib_t
          */
         if (rc == pcmk_rc_ok) {
             rc = resource_clear_node_in_location(rsc_id, host, cib_conn,
-                                                 cib_options, clear_ban_constraints,
-                                                 force);
+                                                 clear_ban_constraints, force);
         }
 
     } else {
@@ -341,9 +341,8 @@ cli_resource_clear(const char *rsc_id, const char *host, GList *allnodes, cib_t
         for(; n; n = n->next) {
             pcmk_node_t *target = n->data;
 
-            rc = cli_resource_clear(rsc_id, target->details->uname, NULL,
-                                    cib_conn, cib_options, clear_ban_constraints,
-                                    force);
+            rc = cli_resource_clear(rsc_id, target->priv->name, NULL,
+                                    cib_conn, clear_ban_constraints, force);
             if (rc != pcmk_rc_ok) {
                 break;
             }
@@ -365,7 +364,7 @@ build_clear_xpath_string(GString *buf, const xmlNode *constraint_node,
                                      "' or @" PCMK_XA_ROLE "='"
                                      PCMK__ROLE_PROMOTED_LEGACY "'";
 
-    CRM_ASSERT(buf != NULL);
+    pcmk__assert(buf != NULL);
     g_string_truncate(buf, 0);
 
     if (!pcmk__starts_with(cons_id, "cli-ban-")
@@ -436,8 +435,8 @@ build_clear_xpath_string(GString *buf, const xmlNode *constraint_node,
 
 // \return Standard Pacemaker return code
 int
-cli_resource_clear_all_expired(xmlNode *root, cib_t *cib_conn, int cib_options,
-                               const char *rsc, const char *node, gboolean promoted_role_only)
+cli_resource_clear_all_expired(xmlNode *root, cib_t *cib_conn, const char *rsc,
+                               const char *node, gboolean promoted_role_only)
 {
     GString *buf = NULL;
     xmlXPathObject *xpathObj = NULL;
@@ -473,12 +472,13 @@ cli_resource_clear_all_expired(xmlNode *root, cib_t *cib_conn, int cib_options,
 
         /* And then finally, see if the date expression is expired.  If so,
          * clear the constraint.
-         *
-         * @COMPAT Check for error once we are rejecting rules with invalid end
          */
         rc = pcmk__xe_get_datetime(date_expr_node, PCMK_XA_END, &end);
         if (rc != pcmk_rc_ok) {
-            crm_trace("Invalid " PCMK_XA_END ": %s", pcmk_rc_str(rc));
+            crm_trace("Date expression %s has invalid " PCMK_XA_END ": %s",
+                      pcmk__s(pcmk__xe_id(date_expr_node), "without ID"),
+                      pcmk_rc_str(rc));
+            continue; // Treat as unexpired
         }
 
         if (crm_time_compare(now, end) == 1) {
@@ -487,18 +487,18 @@ cli_resource_clear_all_expired(xmlNode *root, cib_t *cib_conn, int cib_options,
 
             fragment = pcmk__xe_create(NULL, PCMK_XE_CONSTRAINTS);
             location = pcmk__xe_create(fragment, PCMK_XE_RSC_LOCATION);
-            crm_xml_set_id(location, "%s", pcmk__xe_id(constraint_node));
+            pcmk__xe_set_id(location, "%s", pcmk__xe_id(constraint_node));
             crm_log_xml_info(fragment, "Delete");
 
             rc = cib_conn->cmds->remove(cib_conn, PCMK_XE_CONSTRAINTS, fragment,
-                                        cib_options);
+                                        cib_sync_call);
             rc = pcmk_legacy2rc(rc);
 
             if (rc != pcmk_rc_ok) {
                 goto done;
             }
 
-            free_xml(fragment);
+            pcmk__xml_free(fragment);
         }
 
         crm_time_free(end);
