@@ -21,23 +21,13 @@ static int
 print_constraint(xmlNode *xml_obj, void *userdata)
 {
     pcmk_scheduler_t *scheduler = (pcmk_scheduler_t *) userdata;
-    pcmk__output_t *out = scheduler->priv;
-    xmlNode *lifetime = NULL;
+    pcmk__output_t *out = scheduler->priv->out;
     const char *id = crm_element_value(xml_obj, PCMK_XA_ID);
-    pcmk_rule_input_t rule_input = {
-        .now = scheduler->now,
-    };
 
     if (id == NULL) {
         return pcmk_rc_ok;
     }
 
-    // @COMPAT PCMK__XE_LIFETIME is deprecated
-    lifetime = pcmk__xe_first_child(xml_obj, PCMK__XE_LIFETIME, NULL, NULL);
-    if (pcmk__evaluate_rules(lifetime, &rule_input, NULL) != pcmk_rc_ok) {
-        return pcmk_rc_ok;
-    }
-
     if (!pcmk__xe_is(xml_obj, PCMK_XE_RSC_COLOCATION)) {
         return pcmk_rc_ok;
     }
@@ -67,28 +57,29 @@ cli_resource_print_cts(pcmk_resource_t *rsc, pcmk__output_t *out)
 {
     const char *host = NULL;
     bool needs_quorum = TRUE;
-    const char *rtype = crm_element_value(rsc->xml, PCMK_XA_TYPE);
-    const char *rprov = crm_element_value(rsc->xml, PCMK_XA_PROVIDER);
-    const char *rclass = crm_element_value(rsc->xml, PCMK_XA_CLASS);
+    const char *rtype = crm_element_value(rsc->priv->xml, PCMK_XA_TYPE);
+    const char *rprov = crm_element_value(rsc->priv->xml, PCMK_XA_PROVIDER);
+    const char *rclass = crm_element_value(rsc->priv->xml, PCMK_XA_CLASS);
     pcmk_node_t *node = pcmk__current_node(rsc);
 
-    if (pcmk__str_eq(rclass, PCMK_RESOURCE_CLASS_STONITH, pcmk__str_casei)) {
+    if (pcmk_is_set(rsc->flags, pcmk__rsc_fence_device)) {
         needs_quorum = FALSE;
     } else {
         // @TODO check requires in resource meta-data and rsc_defaults
     }
 
     if (node != NULL) {
-        host = node->details->uname;
+        host = node->priv->name;
     }
 
     out->info(out, "Resource: %s %s %s %s %s %s %s %s %d %lld %#.16llx",
-              rsc->xml->name, rsc->id,
-              rsc->clone_name ? rsc->clone_name : rsc->id, rsc->parent ? rsc->parent->id : "NA",
+              rsc->priv->xml->name, rsc->id,
+              pcmk__s(rsc->priv->history_id, rsc->id),
+              ((rsc->priv->parent == NULL)? "NA" : rsc->priv->parent->id),
               rprov ? rprov : "NA", rclass, rtype, host ? host : "NA", needs_quorum, rsc->flags,
               rsc->flags);
 
-    g_list_foreach(rsc->children, (GFunc) cli_resource_print_cts, out);
+    g_list_foreach(rsc->priv->children, (GFunc) cli_resource_print_cts, out);
 }
 
 // \return Standard Pacemaker return code
@@ -96,7 +87,7 @@ int
 cli_resource_print_operations(const char *rsc_id, const char *host_uname,
                               bool active, pcmk_scheduler_t *scheduler)
 {
-    pcmk__output_t *out = scheduler->priv;
+    pcmk__output_t *out = scheduler->priv->out;
     int rc = pcmk_rc_no_output;
     GList *ops = find_operations(rsc_id, host_uname, active, scheduler);
 
@@ -121,14 +112,14 @@ int
 cli_resource_print(pcmk_resource_t *rsc, pcmk_scheduler_t *scheduler,
                    bool expanded)
 {
-    pcmk__output_t *out = scheduler->priv;
+    pcmk__output_t *out = scheduler->priv->out;
     uint32_t show_opts = pcmk_show_pending;
     GList *all = NULL;
 
     all = g_list_prepend(all, (gpointer) "*");
 
     out->begin_list(out, NULL, NULL, "Resource Config");
-    out->message(out, pcmk__map_element_name(rsc->xml), show_opts, rsc, all,
+    out->message(out, (const char *) rsc->priv->xml->name, show_opts, rsc, all,
                  all);
     out->message(out, "resource-config", rsc, !expanded);
     out->end_list(out);
@@ -160,7 +151,8 @@ attribute_changed_xml(pcmk__output_t *out, va_list args)
 {
     attr_update_data_t *ud = va_arg(args, attr_update_data_t *);
 
-    pcmk__output_xml_create_parent(out, (const char *) ud->rsc->xml->name,
+    pcmk__output_xml_create_parent(out,
+                                   (const char *) ud->rsc->priv->xml->name,
                                    PCMK_XA_ID, ud->rsc->id,
                                    NULL);
 
@@ -261,7 +253,7 @@ agent_status_default(pcmk__output_t *out, va_list args) {
                   class,
                   ((provider == NULL)? "" : ":"),
                   ((provider == NULL)? "" : provider),
-                  type, (int) rc, services_ocf_exitcode_str((int) rc),
+                  type, (int) rc, crm_exit_str(rc),
                   ((exit_reason == NULL)? "" : ": "),
                   ((exit_reason == NULL)? "" : exit_reason));
     } else {
@@ -297,7 +289,7 @@ agent_status_xml(pcmk__output_t *out, va_list args) {
     const char *exit_reason = va_arg(args, const char *);
 
     char *exit_s = pcmk__itoa(rc);
-    const char *message = services_ocf_exitcode_str((int) rc);
+    const char *message = crm_exit_str(rc);
     char *status_s = pcmk__itoa(status);
     const char *execution_message = pcmk_exec_status_str(status);
 
@@ -368,38 +360,6 @@ override_xml(pcmk__output_t *out, va_list args) {
     return pcmk_rc_ok;
 }
 
-PCMK__OUTPUT_ARGS("property-list", "pcmk_resource_t *", "const char *")
-static int
-property_list_default(pcmk__output_t *out, va_list args) {
-    pcmk_resource_t *rsc = va_arg(args, pcmk_resource_t *);
-    const char *attr = va_arg(args, char *);
-
-    const char *value = crm_element_value(rsc->xml, attr);
-
-    if (value != NULL) {
-        out->begin_list(out, NULL, NULL, "Properties");
-        out->list_item(out, attr, "%s", value);
-        out->end_list(out);
-    }
-
-    return pcmk_rc_ok;
-}
-
-PCMK__OUTPUT_ARGS("property-list", "pcmk_resource_t *", "const char *")
-static int
-property_list_text(pcmk__output_t *out, va_list args) {
-    pcmk_resource_t *rsc = va_arg(args, pcmk_resource_t *);
-    const char *attr = va_arg(args, const char *);
-
-    const char *value = crm_element_value(rsc->xml, attr);
-
-    if (value != NULL) {
-        pcmk__formatted_printf(out, "%s\n", value);
-    }
-
-    return pcmk_rc_ok;
-}
-
 PCMK__OUTPUT_ARGS("resource-agent-action", "int", "const char *", "const char *",
                   "const char *", "const char *", "const char *", "GHashTable *",
                   "crm_exit_t", "int", "const char *", "const char *", "const char *")
@@ -451,7 +411,7 @@ resource_agent_action_default(pcmk__output_t *out, va_list args) {
         }
         if (doc != NULL) {
             out->output_xml(out, PCMK_XE_COMMAND, stdout_data);
-            xmlFreeNode(doc);
+            pcmk__xml_free(doc);
         } else {
             out->subprocess_output(out, rc, stdout_data, stderr_data);
         }
@@ -519,7 +479,7 @@ resource_agent_action_xml(pcmk__output_t *out, va_list args) {
         }
         if (doc != NULL) {
             out->output_xml(out, PCMK_XE_COMMAND, stdout_data);
-            xmlFreeNode(doc);
+            pcmk__xml_free(doc);
         } else {
             out->subprocess_output(out, rc, stdout_data, stderr_data);
         }
@@ -535,6 +495,7 @@ resource_check_list_default(pcmk__output_t *out, va_list args) {
     resource_checks_t *checks = va_arg(args, resource_checks_t *);
 
     const pcmk_resource_t *parent = pe__const_top_resource(checks->rsc, false);
+    const pcmk_scheduler_t *scheduler = checks->rsc->priv->scheduler;
 
     if (checks->flags == 0) {
         return pcmk_rc_no_output;
@@ -567,7 +528,7 @@ resource_check_list_default(pcmk__output_t *out, va_list args) {
                        "'%s' cannot run on unhealthy nodes due to "
                        PCMK_OPT_NODE_HEALTH_STRATEGY "='%s'",
                        parent->id,
-                       pcmk__cluster_option(checks->rsc->cluster->config_hash,
+                       pcmk__cluster_option(scheduler->priv->options,
                                             PCMK_OPT_NODE_HEALTH_STRATEGY));
     }
 
@@ -639,11 +600,7 @@ resource_search_list_default(pcmk__output_t *out, va_list args)
             const char *role_text = "";
 
             if (ni->promoted) {
-#ifdef PCMK__COMPAT_2_0
-                role_text = " " PCMK__ROLE_PROMOTED_LEGACY;
-#else
                 role_text = " " PCMK_ROLE_PROMOTED;
-#endif
             }
             out->list_item(out, "node", "resource %s is running on: %s%s",
                            requested_name, ni->node_name, role_text);
@@ -692,7 +649,7 @@ resource_reasons_list_default(pcmk__output_t *out, va_list args)
     pcmk_resource_t *rsc = va_arg(args, pcmk_resource_t *);
     pcmk_node_t *node = va_arg(args, pcmk_node_t *);
 
-    const char *host_uname = (node == NULL)? NULL : node->details->uname;
+    const char *host_uname = (node == NULL)? NULL : node->priv->name;
 
     out->begin_list(out, NULL, NULL, "Resource Reasons");
 
@@ -702,7 +659,8 @@ resource_reasons_list_default(pcmk__output_t *out, va_list args)
 
         for (lpc = resources; lpc != NULL; lpc = lpc->next) {
             pcmk_resource_t *rsc = (pcmk_resource_t *) lpc->data;
-            rsc->fns->location(rsc, &hosts, TRUE);
+
+            rsc->priv->fns->location(rsc, &hosts, pcmk__rsc_node_current);
 
             if (hosts == NULL) {
                 out->list_item(out, "reason", "Resource %s is not running", rsc->id);
@@ -727,8 +685,8 @@ resource_reasons_list_default(pcmk__output_t *out, va_list args)
         cli_resource_check(out, rsc, node);
 
     } else if ((rsc == NULL) && (host_uname != NULL)) {
-        const char* host_uname =  node->details->uname;
-        GList *allResources = node->details->allocated_rsc;
+        const char* host_uname =  node->priv->name;
+        GList *allResources = node->priv->assigned_resources;
         GList *activeResources = node->details->running_rsc;
         GList *unactiveResources = pcmk__subtract_lists(allResources, activeResources, (GCompareFunc) strcmp);
         GList *lpc = NULL;
@@ -754,7 +712,7 @@ resource_reasons_list_default(pcmk__output_t *out, va_list args)
     } else if ((rsc != NULL) && (host_uname == NULL)) {
         GList *hosts = NULL;
 
-        rsc->fns->location(rsc, &hosts, TRUE);
+        rsc->priv->fns->location(rsc, &hosts, pcmk__rsc_node_current);
         out->list_item(out, "reason", "Resource %s is %srunning",
                        rsc->id, (hosts? "" : "not "));
         cli_resource_check(out, rsc, NULL);
@@ -774,7 +732,7 @@ resource_reasons_list_xml(pcmk__output_t *out, va_list args)
     pcmk_resource_t *rsc = va_arg(args, pcmk_resource_t *);
     pcmk_node_t *node = va_arg(args, pcmk_node_t *);
 
-    const char *host_uname = (node == NULL)? NULL : node->details->uname;
+    const char *host_uname = (node == NULL)? NULL : node->priv->name;
 
     xmlNodePtr xml_node = pcmk__output_xml_create_parent(out, PCMK_XE_REASON,
                                                          NULL);
@@ -789,7 +747,7 @@ resource_reasons_list_xml(pcmk__output_t *out, va_list args)
             pcmk_resource_t *rsc = (pcmk_resource_t *) lpc->data;
             const char *running = NULL;
 
-            rsc->fns->location(rsc, &hosts, TRUE);
+            rsc->priv->fns->location(rsc, &hosts, pcmk__rsc_node_current);
             running = pcmk__btoa(hosts != NULL);
 
             pcmk__output_xml_create_parent(out, PCMK_XE_RESOURCE,
@@ -813,8 +771,8 @@ resource_reasons_list_xml(pcmk__output_t *out, va_list args)
         cli_resource_check(out, rsc, node);
 
     } else if ((rsc == NULL) && (host_uname != NULL)) {
-        const char* host_uname =  node->details->uname;
-        GList *allResources = node->details->allocated_rsc;
+        const char* host_uname =  node->priv->name;
+        GList *allResources = node->priv->assigned_resources;
         GList *activeResources = node->details->running_rsc;
         GList *unactiveResources = pcmk__subtract_lists(allResources, activeResources, (GCompareFunc) strcmp);
         GList *lpc = NULL;
@@ -855,7 +813,7 @@ resource_reasons_list_xml(pcmk__output_t *out, va_list args)
     } else if ((rsc != NULL) && (host_uname == NULL)) {
         GList *hosts = NULL;
 
-        rsc->fns->location(rsc, &hosts, TRUE);
+        rsc->priv->fns->location(rsc, &hosts, pcmk__rsc_node_current);
         crm_xml_add(xml_node, PCMK_XA_RUNNING, pcmk__btoa(hosts != NULL));
         cli_resource_check(out, rsc, NULL);
         g_list_free(hosts);
@@ -868,14 +826,14 @@ resource_reasons_list_xml(pcmk__output_t *out, va_list args)
 static void
 add_resource_name(pcmk_resource_t *rsc, pcmk__output_t *out)
 {
-    if (rsc->children == NULL) {
+    if (rsc->priv->children == NULL) {
         /* Sometimes PCMK_XE_RESOURCE might act as a PCMK_XA_NAME instead of an
          * XML element name, depending on whether pcmk__output_enable_list_element
          * was called.
          */
         out->list_item(out, PCMK_XE_RESOURCE, "%s", rsc->id);
     } else {
-        g_list_foreach(rsc->children, (GFunc) add_resource_name, out);
+        g_list_foreach(rsc->priv->children, (GFunc) add_resource_name, out);
     }
 }
 
@@ -906,8 +864,6 @@ static pcmk__message_entry_t fmt_functions[] = {
     { "attribute-list", "text", attribute_list_text },
     { "override", "default", override_default },
     { "override", "xml", override_xml },
-    { "property-list", "default", property_list_default },
-    { "property-list", "text", property_list_text },
     { "resource-agent-action", "default", resource_agent_action_default },
     { "resource-agent-action", "xml", resource_agent_action_xml },
     { "resource-check-list", "default", resource_check_list_default },
