@@ -421,11 +421,12 @@ check_file_exists(const char *filename, bool should_exist, GError **error)
     }
 
     if (should_exist && (stat(filename, &buf) < 0)) {
-        // @COMPAT: Use pcmk_rc2exitc(errno)?
-        exit_code = CRM_EX_NOSUCH;
+        int rc = errno;
+
+        exit_code = pcmk_rc2exitc(rc);
         g_set_error(error, PCMK__EXITC_ERROR, exit_code,
                     "Could not access shadow instance '%s': %s",
-                    options.instance, strerror(errno));
+                    options.instance, strerror(rc));
         return errno;
     }
 
@@ -455,7 +456,7 @@ connect_real_cib(cib_t **real_cib, GError **error)
         return rc;
     }
 
-    rc = cib__signon_attempts(*real_cib, crm_system_name, cib_command, 5);
+    rc = cib__signon_attempts(*real_cib, cib_command, 5);
     rc = pcmk_legacy2rc(rc);
     if (rc != pcmk_rc_ok) {
         exit_code = pcmk_rc2exitc(rc);
@@ -535,7 +536,7 @@ static int
 write_shadow_file(const xmlNode *xml, const char *filename, bool reset,
                   GError **error)
 {
-    int rc = pcmk__xml_write_file(xml, filename, false, NULL);
+    int rc = pcmk__xml_write_file(xml, filename, false);
 
     if (rc != pcmk_rc_ok) {
         exit_code = pcmk_rc2exitc(rc);
@@ -713,7 +714,7 @@ commit_shadow_file(GError **error)
 done:
     free(filename);
     cib_delete(real_cib);
-    free_xml(input);
+    pcmk__xml_free(input);
 }
 
 /*!
@@ -749,7 +750,7 @@ create_shadow_empty(pcmk__output_t *out, GError **error)
 
 done:
     free(filename);
-    free_xml(output);
+    pcmk__xml_free(output);
 }
 
 /*!
@@ -773,26 +774,13 @@ create_shadow_from_cib(pcmk__output_t *out, bool reset, GError **error)
 
     if (!options.force) {
         if (reset) {
-            /* @COMPAT: Reset is dangerous to the shadow file, but to preserve
-             * compatibility we can't require --force unless there's a mismatch.
-             * At a compatibility break, call set_danger_error() with for_shadow
-             * and show_mismatch set to true.
-             */
-            const char *local = getenv("CIB_shadow");
+            const char *reason = "The reset command overwrites the active "
+                                 "shadow configuration";
 
-            if (!pcmk__str_eq(local, options.instance, pcmk__str_null_matches)) {
-                exit_code = CRM_EX_USAGE;
-                g_set_error(error, PCMK__EXITC_ERROR, exit_code,
-                            "The supplied shadow instance (%s) is not the same "
-                            "as the active one (%s).\n"
-                            "To prevent accidental destruction of the shadow "
-                            "file, the --force flag is required in order to "
-                            "proceed.",
-                            options.instance, local);
-                goto done;
-            }
+            exit_code = CRM_EX_USAGE;
+            set_danger_error(reason, true, true, error);
+            goto done;
         }
-
         if (check_file_exists(filename, reset, error) != pcmk_rc_ok) {
             goto done;
         }
@@ -809,7 +797,7 @@ create_shadow_from_cib(pcmk__output_t *out, bool reset, GError **error)
 
 done:
     free(filename);
-    free_xml(output);
+    pcmk__xml_free(output);
 }
 
 /*!
@@ -918,7 +906,7 @@ show_shadow_contents(pcmk__output_t *out, GError **error)
                      options.instance, NULL, output, NULL, shadow_disp_content);
         out->quiet = quiet_orig;
 
-        free_xml(output);
+        pcmk__xml_free(output);
     }
 
 done:
@@ -979,9 +967,9 @@ show_shadow_diff(pcmk__output_t *out, GError **error)
 
 done:
     free(filename);
-    free_xml(old_config);
-    free_xml(new_config);
-    free_xml(diff);
+    pcmk__xml_free(old_config);
+    pcmk__xml_free(new_config);
+    pcmk__xml_free(diff);
 }
 
 /*!
@@ -1131,7 +1119,7 @@ static GOptionEntry command_entries[] = {
 
     { "reset", 'r', G_OPTION_FLAG_NONE, G_OPTION_ARG_CALLBACK, command_cb,
       "Recreate named shadow copy from the active cluster configuration",
-      "name" },
+      "name. Required: --force." },
 
     { "switch", 's', G_OPTION_FLAG_NONE, G_OPTION_ARG_CALLBACK, command_cb,
       "(Advanced) Switch to the named shadow copy", "name" },
