@@ -378,12 +378,12 @@ setup_input(pcmk__output_t *out, const char *input, const char *output,
 
     rc = pcmk__update_configured_schema(&cib_object, false);
     if (rc != pcmk_rc_ok) {
-        free_xml(cib_object);
+        pcmk__xml_free(cib_object);
         return rc;
     }
 
     if (!pcmk__validate_xml(cib_object, NULL, NULL, NULL)) {
-        free_xml(cib_object);
+        pcmk__xml_free(cib_object);
         return pcmk_rc_schema_validation;
     }
 
@@ -396,7 +396,7 @@ setup_input(pcmk__output_t *out, const char *input, const char *output,
         free(pid);
     }
 
-    rc = pcmk__xml_write_file(cib_object, output, false, NULL);
+    rc = pcmk__xml_write_file(cib_object, output, false);
     if (rc != pcmk_rc_ok) {
         g_set_error(error, PCMK__EXITC_ERROR, CRM_EX_CANTCREAT,
                     "Could not create '%s': %s", output, pcmk_rc_str(rc));
@@ -404,7 +404,7 @@ setup_input(pcmk__output_t *out, const char *input, const char *output,
         setenv("CIB_file", output, 1);
     }
 
-    free_xml(cib_object);
+    pcmk__xml_free(cib_object);
     free(local_output);
     return rc;
 }
@@ -508,12 +508,6 @@ main(int argc, char **argv)
 
     if (args->verbosity > 0) {
         options.flags |= pcmk_sim_verbose;
-
-#ifdef PCMK__COMPAT_2_0
-        /* Redirect stderr to stdout so we can grep the output */
-        close(STDERR_FILENO);
-        dup2(STDOUT_FILENO, STDERR_FILENO);
-#endif
     }
 
     scheduler = pe_new_working_set();
@@ -525,15 +519,14 @@ main(int argc, char **argv)
     }
 
     if (pcmk_is_set(options.flags, pcmk_sim_show_scores)) {
-        pcmk__set_scheduler_flags(scheduler, pcmk_sched_output_scores);
+        pcmk__set_scheduler_flags(scheduler, pcmk__sched_output_scores);
     }
     if (pcmk_is_set(options.flags, pcmk_sim_show_utilization)) {
-        pcmk__set_scheduler_flags(scheduler, pcmk_sched_show_utilization);
+        pcmk__set_scheduler_flags(scheduler, pcmk__sched_show_utilization);
     }
-    pcmk__set_scheduler_flags(scheduler, pcmk_sched_no_compat);
 
     if (options.test_dir != NULL) {
-        scheduler->priv = out;
+        scheduler->priv->out = out;
         pcmk__profile_dir(options.test_dir, options.repeat, scheduler,
                           options.use_date);
         rc = pcmk_rc_ok;
