@@ -395,8 +395,7 @@ main(int argc, char **argv)
                     "Could not allocate scheduler data: %s", pcmk_rc_str(rc));
         goto done;
     }
-    pcmk__set_scheduler_flags(scheduler,
-                              pcmk_sched_no_counts|pcmk_sched_no_compat);
+    pcmk__set_scheduler_flags(scheduler, pcmk__sched_no_counts);
 
     cib_conn = cib_new();
     if (cib_conn == NULL) {
@@ -405,7 +404,7 @@ main(int argc, char **argv)
         goto done;
     }
 
-    rc = cib__signon_attempts(cib_conn, crm_system_name, cib_command, 5);
+    rc = cib__signon_attempts(cib_conn, cib_command, 5);
     rc = pcmk_legacy2rc(rc);
 
     if (rc != pcmk_rc_ok) {
@@ -419,7 +418,8 @@ main(int argc, char **argv)
         cib_xml_copy = pcmk__xml_read(options.xml_file);
 
     } else {
-        rc = cib_conn->cmds->query(cib_conn, NULL, &cib_xml_copy, cib_scope_local | cib_sync_call);
+        rc = cib_conn->cmds->query(cib_conn, NULL, &cib_xml_copy,
+                                   cib_sync_call);
         rc = pcmk_legacy2rc(rc);
 
         if (rc != pcmk_rc_ok) {
@@ -439,7 +439,7 @@ main(int argc, char **argv)
     }
 
     scheduler->input = cib_xml_copy;
-    scheduler->now = crm_time_new(NULL);
+    scheduler->priv->now = crm_time_new(NULL);
 
     cluster_status(scheduler);
 
@@ -563,9 +563,10 @@ main(int argc, char **argv)
 
             if (rc == EACCES) {
                 ticket_revoke_warning(options.ticket_id);
-                exit_code = pcmk_rc2exitc(rc);
+                exit_code = CRM_EX_UNSAFE;
                 g_set_error(&error, PCMK__EXITC_ERROR, exit_code,
                             "Ticket modification not allowed without --force");
+                goto done;
             }
         } else {
             rc = pcmk__ticket_set_attr(out, cib_conn, scheduler, options.ticket_id,
@@ -581,9 +582,10 @@ main(int argc, char **argv)
                     ticket_revoke_warning(options.ticket_id);
                 }
 
-                exit_code = pcmk_rc2exitc(rc);
+                exit_code = CRM_EX_UNSAFE;
                 g_set_error(&error, PCMK__EXITC_ERROR, exit_code,
                             "Ticket modification not allowed without --force");
+                goto done;
             }
         }
 
